﻿note
	description: "This class represents the MCI sequencer device."
	status: "See notice at end of class."
	author: "Robin van Ommeren"
	date: "$Date: 2018-02-05 08:33:45 +0000 (Mon, 05 Feb 2018) $"
	revision: "$Revision: 101353 $"

class
	WEX_MCI_SEQUENCER

inherit
	WEX_MCI_DEVICE

create
	make

feature -- Access

	seek_to (a_position: INTEGER)
			-- Position the MIDI file at `position' in milliseconds.
		require
			opened: opened
			a_positive_position: a_position >= 0
			a_meaningful_position: a_position <= media_length
		local
			seek_parms: WEX_MCI_SEEK_PARMS
		do
			create seek_parms.make (parent, a_position)
			seek_device (seek_parms, Mci_to)
		end

	open (a_file: PATH)
			-- Open a Mci device to play a MIDI file.
		require
			not_opened: not opened
			a_file_not_void: a_file /= Void
			a_file_meaningful: not a_file.is_empty
		local
			open_parms: WEX_MCI_OPEN_PARMS
		do
			create open_parms.make (parent, device_name)
			open_parms.set_element_name (a_file.utf_8_name)
			open_device (open_parms, Mci_open_element +
				Mci_open_type)
		end

feature {NONE} -- Implementation

	device_name: STRING
			-- Device name
		once
			Result := "sequencer"
		end

end

--|-------------------------------------------------------------------------
--| WEX, Windows Eiffel library eXtension
--| Copyright (C) 1998  Robin van Ommeren, Andreas Leitner
--| Copyright (C) 2017  Eiffel Software, Alexander Kogtenkov
--| See the file forum.txt included in this package for licensing info.
--|
--| Comments, Questions, Additions to this library? please contact:
--|
--| Robin van Ommeren						Andreas Leitner
--| Eikenlaan 54M								Arndtgasse 1/3/5
--| 7151 WT Eibergen							8010 Graz
--| The Netherlands							Austria
--| email: robin.van.ommeren@wxs.nl		email: andreas.leitner@teleweb.at
--| web: http://home.wxs.nl/~rommeren	web: about:blank
--|-------------------------------------------------------------------------
