note

	description:

		"C code generators"

	library: "Gobo Eiffel Tools Library"
	copyright: "Copyright (c) 2004-2022, Eric Bezault and others"
	license: "MIT License"
	date: "$Date: 2023-03-09 10:35:58 +0000 (Thu, 09 Mar 2023) $"
	revision: "$Revision: 106643 $"

class ET_C_GENERATOR

inherit

	ET_AST_NULL_PROCESSOR
		rename
			make as make_null
		redefine
			process_across_expression,
			process_across_instruction,
			process_assigner_instruction,
			process_assignment,
			process_assignment_attempt,
			process_attribute,
			process_bang_instruction,
			process_binary_integer_constant,
			process_bracket_expression,
			process_c1_character_constant,
			process_c2_character_constant,
			process_c3_character_constant,
			process_call_agent,
			process_check_instruction,
			process_constant_attribute,
			process_convert_builtin_expression,
			process_convert_from_expression,
			process_convert_to_expression,
			process_create_expression,
			process_create_instruction,
			process_current,
			process_current_address,
			process_debug_instruction,
			process_deferred_function,
			process_deferred_procedure,
			process_do_function,
			process_do_function_inline_agent,
			process_do_procedure,
			process_do_procedure_inline_agent,
			process_equality_expression,
			process_explicit_convert_from_expression,
			process_explicit_convert_to_expression,
			process_expression_address,
			process_extended_attribute,
			process_external_function,
			process_external_function_inline_agent,
			process_external_procedure,
			process_external_procedure_inline_agent,
			process_false_constant,
			process_feature_address,
			process_hexadecimal_integer_constant,
			process_identifier,
			process_if_expression,
			process_if_instruction,
			process_infix_cast_expression,
			process_infix_expression,
			process_inspect_expression,
			process_inspect_instruction,
			process_iteration_cursor,
			process_loop_instruction,
			process_manifest_array,
			process_manifest_tuple,
			process_manifest_type,
			process_named_object_test,
			process_object_equality_expression,
			process_object_test,
			process_octal_integer_constant,
			process_old_expression,
			process_old_object_test,
			process_once_function,
			process_once_function_inline_agent,
			process_once_manifest_string,
			process_once_procedure,
			process_once_procedure_inline_agent,
			process_parenthesis_expression,
			process_parenthesis_instruction,
			process_parenthesized_expression,
			process_precursor_expression,
			process_precursor_instruction,
			process_prefix_expression,
			process_qualified_call_expression,
			process_qualified_call_instruction,
			process_quantifier_expression,
			process_regular_integer_constant,
			process_regular_manifest_string,
			process_regular_real_constant,
			process_repeat_instruction,
			process_result,
			process_result_address,
			process_retry_instruction,
			process_semicolon_symbol,
			process_special_manifest_string,
			process_static_call_expression,
			process_static_call_instruction,
			process_strip_expression,
			process_true_constant,
			process_underscored_integer_constant,
			process_underscored_real_constant,
			process_unique_attribute,
			process_unqualified_call_expression,
			process_unqualified_call_instruction,
			process_verbatim_string,
			process_void
		end

	ET_SHARED_TOKEN_CONSTANTS
		export {NONE} all end

	ET_SHARED_IDENTIFIER_TESTER
		export {NONE} all end

	ET_SHARED_DYNAMIC_PRIMARY_TYPE_COMPARATOR_BY_ID
		export {NONE} all end

	KL_SHARED_STREAMS
		export {NONE} all end

	KL_SHARED_FILE_SYSTEM
		export {NONE} all end

	KL_SHARED_EXECUTION_ENVIRONMENT
		export {NONE} all end

	KL_SHARED_STRING_EQUALITY_TESTER
		export {NONE} all end

	ET_SHARED_ISE_VARIABLES
		export {NONE} all end

	KL_IMPORTED_CHARACTER_ROUTINES
		export {NONE} all end

	KL_IMPORTED_INTEGER_ROUTINES
		export {NONE} all end

	KL_IMPORTED_STRING_ROUTINES
		export {NONE} all end

	UT_IMPORTED_FORMATTERS
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make (a_system: like current_dynamic_system; a_system_processor: like system_processor)
			-- Create a new C code generator.
		require
			a_system_not_void: a_system /= Void
			a_system_processor_not_void: a_system_processor /= Void
		local
			l_buffer: STRING
		do
			system_processor := a_system_processor
			current_dynamic_system := a_system
			dynamic_types := a_system.dynamic_types
			short_names := True
			split_mode := True
			split_threshold := default_split_threshold
			use_boehm_gc := current_system.use_boehm_gc
			system_name := "unknown"
			current_file := null_output_stream
			header_file := null_output_stream
			current_type := a_system.unknown_type
			current_feature := dummy_feature
			create extra_dynamic_type_sets.make_with_capacity (50)
			extra_dynamic_type_sets.put_last (a_system.any_type)
			current_dynamic_type_sets := extra_dynamic_type_sets
			create l_buffer.make (1024)
			create current_function_header_buffer.make (l_buffer)
			create l_buffer.make (100000)
			create current_function_body_buffer.make (l_buffer)
			create temp_variables.make (40)
			create used_temp_variables.make (40)
			create free_temp_variables.make (40)
			create frozen_temp_variables.make (40)
			create unused_equality_expressions.make (10)
			create unused_equality_types.make (100)
			create unused_object_equality_types.make (100)
			create unused_unqualified_call_expressions.make (10)
			create unused_qualified_call_expressions.make (10)
			create unused_actual_argument_lists.make (10)
			create unused_result_expressions.make (10)
			create unused_identifiers.make (10)
			create conforming_types.make_with_capacity (100)
			create non_conforming_types.make_with_capacity (100)
			create equality_type_set.make_empty (current_dynamic_system.unknown_type)
			create equality_common_types.make_with_capacity (100)
			create equality_common_reference_types.make_with_capacity (100)
			create unused_standalone_type_sets.make (50)
			create operand_stack.make (5000)
			create call_operands.make (5000)
			create attachment_dynamic_type_ids.make (100)
			create target_dynamic_type_ids.make (100)
			create target_dynamic_types.make_map (100)
			create standalone_type_sets.make_with_capacity (50)
			create deep_twin_types.make (100)
			create deep_equal_types.make (100)
			create deep_feature_target_type_sets.make_map (100)
			create current_object_tests.make (20)
			create current_equalities.make (20)
			create current_agents.make (20)
			create agent_open_operands.make (20)
			create agent_closed_operands.make (20)
			agent_target := tokens.current_keyword
			create agent_arguments.make_with_capacity (100)
			create agent_instruction.make (agent_target, current_feature.static_feature.name, Void)
			create agent_expression.make (agent_target, current_feature.static_feature.name, Void)
			agent_closed_operands_type := current_dynamic_system.unknown_type
			create agent_manifest_tuple.make_with_capacity (20)
			create formal_arguments.make (20)
			create manifest_array_types.make (100)
			create big_manifest_array_types.make (100)
			create manifest_tuple_types.make (100)
			create once_features.make_map (10000)
			create once_per_process_counts.make_filled (0, 1, once_kind_count)
			create once_per_thread_counts.make_filled (0, 1, once_kind_count)
			create constant_features.make_map (10000)
			create inline_constants.make_map (10000)
			create dispose_procedures.make_map (dynamic_types.count)
			create dynamic_type_id_set_names.make_map (10000)
			dynamic_type_id_set_names.set_key_equality_tester (string_equality_tester)
			create called_features.make (1000)
			create called_static_features.make (1000)
			create included_header_filenames.make (100)
			included_header_filenames.set_equality_tester (string_equality_tester)
			create included_cpp_header_filenames.make (50)
			included_cpp_header_filenames.set_equality_tester (string_equality_tester)
			create included_runtime_header_files.make (100)
			included_runtime_header_files.set_key_equality_tester (string_equality_tester)
			create included_runtime_c_files.make (100)
			included_runtime_c_files.set_equality_tester (string_equality_tester)
			create c_filenames.make_map (100)
			c_filenames.set_key_equality_tester (string_equality_tester)
			make_rescue_data
			make_external_regexps
		ensure
			current_dynamic_system_set: current_dynamic_system = a_system
			system_processor_set: system_processor = a_system_processor
		end

feature -- Access

	current_system: ET_SYSTEM
			-- Surrounding Eiffel system
			-- (Note: there is a frozen feature called `system' in
			-- class GENERAL of SmartEiffel 1.0)
		do
			Result := current_dynamic_system.current_system
		ensure
			current_system_not_void: Result /= Void
		end

	current_dynamic_system: ET_DYNAMIC_SYSTEM
			-- Surrounding Eiffel dynamic system

	system_processor: ET_SYSTEM_PROCESSOR
			-- System processor currently used

feature -- Error handling

	error_handler: ET_ERROR_HANDLER
			-- Error handler
		do
			Result := system_processor.error_handler
		ensure
			error_handler_not_void: Result /= Void
		end

feature -- Status report

	short_names: BOOLEAN
			-- Should short names be generated for type and feature names?

	has_fatal_error: BOOLEAN
			-- Has a fatal error occurred when generating `current_dynamic_system'?

feature -- Measurement

	never_void_target_count: INTEGER
			-- Number of calls whose target is guaranteed to be non-Void at compilation time

	can_be_void_target_count: INTEGER
			-- Number of calls for which it was not possible to guarantee at compilation
			-- time whether its target would never be Void

feature -- Compilation options

	console_application_mode: BOOLEAN
			-- Should the generated application be a console application
			-- (or a Windows GUI application)?
		do
			Result := current_system.console_application_mode
		end

	multithreaded_mode: BOOLEAN
			-- Should the generated application be thread-capable?
		do
			Result := current_system.multithreaded_mode
		end

	exception_trace_mode: BOOLEAN
			-- Should the generated application be able to provide an exception trace?
			-- An exception trace is the execution path from the root creation procedure
			-- to the feature where an exception occurred.
		do
			Result := current_system.exception_trace_mode
		end

	total_order_on_reals_mode: BOOLEAN
			-- Should NaN values will be lower than any other real values,
			-- and comparing NaN with another NaN will yield True and not False
			-- as usually done in IEEE arithmetic?
		do
			Result := current_system.total_order_on_reals_mode
		end

	line_generation_mode: BOOLEAN
			-- Should information about positions of constructs in the Eiffel code
			-- be included in the generated C code (e.g. using #line clauses)?
		do
			Result := current_system.line_generation_mode
		end

	current_in_exception_trace: BOOLEAN = False
			-- Should address of current object be displayed when generating an exception trace?

	finalize_mode: BOOLEAN
			-- Compilation with optimizations turned on?

	split_mode: BOOLEAN
			-- Should generated C code be split over several C files
			-- instead of being held in a single possibly large C file?
			-- When in split mode, the next C construct will be generated
			-- in a new C file if the current C file is already larger
			-- than `split_threshold' bytes.

	split_threshold: INTEGER
			-- When in split mode, the next C construct will be generated
			-- in a new C file if the current C file is already larger
			-- than `split_threshold' bytes.

	trace_mode: BOOLEAN
			-- Should the generated application be compiled with trace turned on?
			-- The trace is displayed each time the execution enters or exits
			-- from a feature.
		do
			Result := current_system.trace_mode
		end

	use_boehm_gc: BOOLEAN
			-- Should the application be compiled with the Boehm GC?

feature -- Compilation options setting

	set_finalize_mode (b: BOOLEAN)
			-- Set `finalize_mode' to `b'.
		do
			finalize_mode := b
		ensure
			finalize_mode_set: finalize_mode = b
		end

	set_split_mode (b: BOOLEAN)
			-- Set `split_mode' to `b'.
		do
			split_mode := b
		ensure
			split_mode_set: split_mode = b
		end

	set_split_threshold (v: INTEGER)
			-- Set `split_threshold' to `v'.
		require
			v_positive: v > 0
		do
			split_threshold := v
		ensure
			split_threshold_set: split_threshold = v
		end

	set_use_boehm_gc (b: BOOLEAN)
			-- Set `use_boehm_gc' to `b'.
		do
			use_boehm_gc := b
		ensure
			use_boehm_gc_set: use_boehm_gc = b
		end

feature -- Generation

	generate (a_system_name: STRING)
			-- Generate C code and C compilation script file for `current_dynamic_system'.
			-- Set `has_fatal_error' if a fatal error occurred.
		require
			a_system_name_not_void: a_system_name /= Void
		do
			has_fatal_error := False
			generate_c_code (a_system_name)
			generate_compilation_script (a_system_name)
			c_filenames.wipe_out
		end

feature {NONE} -- Compilation script generation

	generate_compilation_script (a_system_name: STRING)
			-- Generate C compilation script file for `current_dynamic_system'.
			-- Set `has_fatal_error' if a fatal error occurred.
		require
			a_system_name_not_void: a_system_name /= Void
		local
			l_obj_filenames: STRING
			l_variables: like c_config
			l_filename: STRING
			l_base_name: STRING
			i, nb: INTEGER
			l_obj: STRING
			l_env_regexp: RX_PCRE_REGULAR_EXPRESSION
			l_wel_regexp: RX_PCRE_REGULAR_EXPRESSION
			l_com_regexp: RX_PCRE_REGULAR_EXPRESSION
			l_com_runtime_regexp: RX_PCRE_REGULAR_EXPRESSION
			l_replacement: STRING
			l_external_include_pathnames: DS_ARRAYED_LIST [STRING]
			l_external_library_pathnames: DS_ARRAYED_LIST [STRING]
			l_external_object_pathnames: DS_ARRAYED_LIST [STRING]
			l_external_cflags: DS_ARRAYED_LIST [STRING]
			l_external_linker_flags: DS_ARRAYED_LIST [STRING]
			l_includes: STRING
			l_libs: STRING
			l_cflags: STRING
			l_lflags: STRING
			l_cflag: STRING
			l_lflag: STRING
			l_pathname: STRING
			l_cursor: DS_HASH_TABLE_CURSOR [STRING, STRING]
			l_external_c_filenames: DS_HASH_TABLE [STRING, STRING]
		do
			l_base_name := a_system_name
			l_variables := c_config
			create l_env_regexp.make
			l_env_regexp.compile ("\$\(([^)]+)\)")
			create l_wel_regexp.make
			l_wel_regexp.set_case_insensitive (True)
			l_wel_regexp.compile ("(.*[\\/]library[\\/]wel[\\/]).*[\\/](mt)?wel\.lib")
			create l_com_regexp.make
			l_com_regexp.set_case_insensitive (True)
			l_com_regexp.compile ("(.*[\\/]library[\\/]com[\\/]).*[\\/](mt)?com\.(lib|a)")
			create l_com_runtime_regexp.make
			l_com_runtime_regexp.set_case_insensitive (True)
			l_com_runtime_regexp.compile ("(.*[\\/]library[\\/]com[\\/]).*[\\/](mt)?com_runtime\.(lib|a)")
				-- Include files.
			create l_includes.make (256)
			l_external_include_pathnames := current_system.external_include_pathnames
			nb := l_external_include_pathnames.count
			from i := 1 until i > nb loop
				l_pathname := l_external_include_pathnames.item (i)
				l_env_regexp.match (l_pathname)
				l_replacement := STRING_.new_empty_string (l_pathname, 6)
				l_replacement.append_string ("${\1\}")
				l_pathname := Execution_environment.interpreted_string (l_env_regexp.replace_all (l_replacement))
				if i /= 1 then
					l_includes.append_character (' ')
				end
				l_includes.append_string ("-I")
				if l_pathname.starts_with ("%"") then
					l_includes.append_string (l_pathname)
				else
					l_includes.append_character ('%"')
					l_includes.append_string (l_pathname)
					l_includes.append_character ('%"')
				end
				i := i + 1
			end
			l_variables.force (l_includes, "includes")
				-- C flags.
			create l_cflags.make (256)
			l_variables.search ("cflags")
			if l_variables.found then
				l_cflags.append_string (l_variables.found_item)
			end
			l_external_cflags := current_system.external_cflags
			nb := l_external_cflags.count
			from i := 1 until i > nb loop
				l_cflag := l_external_cflags.item (i)
				l_env_regexp.match (l_cflag)
				l_replacement := STRING_.new_empty_string (l_cflag, 6)
				l_replacement.append_string ("${\1\}")
				l_cflag := Execution_environment.interpreted_string (l_env_regexp.replace_all (l_replacement))
				if not l_cflags.is_empty then
					l_cflags.append_character (' ')
				end
				l_cflags.append_string (l_cflag)
				i := i + 1
			end
			l_variables.force (l_cflags, "cflags")
				-- Linker flags.
			create l_lflags.make (256)
			l_variables.search ("lflags")
			if l_variables.found then
				l_lflags.append_string (l_variables.found_item)
			end
			l_external_linker_flags := current_system.external_linker_flags
			nb := l_external_linker_flags.count
			from i := 1 until i > nb loop
				l_lflag := l_external_linker_flags.item (i)
				l_env_regexp.match (l_lflag)
				l_replacement := STRING_.new_empty_string (l_lflag, 6)
				l_replacement.append_string ("${\1\}")
				l_lflag := Execution_environment.interpreted_string (l_env_regexp.replace_all (l_replacement))
				if not l_lflags.is_empty then
					l_lflags.append_character (' ')
				end
				l_lflags.append_string (l_lflag)
				i := i + 1
			end
			l_variables.force (l_lflags, "lflags")
				-- C libraries.
			create l_external_c_filenames.make_map (10)
			l_external_c_filenames.set_key_equality_tester (string_equality_tester)
			create l_libs.make (256)
			l_external_library_pathnames := current_system.external_library_pathnames
			nb := l_external_library_pathnames.count
			from i := 1 until i > nb loop
				l_pathname := l_external_library_pathnames.item (i)
				l_env_regexp.match (l_pathname)
				l_replacement := STRING_.new_empty_string (l_pathname, 6)
				l_replacement.append_string ("${\1\}")
				l_pathname := Execution_environment.interpreted_string (l_env_regexp.replace_all (l_replacement))
				if l_wel_regexp.recognizes (l_pathname) then
					add_external_c_files ("wel", l_wel_regexp.captured_substring (1) + "clib", l_external_c_filenames)
				elseif l_com_regexp.recognizes (l_pathname) then
					add_external_c_files ("com", l_com_regexp.captured_substring (1) + "clib", l_external_c_filenames)
				elseif l_com_runtime_regexp.recognizes (l_pathname) then
					add_external_c_files ("com_runtime", l_com_runtime_regexp.captured_substring (1) + "clib_runtime", l_external_c_filenames)
				else
					if i /= 1 then
						l_libs.append_character (' ')
					end
					if l_pathname.starts_with ("%"") then
						l_libs.append_string (l_pathname)
					else
						l_libs.append_character ('%"')
						l_libs.append_string (l_pathname)
						l_libs.append_character ('%"')
					end
				end
				i := i + 1
			end
			l_variables.force (l_libs, "libs")
				-- Executable name.
			l_variables.force (l_base_name + l_variables.item ("exe"), "exe")
				-- Object files.
			create l_obj_filenames.make (100)
			l_obj := l_variables.item ("obj")
				-- List objects in reverse order so that it looks like
				-- a countdown when compiling since the filenames are numbered.
			l_cursor := c_filenames.new_cursor
			from l_cursor.finish until l_cursor.before loop
				if not l_cursor.is_last then
					l_obj_filenames.append_character (' ')
				end
				l_filename := l_cursor.key
				l_obj_filenames.append_string (l_filename)
				l_obj_filenames.append_string (l_obj)
				l_cursor.back
			end
			l_external_object_pathnames := current_system.external_object_pathnames
			nb := l_external_object_pathnames.count
			from i := 1 until i > nb loop
				l_pathname := l_external_object_pathnames.item (i)
				l_env_regexp.match (l_pathname)
				l_replacement := STRING_.new_empty_string (l_pathname, 6)
				l_replacement.append_string ("${\1\}")
				l_pathname := Execution_environment.interpreted_string (l_env_regexp.replace_all (l_replacement))
				if l_wel_regexp.recognizes (l_pathname) then
					add_external_c_files ("wel", l_wel_regexp.captured_substring (1) + "clib", l_external_c_filenames)
				elseif l_com_regexp.recognizes (l_pathname) then
					add_external_c_files ("com", l_com_regexp.captured_substring (1) + "clib", l_external_c_filenames)
				elseif l_com_runtime_regexp.recognizes (l_pathname) then
					add_external_c_files ("com_runtime", l_com_runtime_regexp.captured_substring (1) + "clib_runtime", l_external_c_filenames)
				else
					l_obj_filenames.append_character (' ')
					if l_pathname.starts_with ("%"") then
						l_obj_filenames.append_string (l_pathname)
					else
						l_obj_filenames.append_character ('%"')
						l_obj_filenames.append_string (l_pathname)
						l_obj_filenames.append_character ('%"')
					end
				end
				i := i + 1
			end
			l_cursor := l_external_c_filenames.new_cursor
			from l_cursor.start until l_cursor.after loop
				l_obj_filenames.append_character (' ')
				l_filename := l_cursor.key
				l_obj_filenames.append_string (l_filename)
				l_obj_filenames.append_string (l_obj)
				l_cursor.forth
			end
			l_variables.force (l_obj_filenames, "objs")
				-- Header file.
			l_variables.force (l_base_name + h_file_extension, "header")
				-- Script.
			generate_compilation_shell_script (l_base_name, l_variables, l_obj_filenames, l_env_regexp, l_external_c_filenames)
			if not operating_system.is_windows then
				generate_compilation_makefile (l_base_name, l_variables)
			end
		end

	generate_compilation_shell_script (a_system_name: STRING; a_variables: like c_config; an_obj_filenames: STRING; an_env_regexp: RX_PCRE_REGULAR_EXPRESSION; an_external_c_filenames: DS_HASH_TABLE [STRING, STRING])
			-- Generate script to compile the C code.
		local
			l_base_name: STRING
			l_script_filename: STRING
			l_file: KL_TEXT_OUTPUT_FILE
			l_link_template: STRING
			l_command_name: STRING
			l_rc_template: STRING
			l_rc_filename: STRING
			l_res_filename: detachable STRING
			l_make_template: STRING
			i, nb: INTEGER
			l_cursor: DS_HASH_TABLE_CURSOR [STRING, STRING]
			l_external_make_pathnames: DS_ARRAYED_LIST [STRING]
			l_pathname: STRING
			l_replacement: STRING
			l_external_resource_pathnames: DS_ARRAYED_LIST [STRING]
			l_cc_template: STRING
			l_filename: STRING
		do
			l_base_name := a_system_name
			if operating_system.is_windows then
				l_script_filename := l_base_name + bat_file_extension
			else
				l_script_filename := l_base_name + sh_file_extension
			end
			create l_file.make (l_script_filename)
			l_file.open_write
			if l_file.is_open_write then
				if operating_system.is_windows then
					l_file.put_line ("@echo off")
				else
					l_file.put_line ("#!/bin/sh")
					if attached ise_variables.ise_platform_value as l_ise_platform_value and then not l_ise_platform_value.is_empty then
							-- Set environment variable $ISE_PLATFORM only if it's not defined yet.
							-- (See https://tldp.org/LDP/Bash-Beginners-Guide/html/sect_10_03.html#sect_10_03_03)
						l_file.put_line ("export " + ise_variables.ise_platform_variable + "=${" + ise_variables.ise_platform_variable + ":-" + l_ise_platform_value + "}")
					end
				end
					-- Make files.
				a_variables.search ("make")
				if a_variables.found then
					l_make_template := a_variables.found_item
					l_external_make_pathnames := current_system.external_make_pathnames
					nb := l_external_make_pathnames.count
					from i := 1 until i > nb loop
						l_pathname := l_external_make_pathnames.item (i)
						an_env_regexp.match (l_pathname)
						l_replacement := STRING_.new_empty_string (l_pathname, 6)
						l_replacement.append_string ("${\1\}")
						l_pathname := Execution_environment.interpreted_string (an_env_regexp.replace_all (l_replacement))
						if not l_pathname.starts_with ("%"") then
							l_pathname := "%"" + l_pathname + "%""
						end
						a_variables.force (l_pathname, "makefile")
						l_command_name := a_variables.expanded_string (l_make_template, False)
						l_file.put_line (l_command_name)
						i := i + 1
					end
				end
					-- Compile files in reverse order so that it looks like
					-- a countdown since the filenames are numbered.
				l_cc_template := a_variables.item ("cc")
				l_cursor := c_filenames.new_cursor
				from l_cursor.finish until l_cursor.before loop
					l_filename := l_cursor.key + l_cursor.item
					a_variables.force (l_filename, "c")
					l_command_name := a_variables.expanded_string (l_cc_template, False)
					l_file.put_line (l_command_name)
					l_cursor.back
				end
				l_cursor := an_external_c_filenames.new_cursor
				from l_cursor.start until l_cursor.after loop
					l_filename := l_cursor.key + l_cursor.item
					a_variables.force (l_filename, "c")
					l_command_name := a_variables.expanded_string (l_cc_template, False)
					l_file.put_line (l_command_name)
					l_cursor.forth
				end
					-- Resource files.
				a_variables.search ("rc")
				if a_variables.found then
					l_rc_template := a_variables.found_item
					l_external_resource_pathnames := current_system.external_resource_pathnames
					nb := l_external_resource_pathnames.count
					from i := 1 until i > nb loop
						l_pathname := l_external_resource_pathnames.item (i)
						an_env_regexp.match (l_pathname)
						l_replacement := STRING_.new_empty_string (l_pathname, 6)
						l_replacement.append_string ("${\1\}")
						l_pathname := Execution_environment.interpreted_string (an_env_regexp.replace_all (l_replacement))
						if file_system.has_extension (l_pathname, resx_file_extension) then
								-- Skip .Net resource files.
							l_res_filename := Void
						elseif file_system.has_extension (l_pathname, res_file_extension) then
							l_res_filename := l_pathname
						else
							l_rc_filename := l_pathname
							if file_system.has_extension (l_pathname, rc_file_extension) then
								l_res_filename := l_pathname.twin
								l_res_filename.remove_tail (rc_file_extension.count)
								l_res_filename.append_string (res_file_extension)
							else
								l_res_filename := l_pathname.twin
								l_res_filename.append_string (res_file_extension)
							end
							a_variables.force (l_rc_filename, "rc_file")
							a_variables.force (l_res_filename, "res_file")
							l_command_name := a_variables.expanded_string (l_rc_template, False)
							l_file.put_line (l_command_name)
						end
						if l_res_filename /= Void then
							an_obj_filenames.append_character (' ')
							if l_res_filename.starts_with ("%"") then
								an_obj_filenames.append_string (l_res_filename)
							else
								an_obj_filenames.append_character ('%"')
								an_obj_filenames.append_string (l_res_filename)
								an_obj_filenames.append_character ('%"')
							end
						end
						i := i + 1
					end
					l_rc_filename :=  l_base_name + rc_file_extension
					if file_system.file_exists (l_rc_filename) then
						l_res_filename := l_base_name + res_file_extension
						a_variables.force (l_rc_filename, "rc_file")
						a_variables.force (l_res_filename, "res_file")
						l_command_name := a_variables.expanded_string (l_rc_template, False)
						l_file.put_line (l_command_name)
						an_obj_filenames.append_character (' ')
						an_obj_filenames.append_string (l_res_filename)
					end
				end
				l_link_template := a_variables.item ("link")
				l_command_name := a_variables.expanded_string (l_link_template, False)
				l_file.put_line (l_command_name)
				l_file.close
					-- Set executable mode.
				l_file.change_mode (0c777)
			else
				set_fatal_error
				report_cannot_write_error (l_script_filename)
			end
		end

	generate_compilation_makefile (a_system_name: STRING; a_variables: like c_config)
			-- Generate Makefile to compile the C code.
		local
			l_base_name: STRING
			l_makefile_filename: STRING
			l_file: KL_TEXT_OUTPUT_FILE
			l_makefile_template: STRING
			l_command_name: STRING
		do
			l_base_name := a_system_name
			l_makefile_filename := l_base_name + make_file_extension
			create l_file.make (l_makefile_filename)
			l_file.open_write
			if l_file.is_open_write then
				l_makefile_template := a_variables.item ("Makefile")
				l_command_name := a_variables.expanded_string (l_makefile_template, False)
				l_file.put_line (l_command_name)
				l_file.close
			else
				set_fatal_error
				report_cannot_write_error (l_makefile_filename)
			end
		end

	c_config: DS_STRING_HASH_TABLE
			-- C compiler configuration
		local
			l_name: detachable STRING
			l_filename: STRING
			l_file: KL_TEXT_INPUT_FILE
			l_default: BOOLEAN
			l_c_config_parser: UT_CONFIG_PARSER
			l_cursor: DS_HASH_TABLE_CURSOR [STRING, STRING]
		do
			l_name := Execution_environment.variable_value ("GOBO_CC")
			if l_name = Void then
				l_filename := file_system.nested_pathname ("${GOBO}", <<"tool", "gec", "config", "c", "default.cfg">>)
				l_filename := Execution_environment.interpreted_string (l_filename)
				create l_file.make (l_filename)
				l_file.open_read
				if l_file.is_open_read then
					l_file.read_line
					if not l_file.end_of_file then
						l_name := STRING_.cloned_string (l_file.last_string)
						STRING_.left_adjust (l_name)
						STRING_.right_adjust (l_name)
					end
					l_file.close
				end
			end
			if l_name = Void then
				l_default := True
				if operating_system.is_windows then
					l_name := "msc"
				else
					l_name := "cc"
				end
			end
			create Result.make_map (10)
			Result.set_key_equality_tester (string_equality_tester)
				-- Put some platform-dependent default values.
			if operating_system.is_windows then
				Result.put_new ("cl -nologo $cflags $includes -c $c", "cc")
				Result.put_new ("link -nologo $lflags -subsystem:console -out:$exe $objs $libs", "link")
				Result.put_new (".obj", "obj")
				Result.put_new (".exe", "exe")
				Result.put_new ("", "cflags")
				Result.put_new ("", "lflags")
			else
				Result.put_new ("cc $cflags $includes -c $c", "cc")
				Result.put_new ("cc $lflags -o $exe $objs $libs", "link")
				Result.put_new (".o", "obj")
				Result.put_new ("", "exe")
				Result.put_new ("", "cflags")
				Result.put_new ("", "lflags")
				Result.put_new (makefile_template, "Makefile")
			end
			l_filename := file_system.nested_pathname ("${GOBO}", <<"tool", "gec", "config", "c", l_name>>)
			l_filename := Execution_environment.interpreted_string (l_filename)
			if not file_system.has_extension (l_filename, cfg_file_extension) then
				l_filename := l_filename + cfg_file_extension
			end
			create l_file.make (l_filename)
			l_file.open_read
			if l_file.is_open_read then
				create l_c_config_parser.make (error_handler)
				if not finalize_mode then
					l_c_config_parser.define_value ("True", "EIF_WORKBENCH")
				end
				if use_boehm_gc then
					l_c_config_parser.define_value ("True", c_ge_use_boehm_gc)
				end
				if console_application_mode then
					l_c_config_parser.define_value ("True", "EIF_CONSOLE")
				end
				if multithreaded_mode then
					l_c_config_parser.define_value ("True", c_ge_use_threads)
				end
				l_c_config_parser.parse_file (l_file)
				l_file.close
				if l_c_config_parser.has_error then
					set_fatal_error
				else
					l_cursor := l_c_config_parser.config_values.new_cursor
					from l_cursor.start until l_cursor.after loop
						Result.force (l_cursor.item, l_cursor.key)
						l_cursor.forth
					end
				end
			elseif not l_default then
				set_fatal_error
				report_cannot_read_error (l_filename)
			end
		ensure
			c_config_not_void: Result /= Void
			no_void_variables: not Result.has_void_item
			cc_defined: Result.has ("cc")
			link_defined: Result.has ("link")
			exe_defined: Result.has ("exe")
			obj_defined: Result.has ("obj")
		end

	add_external_c_files (a_library_name, a_clib_dirname: STRING; a_external_c_filenames: DS_HASH_TABLE [STRING, STRING])
			-- Add C files of `a_library_name' found in `a_clib_dirname' to the compilation script.
		require
			a_library_name_not_void: a_library_name /= Void
			a_clib_dirname_not_void: a_clib_dirname /= Void
			a_external_c_filenames_not_void: a_external_c_filenames /= Void
		local
			l_c_file: detachable KL_TEXT_OUTPUT_FILE
			l_cpp_file: detachable KL_TEXT_OUTPUT_FILE
			l_external_file: detachable KL_TEXT_OUTPUT_FILE
			l_c_filename: STRING
			l_cpp_filename: STRING
			l_external_filename: detachable STRING
			l_dir: KL_DIRECTORY
			l_filename: STRING
			l_extension: STRING
			l_basename: STRING
		do
			l_c_filename := a_library_name + c_file_extension
			l_cpp_filename := a_library_name + "_cpp" + cpp_file_extension
			create l_dir.make (a_clib_dirname)
			l_dir.open_read
			if not l_dir.is_open_read then
				set_fatal_error
				report_cannot_read_error (a_clib_dirname)
			else
				from
					l_dir.read_entry
				until
					l_dir.end_of_input
				loop
					l_filename := l_dir.last_entry
					if l_filename.ends_with (c_file_extension) then
						l_external_file := l_c_file
						l_external_filename := l_c_filename
					elseif l_filename.ends_with (cpp_file_extension) then
						l_external_file := l_cpp_file
						l_external_filename := l_cpp_filename
					else
						l_external_file := Void
						l_external_filename := Void
					end
					if l_external_filename /= Void then
						if l_external_file = Void then
							create l_external_file.make (l_external_filename)
							if l_external_filename = l_c_filename then
								l_c_file := l_external_file
							else
								l_cpp_file := l_external_file
							end
							l_external_file.open_write
							if not l_external_file.is_open_write then
								set_fatal_error
								report_cannot_write_error (l_external_filename)
							else
								l_extension := file_system.extension (l_external_filename)
								l_basename := l_external_filename.twin
								l_basename.remove_tail (l_extension.count)
								a_external_c_filenames.force_last (l_extension, l_basename)
								if multithreaded_mode then
									l_external_file.put_string (c_define)
									l_external_file.put_character (' ')
									l_external_file.put_line (c_ge_use_threads)
									l_external_file.put_new_line
								end
								l_external_file.put_string (c_include)
								l_external_file.put_character (' ')
								l_external_file.put_character ('"')
								l_external_file.put_string ("ge_eiffel.h")
								l_external_file.put_character ('"')
								l_external_file.put_new_line
								l_external_file.put_string (c_include)
								l_external_file.put_character (' ')
								l_external_file.put_character ('"')
								l_external_file.put_string ("ge_gc.h")
								l_external_file.put_character ('"')
								l_external_file.put_new_line
									-- Two header files needed to compile EiffelCOM.
								l_external_file.put_string (c_include)
								l_external_file.put_character (' ')
								l_external_file.put_character ('"')
								l_external_file.put_string ("eif_cecil.h")
								l_external_file.put_character ('"')
								l_external_file.put_new_line
								l_external_file.put_string (c_include)
								l_external_file.put_character (' ')
								l_external_file.put_character ('"')
								l_external_file.put_string ("eif_plug.h")
								l_external_file.put_character ('"')
								l_external_file.put_new_line
							end
						end
						if l_external_file.is_open_write then
							l_filename := file_system.pathname (a_clib_dirname, l_filename)
							l_filename := STRING_.replaced_all_substrings (l_filename, "\", "\\")
							l_external_file.put_string (c_include)
							l_external_file.put_character (' ')
							l_external_file.put_character ('"')
							l_external_file.put_string (l_filename)
							l_external_file.put_character ('"')
							l_external_file.put_new_line
						end
					end
					l_dir.read_entry
				end
				l_dir.close
				if l_c_file /= Void and then l_c_file.is_open_write then
					l_c_file.close
				end
				if l_cpp_file /= Void and then l_cpp_file.is_open_write then
					l_cpp_file.close
				end
			end
		end

feature {NONE} -- C code Generation

	generate_c_code (a_system_name: STRING)
			-- Generate C code for `current_dynamic_system'.
			-- Set `has_fatal_error' if a fatal error occurred.
		require
			a_system_name_not_void: a_system_name /= Void
		local
			old_system_name: STRING
			old_file: KI_TEXT_OUTPUT_STREAM
			old_header_file: KI_TEXT_OUTPUT_STREAM
			l_header_filename: STRING
			l_header_file: KL_TEXT_OUTPUT_FILE
			l_dynamic_feature: ET_DYNAMIC_FEATURE
			old_in_static_feature: BOOLEAN
		do
			old_system_name := system_name
			system_name := a_system_name
			l_header_filename := a_system_name + h_file_extension
			create l_header_file.make (l_header_filename)
			l_header_file.open_write
			if not l_header_file.is_open_write then
				set_fatal_error
				report_cannot_write_error (l_header_filename)
			else
				open_c_file
				open_cpp_file
				old_header_file := header_file
				header_file := l_header_file
				old_file := current_file
				current_file := current_function_body_buffer
				generate_ids
				if trace_mode then
					header_file.put_string (c_define)
					header_file.put_character (' ')
					header_file.put_line (c_eif_trace)
					header_file.put_new_line
				end
				if exception_trace_mode then
					header_file.put_string (c_define)
					header_file.put_character (' ')
					header_file.put_line (c_eif_exception_trace)
					header_file.put_new_line
				end
				if use_boehm_gc then
					header_file.put_string (c_define)
					header_file.put_character (' ')
					header_file.put_line (c_ge_use_boehm_gc)
					header_file.put_new_line
				end
				if multithreaded_mode then
					header_file.put_string (c_define)
					header_file.put_character (' ')
					header_file.put_line (c_ge_use_threads)
					header_file.put_new_line
				end
				if current_system.any_type.base_class.universe.character_type.same_named_type_with_type_marks (current_system.any_type.base_class.universe.character_32_type, tokens.implicit_detachable_type_mark, current_system.any_type, tokens.implicit_detachable_type_mark, current_system.any_type) then
					header_file.put_string (c_define)
					header_file.put_character (' ')
					header_file.put_line (c_eif_character)
					header_file.put_character (' ')
					header_file.put_line (c_eif_character_32)
					header_file.put_new_line
				end
				include_runtime_header_file ("ge_eiffel.h", True, header_file)
				header_file.put_new_line
				if multithreaded_mode then
					include_runtime_header_file ("ge_thread_types.h", True, header_file)
					header_file.put_new_line
				end
				include_runtime_header_file ("ge_once.h", True, header_file)
				header_file.put_new_line
				include_runtime_header_file ("ge_exception.h", True, header_file)
				header_file.put_new_line
				if multithreaded_mode then
					include_runtime_header_file ("ge_thread.h", True, header_file)
					header_file.put_new_line
				end
				include_runtime_header_file ("ge_native_string.h", True, header_file)
				header_file.put_new_line
				include_runtime_header_file ("ge_arguments.h", True, header_file)
				header_file.put_new_line

				include_runtime_header_file ("ge_types.h", True, header_file)
				header_file.put_new_line
				include_runtime_header_file ("ge_string.h", True, header_file)
				header_file.put_new_line
				include_runtime_header_file ("ge_console.h", True, header_file)
				header_file.put_new_line
				include_runtime_header_file ("ge_main.h", True, header_file)
				header_file.put_new_line
				include_runtime_header_file ("ge_gc.h", True, header_file)
				header_file.put_new_line
				include_runtime_header_file ("ge_identified.h", True, header_file)
				header_file.put_new_line
					-- Two header files needed to compile EiffelCOM.
				include_runtime_header_file ("eif_cecil.h", False, header_file)
				include_runtime_header_file ("eif_plug.h", False, header_file)
				print_start_extern_c (header_file)
				print_types (header_file)
				flush_to_c_file
				header_file.put_new_line
				print_default_declarations
				current_file.put_new_line
				flush_to_c_file
				header_file.put_new_line
				print_new_string_8_function
				current_file.put_new_line
				flush_to_c_file
				print_new_string_32_function
				current_file.put_new_line
				flush_to_c_file
				print_new_immutable_string_8_function
				current_file.put_new_line
				flush_to_c_file
				print_new_immutable_string_32_function
				current_file.put_new_line
				flush_to_c_file
					-- Print object allocation functions.
				print_object_allocation_functions
				print_once_per_object_data_allocation_functions
				print_once_per_object_data_dispose_functions
					-- Print Eiffel feature functions.
				if attached current_dynamic_system.root_creation_procedure as l_root_procedure then
					if attached current_dynamic_system.ise_exception_manager_init_exception_manager_feature as l_ise_exception_manager_init_exception_manager_feature then
						register_called_feature (l_ise_exception_manager_init_exception_manager_feature)
					end
					if attached current_dynamic_system.ise_exception_manager_last_exception_feature as l_ise_exception_manager_last_exception_feature then
						register_called_feature (l_ise_exception_manager_last_exception_feature)
					end
					if attached current_dynamic_system.ise_exception_manager_once_raise_feature as l_ise_exception_manager_once_raise_feature then
						register_called_feature (l_ise_exception_manager_once_raise_feature)
					end
					if attached current_dynamic_system.ise_exception_manager_set_exception_data_feature as l_ise_exception_manager_set_exception_data_feature then
						register_called_feature (l_ise_exception_manager_set_exception_data_feature)
					end
					register_called_feature (l_root_procedure)
					from until
						called_features.is_empty and
						called_static_features.is_empty
					loop
						from until called_features.is_empty loop
							l_dynamic_feature := called_features.last
							called_features.remove_last
							print_feature (l_dynamic_feature)
						end
						from until called_static_features.is_empty loop
							l_dynamic_feature := called_static_features.last
							called_static_features.remove_last
							old_in_static_feature := in_static_feature
							in_static_feature := True
							print_feature (l_dynamic_feature)
							in_static_feature := old_in_static_feature
						end
					end
				end
					-- Print polymorphic call functions.
				print_polymorphic_query_call_functions
				print_polymorphic_tuple_label_call_functions
				print_polymorphic_procedure_call_functions
					-- Print features which build manifest arrays.
				from manifest_array_types.start until manifest_array_types.after loop
					print_manifest_array_function (manifest_array_types.item_for_iteration)
					current_file.put_new_line
					flush_to_c_file
					manifest_array_types.forth
				end
				manifest_array_types.wipe_out
					-- Print features which build big manifest arrays.
				from big_manifest_array_types.start until big_manifest_array_types.after loop
					print_big_manifest_array_function (big_manifest_array_types.item_for_iteration)
					current_file.put_new_line
					flush_to_c_file
					big_manifest_array_types.forth
				end
				big_manifest_array_types.wipe_out
					-- Print features which build manifest tuples.
				from manifest_tuple_types.start until manifest_tuple_types.after loop
					print_manifest_tuple_function (manifest_tuple_types.item_for_iteration)
					current_file.put_new_line
					flush_to_c_file
					manifest_tuple_types.forth
				end
				manifest_tuple_types.wipe_out
					-- Print all functions necessary to implement 'deep_twin'.
				print_deep_twin_functions
					-- Print constants declarations.
				print_constants_declaration
				current_file.put_new_line
				flush_to_c_file
				print_init_const_function
				current_file.put_new_line
				flush_to_c_file
					-- Print 'GE_dts' dynamic type id set constants.
				print_dynamic_type_id_set_constants
				flush_to_c_file
					-- Print 'GE_types' array.
				print_types_array
				current_file.put_new_line
				flush_to_c_file
					-- Print 'main' function.
				print_main_function
				current_file.put_new_line
				flush_to_c_file
				print_end_extern_c (header_file)
				header_file.put_new_line
					-- Include runtime C files.
				from included_runtime_c_files.start until included_runtime_c_files.after loop
					print_end_extern_c (current_file)
					include_runtime_file (included_runtime_c_files.item_for_iteration, current_file)
					print_start_extern_c (current_file)
					flush_to_c_file
					included_runtime_c_files.forth
				end
					-- Include runtime header files.
				header_file.put_new_line
				from included_runtime_header_files.start until included_runtime_header_files.after loop
					if not included_runtime_header_files.item_for_iteration then
						include_runtime_file (included_runtime_header_files.key_for_iteration, header_file)
					end
					included_runtime_header_files.forth
				end
					-- Include C++ header files.
				if not included_cpp_header_filenames.is_empty then
					header_file.put_string (c_ifdef)
					header_file.put_character (' ')
					header_file.put_line (c_cplusplus)
					from included_cpp_header_filenames.start until included_cpp_header_filenames.after loop
						header_file.put_string (c_include)
						header_file.put_character (' ')
						header_file.put_string (included_cpp_header_filenames.item_for_iteration)
						header_file.put_new_line
						included_cpp_header_filenames.forth
					end
					header_file.put_line (c_endif)
				end
					-- Include header files.
				from included_header_filenames.start until included_header_filenames.after loop
					header_file.put_string (c_include)
					header_file.put_character (' ')
					header_file.put_string (included_header_filenames.item_for_iteration)
					header_file.put_new_line
					included_header_filenames.forth
				end
				included_header_filenames.wipe_out
				included_runtime_header_files.wipe_out
				included_runtime_c_files.wipe_out
				once_features.wipe_out
				once_per_process_counts.fill_with (0)
				once_per_thread_counts.fill_with (0)
				constant_features.wipe_out
				inline_constants.wipe_out
				dispose_procedures.wipe_out
				dynamic_types := current_dynamic_system.dynamic_types
				dynamic_type_id_set_names.wipe_out
				l_header_file.close
				if
					type_info_ancestors_used or
					type_info_attributes_used or
					type_info_attribute_name_used or
					type_info_attribute_type_id_used or
					type_info_attribute_offset_used or
					type_info_generator_used or
					type_info_name_used or
					type_info_generic_parameters_used or
					type_info_object_size_used
				then
					file_system.rename_file (l_header_filename, l_header_filename + "ge")
					l_header_file.open_write
					if not l_header_file.is_open_write then
						set_fatal_error
						report_cannot_write_error (l_header_filename)
					else
						if type_info_ancestors_used then
							header_file.put_string (c_define)
							header_file.put_character (' ')
							header_file.put_line (c_ge_use_ancestors)
						end
						if type_info_attributes_used then
							header_file.put_string (c_define)
							header_file.put_character (' ')
							header_file.put_line (c_ge_use_attributes)
						end
						if type_info_attribute_name_used then
							header_file.put_string (c_define)
							header_file.put_character (' ')
							header_file.put_line (c_ge_use_attribute_name)
						end
						if type_info_attribute_type_id_used then
							header_file.put_string (c_define)
							header_file.put_character (' ')
							header_file.put_line (c_ge_use_attribute_type_id)
						end
						if type_info_attribute_offset_used then
							header_file.put_string (c_define)
							header_file.put_character (' ')
							header_file.put_line (c_ge_use_attribute_offset)
						end
						if type_info_generator_used then
							header_file.put_string (c_define)
							header_file.put_character (' ')
							header_file.put_line (c_ge_use_type_generator)
						end
						if type_info_name_used then
							header_file.put_string (c_define)
							header_file.put_character (' ')
							header_file.put_line (c_ge_use_type_name)
						end
						if type_info_generic_parameters_used then
							header_file.put_string (c_define)
							header_file.put_character (' ')
							header_file.put_line (c_ge_use_type_generic_parameters)
						end
						if type_info_object_size_used then
							header_file.put_string (c_define)
							header_file.put_character (' ')
							header_file.put_line (c_ge_use_type_object_size)
						end
						include_file (l_header_filename + "ge", header_file)
						l_header_file.close
						file_system.delete_file (l_header_filename + "ge")
					end
					type_info_ancestors_used := False
					type_info_attributes_used := False
					type_info_attribute_name_used := False
					type_info_attribute_type_id_used := False
					type_info_attribute_offset_used := False
					type_info_generator_used := False
					type_info_name_used := False
					type_info_generic_parameters_used := False
					type_info_object_size_used := False
				end
				close_c_file
				close_cpp_file
				header_file := old_header_file
				current_file := old_file
				current_type := current_dynamic_system.unknown_type
				current_feature := dummy_feature
				current_dynamic_type_sets := extra_dynamic_type_sets
			end
			system_name := old_system_name
		end

	generate_ids
			-- Generate types and feature ids.
		local
			l_dynamic_types: DS_ARRAYED_LIST [ET_DYNAMIC_PRIMARY_TYPE]
			l_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
			l_features: ET_DYNAMIC_FEATURE_LIST
			l_feature: ET_DYNAMIC_FEATURE
			j, nb2: INTEGER
			k, nb3: INTEGER
			l_count: INTEGER
			l_id: INTEGER
		do
			if current_dynamic_system.ise_runtime_type_conforms_to_feature /= Void then
				sort_types
			end
			l_dynamic_types := dynamic_types
			nb := l_dynamic_types.count
			from i := 1 until i > nb loop
				l_type := l_dynamic_types.item (i)
				l_count := l_count + 1
				l_type.set_id (l_count)
				if l_type.is_alive or l_type.has_static then
						-- Process dynamic queries.
					l_features := l_type.queries
					nb2 := l_features.count
					from j := 1 until j > nb2 loop
						l_feature := l_features.item (j)
						l_feature.set_id (j)
						if attached l_feature.first_precursor as l_first_precursor then
							l_first_precursor.set_id (1)
							if attached l_feature.other_precursors as l_other_precursors then
								nb3 := l_other_precursors.count
								from k := 2 until k > nb3 loop
									l_other_precursors.item (k).set_id (k)
									k := k + 1
								end
							end
						end
						j := j + 1
					end
						-- Process dynamic procedures.
					l_id := nb2 + 1
					l_features := l_type.procedures
					nb2 := l_features.count
					from j := 1 until j > nb2 loop
						l_feature := l_features.item (j)
						l_feature.set_id (l_id)
						if attached l_feature.first_precursor as l_first_precursor then
							l_first_precursor.set_id (1)
							if attached l_feature.other_precursors as l_other_precursors then
								nb3 := l_other_precursors.count
								from k := 2 until k > nb3 loop
									l_other_precursors.item (k).set_id (k)
									k := k + 1
								end
							end
						end
						l_id := l_id + 1
						j := j + 1
					end
				end
				i := i + 1
			end
		end

feature {NONE} -- Feature generation

	print_features (a_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print features of `a_type' to `current_file' and its signature to `header_file'.
		require
			a_type_not_void: a_type /= Void
		local
			old_type: like current_type
			l_features: ET_DYNAMIC_FEATURE_LIST
			i, nb: INTEGER
		do
			old_type := current_type
			current_type := a_type
			l_features := a_type.queries
			nb := l_features.count
			from i := 1 until i > nb loop
				print_feature (l_features.item (i))
				i := i + 1
			end
			l_features := a_type.procedures
			nb := l_features.count
			from i := 1 until i > nb loop
				print_feature (l_features.item (i))
				i := i + 1
			end
			current_type := old_type
		end

	print_feature (a_feature: ET_DYNAMIC_FEATURE)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
			-- Also print the precursors used in its body (calls to 'precursor').
		require
			a_feature_not_void: a_feature /= Void
		local
			old_type: like current_type
			old_feature: like current_feature
			old_dynamic_type_sets: like current_dynamic_type_sets
			i, nb: INTEGER
			l_equality_types: ET_DYNAMIC_EQUALITY_TYPES
		do
			if not a_feature.is_semistrict (current_dynamic_system) then
				old_type := current_type
				current_type := a_feature.target_type
				old_feature := current_feature
				current_feature := a_feature
				old_dynamic_type_sets := current_dynamic_type_sets
				current_dynamic_type_sets := current_feature.dynamic_type_sets
				a_feature.static_feature.process (Current)
					-- Print declaration of agents used in `a_feature'.
					-- Note that we compute `current_agents.count' at each iteration
					-- because inline agents may contain other inline agents
					-- which will be added to `current_agents' on the fly.
				from i := 1 until i > current_agents.count loop
					print_agent_declaration (i, current_agents.item (i))
					i := i + 1
				end
				current_agents.wipe_out
					-- Print object-test functions.
				nb := current_object_tests.count
				from i := 1 until i > nb loop
					print_object_test_function (i, current_object_tests.item (i))
					i := i + 1
				end
				current_object_tests.wipe_out
					-- Print equality functions.
				nb := current_equalities.count
				from i := 1 until i > nb loop
					l_equality_types := current_equalities.item (i)
					print_equality_function (i, l_equality_types)
					if attached {ET_DYNAMIC_OBJECT_EQUALITY_TYPES} l_equality_types as l_object_equality_types then
						free_object_equality_types (l_object_equality_types)
					else
						free_equality_types (l_equality_types)
					end
					i := i + 1
				end
				current_equalities.wipe_out
				current_dynamic_type_sets := old_dynamic_type_sets
				current_feature := old_feature
				current_type := old_type
			end
		end

	print_address_routine (a_feature: ET_ROUTINE)
			-- Print wrapper of `a_feature' used to pass its address to `current_file' and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
			is_address: current_feature.is_address
		local
			l_result_type_set: detachable ET_DYNAMIC_TYPE_SET
			l_result_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_type: ET_DYNAMIC_PRIMARY_TYPE
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			i, nb_args: INTEGER
			old_file: KI_TEXT_OUTPUT_STREAM
			l_name: ET_IDENTIFIER
		do
			old_file := current_file
			current_file := current_function_header_buffer
			print_feature_name_comment (a_feature, current_type, header_file)
			print_feature_name_comment (a_feature, current_type, current_file)
			l_result_type_set := current_feature.result_type_set
			if l_result_type_set /= Void then
				l_result_type := l_result_type_set.static_type.primary_type
			end
				--
				-- Print signature to `header_file' and `current_file'.
				--
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			if l_result_type /= Void then
				print_type_declaration (l_result_type, header_file)
				print_type_declaration (l_result_type, current_file)
			else
				header_file.put_string (c_void)
				current_file.put_string (c_void)
			end
			header_file.put_character (' ')
			current_file.put_character (' ')
			print_address_routine_name (current_feature, current_type, header_file)
			print_address_routine_name (current_feature, current_type, current_file)
			header_file.put_character ('(')
			current_file.put_character ('(')
			print_type_declaration (current_type, header_file)
			print_type_declaration (current_type, current_file)
			if current_type.is_expanded then
				header_file.put_character ('*')
				current_file.put_character ('*')
			end
			header_file.put_character (' ')
			current_file.put_character (' ')
			print_current_name (header_file)
			print_current_name (current_file)
			l_arguments := a_feature.arguments
			if l_arguments /= Void then
				nb_args := l_arguments.count
				if nb_args > 0 then
					from i := 1 until i > nb_args loop
						header_file.put_character (',')
						current_file.put_character (',')
						header_file.put_character (' ')
						current_file.put_character (' ')
						l_argument_type_set := argument_type_set (i)
						l_argument_type := l_argument_type_set.static_type.primary_type
						print_type_declaration (l_argument_type, header_file)
						print_type_declaration (l_argument_type, current_file)
						header_file.put_character (' ')
						current_file.put_character (' ')
						l_name := l_arguments.formal_argument (i).name
						print_argument_name (l_name, header_file)
						print_argument_name (l_name, current_file)
						i := i + 1
					end
				end
			end
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_indentation
			if l_result_type /= Void then
				current_file.put_string (c_return)
				current_file.put_character (' ')
			end
			print_routine_name (current_feature, current_type, current_file)
			current_file.put_character ('(')
			current_file.put_string (c_ge_current_context)
			current_file.put_character ('(')
			current_file.put_character (')')
			current_file.put_character (',')
			current_file.put_character (' ')
			print_current_name (current_file)
			if l_arguments /= Void and then nb_args > 0 then
				from i := 1 until i > nb_args loop
					current_file.put_character (',')
					current_file.put_character (' ')
					l_name := l_arguments.formal_argument (i).name
					print_argument_name (l_name, current_file)
					i := i + 1
				end
			end
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
				--
				-- Clean up.
				--
			current_file := old_file
				-- Flush to file.
			flush_to_c_file
		end

	print_constants_declaration
			-- Print declarations of constant attributes
			-- to `header_file' and `current_file'.
		local
			l_feature: ET_FEATURE
			l_context: ET_TYPE_CONTEXT
			l_constant_type: ET_DYNAMIC_PRIMARY_TYPE
			l_constant: ET_INLINE_CONSTANT
		do
			from constant_features.start until constant_features.after loop
				l_feature := constant_features.key_for_iteration
				if not once_features.has (l_feature) then
					if not attached l_feature.type as l_type then
							-- Internal feature: a constant attribute has a type.
						set_fatal_error
						error_handler.report_giaaa_error
					else
							-- The deanchored form of the type of the constant feature
							-- should not contain any formal generic parameter (in fact
							-- it should be one of BOOLEAN, CHARACTER possibly sized,
							-- INTEGER possibly sized, NATURAL possible sized, REAL possibly
							-- sized, STRING possibly sized, or 'TYPE [X]' where X is a
							-- stand-alone type), therefore it is OK to use the class where
							-- this feature has been written as context.
						l_context := l_feature.implementation_class
						l_constant_type := current_dynamic_system.dynamic_primary_type (l_type, l_context)
						header_file.put_string (c_extern)
						header_file.put_character (' ')
						print_type_declaration (l_constant_type, header_file)
						print_type_declaration (l_constant_type, current_file)
						header_file.put_character (' ')
						current_file.put_character (' ')
						print_once_value_name (l_feature, header_file)
						print_once_value_name (l_feature, current_file)
						header_file.put_character (';')
						current_file.put_character (';')
						header_file.put_new_line
						current_file.put_new_line
					end
				end
				constant_features.forth
			end
			from inline_constants.start until inline_constants.after loop
				l_constant := inline_constants.key_for_iteration
				l_constant_type := inline_constants.item_for_iteration
				header_file.put_string (c_extern)
				header_file.put_character (' ')
				print_type_declaration (l_constant_type, header_file)
				print_type_declaration (l_constant_type, current_file)
				header_file.put_character (' ')
				current_file.put_character (' ')
				print_inline_constant_name (l_constant, header_file)
				print_inline_constant_name (l_constant, current_file)
				header_file.put_character (';')
				current_file.put_character (';')
				header_file.put_new_line
				current_file.put_new_line
				inline_constants.forth
			end
		end

	print_deferred_function (a_feature: ET_DEFERRED_FUNCTION)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
				-- Internal error: deferred features cannot be executed at run-time.
			set_fatal_error
			error_handler.report_giaaa_error
		end

	print_deferred_procedure (a_feature: ET_DEFERRED_PROCEDURE)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
				-- Internal error: deferred features cannot be executed at run-time.
			set_fatal_error
			error_handler.report_giaaa_error
		end

	print_do_function (a_feature: ET_DO_FUNCTION)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
			print_internal_function (a_feature)
		end

	print_do_procedure (a_feature: ET_DO_PROCEDURE)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
			print_internal_procedure (a_feature)
		end

	print_external_function (a_feature: ET_EXTERNAL_FUNCTION)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
			if current_feature.static_feature /= a_feature then
					-- Internal error: inconsistent `current_feature'.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif in_static_feature then
				print_external_routine (a_feature, False, False)
			else
				if current_feature.is_regular then
					print_external_routine (a_feature, False, False)
				end
				if current_feature.is_address then
					print_address_routine (a_feature)
				end
			end
		end

	print_external_procedure (a_feature: ET_EXTERNAL_PROCEDURE)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
			if current_feature.static_feature /= a_feature then
					-- Internal error: inconsistent `current_feature'.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif in_static_feature then
				print_external_routine (a_feature, False, False)
			else
				if current_feature.is_regular then
					print_external_routine (a_feature, False, False)
				end
				if current_feature.is_creation then
					print_external_routine (a_feature, True, False)
				end
				if current_feature.is_address then
					print_address_routine (a_feature)
				end
			end
		end

	print_external_routine (a_feature: ET_EXTERNAL_ROUTINE; a_creation: BOOLEAN; a_inline: BOOLEAN)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
			-- `a_inline' means that `a_feature' is a C inline or C++ inline feature and
			-- we want to generate its code in a separate function.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
			is_creation: a_creation implies current_feature.is_creation
			one_kind: (not in_static_feature and not a_creation and not a_inline) or (in_static_feature xor a_creation xor a_inline)
		local
			l_result_type_set: detachable ET_DYNAMIC_TYPE_SET
			l_result_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_type: ET_DYNAMIC_PRIMARY_TYPE
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_name: ET_IDENTIFIER
			i, nb_args: INTEGER
			l_language: ET_EXTERNAL_LANGUAGE
			l_language_value: ET_MANIFEST_STRING
			l_language_string: STRING
			l_signature_arguments: detachable STRING
			l_signature_result: detachable STRING
			l_alias_value: ET_MANIFEST_STRING
			l_splitter: ST_SPLITTER
			l_list: DS_LIST [STRING]
			l_struct_field_name: STRING
			l_struct_field_type: detachable STRING
			l_struct_type: STRING
			old_file: KI_TEXT_OUTPUT_STREAM
			l_is_inline: BOOLEAN
			l_is_cpp: BOOLEAN
			l_cpp_class_type: STRING
			l_dll_file: STRING
			l_has_include_files: BOOLEAN
		do
			old_file := current_file
			current_file := current_function_header_buffer
				--
				-- Print signature to `header_file' and `current_file'.
				--
			print_feature_name_comment (a_feature, current_type, header_file)
			print_feature_name_comment (a_feature, current_type, current_file)
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			l_result_type_set := current_feature.result_type_set
			if l_result_type_set /= Void then
				l_result_type := l_result_type_set.static_type.primary_type
				print_type_declaration (l_result_type, header_file)
				print_type_declaration (l_result_type, current_file)
			elseif a_creation then
				print_type_declaration (current_type, header_file)
				print_type_declaration (current_type, current_file)
			else
				header_file.put_string (c_void)
				current_file.put_string (c_void)
			end
			header_file.put_character (' ')
			current_file.put_character (' ')
			if a_inline then
				print_inline_routine_name (current_feature, current_type, header_file)
				print_inline_routine_name (current_feature, current_type, current_file)
				header_file.put_character ('(')
				current_file.put_character ('(')
				header_file.put_string (c_ge_context)
				header_file.put_character ('*')
				header_file.put_character (' ')
				header_file.put_string (c_ac)
				current_file.put_string (c_ge_context)
				current_file.put_character ('*')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
			elseif in_static_feature then
				print_static_routine_name (current_feature, current_type, header_file)
				print_static_routine_name (current_feature, current_type, current_file)
				header_file.put_character ('(')
				current_file.put_character ('(')
				header_file.put_string (c_ge_context)
				header_file.put_character ('*')
				header_file.put_character (' ')
				header_file.put_string (c_ac)
				current_file.put_string (c_ge_context)
				current_file.put_character ('*')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
			elseif a_creation then
				print_creation_procedure_name (current_feature, current_type, header_file)
				print_creation_procedure_name (current_feature, current_type, current_file)
				header_file.put_character ('(')
				current_file.put_character ('(')
				header_file.put_string (c_ge_context)
				header_file.put_character ('*')
				header_file.put_character (' ')
				header_file.put_string (c_ac)
				current_file.put_string (c_ge_context)
				current_file.put_character ('*')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
			else
				print_routine_name (current_feature, current_type, header_file)
				print_routine_name (current_feature, current_type, current_file)
				header_file.put_character ('(')
				current_file.put_character ('(')
				header_file.put_string (c_ge_context)
				header_file.put_character ('*')
				header_file.put_character (' ')
				header_file.put_string (c_ac)
				header_file.put_character (',')
				header_file.put_character (' ')
				current_file.put_string (c_ge_context)
				current_file.put_character ('*')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_type_declaration (current_type, header_file)
				print_type_declaration (current_type, current_file)
				if current_type.is_expanded then
					header_file.put_character ('*')
					current_file.put_character ('*')
				end
				header_file.put_character (' ')
				current_file.put_character (' ')
				print_current_name (header_file)
				print_current_name (current_file)
			end
			l_arguments := a_feature.arguments
			if l_arguments /= Void then
				nb_args := l_arguments.count
				if nb_args > 0 then
					from i := 1 until i > nb_args loop
						header_file.put_character (',')
						current_file.put_character (',')
						header_file.put_character (' ')
						current_file.put_character (' ')
						l_argument_type_set := argument_type_set (i)
						l_argument_type := l_argument_type_set.static_type.primary_type
						print_type_declaration (l_argument_type, header_file)
						print_type_declaration (l_argument_type, current_file)
						header_file.put_character (' ')
						current_file.put_character (' ')
						l_name := l_arguments.formal_argument (i).name
						print_argument_name (l_name, header_file)
						print_argument_name (l_name, current_file)
						i := i + 1
					end
				end
			end
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
				--
				-- Print body to `current_file'.
				--
			current_file.put_character ('{')
			current_file.put_new_line
			indent
				-- Determine the kind of external.
			l_language := a_feature.language
			l_language_value := l_language.manifest_string
			l_language_string := l_language_value.value
			if external_c_inline_regexp.recognizes (l_language_string) then
				l_is_inline := True
			elseif external_cpp_inline_regexp.recognizes (l_language_string) then
				l_is_inline := True
				l_is_cpp := True
			end
				--
				-- Declaration of variables.
				--
				-- Variable for exception trace.
			if exception_trace_mode and not a_inline then
				print_indentation
				current_file.put_string (c_ge_call)
				current_file.put_character (' ')
				current_file.put_string (c_tc)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('{')
				if current_in_exception_trace then
					if in_static_feature or a_creation then
						current_file.put_character ('0')
					else
						print_current_name (current_file)
					end
					current_file.put_character (',')
				end
				print_escaped_string (current_type.base_class.upper_name)
				current_file.put_character (',')
				print_escaped_string (a_feature.lower_name)
				current_file.put_character (',')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_call)
				current_file.put_character ('}')
				current_file.put_character (';')
				current_file.put_new_line
			end
				-- Variable for 'Result' entity.
			if l_result_type /= Void and (not l_is_inline or else ((exception_trace_mode or trace_mode) and not a_inline)) then
				print_indentation
				print_type_declaration (l_result_type, current_file)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_default_entity_value (l_result_type, current_file)
				current_file.put_character (';')
				current_file.put_new_line
			end
				--
				-- Instructions.
				--
			current_file := current_function_body_buffer
			if not a_inline then
					-- Call stack.
				if exception_trace_mode and not a_inline then
					print_indentation
					current_file.put_string (c_ac)
					current_file.put_string (c_arrow)
					current_file.put_string (c_call)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_tc_address)
					current_file.put_character (';')
					current_file.put_new_line
				end
				print_feature_trace_message_call (True)
			end
			if l_result_type_set /= Void and then l_result_type_set.is_empty then
-- TODO: build full dynamic type sets, recursively.
				print_feature_info_message_call ("Dynamic type set not built for external feature ")
			end
			if a_creation then
				print_malloc_current (a_feature)
				if exception_trace_mode and then current_in_exception_trace then
					print_indentation
					current_file.put_string (c_tc)
					current_file.put_character ('.')
					current_file.put_string (c_object)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_current_name (current_file)
					current_file.put_character (';')
					current_file.put_new_line
				end
			end
				-- Put the body of the feature in a block so that there
				-- is no C compilation error if some variables are
				-- declared after the instructions above.
			print_indentation
			current_file.put_character ('{')
			current_file.put_new_line
			if l_is_inline then
				if l_is_cpp then
						-- Regexp: C++ [blocking] inline [use {<include> "," ...}+]
						-- \3: include files
					if external_cpp_inline_regexp.match_count > 3 and then external_cpp_inline_regexp.captured_substring_count (3) > 0 then
						print_external_cpp_includes (external_cpp_inline_regexp.captured_substring (3))
					end
				else
						-- Regexp: C [blocking] inline [use {<include> "," ...}+]
						-- \3: include files
					if external_c_inline_regexp.match_count > 3 and then external_c_inline_regexp.captured_substring_count (3) > 0 then
						print_external_c_includes (external_c_inline_regexp.captured_substring (3))
					end
				end
				if (exception_trace_mode or trace_mode) and not a_inline then
						-- The inline C code may contain 'return' statement.
						-- So we need to enclose its code in a separate C function.
					if l_result_type_set /= Void then
						print_indentation
						print_result_name (current_file)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
					end
					print_inline_routine_name (current_feature, current_type, current_file)
					current_file.put_character ('(')
					current_file.put_string (c_ac)
					if l_arguments /= Void then
						from i := 1 until i > nb_args loop
							current_file.put_character (',')
							current_file.put_character (' ')
							l_name := l_arguments.formal_argument (i).name
							print_argument_name (l_name, current_file)
							i := i + 1
						end
					end
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				else
					print_external_c_inline_body (a_feature)
				end
			elseif a_feature.is_builtin then
				print_external_builtin_body (a_feature)
			elseif external_c_regexp.recognizes (l_language_string) then
					-- Regexp: C [blocking] [signature ["(" {<type> "," ...}* ")"] [":" <type>]] [use {<include> "," ...}+]
					-- \5: has signature arguments
					-- \6: signature arguments
					-- \11: signature result
					-- \18: include files
				if external_c_regexp.match_count > 18 and then external_c_regexp.captured_substring_count (18) > 0 then
					l_has_include_files := True
					print_external_c_includes (external_c_regexp.captured_substring (18))
				end
				if external_c_regexp.match_count > 5 and then external_c_regexp.captured_substring_count (5) > 0 then
					l_signature_arguments := external_c_regexp.captured_substring (6)
				end
				if external_c_regexp.match_count > 11 and then external_c_regexp.captured_substring_count (11) > 0 then
					l_signature_result := external_c_regexp.captured_substring (11)
				end
				if not l_has_include_files then
						-- We need to generate a prototype for this external routine.
					print_external_c_prototype (a_feature.implementation_feature.name, l_arguments, l_result_type_set, l_signature_arguments, l_signature_result, a_feature.alias_clause)
				end
				print_external_c_body (a_feature.implementation_feature.name, l_arguments, l_result_type_set, l_signature_arguments, l_signature_result, a_feature.alias_clause, False)
			elseif external_c_macro_regexp.recognizes (l_language_string) then
					-- Regexp: C [blocking] macro [signature ["(" {<type> "," ...}* ")"] [":" <type>]] use {<include> "," ...}+
					-- \4: has signature arguments
					-- \5: signature arguments
					-- \10: signature result
					-- \17: include files
				print_external_c_includes (external_c_macro_regexp.captured_substring (17))
				if external_c_macro_regexp.match_count > 4 and then external_c_macro_regexp.captured_substring_count (4) > 0 then
					l_signature_arguments := external_c_macro_regexp.captured_substring (5)
				end
				if external_c_macro_regexp.match_count > 10 and then external_c_macro_regexp.captured_substring_count (10) > 0 then
					l_signature_result := external_c_macro_regexp.captured_substring (10)
				end
				print_external_c_body (a_feature.implementation_feature.name, l_arguments, l_result_type_set, l_signature_arguments, l_signature_result, a_feature.alias_clause, True)
			elseif external_c_struct_regexp.recognizes (l_language_string) then
					-- Regexp: C struct <struct-type> (access|get) <field-name> [type <field-type>] use {<include> "," ...}+
					-- \1: struct type
					-- \6: field name
					-- \9: field type
					-- \16: include files
				print_external_c_includes (external_c_struct_regexp.captured_substring (16))
				l_struct_type := external_c_struct_regexp.captured_substring (1)
				l_struct_field_name := external_c_struct_regexp.captured_substring (6)
				if external_c_struct_regexp.match_count > 9 and then external_c_struct_regexp.captured_substring_count (9) > 0 then
					l_struct_field_type := external_c_struct_regexp.captured_substring (9)
				end
				print_external_c_struct_body (l_arguments, l_result_type_set, l_struct_type, l_struct_field_name, l_struct_field_type)
			elseif old_external_c_regexp.recognizes (l_language_string) then
					-- Regexp: C [["(" {<type> "," ...}* ")"] [":" <type>]] ["|" {<include> "," ...}+]
					-- \1: has signature
					-- \2: has signature arguments
					-- \3: signature arguments
					-- \5: signature result
					-- \7: include files
				if old_external_c_regexp.match_count > 7 and then old_external_c_regexp.captured_substring_count (7) > 0 then
					l_has_include_files := True
					print_external_c_includes (old_external_c_regexp.captured_substring (7))
				end
				if old_external_c_regexp.match_count > 1 and then old_external_c_regexp.captured_substring_count (1) > 0 then
					if old_external_c_regexp.match_count > 2 and then old_external_c_regexp.captured_substring_count (2) > 0 then
						l_signature_arguments := old_external_c_regexp.captured_substring (3)
					end
					if old_external_c_regexp.match_count > 5 and then old_external_c_regexp.captured_substring_count (5) > 0 then
						l_signature_result := old_external_c_regexp.captured_substring (5)
					end
				end
				if not l_has_include_files then
						-- We need to generate a prototype for this external routine.
					print_external_c_prototype (a_feature.implementation_feature.name, l_arguments, l_result_type_set, l_signature_arguments, l_signature_result, a_feature.alias_clause)
				end
				print_external_c_body (a_feature.implementation_feature.name, l_arguments, l_result_type_set, l_signature_arguments, l_signature_result, a_feature.alias_clause, False)
			elseif old_external_c_macro_regexp.recognizes (l_language_string) then
					-- Regexp: C "[" macro <include> "]" ["(" {<type> "," ...}* ")"] [":" <type>]
					-- \1: include file
					-- \2: has signature arguments
					-- \3: signature arguments
					-- \5: signature result
				print_external_c_includes (old_external_c_macro_regexp.captured_substring (1))
				if old_external_c_macro_regexp.match_count > 2 and then old_external_c_macro_regexp.captured_substring_count (2) > 0 then
					l_signature_arguments := old_external_c_macro_regexp.captured_substring (3)
					if old_external_c_macro_regexp.match_count > 5 and then old_external_c_macro_regexp.captured_substring_count (5) > 0 then
						l_signature_result := old_external_c_macro_regexp.captured_substring (5)
					end
				end
				print_external_c_body (a_feature.implementation_feature.name, l_arguments, l_result_type_set, l_signature_arguments, l_signature_result, a_feature.alias_clause, True)
			elseif old_external_c_struct_regexp.recognizes (l_language_string) then
					-- Regexp: C "[" struct <include> "]" "(" {<type> "," ...}+ ")" [":" <type>]
					-- \1: include file
					-- \2: signature arguments
					-- \4: signature result
				print_external_c_includes (old_external_c_struct_regexp.captured_substring (1))
				l_signature_arguments := old_external_c_struct_regexp.captured_substring (2)
				if old_external_c_struct_regexp.match_count > 4 and then old_external_c_struct_regexp.captured_substring_count (4) > 0 then
					l_signature_result := old_external_c_struct_regexp.captured_substring (4)
				end
				if attached a_feature.alias_clause as l_alias then
					l_alias_value := l_alias.manifest_string
					l_struct_field_name := l_alias_value.value
				else
					l_struct_field_name := a_feature.implementation_feature.lower_name
				end
				create l_splitter.make_with_separators (",")
				l_list := l_splitter.split (l_signature_arguments)
				inspect l_list.count
				when 1 then
					l_struct_type := l_list.item (1)
					l_struct_field_type := l_signature_result
					print_external_c_struct_body (l_arguments, l_result_type_set, l_struct_type, l_struct_field_name, l_struct_field_type)
				when 2 then
					l_struct_type := l_list.item (1)
					l_struct_field_type := l_list.item (2)
					print_external_c_struct_body (l_arguments, l_result_type_set, l_struct_type, l_struct_field_name, l_struct_field_type)
				else
-- TODO: syntax error
				end
			elseif external_cpp_macro_regexp.recognizes (l_language_string) then
					-- Regexp: C++ [blocking] macro [signature ["(" {<type> "," ...}* ")"] [":" <type>]] use {<include> "," ...}+
					-- \4: has signature arguments
					-- \5: signature arguments
					-- \10: signature result
					-- \17: include files
				l_is_cpp := True
				print_external_cpp_includes (external_c_macro_regexp.captured_substring (17))
				if external_cpp_macro_regexp.match_count > 4 and then external_cpp_macro_regexp.captured_substring_count (4) > 0 then
					l_signature_arguments := external_cpp_macro_regexp.captured_substring (5)
				end
				if external_cpp_macro_regexp.match_count > 10 and then external_cpp_macro_regexp.captured_substring_count (10) > 0 then
					l_signature_result := external_cpp_macro_regexp.captured_substring (10)
				end
				print_external_c_body (a_feature.implementation_feature.name, l_arguments, l_result_type_set, l_signature_arguments, l_signature_result, a_feature.alias_clause, True)
			elseif external_cpp_regexp.recognizes (l_language_string) then
					-- Regexp: C++ [blocking] <class_type> [signature ["(" {<type> "," ...}* ")"] [":" <type>]] [use {<include> "," ...}+]
					-- \2: class type
					-- \5: has signature arguments
					-- \6: signature arguments
					-- \11: signature result
					-- \18: include files
				l_is_cpp := True
				if external_cpp_regexp.match_count > 18 and then external_cpp_regexp.captured_substring_count (18) > 0 then
					l_has_include_files := True
					print_external_cpp_includes (external_cpp_regexp.captured_substring (18))
				end
				l_cpp_class_type := external_cpp_regexp.captured_substring (2)
				if external_cpp_regexp.match_count > 5 and then external_cpp_regexp.captured_substring_count (5) > 0 then
					l_signature_arguments := external_cpp_regexp.captured_substring (6)
				end
				if external_cpp_regexp.match_count > 11 and then external_cpp_regexp.captured_substring_count (11) > 0 then
					l_signature_result := external_cpp_regexp.captured_substring (11)
				end
				if not l_has_include_files then
						-- We need to generate a prototype for this external routine.
					print_external_c_prototype (a_feature.implementation_feature.name, l_arguments, l_result_type_set, l_signature_arguments, l_signature_result, a_feature.alias_clause)
				end
				print_external_cpp_body (a_feature.implementation_feature.name, l_arguments, l_result_type_set, l_cpp_class_type, l_signature_arguments, l_signature_result, a_feature.alias_clause)
			elseif external_cpp_struct_regexp.recognizes (l_language_string) then
					-- Regexp: C++ struct <struct-type> (access|get) <field-name> [type <field-type>] use {<include> "," ...}+
					-- \1: struct type
					-- \6: field name
					-- \9: field type
					-- \16: include files
				l_is_cpp := True
				print_external_cpp_includes (external_cpp_struct_regexp.captured_substring (16))
				l_struct_type := external_cpp_struct_regexp.captured_substring (1)
				l_struct_field_name := external_cpp_struct_regexp.captured_substring (6)
				if external_cpp_struct_regexp.match_count > 9 and then external_cpp_struct_regexp.captured_substring_count (9) > 0 then
					l_struct_field_type := external_cpp_struct_regexp.captured_substring (9)
				end
				print_external_c_struct_body (l_arguments, l_result_type_set, l_struct_type, l_struct_field_name, l_struct_field_type)
			elseif external_dllwin_regexp.recognizes (l_language_string) then
					-- Regexp: [blocking] dllwin <dll_file> [signature ["(" {<type> "," ...}* ")"] [":" <type>]] [use {<include> "," ...}+]
					-- \2: dll file
					-- \5: has signature arguments
					-- \6: signature arguments
					-- \11: signature result
					-- \18: include files
				if external_dllwin_regexp.match_count > 18 and then external_dllwin_regexp.captured_substring_count (18) > 0 then
					print_external_c_includes (external_dllwin_regexp.captured_substring (18))
				end
				l_dll_file := external_dllwin_regexp.captured_substring (2)
				if external_dllwin_regexp.match_count > 5 and then external_dllwin_regexp.captured_substring_count (5) > 0 then
					l_signature_arguments := external_dllwin_regexp.captured_substring (6)
				end
				if external_dllwin_regexp.match_count > 11 and then external_dllwin_regexp.captured_substring_count (11) > 0 then
					l_signature_result := external_dllwin_regexp.captured_substring (11)
				end
				print_external_dllwin_body (a_feature.implementation_feature.name, l_arguments, l_result_type_set, l_dll_file, l_signature_arguments, l_signature_result, a_feature.alias_clause)
			elseif old_external_dllwin32_regexp.recognizes (l_language_string) then
					-- Regexp: C "[" dllwin32 <dll_file> "]" ["(" {<type> "," ...}* ")"] [":" <type>]
					-- \1: dll file
					-- \2: has signature arguments
					-- \3: signature arguments
					-- \5: signature result
				l_dll_file := old_external_dllwin32_regexp.captured_substring (1)
				if old_external_dllwin32_regexp.match_count > 2 and then old_external_dllwin32_regexp.captured_substring_count (2) > 0 then
					l_signature_arguments := old_external_dllwin32_regexp.captured_substring (3)
					if old_external_dllwin32_regexp.match_count > 5 and then old_external_dllwin32_regexp.captured_substring_count (5) > 0 then
						l_signature_result := old_external_dllwin32_regexp.captured_substring (5)
					end
				end
				print_external_dllwin_body (a_feature.implementation_feature.name, l_arguments, l_result_type_set, l_dll_file, l_signature_arguments, l_signature_result, a_feature.alias_clause)
			else
error_handler.report_warning_message ("**** language not recognized: " + l_language_string)
			end
				-- Close the block containing the body of the feature.
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
			if not a_inline then
				print_feature_trace_message_call (False)
					-- Call stack.
				if exception_trace_mode then
					print_indentation
					current_file.put_string (c_ac)
					current_file.put_string (c_arrow)
					current_file.put_string (c_call)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_tc)
					current_file.put_character ('.')
					current_file.put_string (c_caller)
					current_file.put_character (';')
					current_file.put_new_line
				end
			end
			if l_result_type /= Void and (not l_is_inline or else ((exception_trace_mode or trace_mode) and not a_inline)) then
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
			elseif a_creation then
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				print_current_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
			end
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
				--
				-- Clean up.
				--
			current_file := old_file
			reset_temp_variables
				-- Flush to file.
			if l_is_cpp then
				flush_to_cpp_file
			else
				flush_to_c_file
			end
			if (exception_trace_mode or trace_mode) and l_is_inline and not a_inline then
				print_external_routine (a_feature, False, True)
			end
		end

	print_external_builtin_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			valid_feature: current_feature.static_feature = a_feature
		do
			if a_feature.is_function then
				print_external_builtin_function_body (a_feature)
			else
				print_external_builtin_procedure_body (a_feature)
			end
		end

	print_external_builtin_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_class_code
			when {ET_TOKEN_CODES}.builtin_any_class then
				print_external_builtin_any_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_arguments_32_class then
				print_external_builtin_arguments_32_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_boolean_class then
				print_external_builtin_boolean_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_boolean_ref_class then
				print_external_builtin_boolean_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_character_8_class then
				print_external_builtin_character_n_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_character_8_ref_class then
				print_external_builtin_character_n_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_character_32_class then
				print_external_builtin_character_n_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_character_32_ref_class then
				print_external_builtin_character_n_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_com_failure_class then
				print_external_builtin_com_failure_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_exception_manager_class then
				print_external_builtin_exception_manager_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_exception_manager_factory_class then
				print_external_builtin_exception_manager_factory_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_function_class then
				print_external_builtin_function_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_identified_routines_class then
				print_external_builtin_identified_routines_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_integer_8_class then
				print_external_builtin_integer_n_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_integer_8_ref_class then
				print_external_builtin_integer_n_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_integer_16_class then
				print_external_builtin_integer_n_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_integer_16_ref_class then
				print_external_builtin_integer_n_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_integer_32_class then
				print_external_builtin_integer_n_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_integer_32_ref_class then
				print_external_builtin_integer_n_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_integer_64_class then
				print_external_builtin_integer_n_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_integer_64_ref_class then
				print_external_builtin_integer_n_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_class then
				print_external_builtin_ise_runtime_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_memory_class then
				print_external_builtin_memory_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_natural_8_class then
				print_external_builtin_integer_n_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_natural_8_ref_class then
				print_external_builtin_integer_n_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_natural_16_class then
				print_external_builtin_integer_n_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_natural_16_ref_class then
				print_external_builtin_integer_n_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_natural_32_class then
				print_external_builtin_integer_n_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_natural_32_ref_class then
				print_external_builtin_integer_n_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_natural_64_class then
				print_external_builtin_integer_n_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_natural_64_ref_class then
				print_external_builtin_integer_n_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_platform_class then
				print_external_builtin_platform_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_pointer_class then
				print_external_builtin_pointer_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_pointer_ref_class then
				print_external_builtin_pointer_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_real_32_class then
				print_external_builtin_real_n_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_real_32_ref_class then
				print_external_builtin_real_n_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_real_64_class then
				print_external_builtin_real_n_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_real_64_ref_class then
				print_external_builtin_real_n_ref_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_special_class then
				print_external_builtin_special_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_class then
				print_external_builtin_tuple_function_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_type_class then
				print_external_builtin_type_function_body (a_feature)
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_any_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "ANY".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_any: current_feature.is_builtin_any_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_any_conforms_to then
				print_builtin_any_conforms_to_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_any_deep_twin then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_any_deep_twin_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_any_generating_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_any_generating_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_any_generator then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_any_generator_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_any_is_deep_equal then
				print_builtin_any_is_deep_equal_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_any_is_equal then
				print_builtin_any_is_equal_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_any_same_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_any_same_type_call (current_feature, current_type, False)
				print_semicolon_newline
			when {ET_TOKEN_CODES}.builtin_any_standard_is_equal then
				print_builtin_any_standard_is_equal_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_any_standard_twin then
				print_builtin_any_standard_twin_body (a_feature)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_any_tagged_out then
				print_builtin_any_tagged_out_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_any_twin then
				print_builtin_any_twin_body (a_feature)
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_arguments_32_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "ARGUMENTS_32".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_arguments_32: current_feature.is_builtin_arguments_32_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_arguments_32_argument_count then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_arguments_32_argument_count_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_arguments_32_i_th_argument_pointer then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_arguments_32_i_th_argument_pointer_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_arguments_32_i_th_argument_string then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_arguments_32_i_th_argument_string_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_boolean_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "BOOLEAN".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_boolean: current_feature.is_builtin_boolean_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_boolean_and then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_boolean_and_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_boolean_not then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_boolean_not_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_boolean_or then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_boolean_or_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_boolean_xor then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_boolean_xor_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_boolean_ref_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "BOOLEAN_REF".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_boolean_ref: current_feature.is_builtin_boolean_ref_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_boolean_ref_item then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_boolean_ref_item_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_character_n_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in classed "CHARACTER_N".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_character_n: current_feature.is_builtin_character_n_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_character_n_code then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_character_n_code_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_character_n_natural_32_code then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_character_n_natural_32_code_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_character_n_to_character_8 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_character_n_to_character_8_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_character_n_to_character_32 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_character_n_to_character_32_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_character_n_ref_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in classed "CHARACTER_N_REF".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_character_n_ref: current_feature.is_builtin_character_n_ref_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_character_n_ref_item then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_character_n_ref_item_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_com_failure_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "COM_FAILURE".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_com_failure: current_feature.is_builtin_com_failure_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_com_failure_c_strlen then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_com_failure_c_strlen_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_com_failure_ccom_hresult then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_com_failure_ccom_hresult_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_com_failure_ccom_hresult_code then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_com_failure_ccom_hresult_code_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_com_failure_ccom_hresult_facility then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_com_failure_ccom_hresult_facility_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_com_failure_character_size then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_com_failure_character_size_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_com_failure_cwin_error_text then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_com_failure_cwin_error_text_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_exception_manager_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "EXCEPTION_MANAGER".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_exception_manager: current_feature.is_builtin_exception_manager_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_exception_manager_exception_from_code then
				print_builtin_exception_manager_exception_from_code_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_exception_manager_is_caught then
				print_builtin_exception_manager_is_caught_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_exception_manager_is_ignorable then
				print_builtin_exception_manager_is_ignorable_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_exception_manager_is_ignored then
				print_builtin_exception_manager_is_ignored_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_exception_manager_is_raisable then
				print_builtin_exception_manager_is_raisable_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_exception_manager_last_exception then
				print_builtin_exception_manager_last_exception_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_exception_manager_type_of_code then
				print_builtin_exception_manager_type_of_code_body (a_feature)
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_exception_manager_factory_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "EXCEPTION_MANAGER_FACTORY".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_exception_manager_factory: current_feature.is_builtin_exception_manager_factory_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_exception_manager_factory_exception_manager then
				print_builtin_exception_manager_factory_exception_manager_body (a_feature)
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_function_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "FUNCTION".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_function: current_feature.is_builtin_function_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_function_fast_item then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_function_fast_item_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_function_item then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_function_item_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_identified_routines_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "IDENTIFIED_ROUTINES".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_identified_routines: current_feature.is_builtin_identified_routines_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_identified_routines_eif_current_object_id then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_identified_routines_eif_current_object_id_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_identified_routines_eif_id_object then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_identified_routines_eif_id_object_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_identified_routines_eif_is_object_id_of_current then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_identified_routines_eif_is_object_id_of_current_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_identified_routines_eif_object_id then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_identified_routines_eif_object_id_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_integer_n_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in classes "INTEGER_N" and "NATURAL_N".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_integer_n: current_feature.is_builtin_integer_n_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_integer_n_as_integer_8 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_as_integer_8_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_as_integer_16 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_as_integer_16_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_as_integer_32 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_as_integer_32_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_as_integer_64 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_as_integer_64_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_as_natural_8 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_as_natural_8_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_as_natural_16 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_as_natural_16_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_as_natural_32 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_as_natural_32_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_as_natural_64 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_as_natural_64_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_bit_and then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_bit_and_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_bit_not then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_bit_not_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_bit_or then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_bit_or_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_bit_shift_left then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_bit_shift_left_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_bit_shift_right then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_bit_shift_right_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_bit_xor then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_bit_xor_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_identity then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_identity_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_integer_quotient then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_integer_quotient_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_integer_remainder then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_integer_remainder_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_is_less then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_is_less_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_minus then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_minus_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_opposite then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_opposite_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_plus then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_plus_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_power then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_power_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_product then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_product_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_quotient then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_quotient_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_to_character_8 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_to_character_8_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_to_character_32 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_to_character_32_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_to_double then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_to_double_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_to_real then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_to_real_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_to_real_32 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_to_real_32_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_integer_n_to_real_64 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_to_real_64_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_integer_n_ref_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in classes "INTEGER_N_REF" and "NATURAL_N_REF".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_integer_n_ref: current_feature.is_builtin_integer_n_ref_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_integer_n_ref_item then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_integer_n_ref_item_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_ise_runtime_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "ISE_RUNTIME".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_ise_runtime: current_feature.is_builtin_ise_runtime_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_ise_runtime_attached_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_attached_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_boolean_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_boolean_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_boolean_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_boolean_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_character_8_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_character_8_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_character_8_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_character_8_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_character_32_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_character_32_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_character_32_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_character_32_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_check_assert then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_check_assert_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_compiler_version then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_compiler_version_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_detachable_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_detachable_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_dynamic_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_dynamic_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_dynamic_type_at_offset then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_dynamic_type_at_offset_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_eif_gen_param_id then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_eif_gen_param_id_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_field_count_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_field_count_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_field_name_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_field_name_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_field_offset_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_field_offset_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_field_static_type_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_field_static_type_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_field_type_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_field_type_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_generating_type_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_generating_type_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_generating_type_8_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_generating_type_8_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_generator_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_generator_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_generator_8_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_generator_8_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_generic_parameter_count then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_generic_parameter_count_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_in_assertion then
				print_builtin_ise_runtime_in_assertion_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_8_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_integer_8_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_8_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_integer_8_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_16_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_integer_16_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_16_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_integer_16_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_32_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_integer_32_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_32_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_integer_32_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_64_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_integer_64_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_64_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_integer_64_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_attached_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_attached_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_copy_semantics_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_copy_semantics_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_expanded then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_expanded_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_field_expanded_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_field_expanded_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_field_transient_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_field_transient_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_object_marked then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_object_marked_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_special then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_special_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_special_copy_semantics_item then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_special_copy_semantics_item_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_special_of_expanded then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_special_of_expanded_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_special_of_reference then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_special_of_reference_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_special_of_reference_or_basic_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_special_of_reference_or_basic_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_special_of_reference_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_special_of_reference_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_tuple then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_tuple_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_tuple_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_is_tuple_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_8_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_natural_8_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_8_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_natural_8_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_16_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_natural_16_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_16_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_natural_16_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_32_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_natural_32_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_32_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_natural_32_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_64_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_natural_64_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_64_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_natural_64_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_new_instance_of then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_new_instance_of_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_new_special_of_reference_instance_of then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_new_special_of_reference_instance_of_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_new_tuple_instance_of then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_new_tuple_instance_of_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_new_type_instance_of then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_new_type_instance_of_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_object_size then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_object_size_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_once_objects then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_once_objects_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_persistent_field_count_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_persistent_field_count_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_pointer_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_pointer_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_pointer_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_pointer_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_pre_ecma_mapping_status then
				print_builtin_ise_runtime_pre_ecma_mapping_status_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_raw_reference_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_raw_reference_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_raw_reference_field_at_offset then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_raw_reference_field_at_offset_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_real_32_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_real_32_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_real_32_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_real_32_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_real_64_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_real_64_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_real_64_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_real_64_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_reference_field then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_reference_field_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_reference_field_at then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_reference_field_at_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_reference_field_at_offset then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_reference_field_at_offset_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_storable_version_of_type then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_storable_version_of_type_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_type_id_from_name then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_type_id_from_name_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_type_conforms_to then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_ise_runtime_type_conforms_to_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_memory_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "MEMORY".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_memory: current_feature.is_builtin_memory_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_memory_find_referers then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_memory_find_referers_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_platform_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "PLATFORM".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_platform: current_feature.is_builtin_platform_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_platform_boolean_bytes then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_boolean_bytes_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_character_bytes then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_character_bytes_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_double_bytes then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_double_bytes_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_integer_bytes then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_integer_bytes_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_is_64_bits then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_is_64_bits_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_is_dotnet then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_is_dotnet_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_is_mac then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_is_mac_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_is_scoop_capable then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_is_scoop_capable_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_is_thread_capable then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_is_thread_capable_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_is_unix then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_is_unix_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_is_vms then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_is_vms_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_is_vxworks then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_is_vxworks_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_is_windows then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_is_windows_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_pointer_bytes then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_pointer_bytes_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_real_bytes then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_real_bytes_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_platform_wide_character_bytes then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_platform_wide_character_bytes_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_pointer_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "POINTER".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_pointer: current_feature.is_builtin_pointer_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_pointer_hash_code then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_pointer_hash_code_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_pointer_is_default_pointer then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_pointer_is_default_pointer_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_pointer_out then
				print_builtin_pointer_out_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_pointer_plus then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_pointer_plus_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_pointer_to_integer_32 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_pointer_to_integer_32_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_pointer_ref_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "POINTER_REF".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_pointer_ref: current_feature.is_builtin_pointer_ref_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_pointer_ref_item then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_pointer_ref_item_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_real_n_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in classes "REAL_N".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_real_n: current_feature.is_builtin_real_n_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_real_n_ceiling_real_32 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ceiling_real_32_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_ceiling_real_64 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ceiling_real_64_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_floor_real_32 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_floor_real_32_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_floor_real_64 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_floor_real_64_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_identity then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_identity_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_is_equal then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ieee_is_equal_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_is_greater then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ieee_is_greater_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_is_greater_equal then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ieee_is_greater_equal_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_is_less then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ieee_is_less_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_is_less_equal then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ieee_is_less_equal_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_maximum_number then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ieee_maximum_number_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_minimum_number then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ieee_minimum_number_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_is_less then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_is_less_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_is_nan then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_is_nan_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_is_negative_infinity then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_is_negative_infinity_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_is_positive_infinity then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_is_positive_infinity_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_minus then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_minus_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_opposite then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_opposite_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_out then
				print_builtin_real_n_out_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_real_n_plus then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_plus_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_power then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_power_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_product then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_product_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_quotient then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_quotient_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_to_double then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_to_double_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_truncated_to_integer then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_truncated_to_integer_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_truncated_to_integer_64 then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_truncated_to_integer_64_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_truncated_to_real then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_truncated_to_real_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_real_n_ref_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in classes "REAL_N_REF".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_real_n_ref: current_feature.is_builtin_real_n_ref_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_real_n_ref_item then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ref_item_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_ref_nan then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ref_nan_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_ref_negative_infinity then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ref_negative_infinity_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_real_n_ref_positive_infinity then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_real_n_ref_positive_infinity_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_special_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "SPECIAL".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_special: current_feature.is_builtin_special_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_special_aliased_resized_area then
				print_builtin_special_aliased_resized_area_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_special_base_address then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_special_base_address_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_special_capacity then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_special_capacity_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_special_count then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_special_count_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_special_element_size then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_special_element_size_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_special_item then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_special_item_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_tuple_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "TUPLE".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_tuple: current_feature.is_builtin_tuple_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_tuple_boolean_item then
				print_builtin_tuple_boolean_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_character_8_item then
				print_builtin_tuple_character_8_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_character_32_item then
				print_builtin_tuple_character_32_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_count then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_tuple_count_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_tuple_integer_8_item then
				print_builtin_tuple_integer_8_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_integer_16_item then
				print_builtin_tuple_integer_16_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_integer_32_item then
				print_builtin_tuple_integer_32_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_integer_64_item then
				print_builtin_tuple_integer_64_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_item_code then
				print_builtin_tuple_item_code_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_natural_8_item then
				print_builtin_tuple_natural_8_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_natural_16_item then
				print_builtin_tuple_natural_16_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_natural_32_item then
				print_builtin_tuple_natural_32_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_natural_64_item then
				print_builtin_tuple_natural_64_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_object_comparison then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_tuple_object_comparison_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_tuple_pointer_item then
				print_builtin_tuple_pointer_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_real_32_item then
				print_builtin_tuple_real_32_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_real_64_item then
				print_builtin_tuple_real_64_item_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_reference_item then
				print_builtin_tuple_reference_item_body (a_feature)
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_type_function_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in function introduced in class "TYPE".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_type: current_feature.is_builtin_type_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_type_default then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_type_default_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_type_generic_parameter_type then
				print_builtin_type_generic_parameter_type_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_type_generic_parameter_count then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_type_generic_parameter_count_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_type_has_default then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_type_has_default_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_type_is_attached then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_type_is_attached_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_type_is_deferred then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_type_is_deferred_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_type_is_expanded then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_type_is_expanded_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_type_runtime_name then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_type_runtime_name_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_type_type_id then
				fill_call_formal_arguments (a_feature)
				print_indentation_assign_to_result
				print_builtin_type_type_id_call (current_feature, current_type, False)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_class_code
			when {ET_TOKEN_CODES}.builtin_any_class then
				print_external_builtin_any_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_boolean_ref_class then
				print_external_builtin_boolean_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_character_8_ref_class then
				print_external_builtin_character_n_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_character_32_ref_class then
				print_external_builtin_character_n_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_com_failure_class then
				print_external_builtin_com_failure_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_exception_manager_class then
				print_external_builtin_exception_manager_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_identified_routines_class then
				print_external_builtin_identified_routines_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_integer_8_ref_class then
				print_external_builtin_integer_n_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_integer_16_ref_class then
				print_external_builtin_integer_n_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_integer_32_ref_class then
				print_external_builtin_integer_n_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_integer_64_ref_class then
				print_external_builtin_integer_n_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_ise_exception_manager_class then
				print_external_builtin_ise_exception_manager_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_class then
				print_external_builtin_ise_runtime_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_memory_class then
				print_external_builtin_memory_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_natural_8_ref_class then
				print_external_builtin_integer_n_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_natural_16_ref_class then
				print_external_builtin_integer_n_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_natural_32_ref_class then
				print_external_builtin_integer_n_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_natural_64_ref_class then
				print_external_builtin_integer_n_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_pointer_ref_class then
				print_external_builtin_pointer_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_procedure_class then
				print_external_builtin_procedure_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_real_32_ref_class then
				print_external_builtin_real_n_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_real_64_ref_class then
				print_external_builtin_real_n_ref_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_special_class then
				print_external_builtin_special_procedure_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_class then
				print_external_builtin_tuple_procedure_body (a_feature)
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_any_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in class "ANY".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_any: current_feature.is_builtin_any_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_any_copy then
				print_builtin_any_copy_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_any_standard_copy then
				print_builtin_any_standard_copy_body (a_feature)
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_boolean_ref_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in class "BOOLEAN_REF".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_boolean_ref: current_feature.is_builtin_boolean_ref_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_boolean_ref_set_item then
				fill_call_formal_arguments (a_feature)
				print_builtin_boolean_ref_set_item_call (current_feature, current_type, False)
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_character_n_ref_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in classes "CHARACTER_N_REF".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_character_n_ref: current_feature.is_builtin_character_n_ref_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_character_n_ref_set_item then
				fill_call_formal_arguments (a_feature)
				print_builtin_character_n_ref_set_item_call (current_feature, current_type, False)
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_com_failure_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in class "COM_FAILURE".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_com_failure: current_feature.is_builtin_com_failure_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_com_failure_cwin_local_free then
				fill_call_formal_arguments (a_feature)
				print_builtin_com_failure_cwin_local_free_call (current_feature, current_type, False)
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_exception_manager_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in class "EXCEPTION_MANAGER".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_exception_manager: current_feature.is_builtin_exception_manager_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_exception_manager_catch then
				print_builtin_exception_manager_catch_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_exception_manager_ignore then
				print_builtin_exception_manager_ignore_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_exception_manager_raise then
				print_builtin_exception_manager_raise_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_exception_manager_set_is_ignored then
				print_builtin_exception_manager_set_is_ignored_body (a_feature)
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_identified_routines_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in class "IDENTIFIED_ROUTINES".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_identified_routines: current_feature.is_builtin_identified_routines_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_identified_routines_eif_object_id_free then
				fill_call_formal_arguments (a_feature)
				print_builtin_identified_routines_eif_object_id_free_call (current_feature, current_type, False)
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_integer_n_ref_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in classes "INTEGER_N_REF" and "NATURAL_N_REF".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_integer_n_ref: current_feature.is_builtin_integer_n_ref_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_integer_n_ref_set_item then
				fill_call_formal_arguments (a_feature)
				print_builtin_integer_n_ref_set_item_call (current_feature, current_type, False)
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_ise_exception_manager_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in class "ISE_EXCEPTION_MANAGER".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_ise_exception_manager: current_feature.is_builtin_ise_exception_manager_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_ise_exception_manager_developer_raise then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_exception_manager_developer_raise_call (current_feature, current_type, False)
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_ise_runtime_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in class "ISE_RUNTIME".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_ise_runtime: current_feature.is_builtin_ise_runtime_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_ise_runtime_lock_marking then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_lock_marking_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_mark_object then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_mark_object_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_boolean_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_boolean_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_boolean_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_boolean_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_character_8_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_character_8_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_character_8_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_character_8_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_character_32_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_character_32_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_character_32_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_character_32_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_8_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_integer_8_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_8_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_integer_8_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_16_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_integer_16_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_16_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_integer_16_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_32_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_integer_32_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_32_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_integer_32_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_64_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_integer_64_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_64_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_integer_64_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_8_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_natural_8_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_8_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_natural_8_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_16_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_natural_16_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_16_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_natural_16_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_32_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_natural_32_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_32_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_natural_32_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_64_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_natural_64_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_64_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_natural_64_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_pointer_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_pointer_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_pointer_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_pointer_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_pre_ecma_mapping then
				print_builtin_ise_runtime_set_pre_ecma_mapping_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_real_32_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_real_32_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_real_32_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_real_32_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_real_64_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_real_64_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_real_64_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_real_64_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_reference_field then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_reference_field_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_reference_field_at then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_set_reference_field_at_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_unlock_marking then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_unlock_marking_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_ise_runtime_unmark_object then
				fill_call_formal_arguments (a_feature)
				print_builtin_ise_runtime_unmark_object_call (current_feature, current_type, False)
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_memory_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in class "MEMORY".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_memory: current_feature.is_builtin_memory_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_memory_free then
				fill_call_formal_arguments (a_feature)
				print_builtin_memory_free_call (current_feature, current_type, False)
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_pointer_ref_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in class "POINTER_REF".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_pointer_ref: current_feature.is_builtin_pointer_ref_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_pointer_ref_set_item then
				fill_call_formal_arguments (a_feature)
				print_builtin_pointer_ref_set_item_call (current_feature, current_type, False)
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_procedure_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in class "PROCEDURE".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_procedure: current_feature.is_builtin_procedure_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_procedure_call then
				fill_call_formal_arguments (a_feature)
				print_builtin_procedure_call_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_procedure_fast_call then
				fill_call_formal_arguments (a_feature)
				print_builtin_procedure_fast_call_call (current_feature, current_type, False)
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_real_n_ref_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in classes "REAL_N_REF".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_real_n_ref: current_feature.is_builtin_real_n_ref_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_real_n_ref_set_item then
				fill_call_formal_arguments (a_feature)
				print_builtin_real_n_ref_set_item_call (current_feature, current_type, False)
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_special_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in class "SPECIAL".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_special: current_feature.is_builtin_special_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_special_make_empty then
				-- Do nothing: already done in `print_malloc_current'.
			when {ET_TOKEN_CODES}.builtin_special_extend then
				print_builtin_special_extend_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_special_put then
				fill_call_formal_arguments (a_feature)
				print_builtin_special_put_call (current_feature, current_type, False)
				call_operands.wipe_out
			when {ET_TOKEN_CODES}.builtin_special_set_count then
				print_builtin_special_set_count_body (a_feature)
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_builtin_tuple_procedure_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of built-in feature `a_feature'.
			-- `a_feature' is a built-in procedure introduced in class "TUPLE".
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_procedure: a_feature.is_procedure
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_tuple: current_feature.is_builtin_tuple_class
			valid_feature: current_feature.static_feature = a_feature
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_tuple_put_boolean then
				print_builtin_tuple_put_boolean_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_character_8 then
				print_builtin_tuple_put_character_8_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_character_32 then
				print_builtin_tuple_put_character_32_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_integer_8 then
				print_builtin_tuple_put_integer_8_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_integer_16 then
				print_builtin_tuple_put_integer_16_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_integer_32 then
				print_builtin_tuple_put_integer_32_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_integer_64 then
				print_builtin_tuple_put_integer_64_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_natural_8 then
				print_builtin_tuple_put_natural_8_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_natural_16 then
				print_builtin_tuple_put_natural_16_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_natural_32 then
				print_builtin_tuple_put_natural_32_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_natural_64 then
				print_builtin_tuple_put_natural_64_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_pointer then
				print_builtin_tuple_put_pointer_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_real_32 then
				print_builtin_tuple_put_real_32_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_real_64 then
				print_builtin_tuple_put_real_64_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_put_reference then
				print_builtin_tuple_put_reference_body (a_feature)
			when {ET_TOKEN_CODES}.builtin_tuple_set_object_comparison then
				fill_call_formal_arguments (a_feature)
				print_builtin_tuple_set_object_comparison_call (current_feature, current_type, False)
				call_operands.wipe_out
			else
					-- Internal error: unknown built-in feature.
					-- This error should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_external_c_prototype (a_feature_name: detachable ET_FEATURE_NAME;
		a_arguments: detachable ET_FORMAL_ARGUMENT_LIST; a_result_type_set: detachable ET_DYNAMIC_TYPE_SET;
		a_signature_arguments, a_signature_result: detachable STRING;
		a_alias: detachable ET_EXTERNAL_ALIAS)
			-- Print to `header_file' a prototype for the external C function.
			-- If `a_feature_name' is Void then the name of the C function will be found in the alias.
			-- `a_signature_arguments' and `a_signature_result', if not Void,
			-- are the signature types declared in the Language part.
			-- `a_result_type_set' is not Void if the external feature is a query.
		require
			name_not_void: a_feature_name /= Void or else a_alias /= Void
		local
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
			l_alias_value: ET_MANIFEST_STRING
			i, nb_args: INTEGER
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_type: ET_DYNAMIC_PRIMARY_TYPE
			l_actual_parameters: detachable ET_ACTUAL_PARAMETERS
			l_actual_parameter: ET_DYNAMIC_PRIMARY_TYPE
		do
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			if a_signature_result /= Void then
				header_file.put_string (a_signature_result)
			elseif a_result_type_set /= Void then
				l_result_type := a_result_type_set.static_type.primary_type
				print_type_declaration (l_result_type, header_file)
			else
				header_file.put_string (c_void)
			end
			header_file.put_character (' ')
			if a_alias /= Void then
				l_alias_value := a_alias.manifest_string
				header_file.put_string (l_alias_value.value)
			elseif a_feature_name /= Void then
				header_file.put_string (a_feature_name.lower_name)
			else
				check precondition_name_not_void: False then end
			end
			header_file.put_character ('(')
			if a_signature_arguments /= Void then
				header_file.put_string (a_signature_arguments)
			elseif a_arguments /= Void and then not a_arguments.is_empty then
				nb_args := a_arguments.count
				from i := 1 until i > nb_args loop
					if i /= 1 then
						header_file.put_character (',')
					end
					l_argument_type_set := argument_type_set (i)
					l_argument_type := l_argument_type_set.static_type.primary_type
					if l_argument_type.base_class.is_typed_pointer_class then
							-- The argument is declared of type 'TYPED_POINTER [XX]'.
							-- In that case we use the corresponding pointer (i.e.
							-- the first attribute of the object).
						l_actual_parameters := l_argument_type.base_type.actual_parameters
						if l_actual_parameters = Void or else l_actual_parameters.is_empty then
								-- Internal error: TYPED_POINTER [XX] has one generic parameter.
								-- This should have been checked already.
							set_fatal_error
							error_handler.report_giaaa_error
						else
							l_actual_parameter := current_dynamic_system.dynamic_primary_type (l_actual_parameters.type (1), current_system.any_type)
							print_type_declaration (l_actual_parameter, header_file)
							if l_actual_parameter.is_expanded then
								header_file.put_character ('*')
							end
						end
					else
						print_type_declaration (l_argument_type, header_file)
					end
					i := i + 1
				end
			end
			header_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
		end

	print_external_c_body (a_feature_name: detachable ET_FEATURE_NAME;
		a_arguments: detachable ET_FORMAL_ARGUMENT_LIST; a_result_type_set: detachable ET_DYNAMIC_TYPE_SET;
		a_signature_arguments, a_signature_result: detachable STRING;
		a_alias: detachable ET_EXTERNAL_ALIAS; is_macro: BOOLEAN)
			-- Print body of external C function to `current_file'.
			-- If `a_feature_name' is Void then the name of the C function will be found in the alias.
			-- `a_signature_arguments' and `a_signature_result', if not Void,
			-- are the signature types declared in the Language part.
			-- `a_result_type_set' is not Void if the external feature is a query.
		require
			name_not_void: a_feature_name /= Void or else a_alias /= Void
		local
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
			l_alias_value: ET_MANIFEST_STRING
			i, nb_args: INTEGER
			l_splitter: ST_SPLITTER
			l_list: DS_LIST [STRING]
			l_cursor: DS_LIST_CURSOR [STRING]
			l_name: ET_IDENTIFIER
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_type: ET_DYNAMIC_PRIMARY_TYPE
			l_actual_parameters: detachable ET_ACTUAL_PARAMETERS
			l_actual_parameter: ET_DYNAMIC_PRIMARY_TYPE
		do
			print_indentation
			if a_result_type_set /= Void then
				l_result_type := a_result_type_set.static_type.primary_type
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_declaration_type_cast_from_c (l_result_type, current_file)
				current_file.put_character ('(')
			end
			if a_alias /= Void then
				l_alias_value := a_alias.manifest_string
				current_file.put_string (l_alias_value.value)
			elseif a_feature_name /= Void then
				current_file.put_string (a_feature_name.lower_name)
			else
				check precondition_name_not_void: False then end
			end
			if a_arguments /= Void and then not a_arguments.is_empty then
				nb_args := a_arguments.count
				current_file.put_character ('(')
				if a_signature_arguments /= Void then
					create l_splitter.make_with_separators (",")
					l_list := l_splitter.split (a_signature_arguments)
					if l_list.count /= nb_args then
-- TODO: error
					end
					l_cursor := l_list.new_cursor
					l_cursor.start
					from i := 1 until i > nb_args loop
						if i /= 1 then
							current_file.put_character (',')
						end
						if not l_cursor.after then
							current_file.put_character ('(')
							current_file.put_string (l_cursor.item)
							current_file.put_character (')')
							l_cursor.forth
						end
						l_name := a_arguments.formal_argument (i).name
						l_argument_type_set := argument_type_set (i)
						l_argument_type := l_argument_type_set.static_type.primary_type
						if l_argument_type.base_class.is_typed_pointer_class then
								-- The argument is declared of type 'TYPED_POINTER [XX]'.
								-- In that case we use the corresponding pointer (i.e.
								-- the first attribute of the object).
							l_actual_parameters := l_argument_type.base_type.actual_parameters
							if l_actual_parameters = Void or else l_actual_parameters.is_empty then
									-- Internal error: TYPED_POINTER [XX] has one generic parameter.
									-- This should have been checked already.
								set_fatal_error
								error_handler.report_giaaa_error
							else
								l_actual_parameter := current_dynamic_system.dynamic_primary_type (l_actual_parameters.type (1), current_system.any_type)
								current_file.put_character ('(')
								print_type_declaration (l_actual_parameter, current_file)
								if l_actual_parameter.is_expanded then
									current_file.put_character ('*')
								end
								current_file.put_character (')')
								current_file.put_character ('(')
								print_argument_name (l_name, current_file)
								current_file.put_character ('.')
								current_file.put_character ('a')
								current_file.put_character ('1')
								current_file.put_character (')')
							end
						else
							print_argument_name (l_name, current_file)
						end
						i := i + 1
					end
				else
					from i := 1 until i > nb_args loop
						if i /= 1 then
							current_file.put_character (',')
						end
						l_name := a_arguments.formal_argument (i).name
						l_argument_type_set := argument_type_set (i)
						l_argument_type := l_argument_type_set.static_type.primary_type
						if l_argument_type = current_dynamic_system.pointer_type then
								-- When compiling with C++, MSVC++ does not want to convert
								-- 'void*' to non-'void*' implicitly.
							current_file.put_character ('(')
							current_file.put_string (c_eif_native_char)
							current_file.put_character ('*')
							current_file.put_character (')')
							print_argument_name (l_name, current_file)
						elseif l_argument_type.base_class.is_typed_pointer_class then
								-- The argument is declared of type 'TYPED_POINTER [XX]'.
								-- In that case we use the corresponding pointer (i.e.
								-- the first attribute of the object).
							l_actual_parameters := l_argument_type.base_type.actual_parameters
							if l_actual_parameters = Void or else l_actual_parameters.is_empty then
									-- Internal error: TYPED_POINTER [XX] has one generic parameter.
									-- This should have been checked already.
								set_fatal_error
								error_handler.report_giaaa_error
							else
								l_actual_parameter := current_dynamic_system.dynamic_primary_type (l_actual_parameters.type (1), current_system.any_type)
								current_file.put_character ('(')
								print_type_declaration (l_actual_parameter, current_file)
								if l_actual_parameter.is_expanded then
									current_file.put_character ('*')
								end
								current_file.put_character (')')
								current_file.put_character ('(')
								print_argument_name (l_name, current_file)
								current_file.put_character ('.')
								current_file.put_character ('a')
								current_file.put_character ('1')
								current_file.put_character (')')
							end
						else
							print_argument_name (l_name, current_file)
						end
						i := i + 1
					end
				end
				current_file.put_character (')')
			elseif not is_macro then
				current_file.put_character ('(')
				current_file.put_character (')')
			end
			if a_result_type_set /= Void then
				current_file.put_character (')')
			end
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_external_c_struct_body (a_arguments: detachable ET_FORMAL_ARGUMENT_LIST; a_result_type_set: detachable ET_DYNAMIC_TYPE_SET;
		a_struct_type, a_field_name: STRING; a_field_type: detachable STRING)
			-- Print body of external C struct to `current_file'.
			-- `a_result_type_set' is not Void if the external feature is a query.
		require
			a_struct_type_not_void: a_struct_type /= Void
			a_field_name_not_void: a_field_name /= Void
		local
			l_result_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_field_name: STRING
			l_name: ET_IDENTIFIER
			nb_args: INTEGER
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_type: ET_DYNAMIC_PRIMARY_TYPE
			l_actual_parameters: detachable ET_ACTUAL_PARAMETERS
			l_actual_parameter: ET_DYNAMIC_PRIMARY_TYPE
		do
			if a_result_type_set /= Void then
				l_result_type := a_result_type_set.static_type.primary_type
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_declaration_type_cast_from_c (l_result_type, current_file)
				current_file.put_character ('(')
			end
			l_field_name := a_field_name
			if not l_field_name.is_empty then
				inspect l_field_name.item (1)
				when '@' then
					l_field_name := STRING_.cloned_string (l_field_name)
					l_field_name.remove_head (1)
				when '&' then
					l_field_name := STRING_.cloned_string (l_field_name)
					l_field_name.remove_head (1)
					current_file.put_character ('&')
				else
				end
			end
			current_file.put_character ('(')
			current_file.put_character ('(')
			current_file.put_character ('(')
			current_file.put_string (a_struct_type)
			current_file.put_character ('*')
			current_file.put_character (')')
			if a_arguments /= Void then
				nb_args := a_arguments.count
				if nb_args >= 1 then
					l_name := a_arguments.formal_argument (1).name
					l_argument_type_set := dynamic_type_set (l_name)
					l_argument_type := l_argument_type_set.static_type.primary_type
					if l_argument_type.base_class.is_typed_pointer_class then
							-- The argument is declared of type 'TYPED_POINTER [XX]'.
							-- In that case we use the corresponding pointer (i.e.
							-- the first attribute of the object).
						l_actual_parameters := l_argument_type.base_type.actual_parameters
						if l_actual_parameters = Void or else l_actual_parameters.is_empty then
								-- Internal error: TYPED_POINTER [XX] has one generic parameter.
								-- This should have been checked already.
							set_fatal_error
							error_handler.report_giaaa_error
						else
							l_actual_parameter := current_dynamic_system.dynamic_primary_type (l_actual_parameters.type (1), current_system.any_type)
							current_file.put_character ('(')
							print_type_declaration (l_actual_parameter, current_file)
							if l_actual_parameter.is_expanded then
								current_file.put_character ('*')
							end
							current_file.put_character (')')
							current_file.put_character ('(')
							print_argument_name (l_name, current_file)
							current_file.put_character ('.')
							current_file.put_character ('a')
							current_file.put_character ('1')
							current_file.put_character (')')
						end
					else
						print_argument_name (l_name, current_file)
					end
				end
			else
-- TODO: error
			end
			current_file.put_character (')')
			current_file.put_character ('-')
			current_file.put_character ('>')
			current_file.put_string (l_field_name)
			current_file.put_character (')')
			if l_result_type = Void then
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				if a_field_type /= Void then
					current_file.put_character ('(')
					current_file.put_string (a_field_type)
					current_file.put_character (')')
				end
				if a_arguments /= Void and then nb_args >= 2 then
					l_name := a_arguments.formal_argument (2).name
					l_argument_type_set := dynamic_type_set (l_name)
					l_argument_type := l_argument_type_set.static_type.primary_type
					if l_argument_type.base_class.is_typed_pointer_class then
							-- The argument is declared of type 'TYPED_POINTER [XX]'.
							-- In that case we use the corresponding pointer (i.e.
							-- the first attribute of the object).
						l_actual_parameters := l_argument_type.base_type.actual_parameters
						if l_actual_parameters = Void or else l_actual_parameters.is_empty then
								-- Internal error: TYPED_POINTER [XX] has one generic parameter.
								-- This should have been checked already.
							set_fatal_error
							error_handler.report_giaaa_error
						else
							l_actual_parameter := current_dynamic_system.dynamic_primary_type (l_actual_parameters.type (1), current_system.any_type)
							current_file.put_character ('(')
							print_type_declaration (l_actual_parameter, current_file)
							if l_actual_parameter.is_expanded then
								current_file.put_character ('*')
							end
							current_file.put_character (')')
							current_file.put_character ('(')
							print_argument_name (l_name, current_file)
							current_file.put_character ('.')
							current_file.put_character ('a')
							current_file.put_character ('1')
							current_file.put_character (')')
						end
					else
						print_argument_name (l_name, current_file)
					end
				else
-- TODO: error
				end
			end
			if l_result_type /= Void then
				current_file.put_character (')')
			end
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_external_c_inline_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print body of external "C inline" `a_feature' to `current_file'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_alias_value: ET_MANIFEST_STRING
			l_c_code: STRING
			l_argument_name: ET_IDENTIFIER
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_type: ET_DYNAMIC_PRIMARY_TYPE
			l_actual_parameters: detachable ET_ACTUAL_PARAMETERS
			l_actual_parameter: ET_DYNAMIC_PRIMARY_TYPE
			l_name: STRING
			i, j, nb: INTEGER
			i2, nb2: INTEGER
			i3, nb3: INTEGER
			l_max: INTEGER
			l_max_index: INTEGER
			l_semicolon_needed: BOOLEAN
			l_has_result_type_cast: BOOLEAN
			c, c3: CHARACTER
		do
			if attached a_feature.alias_clause as l_alias then
				l_alias_value := l_alias.manifest_string
				l_c_code := l_alias_value.value
					-- Check if it looks like legacy code: a C statement
					-- with no terminating semicolon.
				from
					i := l_c_code.count
				until
					i < 1
				loop
					inspect l_c_code.item (i)
					when ' ', '%N', '%R', '%T' then
						i := i - 1
					when ';' then
							-- This is a terminating semicolon.
						i := 0
					when '}' then
							-- This is a terminating '}'.
						i := 0
					when '/' then
						if i > 1 and then l_c_code.item (i - 1) = '*' then
								-- This is a terminating comment.
						else
								-- There is no terminating semicolon.
							l_semicolon_needed := True

						end
						i := 0
					when 'f' then
						if i > 5 and then c_endif.same_characters (l_c_code, i - 5, i, 1) then
								-- This is a terminating #endif.
						else
								-- There is no terminating semicolon.
							l_semicolon_needed := True

						end
						i := 0
					else
							-- There is no terminating semicolon.
						l_semicolon_needed := True
						i := 0
					end
				end
				if a_feature.is_function and then not l_c_code.has_substring (c_return) then
						-- This looks like legacy code.
						-- With ECMA we need to write the 'return' keyword in the alias clause.
					current_file.put_string (c_return)
					current_file.put_character (' ')
					if attached current_feature.result_type_set as l_result_type_set then
						print_declaration_type_cast_from_c (l_result_type_set.static_type.primary_type, current_file)
						current_file.put_character ('(')
						l_has_result_type_cast := True
					end
					l_semicolon_needed := True
				end
				if attached a_feature.arguments as l_formal_arguments then
					nb := l_c_code.count
					from i := 1 until i > nb loop
						c := l_c_code.item (i)
						if c = '$' then
							i := i + 1
							if i <= nb then
								c := l_c_code.item (i)
								inspect c
								when '$' then
									current_file.put_character ('$')
									i := i + 1
								when 'a'..'z', 'A'..'Z' then
									l_max := 0
									l_max_index := 0
									nb2 := l_formal_arguments.count
									from i2 := 1 until i2 > nb2 loop
										l_name := l_formal_arguments.formal_argument (i2).name.name
										nb3 := l_name.count
										if nb3 > l_max then
											from
												i3 := 1
												j := i
											until
												j > nb or
												i3 > nb3
											loop
												c := l_c_code.item (j)
												c3 := l_name.item (i3)
												if CHARACTER_.as_lower (c3) = CHARACTER_.as_lower (c) then
													i3 := i3 + 1
													j := j + 1
												else
													j := nb + 1
												end
											end
											if i3 > nb3 then
												l_max_index := i2
												l_max := nb3
											end
										end
										i2 := i2 + 1
									end
									if l_max_index /= 0 then
										l_argument_name := l_formal_arguments.formal_argument (l_max_index).name
										l_argument_type_set := argument_type_set (l_max_index)
										l_argument_type := l_argument_type_set.static_type.primary_type
										if l_argument_type.base_class.is_typed_pointer_class then
												-- The argument is declared of type 'TYPED_POINTER [XX]'.
												-- In that case we use the corresponding pointer (i.e.
												-- the first attribute of the object).
											l_actual_parameters := l_argument_type.primary_type.base_type.actual_parameters
											if l_actual_parameters = Void or else l_actual_parameters.is_empty then
													-- Internal error: TYPED_POINTER [XX] has one generic parameter.
													-- This should have been checked already.
												set_fatal_error
												error_handler.report_giaaa_error
											else
												l_actual_parameter := current_dynamic_system.dynamic_primary_type (l_actual_parameters.type (1), current_system.any_type)
												current_file.put_character ('(')
												print_type_declaration (l_actual_parameter, current_file)
												if l_actual_parameter.is_expanded then
													current_file.put_character ('*')
												end
												current_file.put_character (')')
												current_file.put_character ('(')
												print_argument_name (l_argument_name, current_file)
												current_file.put_character ('.')
												current_file.put_character ('a')
												current_file.put_character ('1')
												current_file.put_character (')')
											end
										else
											print_argument_name (l_argument_name, current_file)
										end
										i := i + l_max
									else
										current_file.put_character ('$')
										current_file.put_character (l_c_code.item (i))
										i := i + 1
									end
								else
									current_file.put_character ('$')
									current_file.put_character (c)
									i := i + 1
								end
							else
								current_file.put_character ('$')
							end
						else
							current_file.put_character (c)
							i := i + 1
						end
					end
				else
					current_file.put_string (l_c_code)
				end
				if l_has_result_type_cast then
					current_file.put_character (')')
				end
				if l_semicolon_needed then
					current_file.put_character (';')
				end
				current_file.put_new_line
			end
		end

	print_external_c_includes (a_include_filenames: STRING)
			-- Print C includes declarations to `header_file'.
			-- `a_include_filenames' are the filenames (with the
			-- < > or " " characters included) separated by
			-- commas.
		require
			a_include_filenames_not_void: a_include_filenames /= Void
		local
			l_splitter: ST_SPLITTER
			l_list: DS_LIST [STRING]
			l_cursor: DS_LIST_CURSOR [STRING]
			l_include_filename: STRING
		do
			create l_splitter.make_with_separators (",")
			l_list := l_splitter.split (a_include_filenames)
			l_cursor := l_list.new_cursor
			from l_cursor.start until l_cursor.after loop
				l_include_filename := l_cursor.item
				STRING_.left_adjust (l_include_filename)
				STRING_.right_adjust (l_include_filename)
				include_header_filename (l_include_filename, header_file)
				l_cursor.forth
			end
		end

	print_external_cpp_body (a_feature_name: detachable ET_FEATURE_NAME;
		a_arguments: detachable ET_FORMAL_ARGUMENT_LIST; a_result_type_set: detachable ET_DYNAMIC_TYPE_SET;
		a_cpp_class_type: STRING; a_signature_arguments, a_signature_result: detachable STRING; a_alias: detachable ET_EXTERNAL_ALIAS)
			-- Print body of external C++ function to `current_file'.
			-- If `a_feature_name' is Void then the name of the C++ function will be found in the alias.
			-- `a_cpp_class_type' is the name of the C++ class type.
			-- `a_signature_arguments' and `a_signature_result', if not Void,
			-- are the signature types declared in the Language part.
			-- `a_result_type_set' is not Void if the external feature is a query.
		require
			name_not_void: a_feature_name /= Void or else a_alias /= Void
			a_cpp_class_type_not_void: a_cpp_class_type /= Void
		local
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
			l_alias_value: ET_MANIFEST_STRING
			i, nb_args: INTEGER
			l_splitter: ST_SPLITTER
			l_list: DS_LIST [STRING]
			l_cursor: DS_LIST_CURSOR [STRING]
			l_name: ET_IDENTIFIER
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_type: ET_DYNAMIC_PRIMARY_TYPE
			l_actual_parameters: detachable ET_ACTUAL_PARAMETERS
			l_actual_parameter: ET_DYNAMIC_PRIMARY_TYPE
		do
			print_indentation
			if a_result_type_set /= Void then
				l_result_type := a_result_type_set.static_type.primary_type
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_declaration_type_cast_from_c (l_result_type, current_file)
				current_file.put_character ('(')
			end
			if a_arguments = Void or else a_arguments.is_empty then
-- TODO: error
			else
				nb_args := a_arguments.count
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_string (a_cpp_class_type)
				current_file.put_character ('*')
				current_file.put_character (')')
				l_name := a_arguments.formal_argument (1).name
				l_argument_type_set := dynamic_type_set (l_name)
				l_argument_type := l_argument_type_set.static_type.primary_type
				if l_argument_type.base_class.is_typed_pointer_class then
						-- The argument is declared of type 'TYPED_POINTER [XX]'.
						-- In that case we use the corresponding pointer (i.e.
						-- the first attribute of the object).
					l_actual_parameters := l_argument_type.base_type.actual_parameters
					if l_actual_parameters = Void or else l_actual_parameters.is_empty then
							-- Internal error: TYPED_POINTER [XX] has one generic parameter.
							-- This should have been checked already.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						l_actual_parameter := current_dynamic_system.dynamic_primary_type (l_actual_parameters.type (1), current_system.any_type)
						current_file.put_character ('(')
						print_type_declaration (l_actual_parameter, current_file)
						if l_actual_parameter.is_expanded then
							current_file.put_character ('*')
						end
						current_file.put_character (')')
						current_file.put_character ('(')
						print_argument_name (l_name, current_file)
						current_file.put_character ('.')
						current_file.put_character ('a')
						current_file.put_character ('1')
						current_file.put_character (')')
					end
				else
					print_argument_name (l_name, current_file)
				end
				current_file.put_character (')')
				current_file.put_string (c_arrow)
				if a_alias /= Void then
					l_alias_value := a_alias.manifest_string
					current_file.put_string (l_alias_value.value)
				elseif a_feature_name /= Void then
					current_file.put_string (a_feature_name.lower_name)
				else
					check precondition_name_not_void: False then end
				end
				current_file.put_character ('(')
				if a_signature_arguments /= Void then
					create l_splitter.make_with_separators (",")
					l_list := l_splitter.split (a_signature_arguments)
					if l_list.count /= nb_args - 1 then
-- TODO: error
					end
					l_cursor := l_list.new_cursor
					l_cursor.start
					from i := 2 until i > nb_args loop
						if i /= 2 then
							current_file.put_character (',')
						end
						if not l_cursor.after then
							current_file.put_character ('(')
							current_file.put_string (l_cursor.item)
							current_file.put_character (')')
							l_cursor.forth
						end
						l_name := a_arguments.formal_argument (i).name
						l_argument_type_set := dynamic_type_set (l_name)
						l_argument_type := l_argument_type_set.static_type.primary_type
						if l_argument_type.base_class.is_typed_pointer_class then
								-- The argument is declared of type 'TYPED_POINTER [XX]'.
								-- In that case we use the corresponding pointer (i.e.
								-- the first attribute of the object).
							l_actual_parameters := l_argument_type.base_type.actual_parameters
							if l_actual_parameters = Void or else l_actual_parameters.is_empty then
									-- Internal error: TYPED_POINTER [XX] has one generic parameter.
									-- This should have been checked already.
								set_fatal_error
								error_handler.report_giaaa_error
							else
								l_actual_parameter := current_dynamic_system.dynamic_primary_type (l_actual_parameters.type (1), current_system.any_type)
								current_file.put_character ('(')
								print_type_declaration (l_actual_parameter, current_file)
								if l_actual_parameter.is_expanded then
									current_file.put_character ('*')
								end
								current_file.put_character (')')
								current_file.put_character ('(')
								print_argument_name (l_name, current_file)
								current_file.put_character ('.')
								current_file.put_character ('a')
								current_file.put_character ('1')
								current_file.put_character (')')
							end
						else
							print_argument_name (l_name, current_file)
						end
						i := i + 1
					end
				else
					from i := 2 until i > nb_args loop
						if i /= 2 then
							current_file.put_character (',')
						end
						l_name := a_arguments.formal_argument (i).name
						l_argument_type_set := dynamic_type_set (l_name)
						l_argument_type := l_argument_type_set.static_type.primary_type
						if l_argument_type = current_dynamic_system.pointer_type then
								-- When compiling with C++, MSVC++ does not want to convert
								-- 'void*' to non-'void*' implicitly.
							current_file.put_string ("(char*)")
							print_argument_name (l_name, current_file)
						elseif l_argument_type.base_class.is_typed_pointer_class then
								-- The argument is declared of type 'TYPED_POINTER [XX]'.
								-- In that case we use the corresponding pointer (i.e.
								-- the first attribute of the object).
							l_actual_parameters := l_argument_type.base_type.actual_parameters
							if l_actual_parameters = Void or else l_actual_parameters.is_empty then
									-- Internal error: TYPED_POINTER [XX] has one generic parameter.
									-- This should have been checked already.
								set_fatal_error
								error_handler.report_giaaa_error
							else
								l_actual_parameter := current_dynamic_system.dynamic_primary_type (l_actual_parameters.type (1), current_system.any_type)
								current_file.put_character ('(')
								print_type_declaration (l_actual_parameter, current_file)
								if l_actual_parameter.is_expanded then
									current_file.put_character ('*')
								end
								current_file.put_character (')')
								current_file.put_character ('(')
								print_argument_name (l_name, current_file)
								current_file.put_character ('.')
								current_file.put_character ('a')
								current_file.put_character ('1')
								current_file.put_character (')')
							end
						else
							print_argument_name (l_name, current_file)
						end
						i := i + 1
					end
				end
				current_file.put_character (')')
			end
			if a_result_type_set /= Void then
				current_file.put_character (')')
			end
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_external_cpp_includes (a_include_filenames: STRING)
			-- Print C++ includes declarations to `header_file'.
			-- `a_include_filenames' are the filenames (with the
			-- < > or " " characters included) separated by
			-- commas.
		require
			a_include_filenames_not_void: a_include_filenames /= Void
		local
			l_splitter: ST_SPLITTER
			l_list: DS_LIST [STRING]
			l_cursor: DS_LIST_CURSOR [STRING]
			l_include_filename: STRING
		do
			create l_splitter.make_with_separators (",")
			l_list := l_splitter.split (a_include_filenames)
			l_cursor := l_list.new_cursor
			from l_cursor.start until l_cursor.after loop
				l_include_filename := l_cursor.item
				STRING_.left_adjust (l_include_filename)
				STRING_.right_adjust (l_include_filename)
				include_cpp_header_filename (l_include_filename, header_file)
				l_cursor.forth
			end
		end

	print_external_dllwin_body (a_feature_name: detachable ET_FEATURE_NAME;
		a_arguments: detachable ET_FORMAL_ARGUMENT_LIST; a_result_type_set: detachable ET_DYNAMIC_TYPE_SET;
		a_dll_file: STRING; a_signature_arguments, a_signature_result: detachable STRING;
		a_alias: detachable ET_EXTERNAL_ALIAS)
			-- Print body of external dllwin function to `current_file'.
			-- If `a_feature_name' is Void then the name of the C function in the DLL will be found in the alias.
			-- `a_dll_file' is the name of the DLL file.
			-- `a_signature_arguments' and `a_signature_result', if not Void,
			-- are the signature types declared in the Language part.
			-- `a_result_type_set' is not Void if the external feature is a query.
		require
			name_not_void: a_feature_name /= Void or else a_alias /= Void
			a_dll_file_not_void: a_dll_file /= Void
		local
			l_result_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_alias_value: ET_MANIFEST_STRING
			i, nb_args: INTEGER
			l_splitter: ST_SPLITTER
			l_list: DS_LIST [STRING]
			l_cursor: DS_LIST_CURSOR [STRING]
			l_name: ET_IDENTIFIER
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_type: ET_DYNAMIC_PRIMARY_TYPE
			l_actual_parameters: detachable ET_ACTUAL_PARAMETERS
			l_actual_parameter: ET_DYNAMIC_PRIMARY_TYPE
			l_function_name: STRING
		do
			include_runtime_header_file ("ge_exception.h", False, header_file)
			include_runtime_header_file ("ge_dll.h", False, header_file)
			print_indentation
			current_file.put_line ("static char done = 0;")
			print_indentation
			current_file.put_line ("static EIF_POINTER fp = NULL;")
			print_indentation
			current_file.put_line ("if (!done) {")
			indent
			print_indentation
			current_file.put_line ("HMODULE a_result;")
			print_indentation
			current_file.put_string ("a_result = GE_load_dll(")
			if a_dll_file.is_empty or else a_dll_file.item (1) /= '%"' then
				current_file.put_character ('%"')
				current_file.put_string (a_dll_file)
				current_file.put_character ('%"')
			else
				current_file.put_string (a_dll_file)
			end
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
			print_indentation
			current_file.put_line ("if (a_result == NULL) {")
			indent
			indent
			print_indentation
			current_file.put_string ("/* Cannot load library ")
			current_file.put_string (a_dll_file)
			current_file.put_line (" */")
			dedent
			print_indentation
			current_file.put_string (c_ge_raise)
			current_file.put_character ('(')
			current_file.put_string (c_ge_ex_prog)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
			print_indentation
			current_file.put_string ("fp = (EIF_POINTER)GetProcAddress(a_result,%"")
			if a_alias /= Void then
				l_alias_value := a_alias.manifest_string
				l_function_name := l_alias_value.value
			elseif a_feature_name /= Void then
				l_function_name := a_feature_name.lower_name
			else
				check precondition_name_not_void: False then end
			end
			current_file.put_string (l_function_name)
			current_file.put_line ("%");")
			print_indentation
			current_file.put_line ("if (fp == NULL) {")
			indent
			indent
			print_indentation
			current_file.put_string ("/* Cannot find entry point of ")
			current_file.put_string (l_function_name)
			current_file.put_line (" */")
			dedent
			print_indentation
			current_file.put_string (c_ge_raise)
			current_file.put_character ('(')
			current_file.put_string (c_ge_ex_prog)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
			print_indentation
			current_file.put_line ("done = (char) 1;")
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
			print_indentation
			if a_result_type_set /= Void then
				l_result_type := a_result_type_set.static_type.primary_type
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_declaration_type_cast_from_c (l_result_type, current_file)
				current_file.put_character ('(')
			end
			current_file.put_character ('(')
			current_file.put_character ('(')
			if l_result_type /= Void then
				if a_signature_result /= Void then
					current_file.put_string (a_signature_result)
				else
					print_type_declaration (l_result_type, current_file)
				end
			else
				current_file.put_string (c_void)
			end
			current_file.put_string (" (__stdcall *) ")
			current_file.put_character ('(')
			if a_arguments /= Void and then not a_arguments.is_empty then
				nb_args := a_arguments.count
				if a_signature_arguments /= Void then
					current_file.put_string (a_signature_arguments)
				else
					from i := 1 until i > nb_args loop
						if i /= 1 then
							current_file.put_character (',')
						end
						l_name := a_arguments.formal_argument (i).name
						l_argument_type_set := argument_type_set (i)
						l_argument_type := l_argument_type_set.static_type.primary_type
						if l_argument_type = current_dynamic_system.pointer_type then
								-- When compiling with C++, MSVC++ does not want to convert
								-- 'void*' to non-'void*' implicitly.
							current_file.put_string ("char*")
						elseif l_argument_type.base_class.is_typed_pointer_class then
								-- The argument is declared of type 'TYPED_POINTER [XX]'.
								-- In that case we use the corresponding pointer (i.e.
								-- the first attribute of the object).
							l_actual_parameters := l_argument_type.base_type.actual_parameters
							if l_actual_parameters = Void or else l_actual_parameters.is_empty then
									-- Internal error: TYPED_POINTER [XX] has one generic parameter.
									-- This should have been checked already.
								set_fatal_error
								error_handler.report_giaaa_error
							else
								l_actual_parameter := current_dynamic_system.dynamic_primary_type (l_actual_parameters.type (1), current_system.any_type)
								print_type_declaration (l_actual_parameter, current_file)
								if l_actual_parameter.is_expanded then
									current_file.put_character ('*')
								end
							end
						else
							print_type_declaration (l_argument_type, current_file)
						end
						i := i + 1
					end
				end
			end
			current_file.put_character (')')
			current_file.put_character (')')
			current_file.put_string ("fp")
			current_file.put_character (')')
			if a_arguments /= Void and then nb_args > 0 then
				current_file.put_character ('(')
				if a_signature_arguments /= Void then
					create l_splitter.make_with_separators (",")
					l_list := l_splitter.split (a_signature_arguments)
					if l_list.count /= nb_args then
-- TODO: error
					end
					l_cursor := l_list.new_cursor
					l_cursor.start
					from i := 1 until i > nb_args loop
						if i /= 1 then
							current_file.put_character (',')
						end
						if not l_cursor.after then
							current_file.put_character ('(')
							current_file.put_string (l_cursor.item)
							current_file.put_character (')')
							l_cursor.forth
						end
						l_name := a_arguments.formal_argument (i).name
						l_argument_type_set := argument_type_set (i)
						l_argument_type := l_argument_type_set.static_type.primary_type
						if l_argument_type.base_class.is_typed_pointer_class then
								-- The argument is declared of type 'TYPED_POINTER [XX]'.
								-- In that case we use the corresponding pointer (i.e.
								-- the first attribute of the object).
							l_actual_parameters := l_argument_type.base_type.actual_parameters
							if l_actual_parameters = Void or else l_actual_parameters.is_empty then
									-- Internal error: TYPED_POINTER [XX] has one generic parameter.
									-- This should have been checked already.
								set_fatal_error
								error_handler.report_giaaa_error
							else
								l_actual_parameter := current_dynamic_system.dynamic_primary_type (l_actual_parameters.type (1), current_system.any_type)
								current_file.put_character ('(')
								print_type_declaration (l_actual_parameter, current_file)
								if l_actual_parameter.is_expanded then
									current_file.put_character ('*')
								end
								current_file.put_character (')')
								current_file.put_character ('(')
								print_argument_name (l_name, current_file)
								current_file.put_character ('.')
								current_file.put_character ('a')
								current_file.put_character ('1')
								current_file.put_character (')')
							end
						else
							print_argument_name (l_name, current_file)
						end
						i := i + 1
					end
				else
					from i := 1 until i > nb_args loop
						if i /= 1 then
							current_file.put_character (',')
						end
						l_name := a_arguments.formal_argument (i).name
						l_argument_type_set := argument_type_set (i)
						l_argument_type := l_argument_type_set.static_type.primary_type
						if l_argument_type = current_dynamic_system.pointer_type then
								-- When compiling with C++, MSVC++ does not want to convert
								-- 'void*' to non-'void*' implicitly.
							current_file.put_string ("(char*)")
							print_argument_name (l_name, current_file)
						elseif l_argument_type.base_class.is_typed_pointer_class then
								-- The argument is declared of type 'TYPED_POINTER [XX]'.
								-- In that case we use the corresponding pointer (i.e.
								-- the first attribute of the object).
							l_actual_parameters := l_argument_type.base_type.actual_parameters
							if l_actual_parameters = Void or else l_actual_parameters.is_empty then
									-- Internal error: TYPED_POINTER [XX] has one generic parameter.
									-- This should have been checked already.
								set_fatal_error
								error_handler.report_giaaa_error
							else
								l_actual_parameter := current_dynamic_system.dynamic_primary_type (l_actual_parameters.type (1), current_system.any_type)
								current_file.put_character ('(')
								print_type_declaration (l_actual_parameter, current_file)
								if l_actual_parameter.is_expanded then
									current_file.put_character ('*')
								end
								current_file.put_character (')')
								current_file.put_character ('(')
								print_argument_name (l_name, current_file)
								current_file.put_character ('.')
								current_file.put_character ('a')
								current_file.put_character ('1')
								current_file.put_character (')')
							end
						else
							print_argument_name (l_name, current_file)
						end
						i := i + 1
					end
				end
				current_file.put_character (')')
			else
				current_file.put_character ('(')
				current_file.put_character (')')
			end
			if a_result_type_set /= Void then
				current_file.put_character (')')
			end
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_internal_function (a_feature: ET_INTERNAL_FUNCTION)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
			if current_feature.static_feature /= a_feature then
					-- Internal error: inconsistent `current_feature'.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif in_static_feature then
				print_internal_feature (a_feature, False)
			else
				if current_feature.is_regular then
					print_internal_feature (a_feature, False)
				end
				if current_feature.is_address then
					print_address_routine (a_feature)
				end
			end
		end

	print_internal_procedure (a_feature: ET_INTERNAL_PROCEDURE)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
			if current_feature.static_feature /= a_feature then
					-- Internal error: inconsistent `current_feature'.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif in_static_feature then
				print_internal_feature (a_feature, False)
			else
				if current_feature.is_regular then
					print_internal_feature (a_feature, False)
				end
				if current_feature.is_creation then
					print_internal_feature (a_feature, True)
				end
				if current_feature.is_address then
					print_address_routine (a_feature)
				end
			end
		end

	print_internal_feature (a_feature: ET_INTERNAL_FEATURE; a_creation: BOOLEAN)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
			is_creation: a_creation implies current_feature.is_creation
			one_kind: (not in_static_feature and not a_creation) or (in_static_feature xor a_creation)
		local
			l_result_type_set: detachable ET_DYNAMIC_TYPE_SET
			l_result_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_type: ET_DYNAMIC_PRIMARY_TYPE
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			i, nb_args: INTEGER
			old_file: KI_TEXT_OUTPUT_STREAM
			l_name: ET_IDENTIFIER
			l_is_empty: BOOLEAN
			l_once_kind: INTEGER
			l_once_index: INTEGER
			l_is_once: BOOLEAN
			l_is_once_per_process: BOOLEAN
			l_is_once_per_thread: BOOLEAN
			l_is_once_per_object: BOOLEAN
			l_is_self_initialized_attribute: BOOLEAN
		do
			old_file := current_file
			current_file := current_function_header_buffer
			print_feature_name_comment (a_feature, current_type, header_file)
			print_feature_name_comment (a_feature, current_type, current_file)
			l_result_type_set := current_feature.result_type_set
			if l_result_type_set /= Void then
				l_result_type := l_result_type_set.static_type.primary_type
			end
				--
				-- Print signature to `header_file' and `current_file'.
				--
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			if l_result_type /= Void then
				print_type_declaration (l_result_type, header_file)
				print_type_declaration (l_result_type, current_file)
			elseif a_creation then
				print_type_declaration (current_type, header_file)
				print_type_declaration (current_type, current_file)
			else
				header_file.put_string (c_void)
				current_file.put_string (c_void)
			end
			header_file.put_character (' ')
			current_file.put_character (' ')
			if in_static_feature then
				print_static_routine_name (current_feature, current_type, header_file)
				print_static_routine_name (current_feature, current_type, current_file)
				header_file.put_character ('(')
				current_file.put_character ('(')
				header_file.put_string (c_ge_context)
				header_file.put_character ('*')
				header_file.put_character (' ')
				header_file.put_string (c_ac)
				current_file.put_string (c_ge_context)
				current_file.put_character ('*')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
			elseif a_creation then
				print_creation_procedure_name (current_feature, current_type, header_file)
				print_creation_procedure_name (current_feature, current_type, current_file)
				header_file.put_character ('(')
				current_file.put_character ('(')
				header_file.put_string (c_ge_context)
				header_file.put_character ('*')
				header_file.put_character (' ')
				header_file.put_string (c_ac)
				current_file.put_string (c_ge_context)
				current_file.put_character ('*')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
			else
				print_routine_name (current_feature, current_type, header_file)
				print_routine_name (current_feature, current_type, current_file)
				header_file.put_character ('(')
				current_file.put_character ('(')
				header_file.put_string (c_ge_context)
				header_file.put_character ('*')
				header_file.put_character (' ')
				header_file.put_string (c_ac)
				header_file.put_character (',')
				header_file.put_character (' ')
				current_file.put_string (c_ge_context)
				current_file.put_character ('*')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_type_declaration (current_type, header_file)
				print_type_declaration (current_type, current_file)
				if current_type.is_expanded then
					header_file.put_character ('*')
					current_file.put_character ('*')
				end
				header_file.put_character (' ')
				current_file.put_character (' ')
				print_current_name (header_file)
				print_current_name (current_file)
			end
			l_arguments := a_feature.arguments
			if l_arguments /= Void then
				nb_args := l_arguments.count
				if nb_args > 0 then
					from i := 1 until i > nb_args loop
						header_file.put_character (',')
						current_file.put_character (',')
						header_file.put_character (' ')
						current_file.put_character (' ')
						l_argument_type_set := argument_type_set (i)
						l_argument_type := l_argument_type_set.static_type.primary_type
						print_type_declaration (l_argument_type, header_file)
						print_type_declaration (l_argument_type, current_file)
						header_file.put_character (' ')
						current_file.put_character (' ')
						l_name := l_arguments.formal_argument (i).name
						print_argument_name (l_name, header_file)
						print_argument_name (l_name, current_file)
						i := i + 1
					end
				end
			end
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
			current_file.put_character ('{')
			current_file.put_new_line
			indent
				-- Call stack.
			if exception_trace_mode then
				l_is_empty := not a_creation and then (not attached a_feature.compound as l_compound or else l_compound.is_empty) and then a_feature.rescue_clause = Void
				if not l_is_empty then
					print_indentation
					current_file.put_string (c_ge_call)
					current_file.put_character (' ')
					current_file.put_string (c_tc)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('{')
					if current_in_exception_trace then
						if in_static_feature or a_creation then
							current_file.put_character ('0')
						else
							print_current_name (current_file)
						end
						current_file.put_character (',')
					end
					print_escaped_string (current_type.base_class.upper_name)
					current_file.put_character (',')
					print_escaped_string (a_feature.lower_name)
					current_file.put_character (',')
					current_file.put_string (c_ac)
					current_file.put_string (c_arrow)
					current_file.put_string (c_call)
					current_file.put_character ('}')
					current_file.put_character (';')
					current_file.put_new_line
				end
			end
				--
				-- Print body to `current_file'.
				--
			current_file := current_function_body_buffer
			if a_feature.is_once then
				l_is_once := True
				l_is_once_per_process := a_feature.is_once_per_process
				l_is_once_per_thread := a_feature.is_once_per_thread
				l_is_once_per_object := a_feature.is_once_per_object
				if l_is_once_per_process or l_is_once_per_thread then
					register_once_feature (current_feature)
					l_once_index := once_features.value (a_feature.implementation_feature)
					l_once_kind := once_kind (current_feature)
				end
				if l_is_once_per_object then
						-- Create once-per-object data for 'Current' object if not created yet.
					if multithreaded_mode then
						print_once_per_object_global_mutex_lock
					end
					print_indentation
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					current_file.put_character ('!')
					print_attribute_onces_access (tokens.current_keyword, current_type, False)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					print_indentation
					print_attribute_onces_access (tokens.current_keyword, current_type, False)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_ge_new_once_per_object_data)
					current_file.put_integer (current_type.id)
					current_file.put_character ('(')
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
					if multithreaded_mode then
						print_once_per_object_global_mutex_unlock
					end
				end
				if not multithreaded_mode or else l_is_once_per_thread then
					print_indentation
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_once_status (current_feature, l_once_kind, l_once_index)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					print_raise_once_exception_if_any (current_feature, l_once_kind, l_once_index)
					print_return_once_value (current_feature, l_once_kind, l_once_index)
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_character (' ')
					current_file.put_string (c_else)
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
				else
					if multithreaded_mode and then l_is_once_per_process then
						print_indentation
						current_file.put_string (c_if)
						current_file.put_character (' ')
						current_file.put_character ('(')
						print_thread_safe_once_per_process_status (current_feature, l_once_kind, l_once_index)
						current_file.put_character (')')
						current_file.put_character (' ')
						current_file.put_character ('{')
						current_file.put_new_line
						indent
						print_raise_thread_safe_once_exception_if_any (current_feature, l_once_kind, l_once_index)
						print_return_thread_safe_once_value (current_feature, l_once_kind, l_once_index)
						dedent
						print_indentation
						current_file.put_character ('}')
						current_file.put_character (' ')
						current_file.put_string (c_else)
						current_file.put_character (' ')
						current_file.put_character ('{')
						current_file.put_new_line
						indent
					end
					print_indentation
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					current_file.put_character ('!')
					print_once_mutex_try_lock (current_feature, l_once_kind, l_once_index)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
						-- Wait for other thread to complete (lock/unlock).
					print_once_mutex_lock (current_feature, l_once_kind, l_once_index)
					print_once_mutex_unlock (current_feature, l_once_kind, l_once_index)
					if l_result_type /= Void then
						print_assign_once_value_to_result (current_feature, l_once_kind, l_once_index)
						if l_is_once_per_process then
							print_assign_result_to_thread_safe_once_value (current_feature, l_once_kind, l_once_index)
						end
					end
					if l_is_once_per_process then
						print_assign_once_exception_to_thread_safe_once_exception (current_feature, l_once_kind, l_once_index)
						print_assign_called_to_thread_safe_once_status (current_feature, l_once_kind, l_once_index)
					end
					print_raise_once_exception_if_any (current_feature, l_once_kind, l_once_index)
					print_return_statement (a_feature)
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_character (' ')
					current_file.put_string (c_else)
					current_file.put_character (' ')
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_once_status_is_completed (current_feature, l_once_kind, l_once_index)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					if l_result_type /= Void then
						print_assign_once_value_to_result (current_feature, l_once_kind, l_once_index)
						if l_is_once_per_process then
							print_assign_result_to_thread_safe_once_value (current_feature, l_once_kind, l_once_index)
						end
					end
					if l_is_once_per_process then
						print_assign_once_exception_to_thread_safe_once_exception (current_feature, l_once_kind, l_once_index)
						print_assign_called_to_thread_safe_once_status (current_feature, l_once_kind, l_once_index)
					end
					print_once_mutex_unlock (current_feature, l_once_kind, l_once_index)
					print_raise_once_exception_if_any (current_feature, l_once_kind, l_once_index)
					print_return_statement (a_feature)
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_character (' ')
					current_file.put_string (c_else)
					current_file.put_character (' ')
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_once_status (current_feature, l_once_kind, l_once_index)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					print_return_once_value (current_feature, l_once_kind, l_once_index)
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_character (' ')
					current_file.put_string (c_else)
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
				end
				print_assign_called_to_once_status (current_feature, l_once_kind, l_once_index)
				if l_result_type /= Void then
						-- The following is to make sure that if a once-function is called
						-- recursively, the semantics specified by ECMA will be satisfied.
						-- ECMA 367-2 section 8.23.22 page 124 says that the recursive calls
						-- to the once-function should return the value of 'Result' as it
						-- was when the recursive calls occurred.
					print_assign_result_to_once_value (current_feature, l_once_kind, l_once_index)
				end
				dedent
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
				if multithreaded_mode and then l_is_once_per_process then
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
				end
			elseif a_feature.is_attribute then
					-- Self-initialized attribute.
				l_is_self_initialized_attribute := True
					-- Return the value of the attribute if already initialized.
				print_indentation
				current_file.put_string (c_if)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_attribute_access (current_feature, tokens.current_keyword, current_type, False)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				indent
				current_file.put_string (c_return)
				current_file.put_character (' ')
				print_attribute_access (current_feature, tokens.current_keyword, current_type, False)
				current_file.put_character (';')
				current_file.put_new_line
				dedent
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
			end
				-- Call stack.
			if exception_trace_mode and then not l_is_empty then
				print_indentation
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_call)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_string (c_tc_address)
				current_file.put_character (';')
				current_file.put_new_line
			end
			print_feature_trace_message_call (True)
			if a_creation then
				print_malloc_current (a_feature)
				if exception_trace_mode and then current_in_exception_trace then
					print_indentation
					current_file.put_string (c_tc)
					current_file.put_character ('.')
					current_file.put_string (c_object)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_current_name (current_file)
					current_file.put_character (';')
					current_file.put_new_line
				end
			end
			print_internal_feature_body_declaration (a_feature, l_result_type)
			print_feature_trace_message_call (False)
				-- Call stack.
			if exception_trace_mode and then not l_is_empty then
				print_indentation
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_call)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_string (c_tc)
				current_file.put_character ('.')
				current_file.put_string (c_caller)
				current_file.put_character (';')
				current_file.put_new_line
			end
			if l_is_once then
				if l_result_type /= Void and then (l_result_type.is_expanded and then not l_result_type.is_basic) then
						-- The following is to make sure that if a once-function is called
						-- recursively, the semantics specified by ECMA will be satisfied.
						-- ECMA 367-2 section 8.23.22 page 124 says that the recursive calls
						-- to the once-function should return the value of 'Result' as it
						-- was when the recursive calls occurred.
						-- There is a special treatment for once-per-process and once-per-thread
						-- functions of non-basic expanded types because a boxed version of the
						-- Result (and not the Result itself) is kept in the 'GE_onces' C struct.
						-- So we usede the content of the boxed value in place of Result
						-- in case its fields get modified. Likewise for once-per-object functions
						-- even though the Result is not boxed in that case. (See `print_result'.)
						-- Set it back to Result here.
					print_assign_once_value_to_result (current_feature, l_once_kind, l_once_index)
				end
				if multithreaded_mode then
					if l_is_once_per_process then
						if l_result_type /= Void then
							print_assign_result_to_thread_safe_once_value (current_feature, l_once_kind, l_once_index)
						end
						print_assign_called_to_thread_safe_once_status (current_feature, l_once_kind, l_once_index)
					end
					if not l_is_once_per_thread then
						print_assign_completed_to_once_status (current_feature, l_once_kind, l_once_index)
						print_once_mutex_unlock (current_feature, l_once_kind, l_once_index)
					end
				end
			elseif l_is_self_initialized_attribute then
					-- Record value of self-initialized attribute.
				print_indentation
				print_attribute_access (current_feature, tokens.current_keyword, current_type, False)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
			end
			if l_result_type /= Void then
				print_return_statement (a_feature)
			elseif a_creation then
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				if current_type.is_expanded then
					current_file.put_character ('*')
				end
				print_current_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
			end
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
				--
				-- Clean up.
				--
			current_file := old_file
			reset_temp_variables
				-- Flush to file.
			flush_to_c_file
		end

	print_internal_feature_body_declaration (a_feature: ET_INTERNAL_FEATURE_CLOSURE; a_result_type: detachable ET_DYNAMIC_PRIMARY_TYPE)
			-- Print the local variables declaration, the compound and the
			-- rescue clause of `a_feature' to `current_file'.
		require
			a_feature_not_void: a_feature /= Void
		local
			i, nb: INTEGER
			l_name: ET_IDENTIFIER
			l_local_type_set: ET_DYNAMIC_TYPE_SET
			l_local_type: ET_DYNAMIC_PRIMARY_TYPE
			l_rescue: detachable ET_COMPOUND
			old_file: KI_TEXT_OUTPUT_STREAM
			l_result_written_in_body: BOOLEAN
			l_result_read_in_body: BOOLEAN
			l_result_written_in_rescue: BOOLEAN
			l_result_read_in_rescue: BOOLEAN
			l_is_once: BOOLEAN
			l_once_kind: INTEGER
			l_once_index: INTEGER
		do
			old_file := current_file
			current_file := current_function_header_buffer
				--
				-- Declaration of variables.
				--
				-- Variable for rescue chain.
			if current_agent = Void then
				if current_feature.is_once then
					l_is_once := True
					once_features.search (current_feature.static_feature.implementation_feature)
					if once_features.found then
						l_once_index := once_features.found_item
						l_once_kind := once_kind (current_feature)
					end
				end
			end
			l_rescue := a_feature.rescue_clause
			if l_rescue /= Void or l_is_once then
				print_indentation
				current_file.put_string (c_ge_rescue)
				current_file.put_character (' ')
				current_file.put_character ('r')
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_string (c_uint32_t)
				current_file.put_character (' ')
				current_file.put_string (c_tr)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_in_rescue)
				current_file.put_character (';')
				current_file.put_new_line
			end
				--
				-- Instructions.
				--
			current_file := current_function_body_buffer
			reset_rescue_data
			locals_written := locals_written_in_rescue
			locals_read := locals_read_in_rescue
			if l_rescue /= Void or l_is_once then
				print_indentation
				current_file.put_string (c_if)
				current_file.put_character (' ')
				current_file.put_character ('(')
				current_file.put_string (c_ge_setjmp)
				current_file.put_character ('(')
				current_file.put_character ('r')
				current_file.put_character ('.')
				current_file.put_character ('j')
				current_file.put_character ('b')
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('!')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('0')
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				indent
				if exception_trace_mode and then l_rescue /= Void then
					print_indentation
					current_file.put_string (c_ac)
					current_file.put_string (c_arrow)
					current_file.put_string (c_call)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_tc_address)
					current_file.put_character (';')
					current_file.put_new_line
				end
				print_indentation
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_in_rescue)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_string (c_tr)
				current_file.put_character (' ')
				current_file.put_character ('+')
				current_file.put_character (' ')
				current_file.put_character ('1')
				current_file.put_character (';')
				current_file.put_new_line
				if l_rescue /= Void then
					has_rescue := True
					print_compound (l_rescue)
				end
				if l_is_once then
					print_assign_last_exception_to_once_exception (current_feature, l_once_kind, l_once_index)
					if multithreaded_mode then
						if a_feature.is_once_per_process then
							print_assign_once_exception_to_thread_safe_once_exception (current_feature, l_once_kind, l_once_index)
							print_assign_called_to_thread_safe_once_status (current_feature, l_once_kind, l_once_index)
						end
						if not current_feature.is_once_per_thread then
							print_assign_completed_to_once_status (current_feature, l_once_kind, l_once_index)
							print_once_mutex_unlock (current_feature, l_once_kind, l_once_index)
						end
					end
				end
				if l_rescue /= Void then
					print_indentation
					current_file.put_string (c_ge_raise)
					current_file.put_character ('(')
					current_file.put_string (c_ge_ex_fail)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				else
					print_indentation
					current_file.put_string (c_ge_jump_to_last_rescue)
					current_file.put_character ('(')
					current_file.put_string (c_ac)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				end
				dedent
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
				current_file.put_string (c_ge_retry)
				current_file.put_character (':')
				current_file.put_new_line
				print_indentation
				current_file.put_character ('r')
				current_file.put_character ('.')
				current_file.put_string (c_previous)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_last_rescue)
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_last_rescue)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('&')
				current_file.put_character ('r')
				current_file.put_character (';')
				current_file.put_new_line
			end
			l_result_written_in_rescue := result_written
			l_result_read_in_rescue := result_read
			locals_written := locals_written_in_body
			locals_read := locals_read_in_body
			if attached a_feature.compound as l_compound then
				print_compound (l_compound)
			end
			l_result_written_in_body := result_written
			l_result_read_in_body := result_read
			if l_rescue /= Void or l_is_once then
				print_indentation
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_last_rescue)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('r')
				current_file.put_character ('.')
				current_file.put_string (c_previous)
				current_file.put_character (';')
				current_file.put_new_line
			end
			if a_result_type /= Void then
					-- The 'Result' entity is always implicitly read in the body to return its value.
				l_result_read_in_body := True
			end
				--
				-- Local variables and result declaration.
				--
			current_file := current_function_header_buffer
				-- Variable for 'Result' entity.
			if a_result_type /= Void then
				print_indentation
				print_type_declaration (a_result_type, current_file)
				current_file.put_character (' ')
				if has_rescue and then (l_result_written_in_body or l_result_written_in_rescue) and then (has_retry or l_result_read_in_rescue) then
						-- The implementation of the rescue mechanism in C uses 'setjmp'
						-- and 'longjmp'. The use of these two C functions requires that
						-- any local variable modified between the call to 'setjmp' and
						-- the call to 'longjmp' to be declared as 'volatile', otherwise its
						-- value may be lost after calling 'longjmp' if the C optimizer
						-- decided to implement it with 'register'. For more details, see:
						--  http://www.programmersheaven.com/articles/pathak/article1.htm
						--  http://www.freetype.org/david/reliable-c.html#annex-A
						--  http://msdn2.microsoft.com/en-us/library/xe7acxfb(VS.80).aspx
					current_file.put_string (c_volatile)
					current_file.put_character (' ')
				end
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_default_entity_value (a_result_type, current_file)
				current_file.put_character (';')
				current_file.put_new_line
			end
				-- Local variables.
			if attached a_feature.locals as l_locals then
				nb := l_locals.count
				from i := 1 until i > nb loop
					l_name := l_locals.local_variable (i).name
					l_local_type_set := dynamic_type_set (l_name)
					l_local_type := l_local_type_set.static_type.primary_type
					print_indentation
					print_type_declaration (l_local_type, current_file)
					current_file.put_character (' ')
					if has_rescue and then (locals_written_in_body.has (l_name) or locals_written_in_rescue.has (l_name)) and then (has_retry or locals_read_in_rescue.has (l_name)) then
							-- The implementation of the rescue mechanism in C uses 'setjmp'
							-- and 'longjmp'. The use of these two C functions requires that
							-- any local variable modified between the call to 'setjmp' and
							-- the call to 'longjmp' to be declared as 'volatile', otherwise its
							-- value may be lost after calling 'longjmp' if the C optimizer
							-- decided to implement it with 'register'. For more details, see:
							--  http://www.programmersheaven.com/articles/pathak/article1.htm
							--  http://www.freetype.org/david/reliable-c.html#annex-A
							--  http://msdn2.microsoft.com/en-us/library/xe7acxfb(VS.80).aspx
						current_file.put_string (c_volatile)
						current_file.put_character (' ')
					end
					print_local_name (l_name, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_default_entity_value (l_local_type, current_file)
					current_file.put_character (';')
					current_file.put_new_line
					i := i + 1
				end
			end
				-- Clean up.
			reset_rescue_data
			current_file := old_file
		end

	print_once_function (a_feature: ET_ONCE_FUNCTION)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
			print_internal_function (a_feature)
		end

	print_once_procedure (a_feature: ET_ONCE_PROCEDURE)
			-- Print `a_feature' to `current_file' and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
			print_internal_procedure (a_feature)
		end

	print_attribute (a_feature: ET_ATTRIBUTE)
			-- Print function wrapper for `a_feature' to `current_file'
			-- and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
			if current_feature.static_feature /= a_feature then
					-- Internal error: inconsistent `current_feature'.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_attribute_wrapper (a_feature)
			end
		end

	print_extended_attribute (a_feature: ET_EXTENDED_ATTRIBUTE)
			-- Print function wrapper for `a_feature' to `current_file'
			-- and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
			if a_feature.has_self_initializing_code then
				print_internal_feature (a_feature, False)
			else
				print_attribute (a_feature)
			end
		end

	print_constant_attribute (a_feature: ET_CONSTANT_ATTRIBUTE)
			-- Print function wrapper for `a_feature' to `current_file'
			-- and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
			if current_feature.static_feature /= a_feature then
					-- Internal error: inconsistent `current_feature'.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif in_static_feature then
				print_attribute_wrapper (a_feature)
			else
				if current_feature.is_regular then
					print_attribute_wrapper (a_feature)
				end
			end
		end

	print_unique_attribute (a_feature: ET_UNIQUE_ATTRIBUTE)
			-- Print function wrapper for `a_feature' to `current_file'
			-- and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
		do
			if current_feature.static_feature /= a_feature then
					-- Internal error: inconsistent `current_feature'.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif in_static_feature then
				print_attribute_wrapper (a_feature)
			else
				if current_feature.is_regular then
					print_attribute_wrapper (a_feature)
				end
			end
		end

	print_attribute_wrapper (a_feature: ET_QUERY)
			-- Print function wrapper for `a_feature' to `current_file'
			-- and its signature to `header_file'.
		require
			a_feature_not_void: a_feature /= Void
			no_arguments: a_feature.arguments = Void
			valid_feature: current_feature.static_feature = a_feature
		local
			old_file: KI_TEXT_OUTPUT_STREAM
			l_result_type_set: detachable ET_DYNAMIC_TYPE_SET
			l_result_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_wrapper_expression: ET_UNQUALIFIED_CALL_EXPRESSION
			l_name: ET_FEATURE_NAME
			l_result: ET_RESULT
			l_index: INTEGER
		do
			old_file := current_file
			current_file := current_function_header_buffer
				--
				-- Print signature to `header_file' and `current_file'.
				--
			print_feature_name_comment (a_feature, current_type, header_file)
			print_feature_name_comment (a_feature, current_type, current_file)
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			l_result_type_set := current_feature.result_type_set
			if l_result_type_set = Void then
					-- Internal error: a query has a result type set.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_result_type := l_result_type_set.static_type.primary_type
				print_type_declaration (l_result_type, header_file)
				print_type_declaration (l_result_type, current_file)
				header_file.put_character (' ')
				current_file.put_character (' ')
				if in_static_feature then
					print_static_routine_name (current_feature, current_type, header_file)
					print_static_routine_name (current_feature, current_type, current_file)
					header_file.put_character ('(')
					current_file.put_character ('(')
					header_file.put_string (c_ge_context)
					header_file.put_character ('*')
					header_file.put_character (' ')
					header_file.put_string (c_ac)
					current_file.put_string (c_ge_context)
					current_file.put_character ('*')
					current_file.put_character (' ')
					current_file.put_string (c_ac)
				else
					print_routine_name (current_feature, current_type, header_file)
					print_routine_name (current_feature, current_type, current_file)
					header_file.put_character ('(')
					current_file.put_character ('(')
					header_file.put_string (c_ge_context)
					header_file.put_character ('*')
					header_file.put_character (' ')
					header_file.put_string (c_ac)
					header_file.put_character (',')
					header_file.put_character (' ')
					current_file.put_string (c_ge_context)
					current_file.put_character ('*')
					current_file.put_character (' ')
					current_file.put_string (c_ac)
					current_file.put_character (',')
					current_file.put_character (' ')
					print_type_declaration (current_type, header_file)
					print_type_declaration (current_type, current_file)
					if current_type.is_expanded then
						header_file.put_character ('*')
						current_file.put_character ('*')
					end
					header_file.put_character (' ')
					current_file.put_character (' ')
					print_current_name (header_file)
					print_current_name (current_file)
				end
				header_file.put_character (')')
				current_file.put_character (')')
				header_file.put_character (';')
				header_file.put_new_line
				current_file.put_new_line
					--
					-- Print body to `current_file'.
					--
				current_file.put_character ('{')
				current_file.put_new_line
				indent
					--
					-- Declaration of variables.
					--
					-- Variable for exception trace.
				if exception_trace_mode then
					print_indentation
					current_file.put_string (c_ge_call)
					current_file.put_character (' ')
					current_file.put_string (c_tc)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('{')
					if current_in_exception_trace then
						print_current_name (current_file)
						current_file.put_character (',')
					end
					print_escaped_string (current_type.base_class.upper_name)
					current_file.put_character (',')
					print_escaped_string (a_feature.lower_name)
					current_file.put_character (',')
					current_file.put_string (c_ac)
					current_file.put_string (c_arrow)
					current_file.put_string (c_call)
					current_file.put_character ('}')
					current_file.put_character (';')
					current_file.put_new_line
				end
					-- Variable for 'Result' entity.
				print_indentation
				print_type_declaration (l_result_type, current_file)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_default_entity_value (l_result_type, current_file)
				current_file.put_character (';')
				current_file.put_new_line
					--
					-- Instructions.
					--
				current_file := current_function_body_buffer
					-- Call stack.
				if exception_trace_mode then
					print_indentation
					current_file.put_string (c_ac)
					current_file.put_string (c_arrow)
					current_file.put_string (c_call)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_tc_address)
					current_file.put_character (';')
					current_file.put_new_line
				end
					-- Prepare dynamic type sets of wrapper feature.
				extra_dynamic_type_sets.force_last (l_result_type_set)
				l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
					-- Prepare call expression to attribute feature.
				l_name := a_feature.name
				l_name.set_seed (a_feature.first_seed)
				l_wrapper_expression := new_unqualified_call_expression (l_name, Void)
				l_wrapper_expression.set_index (l_index)
					-- Print assignment of call expression to result entity.
				l_result := tokens.result_keyword
				print_assignment_operand (l_wrapper_expression, l_result_type_set, l_result, l_result_type)
				fill_call_operands (1)
				if call_operands.first /= l_result then
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_expression (call_operands.first)
					current_file.put_character (';')
					current_file.put_new_line
				end
					-- Clean up.
				call_operands.wipe_out
				extra_dynamic_type_sets.remove_last
				free_unqualified_call_expression (l_wrapper_expression)
					-- Call stack.
				if exception_trace_mode then
					print_indentation
					current_file.put_string (c_ac)
					current_file.put_string (c_arrow)
					current_file.put_string (c_call)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_tc)
					current_file.put_character ('.')
					current_file.put_string (c_caller)
					current_file.put_character (';')
					current_file.put_new_line
				end
					-- Return attribute value.
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				dedent
				current_file.put_character ('}')
				current_file.put_new_line
				current_file.put_new_line
			end
				--
				-- Clean up.
				--
			current_file := old_file
			reset_temp_variables
				-- Flush to file.
			flush_to_c_file
		end

feature {NONE} -- Instruction generation

	print_across_instruction (an_instruction: ET_ACROSS_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		do
			print_iteration_instruction (an_instruction)
		end

	print_assigner_instruction (an_instruction: ET_ASSIGNER_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		do
			print_qualified_call_instruction (an_instruction)
		end

	print_assignment (an_instruction: ET_ASSIGNMENT)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		local
			l_source: ET_EXPRESSION
			l_source_type_set: ET_DYNAMIC_TYPE_SET
			l_source_type: ET_DYNAMIC_TYPE
			l_target: ET_WRITABLE
			l_target_type_set: ET_DYNAMIC_TYPE_SET
			l_target_type: ET_DYNAMIC_TYPE
			l_once_kind: INTEGER
			l_once_index: INTEGER
		do
			if line_generation_mode then
				print_position (an_instruction.position, current_feature.static_feature.implementation_class)
			end
			l_source := an_instruction.source
			l_source_type_set := dynamic_type_set (l_source)
			l_target := an_instruction.target
			l_target_type_set := dynamic_type_set (l_target)
			l_source_type := l_source_type_set.static_type
			l_target_type := l_target_type_set.static_type
			print_assignment_operand (l_source, l_source_type_set, l_target, l_target_type)
			fill_call_operands (1)
			if call_operands.first /= l_target then
				print_indentation
				print_writable (l_target)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attachment_expression (call_operands.first, l_source_type_set, l_target_type)
				current_file.put_character (';')
				current_file.put_new_line
			end
			call_operands.wipe_out
			if current_agent = Void and then current_feature.is_once then
					-- Make sure that if a once-function is called
					-- recursively, the semantics specified by ECMA will be satisfied.
					-- ECMA 367-2 section 8.23.22 page 124 says that the recursive calls
					-- to the once-function should return the value of 'Result' as it
					-- was when the recursive calls occurred.
					-- There is a special treatment for once-per-process and once-per-thread
					-- functions of non-basic expanded types because a boxed version of the
					-- Result (and not the Result itself) is kept in the 'GE_onces' C struct.
					-- So we need to use the content of the boxed value in place of Result
					-- in case its fields get modified. Likewise for once-per-object functions
					-- even though the Result is not boxed in that case. (See `print_result'.)
				if l_target.is_result and then not (l_target_type.is_expanded and then not l_target_type.is_basic) then
					once_features.search (current_feature.static_feature.implementation_feature)
					if once_features.found then
						l_once_index := once_features.found_item
						l_once_kind := once_kind (current_feature)
					end
					print_assign_result_to_once_value (current_feature, l_once_kind, l_once_index)
				end
			end
		end

	print_assignment_attempt (an_instruction: ET_ASSIGNMENT_ATTEMPT)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		local
			i, nb: INTEGER
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_source: ET_EXPRESSION
			l_source_type_set: ET_DYNAMIC_TYPE_SET
			l_source_type: ET_DYNAMIC_TYPE
			l_target: ET_WRITABLE
			l_target_type_set: ET_DYNAMIC_TYPE_SET
			l_target_type: ET_DYNAMIC_TYPE
			l_target_primary_type: ET_DYNAMIC_PRIMARY_TYPE
			l_conforming_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			l_non_conforming_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			l_conforming_type_set: ET_DYNAMIC_STANDALONE_TYPE_SET
			l_old_conforming_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			l_can_be_void: BOOLEAN
			l_once_index: INTEGER
			l_once_kind: INTEGER
		do
			if line_generation_mode then
				print_position (an_instruction.position, current_feature.static_feature.implementation_class)
			end
			l_source := an_instruction.source
			l_source_type_set := dynamic_type_set (l_source)
			l_source_type := l_source_type_set.static_type
			l_target := an_instruction.target
			l_target_type_set := dynamic_type_set (l_target)
			l_target_type := l_target_type_set.static_type
			l_target_primary_type := l_target_type.primary_type
			nb := l_source_type_set.count
			l_conforming_types := conforming_types
			l_conforming_types.resize (nb)
			l_non_conforming_types := non_conforming_types
			l_non_conforming_types.resize (nb)
			from i := 1 until i > nb loop
				l_dynamic_type := l_source_type_set.dynamic_type (i)
				if l_dynamic_type.conforms_to_primary_type (l_target_primary_type) then
					l_conforming_types.put_last (l_dynamic_type)
				else
					l_non_conforming_types.put_last (l_dynamic_type)
				end
				i := i + 1
			end
			print_attachment_operand (l_source, l_source_type_set)
			fill_call_operands (1)
			if l_non_conforming_types.is_empty then
					-- Direct assignment.
				print_indentation
				print_writable (l_target)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attachment_expression (call_operands.first, l_source_type_set, l_target_type)
				current_file.put_character (';')
				current_file.put_new_line
			elseif l_conforming_types.is_empty then
				if l_target_type.is_expanded then
					-- ISE 6.3 does not change the value of the target
					-- when the type of the target is expanded and the
					-- assignment attempt fails. Note that it would have
					-- made more sense to set the target to its default
					-- value in that case.
				else
					print_indentation
					print_writable (l_target)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_void)
					current_file.put_character (';')
					current_file.put_new_line
				end
			else
				l_can_be_void := l_source_type_set.can_be_void
				if l_can_be_void then
					print_indentation
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					current_file.put_character ('(')
					print_expression (call_operands.first)
					current_file.put_character (')')
					current_file.put_character ('=')
					current_file.put_character ('=')
					current_file.put_string (c_eif_void)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					if l_target_type.is_expanded then
						-- ISE 6.3 does not change the value of the target
						-- when the type of the target is expanded and the
						-- assignment attempt fails. Note that it would have
						-- made more sense to set the target to its default
						-- value in that case.
					else
						print_indentation
						print_writable (l_target)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_string (c_eif_void)
						current_file.put_character (';')
						current_file.put_new_line
					end
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_character (' ')
					current_file.put_string (c_else)
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
				end
				check
						-- None of `l_non_conforming_types' and `l_conforming_types' are empty.
						-- Therefore the source is polymorphic (at least two dynamic types, one
						-- conforming and one not conforming). As a consequence `l_source_type'
						-- is not expanded because for expanded types there is no type other
						-- than itself that conforms to it.
						-- So we know that a type-id is always available.
					not_expanded: not l_source_type.is_expanded
				end
				print_indentation
				current_file.put_string (c_switch)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_attribute_type_id_access (call_operands.first, l_source_type.primary_type, False)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				l_conforming_type_set := new_standalone_type_set (l_source_type)
				l_old_conforming_types := l_conforming_type_set.dynamic_types
				l_conforming_type_set.reset_with_types (l_source_type, l_conforming_types)
				if l_source_type_set.is_never_void then
					l_conforming_type_set.set_never_void
				end
				if l_non_conforming_types.count < l_conforming_types.count then
					nb := l_non_conforming_types.count
					from i := 1 until i > nb loop
						l_dynamic_type := l_non_conforming_types.dynamic_type (i)
						print_indentation
						current_file.put_string (c_case)
						current_file.put_character (' ')
						current_file.put_integer (l_dynamic_type.id)
						current_file.put_character (':')
						current_file.put_new_line
						i := i + 1
					end
					indent
					if l_target_type.is_expanded then
						-- ISE 6.3 does not change the value of the target
						-- when the type of the target is expanded and the
						-- assignment attempt fails. Note that it would have
						-- made more sense to set the target to its default
						-- value in that case.
					else
						print_indentation
						print_writable (l_target)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_string (c_eif_void)
						current_file.put_character (';')
						current_file.put_new_line
					end
					print_indentation
					current_file.put_string (c_break)
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_string (c_default)
					current_file.put_character (':')
					current_file.put_new_line
					indent
					print_indentation
					print_writable (l_target)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_attachment_expression (call_operands.first, l_conforming_type_set, l_target_type)
					current_file.put_character (';')
					current_file.put_new_line
					dedent
				else
					nb := l_conforming_types.count
					from i := 1 until i > nb loop
						l_dynamic_type := l_conforming_types.dynamic_type (i)
						print_indentation
						current_file.put_string (c_case)
						current_file.put_character (' ')
						current_file.put_integer (l_dynamic_type.id)
						current_file.put_character (':')
						current_file.put_new_line
						i := i + 1
					end
					indent
					print_indentation
					print_writable (l_target)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_attachment_expression (call_operands.first, l_conforming_type_set, l_target_type)
					current_file.put_character (';')
					current_file.put_new_line
					print_indentation
					current_file.put_string (c_break)
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_string (c_default)
					current_file.put_character (':')
					current_file.put_new_line
					indent
					if l_target_type.is_expanded then
						-- ISE 6.3 does not change the value of the target
						-- when the type of the target is expanded and the
						-- assignment attempt fails.
					else
						print_indentation
						print_writable (l_target)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_string (c_eif_void)
						current_file.put_character (';')
						current_file.put_new_line
					end
					dedent
				end
				l_conforming_type_set.reset_with_types (current_dynamic_system.unknown_type, l_old_conforming_types)
				free_standalone_type_set (l_conforming_type_set)
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
				if l_can_be_void then
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
				end
			end
			l_conforming_types.wipe_out
			l_non_conforming_types.wipe_out
			call_operands.wipe_out
			if current_agent = Void and then current_feature.is_once then
					-- Make sure that if a once-function is called
					-- recursively, the semantics specified by ECMA will be satisfied.
					-- ECMA 367-2 section 8.23.22 page 124 says that the recursive calls
					-- to the once-function should return the value of 'Result' as it
					-- was when the recursive calls occurred.
					-- There is a special treatment for once-per-process and once-per-thread
					-- functions of non-basic expanded types because a boxed version of the
					-- Result (and not the Result itself) is kept in the 'GE_onces' C struct.
					-- So we need to use the content of the boxed value in place of Result
					-- in case its fields get modified. Likewise for once-per-object functions.
					-- even though the Result is not boxed in that case. (See `print_result'.)
				if l_target.is_result and then not (l_target_type.is_expanded and then not l_target_type.is_basic) then
					once_features.search (current_feature.static_feature.implementation_feature)
					if once_features.found then
						l_once_index := once_features.found_item
						l_once_kind := once_kind (current_feature)
					end
					print_assign_result_to_once_value (current_feature, l_once_kind, l_once_index)
				end
			end
		end

	print_bang_instruction (an_instruction: ET_BANG_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		do
			print_creation_instruction (an_instruction)
		end

	print_check_instruction (an_instruction: ET_CHECK_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		local
			i, nb: INTEGER
		do
			if attached an_instruction.then_compound as l_then_compound then
				if line_generation_mode then
					print_position (an_instruction.position, current_feature.static_feature.implementation_class)
				end
				print_indentation
				current_file.put_character ('{')
				current_file.put_new_line
				indent
				nb := an_instruction.count
				from i := 1 until i > nb loop
					if attached an_instruction.assertion (i).expression as l_expression then
						print_operand (l_expression)
						fill_call_operands (1)
						print_indentation
						current_file.put_string (c_if)
						current_file.put_character (' ')
						current_file.put_character ('(')
						current_file.put_character ('!')
						current_file.put_character ('(')
						print_expression (call_operands.first)
						call_operands.wipe_out
						current_file.put_character (')')
						current_file.put_character (')')
						current_file.put_character (' ')
						current_file.put_character ('{')
						current_file.put_new_line
						indent
						print_indentation
						current_file.put_string (c_ge_raise)
						current_file.put_character ('(')
						current_file.put_string (c_ge_ex_check)
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
						dedent
						print_indentation
						current_file.put_character ('}')
						current_file.put_new_line
					end
					i := i + 1
				end
				print_compound (l_then_compound)
				dedent
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
			else
					-- Do not check assertions at run-time.
			end
		end

	print_compound (a_compound: ET_COMPOUND)
			-- Print `a_compound'.
		require
			a_compound_not_void: a_compound /= Void
		local
			i, nb: INTEGER
		do
			nb := a_compound.count
			from i := 1 until i > nb loop
				print_instruction (a_compound.item (i))
				i := i + 1
			end
		end

	print_create_instruction (an_instruction: ET_CREATE_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		do
			print_creation_instruction (an_instruction)
		end

	print_creation_instruction (an_instruction: ET_CREATION_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		local
			l_target: ET_WRITABLE
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_static_type: ET_DYNAMIC_PRIMARY_TYPE
			l_seed: INTEGER
			l_actuals: detachable ET_ACTUAL_ARGUMENT_LIST
			l_actual: ET_EXPRESSION
			l_dynamic_procedure: detachable ET_DYNAMIC_FEATURE
			i, nb: INTEGER
			l_once_index: INTEGER
			l_once_kind: INTEGER
		do
			if line_generation_mode then
				print_position (an_instruction.position, current_feature.static_feature.implementation_class)
			end
				-- Look for the static and dynamic types of the creation.
			l_target := an_instruction.target
			l_dynamic_type_set := dynamic_type_set (l_target)
			l_static_type := l_dynamic_type_set.static_type.primary_type
			if attached an_instruction.type as l_type then
				l_dynamic_type := current_dynamic_system.dynamic_primary_type (l_type, current_type.base_type)
			else
					-- Look for the type of the target.
				l_dynamic_type := l_static_type
			end
			if attached an_instruction.creation_call as l_creation_call then
				l_seed := l_creation_call.name.seed
				l_actuals := l_creation_call.arguments
			else
				l_seed := current_system.default_create_seed
				l_actuals := Void
			end
			l_dynamic_procedure := l_dynamic_type.seeded_dynamic_procedure (l_seed, current_dynamic_system)
			if l_dynamic_procedure = Void then
					-- Internal error: there should be a procedure with `l_seed'.
					-- It has been computed in ET_FEATURE_CHECKER or else an
					-- error should have already been reported.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				if l_actuals /= Void then
					nb := l_actuals.count
					from i := 1 until i > nb loop
						l_actual := l_actuals.actual_argument (i)
						print_attachment_operand (l_actual, dynamic_type_set (l_actual))
						i := i + 1
					end
				end
				fill_call_operands (nb)
				print_indentation
				print_writable (l_target)
				print_assign_to
				print_adapted_creation_procedure_call (l_dynamic_procedure, l_dynamic_type, l_static_type)
				print_semicolon_newline
				call_operands.wipe_out
			end
			if current_agent = Void and then current_feature.is_once then
					-- Make sure that if a once-function is called
					-- recursively, the semantics specified by ECMA will be satisfied.
					-- ECMA 367-2 section 8.23.22 page 124 says that the recursive calls
					-- to the once-function should return the value of 'Result' as it
					-- was when the recursive calls occurred.
					-- There is a special treatment for once-per-process and once-per-thread
					-- functions of non-basic expanded types because a boxed version of the
					-- Result (and not the Result itself) is kept in the 'GE_onces' C struct.
					-- So we need to use the content of the boxed value in place of Result
					-- in case its fields get modified. Likewise for once-per-object functions.
					-- even though the Result is not boxed in that case. (See `print_result'.)
				if l_target.is_result and then not (l_static_type.is_expanded and then not l_static_type.is_basic) then
					once_features.search (current_feature.static_feature.implementation_feature)
					if once_features.found then
						l_once_index := once_features.found_item
						l_once_kind := once_kind (current_feature)
					end
					print_assign_result_to_once_value (current_feature, l_once_kind, l_once_index)
				end
			end
		end

	print_debug_instruction (an_instruction: ET_DEBUG_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		do
			-- Do nothing.
		end

	print_if_instruction (an_instruction: ET_IF_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		local
			l_elseif: ET_ELSEIF_PART
			i, nb: INTEGER
		do
			if line_generation_mode then
				print_position (an_instruction.position, current_feature.static_feature.implementation_class)
			end
			print_operand (an_instruction.expression)
			fill_call_operands (1)
			print_indentation
			current_file.put_string (c_if)
			current_file.put_character (' ')
			current_file.put_character ('(')
			print_expression (call_operands.first)
			call_operands.wipe_out
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			if attached an_instruction.then_compound as l_then_compound then
				indent
				print_compound (l_then_compound)
				dedent
			end
			print_indentation
			current_file.put_character ('}')
			if attached an_instruction.elseif_parts as l_elseif_parts then
				nb := l_elseif_parts.count
				from i := 1 until i > nb loop
					l_elseif := l_elseif_parts.item (i)
					current_file.put_character (' ')
					current_file.put_string (c_else)
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					if line_generation_mode then
						print_position (l_elseif.position, current_feature.static_feature.implementation_class)
					end
					indent
					print_operand (l_elseif.expression)
					fill_call_operands (1)
					print_indentation
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_expression (call_operands.first)
					call_operands.wipe_out
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					if attached l_elseif.then_compound as l_then_compound then
						indent
						print_compound (l_then_compound)
						dedent
					end
					print_indentation
					current_file.put_character ('}')
					i := i + 1
				end
			end
			if attached an_instruction.else_compound as l_else_compound then
				current_file.put_character (' ')
				current_file.put_string (c_else)
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				indent
				print_compound (l_else_compound)
				dedent
				print_indentation
				current_file.put_character ('}')
			end
			from i := 1 until i > nb loop
				current_file.put_new_line
				dedent
				print_indentation
				current_file.put_character ('}')
				i := i + 1
			end
			current_file.put_new_line
		end

	print_inspect_instruction (an_instruction: ET_INSPECT_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		local
			l_expression: ET_EXPRESSION
			l_when_part: ET_WHEN_PART
			l_choices: ET_CHOICE_LIST
			l_choice: ET_CHOICE
			i, nb: INTEGER
			j, nb2: INTEGER
			l_has_case: BOOLEAN
			l_lower: detachable ET_CONSTANT
			l_upper: detachable ET_CONSTANT
			l_lower_integer: detachable ET_INTEGER_CONSTANT
			l_upper_integer: detachable ET_INTEGER_CONSTANT
			l_lower_character: detachable ET_CHARACTER_CONSTANT
			l_upper_character: detachable ET_CHARACTER_CONSTANT
			k, nb3: INTEGER
			l_i_nat32, l_nb_nat32: NATURAL_32
			l_value_type_set: ET_DYNAMIC_TYPE_SET
			l_value_type: ET_DYNAMIC_PRIMARY_TYPE
			l_stop: BOOLEAN
		do
-- TODO.
			if line_generation_mode then
				print_position (an_instruction.position, current_feature.static_feature.implementation_class)
			end
			l_expression := an_instruction.conditional.expression
			l_value_type_set := dynamic_type_set (l_expression)
			l_value_type := l_value_type_set.static_type.primary_type
			print_operand (l_expression)
			fill_call_operands (1)
			print_indentation
			current_file.put_string (c_switch)
			current_file.put_character (' ')
			current_file.put_character ('(')
			print_expression (call_operands.first)
			call_operands.wipe_out
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			if attached an_instruction.when_parts as l_when_parts then
				nb := l_when_parts.count
				from i := 1 until i > nb loop
					l_when_part := l_when_parts.item (i)
					if line_generation_mode then
						print_position (l_when_part.position, current_feature.static_feature.implementation_class)
					end
					l_choices := l_when_part.choices
					nb2 := l_choices.count
					if nb2 = 0 then
						-- Do nothing.
					else
						l_has_case := False
						from j := 1 until j > nb2 loop
							l_choice := l_choices.choice (j)
							if l_choice.is_range then
-- TODO
								l_lower := choice_constant (l_choice.lower)
								l_upper := choice_constant (l_choice.upper)
								if attached {ET_INTEGER_CONSTANT} l_lower as l_integer_constant then
									l_lower_integer := l_integer_constant
									l_lower_character := Void
								elseif attached {ET_CHARACTER_CONSTANT} l_lower as l_character_constant then
									l_lower_character := l_character_constant
									l_lower_integer := Void
								else
									l_lower_integer := Void
									l_lower_character := Void
								end
								if attached {ET_INTEGER_CONSTANT} l_upper as l_integer_constant then
									l_upper_integer := l_integer_constant
									l_upper_character := Void
								elseif attached {ET_CHARACTER_CONSTANT} l_upper as l_character_constant then
									l_upper_character := l_character_constant
									l_upper_integer := Void
								else
									l_upper_integer := Void
									l_upper_character := Void
								end
								if l_lower_integer /= Void and l_upper_integer /= Void then
									from
-- TODO: check type of inspect value.
										k := l_lower_integer.to_integer_32
										nb3 := l_upper_integer.to_integer_32
									until
										k > nb3
									loop
										l_has_case := True
										print_indentation
										current_file.put_string (c_case)
										current_file.put_character (' ')
										print_integer_value (k.abs.to_natural_64, k < 0, l_value_type, True)
										current_file.put_character (':')
										current_file.put_new_line
										k := k + 1
									end
								elseif l_lower_character /= Void and l_upper_character /= Void then
									from
										l_i_nat32 := l_lower_character.value.natural_32_code
										l_nb_nat32 := l_upper_character.value.natural_32_code
										l_stop := l_i_nat32 > l_nb_nat32
									until
										l_stop
									loop
										l_has_case := True
										print_indentation
										current_file.put_string (c_case)
										current_file.put_character (' ')
										if current_system.character_32_type.same_named_type (l_value_type.base_type, current_type.base_type, current_type.base_type) then
											current_file.put_string (c_ge_nat32)
											current_file.put_character ('(')
											current_file.put_natural_32 (l_i_nat32)
											current_file.put_character (')')
										else
												-- Type cast needed when compiling with mingw when the character
												-- is represented by '\xyz' where x is not 0.
											print_type_cast (l_value_type, current_file)
											print_escaped_character_8 (l_i_nat32.to_character_8)
										end
										current_file.put_character (':')
										current_file.put_new_line
										if l_i_nat32 = l_nb_nat32 then
											l_stop := True
										else
											l_i_nat32 := l_i_nat32 + 1
										end
									end
								else
-- TODO
error_handler.report_warning_message ("ET_C_GENERATOR.print_inspect_instruction - range")
								end
							else
								l_has_case := True
								print_indentation
								current_file.put_string (c_case)
								current_file.put_character (' ')
								l_lower := choice_constant (l_choice.lower)
								if attached {ET_INTEGER_CONSTANT} l_lower as l_integer_constant then
									print_integer_value (l_integer_constant.value, l_integer_constant.is_negative, l_value_type, True)
								elseif attached {ET_CHARACTER_CONSTANT} l_lower as l_character_constant then
									if current_system.character_32_type.same_named_type (l_value_type.base_type, current_type.base_type, current_type.base_type) then
										current_file.put_string (c_ge_nat32)
										current_file.put_character ('(')
										current_file.put_natural_32 (l_character_constant.value.natural_32_code)
										current_file.put_character (')')
									else
											-- Type cast needed when compiling with mingw when the character
											-- is represented by '\xyz' where x is not 0.
										print_type_cast (l_value_type, current_file)
										print_escaped_character_8 (l_character_constant.value.to_character_8)
									end
								else
									if l_value_type /= dynamic_type_set (l_choice.lower).static_type.primary_type then
										print_type_cast (l_value_type, current_file)
									end
									print_expression (l_choice.lower)
								end
								current_file.put_character (':')
								current_file.put_new_line
							end
							j := j + 1
						end
						if l_has_case then
							indent
							if attached l_when_part.then_compound as l_then_compound then
								print_compound (l_then_compound)
							end
							print_indentation
							current_file.put_string (c_break)
							current_file.put_character (';')
							current_file.put_new_line
							dedent
						end
					end
					i := i + 1
				end
			end
			print_indentation
			current_file.put_string (c_default)
			current_file.put_character (':')
			current_file.put_new_line
			if attached an_instruction.else_compound as l_else_compound then
				indent
				print_compound (l_else_compound)
				print_indentation
				current_file.put_string (c_break)
				current_file.put_character (';')
				current_file.put_new_line
				dedent
			else
				indent
				print_indentation
				current_file.put_string (c_ge_raise)
				current_file.put_character ('(')
				current_file.put_string (c_ge_ex_when)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_string (c_break)
				current_file.put_character (';')
				current_file.put_new_line
				dedent
			end
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
		end

	choice_constant (a_choice_constant: ET_CHOICE_CONSTANT): detachable ET_CONSTANT
			-- Constant value of the inspect choice `a_choice_constant',
			-- or Void if it appears not to be a constant after all
			--
			-- Note that validity `a_choice_constant' as an expression
			-- is assumed to have already been checked.
			-- It will just return Void if something unexpected is encountered.
		require
			a_choice_constant_not_void: a_choice_constant /= Void
		local
			l_query: detachable ET_QUERY
			l_seed: INTEGER
			l_target_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if attached {ET_CONSTANT} a_choice_constant as l_constant then
				Result := l_constant
			else
				if attached {ET_IDENTIFIER} a_choice_constant as l_identifier then
					if l_identifier.is_argument then
							-- This is not a constant.
					elseif l_identifier.is_local then
							-- This is not a constant.
					elseif l_identifier.is_object_test_local then
							-- This is not a constant.
					elseif l_identifier.is_iteration_item then
							-- This is not a constant.
					else
						l_seed := l_identifier.seed
						l_query := current_type.base_class.seeded_query (l_seed)
					end
				elseif attached {ET_UNQUALIFIED_FEATURE_CALL} a_choice_constant as l_unqualified_call and then l_unqualified_call.arguments_count = 0 then
					l_seed := l_unqualified_call.name.seed
					l_query := current_type.base_class.seeded_query (l_seed)
				elseif attached {ET_STATIC_CALL_EXPRESSION} a_choice_constant as l_static_call then
					l_target_type := current_dynamic_system.dynamic_primary_type (l_static_call.type, current_type.base_type)
					l_query := l_target_type.base_class.seeded_query (l_static_call.name.seed)
				end
				if attached {ET_CONSTANT_QUERY} l_query as l_constant_query then
					Result := l_constant_query.constant
				end
			end
		end

	print_instruction (an_instruction: ET_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		do
			an_instruction.process (Current)
		end

	print_iteration_instruction (an_instruction: ET_ITERATION_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		local
			l_expression: ET_EXPRESSION
			l_cursor_name: ET_IDENTIFIER
			l_cursor_type: ET_DYNAMIC_TYPE
			l_cursor_type_set: ET_DYNAMIC_TYPE_SET
		do
			if line_generation_mode then
				print_position (an_instruction.position, current_feature.static_feature.implementation_class)
			end
				-- Declaration of the iteration cursor.
			l_cursor_name := an_instruction.unfolded_cursor_name
			l_cursor_type_set := dynamic_type_set (l_cursor_name)
			l_cursor_type := l_cursor_type_set.static_type
			current_function_header_buffer.put_character ('%T')
			print_type_declaration (l_cursor_type.primary_type, current_function_header_buffer)
			current_function_header_buffer.put_character (' ')
			print_iteration_cursor_name (l_cursor_name, current_function_header_buffer)
			current_function_header_buffer.put_character (';')
			current_function_header_buffer.put_new_line
				-- Call to `new_cursor'.
			print_assignment_operand (an_instruction.new_cursor_expression, l_cursor_type_set, l_cursor_name, l_cursor_type)
			fill_call_operands (1)
			if call_operands.first /= l_cursor_name then
				print_indentation
				print_iteration_cursor_name (l_cursor_name, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attachment_expression (call_operands.first, l_cursor_type_set, l_cursor_type)
				current_file.put_character (';')
				current_file.put_new_line
			end
			call_operands.wipe_out
			if attached an_instruction.from_compound as l_from_compound then
				print_compound (l_from_compound)
			end
			print_indentation
			current_file.put_string (c_while)
			current_file.put_character (' ')
			current_file.put_character ('(')
			current_file.put_character ('1')
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_operand (an_instruction.cursor_after_expression)
			if attached an_instruction.until_conditional as l_until_conditional then
				l_expression := l_until_conditional.expression
				print_operand (l_expression)
				fill_call_operands (2)
			else
				fill_call_operands (1)
			end
			print_indentation
			current_file.put_string (c_if)
			current_file.put_character (' ')
			current_file.put_character ('(')
			current_file.put_character ('(')
			print_expression (call_operands.first)
			current_file.put_character (')')
			if call_operands.count = 2 then
				current_file.put_character ('|')
				current_file.put_character ('|')
				current_file.put_character ('(')
				print_expression (call_operands.item (2))
				current_file.put_character (')')
			end
			call_operands.wipe_out
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_indentation
			current_file.put_string (c_break)
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
			if attached an_instruction.loop_compound as l_loop_compound then
				print_compound (l_loop_compound)
			end
			print_instruction (an_instruction.cursor_forth_instruction)
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
		end

	print_loop_instruction (an_instruction: ET_LOOP_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		local
			l_expression: ET_EXPRESSION
		do
			if attached an_instruction.from_compound as l_from_compound and then not l_from_compound.is_empty then
				print_compound (l_from_compound)
			elseif line_generation_mode then
				print_position (an_instruction.position, current_feature.static_feature.implementation_class)
			end
			print_indentation
			current_file.put_string (c_while)
			current_file.put_character (' ')
			current_file.put_character ('(')
			current_file.put_character ('1')
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			l_expression := an_instruction.until_expression
			if line_generation_mode then
				print_position (l_expression.position, current_feature.static_feature.implementation_class)
			end
			print_operand (l_expression)
			fill_call_operands (1)
			print_indentation
			current_file.put_string (c_if)
			current_file.put_character (' ')
			current_file.put_character ('(')
			print_expression (call_operands.first)
			call_operands.wipe_out
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_indentation
			current_file.put_string (c_break)
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
			if attached an_instruction.loop_compound as l_loop_compound then
				print_compound (l_loop_compound)
			end
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
		end

	print_precursor_instruction (an_instruction: ET_PRECURSOR_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		local
			l_precursor_keyword: ET_PRECURSOR_KEYWORD
			l_procedure: detachable ET_PROCEDURE
			l_parent_type, l_ancestor: detachable ET_BASE_TYPE
			l_class: ET_CLASS
			l_current_class: ET_CLASS
			l_class_impl: ET_CLASS
			l_dynamic_precursor: ET_DYNAMIC_PRECURSOR
			l_parent_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type_set: ET_DYNAMIC_TYPE_SET
			l_actual: ET_EXPRESSION
			had_error: BOOLEAN
		do
			if line_generation_mode then
				print_position (an_instruction.position, current_feature.static_feature.implementation_class)
			end
			if attached an_instruction.arguments as l_actuals then
				nb := l_actuals.count
				from i := 1 until i > nb loop
					l_actual := l_actuals.actual_argument (i)
					l_actual_type_set := dynamic_type_set (l_actual)
					print_attachment_operand (l_actual, l_actual_type_set)
					i := i + 1
				end
			end
			fill_call_operands (nb)
			l_parent_type := an_instruction.parent_type
			if l_parent_type = Void then
					-- Internal error: the Precursor construct should already
					-- have been resolved when flattening the features of the
					-- implementation class of current feature.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				had_error := has_fatal_error
				has_fatal_error := False
				if l_parent_type.is_generic then
					l_current_class := current_type.base_class
					l_class_impl := current_feature.static_feature.implementation_class
					if l_current_class /= l_class_impl then
							-- Resolve generic parameters in the context of `current_type'.
						if not l_current_class.ancestors_built or else l_current_class.has_ancestors_error then
								-- 'ancestor_builder' should already have been executed
								-- on `l_current_class' at this stage, and any error
								-- should have already been reported.
							set_fatal_error
							error_handler.report_giaaa_error
						else
							l_ancestor := l_current_class.ancestor (l_parent_type)
							if l_ancestor = Void then
									-- Internal error: `l_parent_type' is an ancestor
									-- of `l_class_impl', and hence of `l_current_class'.
								set_fatal_error
								error_handler.report_giaaa_error
							else
								l_parent_type := l_ancestor
							end
						end
					end
				end
				if not has_fatal_error then
					has_fatal_error := had_error
					l_precursor_keyword := an_instruction.precursor_keyword
					l_class := l_parent_type.base_class
					l_procedure := l_class.seeded_procedure (l_precursor_keyword.seed)
					if l_procedure = Void then
							-- Internal error: the Precursor construct should
							-- already have been resolved when flattening the
							-- features of `l_class_impl'.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						l_parent_dynamic_type := current_dynamic_system.dynamic_primary_type (l_parent_type, current_type.base_type)
						l_dynamic_precursor := current_feature.dynamic_precursor (l_procedure, l_parent_dynamic_type, current_dynamic_system)
						print_indentation
						if in_static_feature then
							register_called_static_feature (l_dynamic_precursor)
							print_static_routine_name (l_dynamic_precursor, current_type, current_file)
							current_file.put_character ('(')
							current_file.put_string (c_ac)
						else
							register_called_feature (l_dynamic_precursor)
							print_routine_name (l_dynamic_precursor, current_type, current_file)
							current_file.put_character ('(')
							current_file.put_string (c_ac)
							current_file.put_character (',')
							current_file.put_character (' ')
							print_current_name (current_file)
						end
						from i := 1 until i > nb loop
							current_file.put_character (',')
							current_file.put_character (' ')
							l_actual_type_set := dynamic_type_set (call_operands.item (i))
							l_formal_type_set := argument_type_set_in_feature (i, l_dynamic_precursor)
							print_attachment_expression (call_operands.item (i), l_actual_type_set, l_formal_type_set.static_type)
							i := i + 1
						end
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					end
				end
			end
			call_operands.wipe_out
		end

	print_qualified_call_instruction (a_call: ET_QUALIFIED_FEATURE_CALL_INSTRUCTION)
			-- Print qualified call instruction.
		require
			a_call_not_void: a_call /= Void
		local
			l_name: ET_CALL_NAME
			l_target: ET_EXPRESSION
			l_target_type_set: ET_DYNAMIC_TYPE_SET
			l_target_static_type: ET_DYNAMIC_PRIMARY_TYPE
			l_manifest_tuple_operand: detachable ET_MANIFEST_TUPLE
			l_actuals: detachable ET_ACTUAL_ARGUMENTS
			l_actual: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_target_operand: ET_EXPRESSION
			l_seed: INTEGER
			i, nb: INTEGER
			l_target_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			nb2: INTEGER
			l_switch: BOOLEAN
			l_printed: BOOLEAN
		do
			if line_generation_mode then
				print_position (a_call.position, current_feature.static_feature.implementation_class)
			end
			l_target := a_call.target
			l_name := a_call.name
			l_actuals := a_call.arguments
			l_target_type_set := dynamic_type_set (l_target)
			l_target_static_type := l_target_type_set.static_type.primary_type
			l_seed := l_name.seed
				-- Check whether this a call to ROUTINE.call with a manifest
				-- tuple as argument. We have a special treatment in that case
				-- to avoid having to create the manifest tuple when possible.
			if not a_call.is_tuple_label and then l_seed = current_system.routine_call_seed and then not a_call.is_call_agent then
				if l_actuals /= Void and then l_actuals.count = 1 then
					if attached {ET_MANIFEST_TUPLE} l_actuals.actual_argument (1) as l_manifest_tuple then
						print_procedure_target_operand (l_target, l_target_static_type)
						nb := l_manifest_tuple.count
						from i := 1 until i > nb loop
							l_actual := l_manifest_tuple.expression (i)
							l_actual_type_set := dynamic_type_set (l_actual)
							print_attachment_operand (l_actual, l_actual_type_set)
							i := i + 1
						end
						nb := nb + 1
						fill_call_operands (nb)
						l_manifest_tuple_operand := agent_manifest_tuple
						l_manifest_tuple_operand.resize (nb - 1)
						from i := nb until i < 2 loop
							l_manifest_tuple_operand.put_first (call_operands.item (i))
							i := i - 1
						end
						l_manifest_tuple_operand.set_index (l_manifest_tuple.index)
						mark_expressions_frozen (l_manifest_tuple_operand)
						l_target_operand := call_operands.first
						call_operands.wipe_out
						call_operands.put_last (l_target_operand)
						call_operands.put_last (l_manifest_tuple_operand)
						nb := 2
					end
				end
			end
			if l_manifest_tuple_operand = Void then
				print_procedure_target_operand (l_target, l_target_static_type)
				if l_actuals /= Void then
					nb := l_actuals.count
					from i := 1 until i > nb loop
						l_actual := l_actuals.actual_argument (i)
						l_actual_type_set := dynamic_type_set (l_actual)
						print_attachment_operand (l_actual, l_actual_type_set)
						i := i + 1
					end
				end
				nb := nb + 1
				fill_call_operands (nb)
			end
			nb2 := l_target_type_set.count
			if nb2 = 0 then
					-- Call on Void target.
				print_indentation
				current_file.put_character ('(')
				print_procedure_target_expression (call_operands.first, l_target_static_type, True)
				from i := 2 until i > nb loop
					current_file.put_character (',')
					current_file.put_character (' ')
					print_expression (call_operands.item (i))
					i := i + 1
				end
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			elseif nb2 = 1 then
					-- Static binding.
				l_target_dynamic_type := l_target_type_set.dynamic_type (1)
				print_named_procedure_call (a_call.name, l_target_dynamic_type, True)
			else
					-- Dynamic binding.
				if l_name.is_tuple_label then
-- TODO: fields in Tuples are not necessarily aligned, and some need (un)boxing,
-- for example:
--    t1: TUPLE [a: ANY; b: INTEGER]
--    t2: TUPLE [a: INTEGER_64; b: INTEGER]
--    t1 := t2
--    t1.a := {INTEGER_64} 3
--    t1.b := 1
-- But when there are aligned with no (un)boxing, we should avoid having polymorphic calls.
				end
				if l_printed then
					-- Do nothing.
				elseif nb2 > 2 then
					if not attached current_feature.procedure_call (a_call) as l_dynamic_call then
							-- Internal error: the dynamic call should have been created in ET_DYNAMIC_TYPE_BUILDER.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						register_polymorphic_called_features (l_dynamic_call)
						print_indentation
						print_call_name (a_call, current_feature, l_target_static_type, False, current_file)
						current_file.put_character ('(')
						current_file.put_string (c_ac)
						current_file.put_character (',')
						current_file.put_character (' ')
						print_procedure_target_expression (call_operands.first, l_target_static_type, True)
						if l_manifest_tuple_operand /= Void then
							nb := l_manifest_tuple_operand.count
							from i := 1 until i > nb loop
								current_file.put_character (',')
								current_file.put_character (' ')
								print_expression (l_manifest_tuple_operand.expression (i))
								i := i + 1
							end
						else
							from i := 2 until i > nb loop
								current_file.put_character (',')
								current_file.put_character (' ')
								print_expression (call_operands.item (i))
								i := i + 1
							end
						end
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					end
				elseif l_switch then
					print_indentation
					current_file.put_string (c_switch)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_attribute_type_id_access (call_operands.first, l_target_static_type, True)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
						-- First type.
					l_target_dynamic_type := l_target_type_set.dynamic_type (1)
					print_indentation
					current_file.put_string (c_case)
					current_file.put_character (' ')
					current_file.put_integer (l_target_dynamic_type.id)
					current_file.put_character (':')
					current_file.put_new_line
					indent
					print_named_procedure_call (a_call.name, l_target_dynamic_type, False)
					print_indentation
					current_file.put_string (c_break)
					current_file.put_character (';')
					current_file.put_new_line
					dedent
						-- Second type.
					l_target_dynamic_type := l_target_type_set.dynamic_type (2)
					print_indentation
					current_file.put_string (c_case)
					current_file.put_character (' ')
					current_file.put_integer (l_target_dynamic_type.id)
					current_file.put_character (':')
					current_file.put_new_line
					indent
					print_named_procedure_call (a_call.name, l_target_dynamic_type, False)
					print_indentation
					current_file.put_string (c_break)
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
				else
						-- First type.
					l_target_dynamic_type := l_target_type_set.dynamic_type (1)
					print_indentation
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_attribute_type_id_access (call_operands.first, l_target_static_type, True)
					current_file.put_character ('=')
					current_file.put_character ('=')
					current_file.put_integer (l_target_dynamic_type.id)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					print_named_procedure_call (a_call.name, l_target_dynamic_type, False)
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_character (' ')
						-- Second type.
					l_target_dynamic_type := l_target_type_set.dynamic_type (2)
					current_file.put_string (c_else)
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					print_named_procedure_call (a_call.name, l_target_dynamic_type, False)
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
				end
			end
			if l_manifest_tuple_operand /= Void then
				mark_expressions_unfrozen (l_manifest_tuple_operand)
				l_manifest_tuple_operand.wipe_out
			end
			call_operands.wipe_out
		end

	print_repeat_instruction (a_instruction: ET_REPEAT_INSTRUCTION)
			-- Print `a_instruction'.
		require
			a_instruction_not_void: a_instruction /= Void
		do
			print_iteration_instruction (a_instruction)
		end

	print_retry_instruction (an_instruction: ET_RETRY_INSTRUCTION)
			-- Print `an_instruction'.
		require
			an_instruction_not_void: an_instruction /= Void
		do
			if line_generation_mode then
				print_position (an_instruction.position, current_feature.static_feature.implementation_class)
			end
			print_indentation
			current_file.put_string (c_ac)
			current_file.put_string (c_arrow)
			current_file.put_string (c_in_rescue)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_string (c_tr)
			current_file.put_character (';')
			current_file.put_new_line
			print_indentation
			current_file.put_string (c_if)
			current_file.put_character (' ')
			current_file.put_character ('(')
			current_file.put_string (c_tr)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_character ('0')
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_indentation
			current_file.put_string (c_ac)
			current_file.put_string (c_arrow)
			current_file.put_string (c_exception_code)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_character ('0')
			current_file.put_character (';')
			current_file.put_new_line
			print_indentation
			current_file.put_string (c_ac)
			current_file.put_string (c_arrow)
			current_file.put_string (c_exception_tag)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_character ('(')
			current_file.put_string (c_char)
			current_file.put_character ('*')
			current_file.put_character (')')
			current_file.put_character ('0')
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
			print_indentation
			current_file.put_string (c_goto)
			current_file.put_character (' ')
			current_file.put_string (c_ge_retry)
			current_file.put_character (';')
			current_file.put_new_line
			has_retry := True
		end

	print_static_call_instruction (a_instruction: ET_STATIC_CALL_INSTRUCTION)
			-- Print `a_instruction'.
		require
			a_instruction_not_void: a_instruction /= Void
		local
			l_target_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if line_generation_mode then
				print_position (a_instruction.position, current_feature.static_feature.implementation_class)
			end
			l_target_type := current_dynamic_system.dynamic_primary_type (a_instruction.type, current_type.base_type)
			print_instruction_static_call (l_target_type, a_instruction.name, a_instruction.arguments)
		end

	print_instruction_static_call (a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_name: ET_CALL_NAME; a_arguments: detachable ET_ACTUAL_ARGUMENTS)
			-- Print instruction which is expected to be a static call to
			-- feature with the same seed as `a_name' in type `a_target_type'
			-- and with `a_arguments'.
		require
			a_target_type_not_void: a_target_type /= Void
			a_name_not_void: a_name /= Void
		local
			l_dynamic_procedure: detachable ET_DYNAMIC_FEATURE
			l_seed: INTEGER
			i, nb: INTEGER
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type_set: ET_DYNAMIC_TYPE_SET
			l_actual: ET_EXPRESSION
		do
			if a_arguments /= Void then
				nb := a_arguments.count
				from i := 1 until i > nb loop
					l_actual := a_arguments.actual_argument (i)
					l_actual_type_set := dynamic_type_set (l_actual)
					print_attachment_operand (l_actual, l_actual_type_set)
					i := i + 1
				end
			end
			fill_call_operands (nb)
			l_seed := a_name.seed
			l_dynamic_procedure := a_target_type.seeded_dynamic_procedure (l_seed, current_dynamic_system)
			if l_dynamic_procedure = Void then
					-- Internal error: there should be a procedure with `l_seed'.
					-- It has been computed in ET_FEATURE_CHECKER or else an
					-- error should have already been reported.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				register_called_static_feature (l_dynamic_procedure)
				print_indentation
				print_static_routine_name (l_dynamic_procedure, a_target_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				from i := 1 until i > nb loop
					current_file.put_character (',')
					current_file.put_character (' ')
					l_actual_type_set := dynamic_type_set (call_operands.item (i))
					l_formal_type_set := argument_type_set_in_feature (i, l_dynamic_procedure)
					print_attachment_expression (call_operands.item (i), l_actual_type_set, l_formal_type_set.static_type)
					i := i + 1
				end
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
			call_operands.wipe_out
		end

	print_unqualified_call_instruction (a_call: ET_UNQUALIFIED_FEATURE_CALL_INSTRUCTION)
			-- Print unqualified call instruction.
		require
			a_call_not_void: a_call /= Void
		local
			l_actuals: detachable ET_ACTUAL_ARGUMENTS
			l_actual: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_manifest_tuple_operand: detachable ET_MANIFEST_TUPLE
			l_target_operand: ET_EXPRESSION
			l_seed: INTEGER
			i, nb: INTEGER
		do
			if line_generation_mode then
				print_position (a_call.position, current_feature.static_feature.implementation_class)
			end
			if in_static_feature then
				print_instruction_static_call (current_type, a_call.name, a_call.arguments)
			else
				l_actuals := a_call.arguments
				l_seed := a_call.name.seed
					-- Check whether this a call to ROUTINE.call with a manifest
					-- tuple as argument. We have a special treatment in that case
					-- to avoid having to create the manifest tuple when possible.
				if l_seed = current_system.routine_call_seed and then not a_call.is_call_agent then
					if l_actuals /= Void and then l_actuals.count = 1 then
						if attached {ET_MANIFEST_TUPLE} l_actuals.actual_argument (1) as l_manifest_tuple then
							operand_stack.force (tokens.current_keyword)
							nb := l_manifest_tuple.count
							from i := 1 until i > nb loop
								l_actual := l_manifest_tuple.expression (i)
								l_actual_type_set := dynamic_type_set (l_actual)
								print_attachment_operand (l_actual, l_actual_type_set)
								i := i + 1
							end
							nb := nb + 1
							fill_call_operands (nb)
							l_manifest_tuple_operand := agent_manifest_tuple
							l_manifest_tuple_operand.resize (nb - 1)
							from i := nb until i < 2 loop
								l_manifest_tuple_operand.put_first (call_operands.item (i))
								i := i - 1
							end
							l_manifest_tuple_operand.set_index (l_manifest_tuple.index)
							mark_expressions_frozen (l_manifest_tuple_operand)
							l_target_operand := call_operands.first
							call_operands.wipe_out
							call_operands.put_last (l_target_operand)
							call_operands.put_last (l_manifest_tuple_operand)
							nb := 2
						end
					end
				end
				if l_manifest_tuple_operand = Void then
					operand_stack.force (tokens.current_keyword)
					if l_actuals /= Void then
						nb := l_actuals.count
						from i := 1 until i > nb loop
							l_actual := l_actuals.actual_argument (i)
							l_actual_type_set := dynamic_type_set (l_actual)
							print_attachment_operand (l_actual, l_actual_type_set)
							i := i + 1
						end
					end
					nb := nb + 1
					fill_call_operands (nb)
				end
				print_named_procedure_call (a_call.name, current_type, False)
				call_operands.wipe_out
				if l_manifest_tuple_operand /= Void then
					mark_expressions_unfrozen (l_manifest_tuple_operand)
					l_manifest_tuple_operand.wipe_out
				end
			end
		end

feature {NONE} -- Procedure call generation

	print_adapted_creation_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type, a_static_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print to `current_file' a call to creation procedure `a_feature' (static binding).
			-- `a_target_type' is the dynamic type of the target.
			-- `a_static_type' is the static type of the declared type of the target,
			-- used to adapt the result of the creation if needed (see header comment of
			-- `print_adapted_expression_with_agent' for details).
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_creation: a_feature.is_creation
			a_target_type_not_void: a_target_type /= Void
			a_static_type_not_void: a_static_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_adapted_expression_with_agent (agent print_creation_procedure_call (a_feature, a_target_type), a_target_type, a_static_type, False)
		end

	print_creation_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print to `current_file' a call to creation procedure `a_feature' (static binding).
			-- `a_target_type' is the dynamic type of the target.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			i, nb: INTEGER
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type_set: ET_DYNAMIC_TYPE_SET
		do
			register_called_feature (a_feature)
			print_creation_procedure_name (a_feature, a_target_type, current_file)
			current_file.put_character ('(')
			current_file.put_string (c_ac)
			nb := call_operands.count
			from i := 1 until i > nb loop
				print_comma
				l_actual_type_set := dynamic_type_set (call_operands.item (i))
				l_formal_type_set := argument_type_set_in_feature (i, a_feature)
				print_attachment_expression (call_operands.item (i), l_actual_type_set, l_formal_type_set.static_type)
				i := i + 1
			end
			current_file.put_character (')')
		end

	print_named_procedure_call (a_name: ET_CALL_NAME; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_name' (static binding).
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_name_not_void: a_name /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_dynamic_feature: detachable ET_DYNAMIC_FEATURE
			l_seed: INTEGER
		do
			l_seed := a_name.seed
			if a_name.is_tuple_label then
				print_tuple_label_setter_call (a_name, a_target_type, a_check_void_target)
			else
				l_dynamic_feature := a_target_type.seeded_dynamic_procedure (l_seed, current_dynamic_system)
				if l_dynamic_feature = Void then
						-- Internal error: there should be a procedure with `l_seed'.
						-- It has been computed in ET_FEATURE_CHECKER.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					print_procedure_call (l_dynamic_feature, a_target_type, a_check_void_target)
				end
			end
		end

	print_tuple_label_setter_call (a_name: ET_CALL_NAME; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to a Tuple label setter (static binding).
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_name_not_void: a_name /= Void
			tuple_label: a_name.is_tuple_label
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_tuple_expression: ET_EXPRESSION
			l_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_item_type_set: ET_DYNAMIC_TYPE_SET
			l_source: ET_EXPRESSION
			l_source_type_set: ET_DYNAMIC_TYPE_SET
			l_seed: INTEGER
		do
			if call_operands.count /= 2 then
					-- Internal error: the Tuple label setter should have one argument.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				if not attached {ET_DYNAMIC_TUPLE_TYPE} a_target_type as l_tuple_type then
						-- Internal error: the type of the target should be a Tuple type.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					l_seed := a_name.seed
					l_item_type_sets := l_tuple_type.item_type_sets
					if l_seed < 1 or l_seed > l_item_type_sets.count then
							-- Internal error: invalid Tuple label.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						print_indentation
						l_tuple_expression := call_operands.first
						print_attribute_tuple_item_access (l_seed, l_tuple_expression, a_target_type, a_check_void_target)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_character ('(')
						l_source := call_operands.item (2)
						l_source_type_set := dynamic_type_set (l_source)
						l_item_type_set := l_item_type_sets.item (l_seed)
						print_attachment_expression (l_source, l_source_type_set, l_item_type_set.static_type)
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					end
				end
			end
		end

	print_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if a_feature.is_builtin then
				print_builtin_procedure_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_non_inlined_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a non-inlined version of a call to procedure `a_feature' (static binding).
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type_set: ET_DYNAMIC_TYPE_SET
			i, nb: INTEGER
		do
			register_called_feature (a_feature)
			print_indentation
			print_routine_name (a_feature, a_target_type, current_file)
			current_file.put_character ('(')
			current_file.put_string (c_ac)
			current_file.put_character (',')
			current_file.put_character (' ')
			nb := call_operands.count
			print_procedure_target_expression (call_operands.first, a_target_type, a_check_void_target)
			from i := 2 until i > nb loop
				current_file.put_character (',')
				current_file.put_character (' ')
				l_actual_type_set := dynamic_type_set (call_operands.item (i))
				l_formal_type_set := argument_type_set_in_feature (i - 1, a_feature)
				print_attachment_expression (call_operands.item (i), l_actual_type_set, l_formal_type_set.static_type)
				i := i + 1
			end
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_builtin_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_class_code
			when {ET_TOKEN_CODES}.builtin_any_class then
				print_builtin_any_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_boolean_ref_class then
				print_builtin_boolean_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_character_8_ref_class then
				print_builtin_character_n_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_character_32_ref_class then
				print_builtin_character_n_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_com_failure_class then
				print_builtin_com_failure_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_identified_routines_class then
				print_builtin_identified_routines_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_8_ref_class then
				print_builtin_integer_n_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_16_ref_class then
				print_builtin_integer_n_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_32_ref_class then
				print_builtin_integer_n_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_64_ref_class then
				print_builtin_integer_n_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_exception_manager_class then
				print_builtin_ise_exception_manager_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_class then
				print_builtin_ise_runtime_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_memory_class then
				print_builtin_memory_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_natural_8_ref_class then
				print_builtin_integer_n_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_natural_16_ref_class then
				print_builtin_integer_n_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_natural_32_ref_class then
				print_builtin_integer_n_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_natural_64_ref_class then
				print_builtin_integer_n_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_pointer_ref_class then
				print_builtin_pointer_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_procedure_class then
				print_builtin_procedure_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_32_ref_class then
				print_builtin_real_n_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_64_ref_class then
				print_builtin_real_n_ref_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_special_class then
				print_builtin_special_procedure_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_tuple_class then
				print_builtin_tuple_procedure_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_any_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "ANY".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_any: a_feature.is_builtin_any_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_any_copy then
				print_builtin_any_copy_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_any_standard_copy then
				print_builtin_any_standard_copy_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_boolean_ref_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "BOOLEAN_REF".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_boolean_ref: a_feature.is_builtin_boolean_ref_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_boolean_ref_set_item then
				print_builtin_boolean_ref_set_item_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_character_n_ref_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in classes "CHARACTER_N_REF".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_character_n_ref: a_feature.is_builtin_character_n_ref_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_character_n_ref_set_item then
				print_builtin_character_n_ref_set_item_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_com_failure_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "COM_FAILURE".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_com_failure: a_feature.is_builtin_com_failure_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_com_failure_cwin_local_free then
				print_builtin_com_failure_cwin_local_free_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_identified_routines_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "IDENTIFIED_ROUTINES".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_identified_routines: a_feature.is_builtin_identified_routines_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_identified_routines_eif_object_id_free then
				print_builtin_identified_routines_eif_object_id_free_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_integer_n_ref_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in classes "INTEGER_N_REF" and "NATURAL_N_REF".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_integer_n_ref: a_feature.is_builtin_integer_n_ref_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_integer_n_ref_set_item then
				print_builtin_integer_n_ref_set_item_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_ise_exception_manager_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "ISE_EXCEPTION_MANAGER".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_ise_exception_manager: a_feature.is_builtin_ise_exception_manager_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_ise_exception_manager_developer_raise then
				print_builtin_ise_exception_manager_developer_raise_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_ise_runtime_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "ISE_RUNTIME".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_ise_runtime: a_feature.is_builtin_ise_runtime_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_ise_runtime_lock_marking then
				print_builtin_ise_runtime_lock_marking_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_mark_object then
				print_builtin_ise_runtime_mark_object_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_boolean_field then
				print_builtin_ise_runtime_set_boolean_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_boolean_field_at then
				print_builtin_ise_runtime_set_boolean_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_character_8_field then
				print_builtin_ise_runtime_set_character_8_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_character_8_field_at then
				print_builtin_ise_runtime_set_character_8_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_character_32_field then
				print_builtin_ise_runtime_set_character_32_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_character_32_field_at then
				print_builtin_ise_runtime_set_character_32_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_8_field then
				print_builtin_ise_runtime_set_integer_8_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_8_field_at then
				print_builtin_ise_runtime_set_integer_8_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_16_field then
				print_builtin_ise_runtime_set_integer_16_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_16_field_at then
				print_builtin_ise_runtime_set_integer_16_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_32_field then
				print_builtin_ise_runtime_set_integer_32_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_32_field_at then
				print_builtin_ise_runtime_set_integer_32_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_64_field then
				print_builtin_ise_runtime_set_integer_64_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_integer_64_field_at then
				print_builtin_ise_runtime_set_integer_64_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_8_field then
				print_builtin_ise_runtime_set_natural_8_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_8_field_at then
				print_builtin_ise_runtime_set_natural_8_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_16_field then
				print_builtin_ise_runtime_set_natural_16_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_16_field_at then
				print_builtin_ise_runtime_set_natural_16_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_32_field then
				print_builtin_ise_runtime_set_natural_32_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_32_field_at then
				print_builtin_ise_runtime_set_natural_32_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_64_field then
				print_builtin_ise_runtime_set_natural_64_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_natural_64_field_at then
				print_builtin_ise_runtime_set_natural_64_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_pointer_field then
				print_builtin_ise_runtime_set_pointer_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_pointer_field_at then
				print_builtin_ise_runtime_set_pointer_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_real_32_field then
				print_builtin_ise_runtime_set_real_32_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_real_32_field_at then
				print_builtin_ise_runtime_set_real_32_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_real_64_field then
				print_builtin_ise_runtime_set_real_64_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_real_64_field_at then
				print_builtin_ise_runtime_set_real_64_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_reference_field then
				print_builtin_ise_runtime_set_reference_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_set_reference_field_at then
				print_builtin_ise_runtime_set_reference_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_unlock_marking then
				print_builtin_ise_runtime_unlock_marking_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_unmark_object then
				print_builtin_ise_runtime_unmark_object_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_memory_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "MEMORY".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_memory: a_feature.is_builtin_memory_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_memory_free then
				print_builtin_memory_free_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_pointer_ref_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "POINTER_REF".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_pointer_ref: a_feature.is_builtin_pointer_ref_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_pointer_ref_set_item then
				print_builtin_pointer_ref_set_item_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_procedure_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "PROCEDURE".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_procedure: a_feature.is_builtin_procedure_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_procedure_call then
				print_builtin_procedure_call_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_procedure_fast_call then
				print_builtin_procedure_fast_call_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_real_n_ref_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in classes "REAL_N_REF".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_real_n_ref: a_feature.is_builtin_real_n_ref_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_real_n_ref_set_item then
				print_builtin_real_n_ref_set_item_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_special_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "SPECIAL".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_special: a_feature.is_builtin_special_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_special_put then
				print_builtin_special_put_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_tuple_procedure_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to procedure `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "TUPLE".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_tuple: a_feature.is_builtin_tuple_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_tuple_set_object_comparison then
				print_builtin_tuple_set_object_comparison_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_procedure_call (a_feature, a_target_type, a_check_void_target)
			end
		end

feature {NONE} -- Expression generation

	print_across_expression (an_expression: ET_ACROSS_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
			print_iteration_expression (an_expression)
		end

	print_adapted_attribute_access (an_attribute: ET_DYNAMIC_FEATURE; a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to `an_attribute' applied to `a_target' of dynamic type `a_target_type'.
			-- Useful for example when the access to `a_target' needs to be adapted depending on
			-- the expandedness of the dynamic and static types of `a_target'. For example if the
			-- static type is a reference type and the dynamic type is expanded, then `a_target' will
			-- need to be unboxed before accessing its attribute.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			an_attribute_not_void: an_attribute /= Void
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if dynamic_type_set (a_target).is_expanded then
				print_attribute_access (an_attribute, a_target, a_target_type, a_check_void_target)
			else
				print_boxed_attribute_access (an_attribute, a_target, a_target_type, a_check_void_target)
			end
		end

	print_adapted_expression_with_agent (a_print_expression: PROCEDURE;
		a_source_type_set: ET_DYNAMIC_TYPE_SET; a_target_type: ET_DYNAMIC_PRIMARY_TYPE;
		a_use_boxed_pointer: BOOLEAN)
			-- Print to `current_file' an expression (using `a_print_expression') with
			-- dynamic type set `a_source_type_set' when the static type expected
			-- by the caller is `a_target_type'. This is useful for example when the
			-- expression is a call to a query and this call is one of the alternatives
			-- of a polymorphic call whose static type expected by the caller is
			-- `a_target_type'. In that case the result of the query needs to be
			-- adapted to match the kind of result type expected by the caller.
			-- For example, if we have:
			--
			--    l: LIST [ANY]
			--    li: LIST [INTEGER]
			--    ls: LIST [STRING]
			--    ...
			--    a: ANY
			--    if b then
			--       l := li
			--    else
			--       l := ls
			--    end
			--    a := l.item
			--
			-- depending on the value of 'b', the call to 'item' will yield an integer
			-- or a string. If it's an integer (this corresponds to the static type
			-- of `a_source_type_set'), then its value will need to be boxed because
			-- the signature of 'item' in 'LIST [ANY]' is of reference type (this
			-- corresponds to `a_target_type').
			--
			-- `a_use_boxed_pointer' indicates whether we should keep the address of the
			-- expression, and not just its value, if boxing is needed.
			-- Useful when the expression is an expanded attribute, tuple label or
			-- "SPECIAL" item to be used as the target of another call.
			--
			-- Note that contrary to `print_attachment_expression', the result of the
			-- query should not be copied or cloned. It will occur later on when the call
			-- is the source of an assignment or passed as actual argument (in the example
			-- above it will occur when the value of the call to 'l.item' is assigned
			-- to 'a').
			--
			-- And also, contrary to `print_attachment_expression', the static type of
			-- the result of the query is expected to conform to the type of the underlying
			-- call. Unless the query has a result type of the form 'like argument'.
			-- For example 'clone (1)' is expected to be of type "INTEGER" and
			-- 'clone ("gobo")' is expected to be of type "STRING". But the corresponding
			-- generated function for 'clone' will be declared to return an "ANY".
		require
			a_print_expression_not_void: a_print_expression /= Void
			a_source_type_set_not_void: a_source_type_set /= Void
			a_target_type_not_void: a_target_type /= Void
		local
			l_source_type: ET_DYNAMIC_PRIMARY_TYPE
			l_do_check_void: BOOLEAN
		do
			l_source_type := a_source_type_set.static_type.primary_type
			if a_target_type.is_expanded then
					-- Only expanded types conform to expanded types.
					-- So no unboxing is needed, and copy or clone will
					-- not be called at this stage.
					--
					-- However we may need to unbox when the query has
					-- a result type of the form 'like argument' (see
					-- header comment).
				if not l_source_type.is_expanded then
						-- The source object has been boxed.
					current_file.put_character ('(')
					print_boxed_type_cast (a_target_type, current_file)
					current_file.put_character ('(')
					if a_source_type_set.can_be_void  then
						can_be_void_target_count := can_be_void_target_count + 1
						current_file.put_string (c_ge_void)
						current_file.put_character ('(')
						l_do_check_void := True
					else
						never_void_target_count := never_void_target_count + 1
					end
					a_print_expression.call ([])
					if l_do_check_void then
						current_file.put_character (')')
					end
					current_file.put_character (')')
					current_file.put_character (')')
						-- The source object has been boxed.
					current_file.put_string (c_arrow)
					print_boxed_attribute_item_name (a_target_type, current_file)
				else
					a_print_expression.call ([])
				end
			else
				if l_source_type.is_expanded then
						-- We need to box the object, but without triggering a call to
						-- 'copy' (it will be called during subsequent attachment).
					if a_use_boxed_pointer then
						current_file.put_string (c_ge_boxed_pointer)
						current_file.put_integer (l_source_type.id)
						current_file.put_character ('(')
						current_file.put_character ('&')
						current_file.put_character ('(')
						a_print_expression.call ([])
						current_file.put_character (')')
						current_file.put_character (')')
					else
						current_file.put_string (c_ge_boxed)
						current_file.put_integer (l_source_type.id)
						current_file.put_character ('(')
						a_print_expression.call ([])
						current_file.put_character (')')
					end
				else
						-- Even if the dynamic type set of the expression contains
						-- expanded types, there will be no copy or clone
						-- at this stage.
					a_print_expression.call ([])
				end
			end
		end

	print_assignment_operand (a_operand: ET_EXPRESSION; a_source_type_set: ET_DYNAMIC_TYPE_SET; a_target: ET_WRITABLE; a_target_type: ET_DYNAMIC_TYPE)
			-- Print `a_operand' of dynamic type set `a_source_type_set' when it
			-- is the source of an assignment to `a_target' of type `a_target_type'.
			-- Make sure that the object is cloned when its type is expanded.
			-- Note that the object will be boxed or unboxed when needed
			-- later, in the call to `print_attachment_expression'.
		require
			a_operand_not_void: a_operand /= Void
			a_source_type_set_not_void: a_source_type_set /= Void
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if (a_source_type_set.is_expanded = a_target_type.is_expanded) and then not is_twin_needed_in_attachment (a_source_type_set) then
				assignment_target := a_target
			else
				assignment_target := Void
			end
			print_attachment_operand (a_operand, a_source_type_set)
			assignment_target := Void
		end

	print_attachment_expression (an_expression: ET_EXPRESSION; a_source_type_set: ET_DYNAMIC_TYPE_SET; a_target_type: ET_DYNAMIC_TYPE)
			-- Print `an_expression' of dynamic type set `a_source_type_set'
			-- when it is to be attached to an entity of type `a_target_type'.
			-- Make sure that the expression is boxed or unboxed when needed.
			-- Useful when printing the source of an assignment or actual
			-- arguments of a routine.
			-- Note that the cloning of then object when its type is expanded
			-- is done earlier, in the call to `print_attachment_operand'.
			--
			-- Note that the static type of the source does not necessarily
			-- conform to the target type. This can be the case for example
			-- with assignment attempts or because of covariance of arguments
			-- as in the example below:
			--
			--    c1: COMPARABLE
			--    c2: COMPARABLE
			--    ...
			--    c1 := 1
			--    c2 := 2
			--    c1 < c2
			--
			-- At run-time, 'INTEGER.is_less alias "<"' will be called. 'c2' is of static
			-- type COMPARABLE, which does not conform to the expected argument
			-- type INTEGER. But according to the dynamic type set mechanism there
			-- is no CAT-call in that case. So we will just need to unbox the
			-- integer 2 (which had been previously boxed in the assignment to
			-- 'c2') before passing it as argument of 'INTEGER.is_less alias "<"'.
		require
			an_expression_not_void: an_expression /= Void
			a_source_type_set_not_void: a_source_type_set /= Void
			a_target_type_not_void: a_target_type /= Void
		local
			l_source_type: ET_DYNAMIC_TYPE
			l_target_primary_type: ET_DYNAMIC_PRIMARY_TYPE
			l_source_primary_type: ET_DYNAMIC_PRIMARY_TYPE
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_non_conforming_types: DS_ARRAYED_LIST [INTEGER]
			l_has_non_conforming_types: BOOLEAN
			l_dts_ids: STRING
			l_dts_name: detachable STRING
			i, nb: INTEGER
			l_need_unboxing: BOOLEAN
			l_target_type_is_attached: BOOLEAN
		do
			l_source_type := a_source_type_set.static_type
			l_source_primary_type := l_source_type.primary_type
			l_target_primary_type := a_target_type.primary_type
			if not l_source_primary_type.conforms_to_primary_type (l_target_primary_type) then
					-- Make sure that CAT-call errors will be reported at run-time.
				nb := a_source_type_set.count
				l_non_conforming_types := attachment_dynamic_type_ids
				if nb > l_non_conforming_types.capacity then
					l_non_conforming_types.resize (nb)
				end
				from i := 1 until i > nb loop
					l_dynamic_type := a_source_type_set.dynamic_type (i)
					if not l_dynamic_type.conforms_to_primary_type (l_target_primary_type) then
						l_non_conforming_types.put_last (l_dynamic_type.id)
					end
					i := i + 1
				end
				if not l_non_conforming_types.is_empty then
					l_has_non_conforming_types := True
					l_non_conforming_types.sort (dynamic_type_id_sorter)
					nb := l_non_conforming_types.count
					create l_dts_ids.make (5 * nb)
					from i := 1 until i > nb loop
						if i /= 1 then
							l_dts_ids.append_character (',')
						end
						l_dts_ids.append_string (l_non_conforming_types.item (i).out)
						i := i + 1
					end
					dynamic_type_id_set_names.search (l_dts_ids)
					if dynamic_type_id_set_names.found then
						l_dts_name := dynamic_type_id_set_names.found_item
					else
						l_dts_name := c_ge_dts + (dynamic_type_id_set_names.count + 1).out
						dynamic_type_id_set_names.force_last_new (l_dts_name, l_dts_ids)
					end
				end
				l_non_conforming_types.wipe_out
			end
			l_need_unboxing := l_target_primary_type.is_expanded and then (not l_source_primary_type.is_expanded or l_has_non_conforming_types)
			if l_need_unboxing then
				current_file.put_character ('(')
				print_boxed_type_cast (l_target_primary_type, current_file)
				current_file.put_character ('(')
			end
			if l_has_non_conforming_types then
				current_file.put_string (c_ge_catcall)
				current_file.put_character ('(')
			end
			if l_target_primary_type.is_expanded then
				if l_source_primary_type.is_expanded then
					if l_has_non_conforming_types then
							-- We need to box the source object in order
							-- to pass it to 'GE_catcall'.
						print_boxed_expression (an_expression, l_source_primary_type)
					else
						print_expression (an_expression)
					end
				else
						-- The source object has been boxed.
					print_expression (an_expression)
				end
			else
				if l_source_primary_type.is_expanded then
						-- We need to box the source object.
					print_boxed_expression (an_expression, l_source_primary_type)
				else
						-- Deal with void-safety: when `a_target_type' is attached,
						-- then we need to check that we don't pass Void.
					l_target_type_is_attached := a_target_type.is_attached
-- TODO: Improve dynamic type set 'is_never_void' in void-safe mode before
-- using this code. Otherwise we end up with too many unnecessary checks for void.
--					print_non_void_expression (an_expression, l_target_type_is_attached)
					print_expression (an_expression)
				end
			end
			if l_has_non_conforming_types and l_dts_name /= Void then
				current_file.put_character (',')
				current_file.put_string (l_dts_name)
				current_file.put_character (',')
				current_file.put_integer (nb)
				current_file.put_character (')')
			end
			if l_need_unboxing then
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_string (c_arrow)
				print_boxed_attribute_item_name (l_source_primary_type, current_file)
			end
		end

	print_attachment_operand (a_operand: ET_EXPRESSION; a_source_type_set: ET_DYNAMIC_TYPE_SET)
			-- Print `a_operand' of dynamic type set `a_source_type_set'
			-- when it is to be attached to an entity.
			-- Make sure that the object is cloned when its type is expanded.
			-- Useful when printing the source of an assignment or actual arguments
			-- of a routine.
			-- Note that the object will be boxed or unboxed when needed
			-- later, in the call to `print_attachment_expression'.
		require
			a_operand_not_void: a_operand /= Void
			a_source_type_set_not_void: a_source_type_set /= Void
		local
			l_source_type: ET_DYNAMIC_TYPE
			l_source_primary_type: ET_DYNAMIC_PRIMARY_TYPE
			l_twin_feature_name: ET_IDENTIFIER
			l_twin_expression: ET_QUALIFIED_CALL_EXPRESSION
			l_twin_type_set: ET_DYNAMIC_STANDALONE_TYPE_SET
			l_query_call: detachable ET_DYNAMIC_QUALIFIED_QUERY_CALL
			l_index: INTEGER
			l_old_index: INTEGER
			l_checked_void: BOOLEAN
			l_call_operand: ET_EXPRESSION
			l_temp: detachable ET_IDENTIFIER
			l_temp_index: INTEGER
			l_assignment_target: like assignment_target
		do
			l_source_type := a_source_type_set.static_type
			l_source_primary_type := l_source_type.primary_type
			if l_source_primary_type.is_expanded then
				if l_source_primary_type.is_basic and then not l_source_primary_type.has_redefined_copy_routine then
						-- Optimization: no need to explicitly call 'twin' in that case.
					print_operand (a_operand)
				else
					l_twin_feature_name := new_twin_feature_name
					l_twin_expression := new_qualified_call_expression (a_operand, l_twin_feature_name, Void)
					l_twin_expression.set_index (a_operand.index)
					print_operand (l_twin_expression)
					free_qualified_call_expression (l_twin_expression)
					free_twin_feature_name (l_twin_feature_name)
				end
			else
					-- Check to see whether some of the types in the source type set are expanded.
					-- If so, 'twin' needs to be called on them.
				l_twin_type_set := new_standalone_type_set (l_source_type)
				l_twin_type_set.put_expanded_types (a_source_type_set)
				l_twin_type_set.set_never_void
				if l_twin_type_set.is_empty then
					print_operand (a_operand)
					free_standalone_type_set (l_twin_type_set)
				else
					l_assignment_target := assignment_target
					assignment_target := Void
					print_operand (a_operand)
					fill_call_operands (1)
					l_call_operand := call_operands.first
					call_operands.wipe_out
					print_indentation
					if l_assignment_target /= Void then
						operand_stack.force (l_assignment_target)
						print_writable (l_assignment_target)
					else
						l_temp := new_temp_variable (l_source_primary_type)
							-- We will set the index of `l_temp' later because
							-- it could still be used in `call_operands'.
						l_temp_index := l_call_operand.index
						operand_stack.force (l_temp)
						print_temp_name (l_temp, current_file)
					end
					print_assign_to
					current_file.put_character ('(')
					if a_source_type_set.can_be_void and then not l_call_operand.is_never_void then
						l_checked_void := True
						current_file.put_character ('(')
						print_expression (l_call_operand)
						print_and_then
					end
					current_file.put_character ('(')
					current_file.put_string (c_ge_type_infos)
					current_file.put_character ('[')
					print_boxed_attribute_type_id_access (l_call_operand, l_source_primary_type, False)
					current_file.put_character (']')
					current_file.put_character ('.')
					current_file.put_string (c_flags)
					current_file.put_character ('&')
					current_file.put_string (c_ge_type_flag_expanded)
					current_file.put_character (')')
					if l_checked_void then
						current_file.put_character (')')
					end
					current_file.put_character ('?')
					extra_dynamic_type_sets.force_last (l_twin_type_set)
					l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
					l_old_index := l_call_operand.index
					l_call_operand.set_index (l_index)
					l_twin_feature_name := new_twin_feature_name
					l_twin_expression := new_qualified_call_expression (l_call_operand, l_twin_feature_name, Void)
					l_twin_expression.set_index (l_index)
						-- Register the call to 'twin' so that it is handled correctly
						-- in case of polymorphism.
						-- The special treatment for polymorphism only occurs
						-- when the target has more than 2 possible dynamic types.
					if l_twin_type_set.count > 2 then
						create l_query_call.make (l_twin_expression, l_twin_type_set, l_twin_type_set, current_feature, current_type)
						l_source_primary_type.put_query_call (l_query_call)
					end
					print_expression (l_twin_expression)
					l_call_operand.set_index (l_old_index)
					extra_dynamic_type_sets.remove_last
					current_file.put_character (':')
					print_expression (l_call_operand)
					current_file.put_character (')')
					print_semicolon_newline
					if l_temp /= Void and then l_temp_index /= 0 then
							-- We had to wait until this stage to set the index of `l_temp'
							-- because it could have still been used in `call_operands'.
						l_temp.set_index (l_temp_index)
					end
						-- Clean up.
					if l_query_call = Void then
						free_qualified_call_expression (l_twin_expression)
						free_twin_feature_name (l_twin_feature_name)
						free_standalone_type_set (l_twin_type_set)
					end
				end
			end
		end

	print_attribute_access (an_attribute: ET_DYNAMIC_FEATURE; a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to `an_attribute' applied to object `a_target' of type `a_target_type'.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			an_attribute_not_void: an_attribute /= Void
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if a_target_type.is_expanded then
				current_file.put_character ('(')
				print_expression (a_target)
				current_file.put_character (')')
				current_file.put_character ('.')
			else
				current_file.put_character ('(')
				print_type_cast (a_target_type, current_file)
				current_file.put_character ('(')
				print_non_void_expression (a_target, a_check_void_target)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_string (c_arrow)
			end
			print_attribute_name (an_attribute, a_target_type, current_file)
		end

	print_attribute_special_capacity_access (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to 'capacity' pseudo attribute of class SPECIAL applied to object `a_target' of type `a_target_type'.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if a_target_type.is_expanded then
				current_file.put_character ('(')
				print_expression (a_target)
				current_file.put_character (')')
				current_file.put_character ('.')
			else
				current_file.put_character ('(')
				print_type_cast (a_target_type, current_file)
				current_file.put_character ('(')
				print_non_void_expression (a_target, a_check_void_target)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_string (c_arrow)
			end
			print_attribute_special_capacity_name (a_target_type, current_file)
		end

	print_attribute_special_count_access (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to 'count' pseudo attribute of class SPECIAL applied to object `a_target' of type `a_target_type'.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if a_target_type.is_expanded then
				current_file.put_character ('(')
				print_expression (a_target)
				current_file.put_character (')')
				current_file.put_character ('.')
			else
				current_file.put_character ('(')
				print_type_cast (a_target_type, current_file)
				current_file.put_character ('(')
				print_non_void_expression (a_target, a_check_void_target)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_string (c_arrow)
			end
			print_attribute_special_count_name (a_target_type, current_file)
		end

	print_attribute_special_offset_access (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to 'offset' pseudo attribute of class SPECIAL applied to object `a_target' of type `a_target_type'.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if a_target_type.is_expanded then
				current_file.put_character ('(')
				print_expression (a_target)
				current_file.put_character (')')
				current_file.put_character ('.')
			else
				current_file.put_character ('(')
				print_type_cast (a_target_type, current_file)
				current_file.put_character ('(')
				print_non_void_expression (a_target, a_check_void_target)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_string (c_arrow)
			end
			print_attribute_special_offset_name (a_target_type, current_file)
		end

	print_attribute_special_item_access (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to 'item' pseudo attribute of class SPECIAL applied to object `a_target' of type `a_target_type'.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if a_target_type.is_expanded then
				current_file.put_character ('(')
				print_expression (a_target)
				current_file.put_character (')')
				current_file.put_character ('.')
			else
				current_file.put_character ('(')
				print_type_cast (a_target_type, current_file)
				current_file.put_character ('(')
				print_non_void_expression (a_target, a_check_void_target)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_string (c_arrow)
			end
			print_attribute_special_item_name (a_target_type, current_file)
		end

	print_attribute_special_indexed_item_access (an_index: ET_EXPRESSION; a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' access to item at indexed `an_index'
			-- in the 'item' pseudo attribute of class SPECIAL applied to
			-- object `a_target' of type `a_target_type'. `an_index' must
			-- be declared of (i.e. its static type must be) one of the
			-- possibly sized integer types.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			an_index_not_void: an_index /= Void
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			print_attribute_special_item_access (a_target, a_target_type, a_check_void_target)
			current_file.put_character ('[')
			print_expression (an_index)
			current_file.put_character (']')
		end

	print_attribute_tuple_item_access (i: INTEGER; a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to `i'-th 'item' pseudo attribute of class TUPLE
			-- applied to Tuple object `a_target' of type `a_target_type'.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if a_target_type.is_expanded then
				current_file.put_character ('(')
				print_expression (a_target)
				current_file.put_character (')')
				current_file.put_character ('.')
			else
				current_file.put_character ('(')
				print_type_cast (a_target_type, current_file)
				current_file.put_character ('(')
				print_non_void_expression (a_target, a_check_void_target)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_string (c_arrow)
			end
			print_attribute_tuple_item_name (i, a_target_type, current_file)
		end

	print_attribute_type_id_access (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to 'type_id' pseudo attribute applied to object `a_target' of type `a_target_type'.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if a_target_type.is_expanded then
				current_file.put_character ('(')
				print_expression (a_target)
				current_file.put_character (')')
				current_file.put_character ('.')
			else
				current_file.put_character ('(')
				current_file.put_character ('(')
				print_type_declaration (a_target_type, current_file)
				current_file.put_character (')')
				current_file.put_character ('(')
				print_non_void_expression (a_target, a_check_void_target)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_string (c_arrow)
			end
			print_attribute_type_id_name (a_target_type, current_file)
		end

	print_attribute_flags_access (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to 'flags' pseudo attribute applied to object `a_target' of type `a_target_type'.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if a_target_type.is_expanded then
				current_file.put_character ('(')
				print_expression (a_target)
				current_file.put_character (')')
				current_file.put_character ('.')
			else
				current_file.put_character ('(')
				current_file.put_character ('(')
				print_type_declaration (a_target_type, current_file)
				current_file.put_character (')')
				current_file.put_character ('(')
				print_non_void_expression (a_target, a_check_void_target)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_string (c_arrow)
			end
			print_attribute_flags_name (a_target_type, current_file)
		end

	print_attribute_onces_access (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to 'onces' pseudo attribute applied to object `a_target' of type `a_target_type'.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if a_target_type.is_expanded then
				current_file.put_character ('(')
				print_expression (a_target)
				current_file.put_character (')')
				current_file.put_character ('.')
			else
				current_file.put_character ('(')
				print_type_cast (a_target_type, current_file)
				current_file.put_character ('(')
				print_non_void_expression (a_target, a_check_void_target)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_string (c_arrow)
			end
			print_attribute_onces_name (a_target_type, current_file)
		end

	print_binary_integer_constant (a_constant: ET_BINARY_INTEGER_CONSTANT)
			-- Print `a_constant'.
		require
			a_constant_not_void: a_constant /= Void
		do
			print_integer_constant (a_constant)
		end

	print_boxed_attribute_access (an_attribute: ET_DYNAMIC_FEATURE; a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to `an_attribute' applied to `a_target' of type the boxed version of `a_target_type'.
			-- The static type (i.e. declared type) of `a_target' is assumed to be of reference type.
			-- There is no assumption about `a_target_type': it may be reference or expanded.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			an_attribute_not_void: an_attribute /= Void
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if a_target_type.is_expanded then
				current_file.put_character ('(')
				print_boxed_attribute_item_access (a_target, a_target_type, a_check_void_target)
				current_file.put_character (')')
				current_file.put_character ('.')
			else
				current_file.put_character ('(')
				print_type_cast (a_target_type, current_file)
				current_file.put_character ('(')
				print_non_void_expression (a_target, a_check_void_target)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_string (c_arrow)
			end
			print_attribute_name (an_attribute, a_target_type, current_file)
		end

	print_boxed_attribute_flags_access (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to 'flags' pseudo attribute applied to `a_target' of type the boxed version of `a_target_type'.
			-- The static type (i.e. declared type) of `a_target' is assumed to be of reference type.
			-- There is no assumption about `a_target_type': it may be reference or expanded.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			current_file.put_character ('(')
			current_file.put_character ('(')
			print_boxed_type_declaration (a_target_type, current_file)
			current_file.put_character (')')
			current_file.put_character ('(')
			print_non_void_expression (a_target, a_check_void_target)
			current_file.put_character (')')
			current_file.put_character (')')
			current_file.put_string (c_arrow)
			print_attribute_flags_name (a_target_type, current_file)
		end

	print_boxed_attribute_item_access (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to 'item' pseudo attribute applied to `a_target' of type the boxed version of `a_target_type'.
			-- The static type (i.e. declared type) of `a_target' is assumed to be of reference type.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
			a_target_type_expanded: a_target_type.is_expanded
		do
			current_file.put_character ('(')
			print_boxed_type_cast (a_target_type, current_file)
			current_file.put_character ('(')
			print_non_void_expression (a_target, a_check_void_target)
			current_file.put_character (')')
			current_file.put_character (')')
			current_file.put_string (c_arrow)
			print_boxed_attribute_item_name (a_target_type, current_file)
		end

	print_boxed_attribute_pointer_access (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to 'pointer' pseudo attribute applied to `a_target' of type the boxed version of `a_target_type'.
			-- The static type (i.e. declared type) of `a_target' is assumed to be of reference type.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
			a_target_type_expanded: a_target_type.is_expanded
		do
			current_file.put_character ('(')
			print_boxed_type_cast (a_target_type, current_file)
			current_file.put_character ('(')
			print_non_void_expression (a_target, a_check_void_target)
			current_file.put_character (')')
			current_file.put_character (')')
			current_file.put_string (c_arrow)
			print_boxed_attribute_pointer_name (a_target_type, current_file)
		end

	print_boxed_attribute_type_id_access (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print access to 'type_id' pseudo attribute applied to `a_target' of type the boxed version of `a_target_type'.
			-- The static type (i.e. declared type) of `a_target' is assumed to be of reference type.
			-- There is no assumption about `a_target_type': it may be reference or expanded.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			current_file.put_character ('(')
			current_file.put_character ('(')
			print_boxed_type_declaration (a_target_type, current_file)
			current_file.put_character (')')
			current_file.put_character ('(')
			print_non_void_expression (a_target, a_check_void_target)
			current_file.put_character (')')
			current_file.put_character (')')
			current_file.put_string (c_arrow)
			print_attribute_type_id_name (a_target_type, current_file)
		end

	print_boxed_expression (an_expression: ET_EXPRESSION; a_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print boxed version of `an_expression' of type `a_type'.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
		require
			an_expression_not_void: an_expression /= Void
			a_type_not_void: a_type /= Void
			a_type_expanded: a_type.is_expanded
		do
			current_file.put_string (c_ge_boxed)
			current_file.put_integer (a_type.id)
			current_file.put_character ('(')
			print_expression (an_expression)
			current_file.put_character (')')
		end

	print_bracket_expression (an_expression: ET_BRACKET_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
			print_qualified_call_expression (an_expression)
		end

	print_character_constant (a_constant: ET_CHARACTER_CONSTANT)
			-- Print `a_constant'.
		require
			a_constant_not_void: a_constant /= Void
		local
			l_temp: ET_IDENTIFIER
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if in_operand then
				if in_call_target then
					in_operand := False
					l_dynamic_type := dynamic_type_set (a_constant).static_type.primary_type
					l_temp := new_temp_variable (l_dynamic_type)
					print_indentation
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_character_constant (a_constant)
					current_file.put_character (';')
					current_file.put_new_line
					l_temp.set_index (a_constant.index)
					operand_stack.force (l_temp)
					in_operand := True
				else
					operand_stack.force (a_constant)
				end
			else
				l_dynamic_type := dynamic_type_set (a_constant).static_type.primary_type
				print_type_cast (l_dynamic_type, current_file)
				current_file.put_character ('(')
				if current_system.character_32_type.same_named_type (l_dynamic_type.base_type, current_type.base_type, current_type.base_type) then
					current_file.put_string (c_ge_nat32)
					current_file.put_character ('(')
					current_file.put_natural_32 (a_constant.value.natural_32_code)
					current_file.put_character (')')
				else
					print_escaped_character_8 (a_constant.value.to_character_8)
				end
				current_file.put_character (')')
			end
		end

	print_check_void_expression (a_expression: ET_EXPRESSION; a_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void: BOOLEAN)
			-- Print `a_expression'.
			-- `a_dynamic_type' is one of the possible dynamic types of `a_expression'.
			-- `a_check_void' means that we need to check whether the expression is Void or not.
		require
			a_expression_not_void: a_expression /= Void
			a_dynamic_type_not_void: a_dynamic_type /= Void
		local
			l_do_check_void: BOOLEAN
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
		do
			if a_check_void then
				l_dynamic_type_set := dynamic_type_set (a_expression)
				if not a_dynamic_type.is_expanded and then l_dynamic_type_set.can_be_void and not a_expression.is_never_void then
					can_be_void_target_count := can_be_void_target_count + 1
					current_file.put_string (c_ge_void)
					current_file.put_character ('(')
					l_do_check_void := True
				else
					never_void_target_count := never_void_target_count + 1
				end
			end
			a_expression.process (Current)
			if l_do_check_void then
				current_file.put_character (')')
			end
		end

	print_convert_builtin_expression (an_expression: ET_CONVERT_BUILTIN_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_convert_feature: ET_CONVERT_FEATURE
			l_target_type: ET_DYNAMIC_PRIMARY_TYPE
			l_target_type_set: ET_DYNAMIC_TYPE_SET
			l_source_type: ET_DYNAMIC_PRIMARY_TYPE
			l_source_type_set: ET_DYNAMIC_TYPE_SET
			l_temp: detachable ET_IDENTIFIER
			l_temp_index: INTEGER
			l_assignment_target: like assignment_target
		do
			l_convert_feature := an_expression.convert_feature
			l_target_type_set := dynamic_type_set (an_expression)
			l_target_type := l_target_type_set.static_type.primary_type
			l_source_type_set := dynamic_type_set (an_expression.expression)
			l_source_type := l_source_type_set.static_type.primary_type
			if l_source_type.conforms_to_primary_type (l_target_type) then
-- TODO: built-in feature with formal generic parameter? Should not be needed with ECMA Eiffel.
				an_expression.expression.process (Current)
			else
-- TODO: built-in feature between basic types? Should not be needed with ECMA Eiffel.
				l_assignment_target := assignment_target
				assignment_target := Void
				print_operand (an_expression.expression)
				fill_call_operands (1)
				if in_operand then
					if l_assignment_target /= Void then
						operand_stack.force (l_assignment_target)
						print_indentation
						print_writable (l_assignment_target)
					else
						l_temp := new_temp_variable (l_target_type)
							-- We will set the index of `l_temp' later because
							-- it could still be used in `call_operands'.
						l_temp_index := an_expression.index
						operand_stack.force (l_temp)
						print_indentation
						print_temp_name (l_temp, current_file)
					end
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
				end
				print_type_cast (l_target_type, current_file)
				current_file.put_character ('(')
				print_expression (call_operands.item (1))
				current_file.put_character (')')
				if in_operand then
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				end
				call_operands.wipe_out
				if l_temp /= Void and then l_temp_index /= 0 then
						-- We had to wait until this stage to set the index of `l_temp'
						-- because it could have still been used in `call_operands'.
					l_temp.set_index (l_temp_index)
				end
			end
		end

	print_convert_from_expression (an_expression: ET_CONVERT_FROM_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
			print_creation_expression (an_expression)
		end

	print_convert_to_expression (an_expression: ET_CONVERT_TO_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
			print_qualified_call_expression (an_expression)
		end

	print_create_expression (an_expression: ET_CREATE_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
			print_creation_expression (an_expression)
		end

	print_creation_expression (an_expression: ET_CREATION_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_target_type_set: ET_DYNAMIC_TYPE_SET
			l_target_type: ET_DYNAMIC_PRIMARY_TYPE
			l_seed: INTEGER
			i, nb: INTEGER
			l_temp: detachable ET_IDENTIFIER
			l_temp_index: INTEGER
			l_assignment_target: like assignment_target
			l_actual: ET_EXPRESSION
		do
			l_assignment_target := assignment_target
			assignment_target := Void
			l_target_type_set := dynamic_type_set (an_expression)
			l_target_type := l_target_type_set.static_type.primary_type
			if attached an_expression.arguments as l_actuals then
				nb := l_actuals.count
				from i := 1 until i > nb loop
					l_actual := l_actuals.actual_argument (i)
					print_attachment_operand (l_actual, dynamic_type_set (l_actual))
					i := i + 1
				end
			end
			fill_call_operands (nb)
			if in_operand then
				print_indentation
				if l_assignment_target /= Void then
					operand_stack.force (l_assignment_target)
					print_writable (l_assignment_target)
				else
					l_temp := new_temp_variable (l_target_type)
						-- We will set the index of `l_temp' later because
						-- it could still be used in `call_operands'.
					l_temp_index := an_expression.index
					operand_stack.force (l_temp)
					print_temp_name (l_temp, current_file)
				end
				print_assign_to
				current_file.put_character ('(')
			end
			if attached an_expression.name as l_name then
				l_seed := l_name.seed
			else
				l_seed := current_system.default_create_seed
			end
			if not attached l_target_type.seeded_dynamic_procedure (l_seed, current_dynamic_system) as l_procedure then
					-- Internal error: there should be a procedure with `l_seed'.
					-- It has been computed in ET_FEATURE_CHECKER or else an
					-- error should have already been reported.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_creation_procedure_call (l_procedure, l_target_type)
			end
			if in_operand then
				current_file.put_character (')')
				print_semicolon_newline
			end
			call_operands.wipe_out
			if l_temp /= Void and then l_temp_index /= 0 then
					-- We had to wait until this stage to set the index of `l_temp'
					-- because it could have still been used in `call_operands'.
				l_temp.set_index (l_temp_index)
			end
		end

	print_current (an_expression: ET_CURRENT)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
			if current_agent /= Void then
				agent_target.process (Current)
			elseif in_operand then
				operand_stack.force (an_expression)
			elseif attached call_target_type as l_call_target_type then
				check in_call_target: in_call_target end
				if current_type.is_expanded then
					print_current_name (current_file)
				elseif l_call_target_type.is_expanded then
						-- We need to unbox the object and then pass its address.
					print_boxed_attribute_pointer_access (an_expression, l_call_target_type, False)
				else
					print_current_name (current_file)
				end
			elseif not current_type.is_expanded then
				print_current_name (current_file)
			else
				current_file.put_character ('*')
				print_current_name (current_file)
			end
		end

	print_current_address (an_expression: ET_CURRENT_ADDRESS)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_temp: ET_IDENTIFIER
			l_pointer_type: ET_DYNAMIC_PRIMARY_TYPE
			l_pointer: BOOLEAN
		do
			l_pointer_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.pointer_type, current_type.base_type)
			l_dynamic_type_set := dynamic_type_set (an_expression)
			l_dynamic_type := l_dynamic_type_set.static_type.primary_type
			l_pointer := (l_dynamic_type = l_pointer_type)
			if not l_pointer then
					-- $Current is of type TYPED_POINTER.
				l_temp := new_temp_variable (l_dynamic_type)
				l_temp.set_index (an_expression.index)
				operand_stack.force (l_temp)
				l_queries := l_dynamic_type.queries
				if l_queries.is_empty then
						-- Internal error: TYPED_POINTER should have an attribute
						-- `pointer_item' at first position.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					print_indentation
					print_attribute_access (l_queries.first, l_temp, l_dynamic_type, False)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
				end
			end
			if l_pointer and in_operand then
					-- $Current is of type POINTER.
				operand_stack.force (an_expression)
			else
				print_type_cast (l_pointer_type, current_file)
				current_file.put_character ('(')
				if current_type.is_expanded then
					current_file.put_character ('&')
					print_expression (an_expression.current_keyword)
				elseif attached {ET_DYNAMIC_SPECIAL_TYPE} current_type as l_special_type then
					print_attribute_special_item_access (an_expression.current_keyword, l_special_type, False)
				else
					print_expression (an_expression.current_keyword)
				end
				current_file.put_character (')')
			end
			if not l_pointer then
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_equality_expression (a_expression: ET_EQUALITY_EXPRESSION)
			-- Print `a_expression'.
		require
			a_expression_not_void: a_expression /= Void
		local
			l_left_operand: ET_EXPRESSION
			l_right_operand: ET_EXPRESSION
			l_left_type_set: ET_DYNAMIC_TYPE_SET
			l_right_type_set: ET_DYNAMIC_TYPE_SET
			l_equality_types: ET_DYNAMIC_EQUALITY_TYPES
			l_assignment_target: like assignment_target
		do
			l_assignment_target := assignment_target
			assignment_target := Void
			l_left_operand := a_expression.left
			l_right_operand := a_expression.right
			l_left_type_set := dynamic_type_set (l_left_operand)
			l_right_type_set := dynamic_type_set (l_right_operand)
				-- Process operands.
			print_operand (l_left_operand)
				-- Call `print_attachment_operand' and not `print_operand'
				-- because the right operand might be the argument of a call
				-- to 'is_equal' and it will require some cloning if its
				-- type is expanded.
			print_attachment_operand (l_right_operand, l_right_type_set)
			fill_call_operands (2)
			assignment_target := l_assignment_target
			l_equality_types := new_equality_types (l_left_type_set, l_right_type_set)
			print_equality_call (a_expression, a_expression.operator.is_not_equal, l_equality_types)
			if current_equalities.is_empty or else current_equalities.last /= l_equality_types then
				free_equality_types (l_equality_types)
			end
			call_operands.wipe_out
		end

	print_expression (an_expression: ET_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			old_in_operand: BOOLEAN
			old_target_type: like call_target_type
			l_old_in_procedure_call_target: BOOLEAN
		do
			old_in_operand := in_operand
			old_target_type := call_target_type
			l_old_in_procedure_call_target := in_procedure_call_target
			in_operand := False
			in_procedure_call_target := False
			call_target_type := Void
			an_expression.process (Current)
			call_target_type := old_target_type
			in_procedure_call_target := l_old_in_procedure_call_target
			in_operand := old_in_operand
		end

	print_expression_address (an_expression: ET_EXPRESSION_ADDRESS)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp: ET_IDENTIFIER
			l_feature_address: ET_FEATURE_ADDRESS
		do
			l_dynamic_type_set := dynamic_type_set (an_expression.expression)
			l_dynamic_type := l_dynamic_type_set.static_type.primary_type
			l_temp := new_temp_variable (l_dynamic_type)
			mark_temp_variable_frozen (l_temp)
			l_temp.set_index (an_expression.expression.index)
			print_indentation
			print_temp_name (l_temp, current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			print_expression (an_expression.expression)
			current_file.put_character (';')
			current_file.put_new_line
			create l_feature_address.make (l_temp)
			l_feature_address.set_index (an_expression.index)
			print_feature_address (l_feature_address)
		end

	print_false_constant (a_constant: ET_FALSE_CONSTANT)
			-- Print `a_constant'.
		require
			a_constant_not_void: a_constant /= Void
		local
			l_temp: ET_IDENTIFIER
		do
			if in_operand then
				if in_call_target then
					l_temp := new_temp_variable (dynamic_type_set (a_constant).static_type.primary_type)
					print_indentation
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_false)
					current_file.put_character (';')
					current_file.put_new_line
					l_temp.set_index (a_constant.index)
					operand_stack.force (l_temp)
				else
					operand_stack.force (a_constant)
				end
			else
				current_file.put_string (c_eif_false)
			end
		end

	print_feature_address (an_expression: ET_FEATURE_ADDRESS)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_value_type_set: detachable ET_DYNAMIC_TYPE_SET
			l_special_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_temp: ET_IDENTIFIER
			l_pointer: BOOLEAN
			l_name: ET_FEATURE_NAME
			l_name_expression: detachable ET_EXPRESSION
			l_query: detachable ET_DYNAMIC_FEATURE
			l_procedure: detachable ET_DYNAMIC_FEATURE
			l_pointer_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			l_pointer_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.pointer_type, current_type.base_type)
			l_dynamic_type_set := dynamic_type_set (an_expression)
			l_dynamic_type := l_dynamic_type_set.static_type.primary_type
			l_pointer := (l_dynamic_type = l_pointer_type)
			if not l_pointer then
					-- $feature_name is of type TYPED_POINTER.
				l_temp := new_temp_variable (l_dynamic_type)
				l_temp.set_index (an_expression.index)
				operand_stack.force (l_temp)
				l_queries := l_dynamic_type.queries
				if l_queries.is_empty then
						-- Internal error: TYPED_POINTER should have an attribute
						-- `pointer_item' at first position.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					print_indentation
					print_attribute_access (l_queries.first, l_temp, l_dynamic_type, False)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
				end
			end
			if l_pointer and in_operand then
					-- $feature_name is of type POINTER.
				operand_stack.force (an_expression)
			else
				current_file.put_character ('(')
				l_name := an_expression.name
				if l_name.is_argument then
					l_name_expression := l_name.argument_name
				elseif l_name.is_local then
					if has_rescue then
							-- Keep track of the fact that the value of this local variable can
							-- possibly be modified. Useful to determine the 'volatile' status
							-- of the local variable when current feature has a rescue clause.
						locals_written.force_last (l_name.local_name.identifier)
					end
					l_name_expression := l_name.local_name
				elseif l_name.is_object_test_local then
					l_name_expression := l_name.object_test_local_name
				elseif l_name.is_iteration_item then
					l_name_expression := l_name.iteration_item_name
				elseif attached {ET_IDENTIFIER} l_name as l_identifier and then l_identifier.is_temporary then
					l_name_expression := l_identifier
				end
				if l_name_expression /= Void then
					l_value_type_set := dynamic_type_set (l_name_expression)
					if l_value_type_set.is_expanded then
						print_type_cast (l_pointer_type, current_file)
						current_file.put_character ('&')
						print_expression (l_name_expression)
					else
						l_special_type := l_value_type_set.special_type
						if l_special_type /= Void then
							current_file.put_character ('(')
							print_expression (l_name_expression)
							current_file.put_character ('?')
							print_type_cast (l_pointer_type, current_file)
							current_file.put_character ('(')
							current_file.put_string (c_ge_types)
							current_file.put_character ('[')
							print_attribute_type_id_access (l_name_expression, l_value_type_set.static_type.primary_type, False)
							current_file.put_character (']')
							current_file.put_character ('[')
							current_file.put_character ('0')
							current_file.put_character (']')
							current_file.put_character ('.')
							current_file.put_string (c_is_special)
							current_file.put_character ('?')
							print_type_cast (l_pointer_type, current_file)
							print_attribute_special_item_access (l_name_expression, l_special_type, False)
							current_file.put_character (':')
							print_type_cast (l_pointer_type, current_file)
							print_expression (l_name_expression)
							current_file.put_character (')')
							current_file.put_character (':')
							print_type_cast (l_pointer_type, current_file)
							current_file.put_character ('0')
							current_file.put_character (')')
						else
							print_type_cast (l_pointer_type, current_file)
							print_expression (l_name_expression)
						end
					end
				else
					l_query := current_type.seeded_dynamic_query (l_name.seed, current_dynamic_system)
					if l_query /= Void then
						if l_query.is_attribute then
							l_value_type_set := l_query.result_type_set
							if l_value_type_set = Void then
									-- Internal error: we know that `l_query' is an attribute.
								set_fatal_error
								error_handler.report_giaaa_error
							elseif l_value_type_set.is_expanded then
								print_type_cast (l_pointer_type, current_file)
								current_file.put_character ('&')
								current_file.put_character ('(')
								print_attribute_access (l_query, tokens.current_keyword, current_type, False)
								current_file.put_character (')')
							else
								l_special_type := l_value_type_set.special_type
								if l_special_type /= Void then
									l_temp := new_temp_variable (l_value_type_set.static_type.primary_type)
									current_file.put_character ('(')
									current_file.put_character ('(')
									print_temp_name (l_temp, current_file)
									current_file.put_character (' ')
									current_file.put_character ('=')
									current_file.put_character (' ')
									print_attribute_access (l_query, tokens.current_keyword, current_type, False)
									current_file.put_character (')')
									current_file.put_character ('?')
									print_type_cast (l_pointer_type, current_file)
									current_file.put_character ('(')
									current_file.put_string (c_ge_types)
									current_file.put_character ('[')
									print_attribute_type_id_access (l_temp, l_value_type_set.static_type.primary_type, False)
									current_file.put_character (']')
									current_file.put_character ('[')
									current_file.put_character ('0')
									current_file.put_character (']')
									current_file.put_character ('.')
									current_file.put_string (c_is_special)
									current_file.put_character ('?')
									print_type_cast (l_pointer_type, current_file)
									print_attribute_special_item_access (l_temp, l_special_type, False)
									current_file.put_character (':')
									print_type_cast (l_pointer_type, current_file)
									print_attribute_access (l_query, tokens.current_keyword, current_type, False)
									current_file.put_character (')')
									current_file.put_character (':')
									print_type_cast (l_pointer_type, current_file)
									current_file.put_character ('0')
									current_file.put_character (')')
									mark_temp_variable_free (l_temp)
								else
									print_type_cast (l_pointer_type, current_file)
									print_attribute_access (l_query, tokens.current_keyword, current_type, False)
								end
							end
						else
							register_called_feature (l_query)
							print_type_cast (l_pointer_type, current_file)
							current_file.put_character ('(')
							current_file.put_character ('&')
							print_address_routine_name (l_query, current_type, current_file)
							current_file.put_character (')')
						end
					else
						l_procedure := current_type.seeded_dynamic_procedure (l_name.seed, current_dynamic_system)
						if l_procedure /= Void then
							register_called_feature (l_procedure)
							print_type_cast (l_pointer_type, current_file)
							current_file.put_character ('(')
							current_file.put_character ('&')
							print_address_routine_name (l_procedure, current_type, current_file)
							current_file.put_character (')')
						else
								-- Internal error: It has been checked in ET_FEATURE_CHECKER that
								-- we should have either $argument, $local or $feature_name.
							set_fatal_error
							error_handler.report_giaaa_error
						end
					end
				end
				current_file.put_character (')')
			end
			if not l_pointer then
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_formal_argument (a_name: ET_IDENTIFIER)
			-- Print formal argument `a_name'.
		require
			a_name_not_void: a_name /= Void
			a_name_argument: a_name.is_argument
		local
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_static_type: ET_DYNAMIC_TYPE
			l_seed: INTEGER
		do
			if current_agent /= Void then
				l_seed := a_name.seed
				if l_seed < 1 or l_seed > agent_arguments.count then
						-- Internal error: we know at this stage that the
						-- number of formal and actual aguments is the same.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					agent_arguments.actual_argument (l_seed).process (Current)
				end
			elseif in_operand then
				operand_stack.force (a_name)
			elseif attached call_target_type as l_call_target_type then
				check in_call_target: in_call_target end
				l_dynamic_type_set := dynamic_type_set (a_name)
				l_static_type := l_dynamic_type_set.static_type
				if l_static_type.is_expanded then
						-- Pass the address of the expanded object.
					current_file.put_character ('&')
					print_argument_name (a_name, current_file)
				elseif l_call_target_type.is_expanded then
						-- We need to unbox the object and then pass its address.
					print_boxed_attribute_pointer_access (a_name, l_call_target_type, call_target_check_void)
				else
					print_argument_name (a_name, current_file)
				end
			else
				print_argument_name (a_name, current_file)
			end
		end

	print_hexadecimal_integer_constant (a_constant: ET_HEXADECIMAL_INTEGER_CONSTANT)
			-- Print `a_constant'.
		require
			a_constant_not_void: a_constant /= Void
		do
			print_integer_constant (a_constant)
		end

	print_if_expression (a_expression: ET_IF_EXPRESSION)
			-- Print `a_expression'.
		require
			a_expression_not_void: a_expression /= Void
		local
			l_temp: ET_IDENTIFIER
			l_temp_index: INTEGER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_dynamic_type: ET_DYNAMIC_TYPE
			l_expression: ET_EXPRESSION
			l_expression_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_elseif: ET_ELSEIF_EXPRESSION
			i, nb: INTEGER
		do
			assignment_target := Void
				-- Declaration of temporary result.
			l_dynamic_type_set := dynamic_type_set (a_expression)
			l_dynamic_type := l_dynamic_type_set.static_type
			l_temp := new_temp_variable (l_dynamic_type.primary_type)
			mark_temp_variable_frozen (l_temp)
				-- We will set the index of `l_temp' later because
				-- it could still be used in `call_operands'.
			l_temp_index := a_expression.index
			operand_stack.force (l_temp)
			print_operand (a_expression.conditional_expression)
			fill_call_operands (1)
			print_indentation
			current_file.put_string (c_if)
			current_file.put_character (' ')
			current_file.put_character ('(')
			print_expression (call_operands.first)
			call_operands.wipe_out
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			l_expression := a_expression.then_expression
			l_expression_dynamic_type_set := dynamic_type_set (l_expression)
			print_assignment_operand (l_expression, l_expression_dynamic_type_set, l_temp, l_dynamic_type)
			fill_call_operands (1)
			if call_operands.first /= l_temp then
				print_indentation
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attachment_expression (call_operands.first, l_expression_dynamic_type_set, l_dynamic_type)
				current_file.put_character (';')
				current_file.put_new_line
			end
			call_operands.wipe_out
			dedent
			print_indentation
			current_file.put_character ('}')
			if attached a_expression.elseif_parts as l_elseif_parts then
				nb := l_elseif_parts.count
				from i := 1 until i > nb loop
					l_elseif := l_elseif_parts.item (i)
					current_file.put_character (' ')
					current_file.put_string (c_else)
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					print_operand (l_elseif.conditional_expression)
					fill_call_operands (1)
					print_indentation
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_expression (call_operands.first)
					call_operands.wipe_out
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					l_expression := l_elseif.then_expression
					l_expression_dynamic_type_set := dynamic_type_set (l_expression)
					print_assignment_operand (l_expression, l_expression_dynamic_type_set, l_temp, l_dynamic_type)
					fill_call_operands (1)
					if call_operands.first /= l_temp then
						print_indentation
						print_temp_name (l_temp, current_file)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						print_attachment_expression (call_operands.first, l_expression_dynamic_type_set, l_dynamic_type)
						current_file.put_character (';')
						current_file.put_new_line
					end
					call_operands.wipe_out
					dedent
					print_indentation
					current_file.put_character ('}')
					i := i + 1
				end
			end
			current_file.put_character (' ')
			current_file.put_string (c_else)
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			l_expression := a_expression.else_expression
			l_expression_dynamic_type_set := dynamic_type_set (l_expression)
			print_assignment_operand (l_expression, l_expression_dynamic_type_set, l_temp, l_dynamic_type)
			fill_call_operands (1)
			if call_operands.first /= l_temp then
				print_indentation
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attachment_expression (call_operands.first, l_expression_dynamic_type_set, l_dynamic_type)
				current_file.put_character (';')
				current_file.put_new_line
			end
			call_operands.wipe_out
			dedent
			print_indentation
			current_file.put_character ('}')
			from i := 1 until i > nb loop
				current_file.put_new_line
				dedent
				print_indentation
				current_file.put_character ('}')
				i := i + 1
			end
			current_file.put_new_line
			if l_temp_index /= 0 then
					-- We had to wait until this stage to set the index of `l_temp'
					-- because it could have still been used in `call_operands'.
				l_temp.set_index (l_temp_index)
			end
			mark_temp_variable_unfrozen (l_temp)
		end

	print_infix_cast_expression (an_expression: ET_INFIX_CAST_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
			an_expression.expression.process (Current)
		end

	print_infix_expression (an_expression: ET_INFIX_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
			print_qualified_call_expression (an_expression)
		end

	print_integer_constant (a_constant: ET_INTEGER_CONSTANT)
			-- Print `a_constant'.
		require
			a_constant_not_void: a_constant /= Void
		local
			l_temp: ET_IDENTIFIER
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if in_operand then
				if in_call_target then
					in_operand := False
					l_dynamic_type := dynamic_type_set (a_constant).static_type.primary_type
					l_temp := new_temp_variable (l_dynamic_type)
					print_indentation
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_integer_constant (a_constant)
					current_file.put_character (';')
					current_file.put_new_line
					l_temp.set_index (a_constant.index)
					operand_stack.force (l_temp)
					in_operand := True
				else
					operand_stack.force (a_constant)
				end
			else
				l_dynamic_type := dynamic_type_set (a_constant).static_type.primary_type
				print_type_cast (l_dynamic_type, current_file)
				current_file.put_character ('(')
				print_integer_value (a_constant.value, a_constant.is_negative, l_dynamic_type, False)
				current_file.put_character (')')
			end
		end

	print_integer_value (a_abs_value: NATURAL_64; a_is_negative: BOOLEAN; a_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE; a_in_case_statement: BOOLEAN)
			-- Print integer of type `a_dynamic_type' with absolute value `a_abs_value'
			-- with a minus sign if `a_is_negative'.
			-- `a_in_case_statement' is used because lcc-win32 does not consider 64 bit constants as constants in case statement.
			-- We have to cast them down to 32 bit constants, with the risk that they do not fit in 32 bits.
		require
			a_dynamic_type_not_void: a_dynamic_type /= Void
		do
			if current_universe_impl.integer_8_type.same_named_type (a_dynamic_type.base_type, current_type.base_type, current_type.base_type) then
				current_file.put_string (c_ge_int8)
				current_file.put_character ('(')
				if a_is_negative then
					current_file.put_character ('-')
				end
				current_file.put_natural_64 (a_abs_value)
				current_file.put_character (')')
			elseif current_universe_impl.integer_16_type.same_named_type (a_dynamic_type.base_type, current_type.base_type, current_type.base_type) then
				current_file.put_string (c_ge_int16)
				current_file.put_character ('(')
				if a_is_negative then
					current_file.put_character ('-')
				end
				current_file.put_natural_64 (a_abs_value)
				current_file.put_character (')')
			elseif current_universe_impl.integer_32_type.same_named_type (a_dynamic_type.base_type, current_type.base_type, current_type.base_type) then
				if a_is_negative and then a_abs_value = {INTEGER_32}.Max_value.to_natural_64 + 1 then
					current_file.put_string (c_ge_min_int32)
				else
					current_file.put_string (c_ge_int32)
					current_file.put_character ('(')
					if a_is_negative then
						current_file.put_character ('-')
					end
					current_file.put_natural_64 (a_abs_value)
					current_file.put_character (')')
				end
			elseif current_universe_impl.integer_64_type.same_named_type (a_dynamic_type.base_type, current_type.base_type, current_type.base_type) then
				if a_in_case_statement then
					current_file.put_string (c_ge_case_int64)
					current_file.put_character ('(')
				end
				if a_is_negative and then a_abs_value = {INTEGER_64}.Max_value.to_natural_64 + 1 then
					current_file.put_string (c_ge_min_int64)
				else
					current_file.put_string (c_ge_int64)
					current_file.put_character ('(')
					if a_is_negative then
						current_file.put_character ('-')
					end
					current_file.put_natural_64 (a_abs_value)
					current_file.put_character (')')
				end
				if a_in_case_statement then
					current_file.put_character (')')
				end
			elseif current_universe_impl.natural_8_type.same_named_type (a_dynamic_type.base_type, current_type.base_type, current_type.base_type) then
				current_file.put_string (c_ge_nat8)
				current_file.put_character ('(')
				current_file.put_natural_64 (a_abs_value)
				current_file.put_character (')')
			elseif current_universe_impl.natural_16_type.same_named_type (a_dynamic_type.base_type, current_type.base_type, current_type.base_type) then
				current_file.put_string (c_ge_nat16)
				current_file.put_character ('(')
				current_file.put_natural_64 (a_abs_value)
				current_file.put_character (')')
			elseif current_universe_impl.natural_32_type.same_named_type (a_dynamic_type.base_type, current_type.base_type, current_type.base_type) then
				current_file.put_string (c_ge_nat32)
				current_file.put_character ('(')
				current_file.put_natural_64 (a_abs_value)
				current_file.put_character (')')
			elseif current_universe_impl.natural_64_type.same_named_type (a_dynamic_type.base_type, current_type.base_type, current_type.base_type) then
				if a_in_case_statement then
					current_file.put_string (c_ge_case_nat64)
					current_file.put_character ('(')
				end
				current_file.put_string (c_ge_nat64)
				current_file.put_character ('(')
				current_file.put_natural_64 (a_abs_value)
				current_file.put_character (')')
				if a_in_case_statement then
					current_file.put_character (')')
				end
			else
				if a_is_negative then
					current_file.put_character ('-')
				end
				current_file.put_natural_64 (a_abs_value)
			end
		end

	print_inspect_expression (a_expression: ET_INSPECT_EXPRESSION)
			-- Print `a_expression'.
		require
			a_expression_not_void: a_expression /= Void
		local
			l_temp: ET_IDENTIFIER
			l_temp_index: INTEGER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_expression_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_dynamic_type: ET_DYNAMIC_TYPE
			l_expression: ET_EXPRESSION
			l_when_part: ET_WHEN_EXPRESSION
			l_choices: ET_CHOICE_LIST
			l_choice: ET_CHOICE
			i, nb: INTEGER
			j, nb2: INTEGER
			l_has_case: BOOLEAN
			l_lower: detachable ET_CONSTANT
			l_upper: detachable ET_CONSTANT
			l_lower_integer: detachable ET_INTEGER_CONSTANT
			l_upper_integer: detachable ET_INTEGER_CONSTANT
			l_lower_character: detachable ET_CHARACTER_CONSTANT
			l_upper_character: detachable ET_CHARACTER_CONSTANT
			k, nb3: INTEGER
			l_i_nat32, l_nb_nat32: NATURAL_32
			l_value_type_set: ET_DYNAMIC_TYPE_SET
			l_value_type: ET_DYNAMIC_PRIMARY_TYPE
			l_stop: BOOLEAN
		do
-- TODO
			assignment_target := Void
				-- Declaration of temporary result.
			l_dynamic_type_set := dynamic_type_set (a_expression)
			l_dynamic_type := l_dynamic_type_set.static_type
			l_temp := new_temp_variable (l_dynamic_type.primary_type)
			mark_temp_variable_frozen (l_temp)
				-- We will set the index of `l_temp' later because
				-- it could still be used in `call_operands'.
			l_temp_index := a_expression.index
			operand_stack.force (l_temp)
			l_expression := a_expression.conditional.expression
			l_value_type_set := dynamic_type_set (l_expression)
			l_value_type := l_value_type_set.static_type.primary_type
			print_operand (l_expression)
			fill_call_operands (1)
			print_indentation
			current_file.put_string (c_switch)
			current_file.put_character (' ')
			current_file.put_character ('(')
			print_expression (call_operands.first)
			call_operands.wipe_out
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			if attached a_expression.when_parts as l_when_parts then
				nb := l_when_parts.count
				from i := 1 until i > nb loop
					l_when_part := l_when_parts.item (i)
					l_choices := l_when_part.choices
					nb2 := l_choices.count
					if nb2 = 0 then
						-- Do nothing.
					else
						l_has_case := False
						from j := 1 until j > nb2 loop
							l_choice := l_choices.choice (j)
							if l_choice.is_range then
-- TODO
								l_lower := choice_constant (l_choice.lower)
								l_upper := choice_constant (l_choice.upper)
								if attached {ET_INTEGER_CONSTANT} l_lower as l_integer_constant then
									l_lower_integer := l_integer_constant
									l_lower_character := Void
								elseif attached {ET_CHARACTER_CONSTANT} l_lower as l_character_constant then
									l_lower_character := l_character_constant
									l_lower_integer := Void
								else
									l_lower_integer := Void
									l_lower_character := Void
								end
								if attached {ET_INTEGER_CONSTANT} l_upper as l_integer_constant then
									l_upper_integer := l_integer_constant
									l_upper_character := Void
								elseif attached {ET_CHARACTER_CONSTANT} l_upper as l_character_constant then
									l_upper_character := l_character_constant
									l_upper_integer := Void
								else
									l_upper_integer := Void
									l_upper_character := Void
								end
								if l_lower_integer /= Void and l_upper_integer /= Void then
									from
-- TODO: check type of inspect value.
										k := l_lower_integer.to_integer_32
										nb3 := l_upper_integer.to_integer_32
									until
										k > nb3
									loop
										l_has_case := True
										print_indentation
										current_file.put_string (c_case)
										current_file.put_character (' ')
										print_integer_value (k.abs.to_natural_64, k < 0, l_value_type, True)
										current_file.put_character (':')
										current_file.put_new_line
										k := k + 1
									end
								elseif l_lower_character /= Void and l_upper_character /= Void then
									from
										l_i_nat32 := l_lower_character.value.natural_32_code
										l_nb_nat32 := l_upper_character.value.natural_32_code
										l_stop := l_i_nat32 > l_nb_nat32
									until
										l_stop
									loop
										l_has_case := True
										print_indentation
										current_file.put_string (c_case)
										current_file.put_character (' ')
										if current_system.character_32_type.same_named_type (l_value_type.base_type, current_type.base_type, current_type.base_type) then
											current_file.put_string (c_ge_nat32)
											current_file.put_character ('(')
											current_file.put_natural_32 (l_i_nat32)
											current_file.put_character (')')
										else
												-- Type cast needed when compiling with mingw when the character
												-- is represented by '\xyz' where x is not 0.
											print_type_cast (l_value_type, current_file)
											print_escaped_character_8 (l_i_nat32.to_character_8)
										end
										current_file.put_character (':')
										current_file.put_new_line
										if l_i_nat32 = l_nb_nat32 then
											l_stop := True
										else
											l_i_nat32 := l_i_nat32 + 1
										end
									end
								else
-- TODO
error_handler.report_warning_message ("ET_C_GENERATOR.print_inspect_expression - range")
								end
							else
								l_has_case := True
								print_indentation
								current_file.put_string (c_case)
								current_file.put_character (' ')
								l_lower := choice_constant (l_choice.lower)
								if attached {ET_INTEGER_CONSTANT} l_lower as l_integer_constant then
									print_integer_value (l_integer_constant.value, l_integer_constant.is_negative, l_value_type, True)
								elseif attached {ET_CHARACTER_CONSTANT} l_lower as l_character_constant then
									if current_system.character_32_type.same_named_type (l_value_type.base_type, current_type.base_type, current_type.base_type) then
										current_file.put_string (c_ge_nat32)
										current_file.put_character ('(')
										current_file.put_natural_32 (l_character_constant.value.natural_32_code)
										current_file.put_character (')')
									else
											-- Type cast needed when compiling with mingw when the character
											-- is represented by '\xyz' where x is not 0.
										print_type_cast (l_value_type, current_file)
										print_escaped_character_8 (l_character_constant.value.to_character_8)
									end
								else
									if l_value_type /= dynamic_type_set (l_choice.lower).static_type.primary_type then
										print_type_cast (l_value_type, current_file)
									end
									print_expression (l_choice.lower)
								end
								current_file.put_character (':')
								current_file.put_new_line
							end
							j := j + 1
						end
						if l_has_case then
							indent
							l_expression := l_when_part.then_expression
							l_expression_dynamic_type_set := dynamic_type_set (l_expression)
							print_assignment_operand (l_expression, l_expression_dynamic_type_set, l_temp, l_dynamic_type)
							fill_call_operands (1)
							if call_operands.first /= l_temp then
								print_indentation
								print_temp_name (l_temp, current_file)
								current_file.put_character (' ')
								current_file.put_character ('=')
								current_file.put_character (' ')
								print_attachment_expression (call_operands.first, l_expression_dynamic_type_set, l_dynamic_type)
								current_file.put_character (';')
								current_file.put_new_line
							end
							call_operands.wipe_out
							print_indentation
							current_file.put_string (c_break)
							current_file.put_character (';')
							current_file.put_new_line
							dedent
						end
					end
					i := i + 1
				end
			end
			print_indentation
			current_file.put_string (c_default)
			current_file.put_character (':')
			current_file.put_new_line
			if attached a_expression.else_part as l_else_part then
				indent
				l_expression := l_else_part.expression
				l_expression_dynamic_type_set := dynamic_type_set (l_expression)
				print_assignment_operand (l_expression, l_expression_dynamic_type_set, l_temp, l_dynamic_type)
				fill_call_operands (1)
				if call_operands.first /= l_temp then
					print_indentation
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_attachment_expression (call_operands.first, l_expression_dynamic_type_set, l_dynamic_type)
					current_file.put_character (';')
					current_file.put_new_line
				end
				call_operands.wipe_out
				print_indentation
				current_file.put_string (c_break)
				current_file.put_character (';')
				current_file.put_new_line
				dedent
			else
				indent
				print_indentation
				current_file.put_string (c_ge_raise)
				current_file.put_character ('(')
				current_file.put_string (c_ge_ex_when)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_string (c_break)
				current_file.put_character (';')
				current_file.put_new_line
				dedent
			end
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
			if l_temp_index /= 0 then
					-- We had to wait until this stage to set the index of `l_temp'
					-- because it could have still been used in `call_operands'.
				l_temp.set_index (l_temp_index)
			end
			mark_temp_variable_unfrozen (l_temp)
		end

	print_iteration_cursor (a_iteration_cursor: ET_ITERATION_CURSOR)
			-- Print `a_iteration_cursor'.
		require
			a_iteration_cursor_not_void: a_iteration_cursor /= Void
		local
			l_name: ET_IDENTIFIER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_static_type: ET_DYNAMIC_TYPE
		do
			l_name := a_iteration_cursor.item_name
			if in_operand then
				operand_stack.force (a_iteration_cursor)
			elseif attached call_target_type as l_call_target_type then
				check in_call_target: in_call_target end
				l_dynamic_type_set := dynamic_type_set (a_iteration_cursor)
				l_static_type := l_dynamic_type_set.static_type
				if l_static_type.is_expanded then
						-- Pass the address of the expanded object.
					current_file.put_character ('&')
					print_iteration_cursor_name (l_name, current_file)
				elseif l_call_target_type.is_expanded then
						-- We need to unbox the object and then pass its address.
					print_boxed_attribute_pointer_access (l_name, l_call_target_type, call_target_check_void)
				else
					print_iteration_cursor_name (l_name, current_file)
				end
			else
				print_iteration_cursor_name (l_name, current_file)
			end
		end

	print_iteration_item (a_name: ET_IDENTIFIER)
			-- Print iteration item `a_name'.
		require
			a_name_not_void: a_name /= Void
			a_name_iteration_item: a_name.is_iteration_item
		local
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_static_type: ET_DYNAMIC_TYPE
			l_current_closure: ET_CLOSURE
			l_iteration_component: ET_ITERATION_COMPONENT
			l_seed: INTEGER
			l_done: BOOLEAN
		do
			if attached {ET_INLINE_AGENT} current_agent as l_inline_agent then
				l_current_closure := l_inline_agent
			else
				l_current_closure := current_feature.static_feature
			end
			if attached l_current_closure.iteration_components as l_iteration_components then
				l_seed := a_name.seed
				if l_seed >= 1 and l_seed <= l_iteration_components.count then
					l_iteration_component := l_iteration_components.iteration_component (l_seed)
					if a_name /= l_iteration_component.unfolded_cursor_name and then not l_iteration_component.has_cursor_name then
							-- We are not in the case of 'across ... as ...' with 'obsolete_iteration_mode' set to True.
							-- Print the unfolded form of iteration item: 'unfolded_cursor_name.item'
						print_qualified_call_expression (l_iteration_component.cursor_item_expression)
						l_done := True
					end
				end
			end
			if l_done then
				-- Already printed.
			elseif in_operand then
				operand_stack.force (a_name)
			elseif attached call_target_type as l_call_target_type then
				check in_call_target: in_call_target end
				l_dynamic_type_set := dynamic_type_set (a_name)
				l_static_type := l_dynamic_type_set.static_type
				if l_static_type.is_expanded then
						-- Pass the address of the expanded object.
					current_file.put_character ('&')
					print_iteration_cursor_name (a_name, current_file)
				elseif l_call_target_type.is_expanded then
						-- We need to unbox the object and then pass its address.
					print_boxed_attribute_pointer_access (a_name, l_call_target_type, call_target_check_void)
				else
					print_iteration_cursor_name (a_name, current_file)
				end
			else
				print_iteration_cursor_name (a_name, current_file)
			end
		end

	print_iteration_expression (an_expression: ET_ITERATION_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_temp: ET_IDENTIFIER
			l_temp_index: INTEGER
			l_expression: ET_EXPRESSION
			l_cursor_name: ET_IDENTIFIER
			l_cursor_type: ET_DYNAMIC_TYPE
			l_cursor_type_set: ET_DYNAMIC_TYPE_SET
			l_boolean_type: ET_DYNAMIC_PRIMARY_TYPE
			l_expression_dynamic_type_set: ET_DYNAMIC_TYPE_SET
		do
			assignment_target := Void
				-- Declaration of the iteration cursor.
			l_cursor_name := an_expression.unfolded_cursor_name
			l_cursor_type_set := dynamic_type_set (l_cursor_name)
			l_cursor_type := l_cursor_type_set.static_type
			current_function_header_buffer.put_character ('%T')
			print_type_declaration (l_cursor_type.primary_type, current_function_header_buffer)
			current_function_header_buffer.put_character (' ')
			print_iteration_cursor_name (l_cursor_name, current_function_header_buffer)
			current_function_header_buffer.put_character (';')
			current_function_header_buffer.put_new_line
				-- Call to `new_cursor'.
			print_assignment_operand (an_expression.new_cursor_expression, l_cursor_type_set, l_cursor_name, l_cursor_type)
			fill_call_operands (1)
			if call_operands.first /= l_cursor_name then
				print_indentation
				print_iteration_cursor_name (l_cursor_name, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attachment_expression (call_operands.first, l_cursor_type_set, l_cursor_type)
				current_file.put_character (';')
				current_file.put_new_line
			end
			call_operands.wipe_out
				-- Declaration of temporary result.
			l_boolean_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.boolean_type, current_type.base_type)
			l_temp := new_temp_variable (l_boolean_type)
				-- We will set the index of `l_temp' later because
				-- it could still be used in `call_operands'.
			l_temp_index := an_expression.index
			operand_stack.force (l_temp)
			print_indentation
			print_temp_name (l_temp, current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			if an_expression.is_all then
				current_file.put_string (c_eif_true)
			else
				current_file.put_string (c_eif_false)
			end
			current_file.put_character (';')
			current_file.put_new_line
			print_indentation
			current_file.put_string (c_while)
			current_file.put_character (' ')
			current_file.put_character ('(')
			current_file.put_character ('1')
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_operand (an_expression.cursor_after_expression)
			if attached an_expression.until_conditional as l_until_conditional then
				l_expression := l_until_conditional.expression
				print_operand (l_expression)
				fill_call_operands (2)
			else
				fill_call_operands (1)
			end
			print_indentation
			current_file.put_string (c_if)
			current_file.put_character (' ')
			current_file.put_character ('(')
			current_file.put_character ('(')
			if an_expression.is_all then
				current_file.put_character ('!')
			end
			print_temp_name (l_temp, current_file)
			current_file.put_character (')')
			current_file.put_character ('|')
			current_file.put_character ('|')
			current_file.put_character ('(')
			print_expression (call_operands.first)
			current_file.put_character (')')
			if call_operands.count = 2 then
				current_file.put_character ('|')
				current_file.put_character ('|')
				current_file.put_character ('(')
				print_expression (call_operands.item (2))
				current_file.put_character (')')
			end
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			call_operands.wipe_out
			indent
			print_indentation
			current_file.put_string (c_break)
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
				-- Iteration expression.
			l_expression := an_expression.iteration_expression
			l_expression_dynamic_type_set := dynamic_type_set (l_expression)
			print_assignment_operand (l_expression, l_expression_dynamic_type_set, l_temp, l_boolean_type)
			fill_call_operands (1)
			if call_operands.first /= l_temp then
				print_indentation
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attachment_expression (call_operands.first, l_expression_dynamic_type_set, l_boolean_type)
				current_file.put_character (';')
				current_file.put_new_line
			end
			call_operands.wipe_out
				-- Call to `forth'.
			print_instruction (an_expression.cursor_forth_instruction)
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
			if l_temp_index /= 0 then
					-- We had to wait until this stage to set the index of `l_temp'
					-- because it could have still been used in `call_operands'.
				l_temp.set_index (l_temp_index)
			end
		end

	print_local_variable (a_name: ET_IDENTIFIER)
			-- Print local variable `a_name'.
		require
			a_name_not_void: a_name /= Void
			a_name_local: a_name.is_local
		local
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_static_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if in_operand then
				operand_stack.force (a_name)
			else
				if has_rescue then
					-- Keep track of the fact that this local variable has been
					-- read. Useful to determine the 'volatile' status of the local
					-- variable when current feature has a rescue clause.
					locals_read.force_last (a_name)
				end
				if attached call_target_type as l_call_target_type then
					check in_call_target: in_call_target end
					l_dynamic_type_set := dynamic_type_set (a_name)
					l_static_type := l_dynamic_type_set.static_type.primary_type
					if l_static_type.is_expanded then
							-- Pass the address of the expanded object.
						current_file.put_character ('(')
						if has_rescue then
								-- When there is a rescue clause, the local variable might
								-- have been declared with a 'volatile' qualifier. In that
								-- case we need a type cast here to avoid C compiler warning
								-- (at least with MSVC 12).
							current_file.put_character ('(')
							print_type_declaration (l_static_type, current_file)
							current_file.put_character ('*')
							current_file.put_character (')')
						end
						current_file.put_character ('&')
						print_local_name (a_name, current_file)
						current_file.put_character (')')
					elseif l_call_target_type.is_expanded then
							-- We need to unbox the object and then pass its address.
						print_boxed_attribute_pointer_access (a_name, l_call_target_type, call_target_check_void)
					else
						print_local_name (a_name, current_file)
					end
				else
					print_local_name (a_name, current_file)
				end
			end
		end

	print_manifest_array (an_expression: ET_MANIFEST_ARRAY)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_dynamic_type_set: detachable ET_DYNAMIC_TYPE_SET
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp: detachable ET_IDENTIFIER
			l_temp_index: INTEGER
			i, nb, nb2: INTEGER
			l_assignment_target: like assignment_target
			l_int_promoted: BOOLEAN
			l_double_promoted: BOOLEAN
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_integer_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_static_item_type: detachable ET_DYNAMIC_TYPE
			l_static_item_primary_type: ET_DYNAMIC_PRIMARY_TYPE
			l_dynamic_item_type_set: ET_DYNAMIC_TYPE_SET
			l_item: ET_EXPRESSION
			l_argument_limit: INTEGER
		do
			--l_argument_limit := 253 -- tcc limitation
			l_argument_limit := 1024 -- lcc-win32 limitation
			l_assignment_target := assignment_target
			assignment_target := Void
			l_dynamic_type := dynamic_type_set (an_expression).static_type.primary_type
			if l_dynamic_type.attribute_count < 3 then
					-- Internal error: class "ARRAY" should have at least the
					-- features 'area', 'lower' and 'upper' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_queries := l_dynamic_type.queries
				l_dynamic_type_set := l_queries.first.result_type_set
				if l_dynamic_type_set = Void then
						-- Error in feature 'area', already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
					set_fatal_error
					error_handler.report_giaaa_error
				elseif not attached {ET_DYNAMIC_SPECIAL_TYPE} l_dynamic_type_set.static_type.primary_type as l_special_type then
						-- Internal error: it has already been checked in ET_DYNAMIC_SYSTEM.compile_kernel
						-- that the attribute `area' is of SPECIAL type.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					l_static_item_type := l_special_type.item_type_set.static_type
				end
				l_dynamic_type_set := l_queries.item (3).result_type_set
				if l_dynamic_type_set = Void then
						-- Error in feature 'upper', already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					l_integer_type := l_dynamic_type_set.static_type.primary_type
				end
			end
			if l_static_item_type /= Void and l_integer_type /= Void then
				l_static_item_primary_type := l_static_item_type.primary_type
				nb := an_expression.count
				from i := 1 until i > nb loop
					l_item := an_expression.expression (i)
					l_dynamic_item_type_set := dynamic_type_set (l_item)
					print_attachment_operand (l_item, l_dynamic_item_type_set)
					i := i + 1
				end
				fill_call_operands (nb)
				if
					l_static_item_primary_type = current_dynamic_system.boolean_type or
					l_static_item_primary_type = current_dynamic_system.character_8_type or
					l_static_item_primary_type = current_dynamic_system.integer_8_type or
					l_static_item_primary_type = current_dynamic_system.natural_8_type or
					l_static_item_primary_type = current_dynamic_system.integer_16_type or
					l_static_item_primary_type = current_dynamic_system.natural_16_type
				then
						-- ISO C 99 says that through "..." the types are promoted to
						-- 'int', and that promotion to 'int' leaves the type unchanged
						-- if all values cannot be represented with an 'int' or
						-- 'unsigned int'.
					l_int_promoted := True
				elseif
					l_static_item_primary_type = current_dynamic_system.real_32_type
				then
						-- ISO C 99 says that 'float' is promoted to 'double' when
						-- passed as argument of a function.
					l_double_promoted := True
				end
				manifest_array_types.force_last (l_dynamic_type)
					-- `nb2' is the maximum number of array elements that can be passed to the
					-- function 'GE_ma<type-id>' based on C compiler limitation.
				nb2 := nb.min (l_argument_limit - 2)
				if in_operand then
					if l_assignment_target /= Void and nb <= nb2 then
						operand_stack.force (l_assignment_target)
						print_indentation
						print_writable (l_assignment_target)
					else
						l_temp := new_temp_variable (l_dynamic_type)
							-- We will set the index of `l_temp' later because
							-- it could still be used in `call_operands'.
						l_temp_index := an_expression.index
						operand_stack.force (l_temp)
						print_indentation
						print_temp_name (l_temp, current_file)
					end
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
				end
				current_file.put_string (c_ge_ma)
				current_file.put_integer (l_dynamic_type.id)
				current_file.put_character ('(')
				print_type_cast (l_integer_type, current_file)
				current_file.put_integer (nb)
				current_file.put_character (',')
				print_type_cast (l_integer_type, current_file)
				current_file.put_integer (nb2)
				from i := 1 until i > nb loop
					current_file.put_character (',')
						-- Print one item per line for better readability,
						-- avoiding too long lines for big arrays.
					current_file.put_new_line
					l_item := call_operands.item (i)
					l_dynamic_item_type_set := dynamic_type_set (l_item)
					if l_int_promoted then
						current_file.put_character ('(')
						current_file.put_string (c_int)
						current_file.put_character (')')
						current_file.put_character ('(')
						print_attachment_expression (l_item, l_dynamic_item_type_set, l_static_item_type)
						current_file.put_character (')')
					elseif l_double_promoted then
						current_file.put_character ('(')
						current_file.put_string (c_double)
						current_file.put_character (')')
						current_file.put_character ('(')
						print_attachment_expression (l_item, l_dynamic_item_type_set, l_static_item_type)
						current_file.put_character (')')
					else
						print_attachment_expression (l_item, l_dynamic_item_type_set, l_static_item_type)
					end
					i := i + 1
					if i > nb2 and i <= nb then
							-- `nb2' is the maximum number of array elements that can be passed to the
							-- function 'GE_ma<type-id>' based on C compiler limitation.
						check l_temp_not_void: l_temp /= Void then
							nb2 := (nb - i + 1).min (l_argument_limit - 3)
							current_file.put_character (')')
							current_file.put_character (';')
							current_file.put_new_line
							print_indentation
							current_file.put_string (c_ge_bma)
							current_file.put_integer (l_dynamic_type.id)
							current_file.put_character ('(')
							print_expression (l_temp)
							current_file.put_character (',')
							print_type_cast (l_integer_type, current_file)
							current_file.put_integer (i - 1)
							current_file.put_character (',')
							print_type_cast (l_integer_type, current_file)
							current_file.put_integer (nb2)
							nb2 := nb2 + i - 1
							big_manifest_array_types.force_last (l_dynamic_type)
						end
					end
				end
				current_file.put_character (')')
				if in_operand then
					current_file.put_character (';')
					current_file.put_new_line
				end
				call_operands.wipe_out
				if l_temp /= Void and then l_temp_index /= 0 then
						-- We had to wait until this stage to set the index of `l_temp'
						-- because it could have still been used in `call_operands'.
					l_temp.set_index (l_temp_index)
				end
			end
		end

	print_manifest_string (a_string: ET_MANIFEST_STRING)
			-- Print `a_string'.
		require
			a_string_not_void: a_string /= Void
		local
			l_string: STRING
			l_atomic: BOOLEAN
			l_temp: ET_IDENTIFIER
			l_assignment_target: like assignment_target
			l_string_type: ET_DYNAMIC_PRIMARY_TYPE
		do
				-- We consider manifest strings as being atomic expressions.
			l_atomic := True
			l_assignment_target := assignment_target
			assignment_target := Void
			if in_operand then
				if l_atomic then
					operand_stack.force (a_string)
				else
					if l_assignment_target /= Void then
						operand_stack.force (l_assignment_target)
						print_indentation
						print_writable (l_assignment_target)
					else
						l_string_type := dynamic_type_set (a_string).static_type.primary_type
						l_temp := new_temp_variable (l_string_type)
						l_temp.set_index (a_string.index)
						operand_stack.force (l_temp)
						print_indentation
						print_temp_name (l_temp, current_file)
					end
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
				end
			end
			if not l_atomic or not in_operand then
				l_string := a_string.value
				l_string_type := dynamic_type_set (a_string).static_type.primary_type
				if current_system.string_32_type.same_named_type_with_type_marks (l_string_type.base_type, tokens.implicit_detachable_type_mark, current_system.any_type, tokens.implicit_detachable_type_mark, current_system.any_type) then
					current_file.put_string (c_ge_ms32_from_utf32le)
					current_file.put_character ('(')
					print_utf8_as_escaped_string_32 (l_string)
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_integer ({UC_UTF8_ROUTINES}.unicode_character_count (l_string))
					current_file.put_character (')')
				elseif current_system.immutable_string_32_type.same_named_type_with_type_marks (l_string_type.base_type, tokens.implicit_detachable_type_mark, current_system.any_type, tokens.implicit_detachable_type_mark, current_system.any_type) then
					current_file.put_string (c_ge_ims32_from_utf32le)
					current_file.put_character ('(')
					print_utf8_as_escaped_string_32 (l_string)
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_integer ({UC_UTF8_ROUTINES}.unicode_character_count (l_string))
					current_file.put_character (')')
				elseif current_system.immutable_string_8_type.same_named_type_with_type_marks (l_string_type.base_type, tokens.implicit_detachable_type_mark, current_system.any_type, tokens.implicit_detachable_type_mark, current_system.any_type) then
					current_file.put_string (c_ge_ims8)
					current_file.put_character ('(')
					print_utf8_as_escaped_string_8 (l_string)
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_integer ({UC_UTF8_ROUTINES}.unicode_character_count (l_string))
					current_file.put_character (')')
				else
					current_file.put_string (c_ge_ms8)
					current_file.put_character ('(')
					print_utf8_as_escaped_string_8 (l_string)
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_integer ({UC_UTF8_ROUTINES}.unicode_character_count (l_string))
					current_file.put_character (')')
				end
			end
			if not l_atomic and in_operand then
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_manifest_tuple (an_expression: ET_MANIFEST_TUPLE)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_temp: detachable ET_IDENTIFIER
			l_temp_index: INTEGER
			i, nb: INTEGER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_assignment_target: like assignment_target
			l_item_type_set: ET_DYNAMIC_TYPE_SET
			l_item: ET_EXPRESSION
		do
			l_assignment_target := assignment_target
			assignment_target := Void
			l_dynamic_type_set := dynamic_type_set (an_expression)
			if not attached {ET_DYNAMIC_TUPLE_TYPE} l_dynamic_type_set.static_type.primary_type as l_tuple_type then
					-- Internal error: the dynamic type of `an_expression'
					-- should be a Tuple_type.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				nb := an_expression.count
				manifest_tuple_types.force_last (l_tuple_type)
				if in_operand then
					from i := 1 until i > nb loop
						l_item := an_expression.expression (i)
						l_item_type_set := dynamic_type_set (l_item)
						print_attachment_operand (l_item, l_item_type_set)
						i := i + 1
					end
					fill_call_operands (nb)
					if l_assignment_target /= Void then
						operand_stack.force (l_assignment_target)
						print_indentation
						print_writable (l_assignment_target)
					else
						l_temp := new_temp_variable (l_tuple_type)
							-- We will set the index of `l_temp' later because
							-- it could still be used in `call_operands'.
						l_temp_index := an_expression.index
						operand_stack.force (l_temp)
						print_indentation
						print_temp_name (l_temp, current_file)
					end
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
				end
				current_file.put_string (c_ge_mt)
				current_file.put_integer (l_tuple_type.id)
				current_file.put_character ('(')
				from i := 1 until i > nb loop
					if i /= 1 then
						current_file.put_character (',')
					end
					if in_operand then
						l_item := call_operands.item (i)
					else
						l_item := an_expression.expression (i)
					end
					l_item_type_set := dynamic_type_set (l_item)
					print_attachment_expression (l_item, l_item_type_set, l_item_type_set.static_type)
					i := i + 1
				end
				current_file.put_character (')')
				if in_operand then
					current_file.put_character (';')
					current_file.put_new_line
					call_operands.wipe_out
					if l_temp /= Void and then l_temp_index /= 0 then
							-- We had to wait until this stage to set the index of `l_temp'
							-- because it could have still been used in `call_operands'.
						l_temp.set_index (l_temp_index)
					end
				end
			end
		end

	print_manifest_type (an_expression: ET_MANIFEST_TYPE)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_type: ET_DYNAMIC_TYPE
			l_meta_type: detachable ET_DYNAMIC_PRIMARY_TYPE
		do
			if in_operand then
				operand_stack.force (an_expression)
			else
				l_type := current_dynamic_system.dynamic_type (an_expression.type, current_type.base_type)
				l_meta_type := l_type.meta_type
				if l_meta_type = Void then
						-- Internal error: the meta type of this type should have been
						-- computed. It is nothing else than the type of this manifest
						-- type expression.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					current_file.put_character ('(')
					current_file.put_character ('(')
					print_type_declaration (l_meta_type, current_file)
					current_file.put_character (')')
					current_file.put_character ('&')
					current_file.put_character ('(')
					current_file.put_string (c_ge_types)
					current_file.put_character ('[')
					current_file.put_integer (l_type.primary_type.id)
					current_file.put_character (']')
					current_file.put_character ('[')
					if l_type = l_type.primary_type then
						current_file.put_integer (0)
					else
						current_file.put_integer (1)
					end
					current_file.put_character (']')
					current_file.put_character (')')
					current_file.put_character (')')
				end
			end
		end

	print_non_void_expression (an_expression: ET_EXPRESSION; a_check_void: BOOLEAN)
			-- Print `an_expression' which needs to be non-void.
			-- The static type (i.e. declared type) of `an_expression' is assumed to be of reference type.
			-- Useful for example when we want to get access to an attribute of this object.
			-- `a_check_void' means that we do need to check whether it is actually Void or not.
		require
			an_expression_not_void: an_expression /= Void
		local
			old_in_operand: BOOLEAN
			old_target_type: like call_target_type
			l_old_in_procedure_call_target: BOOLEAN
		do
			old_in_operand := in_operand
			old_target_type := call_target_type
			l_old_in_procedure_call_target := in_procedure_call_target
			in_operand := False
			in_procedure_call_target := False
			call_target_type := Void
			print_check_void_expression (an_expression, current_dynamic_system.none_type, a_check_void)
			call_target_type := old_target_type
			in_procedure_call_target := l_old_in_procedure_call_target
			in_operand := old_in_operand
		end

	print_object_equality_expression (a_expression: ET_OBJECT_EQUALITY_EXPRESSION)
			-- Print `a_expression'.
		require
			a_expression_not_void: a_expression /= Void
		local
			l_left_operand: ET_EXPRESSION
			l_right_operand: ET_EXPRESSION
			l_left_type_set: ET_DYNAMIC_TYPE_SET
			l_right_type_set: ET_DYNAMIC_TYPE_SET
			l_equality_types: ET_DYNAMIC_OBJECT_EQUALITY_TYPES
			l_assignment_target: like assignment_target
		do
			l_assignment_target := assignment_target
			assignment_target := Void
			l_left_operand := a_expression.left
			l_right_operand := a_expression.right
			l_left_type_set := dynamic_type_set (l_left_operand)
			l_right_type_set := dynamic_type_set (l_right_operand)
				-- Process operands.
			print_operand (l_left_operand)
				-- Call `print_attachment_operand' and not `print_operand'
				-- because the right operand might be the argument of a call
				-- to 'is_equal' and it will require some cloning if its
				-- type is expanded.
			print_attachment_operand (l_right_operand, l_right_type_set)
			fill_call_operands (2)
			assignment_target := l_assignment_target
			l_equality_types := new_object_equality_types (l_left_type_set, l_right_type_set)
			print_equality_call (a_expression, a_expression.operator.is_not_tilde, l_equality_types)
			if current_equalities.is_empty or else current_equalities.last /= l_equality_types then
				free_object_equality_types (l_equality_types)
			end
			call_operands.wipe_out
		end

	print_object_test (an_expression: ET_OBJECT_TEST)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_assignment_target: like assignment_target
			l_temp: detachable ET_IDENTIFIER
			l_temp_index: INTEGER
			i, nb: INTEGER
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_source: ET_EXPRESSION
			l_source_type_set: ET_DYNAMIC_TYPE_SET
			l_source_type: ET_DYNAMIC_TYPE
			l_target_type_set: ET_DYNAMIC_TYPE_SET
			l_target_type: ET_DYNAMIC_TYPE
			l_target_primary_type: ET_DYNAMIC_PRIMARY_TYPE
			nb_conforming_types: INTEGER
			nb_non_conforming_types: INTEGER
			l_can_be_void: BOOLEAN
			l_name: detachable ET_IDENTIFIER
			l_type: detachable ET_TYPE
		do
			l_assignment_target := assignment_target
			assignment_target := Void
			l_source := an_expression.expression
			l_source_type_set := dynamic_type_set (l_source)
			l_source_type := l_source_type_set.static_type
			l_name := an_expression.name
			l_type := an_expression.type
			if l_name /= Void then
				l_target_type_set := dynamic_type_set (l_name)
				l_target_type := l_target_type_set.static_type
			elseif l_type /= Void then
				l_target_type := current_dynamic_system.dynamic_type (l_type, current_type.base_type)
			else
				l_target_type := l_source_type
			end
			l_target_primary_type := l_target_type.primary_type
			print_attachment_operand (l_source, l_source_type_set)
			fill_call_operands (1)
			if in_operand then
				if l_assignment_target /= Void then
					operand_stack.force (l_assignment_target)
					print_indentation
					print_writable (l_assignment_target)
				else
					l_temp := new_temp_variable (dynamic_type_set (an_expression).static_type.primary_type)
						-- We will set the index of `l_temp' later because
						-- it could still be used in `call_operands'.
					l_temp_index := an_expression.index
					operand_stack.force (l_temp)
					print_indentation
					print_temp_name (l_temp, current_file)
				end
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
			end
				-- Declaration of the object-test local.
			if l_name /= Void then
				current_function_header_buffer.put_character ('%T')
				print_type_declaration (l_target_primary_type, current_function_header_buffer)
				current_function_header_buffer.put_character (' ')
				print_object_test_local_name (l_name, current_function_header_buffer)
				current_function_header_buffer.put_character (' ')
				current_function_header_buffer.put_character ('=')
				current_function_header_buffer.put_character (' ')
				print_default_entity_value (l_target_primary_type, current_function_header_buffer)
				current_function_header_buffer.put_character (';')
				current_function_header_buffer.put_new_line
			end
				-- Check type conformance.
			nb := l_source_type_set.count
			if l_type = Void then
				nb_conforming_types := nb
				nb_non_conforming_types := 0
			else
				from i := 1 until i > nb loop
					l_dynamic_type := l_source_type_set.dynamic_type (i)
					if l_dynamic_type.conforms_to_primary_type (l_target_primary_type) then
						nb_conforming_types := nb_conforming_types + 1
					else
						nb_non_conforming_types := nb_non_conforming_types + 1
					end
					i := i + 1
				end
			end
			if nb_non_conforming_types = 0 then
					-- All types in the source type set conform to the target type.
				l_can_be_void := l_source_type_set.can_be_void
				if l_can_be_void then
					current_file.put_character ('(')
					print_expression (call_operands.first)
					current_file.put_character ('?')
					current_file.put_character ('(')
				end
				if l_name /= Void then
					current_file.put_character ('(')
					print_object_test_local_name (l_name, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_attachment_expression (call_operands.first, l_source_type_set, l_target_type)
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_string (c_eif_true)
					current_file.put_character (')')
				else
					current_file.put_string (c_eif_true)
				end
				if l_can_be_void then
					current_file.put_character (')')
					current_file.put_character (':')
					current_file.put_string (c_eif_false)
					current_file.put_character (')')
				end
			elseif nb_conforming_types = 0 then
					-- All types in the source type set conform to the target type.
				current_file.put_string (c_eif_false)
			else
				current_object_tests.force_last (an_expression)
				print_object_test_function_name (current_object_tests.count, current_feature, current_type, current_file)
				current_file.put_character ('(')
				print_expression (call_operands.first)
				if l_name /= Void then
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_character ('&')
					print_object_test_local_name (l_name, current_file)
				end
				current_file.put_character (')')
			end
			call_operands.wipe_out
			if in_operand then
				current_file.put_character (';')
				current_file.put_new_line
				if l_temp /= Void and then l_temp_index /= 0 then
						-- We had to wait until this stage to set the index of `l_temp'
						-- because it could have still been used in `call_operands'.
					l_temp.set_index (l_temp_index)
				end
			end
		end

	print_object_test_local (a_name: ET_IDENTIFIER)
			-- Print object-test local `a_name'.
		require
			a_name_not_void: a_name /= Void
			a_name_object_test_local: a_name.is_object_test_local
		local
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_static_type: ET_DYNAMIC_TYPE
		do
			if in_operand then
				operand_stack.force (a_name)
			elseif attached call_target_type as l_call_target_type then
				check in_call_target: in_call_target end
				l_dynamic_type_set := dynamic_type_set (a_name)
				l_static_type := l_dynamic_type_set.static_type
				if l_static_type.is_expanded then
						-- Pass the address of the expanded object.
					current_file.put_character ('&')
					print_object_test_local_name (a_name, current_file)
				elseif l_call_target_type.is_expanded then
						-- We need to unbox the object and then pass its address.
					print_boxed_attribute_pointer_access (a_name, l_call_target_type, call_target_check_void)
				else
					print_object_test_local_name (a_name, current_file)
				end
			else
				print_object_test_local_name (a_name, current_file)
			end
		end

	print_octal_integer_constant (a_constant: ET_OCTAL_INTEGER_CONSTANT)
			-- Print `a_constant'.
		require
			a_constant_not_void: a_constant /= Void
		do
			print_integer_constant (a_constant)
		end

	print_old_expression (an_expression: ET_OLD_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
-- TODO.
error_handler.report_warning_message ("ET_C_GENERATOR.print_old_expression")
			an_expression.expression.process (Current)
		end

	print_once_manifest_string (an_expression: ET_ONCE_MANIFEST_STRING)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if in_operand then
				operand_stack.force (an_expression)
			else
				l_dynamic_type := dynamic_type_set (an_expression).static_type.primary_type
				inline_constants.force_last (l_dynamic_type, an_expression)
				print_inline_constant_name (an_expression, current_file)
			end
		end

	print_operand (an_operand: ET_EXPRESSION)
			-- Print `an_operand'.
		require
			an_operand_not_void: an_operand /= Void
		local
			old_in_operand: BOOLEAN
			old_target_type: like call_target_type
			l_old_in_procedure_call_target: BOOLEAN
		do
			old_in_operand := in_operand
			old_target_type := call_target_type
			l_old_in_procedure_call_target := in_procedure_call_target
			in_operand := True
			in_procedure_call_target := False
			call_target_type := Void
			an_operand.process (Current)
			call_target_type := old_target_type
			in_procedure_call_target := l_old_in_procedure_call_target
			in_operand := old_in_operand
		end

	print_parenthesized_expression (an_expression: ET_PARENTHESIZED_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
			an_expression.expression.process (Current)
		end

	print_precursor_expression (an_expression: ET_PRECURSOR_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_precursor_keyword: ET_PRECURSOR_KEYWORD
			l_query: detachable ET_QUERY
			l_parent_type, l_ancestor: detachable ET_BASE_TYPE
			l_class: ET_CLASS
			l_current_class: ET_CLASS
			l_class_impl: ET_CLASS
			l_dynamic_precursor: ET_DYNAMIC_PRECURSOR
			l_parent_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
			l_temp: detachable ET_IDENTIFIER
			l_temp_index: INTEGER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_assignment_target: like assignment_target
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type_set: ET_DYNAMIC_TYPE_SET
			l_actual: ET_EXPRESSION
			had_error: BOOLEAN
		do
			l_assignment_target := assignment_target
			assignment_target := Void
			if attached an_expression.arguments as l_actuals then
				nb := l_actuals.count
				from i := 1 until i > nb loop
					l_actual := l_actuals.actual_argument (i)
					l_actual_type_set := dynamic_type_set (l_actual)
					print_attachment_operand (l_actual, l_actual_type_set)
					i := i + 1
				end
			end
			fill_call_operands (nb)
			l_parent_type := an_expression.parent_type
			if l_parent_type = Void then
					-- Internal error: the Precursor construct should already
					-- have been resolved when flattening the features of the
					-- implementation class of current feature.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				had_error := has_fatal_error
				has_fatal_error := False
				if l_parent_type.is_generic then
					l_current_class := current_type.base_class
					l_class_impl := current_feature.static_feature.implementation_class
					if l_current_class /= l_class_impl then
							-- Resolve generic parameters in the context of `current_type'.
						l_current_class.process (system_processor.ancestor_builder)
						if not l_current_class.ancestors_built or else l_current_class.has_ancestors_error then
								-- 'ancestor_builder' should already have been executed
								-- on `l_current_class' at this stage, and any error
								-- should have already been reported.
							set_fatal_error
							error_handler.report_giaaa_error
						else
							l_ancestor := l_current_class.ancestor (l_parent_type)
							if l_ancestor = Void then
									-- Internal error: `l_parent_type' is an ancestor
									-- of `l_class_impl', and hence of `l_current_class'.
								set_fatal_error
								error_handler.report_giaaa_error
							else
								l_parent_type := l_ancestor
							end
						end
					end
				end
				if not has_fatal_error then
					has_fatal_error := had_error
					l_precursor_keyword := an_expression.precursor_keyword
					l_class := l_parent_type.base_class
					l_query := l_class.seeded_query (l_precursor_keyword.seed)
					if l_query = Void then
							-- Internal error: the Precursor construct should
							-- already have been resolved when flattening the
							-- features of `l_class_impl'.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						if in_operand then
							if l_assignment_target /= Void then
								operand_stack.force (l_assignment_target)
								print_indentation
								print_writable (l_assignment_target)
							else
								l_dynamic_type_set := dynamic_type_set (an_expression)
								l_dynamic_type := l_dynamic_type_set.static_type.primary_type
								l_temp := new_temp_variable (l_dynamic_type)
									-- We will set the index of `l_temp' later because
									-- it could still be used in `call_operands'.
								l_temp_index := an_expression.index
								operand_stack.force (l_temp)
								print_indentation
								print_temp_name (l_temp, current_file)
							end
							current_file.put_character (' ')
							current_file.put_character ('=')
							current_file.put_character (' ')
							current_file.put_character ('(')
						end
						l_parent_dynamic_type := current_dynamic_system.dynamic_primary_type (l_parent_type, current_type.base_type)
						l_dynamic_precursor := current_feature.dynamic_precursor (l_query, l_parent_dynamic_type, current_dynamic_system)
						if in_static_feature then
							register_called_static_feature (l_dynamic_precursor)
							print_static_routine_name (l_dynamic_precursor, current_type, current_file)
							current_file.put_character ('(')
							current_file.put_string (c_ac)
						else
							register_called_feature (l_dynamic_precursor)
							print_routine_name (l_dynamic_precursor, current_type, current_file)
							current_file.put_character ('(')
							current_file.put_string (c_ac)
							current_file.put_character (',')
							current_file.put_character (' ')
							print_current_name (current_file)
						end
						from i := 1 until i > nb loop
							current_file.put_character (',')
							current_file.put_character (' ')
							l_actual_type_set := dynamic_type_set (call_operands.item (i))
							l_formal_type_set := argument_type_set_in_feature (i, l_dynamic_precursor)
							print_attachment_expression (call_operands.item (i), l_actual_type_set, l_formal_type_set.static_type)
							i := i + 1
						end
						current_file.put_character (')')
						if in_operand then
							current_file.put_character (')')
							current_file.put_character (';')
							current_file.put_new_line
						end
					end
				end
			end
			call_operands.wipe_out
			if l_temp /= Void and then l_temp_index /= 0 then
					-- We had to wait until this stage to set the index of `l_temp'
					-- because it could have still been used in `call_operands'.
				l_temp.set_index (l_temp_index)
			end
		end

	print_prefix_expression (an_expression: ET_PREFIX_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
			print_qualified_call_expression (an_expression)
		end

	print_procedure_target_expression (an_expression: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print `an_expression' when appearing as the target of a procedure call.
			-- `a_target_type' is one of the possible dynamic types of `an_expression'.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			an_expression_not_void: an_expression /= Void
			a_target_type_not_void: a_target_type /= Void
		local
			old_in_operand: BOOLEAN
			old_target_type: like call_target_type
			old_target_check_void: BOOLEAN
			l_old_in_procedure_call_target: BOOLEAN
		do
			old_in_operand := in_operand
			old_target_type := call_target_type
			old_target_check_void := call_target_check_void
			l_old_in_procedure_call_target := in_procedure_call_target
			in_operand := False
			call_target_type := a_target_type
			in_procedure_call_target := True
			call_target_check_void := a_check_void_target
			print_check_void_expression (an_expression, a_target_type, a_check_void_target)
			call_target_check_void := old_target_check_void
			in_procedure_call_target := l_old_in_procedure_call_target
			call_target_type := old_target_type
			in_operand := old_in_operand
		end

	print_procedure_target_operand (an_operand: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print `an_operand' when appearing as the target of a procedure call.
			-- `a_target_type' is one of the possible dynamic types of `an_operand'.
		require
			an_operand_not_void: an_operand /= Void
			a_target_type_not_void: a_target_type /= Void
		local
			old_in_operand: BOOLEAN
			old_target_type: like call_target_type
			l_old_in_procedure_call_target: BOOLEAN
		do
			old_in_operand := in_operand
			old_target_type := call_target_type
			l_old_in_procedure_call_target := in_procedure_call_target
			in_operand := True
			call_target_type := a_target_type
			in_procedure_call_target := True
			an_operand.process (Current)
			in_procedure_call_target := l_old_in_procedure_call_target
			call_target_type := old_target_type
			in_operand := old_in_operand
		end

	print_qualified_call_expression (a_call: ET_QUALIFIED_FEATURE_CALL_EXPRESSION)
			-- Print qualified call expression.
		require
			a_call_not_void: a_call /= Void
		local
			l_name: ET_CALL_NAME
			l_target: ET_EXPRESSION
			l_target_type_set: ET_DYNAMIC_TYPE_SET
			l_actuals: detachable ET_ACTUAL_ARGUMENTS
			l_query: detachable ET_QUERY
			l_seed: INTEGER
			i, nb: INTEGER
			l_target_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_target_static_type: ET_DYNAMIC_PRIMARY_TYPE
			nb2: INTEGER
			l_call_type_set: ET_DYNAMIC_TYPE_SET
			l_call_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp: detachable ET_IDENTIFIER
			l_temp_index: INTEGER
			l_temp_target: detachable ET_IDENTIFIER
			l_assignment_target: like assignment_target
			l_semistrict_target: ET_WRITABLE
			l_implies: BOOLEAN
			l_or: BOOLEAN
			l_printed: BOOLEAN
			l_manifest_tuple_operand: detachable ET_MANIFEST_TUPLE
			l_target_operand: ET_EXPRESSION
			l_force_result_boxing: BOOLEAN
			l_dynamic_call: detachable ET_DYNAMIC_QUALIFIED_QUERY_CALL
			l_actual: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
		do
			l_assignment_target := assignment_target
			assignment_target := Void
			l_target := a_call.target
			l_name := a_call.name
			l_actuals := a_call.arguments
			l_target_type_set := dynamic_type_set (l_target)
			l_target_static_type := l_target_type_set.static_type.primary_type
				-- Note that `l_dynamic_call' may be Void at this stage because in the
				-- implementation of the built-in feature 'ANY.standard_copy' we create
				-- non-polymorphic calls which have no associated dynamic calls.
			l_dynamic_call := current_feature.query_call (a_call)
			l_call_type_set := dynamic_type_set (a_call)
			l_call_type := l_call_type_set.static_type.primary_type
			if
				in_call_target and then
				l_call_type.is_expanded and then
				(not l_call_type.is_basic or else in_procedure_call_target) and then
				(l_dynamic_call = Void or else l_dynamic_call.has_field_access (current_dynamic_system))
			then
					-- We want the result to be boxed even though the result static type is expanded.
					-- This is needed when the call is an attribute, tuple label, "SPECIAL" item or
					-- a once function, the call result type is expanded, and it is the target of
					-- another call. In that case we force the result type to be of reference type in
					-- order to force the boxing of the result so that the other call is applied on
					-- the result object itself and not on a clone of this object.
					-- Note that for optimization we don't force boxing when the type is a basic
					-- type unless the result object is the target of a procedure call (e.g.
					-- 'copy', 'standard_copy' or 'set_item' which will overwrite the value of
					-- the target).
				l_call_type := current_dynamic_system.any_type
				l_force_result_boxing := True
			end
			if
				l_target_static_type = current_dynamic_system.boolean_type and in_operand and then
				(l_name.is_infix_and_then or l_name.is_infix_or_else or l_name.is_infix_implies or
				l_name.is_infix_and or l_name.is_infix_or or
				l_name.is_infix_and_then_symbol or l_name.is_infix_or_else_symbol or
				l_name.is_infix_implies_symbol or l_name.is_infix_and_symbol or l_name.is_infix_or_symbol)
			then
				print_operand (l_target)
				fill_call_operands (1)
				if l_actuals = Void or else l_actuals.count /= 1 then
						-- Internal error: these semistrict operators
						-- have exactly one argument. This is guaranteed
						-- by the fact that they are infix features.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					print_indentation
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					l_or := l_name.is_infix_or_else or l_name.is_infix_or or l_name.is_infix_or_else_symbol or l_name.is_infix_or_symbol
					if l_or then
						current_file.put_character ('!')
						current_file.put_character ('(')
					end
					print_expression (call_operands.first)
					if l_or then
						current_file.put_character (')')
					end
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					if l_assignment_target /= Void then
						l_semistrict_target := l_assignment_target
						if attached {ET_IDENTIFIER} l_assignment_target as l_identifier and then l_identifier.is_temporary then
							l_temp := l_identifier
						end
					else
						if attached {ET_IDENTIFIER} call_operands.first as l_identifier and then l_identifier.is_temporary then
							l_temp_target := l_identifier
							l_temp := l_temp_target
							mark_temp_variable_used (l_temp)
						else
							l_temp := new_temp_variable (l_target_static_type)
								-- We will set the index of `l_temp' later because
								-- it could still be used in `call_operands'.
							l_temp_index := call_operands.first.index
						end
						l_semistrict_target := l_temp
					end
					call_operands.wipe_out
					l_actual := l_actuals.actual_argument (1)
					l_actual_type_set := dynamic_type_set (l_actual)
					print_assignment_operand (l_actual, l_actual_type_set, l_semistrict_target, l_target_static_type)
					fill_call_operands (1)
					if call_operands.first /= l_semistrict_target then
						print_indentation
						print_writable (l_semistrict_target)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_character ('(')
						print_expression (call_operands.first)
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					elseif l_semistrict_target = l_temp then
						check l_temp_not_void: l_temp /= Void then
							mark_temp_variable_used (l_temp)
						end
					end
					dedent
					print_indentation
					current_file.put_character ('}')
					l_implies := l_name.is_infix_implies or l_name.is_infix_implies_symbol
					if not l_implies and l_semistrict_target = l_temp_target then
						current_file.put_new_line
					else
						current_file.put_character (' ')
						current_file.put_string (c_else)
						current_file.put_character (' ')
						current_file.put_character ('{')
						current_file.put_new_line
						indent
						print_indentation
						print_writable (l_semistrict_target)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						if l_or or l_implies then
							current_file.put_string (c_eif_true)
						else
							current_file.put_string (c_eif_false)
						end
						current_file.put_character (';')
						current_file.put_new_line
						dedent
						print_indentation
						current_file.put_character ('}')
						current_file.put_new_line
					end
					operand_stack.force (l_semistrict_target)
				end
			else
				l_seed := l_name.seed
					-- Check whether this a call to FUNCTION.item with a manifest
					-- tuple as argument. We have a special treatment in that case
					-- to avoid having to create the manifest tuple when possible.
				if not a_call.is_tuple_label and then l_seed = current_system.function_item_seed and then not a_call.is_call_agent then
					if l_actuals /= Void and then l_actuals.count = 1 then
						if attached {ET_MANIFEST_TUPLE} l_actuals.actual_argument (1) as l_manifest_tuple then
							print_target_operand (l_target, l_target_static_type)
							nb := l_manifest_tuple.count
							from i := 1 until i > nb loop
								l_actual := l_manifest_tuple.expression (i)
								l_actual_type_set := dynamic_type_set (l_actual)
								print_attachment_operand (l_actual, l_actual_type_set)
								i := i + 1
							end
							nb := nb + 1
							fill_call_operands (nb)
							l_manifest_tuple_operand := agent_manifest_tuple
							l_manifest_tuple_operand.resize (nb - 1)
							from i := nb until i < 2 loop
								l_manifest_tuple_operand.put_first (call_operands.item (i))
								i := i - 1
							end
							l_manifest_tuple_operand.set_index (l_manifest_tuple.index)
							mark_expressions_frozen (l_manifest_tuple_operand)
							l_target_operand := call_operands.first
							call_operands.wipe_out
							call_operands.put_last (l_target_operand)
							call_operands.put_last (l_manifest_tuple_operand)
							nb := 2
						end
					end
				end
				if l_manifest_tuple_operand = Void then
					print_target_operand (l_target, l_target_static_type)
					if l_actuals /= Void then
						nb := l_actuals.count
						from i := 1 until i > nb loop
							l_actual := l_actuals.actual_argument (i)
							l_actual_type_set := dynamic_type_set (l_actual)
							print_attachment_operand (l_actual, l_actual_type_set)
							i := i + 1
						end
					end
					nb := nb + 1
					fill_call_operands (nb)
				end
				if in_operand then
					print_indentation
					if l_assignment_target /= Void then
						operand_stack.force (l_assignment_target)
						print_writable (l_assignment_target)
					else
						l_temp := new_temp_variable (l_call_type)
							-- We will set the index of `l_temp' later because
							-- it could still be used in `call_operands'.
						l_temp_index := a_call.index
						if l_force_result_boxing then
								-- `current_dynamic_system.any_type' is the first type set
								-- in `extra_dynamic_type_sets'.
							l_temp_index := current_dynamic_type_sets.count + 1
						end
						operand_stack.force (l_temp)
						print_temp_name (l_temp, current_file)
					end
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
				end
				nb2 := l_target_type_set.count
				if nb2 = 0 then
						-- Call on Void target.
					current_file.put_character ('(')
					print_target_expression (call_operands.first, l_target_static_type, True)
					from i := 2 until i > nb loop
						current_file.put_character (',')
						current_file.put_character (' ')
						print_expression (call_operands.item (i))
						i := i + 1
					end
					current_file.put_character (',')
					current_file.put_character (' ')
					print_typed_default_entity_value (l_call_type, current_file)
					current_file.put_character (')')
				elseif nb2 = 1 then
						-- Static binding.
					l_target_dynamic_type := l_target_type_set.dynamic_type (1)
					print_adapted_named_query_call (l_name, l_target_dynamic_type, l_call_type, True)
				else
						-- Dynamic binding.
					if l_name.is_tuple_label then
-- TODO: fields in Tuples are not necessarily aligned, and some need (un)boxing,
-- for example:
--    t1: TUPLE [a: ANY; b: INTEGER]
--    t2: TUPLE [a: CHARACTER; b: INTEGER]
--    t1 := t2
--    any := t1.a
--    int := t1.b
-- But when there are aligned with no (un)boxing, we should avoid having polymorphic calls.
					else
						l_query := l_target_static_type.base_class.seeded_query (l_seed)
						if l_query = Void then
								-- Internal error: there should be a query with `a_seed'.
								-- It has been computed in ET_FEATURE_CHECKER.
							set_fatal_error
							error_handler.report_giaaa_error
							l_printed := True
						else
							if l_query.is_constant_attribute then
								l_target_dynamic_type := l_target_type_set.dynamic_type (1)
								print_adapted_named_query_call (l_name, l_target_dynamic_type, l_call_type, True)
								l_printed := True
							elseif l_query.is_unique_attribute then
								l_target_dynamic_type := l_target_type_set.dynamic_type (1)
								print_adapted_named_query_call (l_name, l_target_dynamic_type, l_call_type, True)
								l_printed := True
							end
						end
					end
					if l_printed then
						-- Do nothing.
					elseif nb2 = 2 then
							-- First type.
						l_target_dynamic_type := l_target_type_set.dynamic_type (1)
						current_file.put_character ('(')
						current_file.put_character ('(')
						print_attribute_type_id_access (call_operands.first, l_target_static_type, True)
						current_file.put_character ('=')
						current_file.put_character ('=')
						current_file.put_integer (l_target_dynamic_type.id)
						current_file.put_character (')')
						current_file.put_character ('?')
						print_adapted_named_query_call (l_name, l_target_dynamic_type, l_call_type, False)
						current_file.put_character (':')
							-- Second type.
						l_target_dynamic_type := l_target_type_set.dynamic_type (2)
						print_adapted_named_query_call (l_name, l_target_dynamic_type, l_call_type, False)
						current_file.put_character (')')
					elseif l_dynamic_call = Void then
							-- Internal error: the dynamic call should have been created in ET_DYNAMIC_TYPE_BUILDER.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						l_dynamic_call.set_force_result_boxing (l_force_result_boxing)
						register_polymorphic_called_features (l_dynamic_call)
						print_call_name (a_call, current_feature, l_target_static_type, l_force_result_boxing, current_file)
						current_file.put_character ('(')
						current_file.put_string (c_ac)
						current_file.put_character (',')
						current_file.put_character (' ')
						print_target_expression (call_operands.first, l_target_static_type, True)
						if l_manifest_tuple_operand /= Void then
							nb := l_manifest_tuple_operand.count
							from i := 1 until i > nb loop
								current_file.put_character (',')
								current_file.put_character (' ')
								print_expression (l_manifest_tuple_operand.expression (i))
								i := i + 1
							end
						else
							from i := 2 until i > nb loop
								current_file.put_character (',')
								current_file.put_character (' ')
								print_expression (call_operands.item (i))
								i := i + 1
							end
						end
						current_file.put_character (')')
					end
				end
				if in_operand then
					current_file.put_character (')')
					print_semicolon_newline
				end
			end
			call_operands.wipe_out
			if l_manifest_tuple_operand /= Void then
				mark_expressions_unfrozen (l_manifest_tuple_operand)
				l_manifest_tuple_operand.wipe_out
			end
			if l_temp /= Void and then l_temp_index /= 0 then
					-- We had to wait until this stage to set the index of `l_temp'
					-- because it could have still been used in `call_operands'.
				l_temp.set_index (l_temp_index)
			end
		end

	print_quantifier_expression (a_expression: ET_QUANTIFIER_EXPRESSION)
			-- Print `a_expression'.
		require
			a_expression_not_void: a_expression /= Void
		do
			print_iteration_expression (a_expression)
		end

	print_regular_integer_constant (a_constant: ET_REGULAR_INTEGER_CONSTANT)
			-- Check validity of `a_constant'.
		require
			a_constant_not_void: a_constant /= Void
		do
			print_integer_constant (a_constant)
		end

	print_regular_manifest_string (a_string: ET_REGULAR_MANIFEST_STRING)
			-- Print `a_string'.
		require
			a_string_not_void: a_string /= Void
		do
			print_manifest_string (a_string)
		end

	print_regular_real_constant (a_constant: ET_REGULAR_REAL_CONSTANT)
			-- Print `a_constant'.
		require
			a_constant_not_void: a_constant /= Void
		local
			l_temp: ET_IDENTIFIER
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if in_operand then
				if in_call_target then
					in_operand := False
					l_dynamic_type := dynamic_type_set (a_constant).static_type.primary_type
					l_temp := new_temp_variable (l_dynamic_type)
					print_indentation
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_regular_real_constant (a_constant)
					current_file.put_character (';')
					current_file.put_new_line
					l_temp.set_index (a_constant.index)
					operand_stack.force (l_temp)
					in_operand := True
				else
					operand_stack.force (a_constant)
				end
			else
				l_dynamic_type := dynamic_type_set (a_constant).static_type.primary_type
				print_type_cast (l_dynamic_type, current_file)
				current_file.put_character ('(')
				if a_constant.is_negative then
					current_file.put_character ('-')
				end
				current_file.put_string (a_constant.literal)
				current_file.put_character (')')
			end
		end

	print_result (an_expression: ET_RESULT)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_static_type: ET_DYNAMIC_PRIMARY_TYPE
			l_once_kind: INTEGER
			l_once_index: INTEGER
		do
			if in_operand then
				operand_stack.force (an_expression)
			else
				if attached call_target_type as l_call_target_type then
					check in_call_target: in_call_target end
					if has_rescue then
							-- Keep track of the fact that the value of the 'Result' entity has
							-- been read. Useful to determine the 'volatile' status of the 'Result'
							-- entity when current feature has a rescue clause.
						result_read := True
					end
					l_dynamic_type_set := dynamic_type_set (an_expression)
					l_static_type := l_dynamic_type_set.static_type.primary_type
					if l_static_type.is_expanded then
							-- Pass the address of the expanded object.
						current_file.put_character ('(')
						if has_rescue then
								-- When there is a rescue clause, the result entity might
								-- have been declared with a 'volatile' qualifier. In that
								-- case we need a type cast here to avoid C compiler warning
								-- (at least with MSVC 12).
							current_file.put_character ('(')
							print_type_declaration (l_static_type, current_file)
							current_file.put_character ('*')
							current_file.put_character (')')
						end
						if current_agent = Void and then current_feature.is_once then
								-- Make sure that if a once-function is called
								-- recursively, the semantics specified by ECMA will be satisfied.
								-- ECMA 367-2 section 8.23.22 page 124 says that the recursive calls
								-- to the once-function should return the value of 'Result' as it
								-- was when the recursive calls occurred.
								-- There is a special treatment for once-per-process and once-per-thread
								-- functions of non-basic expanded types because a boxed version of the
								-- Result (and not the Result itself) is kept in the 'GE_onces' C struct.
								-- So we need to use the content of the boxed value in place of Result
								-- in case its fields get modified. Likewise for once-per-object functions
								-- even though the Result is not boxed in that case.
							if not l_static_type.is_basic then
								if current_feature.is_once_per_object then
									current_file.put_character ('&')
									print_once_value (current_feature, 0, 0)
								else
									once_features.search (current_feature.static_feature.implementation_feature)
									if once_features.found then
										l_once_index := once_features.found_item
										l_once_kind := once_kind (current_feature)
									end
										-- The once value has been boxed.
										-- It needs to be unboxed.
									current_file.put_character ('(')
									current_file.put_character ('(')
									print_boxed_type_cast (l_static_type, current_file)
									current_file.put_character ('(')
									print_once_value (current_feature, l_once_kind, l_once_index)
									current_file.put_character (')')
									current_file.put_character (')')
									current_file.put_string (c_arrow)
									print_boxed_attribute_pointer_name (l_static_type, current_file)
									current_file.put_character (')')
								end
							else
								current_file.put_character ('&')
								print_result_name (current_file)
							end
						else
							current_file.put_character ('&')
							print_result_name (current_file)
						end
						current_file.put_character (')')
					elseif l_call_target_type.is_expanded then
							-- We need to unbox the object and then pass its address.
						print_boxed_attribute_pointer_access (an_expression, l_call_target_type, call_target_check_void)
					else
						print_result_name (current_file)
					end
				elseif current_agent = Void and then current_feature.is_once then
						-- Make sure that if a once-function is called
						-- recursively, the semantics specified by ECMA will be satisfied.
						-- ECMA 367-2 section 8.23.22 page 124 says that the recursive calls
						-- to the once-function should return the value of 'Result' as it
						-- was when the recursive calls occurred.
						-- There is a special treatment for once-per-process and once-per-thread
						-- functions of non-basic expanded types because a boxed version of the
						-- Result (and not the Result itself) is kept in the 'GE_onces' C struct.
						-- So we need to use the content of the boxed value in place of Result
						-- in case its fields get modified. Likewise for once-per-object functions
						-- even though the Result is not boxed in that case.
					l_dynamic_type_set := dynamic_type_set (an_expression)
					l_static_type := l_dynamic_type_set.static_type.primary_type
					if l_static_type.is_expanded and then not l_static_type.is_basic then
						if current_feature.is_once_per_object then
							print_once_value (current_feature, 0, 0)
						else
							once_features.search (current_feature.static_feature.implementation_feature)
							if once_features.found then
								l_once_index := once_features.found_item
								l_once_kind := once_kind (current_feature)
							end
								-- The once value has been boxed.
								-- It needs to be unboxed.
							current_file.put_character ('(')
							print_boxed_type_cast (l_static_type, current_file)
							current_file.put_character ('(')
							print_once_value (current_feature, l_once_kind, l_once_index)
							current_file.put_character (')')
							current_file.put_character (')')
							current_file.put_string (c_arrow)
							print_boxed_attribute_item_name (l_static_type, current_file)
						end
					else
						print_result_name (current_file)
					end
				else
					print_result_name (current_file)
				end
			end
		end

	print_result_address (an_expression: ET_RESULT_ADDRESS)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		local
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_result_type_set: detachable ET_DYNAMIC_TYPE_SET
			l_temp: ET_IDENTIFIER
			l_pointer: BOOLEAN
			l_pointer_type: ET_DYNAMIC_PRIMARY_TYPE
			l_static_type: ET_DYNAMIC_PRIMARY_TYPE
			l_once_kind: INTEGER
			l_once_index: INTEGER
		do
			l_pointer_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.pointer_type, current_type.base_type)
			if has_rescue then
					-- Keep track of the fact that the value of the 'Result' entity can
					-- possibly be modified and read. Useful to determine the 'volatile'
					-- status of the 'Result' entity when current feature has a rescue clause.
				result_read := True
				result_written := True
			end
			l_dynamic_type_set := dynamic_type_set (an_expression)
			l_dynamic_type := l_dynamic_type_set.static_type.primary_type
			l_pointer := (l_dynamic_type = l_pointer_type)
			if not l_pointer then
					-- $Result is of type TYPED_POINTER.
				l_temp := new_temp_variable (l_dynamic_type)
				l_temp.set_index (an_expression.index)
				operand_stack.force (l_temp)
				l_queries := l_dynamic_type.queries
				if l_queries.is_empty then
						-- Internal error: TYPED_POINTER should have an attribute
						-- `pointer_item' at first position.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					print_indentation
					print_attribute_access (l_queries.first, l_temp, l_dynamic_type, False)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
				end
			end
			if l_pointer and in_operand then
					-- $Result is of type POINTER.
				operand_stack.force (an_expression)
			else
				current_file.put_character ('(')
				l_result_type_set := current_feature.result_type_set
				if l_result_type_set = Void then
						-- Internal error: it should have been checked elsewhere that
						-- the current feature is a function.
					set_fatal_error
					error_handler.report_giaaa_error
				elseif l_result_type_set.is_expanded then
					print_type_cast (l_pointer_type, current_file)
					if current_agent = Void and then current_feature.is_once then
							-- Make sure that if a once-function is called
							-- recursively, the semantics specified by ECMA will be satisfied.
							-- ECMA 367-2 section 8.23.22 page 124 says that the recursive calls
							-- to the once-function should return the value of 'Result' as it
							-- was when the recursive calls occurred.
							-- There is a special treatment for once-per-process and once-per-thread
							-- functions of non-basic expanded types because a boxed version of the
							-- Result (and not the Result itself) is kept in the 'GE_onces' C struct.
							-- So we need to use the content of the boxed value in place of Result
							-- in case its fields get modified. Likewise for once-per-object functions
							-- even though the Result is not boxed in that case.
						l_static_type := l_result_type_set.static_type.primary_type
						if not l_static_type.is_basic then
							if current_feature.is_once_per_object then
								current_file.put_character ('&')
								print_once_value (current_feature, 0, 0)
							else
								once_features.search (current_feature.static_feature.implementation_feature)
								if once_features.found then
									l_once_index := once_features.found_item
									l_once_kind := once_kind (current_feature)
								end									-- The once value has been boxed.
									-- It needs to be unboxed.
								current_file.put_character ('(')
								print_boxed_type_cast (l_static_type, current_file)
								current_file.put_character ('(')
								print_once_value (current_feature, l_once_kind, l_once_index)
								current_file.put_character (')')
								current_file.put_character (')')
								current_file.put_string (c_arrow)
								print_boxed_attribute_pointer_name (l_static_type, current_file)
							end
						else
							current_file.put_character ('&')
							print_result_name (current_file)
						end
					else
						current_file.put_character ('&')
						print_result_name (current_file)
					end
				elseif attached l_result_type_set.special_type as l_special_type then
					current_file.put_character ('(')
					print_result_name (current_file)
					current_file.put_character ('?')
					print_type_cast (l_pointer_type, current_file)
					current_file.put_character ('(')
					current_file.put_string (c_ge_types)
					current_file.put_character ('[')
					print_attribute_type_id_access (tokens.result_keyword, l_result_type_set.static_type.primary_type, False)
					current_file.put_character (']')
					current_file.put_character ('[')
					current_file.put_character ('0')
					current_file.put_character (']')
					current_file.put_character ('.')
					current_file.put_string (c_is_special)
					current_file.put_character ('?')
					print_type_cast (l_pointer_type, current_file)
					print_attribute_special_item_access (an_expression.result_keyword, l_special_type, False)
					current_file.put_character (':')
					print_type_cast (l_pointer_type, current_file)
					print_result_name (current_file)
					current_file.put_character (')')
					current_file.put_character (':')
					print_type_cast (l_pointer_type, current_file)
					current_file.put_character ('0')
					current_file.put_character (')')
				else
					print_type_cast (l_pointer_type, current_file)
					print_result_name (current_file)
				end
				current_file.put_character (')')
			end
			if not l_pointer then
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_special_manifest_string (a_string: ET_SPECIAL_MANIFEST_STRING)
			-- Print `a_string'.
		require
			a_string_not_void: a_string /= Void
		do
			print_manifest_string (a_string)
		end

	print_static_call_expression (a_expression: ET_STATIC_CALL_EXPRESSION)
			-- Print `a_expression'.
		require
			a_expression_not_void: a_expression /= Void
		local
			l_target_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			l_target_type := current_dynamic_system.dynamic_primary_type (a_expression.type, current_type.base_type)
			print_expression_static_call (a_expression, l_target_type, a_expression.name, a_expression.arguments)
		end

	print_expression_static_call (a_expression: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_name: ET_CALL_NAME; a_arguments: detachable ET_ACTUAL_ARGUMENTS)
			-- Print `a_expression', which is expected to be a static call to
			-- feature with the same seed as `a_name' in type `a_target_type'
			-- and with `a_arguments'.
		require
			a_expression_not_void: a_expression /= Void
			a_target_type_not_void: a_target_type /= Void
			a_name_not_void: a_name /= Void
		local
			l_query: detachable ET_QUERY
			l_dynamic_feature: ET_DYNAMIC_FEATURE
			l_seed: INTEGER
			i, nb: INTEGER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp: detachable ET_IDENTIFIER
			l_temp_index: INTEGER
			l_assignment_target: like assignment_target
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type_set: ET_DYNAMIC_TYPE_SET
			old_index: INTEGER
			l_constant: ET_CONSTANT
			l_once_feature: ET_FEATURE
			l_actual: ET_EXPRESSION
		do
			l_assignment_target := assignment_target
			assignment_target := Void
			if a_arguments /= Void then
				nb := a_arguments.count
				from i := 1 until i > nb loop
					l_actual := a_arguments.actual_argument (i)
					l_actual_type_set := dynamic_type_set (l_actual)
					print_attachment_operand (l_actual, l_actual_type_set)
					i := i + 1
				end
			end
			fill_call_operands (nb)
			if in_operand then
				if l_assignment_target /= Void then
					operand_stack.force (l_assignment_target)
					print_indentation
					print_writable (l_assignment_target)
				else
					l_dynamic_type_set := dynamic_type_set (a_expression)
					l_dynamic_type := l_dynamic_type_set.static_type.primary_type
					l_temp := new_temp_variable (l_dynamic_type)
						-- We will set the index of `l_temp' later because
						-- it could still be used in `call_operands'.
					l_temp_index := a_expression.index
					operand_stack.force (l_temp)
					print_indentation
					print_temp_name (l_temp, current_file)
				end
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('(')
			end
			l_seed := a_name.seed
			l_query := a_target_type.base_class.seeded_query (l_seed)
			if l_query = Void then
					-- Internal error: there should be a query with `l_seed'.
					-- It has been computed in ET_FEATURE_CHECKER or else an
					-- error should have already been reported.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_query.is_attribute then
					-- Internal error: no object available.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_query.is_constant_attribute then
				if not attached {ET_CONSTANT_ATTRIBUTE} l_query as l_constant_attribute then
						-- Internal error.
					set_fatal_error
					error_handler.report_giaaa_error
				elseif attached {ET_MANIFEST_STRING} l_constant_attribute.constant as l_string_constant then
					l_once_feature := l_constant_attribute.implementation_feature
					constant_features.force_last (l_string_constant, l_once_feature)
					print_once_value_name (l_once_feature, current_file)
				else
					l_constant := l_constant_attribute.constant
					old_index := l_constant.index
					l_constant.set_index (a_expression.index)
					print_expression (l_constant)
					l_constant.set_index (old_index)
				end
			elseif l_query.is_unique_attribute then
				if not attached {ET_UNIQUE_ATTRIBUTE} l_query as l_unique_attribute then
						-- Internal error.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					l_dynamic_type_set := dynamic_type_set (a_expression)
					l_dynamic_type := l_dynamic_type_set.static_type.primary_type
					print_type_cast (l_dynamic_type, current_file)
					current_file.put_character ('(')
					current_file.put_integer (l_unique_attribute.constant.to_integer_32)
					current_file.put_character (')')
				end
			else
				l_dynamic_feature := a_target_type.dynamic_query (l_query, current_dynamic_system)
				register_called_static_feature (l_dynamic_feature)
				print_static_routine_name (l_dynamic_feature, a_target_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				from i := 1 until i > nb loop
					current_file.put_character (',')
					current_file.put_character (' ')
					l_actual_type_set := dynamic_type_set (call_operands.item (i))
					l_formal_type_set := argument_type_set_in_feature (i, l_dynamic_feature)
					print_attachment_expression (call_operands.item (i), l_actual_type_set, l_formal_type_set.static_type)
					i := i + 1
				end
				current_file.put_character (')')
			end
			if in_operand then
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
			call_operands.wipe_out
			if l_temp /= Void and then l_temp_index /= 0 then
					-- We had to wait until this stage to set the index of `l_temp'
					-- because it could have still been used in `call_operands'.
				l_temp.set_index (l_temp_index)
			end
		end

	print_strip_expression (an_expression: ET_STRIP_EXPRESSION)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
			if in_operand then
				operand_stack.force (an_expression)
			else
-- TODO.
error_handler.report_warning_message ("ET_C_GENERATOR.print_strip_expression")
				current_file.put_string (c_eif_void)
			end
		end

	print_target_expression (an_expression: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print `an_expression' when appearing as the target of a call.
			-- `a_target_type' is one of the possible dynamic types of `an_expression'.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
		require
			an_expression_not_void: an_expression /= Void
			a_target_type_not_void: a_target_type /= Void
		local
			old_in_operand: BOOLEAN
			old_target_type: like call_target_type
			old_target_check_void: BOOLEAN
			l_old_in_procedure_call_target: BOOLEAN
		do
			old_in_operand := in_operand
			old_target_type := call_target_type
			old_target_check_void := call_target_check_void
			l_old_in_procedure_call_target := in_procedure_call_target
			in_operand := False
			call_target_type := a_target_type
			in_procedure_call_target := False
			call_target_check_void := a_check_void_target
			print_check_void_expression (an_expression, a_target_type, a_check_void_target)
			call_target_check_void := old_target_check_void
			in_procedure_call_target := l_old_in_procedure_call_target
			call_target_type := old_target_type
			in_operand := old_in_operand
		end

	print_target_operand (an_operand: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print `an_operand' when appearing as the target of a call.
			-- `a_target_type' is one of the possible dynamic types of `an_operand'.
		require
			an_operand_not_void: an_operand /= Void
			a_target_type_not_void: a_target_type /= Void
		local
			old_in_operand: BOOLEAN
			old_target_type: like call_target_type
			l_old_in_procedure_call_target: BOOLEAN
		do
			old_in_operand := in_operand
			old_target_type := call_target_type
			l_old_in_procedure_call_target := in_procedure_call_target
			in_operand := True
			call_target_type := a_target_type
			in_procedure_call_target := False
			an_operand.process (Current)
			in_procedure_call_target := l_old_in_procedure_call_target
			call_target_type := old_target_type
			in_operand := old_in_operand
		end

	print_temporary_variable (a_name: ET_IDENTIFIER)
			-- Print temporary variable `a_name'.
		require
			a_name_not_void: a_name /= Void
			a_name_temporary: a_name.is_temporary
		do
			if in_operand then
				operand_stack.force (a_name)
			else
				if attached call_target_type as l_call_target_type then
					check in_call_target: in_call_target end
					if not is_temp_variable_known (a_name) then
							-- Internal error: the temporary
							-- variable should be known at this stage.
						set_fatal_error
						error_handler.report_giaaa_error
					elseif not attached temp_variable_type (a_name) as l_static_type then
							-- Internal error: the type of the temporary
							-- variable should be known at this stage.
						set_fatal_error
						error_handler.report_giaaa_error
					elseif l_static_type.is_expanded then
							-- Pass the address of the expanded object.
						current_file.put_character ('&')
						print_temp_name (a_name, current_file)
					elseif l_call_target_type.is_expanded then
							-- We need to unbox the object and then pass its address.
						print_boxed_attribute_pointer_access (a_name, l_call_target_type, call_target_check_void)
					else
						print_temp_name (a_name, current_file)
					end
				else
					print_temp_name (a_name, current_file)
				end
			end
		end

	print_true_constant (a_constant: ET_TRUE_CONSTANT)
			-- Print `a_constant'.
		require
			a_constant_not_void: a_constant /= Void
		local
			l_temp: ET_IDENTIFIER
		do
			if in_operand then
				if in_call_target then
					l_temp := new_temp_variable (dynamic_type_set (a_constant).static_type.primary_type)
					print_indentation
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_true)
					current_file.put_character (';')
					current_file.put_new_line
					l_temp.set_index (a_constant.index)
					operand_stack.force (l_temp)
				else
					operand_stack.force (a_constant)
				end
			else
				current_file.put_string (c_eif_true)
			end
		end

	print_unboxed_expression (an_expression: ET_EXPRESSION; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void: BOOLEAN)
			-- Print to `current_file' unboxed version of `an_expression' of type `a_type'.
			-- There is no assumption about static type (i.e. declared type) of
			-- `an_expression' or about `a_type': they may be reference or expanded.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
			-- `a_check_void' means that we need to check whether the expression is Void or not.
		require
			an_expression_not_void: an_expression /= Void
			a_type_not_void: a_type /= Void
		local
			l_static_type: ET_DYNAMIC_TYPE
		do
			if a_type.is_expanded then
				l_static_type := dynamic_type_set (an_expression).static_type
				if l_static_type.is_expanded then
					print_expression (an_expression)
				else
						-- We need to unbox the object.
					current_file.put_character ('*')
					print_boxed_attribute_pointer_access (an_expression, a_type, a_check_void)
				end
			else
				print_expression (an_expression)
			end
		end

	print_underscored_integer_constant (a_constant: ET_UNDERSCORED_INTEGER_CONSTANT)
			-- Print `a_constant'.
		require
			a_constant_not_void: a_constant /= Void
		do
			print_integer_constant (a_constant)
		end

	print_underscored_real_constant (a_constant: ET_UNDERSCORED_REAL_CONSTANT)
			-- Print `a_constant'.
		require
			a_constant_not_void: a_constant /= Void
		local
			i, nb: INTEGER
			l_literal: STRING
			l_temp: ET_IDENTIFIER
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if in_operand then
				if in_call_target then
					in_operand := False
					l_dynamic_type := dynamic_type_set (a_constant).static_type.primary_type
					l_temp := new_temp_variable (l_dynamic_type)
					print_indentation
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_underscored_real_constant (a_constant)
					current_file.put_character (';')
					current_file.put_new_line
					l_temp.set_index (a_constant.index)
					operand_stack.force (l_temp)
					in_operand := True
				else
					operand_stack.force (a_constant)
				end
			else
				l_dynamic_type := dynamic_type_set (a_constant).static_type.primary_type
				print_type_cast (l_dynamic_type, current_file)
				current_file.put_character ('(')
				if a_constant.is_negative then
					current_file.put_character ('-')
				end
				l_literal := a_constant.literal
				nb := l_literal.count
				from i := 1 until i > nb loop
					if l_literal.item (i) /= '_' then
						current_file.put_character (l_literal.item (i))
					end
					i := i + 1
				end
				current_file.put_character (')')
			end
		end

	print_unqualified_call_expression (a_call: ET_UNQUALIFIED_FEATURE_CALL_EXPRESSION)
			-- Print unqualified call expression.
		require
			a_call_not_void: a_call /= Void
		local
			l_actuals: detachable ET_ACTUAL_ARGUMENTS
			l_actual: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			i, nb: INTEGER
			l_call_type_set: ET_DYNAMIC_TYPE_SET
			l_call_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp: detachable ET_IDENTIFIER
			l_temp_index: INTEGER
			l_assignment_target: like assignment_target
			l_dynamic_feature: detachable ET_DYNAMIC_FEATURE
			l_once_feature: ET_FEATURE
			l_old_call_target: detachable ET_EXPRESSION
			l_manifest_tuple_operand: detachable ET_MANIFEST_TUPLE
			l_target_operand: ET_EXPRESSION
			l_seed: INTEGER
			old_index: INTEGER
			l_constant: ET_CONSTANT
			l_force_result_boxing: BOOLEAN
		do
			if in_static_feature then
				print_expression_static_call (a_call, current_type, a_call.name, a_call.arguments)
			else
				l_assignment_target := assignment_target
				assignment_target := Void
				l_call_type_set := dynamic_type_set (a_call)
				l_dynamic_feature := current_type.seeded_dynamic_query (a_call.name.seed, current_dynamic_system)
				if l_dynamic_feature = Void then
						-- Internal error: there should be a query for that seed.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					l_call_type := l_call_type_set.static_type.primary_type
					if
						l_dynamic_feature.is_attribute and then
						(not attached {ET_EXTENDED_ATTRIBUTE} l_dynamic_feature.static_feature as l_extended_attribute or else
						not l_extended_attribute.has_self_initializing_code or else
						not attached l_dynamic_feature.result_type_set as l_result_type_set or else
						l_result_type_set.static_type.is_self_initializing)
					then
							-- This is an attribute for which there is no self-initialization code
							-- to be executed.
						if in_operand then
							operand_stack.force (a_call)
						elseif l_dynamic_feature.is_builtin then
								-- This is a built-in attribute (such as feature 'item' in 'INTEGER_REF').
								-- The call might be the operand (target or argument) of another call.
								-- Therefore we need to take some care when dealing with `call_operands'.
								-- We need to keep track of the operands of this possible other call
								-- while processing the current call to the built-in attribute, for
								-- which only the target ('Current') needs to be on `call_operands'.
							if call_operands.is_empty then
								call_operands.force_last (tokens.current_keyword)
							else
								l_old_call_target := call_operands.first
								call_operands.replace (tokens.current_keyword, 1)
							end
							if in_call_target and l_call_type.is_expanded then
									-- Pass the address of the built-in attribute expanded object.
								current_file.put_character ('&')
								current_file.put_character ('(')
									-- No need to call `print_adapted_query_call' here because an
									-- unqualified call is not polymorphic, and therefore the type
									-- of the query is exactly the type expected by the caller.
								print_query_call (l_dynamic_feature, current_type, False)
								current_file.put_character (')')
							else
									-- No need to call `print_adapted_query_call' here because an
									-- unqualified call is not polymorphic, and therefore the type
									-- of the query is exactly the type expected by the caller.
								print_query_call (l_dynamic_feature, current_type, False)
							end
							if l_old_call_target /= Void then
								call_operands.replace (l_old_call_target, 1)
								l_old_call_target := Void
							else
								call_operands.wipe_out
							end
						elseif attached call_target_type as l_call_target_type then
							check in_call_target: in_call_target end
							if l_call_type.is_expanded then
									-- Pass the address of the expanded object.
								current_file.put_character ('&')
								current_file.put_character ('(')
								print_attribute_access (l_dynamic_feature, tokens.current_keyword, current_type, False)
								current_file.put_character (')')
							elseif l_call_target_type.is_expanded then
									-- We need to unbox the object and then pass its address.
								current_file.put_character ('&')
								current_file.put_character ('(')
								print_boxed_attribute_item_access (a_call, l_call_target_type, call_target_check_void)
								current_file.put_character (')')
							else
								print_attribute_access (l_dynamic_feature, tokens.current_keyword, current_type, False)
							end
						else
							print_attribute_access (l_dynamic_feature, tokens.current_keyword, current_type, False)
						end
					elseif l_dynamic_feature.is_constant_attribute then
						if in_operand then
							if in_call_target then
								if l_call_type.is_expanded then
									in_operand := False
									l_temp := new_temp_variable (l_call_type)
									print_indentation
									print_temp_name (l_temp, current_file)
									current_file.put_character (' ')
									current_file.put_character ('=')
									current_file.put_character (' ')
									print_unqualified_call_expression (a_call)
									current_file.put_character (';')
									current_file.put_new_line
									l_temp.set_index (a_call.index)
									operand_stack.force (l_temp)
									in_operand := True
								else
									operand_stack.force (a_call)
								end
							else
								operand_stack.force (a_call)
							end
						elseif attached {ET_CONSTANT_ATTRIBUTE} l_dynamic_feature.static_feature as l_constant_attribute then
							if attached {ET_MANIFEST_STRING} l_constant_attribute.constant as l_string_constant then
								l_once_feature := l_constant_attribute.implementation_feature
								constant_features.force_last (l_string_constant, l_once_feature)
								print_once_value_name (l_once_feature, current_file)
							else
								l_constant := l_constant_attribute.constant
								old_index := l_constant.index
								l_constant.set_index (a_call.index)
								print_expression (l_constant)
								l_constant.set_index (old_index)
							end
						end
					else
						if
							in_call_target and then
							(l_dynamic_feature.is_builtin_special_item or l_dynamic_feature.is_once) and then
							l_call_type.is_expanded and then
							(not l_call_type.is_basic or else in_procedure_call_target)
						then
								-- We want the result be boxed even though the result static type is expanded.
								-- This is needed when the call is a "SPECIAL" item or a once function, the call
								-- result type is expanded, and it is the target of another call. In that case we
								-- force the result type to be of reference type in order to force the boxing of
								-- the result so that the other call is applied on the result object itself and
								-- not on a copy of this object.
								-- Note that for optimization we don't force boxing when the type is a basic
								-- type unless the result object is the target of a procedure call (e.g.
								-- 'copy', 'standard_copy' or 'set_item' which will overwrite the value of
								-- the target).
							l_call_type := current_dynamic_system.any_type
							l_force_result_boxing := True
						end
						l_actuals := a_call.arguments
						l_seed := a_call.name.seed
							-- Check whether this a call to FUNCTION.item with a manifest
							-- tuple as argument. We have a special treatment in that case
							-- to avoid having to create the manifest tuple when possible.
						if l_seed = current_system.function_item_seed and then not a_call.is_call_agent then
							if l_actuals /= Void and then l_actuals.count = 1 then
								if attached {ET_MANIFEST_TUPLE} l_actuals.actual_argument (1) as l_manifest_tuple then
									operand_stack.force (tokens.current_keyword)
									nb := l_manifest_tuple.count
									from i := 1 until i > nb loop
										l_actual := l_manifest_tuple.expression (i)
										l_actual_type_set := dynamic_type_set (l_actual)
										print_attachment_operand (l_actual, l_actual_type_set)
										i := i + 1
									end
									nb := nb + 1
									fill_call_operands (nb)
									l_manifest_tuple_operand := agent_manifest_tuple
									l_manifest_tuple_operand.resize (nb - 1)
									from i := nb until i < 2 loop
										l_manifest_tuple_operand.put_first (call_operands.item (i))
										i := i - 1
									end
									l_manifest_tuple_operand.set_index (l_manifest_tuple.index)
									mark_expressions_frozen (l_manifest_tuple_operand)
									l_target_operand := call_operands.first
									call_operands.wipe_out
									call_operands.put_last (l_target_operand)
									call_operands.put_last (l_manifest_tuple_operand)
									nb := 2
								end
							end
						end
						if l_manifest_tuple_operand = Void then
							operand_stack.force (tokens.current_keyword)
							if l_actuals /= Void then
								nb := l_actuals.count
								from i := 1 until i > nb loop
									l_actual := l_actuals.actual_argument (i)
									l_actual_type_set := dynamic_type_set (l_actual)
									print_attachment_operand (l_actual, l_actual_type_set)
									i := i + 1
								end
							end
							nb := nb + 1
							fill_call_operands (nb)
						end
						if in_operand then
							if l_assignment_target /= Void then
								operand_stack.force (l_assignment_target)
								print_indentation
								print_writable (l_assignment_target)
							else
								l_temp := new_temp_variable (l_call_type)
									-- We will set the index of `l_temp' later because
									-- it could still be used in `call_operands'.
								l_temp_index := a_call.index
								if l_force_result_boxing then
										-- `current_dynamic_system.any_type' is the first type set
										-- in `extra_dynamic_type_sets'.
									l_temp_index := current_dynamic_type_sets.count + 1
								end
								operand_stack.force (l_temp)
								print_indentation
								print_temp_name (l_temp, current_file)
							end
							current_file.put_character (' ')
							current_file.put_character ('=')
							current_file.put_character (' ')
							current_file.put_character ('(')
						end
							-- When there is no result type of the form 'like argument',
							-- then there is no need to call `print_adapted_query_call'
							-- because an unqualified call is not polymorphic, and therefore
							-- the type of the query is exactly the type expected by
							-- the caller. This is another story with 'like argument'.
							-- For example 'clone (1)' is expected to be of type INTEGER
							-- and 'clone ("gobo")' is expected to be of type STRING.
							-- But the corresponding generated function for 'clone' will
							-- be declared to return an ANY. Hence the need for a call
							-- to `print_adapted_query_call' in this case.
							-- There is also the case when `l_force_result_boxing' is True.
						print_adapted_query_call (l_dynamic_feature, current_type, l_call_type, False)
						if in_operand then
							current_file.put_character (')')
							current_file.put_character (';')
							current_file.put_new_line
						end
						call_operands.wipe_out
						if l_manifest_tuple_operand /= Void then
							mark_expressions_unfrozen (l_manifest_tuple_operand)
							l_manifest_tuple_operand.wipe_out
						end
						if l_temp /= Void and then l_temp_index /= 0 then
								-- We had to wait until this stage to set the index of `l_temp'
								-- because it could have still been used in `call_operands'.
							l_temp.set_index (l_temp_index)
						end
					end
				end
			end
		end

	print_verbatim_string (a_string: ET_VERBATIM_STRING)
			-- Print `a_string'.
		require
			a_string_not_void: a_string /= Void
		do
			print_manifest_string (a_string)
		end

	print_void (an_expression: ET_VOID)
			-- Print `an_expression'.
		require
			an_expression_not_void: an_expression /= Void
		do
			if in_operand then
				operand_stack.force (an_expression)
			else
				current_file.put_string (c_eif_void)
			end
		end

	print_writable (a_writable: ET_WRITABLE)
			-- Print `a_writable'.
		require
			a_writable_not_void: a_writable /= Void
		local
			l_query: detachable ET_DYNAMIC_FEATURE
			l_seed: INTEGER
			old_in_operand: BOOLEAN
			old_target_type: like call_target_type
			l_old_in_procedure_call_target: BOOLEAN
			l_was_read: BOOLEAN
		do
			old_in_operand := in_operand
			old_target_type := call_target_type
			l_old_in_procedure_call_target := in_procedure_call_target
			in_operand := False
			in_procedure_call_target := False
			call_target_type := Void
			if attached {ET_IDENTIFIER} a_writable as l_identifier then
				if l_identifier.is_argument then
					print_formal_argument (l_identifier)
				elseif l_identifier.is_temporary then
					print_temporary_variable (l_identifier)
				elseif l_identifier.is_iteration_item then
					print_iteration_item (l_identifier)
				elseif l_identifier.is_local then
					if has_rescue then
							-- Keep track of the fact that the value of this local variable
							-- has been modified. Useful to determine the 'volatile' status
							-- of the local variable when current feature or inline agent
							-- has a rescue clause.
							--
							-- Set `has_rescue' to False before calling `print_local_variable'
							-- because the fact that we are in a rescue clause has already
							-- been taken into account.
						locals_written.force_last (l_identifier)
						l_was_read := locals_read.has (l_identifier)
						has_rescue := False
						print_local_variable (l_identifier)
						has_rescue := True
						if not l_was_read and locals_read.has (l_identifier) then
							locals_read.remove (l_identifier)
						end
					else
						print_local_variable (l_identifier)
					end
				else
					l_seed := l_identifier.seed
					l_query := current_type.seeded_dynamic_query (l_seed, current_dynamic_system)
					if l_query = Void then
							-- Internal error: there should be a query with `l_seed'.
							-- It has been computed in ET_FEATURE_CHECKER.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						print_attribute_access (l_query, tokens.current_keyword, current_type, False)
					end
				end
			else
				check
					is_result: a_writable.is_result
				end
				if has_rescue then
						-- Keep track of the fact that the value of the 'Result' entity
						-- has been modified. Useful to determine the 'volatile' status
						-- of the 'Result' entity when current feature or inline agent
						-- has a rescue clause.
						--
						-- Set `has_rescue' to False before calling `print_local_variable'
						-- because the fact that we are in a rescue clause has already
						-- been taken into account.
					result_written := True
					l_was_read := result_read
					has_rescue := False
					a_writable.process (Current)
					has_rescue := True
					result_read := l_was_read
				else
					a_writable.process (Current)
				end
			end
			call_target_type := old_target_type
			in_procedure_call_target := l_old_in_procedure_call_target
			in_operand := old_in_operand
		end

feature {NONE} -- Equality generation

	print_equality_call (a_expression: ET_EXPRESSION; a_negated: BOOLEAN; a_equality_types: ET_DYNAMIC_EQUALITY_TYPES)
			-- Print `a_expression' which is supposed to be equality expression
			-- ('=' or '/=' as well as '~' or '/~') with operand types `a_equality_types'.
			-- `a_negated' means that the operator is either "/=" or "/~".
			-- Operands can be found in `call_operands'.
		require
			a_expression_not_void: a_expression /= Void
			a_equality_types_not_void: a_equality_types /= Void
		local
			l_common_object_equality_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			l_common_reference_equality_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			l_temp: detachable ET_IDENTIFIER
			l_temp_index: INTEGER
			l_assignment_target: like assignment_target
			l_left_operand: ET_EXPRESSION
			l_right_operand: ET_EXPRESSION
			l_left_type_set: ET_DYNAMIC_TYPE_SET
			l_right_type_set: ET_DYNAMIC_TYPE_SET
			l_boolean_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			l_assignment_target := assignment_target
			assignment_target := Void
			l_left_type_set := a_equality_types.left
			l_right_type_set := a_equality_types.right
			l_boolean_type := dynamic_type_set (a_expression).static_type.primary_type
			if in_operand then
				if l_assignment_target /= Void then
					operand_stack.force (l_assignment_target)
					print_indentation
					print_writable (l_assignment_target)
				else
					l_temp := new_temp_variable (l_boolean_type)
						-- We will set the index of `l_temp' later because
						-- it could still be used in `call_operands'.
					l_temp_index := a_expression.index
					operand_stack.force (l_temp)
					print_indentation
					print_temp_name (l_temp, current_file)
				end
				print_assign_to
			end
				-- Find out which types are in the dynamic type set of both operands.
			l_common_object_equality_types := equality_common_types
			l_common_object_equality_types.resize (l_left_type_set.count.min (l_right_type_set.count))
			l_common_reference_equality_types := equality_common_reference_types
			l_common_reference_equality_types.resize (l_left_type_set.count.min (l_right_type_set.count))
			a_equality_types.add_common_types_to_list (l_common_object_equality_types, l_common_reference_equality_types)
			if l_common_object_equality_types.is_empty then
				print_equality_call_with_zero_object_equality (a_negated, l_common_object_equality_types, l_common_reference_equality_types)
			elseif l_common_object_equality_types.count = 1 then
				print_equality_call_with_one_object_equality (a_negated, l_common_object_equality_types, l_common_reference_equality_types)
			else
				l_left_operand := call_operands.first
				l_right_operand := call_operands.item (2)
				current_equalities.force_last (a_equality_types)
				if a_negated then
					print_declaration_type_cast (l_boolean_type, current_file)
					current_file.put_character ('(')
					current_file.put_string (c_not)
				end
				print_equality_function_name (current_equalities.count, current_feature, current_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				print_comma
				print_expression (l_left_operand)
				print_comma
				print_expression (l_right_operand)
				current_file.put_character (')')
				if a_negated then
					current_file.put_character (')')
				end
			end
			if in_operand then
				print_semicolon_newline
			end
			l_common_object_equality_types.wipe_out
			l_common_reference_equality_types.wipe_out
			if l_temp /= Void and then l_temp_index /= 0 then
					-- We had to wait until this stage to set the index of `l_temp'
					-- because it could have still been used in `call_operands'.
				l_temp.set_index (l_temp_index)
			end
		end

	print_equality_call_with_zero_object_equality (a_negated: BOOLEAN; a_common_object_equality_types, a_common_reference_equality_types: ET_DYNAMIC_PRIMARY_TYPES)
			-- Print equality call when there is no type in common between
			-- the dynamic type sets of the left and right operands for which
			-- object equality is needed.
			-- `a_negated' means that the operator is either "/=" or "/~".
			-- Operands can be found in `call_operands'.
		require
			a_common_object_equality_types_not_void: a_common_object_equality_types /= Void
			a_common_object_equality_types_is_empty: a_common_object_equality_types.is_empty
			a_common_reference_equality_types_not_void: a_common_reference_equality_types /= Void
			two_operands: call_operands.count = 2
		local
			l_left_operand: ET_EXPRESSION
			l_right_operand: ET_EXPRESSION
			l_left_type_set: ET_DYNAMIC_TYPE_SET
			l_right_type_set: ET_DYNAMIC_TYPE_SET
			l_eif_true: STRING
			l_eif_false: STRING
			l_equal: STRING
			l_boolean_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if a_negated then
				l_eif_true := c_eif_false
				l_eif_false := c_eif_true
				l_equal := c_not_equal
			else
				l_eif_true := c_eif_true
				l_eif_false := c_eif_false
				l_equal := c_equal
			end
			l_left_operand := call_operands.first
			l_right_operand := call_operands.item (2)
			l_left_type_set := dynamic_type_set (l_left_operand)
			l_right_type_set := dynamic_type_set (l_right_operand)
			if l_left_type_set.is_expanded or l_right_type_set.is_expanded then
				current_file.put_string (l_eif_false)
			elseif l_left_type_set.is_empty and l_right_type_set.is_empty then
					-- We know for sure that both operands are Void.
				current_file.put_string (l_eif_true)
			elseif a_common_reference_equality_types.is_empty and (not l_left_type_set.can_be_void or not l_right_type_set.can_be_void) then
					-- There is no type in common between the dynamic type sets
					-- of the left and right operands. The only way for the equality
					-- to succeed is that both operands be Void.
				current_file.put_string (l_eif_false)
			else
				l_boolean_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.boolean_type, current_type.base_type)
				print_declaration_type_cast (l_boolean_type, current_file)
				current_file.put_character ('(')
				print_expression (l_left_operand)
				current_file.put_string (l_equal)
				print_expression (l_right_operand)
				current_file.put_character (')')
			end
		end

	print_equality_call_with_one_object_equality (a_negated: BOOLEAN; a_common_object_equality_types, a_common_reference_equality_types: ET_DYNAMIC_PRIMARY_TYPES)
			-- Print equality call when there is exactly one type in common between
			-- the dynamic type sets of the left and right operands for which
			-- object equality is needed.
			-- `a_negated' means that the operator is either "/=" or "/~".
			-- Operands can be found in `call_operands'.
		require
			a_common_object_equality_types_not_void: a_common_object_equality_types /= Void
			a_common_object_equality_types_count_one: a_common_object_equality_types.count = 1
			a_common_reference_equality_types_not_void: a_common_reference_equality_types /= Void
			two_operands: call_operands.count = 2
		local
			l_left_operand: ET_EXPRESSION
			l_right_operand: ET_EXPRESSION
			l_left_type_set: ET_DYNAMIC_TYPE_SET
			l_right_type_set: ET_DYNAMIC_TYPE_SET
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_parenthese_count: INTEGER
			l_formal_type: ET_DYNAMIC_TYPE
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_left_static_type: ET_DYNAMIC_PRIMARY_TYPE
			l_right_static_type: ET_DYNAMIC_PRIMARY_TYPE
			l_eif_true: STRING
			l_eif_false: STRING
			l_equal: STRING
			l_index: INTEGER
			l_old_index: INTEGER
			l_boolean_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if a_negated then
				l_eif_true := c_eif_false
				l_eif_false := c_eif_true
				l_equal := c_not_equal
			else
				l_eif_true := c_eif_true
				l_eif_false := c_eif_false
				l_equal := c_equal
			end
			l_left_operand := call_operands.first
			l_right_operand := call_operands.item (2)
			l_left_type_set := dynamic_type_set (l_left_operand)
			l_right_type_set := dynamic_type_set (l_right_operand)
			l_left_static_type := l_left_type_set.static_type.primary_type
			l_right_static_type := l_right_type_set.static_type.primary_type
			l_dynamic_type := a_common_object_equality_types.dynamic_type (1)
			if not attached l_dynamic_type.seeded_dynamic_query (current_system.is_equal_seed, current_dynamic_system) as l_is_equal_feature then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				if l_left_type_set.can_be_void then
					l_parenthese_count := l_parenthese_count + 1
					current_file.put_character ('(')
					current_file.put_character ('(')
					print_expression (l_left_operand)
					print_equal_to
					current_file.put_string (c_eif_void)
					current_file.put_character (')')
					current_file.put_character ('?')
					if l_right_type_set.can_be_void then
						l_boolean_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.boolean_type, current_type.base_type)
						print_declaration_type_cast (l_boolean_type, current_file)
						current_file.put_character ('(')
						print_expression (l_right_operand)
						current_file.put_string (l_equal)
						current_file.put_string (c_eif_void)
						current_file.put_character (')')
					else
						current_file.put_string (l_eif_false)
					end
					current_file.put_character (':')
				elseif l_right_type_set.can_be_void then
					l_parenthese_count := l_parenthese_count + 1
					current_file.put_character ('(')
					current_file.put_character ('(')
					print_expression (l_right_operand)
					print_equal_to
					current_file.put_string (c_eif_void)
					current_file.put_character (')')
					current_file.put_character ('?')
					current_file.put_string (l_eif_false)
					current_file.put_character (':')
				end
					-- Check that both operands are of the same type.
				if l_left_type_set.count > 1 and l_right_type_set.count > 1 then
						-- Both operands can have several types in their type set.
						-- Check that they have the same type.
					l_parenthese_count := l_parenthese_count + 1
					current_file.put_character ('(')
					current_file.put_character ('(')
					print_attribute_type_id_access (l_left_operand, l_left_static_type, False)
					print_equal_to
					print_attribute_type_id_access (l_right_operand, l_right_static_type, False)
					current_file.put_character (')')
					current_file.put_character ('?')
				elseif l_left_type_set.count > 1 then
						-- We know that the type set of the right operand contains
						-- only one type, which is `l_dynamic_type'. So we just have
						-- to check that the left operand is of that type.
					l_parenthese_count := l_parenthese_count + 1
					current_file.put_character ('(')
					current_file.put_character ('(')
					print_attribute_type_id_access (l_left_operand, l_left_static_type, False)
					print_equal_to
					current_file.put_integer (l_dynamic_type.id)
					current_file.put_character (')')
					current_file.put_character ('?')
				elseif l_right_type_set.count > 1 then
						-- We know that the type set of the left operand contains
						-- only one type, which is `l_dynamic_type'. So we just have
						-- to check that the right operand is of that type.
					l_parenthese_count := l_parenthese_count + 1
					current_file.put_character ('(')
					current_file.put_character ('(')
					print_attribute_type_id_access (l_right_operand, l_right_static_type, False)
					print_equal_to
					current_file.put_integer (l_dynamic_type.id)
					current_file.put_character (')')
					current_file.put_character ('?')
				else
						-- The type sets of both operands contain only one type,
						-- which is `l_dynamic_type'. So we know for sure that
						-- both operands are of the same type.
				end
					-- Call 'is_equal'.
				l_formal_type := argument_type_set_in_feature (1, l_is_equal_feature).static_type
				if not l_dynamic_type.conforms_to_primary_type (l_formal_type.primary_type) then
						-- We won't be able to call 'is_equal' because the type of the
						-- actual argument does not conform to formal type. The only
						-- way of the equality to be True as to have both operands Void.
					current_file.put_string (l_eif_false)
				else
						-- The main purpose of object equality is to be CAT-call free.
						-- Therefore we have to make sure that `l_dynamic_type' is the
						-- only type in the dynamic type set of the right operand when
						-- calling `print_attachment_expression'.
					if l_right_static_type = l_dynamic_type then
						l_actual_type_set := l_dynamic_type
					elseif l_right_type_set.count = 1 then
						l_actual_type_set := l_right_type_set
					else
						equality_type_set.reset (l_right_static_type)
						equality_type_set.put_type (l_dynamic_type)
						if l_right_type_set.is_never_void then
							equality_type_set.set_never_void
						end
						l_actual_type_set := equality_type_set
					end
					extra_dynamic_type_sets.force_last (l_actual_type_set)
					l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
					l_old_index := l_right_operand.index
					l_right_operand.set_index (l_index)
					if a_negated then
						l_boolean_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.boolean_type, current_type.base_type)
						print_declaration_type_cast (l_boolean_type, current_file)
						current_file.put_character ('(')
						current_file.put_string (c_not)
					end
					if not l_dynamic_type.has_redefined_is_equal_routine then
						print_builtin_any_is_equal_call (l_is_equal_feature, l_dynamic_type, False)
					else
						register_called_feature (l_is_equal_feature)
						print_routine_name (l_is_equal_feature, l_dynamic_type, current_file)
						current_file.put_character ('(')
						current_file.put_string (c_ac)
						print_comma
						print_target_expression (l_left_operand, l_dynamic_type, False)
						print_comma
						print_attachment_expression (l_right_operand, l_actual_type_set, l_formal_type)
						current_file.put_character (')')
					end
					if a_negated then
						current_file.put_character (')')
					end
					l_right_operand.set_index (l_old_index)
					extra_dynamic_type_sets.remove_last
						-- Clean up `equality_type_set'.
					if l_actual_type_set = equality_type_set then
						equality_type_set.reset (current_dynamic_system.unknown_type)
					end
				end
				if l_left_type_set.count > 1 or l_right_type_set.count > 1 then
					current_file.put_character (':')
						-- Reference equality.
					if a_common_reference_equality_types.is_empty then
						current_file.put_string (l_eif_false)
					else
						print_equality_call_with_zero_object_equality (a_negated, current_dynamic_system.unknown_type, a_common_reference_equality_types)
					end
				end
				from until l_parenthese_count = 0 loop
					current_file.put_character (')')
					l_parenthese_count := l_parenthese_count - 1
				end
			end
		end

	print_equality_function (i: INTEGER; a_equality_types: ET_DYNAMIC_EQUALITY_TYPES)
			-- Print function corresponding to `i'-th equality ('=' or '/=', as well as
			-- '~' or '/~') with operand types `a_equality_types' to `current_file' and
			-- its signature to `header_file'.
		require
			a_equality_types_not_void: a_equality_types /= Void
		local
			old_file: KI_TEXT_OUTPUT_STREAM
			l_common_object_equality_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			l_common_reference_equality_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
			l_left_type_set: ET_DYNAMIC_TYPE_SET
			l_right_type_set: ET_DYNAMIC_TYPE_SET
			l_left_static_type: ET_DYNAMIC_PRIMARY_TYPE
			l_right_static_type: ET_DYNAMIC_PRIMARY_TYPE
			l_left_operand: ET_IDENTIFIER
			l_right_operand: ET_IDENTIFIER
			l_index: INTEGER
		do
			l_left_type_set := a_equality_types.left
			l_right_type_set := a_equality_types.right
			l_left_static_type := l_left_type_set.static_type.primary_type
			l_right_static_type := l_right_type_set.static_type.primary_type
				-- Print signature to `header_file' and `current_file'.
			old_file := current_file
			current_file := current_function_header_buffer
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			l_result_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.boolean_type, current_type.base_type)
			print_type_declaration (l_result_type, header_file)
			print_type_declaration (l_result_type, current_file)
			header_file.put_character (' ')
			current_file.put_character (' ')
			print_equality_function_name (i, current_feature, current_type, header_file)
			print_equality_function_name (i, current_feature, current_type, current_file)
			header_file.put_character ('(')
			current_file.put_character ('(')
			header_file.put_string (c_ge_context)
			header_file.put_character ('*')
			header_file.put_character (' ')
			header_file.put_string (c_ac)
			header_file.put_character (',')
			header_file.put_character (' ')
			current_file.put_string (c_ge_context)
			current_file.put_character ('*')
			current_file.put_character (' ')
			current_file.put_string (c_ac)
			current_file.put_character (',')
			current_file.put_character (' ')
			l_left_operand := formal_argument (1)
			extra_dynamic_type_sets.force_last (l_left_type_set)
			l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
			l_left_operand.set_index (l_index)
			print_type_declaration (l_left_static_type, header_file)
			header_file.put_character (' ')
			print_argument_name (l_left_operand, header_file)
			header_file.put_character (',')
			header_file.put_character (' ')
			print_type_declaration (l_left_static_type, current_file)
			current_file.put_character (' ')
			print_argument_name (l_left_operand, current_file)
			current_file.put_character (',')
			current_file.put_character (' ')
			l_right_operand := formal_argument (2)
			extra_dynamic_type_sets.force_last (l_right_type_set)
			l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
			l_right_operand.set_index (l_index)
			print_type_declaration (l_right_static_type, header_file)
			header_file.put_character (' ')
			print_argument_name (l_right_operand, header_file)
			print_type_declaration (l_right_static_type, current_file)
			current_file.put_character (' ')
			print_argument_name (l_right_operand, current_file)
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
				-- Print body to `current_file'.
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			current_file := current_function_body_buffer
				-- Find out which types are in the dynamic type set of both operands.
			l_common_object_equality_types := equality_common_types
			l_common_object_equality_types.resize (l_left_type_set.count.min (l_right_type_set.count))
			l_common_reference_equality_types := equality_common_reference_types
			l_common_reference_equality_types.resize (l_left_type_set.count.min (l_right_type_set.count))
			a_equality_types.add_common_types_to_list (l_common_object_equality_types, l_common_reference_equality_types)
			operand_stack.force (l_left_operand)
			operand_stack.force (l_right_operand)
			fill_call_operands (2)
			print_equality_function_body (l_common_object_equality_types, l_common_reference_equality_types)
			call_operands.wipe_out
			l_common_object_equality_types.wipe_out
			l_common_reference_equality_types.wipe_out
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
			l_left_operand.set_index (1)
			l_right_operand.set_index (2)
			extra_dynamic_type_sets.remove_last
			extra_dynamic_type_sets.remove_last
			current_file := old_file
			flush_to_c_file
			reset_temp_variables
		end

	print_equality_function_body (a_common_object_equality_types, a_common_reference_equality_types: ET_DYNAMIC_PRIMARY_TYPES)
			-- Print equality function body.
			-- Operands can be found in `call_operands'.
		require
			a_common_object_equality_types_not_void: a_common_object_equality_types /= Void
			a_common_reference_equality_types_not_void: a_common_reference_equality_types /= Void
			two_operands: call_operands.count = 2
		local
			l_left_operand: ET_EXPRESSION
			l_right_operand: ET_EXPRESSION
			l_left_type_set: ET_DYNAMIC_TYPE_SET
			l_right_type_set: ET_DYNAMIC_TYPE_SET
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_formal_type: ET_DYNAMIC_TYPE
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_left_static_type: ET_DYNAMIC_PRIMARY_TYPE
			l_right_static_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
			l_index: INTEGER
			l_old_index: INTEGER
			l_boolean_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if a_common_object_equality_types.is_empty then
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				print_equality_call_with_zero_object_equality (False, a_common_object_equality_types, a_common_reference_equality_types)
				print_semicolon_newline
			elseif a_common_object_equality_types.count = 1 then
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				print_equality_call_with_one_object_equality (False, a_common_object_equality_types, a_common_reference_equality_types)
				print_semicolon_newline
			else
				l_left_operand := call_operands.first
				l_right_operand := call_operands.item (2)
				l_left_type_set := dynamic_type_set (l_left_operand)
				l_right_type_set := dynamic_type_set (l_right_operand)
				l_left_static_type := l_left_type_set.static_type.primary_type
				l_right_static_type := l_right_type_set.static_type.primary_type
				if l_left_type_set.can_be_void then
					print_indentation_if
					print_expression (l_left_operand)
					print_equal_to
					current_file.put_string (c_eif_void)
					print_then_newline
					indent
					print_indentation
					current_file.put_string (c_return)
					current_file.put_character (' ')
					if l_right_type_set.can_be_void then
						l_boolean_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.boolean_type, current_type.base_type)
						print_declaration_type_cast (l_boolean_type, current_file)
						current_file.put_character ('(')
						print_expression (l_right_operand)
						print_equal_to
						current_file.put_string (c_eif_void)
						current_file.put_character (')')
					else
						current_file.put_string (c_eif_false)
					end
					print_semicolon_newline
					dedent
					print_indentation_end_newline
				end
				if l_right_type_set.can_be_void then
					print_indentation_if
					print_expression (l_right_operand)
					print_equal_to
					current_file.put_string (c_eif_void)
					print_then_newline
					indent
					print_indentation
					current_file.put_string (c_return)
					current_file.put_character (' ')
					current_file.put_string (c_eif_false)
					print_semicolon_newline
					dedent
					print_indentation_end_newline
				end
				print_indentation
				current_file.put_string (c_switch)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_attribute_type_id_access (l_left_operand, l_left_static_type, False)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				nb := a_common_object_equality_types.count
				from i := 1 until i > nb loop
					l_dynamic_type := a_common_object_equality_types.dynamic_type (i)
					print_indentation
					current_file.put_string (c_case)
					current_file.put_character (' ')
					current_file.put_integer (l_dynamic_type.id)
					current_file.put_character (':')
					current_file.put_new_line
					indent
					if not attached l_dynamic_type.seeded_dynamic_query (current_system.is_equal_seed, current_dynamic_system) as l_is_equal_feature then
							-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						l_formal_type := argument_type_set_in_feature (1, l_is_equal_feature).static_type
						if not l_dynamic_type.conforms_to_primary_type (l_formal_type.primary_type) then
								-- We won't be able to call 'is_equal' because the type of the
								-- actual argument does not conform to formal type.
							print_indentation
							current_file.put_string (c_return)
							current_file.put_character (' ')
							current_file.put_string (c_eif_false)
							print_semicolon_newline
						else
							print_indentation
							current_file.put_string (c_return)
							current_file.put_character (' ')
							l_boolean_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.boolean_type, current_type.base_type)
							print_declaration_type_cast (l_boolean_type, current_file)
							current_file.put_character ('(')
							current_file.put_character ('(')
							print_attribute_type_id_access (l_right_operand, l_right_static_type, False)
							print_equal_to
							current_file.put_integer (l_dynamic_type.id)
							current_file.put_character (')')
							print_and_then
								-- Call 'is_equal'.
								--
								-- The main purpose of object equality is to be CAT-call free.
								-- Therefore we have to make sure that `l_dynamic_type' is the
								-- only type in the dynamic type set of the right operand when
								-- calling `print_attachment_expression'.
							if l_right_static_type = l_dynamic_type then
								l_actual_type_set := l_dynamic_type
							else
								equality_type_set.reset (l_right_static_type)
								equality_type_set.put_type (l_dynamic_type)
								if l_right_type_set.is_never_void then
									equality_type_set.set_never_void
								end
								l_actual_type_set := equality_type_set
							end
							extra_dynamic_type_sets.force_last (l_actual_type_set)
							l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
							l_old_index := l_right_operand.index
							l_right_operand.set_index (l_index)
							if not l_dynamic_type.has_redefined_is_equal_routine then
								print_builtin_any_is_equal_call (l_is_equal_feature, l_dynamic_type, False)
							else
								register_called_feature (l_is_equal_feature)
								print_routine_name (l_is_equal_feature, l_dynamic_type, current_file)
								current_file.put_character ('(')
								current_file.put_string (c_ac)
								print_comma
								print_target_expression (l_left_operand, l_dynamic_type, False)
								print_comma
								print_attachment_expression (l_right_operand, l_actual_type_set, l_formal_type)
								current_file.put_character (')')
							end
							l_right_operand.set_index (l_old_index)
							extra_dynamic_type_sets.remove_last
								-- Clean up `equality_type_set'.
							if l_actual_type_set = equality_type_set then
								equality_type_set.reset (current_dynamic_system.unknown_type)
							end
							current_file.put_character (')')
							print_semicolon_newline
						end
					end
					dedent
					i := i + 1
				end
					-- Reference equality.
				print_indentation
				current_file.put_string (c_default)
				current_file.put_character (':')
				current_file.put_new_line
				indent
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				if a_common_reference_equality_types.is_empty then
					current_file.put_string (c_eif_false)
				else
					print_equality_call_with_zero_object_equality (False, current_dynamic_system.unknown_type, a_common_reference_equality_types)
				end
				print_semicolon_newline
				dedent
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
			end
		end

feature {NONE} -- Object-test generation

	print_object_test_function (i: INTEGER; a_object_test: ET_OBJECT_TEST)
			-- Print function corresponding to `i'-th object-test `a_object_test'
			-- to `current_file' and its signature to `header_file'.
			-- This function will return True or False depending if it's
			-- successful or not. And if the object-test has a local name,
			-- then it will attach, when the object-test is successful,
			-- the value of the expression passed as first argument to the
			-- object-test local whose address is passed as second argument.
		require
			a_object_test_not_void: a_object_test /= Void
		local
			old_file: KI_TEXT_OUTPUT_STREAM
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
			j, nb: INTEGER
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_source_type_set: ET_DYNAMIC_TYPE_SET
			l_source_type: ET_DYNAMIC_TYPE
			l_source_primary_type: ET_DYNAMIC_PRIMARY_TYPE
			l_target_type_set: ET_DYNAMIC_TYPE_SET
			l_target_type: ET_DYNAMIC_TYPE
			l_target_primary_type: ET_DYNAMIC_PRIMARY_TYPE
			l_conforming_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			l_non_conforming_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			l_conforming_type_set: ET_DYNAMIC_STANDALONE_TYPE_SET
			l_old_conforming_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			l_target_argument: detachable ET_IDENTIFIER
			l_source_argument: ET_IDENTIFIER
			l_can_be_void: BOOLEAN
			l_name: detachable ET_IDENTIFIER
			l_type: detachable ET_TYPE
		do
			l_name := a_object_test.name
			l_type := a_object_test.type
			l_source_type_set := dynamic_type_set (a_object_test.expression)
			l_source_type := l_source_type_set.static_type
			l_source_primary_type := l_source_type.primary_type
			if l_name /= Void then
				l_target_type_set := dynamic_type_set (l_name)
				l_target_type := l_target_type_set.static_type
			elseif l_type /= Void then
				l_target_type := current_dynamic_system.dynamic_type (l_type, current_type.base_type)
			else
				l_target_type := l_source_type
			end
			l_target_primary_type := l_target_type.primary_type
				-- Print signature to `header_file' and `current_file'.
			old_file := current_file
			current_file := current_function_header_buffer
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			l_result_type := dynamic_type_set (a_object_test).static_type.primary_type
			print_type_declaration (l_result_type, header_file)
			print_type_declaration (l_result_type, current_file)
			header_file.put_character (' ')
			current_file.put_character (' ')
			print_object_test_function_name (i, current_feature, current_type, header_file)
			print_object_test_function_name (i, current_feature, current_type, current_file)
			header_file.put_character ('(')
			current_file.put_character ('(')
			l_source_argument := formal_argument (1)
			print_type_declaration (l_source_primary_type, header_file)
			header_file.put_character (' ')
			print_argument_name (l_source_argument, header_file)
			print_type_declaration (l_source_primary_type, current_file)
			current_file.put_character (' ')
			print_argument_name (l_source_argument, current_file)
			if l_name /= Void then
				l_target_argument := formal_argument (2)
				header_file.put_character (',')
				header_file.put_character (' ')
				print_type_declaration (l_target_primary_type, header_file)
				header_file.put_character ('*')
				header_file.put_character (' ')
				print_argument_name (l_target_argument, header_file)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_type_declaration (l_target_primary_type, current_file)
				current_file.put_character ('*')
				current_file.put_character (' ')
				print_argument_name (l_target_argument, current_file)
			end
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
				-- Print body to `current_file'.
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			current_file := current_function_body_buffer
			nb := l_source_type_set.count
			l_conforming_types := conforming_types
			l_conforming_types.resize (nb)
			l_non_conforming_types := non_conforming_types
			l_non_conforming_types.resize (nb)
			from j := 1 until j > nb loop
				l_dynamic_type := l_source_type_set.dynamic_type (j)
				if l_dynamic_type.conforms_to_primary_type (l_target_type.primary_type) then
					l_conforming_types.put_last (l_dynamic_type)
				else
					l_non_conforming_types.put_last (l_dynamic_type)
				end
				j := j + 1
			end
			if l_conforming_types.is_empty then
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				current_file.put_string (c_eif_false)
				current_file.put_character (';')
				current_file.put_new_line
			else
				l_can_be_void := l_source_type_set.can_be_void
				if l_can_be_void then
					print_indentation
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					current_file.put_character ('(')
					print_expression (l_source_argument)
					current_file.put_character (')')
					current_file.put_character ('=')
					current_file.put_character ('=')
					current_file.put_string (c_eif_void)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					print_indentation
					current_file.put_string (c_return)
					current_file.put_character (' ')
					current_file.put_string (c_eif_false)
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_character (' ')
					current_file.put_string (c_else)
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
				end
				if l_non_conforming_types.is_empty then
						-- Direct assignment.
					if l_target_argument /= Void then
						check l_name_not_void: l_name /= Void end
						print_indentation
						current_file.put_character ('*')
						print_argument_name (l_target_argument, current_file)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						print_attachment_expression (l_source_argument, l_source_type_set, l_target_type)
						current_file.put_character (';')
						current_file.put_new_line
					end
					print_indentation
					current_file.put_string (c_return)
					current_file.put_character (' ')
					current_file.put_string (c_eif_true)
					current_file.put_character (';')
					current_file.put_new_line
				else
					check
							-- None of `l_non_conforming_types' and `l_conforming_types' are empty.
							-- Therefore the source is polymorphic (at least two dynamic types, one
							-- conforming and one not conforming). As a consequence `l_source_type'
							-- is not expanded because for expanded types there is no type other
							-- than itself that conforms to it.
							-- So we know that a type-id is always available.
						not_expanded: not l_source_type.is_expanded
					end
					print_indentation
					current_file.put_string (c_switch)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_attribute_type_id_access (l_source_argument, l_source_primary_type, False)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					l_conforming_type_set := new_standalone_type_set (l_source_type)
					l_old_conforming_types := l_conforming_type_set.dynamic_types
					l_conforming_type_set.reset_with_types (l_source_type, l_conforming_types)
					if l_source_type_set.is_never_void then
						l_conforming_type_set.set_never_void
					end
					if l_non_conforming_types.count < l_conforming_types.count then
						nb := l_non_conforming_types.count
						from j := 1 until j > nb loop
							l_dynamic_type := l_non_conforming_types.dynamic_type (j)
							print_indentation
							current_file.put_string (c_case)
							current_file.put_character (' ')
							current_file.put_integer (l_dynamic_type.id)
							current_file.put_character (':')
							current_file.put_new_line
							j := j + 1
						end
						indent
						print_indentation
						current_file.put_string (c_return)
						current_file.put_character (' ')
						current_file.put_string (c_eif_false)
						current_file.put_character (';')
						current_file.put_new_line
						dedent
						print_indentation
						current_file.put_string (c_default)
						current_file.put_character (':')
						current_file.put_new_line
						indent
						if l_target_argument /= Void then
							check l_name_not_void: l_name /= Void end
							print_indentation
							current_file.put_character ('*')
							print_argument_name (l_target_argument, current_file)
							current_file.put_character (' ')
							current_file.put_character ('=')
							current_file.put_character (' ')
							print_attachment_expression (l_source_argument, l_conforming_type_set, l_target_type)
							current_file.put_character (';')
							current_file.put_new_line
						end
						print_indentation
						current_file.put_string (c_return)
						current_file.put_character (' ')
						current_file.put_string (c_eif_true)
						current_file.put_character (';')
						current_file.put_new_line
						dedent
					else
						nb := l_conforming_types.count
						from j := 1 until j > nb loop
							l_dynamic_type := l_conforming_types.dynamic_type (j)
							print_indentation
							current_file.put_string (c_case)
							current_file.put_character (' ')
							current_file.put_integer (l_dynamic_type.id)
							current_file.put_character (':')
							current_file.put_new_line
							j := j + 1
						end
						indent
						if l_target_argument /= Void then
							check l_name_not_void: l_name /= Void end
							print_indentation
							current_file.put_character ('*')
							print_argument_name (l_target_argument, current_file)
							current_file.put_character (' ')
							current_file.put_character ('=')
							current_file.put_character (' ')
							print_attachment_expression (l_source_argument, l_conforming_type_set, l_target_type)
							current_file.put_character (';')
							current_file.put_new_line
						end
						print_indentation
						current_file.put_string (c_return)
						current_file.put_character (' ')
						current_file.put_string (c_eif_true)
						current_file.put_character (';')
						current_file.put_new_line
						dedent
						print_indentation
						current_file.put_string (c_default)
						current_file.put_character (':')
						current_file.put_new_line
						indent
						print_indentation
						current_file.put_string (c_return)
						current_file.put_character (' ')
						current_file.put_string (c_eif_false)
						current_file.put_character (';')
						current_file.put_new_line
						dedent
					end
					l_conforming_type_set.reset_with_types (current_dynamic_system.unknown_type, l_old_conforming_types)
					free_standalone_type_set (l_conforming_type_set)
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
				end
				if l_can_be_void then
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
				end
			end
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
			l_conforming_types.wipe_out
			l_non_conforming_types.wipe_out
			current_file := old_file
			flush_to_c_file
			reset_temp_variables
		end

feature {NONE} -- Query call generation

	print_adapted_named_query_call (a_name: ET_CALL_NAME; a_target_type, a_result_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_name' (static binding).
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- `a_result_type' is the static type of the result expected by the caller,
			-- used to adapt the result of the query if needed (see header comment of
			-- `print_adapted_expression_with_agent' for details).
			-- Operands can be found in `call_operands'.
		require
			a_target_type_not_void: a_target_type /= Void
			a_result_type_not_void: a_result_type /= Void
			a_name_not_void: a_name /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_seed: INTEGER
			l_tuple_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_query_type_set: ET_DYNAMIC_TYPE_SET
		do
			l_seed := a_name.seed
			if a_name.is_tuple_label then
				if not attached {ET_DYNAMIC_TUPLE_TYPE} a_target_type as l_tuple_type then
						-- Internal error: if `a_name' is a Tuple label then the
						-- target type should be a Tuple_type.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					l_tuple_item_type_sets := l_tuple_type.item_type_sets
					if l_seed < 1 or l_seed > l_tuple_item_type_sets.count then
							-- Internal error: invalid Tuple label.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						l_query_type_set := l_tuple_item_type_sets.item (l_seed)
						print_adapted_expression_with_agent (agent print_attribute_tuple_item_access (l_seed, call_operands.first, a_target_type, a_check_void_target), l_query_type_set, a_result_type, in_call_target)
					end
				end
			elseif attached a_target_type.seeded_dynamic_query (l_seed, current_dynamic_system) as l_dynamic_feature then
				print_adapted_query_call (l_dynamic_feature, a_target_type, a_result_type, a_check_void_target)
			else
					-- Internal error: there should be a query with `l_seed'.
					-- It has been computed in ET_FEATURE_CHECKER.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_adapted_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type, a_result_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- `a_result_type' is the static type of the result expected by the caller,
			-- used to adapt the result of the query if needed (see header comment of
			-- `print_adapted_expression_with_agent' for details).
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_target_type_not_void: a_target_type /= Void
			a_result_type_not_void: a_result_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_static_query: ET_FEATURE
			l_use_boxed_pointer: BOOLEAN
		do
			l_static_query := a_feature.static_feature
			if attached {ET_EXTENDED_ATTRIBUTE} l_static_query as l_extended_attribute then
				print_adapted_extended_attribute_call (a_feature, l_extended_attribute, a_target_type, a_result_type, a_check_void_target)
			elseif a_feature.is_once then
				print_adapted_once_function_call (a_feature, a_target_type, a_result_type, a_check_void_target)
			elseif not attached a_feature.result_type_set as l_query_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_use_boxed_pointer := in_call_target and then (attached {ET_ATTRIBUTE} l_static_query or a_feature.is_builtin_special_item)
				print_adapted_expression_with_agent (agent print_query_call (a_feature, a_target_type, a_check_void_target), l_query_type_set, a_result_type, l_use_boxed_pointer)
			end
		end

	print_adapted_extended_attribute_call (a_feature: ET_DYNAMIC_FEATURE; a_static_feature: ET_EXTENDED_ATTRIBUTE; a_target_type, a_result_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to extended attribute `a_feature' (static binding).
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- `a_result_type' is the static type of the result expected by the caller,
			-- used to adapt the result of the query if needed (see header comment of
			-- `print_adapted_expression_with_agent' for details).
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_static_feature: a_static_feature = a_feature.static_feature
			a_target_type_not_void: a_target_type /= Void
			a_result_type_not_void: a_result_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_query_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif
				a_static_feature.has_self_initializing_code and then
				not l_query_type_set.static_type.is_self_initializing
			then
					-- Note: If there is no attribute initialization code, then
					-- this is a regular attribute, possibly with assertions
					-- or a 'note' clause.
					-- Note: The semantics rule MEVS, in ECMA-367 3-36, section 8.19.20,
					-- says that the attribute initialization code is not executed
					-- if the type of the attribute is self-initializing.
				l_target := call_operands.first
				current_file.put_character ('(')
					-- Initialize the attribute if not already done.
				print_query_call (a_feature, a_target_type, a_check_void_target)
				print_comma
					-- Then return the attribute (or its address if it is the target of another call).
				print_adapted_expression_with_agent (agent print_adapted_attribute_access (a_feature, l_target, a_target_type, a_check_void_target), l_query_type_set, a_result_type, in_call_target)
				current_file.put_character (')')
			else
				print_adapted_expression_with_agent (agent print_query_call (a_feature, a_target_type, a_check_void_target), l_query_type_set, a_result_type, in_call_target)
			end
		end

	print_adapted_once_function_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type, a_result_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to once function `a_feature' (static binding).
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- `a_result_type' is the static type of the result expected by the caller,
			-- used to adapt the result of the once function if needed (see header comment of
			-- `print_adapted_expression_with_agent' for details).
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_once: a_feature.is_once
			a_target_type_not_void: a_target_type /= Void
			a_result_type_not_void: a_result_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_once_index: INTEGER
			l_once_kind: INTEGER
		do
			if not attached a_feature.result_type_set as l_query_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_query_type_set.is_expanded and then not a_result_type.is_expanded then
				current_file.put_character ('(')
					-- Call the once function in order to be sure that it have been executed at least once.
				print_query_call (a_feature, a_target_type, a_check_void_target)
				print_comma
					-- Then return the boxed value.
				register_once_feature (a_feature)
				once_features.search (a_feature.static_feature.implementation_feature)
				if once_features.found then
					l_once_index := once_features.found_item
					l_once_kind := once_kind (a_feature)
				end
				if a_feature.is_once_per_object or l_query_type_set.static_type.is_basic then
					print_adapted_expression_with_agent (agent print_once_value (a_feature, l_once_kind, l_once_index), l_query_type_set, a_result_type, in_call_target)
				else
						-- Note that for once-per-process and once-per-thread functions with
						-- non-basic expanded types, the value is already boxed.
					print_once_value (a_feature, l_once_kind, l_once_index)
				end
				current_file.put_character (')')
			else
				print_adapted_expression_with_agent (agent print_query_call (a_feature, a_target_type, a_check_void_target), l_query_type_set, a_result_type, False)
			end
		end

	print_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_static_query: ET_FEATURE
			l_once_feature: ET_FEATURE
			l_target: ET_EXPRESSION
			old_index: INTEGER
			l_constant: ET_CONSTANT
			old_type: like current_type
			old_feature: like current_feature
		do
			l_static_query := a_feature.static_feature
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif attached {ET_CONSTANT_ATTRIBUTE} l_static_query as l_constant_attribute then
-- TODO: Need to check whether the target is Void or not, even though it does not matter here.
				if attached {ET_MANIFEST_STRING} l_constant_attribute.constant as l_string_constant then
					l_once_feature := l_constant_attribute.implementation_feature
					constant_features.force_last (l_string_constant, l_once_feature)
					print_once_value_name (l_once_feature, current_file)
				else
					old_type := current_type
					current_type := a_feature.target_type
					old_feature := current_feature
					current_feature := a_feature
					l_constant := l_constant_attribute.constant
					old_index := l_constant.index
					extra_dynamic_type_sets.force_last (l_result_type_set)
					l_constant.set_index (current_dynamic_type_sets.count + extra_dynamic_type_sets.count)
					print_expression (l_constant)
					l_constant.set_index (old_index)
					extra_dynamic_type_sets.remove_last
					current_feature := old_feature
					current_type := old_type
				end
			elseif attached {ET_UNIQUE_ATTRIBUTE} l_static_query as l_unique_attribute then
-- TODO: Need to check whether the target is Void or not, even though it does not matter here.
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_integer (l_unique_attribute.constant.to_integer_32)
				current_file.put_character (')')
			elseif
				attached {ET_EXTENDED_ATTRIBUTE} l_static_query as l_extended_attribute and then
				l_extended_attribute.has_self_initializing_code and then
				not l_result_type_set.static_type.is_self_initializing
			then
					-- Note: If there is no attribute initialization code, then
					-- this is a regular attribute, possibly with assertions
					-- or a 'note' clause.
					-- Note: The semantics rule MEVS, in ECMA-367 3-36, section 8.19.20,
					-- says that the attribute initialization code is not executed
					-- if the type of the attribute is self-initializing.
					--
					-- Self-initialization code for this attribute.
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			elseif attached {ET_ATTRIBUTE} l_static_query then
				l_target := call_operands.first
				print_adapted_attribute_access (a_feature, l_target, a_target_type, a_check_void_target)
			elseif a_feature.is_builtin then
				print_builtin_query_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_non_inlined_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a non-inlined version of a call to query `a_feature' (static binding).
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			i, nb: INTEGER
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type_set: ET_DYNAMIC_TYPE_SET
		do
			register_called_feature (a_feature)
			print_routine_name (a_feature, a_target_type, current_file)
			current_file.put_character ('(')
			current_file.put_string (c_ac)
			current_file.put_character (',')
			current_file.put_character (' ')
			nb := call_operands.count
			print_target_expression (call_operands.first, a_target_type, a_check_void_target)
			from i := 2 until i > nb loop
				current_file.put_character (',')
				current_file.put_character (' ')
				l_actual_type_set := dynamic_type_set (call_operands.item (i))
				l_formal_type_set := argument_type_set_in_feature (i - 1, a_feature)
				print_attachment_expression (call_operands.item (i), l_actual_type_set, l_formal_type_set.static_type)
				i := i + 1
			end
			current_file.put_character (')')
		end

	print_builtin_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_class_code
			when {ET_TOKEN_CODES}.builtin_any_class then
				print_builtin_any_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_arguments_32_class then
				print_builtin_arguments_32_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_boolean_class then
				print_builtin_boolean_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_boolean_ref_class then
				print_builtin_boolean_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_character_8_class then
				print_builtin_character_n_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_character_8_ref_class then
				print_builtin_character_n_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_character_32_class then
				print_builtin_character_n_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_character_32_ref_class then
				print_builtin_character_n_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_com_failure_class then
				print_builtin_com_failure_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_function_class then
				print_builtin_function_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_identified_routines_class then
				print_builtin_identified_routines_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_8_class then
				print_builtin_integer_n_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_8_ref_class then
				print_builtin_integer_n_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_16_class then
				print_builtin_integer_n_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_16_ref_class then
				print_builtin_integer_n_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_32_class then
				print_builtin_integer_n_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_32_ref_class then
				print_builtin_integer_n_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_64_class then
				print_builtin_integer_n_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_64_ref_class then
				print_builtin_integer_n_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_class then
				print_builtin_ise_runtime_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_memory_class then
				print_builtin_memory_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_natural_8_class then
				print_builtin_integer_n_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_natural_8_ref_class then
				print_builtin_integer_n_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_natural_16_class then
				print_builtin_integer_n_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_natural_16_ref_class then
				print_builtin_integer_n_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_natural_32_class then
				print_builtin_integer_n_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_natural_32_ref_class then
				print_builtin_integer_n_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_natural_64_class then
				print_builtin_integer_n_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_natural_64_ref_class then
				print_builtin_integer_n_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_class then
				print_builtin_platform_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_pointer_class then
				print_builtin_pointer_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_pointer_ref_class then
				print_builtin_pointer_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_32_class then
				print_builtin_real_n_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_32_ref_class then
				print_builtin_real_n_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_64_class then
				print_builtin_real_n_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_64_ref_class then
				print_builtin_real_n_ref_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_special_class then
				print_builtin_special_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_tuple_class then
				print_builtin_tuple_query_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_type_class then
				print_builtin_type_query_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_any_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "ANY".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_any: a_feature.is_builtin_any_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_any_deep_twin then
				print_builtin_any_deep_twin_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_any_generating_type then
				print_builtin_any_generating_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_any_generator then
				print_builtin_any_generator_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_any_is_equal then
				print_builtin_any_is_equal_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_any_same_type then
				print_builtin_any_same_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_any_standard_is_equal then
				print_builtin_any_standard_is_equal_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_any_standard_twin then
				print_builtin_any_standard_twin_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_any_twin then
				print_builtin_any_twin_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_arguments_32_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "ARGUMENTS_32".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_arguments_32: a_feature.is_builtin_arguments_32_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_arguments_32_argument_count then
				print_builtin_arguments_32_argument_count_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_arguments_32_i_th_argument_pointer then
				print_builtin_arguments_32_i_th_argument_pointer_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_arguments_32_i_th_argument_string then
				print_builtin_arguments_32_i_th_argument_string_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_boolean_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "BOOLEAN".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_boolean: a_feature.is_builtin_boolean_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_boolean_and then
				print_builtin_boolean_and_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_boolean_and_then then
				print_builtin_boolean_and_then_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_boolean_implies then
				print_builtin_boolean_implies_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_boolean_not then
				print_builtin_boolean_not_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_boolean_or then
				print_builtin_boolean_or_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_boolean_or_else then
				print_builtin_boolean_or_else_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_boolean_xor then
				print_builtin_boolean_xor_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_boolean_ref_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "BOOLEAN_REF".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_boolean_ref: a_feature.is_builtin_boolean_ref_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_boolean_ref_item then
				print_builtin_boolean_ref_item_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_character_n_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in classes "CHARACTER_N".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_target_type_not_void: a_target_type /= Void
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_character_n: a_feature.is_builtin_character_n_class
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_character_n_code then
				print_builtin_character_n_code_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_character_n_natural_32_code then
				print_builtin_character_n_natural_32_code_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_character_n_to_character_8 then
				print_builtin_character_n_to_character_8_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_character_n_to_character_32 then
				print_builtin_character_n_to_character_32_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_character_n_ref_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in classes "CHARACTER_N_REF".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_target_type_not_void: a_target_type /= Void
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_character_n_ref: a_feature.is_builtin_character_n_ref_class
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_character_n_ref_item then
				print_builtin_character_n_ref_item_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_com_failure_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "COM_FAILURE".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_com_failure: a_feature.is_builtin_com_failure_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_com_failure_c_strlen then
				print_builtin_com_failure_c_strlen_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_com_failure_ccom_hresult then
				print_builtin_com_failure_ccom_hresult_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_com_failure_ccom_hresult_code then
				print_builtin_com_failure_ccom_hresult_code_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_com_failure_ccom_hresult_facility then
				print_builtin_com_failure_ccom_hresult_facility_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_com_failure_character_size then
				print_builtin_com_failure_character_size_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_com_failure_cwin_error_text then
				print_builtin_com_failure_cwin_error_text_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_function_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "FUNCTION".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_function: a_feature.is_builtin_function_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_function_fast_item then
				print_builtin_function_fast_item_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_function_item then
				print_builtin_function_item_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_identified_routines_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "IDENTIFIED_ROUTINES".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_identified_routines: a_feature.is_builtin_identified_routines_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_identified_routines_eif_current_object_id then
				print_builtin_identified_routines_eif_current_object_id_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_identified_routines_eif_id_object then
				print_builtin_identified_routines_eif_id_object_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_identified_routines_eif_is_object_id_of_current then
				print_builtin_identified_routines_eif_is_object_id_of_current_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_identified_routines_eif_object_id then
				print_builtin_identified_routines_eif_object_id_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_integer_n_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in classes "INTEGER_N" and "NATURAL_N".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_integer_n: a_feature.is_builtin_integer_n_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_integer_n_as_integer_8 then
				print_builtin_integer_n_as_integer_8_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_as_integer_16 then
				print_builtin_integer_n_as_integer_16_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_as_integer_32 then
				print_builtin_integer_n_as_integer_32_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_as_integer_64 then
				print_builtin_integer_n_as_integer_64_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_as_natural_8 then
				print_builtin_integer_n_as_natural_8_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_as_natural_16 then
				print_builtin_integer_n_as_natural_16_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_as_natural_32 then
				print_builtin_integer_n_as_natural_32_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_as_natural_64 then
				print_builtin_integer_n_as_natural_64_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_bit_and then
				print_builtin_integer_n_bit_and_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_bit_not then
				print_builtin_integer_n_bit_not_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_bit_or then
				print_builtin_integer_n_bit_or_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_bit_shift_left then
				print_builtin_integer_n_bit_shift_left_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_bit_shift_right then
				print_builtin_integer_n_bit_shift_right_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_bit_xor then
				print_builtin_integer_n_bit_xor_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_identity then
				print_builtin_integer_n_identity_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_integer_quotient then
				print_builtin_integer_n_integer_quotient_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_integer_remainder then
				print_builtin_integer_n_integer_remainder_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_is_less then
				print_builtin_integer_n_is_less_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_minus then
				print_builtin_integer_n_minus_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_opposite then
				print_builtin_integer_n_opposite_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_plus then
				print_builtin_integer_n_plus_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_power then
				print_builtin_integer_n_power_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_product then
				print_builtin_integer_n_product_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_quotient then
				print_builtin_integer_n_quotient_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_to_character_8 then
				print_builtin_integer_n_to_character_8_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_to_character_32 then
				print_builtin_integer_n_to_character_32_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_to_double then
				print_builtin_integer_n_to_double_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_to_real then
				print_builtin_integer_n_to_real_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_to_real_32 then
				print_builtin_integer_n_to_real_32_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_integer_n_to_real_64 then
				print_builtin_integer_n_to_real_64_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_integer_n_ref_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in classes "INTEGER_N_REF" and "NATURAL_N_REF".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_integer_n_ref: a_feature.is_builtin_integer_n_ref_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_integer_n_ref_item then
				print_builtin_integer_n_ref_item_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_ise_runtime_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "ISE_RUNTIME".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_ise_runtime: a_feature.is_builtin_ise_runtime_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_ise_runtime_attached_type then
				print_builtin_ise_runtime_attached_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_boolean_field then
				print_builtin_ise_runtime_boolean_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_boolean_field_at then
				print_builtin_ise_runtime_boolean_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_character_8_field then
				print_builtin_ise_runtime_character_8_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_character_8_field_at then
				print_builtin_ise_runtime_character_8_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_character_32_field then
				print_builtin_ise_runtime_character_32_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_character_32_field_at then
				print_builtin_ise_runtime_character_32_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_check_assert then
				print_builtin_ise_runtime_check_assert_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_compiler_version then
				print_builtin_ise_runtime_compiler_version_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_detachable_type then
				print_builtin_ise_runtime_detachable_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_dynamic_type then
				print_builtin_ise_runtime_dynamic_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_dynamic_type_at_offset then
				print_builtin_ise_runtime_dynamic_type_at_offset_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_eif_gen_param_id then
				print_builtin_ise_runtime_eif_gen_param_id_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_field_count_of_type then
				print_builtin_ise_runtime_field_count_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_field_name_of_type then
				print_builtin_ise_runtime_field_name_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_field_offset_of_type then
				print_builtin_ise_runtime_field_offset_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_field_static_type_of_type then
				print_builtin_ise_runtime_field_static_type_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_field_type_of_type then
				print_builtin_ise_runtime_field_type_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_generating_type_of_type then
				print_builtin_ise_runtime_generating_type_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_generating_type_8_of_type then
				print_builtin_ise_runtime_generating_type_8_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_generator_of_type then
				print_builtin_ise_runtime_generator_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_generator_8_of_type then
				print_builtin_ise_runtime_generator_8_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_generic_parameter_count then
				print_builtin_ise_runtime_generic_parameter_count_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_8_field then
				print_builtin_ise_runtime_integer_8_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_8_field_at then
				print_builtin_ise_runtime_integer_8_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_16_field then
				print_builtin_ise_runtime_integer_16_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_16_field_at then
				print_builtin_ise_runtime_integer_16_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_32_field then
				print_builtin_ise_runtime_integer_32_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_32_field_at then
				print_builtin_ise_runtime_integer_32_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_64_field then
				print_builtin_ise_runtime_integer_64_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_integer_64_field_at then
				print_builtin_ise_runtime_integer_64_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_attached_type then
				print_builtin_ise_runtime_is_attached_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_copy_semantics_field then
				print_builtin_ise_runtime_is_copy_semantics_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_expanded then
				print_builtin_ise_runtime_is_expanded_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_field_expanded_of_type then
				print_builtin_ise_runtime_is_field_expanded_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_field_transient_of_type then
				print_builtin_ise_runtime_is_field_transient_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_object_marked then
				print_builtin_ise_runtime_is_object_marked_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_special then
				print_builtin_ise_runtime_is_special_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_special_copy_semantics_item then
				print_builtin_ise_runtime_is_special_copy_semantics_item_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_special_of_expanded then
				print_builtin_ise_runtime_is_special_of_expanded_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_special_of_reference then
				print_builtin_ise_runtime_is_special_of_reference_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_special_of_reference_or_basic_type then
				print_builtin_ise_runtime_is_special_of_reference_or_basic_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_special_of_reference_type then
				print_builtin_ise_runtime_is_special_of_reference_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_tuple then
				print_builtin_ise_runtime_is_tuple_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_is_tuple_type then
				print_builtin_ise_runtime_is_tuple_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_8_field then
				print_builtin_ise_runtime_natural_8_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_8_field_at then
				print_builtin_ise_runtime_natural_8_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_16_field then
				print_builtin_ise_runtime_natural_16_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_16_field_at then
				print_builtin_ise_runtime_natural_16_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_32_field then
				print_builtin_ise_runtime_natural_32_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_32_field_at then
				print_builtin_ise_runtime_natural_32_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_64_field then
				print_builtin_ise_runtime_natural_64_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_natural_64_field_at then
				print_builtin_ise_runtime_natural_64_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_new_instance_of then
				print_builtin_ise_runtime_new_instance_of_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_new_special_of_reference_instance_of then
				print_builtin_ise_runtime_new_special_of_reference_instance_of_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_new_tuple_instance_of then
				print_builtin_ise_runtime_new_tuple_instance_of_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_new_type_instance_of then
				print_builtin_ise_runtime_new_type_instance_of_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_object_size then
				print_builtin_ise_runtime_object_size_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_once_objects then
				print_builtin_ise_runtime_once_objects_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_persistent_field_count_of_type then
				print_builtin_ise_runtime_persistent_field_count_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_pointer_field then
				print_builtin_ise_runtime_pointer_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_pointer_field_at then
				print_builtin_ise_runtime_pointer_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_raw_reference_field_at then
				print_builtin_ise_runtime_raw_reference_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_raw_reference_field_at_offset then
				print_builtin_ise_runtime_raw_reference_field_at_offset_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_real_32_field then
				print_builtin_ise_runtime_real_32_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_real_32_field_at then
				print_builtin_ise_runtime_real_32_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_real_64_field then
				print_builtin_ise_runtime_real_64_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_real_64_field_at then
				print_builtin_ise_runtime_real_64_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_reference_field then
				print_builtin_ise_runtime_reference_field_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_reference_field_at then
				print_builtin_ise_runtime_reference_field_at_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_reference_field_at_offset then
				print_builtin_ise_runtime_reference_field_at_offset_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_storable_version_of_type then
				print_builtin_ise_runtime_storable_version_of_type_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_type_conforms_to then
				print_builtin_ise_runtime_type_conforms_to_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_ise_runtime_type_id_from_name then
				print_builtin_ise_runtime_type_id_from_name_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_memory_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "MEMORY".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_memory: a_feature.is_builtin_memory_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_memory_find_referers then
				print_builtin_memory_find_referers_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_platform_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "PLATFORM".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_platform: a_feature.is_builtin_platform_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_platform_boolean_bytes then
				print_builtin_platform_boolean_bytes_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_character_bytes then
				print_builtin_platform_character_bytes_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_double_bytes then
				print_builtin_platform_double_bytes_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_integer_bytes then
				print_builtin_platform_integer_bytes_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_is_64_bits then
				print_builtin_platform_is_64_bits_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_is_dotnet then
				print_builtin_platform_is_dotnet_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_is_mac then
				print_builtin_platform_is_mac_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_is_scoop_capable then
				print_builtin_platform_is_scoop_capable_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_is_thread_capable then
				print_builtin_platform_is_thread_capable_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_is_unix then
				print_builtin_platform_is_unix_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_is_vms then
				print_builtin_platform_is_vms_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_is_vxworks then
				print_builtin_platform_is_vxworks_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_is_windows then
				print_builtin_platform_is_windows_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_pointer_bytes then
				print_builtin_platform_pointer_bytes_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_real_bytes then
				print_builtin_platform_real_bytes_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_platform_wide_character_bytes then
				print_builtin_platform_wide_character_bytes_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_pointer_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "POINTER".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_pointer: a_feature.is_builtin_pointer_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_pointer_hash_code then
				print_builtin_pointer_hash_code_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_pointer_is_default_pointer then
				print_builtin_pointer_is_default_pointer_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_pointer_plus then
				print_builtin_pointer_plus_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_pointer_to_integer_32 then
				print_builtin_pointer_to_integer_32_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_pointer_ref_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "POINTER_REF".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_pointer_ref: a_feature.is_builtin_pointer_ref_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_pointer_ref_item then
				print_builtin_pointer_ref_item_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_real_n_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in classes "REAL_N".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_target_type_not_void: a_target_type /= Void
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_real_n: a_feature.is_builtin_real_n_class
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_real_n_ceiling_real_32 then
				print_builtin_real_n_ceiling_real_32_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_ceiling_real_64 then
				print_builtin_real_n_ceiling_real_64_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_floor_real_32 then
				print_builtin_real_n_floor_real_32_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_floor_real_64 then
				print_builtin_real_n_floor_real_64_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_identity then
				print_builtin_real_n_identity_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_is_equal then
				print_builtin_real_n_ieee_is_equal_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_is_greater then
				print_builtin_real_n_ieee_is_greater_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_is_greater_equal then
				print_builtin_real_n_ieee_is_greater_equal_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_is_less then
				print_builtin_real_n_ieee_is_less_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_is_less_equal then
				print_builtin_real_n_ieee_is_less_equal_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_maximum_number then
				print_builtin_real_n_ieee_maximum_number_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_ieee_minimum_number then
				print_builtin_real_n_ieee_minimum_number_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_is_less then
				print_builtin_real_n_is_less_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_is_nan then
				print_builtin_real_n_is_nan_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_is_negative_infinity then
				print_builtin_real_n_is_negative_infinity_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_is_positive_infinity then
				print_builtin_real_n_is_positive_infinity_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_minus then
				print_builtin_real_n_minus_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_opposite then
				print_builtin_real_n_opposite_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_plus then
				print_builtin_real_n_plus_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_power then
				print_builtin_real_n_power_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_product then
				print_builtin_real_n_product_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_quotient then
				print_builtin_real_n_quotient_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_to_double then
				print_builtin_real_n_to_double_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_truncated_to_integer then
				print_builtin_real_n_truncated_to_integer_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_truncated_to_integer_64 then
				print_builtin_real_n_truncated_to_integer_64_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_truncated_to_real then
				print_builtin_real_n_truncated_to_real_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_real_n_ref_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in classes "REAL_N_REF".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_target_type_not_void: a_target_type /= Void
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_real_n_ref: a_feature.is_builtin_real_n_ref_class
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_real_n_ref_item then
				print_builtin_real_n_ref_item_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_ref_nan then
				print_builtin_real_n_ref_nan_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_ref_negative_infinity then
				print_builtin_real_n_ref_negative_infinity_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_real_n_ref_positive_infinity then
				print_builtin_real_n_ref_positive_infinity_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_special_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "SPECIAL".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_special: a_feature.is_builtin_special_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_special_base_address then
				print_builtin_special_base_address_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_special_capacity then
				print_builtin_special_capacity_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_special_count then
				print_builtin_special_count_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_special_element_size then
				print_builtin_special_element_size_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_special_item then
				print_builtin_special_item_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_tuple_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "TUPLE".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_tuple: a_feature.is_builtin_tuple_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_tuple_count then
				print_builtin_tuple_count_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_tuple_object_comparison then
				print_builtin_tuple_object_comparison_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

	print_builtin_type_query_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call to query `a_feature' (static binding).
			-- `a_feature' is a built-in feature introduced in class "TYPE".
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			-- Note that the result of the query is not adapted to match the kind
			-- of result type expected by the caller. It is recommended to use
			-- `print_adapted_query_call' whenever possible.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_query: a_feature.is_query
			a_feature_is_builtin: a_feature.is_builtin
			a_feature_is_builtin_type: a_feature.is_builtin_type_class
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			inspect a_feature.builtin_feature_code
			when {ET_TOKEN_CODES}.builtin_type_default then
				print_builtin_type_default_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_type_generic_parameter_count then
				print_builtin_type_generic_parameter_count_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_type_has_default then
				print_builtin_type_has_default_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_type_is_attached then
				print_builtin_type_is_attached_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_type_is_deferred then
				print_builtin_type_is_deferred_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_type_is_expanded then
				print_builtin_type_is_expanded_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_type_type_id then
				print_builtin_type_type_id_call (a_feature, a_target_type, a_check_void_target)
			when {ET_TOKEN_CODES}.builtin_type_runtime_name then
				print_builtin_type_runtime_name_call (a_feature, a_target_type, a_check_void_target)
			else
				print_non_inlined_query_call (a_feature, a_target_type, a_check_void_target)
			end
		end

feature {NONE} -- Agent generation

	print_agent (an_agent: ET_AGENT)
			-- Print `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		local
			l_temp: detachable ET_IDENTIFIER
			l_temp_index: INTEGER
			i, nb: INTEGER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_assignment_target: like assignment_target
			l_agent_type: ET_DYNAMIC_PRIMARY_TYPE
			nb_operands: INTEGER
			l_operand: ET_EXPRESSION
			l_operand_type_set: ET_DYNAMIC_TYPE_SET
		do
			l_assignment_target := assignment_target
			assignment_target := Void
			l_dynamic_type_set := dynamic_type_set (an_agent)
			l_agent_type := l_dynamic_type_set.static_type.primary_type
			if not an_agent.is_qualified_call then
				print_attachment_operand (tokens.current_keyword, current_type)
				nb_operands := 1
			elseif attached {ET_EXPRESSION} an_agent.target as l_closed_operand then
				print_attachment_operand (l_closed_operand, dynamic_type_set (l_closed_operand))
				nb_operands := 1
			end
			if attached an_agent.arguments as l_arguments then
				nb := l_arguments.count
				from i := 1 until i > nb loop
					if attached {ET_EXPRESSION} l_arguments.actual_argument (i) as l_closed_operand then
						print_attachment_operand (l_closed_operand, dynamic_type_set (l_closed_operand))
						nb_operands := nb_operands + 1
					end
					i := i + 1
				end
			end
			fill_call_operands (nb_operands)
			current_agents.force_last (an_agent)
			if in_operand then
				print_indentation
				if l_assignment_target /= Void then
					operand_stack.force (l_assignment_target)
					print_writable (l_assignment_target)
				else
					l_temp := new_temp_variable (l_agent_type)
						-- We will set the index of `l_temp' later because
						-- it could still be used in `call_operands'.
					l_temp_index := an_agent.index
					operand_stack.force (l_temp)
					print_temp_name (l_temp, current_file)
				end
				print_assign_to
			end
			print_agent_creation_name (current_agents.count, current_feature, current_type, current_file)
			current_file.put_character ('(')
			current_file.put_string (c_ac)
			from i := 1 until i > nb_operands loop
				print_comma
				l_operand := call_operands.item (i)
				l_operand_type_set := dynamic_type_set (l_operand)
					-- Note that when some closed operands have the copy semantics, they will be cloned
					-- twice: once when creating the agent (here) because they will be used to create
					-- the tuple `ROUTINE.closed_operands`, and once when calling the agent because they
					-- will be used as actual arguments of the underlying feature.
				print_attachment_expression (l_operand, l_operand_type_set, l_operand_type_set.static_type)
				i := i + 1
			end
			current_file.put_character (')')
			if in_operand then
				print_semicolon_newline
			end
			call_operands.wipe_out
			if l_temp /= Void and then l_temp_index /= 0 then
					-- We had to wait until this stage to set the index of `l_temp'
					-- because it could have still been used in `call_operands'.
				l_temp.set_index (l_temp_index)
			end
		end

	print_call_agent (an_agent: ET_CALL_AGENT)
			-- Print `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
			print_agent (an_agent)
		end

	print_do_function_inline_agent (an_agent: ET_DO_FUNCTION_INLINE_AGENT)
			-- Print `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
			print_agent (an_agent)
		end

	print_do_procedure_inline_agent (an_agent: ET_DO_PROCEDURE_INLINE_AGENT)
			-- Print `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
			print_agent (an_agent)
		end

	print_external_function_inline_agent (an_agent: ET_EXTERNAL_FUNCTION_INLINE_AGENT)
			-- Print `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
			print_agent (an_agent)
		end

	print_external_procedure_inline_agent (an_agent: ET_EXTERNAL_PROCEDURE_INLINE_AGENT)
			-- Print `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
			print_agent (an_agent)
		end

	print_once_function_inline_agent (an_agent: ET_ONCE_FUNCTION_INLINE_AGENT)
			-- Print `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
			if an_agent.is_once_per_object then
error_handler.report_warning_message ("ET_C_GENERATOR.print_once_function_inline_agent: once key %"OBJECT%" not supported.")
			end
			print_agent (an_agent)
		end

	print_once_procedure_inline_agent (an_agent: ET_ONCE_PROCEDURE_INLINE_AGENT)
			-- Print `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
			if an_agent.is_once_per_object then
error_handler.report_warning_message ("ET_C_GENERATOR.print_once_procedure_inline_agent: once key %"OBJECT%" not supported.")
			end
			print_agent (an_agent)
		end

	print_agent_open_operand (a_name: ET_IDENTIFIER)
			-- Print agent open operand `a_name'.
		require
			a_name_not_void: a_name /= Void
			a_name_open_operand: a_name.is_agent_open_operand
		local
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_static_type: ET_DYNAMIC_TYPE
			l_agent: like current_agent
		do
			if in_operand then
				if in_call_target and then a_name = agent_target then
					print_agent_open_target_operand (a_name)
				else
					operand_stack.force (a_name)
				end
			else
					-- Make sure that formal arguments are interpreted in the context
					-- of `current_feature' and not in the context of `current_agent'.
					-- For that we temporarily set `current_agent' to Void.
					-- It's reset to its original value when existing this routine.
				l_agent := current_agent
				current_agent := Void
				if attached call_target_type as l_call_target_type then
					check in_call_target: in_call_target end
					l_dynamic_type_set := dynamic_type_set (a_name)
					l_static_type := l_dynamic_type_set.static_type
					if l_static_type.is_expanded then
							-- Pass the address of the expanded object.
						current_file.put_character ('&')
						print_agent_open_operand_access (a_name)
					elseif l_call_target_type.is_expanded then
							-- We need to unbox the object and then pass its address.
						print_boxed_attribute_pointer_access (a_name, l_call_target_type, call_target_check_void)
					else
						print_agent_open_operand_access (a_name)
					end
				else
					print_agent_open_operand_access (a_name)
				end
				current_agent := l_agent
			end
		end

	print_agent_closed_operand (a_name: ET_IDENTIFIER)
			-- Print agent closed operand `a_name'.
		require
			a_name_not_void: a_name /= Void
			a_name_closed_operand: a_name.is_agent_closed_operand
		local
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_static_type: ET_DYNAMIC_TYPE
			l_agent: like current_agent
		do
				-- Make sure that formal arguments are interpreted in the context
				-- of `current_feature' and not in the context of `current_agent'.
				-- For that we temporarily set `current_agent' to Void.
				-- It's reset to its original value when existing this routine.
			l_agent := current_agent
			current_agent := Void
			if in_operand then
				operand_stack.force (a_name)
			elseif attached call_target_type as l_call_target_type then
				check in_call_target: in_call_target end
				l_dynamic_type_set := dynamic_type_set (a_name)
				l_static_type := l_dynamic_type_set.static_type
				if l_static_type.is_expanded then
						-- Pass the address of the expanded object.
					current_file.put_character ('&')
					print_agent_closed_operand_access (a_name, formal_argument (1))
				elseif l_call_target_type.is_expanded then
						-- We need to unbox the object and then pass its address.
					print_boxed_attribute_pointer_access (a_name, l_call_target_type, call_target_check_void)
				else
					print_agent_closed_operand_access (a_name, formal_argument (1))
				end
			else
				print_agent_closed_operand_access (a_name, formal_argument (1))
			end
			current_agent := l_agent
		end

	print_agent_open_target_operand (a_name: ET_IDENTIFIER)
			-- Print agent open operand `a_name' when it's the target
			-- of the call to the feature of the agent.
		require
			a_name_not_void: a_name /= Void
			a_name_open_operand: a_name.is_agent_open_operand
			in_operand: in_operand
			in_call_target: in_call_target
			is_agent_target: a_name = agent_target
		local
			l_tuple: ET_IDENTIFIER
			l_old_index: INTEGER
			l_agent: like current_agent
			l_tuple_type_set: ET_DYNAMIC_STANDALONE_TYPE_SET
			l_tuple_label: ET_IDENTIFIER
			l_tuple_item_expression: ET_QUALIFIED_CALL_EXPRESSION
			l_query_call: detachable ET_DYNAMIC_QUALIFIED_QUERY_CALL
			l_tuple_item_type_set: ET_DYNAMIC_TYPE_SET
			l_tuple_item_static_type: ET_DYNAMIC_PRIMARY_TYPE
		do
				-- Make sure that formal arguments are interpreted in the context
				-- of `current_feature' and not in the context of `current_agent'.
				-- For that we temporarily set `current_agent' to Void.
				-- It's reset to its original value when existing this routine.
			l_agent := current_agent
			current_agent := Void
				-- We need to use the  first item of the tuple containing the open operands
				-- as target of the call, not a clone of this object even if it has copy semantics.
				-- Extract it from the tuple containing the open operands, if any.
			l_tuple_item_type_set := dynamic_type_set (a_name)
			if
				l_agent /= Void and then
				attached {ET_DYNAMIC_ROUTINE_TYPE} dynamic_type_set (l_agent).static_type.primary_type as l_agent_type and then
				attached l_agent_type.open_operand_tuple_type_set as l_open_operand_tuple_type_set and then
				attached call_target_type as l_call_target_type and then
				l_tuple_item_type_set.has_expanded
			then
				l_tuple := formal_argument (2)
				l_tuple_type_set := new_standalone_type_set (l_open_operand_tuple_type_set.static_type)
				l_tuple_type_set.put_types (l_open_operand_tuple_type_set)
				l_tuple_type_set.set_never_void
				extra_dynamic_type_sets.force_last (l_tuple_type_set)
				l_old_index := l_tuple.index
				l_tuple.set_index (current_dynamic_type_sets.count + extra_dynamic_type_sets.count)
				print_indentation_if
				print_argument_name (l_tuple, current_file)
				print_then_newline
				indent
				l_tuple_label := new_tuple_label (1)
				l_tuple_item_expression := new_qualified_call_expression (l_tuple, l_tuple_label, Void)
				l_tuple_item_expression.set_index (a_name.index)
					-- Register the call to extract the tuple item so that
					-- it is handled correctly in case of polymorphism.
					-- The special treatment for polymorphism only occurs
					-- when the target has more than 2 possible dynamic types.
				if l_tuple_type_set.count > 2 then
					create l_query_call.make (l_tuple_item_expression, l_tuple_type_set, l_tuple_item_type_set, current_feature, current_type)
					l_tuple_type_set.static_type.primary_type.put_query_call (l_query_call)
				end
				if l_agent_type.result_type_set = Void then
					print_procedure_target_operand (l_tuple_item_expression, l_call_target_type)
				else
					print_target_operand (l_tuple_item_expression, l_call_target_type)
				end
					-- Clean up.
				l_tuple.set_index (l_old_index)
				extra_dynamic_type_sets.remove_last
				if l_query_call = Void then
					free_qualified_call_expression (l_tuple_item_expression)
					free_tuple_label (l_tuple_label)
					free_standalone_type_set (l_tuple_type_set)
				end
				dedent
				print_indentation_else_newline
				indent
				if
					attached {ET_IDENTIFIER} operand_stack.item as l_temp and then
					l_temp.is_temporary and then
					is_temp_variable_known (l_temp) and then
					attached temp_variable_type (l_temp) as l_temp_type
				then
					print_indentation
					print_temp_name (l_temp, current_file)
					print_assign_to
					l_tuple_item_static_type := l_tuple_item_type_set.static_type.primary_type
					if l_temp_type.is_expanded then
						if l_tuple_item_static_type.is_expanded then
							print_agent_open_operand_access (a_name)
						else
								-- We need to unbox the object.
							print_boxed_attribute_item_access (a_name, l_tuple_item_static_type, call_target_check_void)
						end
					else
						if l_tuple_item_static_type.is_expanded then
								-- We need to box the object.
							print_boxed_expression (a_name, l_tuple_item_static_type)
						else
								-- Already boxed.
							print_agent_open_operand_access (a_name)
						end
					end
					print_semicolon_newline
				else
						-- Internal error: a temporary variable should have properly been declared
						-- when printing `l_tuple_item_expression'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				dedent
				print_indentation_end_newline
			else
				operand_stack.force (a_name)
			end
			current_agent := l_agent
		end

	print_agent_open_operand_access (a_name: ET_IDENTIFIER)
			-- Print to `current_file' an access to agent open operand `a_name'.
		require
			a_name_not_void: a_name /= Void
			a_name_open_operand: a_name.is_agent_open_operand
		do
				-- The first two formal arguments are the tuple containing the closed
				-- operands and the tuple containing the open operands (if the latter
				-- is not a manifest tupe and is not empty, null otherwise).
			print_argument_name (formal_argument (a_name.seed + 2), current_file)
		end

	print_agent_closed_operand_access (a_name: ET_IDENTIFIER; a_target: ET_EXPRESSION)
			-- Print to `current_file' an access to agent closed operand `a_name' applied to `a_target'.
			-- `a_target' is meant to be the Tuple corresponding to the attribute 'closed_operands'
			-- in the Agent type.
		require
			a_name_not_void: a_name /= Void
			a_name_closed_operand: a_name.is_agent_closed_operand
			a_target_not_void: a_target /= Void
		do
			print_attribute_tuple_item_access (a_name.seed, a_target, agent_closed_operands_type, False)
		end

	print_agent_declaration (i: INTEGER; an_agent: ET_AGENT)
			-- Print declaration of `i'-th agent `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		local
			l_open_operand: ET_IDENTIFIER
			l_closed_operand: ET_IDENTIFIER
			l_open_index: INTEGER
			l_closed_index: INTEGER
			nb_open_operands: INTEGER
			nb_closed_operands: INTEGER
			l_target: ET_AGENT_TARGET
			l_result: detachable ET_RESULT
			l_arguments: detachable ET_AGENT_ARGUMENT_OPERANDS
			l_argument: ET_AGENT_ARGUMENT_OPERAND
			j, nb: INTEGER
			l_result_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_type: ET_DYNAMIC_PRIMARY_TYPE
			old_closed_operands_type: ET_DYNAMIC_PRIMARY_TYPE
			l_parameters: ET_ACTUAL_PARAMETER_LIST
			l_tuple_type: ET_TUPLE_TYPE
			old_file: KI_TEXT_OUTPUT_STREAM
		do
				--
				-- Determine agent type.
				--
			if not attached {ET_DYNAMIC_ROUTINE_TYPE} dynamic_type_set (an_agent).static_type.primary_type as l_agent_type then
					-- Internal error: the type of `an_agent' should be an Agent type.
				set_fatal_error
				error_handler.report_giaaa_error
			else
					--
					-- Determine open and closed operands.
					--
				l_target := an_agent.target
				l_result := an_agent.implicit_result
				if l_target.is_open_operand then
					nb_open_operands := 1
					if agent_open_operands.count < nb_open_operands then
						create l_open_operand.make ("a" + nb_open_operands.out)
						l_open_operand.set_agent_open_operand (True)
						l_open_operand.set_seed (nb_open_operands)
						agent_open_operands.force_last (l_open_operand)
					else
						l_open_operand := agent_open_operands.item (nb_open_operands)
					end
					l_open_operand.set_index (l_target.index)
				else
					nb_closed_operands := 1
					if agent_closed_operands.count < nb_closed_operands then
						create l_closed_operand.make ("z" + nb_closed_operands.out)
						l_closed_operand.set_agent_closed_operand (True)
						l_closed_operand.set_seed (nb_closed_operands)
						agent_closed_operands.force_last (l_closed_operand)
					else
						l_closed_operand := agent_closed_operands.item (nb_closed_operands)
					end
					l_closed_operand.set_index (l_target.index)
				end
				l_arguments := an_agent.arguments
				if l_arguments /= Void then
					nb := l_arguments.count
					from j := 1 until j > nb loop
						l_argument := l_arguments.actual_argument (j)
						if l_argument.is_open_operand then
							nb_open_operands := nb_open_operands + 1
							if agent_open_operands.count < nb_open_operands then
								create l_open_operand.make ("a" + nb_open_operands.out)
								l_open_operand.set_agent_open_operand (True)
								l_open_operand.set_seed (nb_open_operands)
								agent_open_operands.force_last (l_open_operand)
							else
								l_open_operand := agent_open_operands.item (nb_open_operands)
							end
							l_open_operand.set_index (l_argument.index)
						else
							nb_closed_operands := nb_closed_operands + 1
							if agent_closed_operands.count < nb_closed_operands then
								create l_closed_operand.make ("z" + nb_closed_operands.out)
								l_closed_operand.set_agent_closed_operand (True)
								l_closed_operand.set_seed (nb_closed_operands)
								agent_closed_operands.force_last (l_closed_operand)
							else
								l_closed_operand := agent_closed_operands.item (nb_closed_operands)
							end
							l_closed_operand.set_index (l_argument.index)
						end
						j := j + 1
					end
				end
				old_closed_operands_type := agent_closed_operands_type
				create l_parameters.make_with_capacity (nb_closed_operands)
				from j := nb_closed_operands until j < 1 loop
					l_parameters.put_first (dynamic_type_set (agent_closed_operands.item (j)).static_type.base_type)
					j := j - 1
				end
				create l_tuple_type.make (tokens.implicit_attached_type_mark, l_parameters, l_agent_type.base_class.universe.tuple_type.named_base_class)
				agent_closed_operands_type := current_dynamic_system.dynamic_primary_type (l_tuple_type, current_system.any_type)
					--
					-- Print function associated with the agent to `current_file'.
					--
				old_file := current_file
				current_file := current_function_header_buffer
				current_file.put_character ('/')
				current_file.put_character ('*')
				current_file.put_character (' ')
				current_file.put_string ("Function for agent #")
				current_file.put_integer (i)
				current_file.put_string (" in feature ")
				current_file.put_string (current_type.base_type.unaliased_to_text)
				current_file.put_character ('.')
				current_file.put_string (STRING_.replaced_all_substrings (current_feature.static_feature.lower_name, "*/", "star/"))
				current_file.put_character (' ')
				current_file.put_character ('*')
				current_file.put_character ('/')
				current_file.put_new_line
				if l_result = Void then
						-- Procedure.
					current_file.put_string (c_void)
				else
						-- Query or Tuple label.
					l_result_type := dynamic_type_set (l_result).static_type.primary_type
					print_type_declaration (l_result_type, current_file)
				end
				current_file.put_character (' ')
				print_agent_function_name (i, current_feature, current_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ge_context)
				current_file.put_character ('*')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
					-- Tuple with closed operands.
				print_type_declaration (agent_closed_operands_type, current_file)
				current_file.put_character (' ')
				print_argument_name (formal_argument (1), current_file)
				current_file.put_character (',')
				current_file.put_character (' ')
					-- Tuple with open operands, if any.
					-- Null if manifest tuple or empty tuple.
				if attached l_agent_type.open_operand_tuple_type_set as l_open_operand_tuple_type_set then
					print_type_declaration (l_open_operand_tuple_type_set.static_type.primary_type, current_file)
				else
					print_eif_any_type_name (current_file)
					current_file.put_character ('*')
				end
				current_file.put_character (' ')
				print_argument_name (formal_argument (2), current_file)
				from j := 1 until j > nb_open_operands loop
					l_open_operand := agent_open_operands.item (j)
					l_type := dynamic_type_set (l_open_operand).static_type.primary_type
					current_file.put_character (',')
					current_file.put_character (' ')
					print_type_declaration (l_type, current_file)
					current_file.put_character (' ')
					print_agent_open_operand_access (l_open_operand)
					j := j + 1
				end
				current_file.put_character (')')
				current_file.put_new_line
				current_file.put_character ('{')
				current_file.put_new_line
				indent
					--
					-- Declaration of variables.
					--
					-- Variable for exception trace.
				if exception_trace_mode then
					print_indentation
					current_file.put_string (c_ge_call)
					current_file.put_character (' ')
					current_file.put_string (c_tc)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('{')
					if current_in_exception_trace then
						print_current_name (current_file)
						current_file.put_character (',')
					end
					if l_result /= Void then
						print_escaped_string (tokens.capitalized_function_name)
						current_file.put_character (',')
						print_escaped_string (tokens.item_name)
					else
						print_escaped_string (tokens.capitalized_procedure_name)
						current_file.put_character (',')
						print_escaped_string (tokens.call_name)
					end
					current_file.put_character (',')
					current_file.put_string (c_ac)
					current_file.put_string (c_arrow)
					current_file.put_string (c_call)
					current_file.put_character ('}')
					current_file.put_character (';')
					current_file.put_new_line
				end
					-- Make sure that `agent_target' is set correctly
					-- before calling `print_agent_body_declaration'.
				if l_target.is_open_operand then
					agent_target := agent_open_operands.item (1)
				else
					agent_target := agent_closed_operands.item (1)
				end
					-- Make sure that `agent_arguments' is set correctly
					-- before calling `print_agent_body_declaration'.
				agent_arguments.wipe_out
				if l_arguments /= Void then
					nb := l_arguments.count
					if agent_arguments.capacity < nb then
						agent_arguments.resize (nb)
					end
					l_open_index := nb_open_operands
					l_closed_index := nb_closed_operands
					from j := nb until j < 1 loop
						l_argument := l_arguments.actual_argument (j)
						if l_argument.is_open_operand then
							agent_arguments.put_first (agent_open_operands.item (l_open_index))
							l_open_index := l_open_index - 1
						else
							agent_arguments.put_first (agent_closed_operands.item (l_closed_index))
							l_closed_index := l_closed_index - 1
						end
						j := j - 1
					end
				end
					--
					-- Instructions.
					--
				current_file := current_function_body_buffer
					-- Call stack.
				if exception_trace_mode then
					print_indentation
					current_file.put_string (c_ac)
					current_file.put_string (c_arrow)
					current_file.put_string (c_call)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_tc_address)
					current_file.put_character (';')
					current_file.put_new_line
				end
				print_agent_body_declaration (an_agent)
					-- Call stack.
				if exception_trace_mode then
					print_indentation
					current_file.put_string (c_ac)
					current_file.put_string (c_arrow)
					current_file.put_string (c_call)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_tc)
					current_file.put_character ('.')
					current_file.put_string (c_caller)
					current_file.put_character (';')
					current_file.put_new_line
				end
				if l_result_type /= Void then
					print_indentation
					current_file.put_string (c_return)
					current_file.put_character (' ')
					print_result_name (current_file)
					current_file.put_character (';')
					current_file.put_new_line
				end
				dedent
				current_file.put_character ('}')
				current_file.put_new_line
				current_file.put_new_line
				current_file := old_file
					-- Do not flush code to C file here (no call to `flush_to_c_file')
					-- because we want the function associated with this agent and the
					-- creation of the agent (printed just below) to be in the same C file.
					-- Indeed, the associated function is not declared in the C header file,
					-- but it is used in the creation of the agent.
				reset_temp_variables
					--
					-- Print creation function of the agent.
					--
					-- Print signature to `header_file' and `current_file'.
				header_file.put_character ('/')
				header_file.put_character ('*')
				header_file.put_character (' ')
				header_file.put_string ("Creation of agent #")
				header_file.put_integer (i)
				header_file.put_string (" in feature ")
				header_file.put_string (current_type.base_type.unaliased_to_text)
				header_file.put_character ('.')
				header_file.put_string (STRING_.replaced_all_substrings (current_feature.static_feature.lower_name, "*/", "star/"))
				header_file.put_character (' ')
				header_file.put_character ('*')
				header_file.put_character ('/')
				header_file.put_new_line
				current_file.put_character ('/')
				current_file.put_character ('*')
				current_file.put_character (' ')
				current_file.put_string ("Creation of agent #")
				current_file.put_integer (i)
				current_file.put_string (" in feature ")
				current_file.put_string (current_type.base_type.unaliased_to_text)
				current_file.put_character ('.')
				current_file.put_string (STRING_.replaced_all_substrings (current_feature.static_feature.lower_name, "*/", "star/"))
				current_file.put_character (' ')
				current_file.put_character ('*')
				current_file.put_character ('/')
				current_file.put_new_line
				header_file.put_string (c_extern)
				header_file.put_character (' ')
				print_type_declaration (l_agent_type, header_file)
				print_type_declaration (l_agent_type, current_file)
				header_file.put_character (' ')
				current_file.put_character (' ')
				print_agent_creation_name (i, current_feature, current_type, header_file)
				print_agent_creation_name (i, current_feature, current_type, current_file)
				header_file.put_character ('(')
				current_file.put_character ('(')
				header_file.put_string (c_ge_context)
				current_file.put_string (c_ge_context)
				header_file.put_character ('*')
				current_file.put_character ('*')
				header_file.put_character (' ')
				current_file.put_character (' ')
				header_file.put_string (c_ac)
				current_file.put_string (c_ac)
				from j := 1 until j > nb_closed_operands loop
					header_file.put_character (',')
					header_file.put_character (' ')
					current_file.put_character (',')
					current_file.put_character (' ')
					l_type := dynamic_type_set (agent_closed_operands.item (j)).static_type.primary_type
					print_type_declaration (l_type, header_file)
					print_type_declaration (l_type, current_file)
					header_file.put_character (' ')
					current_file.put_character (' ')
					header_file.put_character ('a')
					current_file.put_character ('a')
					header_file.put_integer (j)
					current_file.put_integer (j)
					j := j + 1
				end
				header_file.put_character (')')
				current_file.put_character (')')
				header_file.put_character (';')
				header_file.put_new_line
				current_file.put_new_line
					-- Print body to `current_file'.
				current_file.put_character ('{')
				current_file.put_new_line
				indent
				print_indentation
				print_type_declaration (l_agent_type, current_file)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				print_type_declaration (agent_closed_operands_type, current_file)
				current_file.put_character (' ')
				print_temp_name (temp_variable, current_file)
				current_file.put_character (';')
				current_file.put_new_line
				extra_dynamic_type_sets.force_last (agent_closed_operands_type)
				temp_variable.set_index (current_dynamic_type_sets.count + extra_dynamic_type_sets.count)
					-- Create agent object.
				print_indentation
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_string (c_ge_new)
				current_file.put_integer (l_agent_type.id)
				current_file.put_character ('(')
				current_file.put_string (c_eif_true)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
					-- Build 'closed_operands'.
				print_indentation
				print_temp_name (temp_variable, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_string (c_ge_new)
				current_file.put_integer (agent_closed_operands_type.id)
				current_file.put_character ('(')
				current_file.put_string (c_eif_true)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				from j := 1 until j > nb_closed_operands loop
					print_indentation
					print_agent_closed_operand_access (agent_closed_operands.item (j), temp_variable)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('a')
					current_file.put_integer (j)
					current_file.put_character (';')
					current_file.put_new_line
					j := j + 1
				end
					-- Call 'set_rout_disp_final'.
				if not attached l_agent_type.set_rout_disp_final_feature as l_set_rout_disp_final_feature then
						-- Internal error: the Agent type should have a procedure 'set_rout_disp_final'.
						-- See ET_DYNAMIC_SYSTEM.compile_kernel.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					register_called_feature (l_set_rout_disp_final_feature)
					print_indentation
					print_routine_name (l_set_rout_disp_final_feature, l_agent_type, current_file)
					current_file.put_character ('(')
					current_file.put_string (c_ac)
					current_file.put_character (',')
					current_file.put_character (' ')
					if l_agent_type.is_expanded then
						current_file.put_character ('&')
						print_result_name (current_file)
					else
						print_result_name (current_file)
					end
					current_file.put_character (',')
					current_file.put_character (' ')
						-- 'a_rout_disp'.
					print_declaration_type_cast (current_dynamic_system.pointer_type, current_file)
					current_file.put_character ('&')
					print_agent_function_name (i, current_feature, current_type, current_file)
					current_file.put_character (',')
					current_file.put_character (' ')
						-- 'a_encaps_rout_disp'.
					print_declaration_type_cast (current_dynamic_system.pointer_type, current_file)
					current_file.put_character ('&')
					print_agent_function_name (i, current_feature, current_type, current_file)
					current_file.put_character (',')
					current_file.put_character (' ')
						-- 'a_calc_rout_addr'.
					print_declaration_type_cast (current_dynamic_system.pointer_type, current_file)
					current_file.put_character ('0')
					current_file.put_character (',')
					current_file.put_character (' ')
						-- 'a_closed_operands'.
					print_temp_name (temp_variable, current_file)
					current_file.put_character (',')
					current_file.put_character (' ')
						-- 'a_is_target_closed'.
					if l_target.is_open_operand then
						current_file.put_string (c_eif_false)
					else
						current_file.put_string (c_eif_true)
					end
					current_file.put_character (',')
					current_file.put_character (' ')
						-- 'a_open_count'.
					current_file.put_integer (nb_open_operands)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				end
				extra_dynamic_type_sets.remove_last
					-- Return the agent object.
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				dedent
				current_file.put_character ('}')
				current_file.put_new_line
				current_file.put_new_line
				flush_to_c_file
				agent_closed_operands_type := old_closed_operands_type
			end
		end

	print_agent_body_declaration (an_agent: ET_AGENT)
			-- Print body of declaration of agent `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		local
			l_old_agent: like current_agent
		do
			l_old_agent := current_agent
			current_agent := an_agent
			an_agent.process (Current)
			current_agent := l_old_agent
		end

	print_call_agent_body_declaration (an_agent: ET_CALL_AGENT)
			-- Print body of declaration of call agent `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		local
			l_result: detachable ET_RESULT
			l_arguments: detachable ET_AGENT_ARGUMENT_OPERANDS
			l_name: ET_FEATURE_NAME
			l_result_type_set: ET_DYNAMIC_TYPE_SET
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
			old_file: KI_TEXT_OUTPUT_STREAM
		do
			old_file := current_file
			l_name := an_agent.name
			l_arguments := an_agent.arguments
			l_result := an_agent.implicit_result
			if l_result /= Void then
					-- Query or Tuple label.
				l_result_type_set := dynamic_type_set (l_result)
				current_file := current_function_header_buffer
				l_result_type := l_result_type_set.static_type.primary_type
				print_indentation
				print_type_declaration (l_result_type, current_file)
				current_file.put_character (' ')
				print_result_name (current_file)
				print_semicolon_newline
				current_file := current_function_body_buffer
				agent_expression.set_target (agent_target)
				agent_expression.set_name (l_name)
				if l_arguments /= Void then
					agent_expression.set_arguments (agent_arguments)
				else
					agent_expression.set_arguments (Void)
				end
				agent_expression.set_index (l_result.index)
					-- Note that we don't call `print_attachment_operand' or
					-- `print_assignment_operand' here because we don't want
					-- the result of the call to the feature `l_name' to be
					-- cloned (if it has the copy semantics) at this stage
					-- (e.g. `agent a.some_function_with_expanded_result`).
					-- This will be done later if the result is assigned to
					-- some entity (e.g. `x := my_agent.item ([])`).
				print_operand (agent_expression)
				fill_call_operands (1)
				print_indentation_assign_to_result
				print_expression (call_operands.first)
				print_semicolon_newline
				call_operands.wipe_out
			else
					-- Procedure.
				current_file := current_function_body_buffer
				agent_instruction.set_target (agent_target)
				agent_instruction.set_name (l_name)
				if l_arguments /= Void then
					agent_instruction.set_arguments (agent_arguments)
				else
					agent_instruction.set_arguments (Void)
				end
				print_qualified_call_instruction (agent_instruction)
			end
				-- Clean up.
			current_file := old_file
		end

	print_do_function_inline_agent_body_declaration (an_agent: ET_DO_FUNCTION_INLINE_AGENT)
			-- Print body of declaration of inline agent `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
			print_internal_routine_inline_agent_body_declaration (an_agent)
		end

	print_do_procedure_inline_agent_body_declaration (an_agent: ET_DO_PROCEDURE_INLINE_AGENT)
			-- Print body of declaration of inline agent `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
			print_internal_routine_inline_agent_body_declaration (an_agent)
		end

	print_external_function_inline_agent_body_declaration (an_agent: ET_EXTERNAL_FUNCTION_INLINE_AGENT)
			-- Print body of declaration of inline agent `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
-- TODO
error_handler.report_warning_message ("ET_C_GENERATOR.print_external_function_inline_agent_body_declaration not implemented")
		end

	print_external_procedure_inline_agent_body_declaration (an_agent: ET_EXTERNAL_PROCEDURE_INLINE_AGENT)
			-- Print body of declaration of inline agent `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
-- TODO
error_handler.report_warning_message ("ET_C_GENERATOR.print_external_procedure_inline_agent_body_declaration not implemented")
		end

	print_once_function_inline_agent_body_declaration (an_agent: ET_ONCE_FUNCTION_INLINE_AGENT)
			-- Print body of declaration of inline agent `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
-- TODO
error_handler.report_warning_message ("ET_C_GENERATOR.print_once_function_inline_agent_body_declaration not implemented")
			print_internal_routine_inline_agent_body_declaration (an_agent)
		end

	print_once_procedure_inline_agent_body_declaration (an_agent: ET_ONCE_PROCEDURE_INLINE_AGENT)
			-- Print body of declaration of inline agent `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		do
-- TODO
error_handler.report_warning_message ("ET_C_GENERATOR.print_once_procedure_inline_agent_body_declaration not implemented")
			print_internal_routine_inline_agent_body_declaration (an_agent)
		end

	print_internal_routine_inline_agent_body_declaration (an_agent: ET_INTERNAL_ROUTINE_INLINE_AGENT)
			-- Print body of declaration of inline agent `an_agent'.
		require
			an_agent_not_void: an_agent /= Void
		local
			l_result: detachable ET_RESULT
			l_result_type_set: ET_DYNAMIC_TYPE_SET
			l_result_type: detachable ET_DYNAMIC_PRIMARY_TYPE
		do
-- TODO: handle case of once-routines
			print_agent_trace_message_call (an_agent, True)
			l_result := an_agent.implicit_result
			if l_result /= Void then
				l_result_type_set := dynamic_type_set (l_result)
				l_result_type := l_result_type_set.static_type.primary_type
			end
			print_internal_feature_body_declaration (an_agent, l_result_type)
			print_agent_trace_message_call (an_agent, False)
		end

	agent_instruction: ET_QUALIFIED_CALL_INSTRUCTION
			-- Agent instruction

	agent_expression: ET_QUALIFIED_CALL_EXPRESSION
			-- Agent expression

	agent_target: ET_EXPRESSION
			-- Agent target

	agent_arguments: ET_ACTUAL_ARGUMENT_LIST
			-- Agent arguments

	agent_open_operands: DS_ARRAYED_LIST [ET_IDENTIFIER]
			-- Agent open operands

	agent_closed_operands: DS_ARRAYED_LIST [ET_IDENTIFIER]
			-- Agent closed operands

	agent_closed_operands_type: ET_DYNAMIC_PRIMARY_TYPE
			-- Type of attribute 'closed_operands' in Agent type

feature {NONE} -- Polymorphic call functions generation

	print_polymorphic_query_call_functions
			-- Print polymorphic query call functions.
		local
			l_dynamic_types: DS_ARRAYED_LIST [ET_DYNAMIC_PRIMARY_TYPE]
			l_type: ET_DYNAMIC_PRIMARY_TYPE
			l_first_call: detachable ET_DYNAMIC_QUALIFIED_QUERY_CALL
			l_last_call: ET_DYNAMIC_QUALIFIED_QUERY_CALL
			l_call: detachable ET_DYNAMIC_QUALIFIED_QUERY_CALL
			l_previous_call: ET_DYNAMIC_QUALIFIED_QUERY_CALL
			l_seed: INTEGER
			i, nb: INTEGER
			l_static_call: ET_CALL_COMPONENT
			l_force_result_boxing: BOOLEAN
		do
			l_dynamic_types := dynamic_types
			nb := l_dynamic_types.count
			from i := 1 until i > nb loop
				l_type := l_dynamic_types.item (i)
				from
					l_first_call := l_type.query_calls
				until
					l_first_call = Void
				loop
					if not l_first_call.is_generated then
							-- It's not used in the generated C code.
							-- For example, polymorphic calls with one or two target
							-- dynamic types are handled differently.
							-- (See `print_qualified_call_expression'.)
						l_first_call := l_first_call.next
					elseif l_first_call.is_tuple_label then
							-- Polymorphic calls to Tuple labels are handled differently.
						l_first_call := l_first_call.next
					else
						l_static_call := l_first_call.static_call
						l_seed := l_static_call.name.seed
						from
							l_last_call := l_first_call
							l_previous_call := l_first_call
							l_call := l_first_call.next
							l_force_result_boxing := l_first_call.force_result_boxing
						until
							l_call = Void
						loop
							if
								l_call.is_generated and then
								(l_call.static_call.name.seed = l_seed and not l_call.is_tuple_label) and then
								l_force_result_boxing = l_call.force_result_boxing and then
								same_declared_signature (l_call, l_first_call)
							then
								if l_call /= l_last_call.next then
									l_previous_call.set_next (l_call.next)
									l_call.set_next (l_last_call.next)
									l_last_call.set_next (l_call)
									l_last_call := l_call
									l_call := l_previous_call
								else
									l_last_call := l_call
								end
							end
							l_previous_call := l_call
							l_call := l_call.next
						end
						print_polymorphic_call_function (l_first_call, l_last_call, l_type)
						l_first_call := l_last_call.next
					end
				end
				i := i + 1
			end
		end

	print_polymorphic_tuple_label_call_functions
			-- Print polymorphic call to Tuple label functions.
		local
			l_dynamic_types: DS_ARRAYED_LIST [ET_DYNAMIC_PRIMARY_TYPE]
			l_first_call: detachable ET_DYNAMIC_QUALIFIED_QUERY_CALL
			l_last_call: ET_DYNAMIC_QUALIFIED_QUERY_CALL
			l_call: detachable ET_DYNAMIC_QUALIFIED_QUERY_CALL
			l_previous_call: ET_DYNAMIC_QUALIFIED_QUERY_CALL
			l_seed: INTEGER
			i, nb: INTEGER
			l_static_call: ET_CALL_COMPONENT
			l_force_result_boxing: BOOLEAN
		do
			l_dynamic_types := dynamic_types
			nb := l_dynamic_types.count
			from i := 1 until i > nb loop
				if attached {ET_DYNAMIC_TUPLE_TYPE} l_dynamic_types.item (i) as l_tuple_type then
					from
						l_first_call := l_tuple_type.query_calls
					until
						l_first_call = Void
					loop
						if not l_first_call.is_generated then
								-- It's not used in the generated C code.
								-- For example, polymorphic calls with one or two target
								-- dynamic types are handled differently.
								-- (See `print_qualified_call_expression'.)
							l_first_call := l_first_call.next
						elseif not l_first_call.is_tuple_label then
								-- Skip calls that are not to a Tuple label.
							l_first_call := l_first_call.next
						else
							l_static_call := l_first_call.static_call
							l_seed := l_static_call.name.seed
							from
								l_last_call := l_first_call
								l_previous_call := l_first_call
								l_call := l_first_call.next
								l_force_result_boxing := l_first_call.force_result_boxing
							until
								l_call = Void
							loop
								if
									l_call.is_generated and then
									(l_call.static_call.name.seed = l_seed and l_call.is_tuple_label) and then
									l_force_result_boxing = l_call.force_result_boxing and then
									same_declared_signature (l_call, l_first_call)
								then
									if l_call /= l_last_call.next then
										l_previous_call.set_next (l_call.next)
										l_call.set_next (l_last_call.next)
										l_last_call.set_next (l_call)
										l_last_call := l_call
										l_call := l_previous_call
									else
										l_last_call := l_call
									end
								end
								l_previous_call := l_call
								l_call := l_call.next
							end
							print_polymorphic_call_function (l_first_call, l_last_call, l_tuple_type)
							l_first_call := l_last_call.next
						end
					end
				end
				i := i + 1
			end
		end

	print_polymorphic_procedure_call_functions
			-- Print polymorphic procedure call functions.
		local
			l_dynamic_types: DS_ARRAYED_LIST [ET_DYNAMIC_PRIMARY_TYPE]
			l_type: ET_DYNAMIC_PRIMARY_TYPE
			l_first_call: detachable ET_DYNAMIC_QUALIFIED_PROCEDURE_CALL
			l_last_call: ET_DYNAMIC_QUALIFIED_PROCEDURE_CALL
			l_call: detachable ET_DYNAMIC_QUALIFIED_PROCEDURE_CALL
			l_previous_call: ET_DYNAMIC_QUALIFIED_PROCEDURE_CALL
			l_seed: INTEGER
			i, nb: INTEGER
			l_static_call: ET_CALL_COMPONENT
			l_is_tuple_label: BOOLEAN
		do
			l_dynamic_types := dynamic_types
			nb := l_dynamic_types.count
			from i := 1 until i > nb loop
				l_type := l_dynamic_types.item (i)
				from
					l_first_call := l_type.procedure_calls
				until
					l_first_call = Void
				loop
					if not l_first_call.is_generated then
							-- It's not used in the generated C code.
							-- For example, polymorphic calls with one or two target
							-- dynamic types are handled differently.
							-- (See `print_qualified_call_instruction'.)
						l_first_call := l_first_call.next
					else
						l_static_call := l_first_call.static_call
						l_seed := l_static_call.name.seed
						l_is_tuple_label := l_static_call.is_tuple_label
						from
							l_last_call := l_first_call
							l_previous_call := l_first_call
							l_call := l_first_call.next
						until
							l_call = Void
						loop
							if
								l_call.is_generated and then
								(l_call.static_call.name.seed = l_seed and l_call.is_tuple_label = l_is_tuple_label) and then
								same_declared_signature (l_call, l_first_call)
							then
								if l_call /= l_last_call.next then
									l_previous_call.set_next (l_call.next)
									l_call.set_next (l_last_call.next)
									l_last_call.set_next (l_call)
									l_last_call := l_call
									l_call := l_previous_call
								else
									l_last_call := l_call
								end
							end
							l_previous_call := l_call
							l_call := l_call.next
						end
						print_polymorphic_call_function (l_first_call, l_last_call, l_type)
						l_first_call := l_last_call.next
					end
				end
				i := i + 1
			end
		end

	print_polymorphic_call_function (a_first_call, a_last_call: ET_DYNAMIC_QUALIFIED_CALL; a_target_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print to `current_file' dynamic binding code for the calls between `a_first_call'
			-- and `a_last_call' whose target static type is `a_target_type'.
			-- The generated code uses either a switch-statement or binary search to find out
			-- which feature to execute.
		require
			a_first_call_not_void: a_first_call /= Void
			a_last_call_not_void: a_last_call /= Void
			a_target_type_not_void: a_target_type /= Void
		local
			l_static_call: ET_CALL_COMPONENT
			l_caller: ET_DYNAMIC_FEATURE
			i, nb: INTEGER
			l_actual_arguments: detachable ET_ARGUMENT_OPERANDS
			l_formal_arguments_count: INTEGER
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_type: ET_DYNAMIC_PRIMARY_TYPE
			l_target_dynamic_type_ids: DS_ARRAYED_LIST [INTEGER]
			l_target_dynamic_types: DS_HASH_TABLE [ET_DYNAMIC_PRIMARY_TYPE, INTEGER]
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_switch: BOOLEAN
			l_temp: ET_IDENTIFIER
			l_name: ET_IDENTIFIER
			l_call, l_last_call_next: detachable ET_DYNAMIC_QUALIFIED_CALL
			l_target_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_type_sets: ET_DYNAMIC_STANDALONE_TYPE_SET_LIST
			l_formal_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_result_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_index: INTEGER
			l_feature: detachable ET_FEATURE
			l_seed: INTEGER
			l_manifest_tuple: detachable ET_MANIFEST_TUPLE
			l_manifest_tuple_operand: detachable ET_MANIFEST_TUPLE
			old_type: like current_type
			old_feature: like current_feature
			old_dynamic_type_sets: like current_dynamic_type_sets
			l_old_call_target_type: like call_target_type
			l_force_result_boxing: BOOLEAN
		do
			l_old_call_target_type := call_target_type
			call_target_type := Void
			old_type := current_type
			current_type := a_target_type
			old_feature := current_feature
			current_feature := dummy_feature
			old_dynamic_type_sets := current_dynamic_type_sets
			l_argument_type_sets := standalone_type_sets
			current_dynamic_type_sets := l_argument_type_sets
			l_static_call := a_first_call.static_call
			l_caller := a_first_call.current_feature
			l_actual_arguments := l_static_call.arguments
			if l_static_call.is_tuple_label then
				call_operands.wipe_out
				call_operands.force_last (tokens.current_keyword)
				l_index := l_static_call.name.seed
				if not attached {ET_DYNAMIC_TUPLE_TYPE} a_target_type as l_tuple_type then
						-- Internal error: a Tuple label can only be applied to a tuple object.
					set_fatal_error
					error_handler.report_giaaa_error
				elseif not l_tuple_type.item_type_sets.valid_index (l_index) then
						-- Internal error: invalid Tuple label.
					set_fatal_error
					error_handler.report_giaaa_error
				elseif l_actual_arguments /= Void and then not l_actual_arguments.is_empty then
						-- This is a Tuple label setter.
					l_formal_arguments_count := 1
					call_operands.force_last (formal_argument (1))
				else
						-- This is a Tuple label getter.
					l_result_type := l_tuple_type.item_type_sets.item (l_index).static_type.primary_type
				end
			else
				l_feature := a_first_call.static_feature
				if l_feature = Void then
					call_operands.wipe_out
					call_operands.force_last (tokens.current_keyword)
						-- Internal error: there should be a feature with that seed.
						-- It has been computed in ET_FEATURE_FLATTENER.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					if attached l_feature.type as l_feature_type then
						check is_query: l_feature.is_query end
						l_result_type := current_dynamic_system.dynamic_primary_type (l_feature_type, a_target_type.base_type)
					end
					l_formal_arguments := l_feature.arguments
					l_formal_arguments_count := l_feature.arguments_count
					fill_call_formal_arguments (l_feature)
						-- Check to see whether this is a call to ROUTINE.call or FUNCTION.item
						-- with a manifest tuple passed as argument. We have a special treatment
						-- in that case to avoid having to create the manifest tuple when possible.
					if l_formal_arguments_count = 1 then
						l_seed := l_static_call.name.seed
						if (l_seed = current_system.routine_call_seed or l_seed = current_system.function_item_seed) and then not l_static_call.is_call_agent then
							if l_actual_arguments /= Void and then l_actual_arguments.count = 1 then
								if attached {ET_MANIFEST_TUPLE} l_actual_arguments.actual_argument (1) as l_attached_manifest_tuple then
									l_manifest_tuple := l_attached_manifest_tuple
									nb := l_manifest_tuple.count
									l_manifest_tuple_operand := agent_manifest_tuple
									l_manifest_tuple_operand.resize (nb)
									from i := nb until i < 1 loop
										l_manifest_tuple_operand.put_first (formal_argument (i))
										i := i - 1
									end
									l_manifest_tuple_operand.set_index (nb + 1)
									call_operands.replace (l_manifest_tuple_operand, 2)
								end
							end
						end
					end
				end
			end
				-- Compute dynamic type sets.
			l_target_dynamic_type_ids := target_dynamic_type_ids
			l_target_dynamic_types := target_dynamic_types
			from
				l_call := a_first_call
				l_last_call_next := a_last_call.next
			until
				l_call = Void or else l_call = l_last_call_next
			loop
				l_target_type_set := l_call.target_type_set
				nb := l_target_type_set.count
				from i := 1 until i > nb loop
					l_dynamic_type := l_target_type_set.dynamic_type (i)
					if not l_target_dynamic_types.has (l_dynamic_type.id) then
						l_target_dynamic_type_ids.force_last (l_dynamic_type.id)
						l_target_dynamic_types.force_last_new (l_dynamic_type, l_dynamic_type.id)
					end
					i := i + 1
				end
				l_call := l_call.next
			end
			l_target_dynamic_type_ids.sort (dynamic_type_id_sorter)
				-- Print feature signature.
			print_call_name_comment (l_static_call, l_caller, a_target_type, header_file)
			print_call_name_comment (l_static_call, l_caller, a_target_type, current_file)
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			if l_result_type /= Void then
				if a_first_call.force_result_boxing then
						-- We want the result be boxed even though the result static type is expanded.
						-- This is needed when the call is an attribute, tuple label or "SPECIAL"
						-- item, the call result type is expanded, and it is the target of another call.
						-- In that case we force the result type to be of reference type in order
						-- to force the boxing of the result so that the other call is applied on
						-- the result object itself and not on a copy of this object.
					l_result_type := current_dynamic_system.any_type
					call_target_type := l_result_type
					l_force_result_boxing := True
				end
				print_type_declaration (l_result_type, header_file)
				print_type_declaration (l_result_type, current_file)
			else
				header_file.put_string (c_void)
				current_file.put_string (c_void)
			end
			header_file.put_character (' ')
			current_file.put_character (' ')
			print_call_name (l_static_call, l_caller, a_target_type, l_force_result_boxing, header_file)
			print_call_name (l_static_call, l_caller, a_target_type, l_force_result_boxing, current_file)
			header_file.put_character ('(')
			current_file.put_character ('(')
			header_file.put_string (c_ge_context)
			header_file.put_character ('*')
			header_file.put_character (' ')
			header_file.put_string (c_ac)
			header_file.put_character (',')
			header_file.put_character (' ')
			current_file.put_string (c_ge_context)
			current_file.put_character ('*')
			current_file.put_character (' ')
			current_file.put_string (c_ac)
			current_file.put_character (',')
			current_file.put_character (' ')
			print_type_declaration (a_target_type, header_file)
			print_type_declaration (a_target_type, current_file)
			if a_target_type.is_expanded then
				header_file.put_character ('*')
				current_file.put_character ('*')
			end
			header_file.put_character (' ')
			current_file.put_character (' ')
			print_current_name (header_file)
			print_current_name (current_file)
			if l_actual_arguments /= Void then
				nb := l_actual_arguments.count
				if nb /= l_formal_arguments_count then
						-- Internal error: we should have checked at this stage that
						-- the number of actual arguments should be the same as the
						-- number of formal arguments.
					set_fatal_error
					error_handler.report_giaaa_error
				elseif l_manifest_tuple /= Void then
					nb := l_manifest_tuple.count
					from i := 1 until i > nb loop
						header_file.put_character (',')
						current_file.put_character (',')
						header_file.put_character (' ')
						current_file.put_character (' ')
						l_argument_type_set := dynamic_type_set_in_feature (l_manifest_tuple.expression (i), l_caller)
						l_argument_type := l_argument_type_set.static_type.primary_type
						print_type_declaration (l_argument_type, header_file)
						print_type_declaration (l_argument_type, current_file)
						header_file.put_character (' ')
						current_file.put_character (' ')
						l_name := formal_argument (i)
						print_argument_name (l_name, header_file)
						print_argument_name (l_name, current_file)
						i := i + 1
					end
				elseif nb > 0 then
					from i := 1 until i > nb loop
						header_file.put_character (',')
						current_file.put_character (',')
						header_file.put_character (' ')
						current_file.put_character (' ')
						l_argument_type_set := dynamic_type_set_in_feature (l_actual_arguments.actual_argument (i), l_caller)
						l_argument_type := l_argument_type_set.static_type.primary_type
						print_type_declaration (l_argument_type, header_file)
						print_type_declaration (l_argument_type, current_file)
						header_file.put_character (' ')
						current_file.put_character (' ')
						if l_formal_arguments /= Void then
							l_name := l_formal_arguments.formal_argument (i).name
						else
							l_name := formal_argument (i)
						end
						print_argument_name (l_name, header_file)
						print_argument_name (l_name, current_file)
						i := i + 1
					end
				end
			end
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
				-- Print feature body.
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			if l_switch or l_target_dynamic_type_ids.count > 20 then
					-- Use switch statement.
				print_indentation
				current_file.put_string (c_switch)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_attribute_type_id_access (tokens.current_keyword, a_target_type, False)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				nb := l_target_dynamic_type_ids.count
				from i := 1 until i > nb loop
					l_dynamic_type := l_target_dynamic_types.item (l_target_dynamic_type_ids.item (i))
					print_indentation
					current_file.put_string (c_case)
					current_file.put_character (' ')
					current_file.put_integer (l_dynamic_type.id)
					current_file.put_character (':')
					current_file.put_new_line
					indent
					if l_result_type /= Void then
						print_indentation
						current_file.put_string (c_return)
						current_file.put_character (' ')
						current_file.put_character ('(')
						set_polymorphic_call_argument_type_sets (l_argument_type_sets, a_first_call, a_last_call, l_dynamic_type)
						print_adapted_named_query_call (l_static_call.name, l_dynamic_type, l_result_type, False)
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					else
						set_polymorphic_call_argument_type_sets (l_argument_type_sets, a_first_call, a_last_call, l_dynamic_type)
						print_named_procedure_call (l_static_call.name, l_dynamic_type, False)
						print_indentation
						current_file.put_string (c_break)
						current_file.put_character (';')
						current_file.put_new_line
					end
					dedent
					i := i + 1
				end
				if l_result_type /= Void then
					print_indentation
					current_file.put_string (c_default)
					current_file.put_character (':')
					current_file.put_new_line
					indent
					print_indentation
					current_file.put_string (c_return)
					current_file.put_character (' ')
					print_default_entity_value (l_result_type, current_file)
					current_file.put_character (';')
					current_file.put_new_line
					dedent
				end
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
			else
				print_indentation
				current_file.put_string (c_eif_type_index)
				current_file.put_character (' ')
				l_temp := temp_variable
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attribute_type_id_access (tokens.current_keyword, a_target_type, False)
				current_file.put_character (';')
				current_file.put_new_line
					-- Use binary search.
				print_binary_search_polymorphic_calls (a_first_call, a_last_call, l_result_type, 1, l_target_dynamic_type_ids.count, l_target_dynamic_type_ids, l_target_dynamic_types)
			end
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
			call_operands.wipe_out
			if l_manifest_tuple_operand /= Void then
				l_manifest_tuple_operand.wipe_out
			end
			flush_to_c_file
			l_target_dynamic_type_ids.wipe_out
			l_target_dynamic_types.wipe_out
			current_dynamic_type_sets := old_dynamic_type_sets
			current_feature := old_feature
			current_type := old_type
			call_target_type := l_old_call_target_type
		end

	print_binary_search_polymorphic_calls (a_first_call, a_last_call: ET_DYNAMIC_QUALIFIED_CALL; a_result_type: detachable ET_DYNAMIC_PRIMARY_TYPE; l, u: INTEGER; a_target_dynamic_type_ids: DS_ARRAYED_LIST [INTEGER]; a_target_dynamic_types: DS_HASH_TABLE [ET_DYNAMIC_PRIMARY_TYPE, INTEGER])
			-- Print to `current_file' dynamic binding code for the calls between `a_first_call'
			-- and `a_last_call' whose target dynamic types are those stored in `a_target_dynamic_types'
			-- whose type-id is itself stored between indexes `l' and `u' in `a_target_dynamic_type_ids'.
			-- The generated code uses binary search to find out which feature to execute.
		require
			a_first_call_not_void: a_first_call /= Void
			a_last_call_not_void: a_last_call /= Void
			a_target_dynamic_type_ids_not_void: a_target_dynamic_type_ids /= Void
			a_target_dynamic_types_not_void: a_target_dynamic_types /= Void
			no_void_target_dynamic_type: not a_target_dynamic_types.has_void_item
			consistent_count: a_target_dynamic_types.count = a_target_dynamic_type_ids.count
			l_large_enough: l >= 1
			l_small_enough: l <= u
			u_small_enough: u <= a_target_dynamic_type_ids.count
		local
			t: INTEGER
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_type_id: INTEGER
			l_temp: ET_IDENTIFIER
			l_static_call: ET_CALL_COMPONENT
			l_argument_type_sets: ET_DYNAMIC_STANDALONE_TYPE_SET_LIST
		do
			l_argument_type_sets := standalone_type_sets
			l_static_call := a_first_call.static_call
			l_temp := temp_variable
			if l = u then
				l_type_id := a_target_dynamic_type_ids.item (l)
				l_dynamic_type := a_target_dynamic_types.item (l_type_id)
				print_indentation
				if a_result_type /= Void then
					current_file.put_string (c_return)
					current_file.put_character (' ')
					current_file.put_character ('(')
					set_polymorphic_call_argument_type_sets (l_argument_type_sets, a_first_call, a_last_call, l_dynamic_type)
					print_adapted_named_query_call (l_static_call.name, l_dynamic_type, a_result_type, False)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				else
					set_polymorphic_call_argument_type_sets (l_argument_type_sets, a_first_call, a_last_call, l_dynamic_type)
					print_named_procedure_call (l_static_call.name, l_dynamic_type, False)
				end
			elseif l + 1 = u then
				l_type_id := a_target_dynamic_type_ids.item (l)
				l_dynamic_type := a_target_dynamic_types.item (l_type_id)
				current_file.put_string (c_if)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_temp_name (l_temp, current_file)
				current_file.put_character ('=')
				current_file.put_character ('=')
				current_file.put_integer (l_type_id)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				print_indentation
				if a_result_type /= Void then
					current_file.put_string (c_return)
					current_file.put_character (' ')
					current_file.put_character ('(')
					set_polymorphic_call_argument_type_sets (l_argument_type_sets, a_first_call, a_last_call, l_dynamic_type)
					print_adapted_named_query_call (l_static_call.name, l_dynamic_type, a_result_type, False)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				else
					set_polymorphic_call_argument_type_sets (l_argument_type_sets, a_first_call, a_last_call, l_dynamic_type)
					print_named_procedure_call (l_static_call.name, l_dynamic_type, False)
				end
				current_file.put_character ('}')
				current_file.put_character (' ')
				current_file.put_string (c_else)
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				l_type_id := a_target_dynamic_type_ids.item (u)
				l_dynamic_type := a_target_dynamic_types.item (l_type_id)
				print_indentation
				if a_result_type /= Void then
					current_file.put_string (c_return)
					current_file.put_character (' ')
					current_file.put_character ('(')
					set_polymorphic_call_argument_type_sets (l_argument_type_sets, a_first_call, a_last_call, l_dynamic_type)
					print_adapted_named_query_call (l_static_call.name, l_dynamic_type, a_result_type, False)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				else
					set_polymorphic_call_argument_type_sets (l_argument_type_sets, a_first_call, a_last_call, l_dynamic_type)
					print_named_procedure_call (l_static_call.name, l_dynamic_type, False)
				end
				current_file.put_character ('}')
				current_file.put_new_line
			else
				t := l + (u - l) // 2
				l_type_id := a_target_dynamic_type_ids.item (t)
				current_file.put_string (c_if)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_temp_name (l_temp, current_file)
				current_file.put_character ('<')
				current_file.put_character ('=')
				current_file.put_integer (l_type_id)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				print_binary_search_polymorphic_calls (a_first_call, a_last_call, a_result_type, l, t, a_target_dynamic_type_ids, a_target_dynamic_types)
				current_file.put_character ('}')
				current_file.put_character (' ')
				current_file.put_string (c_else)
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				print_binary_search_polymorphic_calls (a_first_call, a_last_call, a_result_type, t + 1, u, a_target_dynamic_type_ids, a_target_dynamic_types)
				current_file.put_character ('}')
				current_file.put_new_line
			end
		end

	same_declared_signature (a_call1, a_call2: ET_DYNAMIC_QUALIFIED_CALL): BOOLEAN
			-- Do `a_call1' and `a_call2' have the same declared signature?
			-- Note1: Reference types are considered the same declared type.
			-- Note2: If the calls are idenfied to be to ROUTINE.call or FUNCTION.item
			-- and one of the calls has a manifest tuple as argument, then the other
			-- should also have a manifest tuple of the exact same type (note1 does not
			-- apply for the types of the manifest tuples here).
		require
			a_call1_not_void: a_call1 /= Void
			a_call2_not_void: a_call2 /= Void
			same_target_type: a_call1.target_type_set.static_type = a_call2.target_type_set.static_type
			same_seed: a_call1.static_call.name.seed = a_call2.static_call.name.seed
			same_kind: a_call1.is_tuple_label = a_call2.is_tuple_label
		local
			l_args1, l_args2: detachable ET_ARGUMENT_OPERANDS
			l_feature1, l_feature2: ET_DYNAMIC_FEATURE
			l_type_set1, l_type_set2: ET_DYNAMIC_TYPE_SET
			l_seed: INTEGER
			i, nb: INTEGER
		do
				-- The calls have the same seed with the same target type,
				-- so there result types if any are the same.
				-- Look at there actual argument types now.
			l_args1 := a_call1.static_call.arguments
			l_feature1 := a_call1.current_feature
			l_args2 := a_call2.static_call.arguments
			l_feature2 := a_call2.current_feature
			if l_args1 = Void then
				Result := (l_args2 = Void or else l_args2.is_empty)
			elseif l_args2 = Void then
				Result := l_args1.is_empty
			else
				nb := l_args1.count
				if l_args2.count = nb then
					Result := True
					from i := 1 until i > nb loop
						l_type_set1 := dynamic_type_set_in_feature (l_args1.actual_argument (i), l_feature1)
						l_type_set2 := dynamic_type_set_in_feature (l_args2.actual_argument (i), l_feature2)
						if nb = 1 then
							l_seed := a_call1.static_call.name.seed
							if not a_call1.is_tuple_label and then (l_seed = current_system.routine_call_seed or l_seed = current_system.function_item_seed) and then (not a_call1.is_call_agent and not a_call2.is_call_agent) then
								if attached {ET_MANIFEST_TUPLE} l_args1.actual_argument (1) then
									Result := attached {ET_MANIFEST_TUPLE} l_args2.actual_argument (1) and then (l_type_set1.static_type = l_type_set2.static_type)
								elseif attached {ET_MANIFEST_TUPLE} l_args2.actual_argument (1) then
									Result := False
								else
									Result := same_declaration_types (l_type_set1.static_type.primary_type, l_type_set2.static_type.primary_type)
								end
							else
								Result := same_declaration_types (l_type_set1.static_type.primary_type, l_type_set2.static_type.primary_type)
							end
							i := i + 1
						elseif same_declaration_types (l_type_set1.static_type.primary_type, l_type_set2.static_type.primary_type) then
							i := i + 1
						else
							Result := False
							i := nb + 1 -- Jump out of the loop.
						end
					end
				end
			end
		end

	set_polymorphic_call_argument_type_sets (an_argument_type_sets: ET_DYNAMIC_STANDALONE_TYPE_SET_LIST; a_first_call, a_last_call: ET_DYNAMIC_QUALIFIED_CALL; a_target_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Set `an_argument_type_sets' as the union of actual argument type
			-- sets for the calls between `a_first_call' and `a_last_call' with
			-- target type `a_target_type'.
		require
			an_argument_type_sets_not_void: an_argument_type_sets /= Void
			a_first_call_not_void: a_first_call /= Void
			a_last_call_not_void: a_last_call /= Void
			a_target_type_not_void: a_target_type /= Void
		local
			l_actual_arguments: detachable ET_ARGUMENT_OPERANDS
			i, nb, nb_args: INTEGER
			l_call: detachable ET_DYNAMIC_QUALIFIED_CALL
			l_caller: ET_DYNAMIC_FEATURE
			l_is_first: BOOLEAN
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_standalone_type_set: ET_DYNAMIC_STANDALONE_TYPE_SET
			l_last_call_next: detachable ET_DYNAMIC_QUALIFIED_CALL
			l_seed: INTEGER
			l_manifest_tuple: detachable ET_MANIFEST_TUPLE
		do
			l_actual_arguments := a_first_call.static_call.arguments
			if l_actual_arguments /= Void and then l_actual_arguments.count > 0 then
				nb_args := l_actual_arguments.count
				nb := nb_args
				if nb_args = 1 then
					l_seed := a_first_call.static_call.name.seed
					if not a_first_call.is_tuple_label and then (l_seed = current_system.routine_call_seed or l_seed = current_system.function_item_seed) and then not a_first_call.is_call_agent then
						if attached {ET_MANIFEST_TUPLE} l_actual_arguments.actual_argument (1) as l_attached_manifest_tuple then
							l_manifest_tuple := l_attached_manifest_tuple
							nb := l_manifest_tuple.count + 1
						end
					end
				end
				if an_argument_type_sets.count < nb then
					an_argument_type_sets.resize (nb)
					from
						i := an_argument_type_sets.count + 1
					until
						i > nb
					loop
						create l_standalone_type_set.make (current_dynamic_system.any_type)
						an_argument_type_sets.put_last (l_standalone_type_set)
						i := i + 1
					end
				end
				if l_manifest_tuple /= Void then
					nb := l_manifest_tuple.count
					l_argument_type_set := dynamic_type_set_in_feature (l_manifest_tuple, a_first_call.current_feature)
					l_standalone_type_set := an_argument_type_sets.item (nb + 1)
					l_standalone_type_set.reset (l_argument_type_set.static_type)
					l_standalone_type_set.put_type (l_argument_type_set.static_type.primary_type)
					l_standalone_type_set.set_never_void
				end
				from
					l_call := a_first_call
					l_is_first := True
					l_last_call_next := a_last_call.next
				until
					l_call = Void or else l_call = l_last_call_next
				loop
					if l_call.target_type_set.has_type (a_target_type) then
						l_caller := l_call.current_feature
						l_actual_arguments := l_call.static_call.arguments
						if l_actual_arguments = Void or else l_actual_arguments.count /= nb_args then
								-- Internal error: all calls should have the same signature.
								-- This is checked by the call to `same_declared_signature'.
							set_fatal_error
							error_handler.report_giaaa_error
							l_call := a_last_call
						elseif l_manifest_tuple /= Void then
							if not attached {ET_MANIFEST_TUPLE} l_actual_arguments.actual_argument (1) as l_attached_manifest_tuple or else l_attached_manifest_tuple.count /= nb then
									-- Internal error: all calls should have the same signature.
									-- This is checked by the call to `same_declared_signature'.
								set_fatal_error
								error_handler.report_giaaa_error
								l_manifest_tuple := Void
								l_call := a_last_call
							else
								l_manifest_tuple := l_attached_manifest_tuple
								from i := 1 until i > nb loop
									l_argument_type_set := dynamic_type_set_in_feature (l_manifest_tuple.expression (i), l_caller)
									l_standalone_type_set := an_argument_type_sets.item (i)
									if l_is_first then
										l_standalone_type_set.reset (l_argument_type_set.static_type)
										l_standalone_type_set.set_never_void
									elseif l_argument_type_set.static_type.conforms_to_type (l_standalone_type_set.static_type) then
										-- Nothing to be done.
									elseif l_standalone_type_set.static_type.conforms_to_type (l_argument_type_set.static_type) then
										l_standalone_type_set.set_static_type (l_argument_type_set.static_type)
									else
										l_standalone_type_set.set_static_type (current_dynamic_system.any_type)
									end
									l_standalone_type_set.put_types (l_argument_type_set)
									if not l_argument_type_set.is_never_void then
										l_standalone_type_set.propagate_can_be_void (l_argument_type_set)
									end
									i := i + 1
								end
							end
						else
							from i := 1 until i > nb_args loop
								l_argument_type_set := dynamic_type_set_in_feature (l_actual_arguments.actual_argument (i), l_caller)
								l_standalone_type_set := an_argument_type_sets.item (i)
								if l_is_first then
									l_standalone_type_set.reset (l_argument_type_set.static_type)
									l_standalone_type_set.set_never_void
								elseif l_argument_type_set.static_type.conforms_to_type (l_standalone_type_set.static_type) then
									-- Nothing to be done.
								elseif l_standalone_type_set.static_type.conforms_to_type (l_argument_type_set.static_type) then
									l_standalone_type_set.set_static_type (l_argument_type_set.static_type)
								else
									l_standalone_type_set.set_static_type (current_dynamic_system.any_type)
								end
								l_standalone_type_set.put_types (l_argument_type_set)
								if not l_argument_type_set.is_never_void then
									l_standalone_type_set.propagate_can_be_void (l_argument_type_set)
								end
								i := i + 1
							end
						end
						l_is_first := False
					end
					l_call := l_call.next
				end
			end
		end

feature {NONE} -- Deep features generation

	print_deep_twin_functions
			-- Print all functions necessary to implement 'deep_twin'.
		do
			if not deep_twin_types.is_empty then
				include_runtime_header_file ("ge_deep.h", True, header_file)
					-- Be aware that `print_deep_twin_function' can add
					-- new types at the end of `deep_twin_types'.
				from deep_twin_types.start until deep_twin_types.after loop
					print_deep_twin_function (deep_twin_types.item_for_iteration)
					deep_twin_types.forth
				end
				from deep_feature_target_type_sets.start until deep_feature_target_type_sets.after loop
					print_deep_twin_polymorphic_call_function (deep_feature_target_type_sets.item_for_iteration)
					deep_feature_target_type_sets.forth
				end
				deep_twin_types.wipe_out
				deep_feature_target_type_sets.wipe_out
			end
		end

	print_deep_twin_function (a_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print 'GE_deep_twin' function for type `a_type' to `current_file'
			-- and its signature to `header_file'.
		require
			a_type_not_void: a_type /= Void
		local
			l_has_nested_reference_attributes: BOOLEAN
			i, nb: INTEGER
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_attribute: ET_DYNAMIC_FEATURE
			l_attribute_type_set: detachable ET_DYNAMIC_TYPE_SET
			l_attribute_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			old_type: like current_type
			old_file: KI_TEXT_OUTPUT_STREAM
			l_temp: ET_IDENTIFIER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_capacity_type: ET_DYNAMIC_PRIMARY_TYPE
			l_special_type: detachable ET_DYNAMIC_SPECIAL_TYPE
		do
			old_type := current_type
			current_type := a_type
				-- Print signature to `header_file' and `current_file'.
			old_file := current_file
			current_file := current_function_header_buffer
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			print_type_declaration (a_type, header_file)
			print_type_declaration (a_type, current_file)
			header_file.put_character (' ')
			current_file.put_character (' ')
			header_file.put_string (c_ge_deep_twin)
			current_file.put_string (c_ge_deep_twin)
			header_file.put_integer (a_type.id)
			current_file.put_integer (a_type.id)
			header_file.put_character ('(')
			current_file.put_character ('(')
			print_type_declaration (current_type, header_file)
			print_type_declaration (current_type, current_file)
			if current_type.is_expanded then
				header_file.put_character ('*')
				current_file.put_character ('*')
			end
			header_file.put_character (' ')
			current_file.put_character (' ')
			print_current_name (header_file)
			print_current_name (current_file)
			header_file.put_character (',')
			header_file.put_character (' ')
			current_file.put_character (',')
			current_file.put_character (' ')
			header_file.put_string (c_ge_deep)
			header_file.put_character ('*')
			header_file.put_character (' ')
			header_file.put_character ('d')
			current_file.put_string (c_ge_deep)
			current_file.put_character ('*')
			current_file.put_character (' ')
			current_file.put_character ('d')
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
				-- Print body to `current_file'.
			current_file.put_character ('{')
			current_file.put_new_line
			indent
				-- Declare the Result entity.
			print_indentation
			print_type_declaration (a_type, current_file)
			current_file.put_character (' ')
			print_result_name (current_file)
			current_file.put_character (';')
			current_file.put_new_line
			current_file := current_function_body_buffer
			if a_type.base_class.is_type_class then
-- TODO: this built-in routine could be inlined.
					-- Cannot have two instances of class TYPE representing the same Eiffel type.
				print_indentation
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_current_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
			else
				l_has_nested_reference_attributes := a_type.has_nested_reference_attributes
				if l_has_nested_reference_attributes then
					print_indentation
					current_file.put_string (c_ge_deep)
					current_file.put_character ('*')
					current_file.put_character (' ')
					current_file.put_character ('t')
					current_file.put_character ('0')
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('d')
					current_file.put_character (';')
					current_file.put_new_line
				end
				if attached {ET_DYNAMIC_SPECIAL_TYPE} a_type as l_attached_special_type then
					l_special_type := l_attached_special_type
					l_attribute_type_set := l_special_type.item_type_set
					l_attribute_type := l_attribute_type_set.static_type.primary_type
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_ge_new)
					current_file.put_integer (l_special_type.id)
					current_file.put_character ('(')
					print_attribute_special_capacity_access (tokens.current_keyword, l_special_type, False)
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_string (c_eif_false)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					print_indentation
					if not l_special_type.is_expanded then
						current_file.put_character ('*')
					end
					print_type_cast (l_special_type, current_file)
					current_file.put_character ('(')
					print_result_name (current_file)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					if not l_special_type.is_expanded then
						current_file.put_character ('*')
					end
					print_type_cast (l_special_type, current_file)
					current_file.put_character ('(')
					print_current_name (current_file)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					if not l_has_nested_reference_attributes then
							-- Copy items if they are reference objects or expanded
							-- objects containing (recursively) reference attributes.
						print_indentation
						current_file.put_string (c_memcpy)
						current_file.put_character ('(')
						print_attribute_special_item_access (tokens.result_keyword, l_special_type, False)
						current_file.put_character (',')
						print_attribute_special_item_access (tokens.current_keyword, l_special_type, False)
						current_file.put_character (',')
						print_attribute_special_count_access (tokens.current_keyword, l_special_type, False)
						current_file.put_character ('*')
						current_file.put_string (c_sizeof)
						current_file.put_character ('(')
						print_type_declaration (l_attribute_type, current_file)
						current_file.put_character (')')
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					end
						-- Clear item slots beyond 'count' if needed.
					current_file.put_string (c_ifndef)
					current_file.put_character (' ')
					if l_special_type.has_nested_reference_fields then
						current_file.put_line (c_ge_malloc_cleared)
					else
						current_file.put_line (c_ge_malloc_atomic_cleared)
					end
					print_indentation
					current_file.put_string (c_memset)
					current_file.put_character ('(')
					print_attribute_special_item_access (tokens.current_keyword, l_special_type, False)
					current_file.put_character ('+')
					print_attribute_special_count_access (tokens.current_keyword, l_special_type, False)
					current_file.put_character (',')
					current_file.put_character ('0')
					current_file.put_character (',')
					current_file.put_character ('(')
					print_attribute_special_capacity_access (tokens.current_keyword, l_special_type, False)
					current_file.put_character ('-')
					print_attribute_special_count_access (tokens.current_keyword, l_special_type, False)
					current_file.put_character (')')
					current_file.put_character ('*')
					current_file.put_string (c_sizeof)
					current_file.put_character ('(')
					print_type_declaration (l_attribute_type, current_file)
					current_file.put_character (')')
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					current_file.put_line (c_endif)
				elseif a_type.is_expanded then
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('*')
					print_current_name (current_file)
					current_file.put_character (';')
					current_file.put_new_line
				else
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_ge_new)
					current_file.put_integer (a_type.id)
					current_file.put_character ('(')
					current_file.put_string (c_eif_false)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					print_indentation
					current_file.put_character ('*')
					print_type_cast (a_type, current_file)
					current_file.put_character ('(')
					print_result_name (current_file)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('*')
					print_type_cast (a_type, current_file)
					current_file.put_character ('(')
					print_current_name (current_file)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				end
				if l_has_nested_reference_attributes then
						-- Allocate a 'GE_deep' struct to keep track of already twined
						-- reference objects, or use 'd' if not a null pointer (which
						-- means that the current object is not the root of the deep twin).
					print_indentation
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					current_file.put_character ('!')
					current_file.put_character ('t')
					current_file.put_character ('0')
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					print_indentation
					current_file.put_string ("t0 = GE_deep_new();")
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
					if not a_type.is_expanded then
							-- Keep track of reference objects already twined.
						print_indentation
						current_file.put_string ("GE_deep_put")
						current_file.put_character ('(')
						print_current_name (current_file)
						current_file.put_character (',')
						current_file.put_character (' ')
						print_result_name (current_file)
						current_file.put_character (',')
						current_file.put_character (' ')
						current_file.put_character ('t')
						current_file.put_character ('0')
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					end
						-- Twin reference attributes or expanded attributes that
						-- contain themselves (recursively) reference attributes.
					if l_special_type /= Void then
						check l_attribute_type_set_already_computed: l_attribute_type_set /= Void then end
						check l_attribute_type_already_computed: l_attribute_type /= Void then end
							-- Twin items.
						if l_special_type.attribute_count < 2 then
								-- Internal error: class "SPECIAL" should have at least the
								-- features 'count' and 'capacity' as first features.
								-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
							set_fatal_error
							error_handler.report_giaaa_error
						else
							l_dynamic_type_set := result_type_set_in_feature (l_special_type.queries.item (2))
							l_capacity_type := l_dynamic_type_set.static_type.primary_type
							if l_attribute_type_set.is_empty then
									-- If the dynamic type set of the items is empty,
									-- then the items is always Void. No need to twin
									-- it in that case.
							elseif l_attribute_type.is_expanded then
									-- If the items are expanded.
									-- We need to deep twin them only if they themselves contain
									-- (recursively) reference attributes. Otherwise we can copy
									-- their contents without further ado.
								if l_attribute_type.has_nested_reference_attributes then
									l_temp := new_temp_variable (l_capacity_type)
									print_indentation
									current_file.put_string (c_for)
									current_file.put_character (' ')
									current_file.put_character ('(')
									print_temp_name (l_temp, current_file)
									current_file.put_character (' ')
									current_file.put_character ('=')
									current_file.put_character (' ')
									print_attribute_special_count_access (tokens.current_keyword, l_special_type, False)
									current_file.put_character (' ')
									current_file.put_character ('-')
									current_file.put_character (' ')
									current_file.put_character ('1')
									current_file.put_character (';')
									current_file.put_character (' ')
									print_temp_name (l_temp, current_file)
									current_file.put_character (' ')
									current_file.put_character ('>')
									current_file.put_character ('=')
									current_file.put_character (' ')
									current_file.put_character ('0')
									current_file.put_character (';')
									current_file.put_character (' ')
									print_temp_name (l_temp, current_file)
									current_file.put_character ('-')
									current_file.put_character ('-')
									current_file.put_character (')')
									current_file.put_character (' ')
									current_file.put_character ('{')
									current_file.put_new_line
									indent
									print_set_deep_twined_attribute (l_attribute_type_set, agent print_attribute_special_indexed_item_access (l_temp,  tokens.current_keyword, a_type, False), agent print_attribute_special_indexed_item_access (l_temp,  tokens.result_keyword, a_type, False))
									dedent
									print_indentation
									current_file.put_character ('}')
									current_file.put_new_line
									mark_temp_variable_free (l_temp)
								end
							else
									-- We are in the case of reference items.
								l_temp := new_temp_variable (l_capacity_type)
								print_indentation
								current_file.put_string (c_for)
								current_file.put_character (' ')
								current_file.put_character ('(')
								print_temp_name (l_temp, current_file)
								current_file.put_character (' ')
								current_file.put_character ('=')
								current_file.put_character (' ')
								print_attribute_special_count_access (tokens.current_keyword, l_special_type, False)
								current_file.put_character (' ')
								current_file.put_character ('-')
								current_file.put_character (' ')
								current_file.put_character ('1')
								current_file.put_character (';')
								current_file.put_character (' ')
								print_temp_name (l_temp, current_file)
								current_file.put_character (' ')
								current_file.put_character ('>')
								current_file.put_character ('=')
								current_file.put_character (' ')
								current_file.put_character ('0')
								current_file.put_character (';')
								current_file.put_character (' ')
								print_temp_name (l_temp, current_file)
								current_file.put_character ('-')
								current_file.put_character ('-')
								current_file.put_character (')')
								current_file.put_character (' ')
								current_file.put_character ('{')
								current_file.put_new_line
								indent
								print_set_deep_twined_attribute (l_attribute_type_set, agent print_attribute_special_indexed_item_access (l_temp,  tokens.current_keyword, a_type, False), agent print_attribute_special_indexed_item_access (l_temp,  tokens.result_keyword, a_type, False))
								dedent
								print_indentation
								current_file.put_character ('}')
								current_file.put_new_line
								mark_temp_variable_free (l_temp)
							end
						end
					else
						l_queries := a_type.queries
						nb := a_type.attribute_count
						from i := 1 until i > nb loop
							l_attribute := l_queries.item (i)
							l_attribute_type_set := l_attribute.result_type_set
							if l_attribute_type_set = Void then
									-- Internal error: an attribute should have a result type.
								set_fatal_error
								error_handler.report_giaaa_error
							elseif l_attribute_type_set.is_empty then
									-- If the dynamic type set of the attribute is empty,
									-- then this attribute is always Void. No need to twin
									-- it in that case.
							else
								l_attribute_type := l_attribute_type_set.static_type.primary_type
								if l_attribute_type.is_expanded then
										-- If the attribute is expanded, then its contents has
										-- already been copied. We need to deep twin it only if
										-- it itself contains (recursively) reference attributes.
									if l_attribute_type.has_nested_reference_attributes then
										print_set_deep_twined_attribute (l_attribute_type_set, agent print_attribute_access (l_attribute, tokens.result_keyword, a_type, False), agent print_attribute_access (l_attribute, tokens.result_keyword, a_type, False))
									end
								else
									print_set_deep_twined_attribute (l_attribute_type_set, agent print_attribute_access (l_attribute, tokens.result_keyword, a_type, False), agent print_attribute_access (l_attribute, tokens.result_keyword, a_type, False))
								end
							end
							i := i + 1
						end
						if attached {ET_DYNAMIC_TUPLE_TYPE} a_type as l_tuple_type then
							l_item_type_sets := l_tuple_type.item_type_sets
							nb := l_item_type_sets.count
							from i := 1 until i > nb loop
								l_attribute_type_set := l_item_type_sets.item (i)
								l_attribute_type := l_attribute_type_set.static_type.primary_type
								if l_attribute_type_set.is_empty then
										-- If the dynamic type set of the item is empty,
										-- then this item is always Void. No need to twin
										-- it in that case.
								elseif l_attribute_type.is_expanded then
										-- If the item is expanded, then its contents has
										-- already been copied. We need to deep twin it only if
										-- it itself contains (recursively) reference attributes.
									if l_attribute_type.has_nested_reference_attributes then
										print_set_deep_twined_attribute (l_attribute_type_set, agent print_attribute_tuple_item_access (i, tokens.result_keyword, a_type, False), agent print_attribute_tuple_item_access (i, tokens.result_keyword, a_type, False))
									end
								else
									print_set_deep_twined_attribute (l_attribute_type_set, agent print_attribute_tuple_item_access (i, tokens.result_keyword, a_type, False), agent print_attribute_tuple_item_access (i, tokens.result_keyword, a_type, False))
								end
								i := i + 1
							end
						end
					end
						-- Free previously allocated 'GE_deep' struct, if any (i.e. if
						-- the current object was the root object of the deep twin).
					print_indentation
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					current_file.put_character ('t')
					current_file.put_character ('0')
					current_file.put_character (' ')
					current_file.put_character ('!')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('d')
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					print_indentation
					current_file.put_string ("GE_deep_free(t0);")
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
				elseif not a_type.is_expanded then
						-- Keep track of reference objects already twined.
						-- If 'd' is a null pointer, then there is no need
						-- to keep track of this object because it is the
						-- only object to be twined: it is the root object
						-- of the deep twin ('d' being a null pointer) and
						-- it has not reference attributes.
					print_indentation
					current_file.put_string (c_if)
					current_file.put_character (' ')
					current_file.put_character ('(')
					current_file.put_character ('d')
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
					print_indentation
					current_file.put_string ("GE_deep_put")
					current_file.put_character ('(')
					print_current_name (current_file)
					current_file.put_character (',')
					current_file.put_character (' ')
					print_result_name (current_file)
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_character ('d')
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
				end
			end
				-- Return the deep twined object.
			print_indentation
			current_file.put_string (c_return)
			current_file.put_character (' ')
			print_result_name (current_file)
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
			current_file := old_file
			flush_to_c_file
			reset_temp_variables
			current_type := old_type
		end

	print_deep_twin_polymorphic_call_function (a_target_type_set: ET_DYNAMIC_TYPE_SET)
			-- Print 'GE_deep_twin<type-id>x' function to `current_file' and its signature to `header_file'.
			-- 'GE_deep_twin<type-id>x' corresponds to a polymorphic call to 'deep_twin'
			-- whose target has `a_target_type_set' as dynamic type set.
			-- 'type-id' is the type-id of the static type of the target.
		require
			a_target_type_set_not_void: a_target_type_set /= Void
		local
			l_target_dynamic_type_ids: DS_ARRAYED_LIST [INTEGER]
			l_target_dynamic_types: DS_HASH_TABLE [ET_DYNAMIC_PRIMARY_TYPE, INTEGER]
			l_static_type: ET_DYNAMIC_PRIMARY_TYPE
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_type_id: INTEGER
			l_temp: ET_IDENTIFIER
			i, nb: INTEGER
			l_switch: BOOLEAN
			old_type: like current_type
		do
			l_static_type := a_target_type_set.static_type.primary_type
			old_type := current_type
			current_type := l_static_type
				-- Print signature to `header_file' and `current_file'.
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			print_type_declaration (l_static_type, header_file)
			print_type_declaration (l_static_type, current_file)
			header_file.put_character (' ')
			current_file.put_character (' ')
			header_file.put_string (c_ge_deep_twin)
			current_file.put_string (c_ge_deep_twin)
			header_file.put_integer (l_static_type.id)
			current_file.put_integer (l_static_type.id)
			header_file.put_character ('x')
			current_file.put_character ('x')
			header_file.put_character ('(')
			current_file.put_character ('(')
			print_type_declaration (current_type, header_file)
			print_type_declaration (current_type, current_file)
			if current_type.is_expanded then
				header_file.put_character ('*')
				current_file.put_character ('*')
			end
			header_file.put_character (' ')
			current_file.put_character (' ')
			print_current_name (header_file)
			print_current_name (current_file)
			header_file.put_character (',')
			header_file.put_character (' ')
			current_file.put_character (',')
			current_file.put_character (' ')
			header_file.put_string (c_ge_deep)
			header_file.put_character ('*')
			header_file.put_character (' ')
			header_file.put_character ('t')
			header_file.put_character ('0')
			current_file.put_string (c_ge_deep)
			current_file.put_character ('*')
			current_file.put_character (' ')
			current_file.put_character ('t')
			current_file.put_character ('0')
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
				-- Print body to `current_file'.
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			l_target_dynamic_type_ids := target_dynamic_type_ids
			l_target_dynamic_types := target_dynamic_types
			nb := a_target_type_set.count
			from i := 1 until i > nb loop
				l_dynamic_type := a_target_type_set.dynamic_type (i)
				l_type_id := l_dynamic_type.id
				l_target_dynamic_type_ids.force_last (l_type_id)
				l_target_dynamic_types.force_last (l_dynamic_type, l_type_id)
				i := i + 1
			end
			l_target_dynamic_type_ids.sort (dynamic_type_id_sorter)
			if l_switch or l_target_dynamic_type_ids.count > 20 then
					-- Use switch statement.
				print_indentation
				current_file.put_string (c_switch)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_attribute_type_id_access (tokens.current_keyword, l_static_type, False)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				nb := l_target_dynamic_type_ids.count
				from i := 1 until i > nb loop
					l_type_id := l_target_dynamic_type_ids.item (i)
					l_dynamic_type := l_target_dynamic_types.item (l_type_id)
					print_indentation
					current_file.put_string (c_case)
					current_file.put_character (' ')
					current_file.put_integer (l_type_id)
					current_file.put_character (':')
					current_file.put_new_line
					indent
					print_indentation
					current_file.put_string (c_return)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_adapted_deep_twin_call (tokens.current_keyword, l_dynamic_type, l_static_type)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					i := i + 1
				end
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
			else
				print_indentation
				current_file.put_string (c_eif_type_index)
				current_file.put_character (' ')
				l_temp := temp_variable
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attribute_type_id_access (tokens.current_keyword, l_static_type, False)
				current_file.put_character (';')
				current_file.put_new_line
					-- Use binary search.
				print_deep_twin_binary_search_polymorphic_call (l_static_type, 1, l_target_dynamic_type_ids.count, l_target_dynamic_type_ids, l_target_dynamic_types)
			end
			l_target_dynamic_type_ids.wipe_out
			l_target_dynamic_types.wipe_out
			print_indentation
			current_file.put_string (c_return)
			current_file.put_character (' ')
			print_default_entity_value (l_static_type, current_file)
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
			flush_to_c_file
			current_type := old_type
		end

	print_deep_twin_binary_search_polymorphic_call (a_target_static_type: ET_DYNAMIC_PRIMARY_TYPE; l, u: INTEGER; a_target_dynamic_type_ids: DS_ARRAYED_LIST [INTEGER]; a_target_dynamic_types: DS_HASH_TABLE [ET_DYNAMIC_PRIMARY_TYPE, INTEGER])
			-- Print to `current_file' dynamic binding code for the call to 'GE_deep_twin'
			-- whose target's static type is `a_target_static_type' and whose target's
			-- dynamic types are those stored in `a_target_dynamic_types' whose type-id is
			-- itself stored between indexes `l' and `u' in `a_target_dynamic_type_ids'.
			-- The generated code uses binary search to find out which feature to execute.
		require
			a_target_static_type_not_void: a_target_static_type /= Void
			a_target_dynamic_type_ids_not_void: a_target_dynamic_type_ids /= Void
			a_target_dynamic_types_not_void: a_target_dynamic_types /= Void
			no_void_target_dynamic_type: not a_target_dynamic_types.has_void_item
			consistent_count: a_target_dynamic_types.count = a_target_dynamic_type_ids.count
			l_large_enough: l >= 1
			l_small_enough: l <= u
			u_small_enough: u <= a_target_dynamic_type_ids.count
		local
			t: INTEGER
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_type_id: INTEGER
			l_temp: ET_IDENTIFIER
		do
			l_temp := temp_variable
			if l = u then
				l_type_id := a_target_dynamic_type_ids.item (l)
				l_dynamic_type := a_target_dynamic_types.item (l_type_id)
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_adapted_deep_twin_call (tokens.current_keyword, l_dynamic_type, a_target_static_type)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			elseif l + 1 = u then
				l_type_id := a_target_dynamic_type_ids.item (l)
				l_dynamic_type := a_target_dynamic_types.item (l_type_id)
				current_file.put_string (c_if)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_temp_name (l_temp, current_file)
				current_file.put_character ('=')
				current_file.put_character ('=')
				current_file.put_integer (l_type_id)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_adapted_deep_twin_call (tokens.current_keyword, l_dynamic_type, a_target_static_type)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				current_file.put_character ('}')
				current_file.put_character (' ')
				current_file.put_string (c_else)
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				l_type_id := a_target_dynamic_type_ids.item (u)
				l_dynamic_type := a_target_dynamic_types.item (l_type_id)
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_adapted_deep_twin_call (tokens.current_keyword, l_dynamic_type, a_target_static_type)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				current_file.put_character ('}')
				current_file.put_new_line
			else
				t := l + (u - l) // 2
				l_type_id := a_target_dynamic_type_ids.item (t)
				current_file.put_string (c_if)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_temp_name (l_temp, current_file)
				current_file.put_character ('<')
				current_file.put_character ('=')
				current_file.put_integer (l_type_id)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				print_deep_twin_binary_search_polymorphic_call (a_target_static_type, l, t, a_target_dynamic_type_ids, a_target_dynamic_types)
				current_file.put_character ('}')
				current_file.put_character (' ')
				current_file.put_string (c_else)
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				print_deep_twin_binary_search_polymorphic_call (a_target_static_type, t + 1, u, a_target_dynamic_type_ids, a_target_dynamic_types)
				current_file.put_character ('}')
				current_file.put_new_line
			end
		end

	print_set_deep_twined_attribute (an_attribute_type_set: ET_DYNAMIC_TYPE_SET; a_print_attribute_access_read, a_print_attribute_access_write: PROCEDURE)
			-- Print to `current_file' the instructions needed to deep twin an attribute
			-- of `current_type' whose dynamic type set is `an_attribute_type_set'.
			-- `a_print_attribute_access_read' and `a_print_attribute_access_write' are
			-- used to print to `current_file' the code to access this attribute in
			-- read and write mode. Indeed, it can be a "regular" attribute, but it
			-- can also be items of a SPECIAL object, fields of a TUPLE object, closed
			-- operands of an Agent object, ...
		require
			an_attribute_type_set_not_void: an_attribute_type_set /= Void
			an_attribute_type_set_not_empty: not an_attribute_type_set.is_empty
			a_print_attribute_access_read_not_void: a_print_attribute_access_read /= Void
			a_print_attribute_access_write_not_void: a_print_attribute_access_write /= Void
		local
			l_attribute_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp1, l_temp2: ET_IDENTIFIER
		do
			l_attribute_type := an_attribute_type_set.static_type.primary_type
			l_temp1 := new_temp_variable (l_attribute_type)
			print_indentation
			print_temp_name (l_temp1, current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			a_print_attribute_access_read.call ([])
			current_file.put_character (';')
			current_file.put_new_line
			if l_attribute_type.is_expanded then
					-- No need to test whether the attribute is Void or not:
					-- expanded attributes are never Void.
				print_indentation
				a_print_attribute_access_write.call ([])
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_deep_twined_attribute (l_temp1, an_attribute_type_set)
				current_file.put_character (';')
				current_file.put_new_line
			else
					-- If the attribute is Void, then there is no need to twin it.
				print_indentation
				current_file.put_string (c_if)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_temp_name (l_temp1, current_file)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				indent
				l_temp2 := new_temp_variable (l_attribute_type)
				print_indentation
				print_temp_name (l_temp2, current_file)
				current_file.put_string (" = GE_deep_item(")
				print_temp_name (l_temp1, current_file)
				current_file.put_string (", t0);")
				current_file.put_new_line
				print_indentation
				a_print_attribute_access_write.call ([])
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_temp_name (l_temp2, current_file)
				current_file.put_character ('?')
					-- The object has not been twined yet.
				print_temp_name (l_temp2, current_file)
				current_file.put_character (':')
				current_file.put_character ('(')
				print_deep_twined_attribute (l_temp1, an_attribute_type_set)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				dedent
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
				mark_temp_variable_free (l_temp2)
			end
			mark_temp_variable_free (l_temp1)
		end

	print_deep_twined_attribute (an_attribute: ET_EXPRESSION; an_attribute_type_set: ET_DYNAMIC_TYPE_SET)
			-- Print to `current_file' deep twined version of the attribute `an_attribute'
			-- belonging to `current_type', with dynamic type set `an_attribute_type_set'.
			-- The test for Void-ness of the attribute is assumed to have
			-- been generated elsewhere. And the attribute is assumed not to
			-- have been deep twined already.
		require
			an_attribute_not_void: an_attribute /= Void
			an_attribute_type_set_not_void: an_attribute_type_set /= Void
			an_attribute_type_set_not_empty: not an_attribute_type_set.is_empty
		local
			i, nb: INTEGER
			l_attribute_type: ET_DYNAMIC_PRIMARY_TYPE
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			l_standalone_type_set: ET_DYNAMIC_STANDALONE_TYPE_SET
		do
			l_attribute_type := an_attribute_type_set.static_type.primary_type
			nb := an_attribute_type_set.count
			if nb = 1 then
					-- Monomorphic call.
				l_dynamic_type := an_attribute_type_set.dynamic_type (1)
				deep_twin_types.force_last (l_dynamic_type)
				print_adapted_deep_twin_call (an_attribute, l_dynamic_type, l_attribute_type)
			elseif nb = 2 then
					-- Polymorphic with only two possible types at run-time.
				l_dynamic_type := an_attribute_type_set.dynamic_type (1)
				deep_twin_types.force_last (l_dynamic_type)
				current_file.put_character ('(')
				current_file.put_character ('(')
				print_attribute_type_id_access (an_attribute, l_attribute_type, False)
				current_file.put_character ('=')
				current_file.put_character ('=')
				current_file.put_integer (l_dynamic_type.id)
				current_file.put_character (')')
				current_file.put_character ('?')
				print_adapted_deep_twin_call (an_attribute, l_dynamic_type, l_attribute_type)
				current_file.put_character (':')
				l_dynamic_type := an_attribute_type_set.dynamic_type (2)
				deep_twin_types.force_last (l_dynamic_type)
				print_adapted_deep_twin_call (an_attribute, l_dynamic_type, l_attribute_type)
				current_file.put_character (')')
			else
					-- Polymorphic with more than two possible types at run-time.
					-- Wrap this polymorphic call into a funtion that will be
					-- shared by other polymorphic calls having the same target
					-- static type.
					--
					-- First, register all what is needed so that this shared
					-- function will be generated correctly.
				deep_feature_target_type_sets.search (l_attribute_type)
				if deep_feature_target_type_sets.found then
					l_standalone_type_set := deep_feature_target_type_sets.found_item
				else
					if standalone_type_sets.count > deep_feature_target_type_sets.count then
						l_standalone_type_set := standalone_type_sets.item (deep_feature_target_type_sets.count + 1)
						l_standalone_type_set.reset (l_attribute_type)
					else
						create l_standalone_type_set.make (l_attribute_type)
						standalone_type_sets.force_last (l_standalone_type_set)
					end
					deep_feature_target_type_sets.force_last_new (l_standalone_type_set, l_attribute_type)
				end
				l_standalone_type_set.put_types (an_attribute_type_set)
				from i := 1 until i > nb loop
					deep_twin_types.force_last (an_attribute_type_set.dynamic_type (i))
					i := i + 1
				end
					-- Now call the shared function that will trigger the
					-- polymorphic call.
				current_file.put_string (c_ge_deep_twin)
				current_file.put_integer (l_attribute_type.id)
				current_file.put_character ('x')
				current_file.put_character ('(')
				print_target_expression (an_attribute, l_attribute_type, False)
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_character ('t')
				current_file.put_character ('0')
				current_file.put_character (')')
			end
		end

	print_adapted_deep_twin_call (a_target: ET_EXPRESSION; a_target_type, a_result_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print to `current_file' a call to the 'GE_deep_twin' function that
			-- will deep twin `a_target' of type `a_target_type'.
			-- `a_result_type' is the static type of the result expected by the caller,
			-- used to adapt the result of 'GE_deep_twin' if needed (see header comment
			-- of `print_adapted_expression_with_agent' for details).
			-- The test for Void-ness of the target is assumed to have
			-- been generated elsewhere.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
			a_result_type_not_void: a_result_type /= Void
		do
			print_adapted_expression_with_agent (agent print_deep_twin_call (a_target, a_target_type), a_target_type, a_result_type, False)
		end

	print_deep_twin_call (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print to `current_file' a call to the 'GE_deep_twin' function that
			-- will deep twin `a_target' of type `a_target_type'.
			-- The test for Void-ness of the target is assumed to have
			-- been generated elsewhere.
			-- Note that the result of 'GE_deep_twin' is not adapted to match the
			-- kind of result type expected by the caller. It is recommended to
			-- use `print_adapted_deep_twin_call' whenever possible.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			current_file.put_string (c_ge_deep_twin)
			current_file.put_integer (a_target_type.id)
			current_file.put_character ('(')
			print_target_expression (a_target, a_target_type, False)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_character ('t')
			current_file.put_character ('0')
			current_file.put_character (')')
		end

	deep_twin_types: DS_HASH_SET [ET_DYNAMIC_PRIMARY_TYPE]
			-- Types of object that need to be deep twined

	deep_equal_types: DS_HASH_SET [ET_DYNAMIC_PRIMARY_TYPE]
			-- Types of object that need deep equality

	deep_feature_target_type_sets: DS_HASH_TABLE [ET_DYNAMIC_STANDALONE_TYPE_SET, ET_DYNAMIC_PRIMARY_TYPE]
			-- Dynamic type sets of target of deep feature (deep twin or deep equal),
			-- indexed by target static type

feature {NONE} -- Built-in feature generation

	print_builtin_procedure_c_call (a_feature: ET_DYNAMIC_FEATURE; a_c_function_name: STRING; a_static: BOOLEAN; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding a built-in feature which is a call to a C function
			-- named `a_c_function_name'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_static' means that 'Current' is not passed to the C function.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			a_c_function_name_not_void: a_c_function_name /= Void
			a_c_function_name_not_empty: not a_c_function_name.is_empty
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
			i, nb: INTEGER
		do
			nb := a_feature.static_feature.arguments_count
			if call_operands.count /= nb + 1 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				current_file.put_string (a_c_function_name)
				current_file.put_character ('(')
				from
					if a_static then
						i := 1
					end
				until
					i > nb
				loop
					current_file.put_character ('(')
					if i = 0 then
						l_argument := call_operands.first
						l_actual_type_set := dynamic_type_set (l_argument)
						l_formal_type := a_target_type
					else
						l_argument := call_operands.item (i + 1)
						l_actual_type_set := dynamic_type_set (l_argument)
						l_formal_type := argument_type_set_in_feature (i, a_feature).static_type
					end
					print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
					current_file.put_character (')')
					if i /= nb then
						current_file.put_character (',')
						current_file.put_character (' ')
					end
					i := i + 1
				end
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_query_c_call (a_feature: ET_DYNAMIC_FEATURE; a_c_function_name: STRING; a_static: BOOLEAN; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding a built-in feature which is a call to a C function
			-- named `a_c_function_name'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_static' means that 'Current' is not passed to the C function.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			a_c_function_name_not_void: a_c_function_name /= Void
			a_c_function_name_not_empty: not a_c_function_name.is_empty
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
			i, nb: INTEGER
		do
			nb := a_feature.static_feature.arguments_count
			if call_operands.count /= nb + 1 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_declaration_type_cast_from_c (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (a_c_function_name)
				current_file.put_character ('(')
				from
					if a_static then
						i := 1
					end
				until
					i > nb
				loop
					current_file.put_character ('(')
					if i = 0 then
						l_argument := call_operands.first
						l_actual_type_set := dynamic_type_set (l_argument)
						l_formal_type := a_target_type
					else
						l_argument := call_operands.item (i + 1)
						l_actual_type_set := dynamic_type_set (l_argument)
						l_formal_type := argument_type_set_in_feature (i, a_feature).static_type
					end
					print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
					current_file.put_character (')')
					if i /= nb then
						current_file.put_character (',')
						current_file.put_character (' ')
					end
					i := i + 1
				end
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_any_conforms_to_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'ANY.conforms_to'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_conforming_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			l_non_conforming_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
		do
			l_arguments := a_feature.arguments
			if l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parse.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument_type_set := argument_type_set (1)
				nb := l_argument_type_set.count
				l_conforming_types := conforming_types
				l_conforming_types.resize (nb)
				l_non_conforming_types := non_conforming_types
				l_non_conforming_types.resize (nb)
				from i := 1 until i > nb loop
					l_dynamic_type := l_argument_type_set.dynamic_type (i)
					if current_type.conforms_to_primary_type (l_dynamic_type) then
						l_conforming_types.put_last (l_dynamic_type)
					else
						l_non_conforming_types.put_last (l_dynamic_type)
					end
					i := i + 1
				end
				if l_non_conforming_types.is_empty then
						-- `current_type' conforms to all types of `l_argument_type_set'.
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_true)
					current_file.put_character (';')
					current_file.put_new_line
				elseif l_conforming_types.is_empty then
						-- `current_type' conforms to none of the types of `l_argument_type_set'.
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_false)
					current_file.put_character (';')
					current_file.put_new_line
				elseif l_non_conforming_types.count < l_conforming_types.count then
					print_indentation
					current_file.put_string (c_switch)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_attribute_type_id_access (l_arguments.formal_argument (1).name, current_dynamic_system.any_type, True)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					nb := l_non_conforming_types.count
					from i := 1 until i > nb loop
						l_dynamic_type := l_non_conforming_types.dynamic_type (i)
						print_indentation
						current_file.put_string (c_case)
						current_file.put_character (' ')
						current_file.put_integer (l_dynamic_type.id)
						current_file.put_character (':')
						current_file.put_new_line
						i := i + 1
					end
					indent
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_false)
					current_file.put_character (';')
					current_file.put_new_line
					print_indentation
					current_file.put_string (c_break)
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_string (c_default)
					current_file.put_character (':')
					current_file.put_new_line
					indent
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_true)
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_character ('}')
				else
					print_indentation
					current_file.put_string (c_switch)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_attribute_type_id_access (l_arguments.formal_argument (1).name, current_dynamic_system.any_type, True)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					nb := l_conforming_types.count
					from i := 1 until i > nb loop
						l_dynamic_type := l_conforming_types.dynamic_type (i)
						print_indentation
						current_file.put_string (c_case)
						current_file.put_character (' ')
						current_file.put_integer (l_dynamic_type.id)
						current_file.put_character (':')
						current_file.put_new_line
						i := i + 1
					end
					indent
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_true)
					current_file.put_character (';')
					current_file.put_new_line
					print_indentation
					current_file.put_string (c_break)
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_string (c_default)
					current_file.put_character (':')
					current_file.put_new_line
					indent
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_false)
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_character ('}')
				end
				l_conforming_types.wipe_out
				l_non_conforming_types.wipe_out
			end
		end

	print_builtin_any_copy_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'ANY.copy'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_any_standard_copy_body (a_feature)
		end

	print_builtin_any_copy_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ANY.copy'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_any_standard_copy_call (a_feature, a_target_type, a_check_void_target)
		end

	print_builtin_any_deep_twin_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ANY.deep_twin'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			deep_twin_types.force_last (a_target_type)
			current_file.put_string (c_ge_deep_twin)
			current_file.put_integer (a_target_type.id)
			current_file.put_character ('(')
			print_target_expression (call_operands.first, a_target_type, a_check_void_target)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_character ('0')
			current_file.put_character (')')
		end

	print_builtin_any_generating_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ANY.generating_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_meta_type: detachable ET_DYNAMIC_PRIMARY_TYPE
		do
			l_meta_type := a_target_type.meta_type
			if l_meta_type = Void then
					-- Internal error: the meta type of the target type should
					-- have been computed when analyzing the dynamic type sets of
					-- `a_feature'.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				current_file.put_character ('(')
				print_type_declaration (l_meta_type, current_file)
				current_file.put_character (')')
				current_file.put_character ('&')
				current_file.put_character ('(')
				current_file.put_string (c_ge_types)
				current_file.put_character ('[')
				current_file.put_integer (a_target_type.id)
				current_file.put_character (']')
				current_file.put_character ('[')
				current_file.put_integer (0)
				current_file.put_character (']')
				current_file.put_character (')')
			end
		end

	print_builtin_any_generator_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ANY.generator'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_string: STRING
		do
			if attached a_feature.result_type_set as l_result_type_set and then l_result_type_set.static_type.base_type.same_named_type_with_type_marks (current_system.string_32_type, tokens.implicit_detachable_type_mark, current_system.any_type, tokens.implicit_detachable_type_mark, current_system.any_type) then
				current_file.put_string (c_ge_ms32)
			else
				current_file.put_string (c_ge_ms8)
			end
			current_file.put_character ('(')
			l_string := a_target_type.base_class.upper_name
			print_escaped_string (l_string)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_integer (l_string.count)
			current_file.put_character (')')
		end

	print_builtin_any_is_deep_equal_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' then body of `a_feature' corresponding
			-- to built-in feature 'ANY.is_deep_equal'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
-- TODO
error_handler.report_warning_message ("ET_C_GENERATOR.print_builtin_any_is_deep_equal_body not implemented")
		end

	print_builtin_any_is_equal_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'ANY.is_equal'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_any_standard_is_equal_body (a_feature)
		end

	print_builtin_any_is_equal_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ANY.is_equal'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_any_standard_is_equal_call (a_feature, a_target_type, a_check_void_target)
		end

	print_builtin_any_same_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ANY.same_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_static_type: ET_DYNAMIC_PRIMARY_TYPE
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				l_argument_type_set := dynamic_type_set (l_argument)
				l_argument_static_type := l_argument_type_set.static_type.primary_type
				if not l_argument_type_set.has_type (a_target_type) then
					current_file.put_string (c_eif_false)
				elseif l_argument_type_set.count = 1 then
						-- `a_target_type' is one of the types held in `l_argument_type_set'
						-- (see the if-branch above). Now we know that it is the only one.
					current_file.put_string (c_eif_true)
				else
					l_result_type := l_result_type_set.static_type.primary_type
					print_declaration_type_cast (l_result_type, current_file)
					current_file.put_character ('(')
						-- We know that the argument is equipped with a type-id
						-- attribute, otherwise `l_argument_static_type' would have
						-- to be an expanded type, and this is covered by the
						-- elseif-branch just above. Indeed expanded types cannot
						-- be polymorphic, so the number of types in `l_argument_type_set'
						-- can only be one.
					print_attribute_type_id_access (l_argument, l_argument_static_type, True)
					print_equal_to
					current_file.put_integer (a_target_type.id)
					current_file.put_character (')')
				end
			end
		end

	print_builtin_any_standard_is_equal_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'ANY.standard_is_equal'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_argument: ET_EXPRESSION
			l_target: ET_EXPRESSION
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_static_type: ET_DYNAMIC_PRIMARY_TYPE
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
			l_has_nested_custom_standard_is_equal_routine: BOOLEAN
		do
			l_arguments := a_feature.arguments
			if l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by ET_BUILTIN_FEATURE_CHECKER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached current_feature.result_type_set as l_result_type_set then
					-- Internal error: `current_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				fill_call_formal_arguments (a_feature)
				l_argument_type_set := argument_type_set_in_feature (1, current_feature)
				l_argument_static_type := l_argument_type_set.static_type.primary_type
				l_result_type := l_result_type_set.static_type.primary_type
				l_has_nested_custom_standard_is_equal_routine := current_type.has_nested_custom_standard_is_equal_routine
					-- The postcondition of 'ANY.standard_is_equal' indicates
					-- that the objects need to be of the same type.
				if not l_argument_type_set.has_type (current_type) then
						-- The objects are not of the same type.
					print_indentation_assign_to_result
					current_file.put_string (c_eif_false)
					print_semicolon_newline
				elseif not l_has_nested_custom_standard_is_equal_routine and current_type.is_expanded then
					print_indentation_assign_to_result
					print_builtin_any_standard_is_equal_call_with_expanded (current_feature, current_type, l_argument_static_type, l_argument_type_set, l_result_type, False)
					print_semicolon_newline
				elseif not l_has_nested_custom_standard_is_equal_routine and l_argument_type_set.count = 1 then
						-- `current_type' is one of the types held in `l_argument_type_set'
						-- (see the if-branch above). So when `l_argument_type_set' has
						-- exactly one type then we know that `current_type' is this type.
					print_indentation_assign_to_result
					print_builtin_any_standard_is_equal_call_with_reference (current_feature, current_type, l_argument_static_type, l_argument_type_set, l_result_type, False)
					print_semicolon_newline
				else
					l_target := call_operands.first
					l_argument := call_operands.item (2)
					print_indentation_assign_to_result
					if l_argument_type_set.count = 1 then
							-- `current_type' is one of the types held in `l_argument_type_set'
							-- (see the if-branch above). So when `l_argument_type_set' has
							-- exactly one type then we know that `current_type' is this type.
						current_file.put_string (c_eif_true)
					else
							-- We know that the argument is equipped with a type-id:
							-- * if 'l_argument_type_set.count = 0' then the argument is Void,
							--   so the argument static type has to be a reference type.
							-- * if 'l_argument_type_set.count > 1' then the argument is
							--   polymorphic.
						print_declaration_type_cast (l_result_type, current_file)
						current_file.put_character ('(')
						print_attribute_type_id_access (l_argument, l_argument_static_type, True)
						print_equal_to
						current_file.put_integer (current_type.id)
						current_file.put_character (')')
					end
					print_semicolon_newline
					if l_has_nested_custom_standard_is_equal_routine then
						print_builtin_any_standard_is_equal_attributes (l_argument, l_target)
						if attached {ET_DYNAMIC_SPECIAL_TYPE} current_type as l_special_type then
							print_builtin_any_standard_is_equal_special_items (l_argument, l_target, l_special_type)
						end
						if attached {ET_DYNAMIC_TUPLE_TYPE} current_type as l_tuple_type then
							print_builtin_any_standard_is_equal_tuple_items (l_argument, l_target, l_tuple_type)
						end
					elseif
						current_type.attribute_count > 0 or else
						current_type.is_special or else
						(attached {ET_DYNAMIC_TUPLE_TYPE} current_type as l_tuple_type and then not l_tuple_type.item_type_sets.is_empty) or else
						not current_type.base_class.is_type_class
					then
						print_indentation_assign_to_result
						print_declaration_type_cast (l_result_type, current_file)
						current_file.put_character ('(')
						print_result_name (current_file)
						print_and_then
						print_builtin_any_standard_is_equal_call_with_reference (current_feature, current_type, l_argument_static_type, l_argument_type_set, l_result_type, False)
						current_file.put_character (')')
						print_semicolon_newline
					end
				end
				call_operands.wipe_out
			end
		end

	print_builtin_any_standard_is_equal_field (a_target_field, a_source_field: ET_EXPRESSION; a_field_type_set: ET_DYNAMIC_TYPE_SET)
			-- Print code to check whether the fields `a_target_field' and `a_source_field'
			-- can be considered equal when calling 'standard_is_equal' on their enclosing objects.
			-- A field is either an attribute, a tuple label or a "SPECIAL" item.
			-- `a_target_field' and `a_source_field' have the same dynamic type set `a_field_type_set'.
		require
			a_target_field_not_void: a_target_field /= Void
			a_source_field_not_void: a_source_field /= Void
			a_field_type_set_not_void: a_field_type_set /= Void
		local
			l_field_type: ET_DYNAMIC_PRIMARY_TYPE
			l_equality_expression: ET_EQUALITY_EXPRESSION
			l_index: INTEGER
			l_boolean_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			l_field_type := a_field_type_set.static_type.primary_type
			if l_field_type.is_expanded then
					-- Copy semantics.
				if not attached l_field_type.seeded_dynamic_query (current_system.is_equal_seed, current_dynamic_system) as l_is_equal_feature then
						-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
					set_fatal_error
					error_handler.report_giaaa_error
				else
						-- Call 'is_equal'.
					print_target_operand (a_target_field, l_field_type)
					print_attachment_operand (a_source_field, a_field_type_set)
					fill_call_operands (2)
					print_indentation_assign_to_result
					l_boolean_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.boolean_type, current_type.base_type)
					print_declaration_type_cast (l_boolean_type, current_file)
					current_file.put_character ('(')
					print_result_name (current_file)
					print_and_then
					register_called_feature (l_is_equal_feature)
					print_query_call (l_is_equal_feature, l_field_type, False)
					current_file.put_character (')')
					print_semicolon_newline
					call_operands.wipe_out
				end
			elseif a_field_type_set.has_expanded then
					-- Reference field which may be attached to an object with copy semantics.
				extra_dynamic_type_sets.force_last (current_dynamic_system.boolean_type)
				l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
				l_equality_expression := new_equality_expression (a_target_field, tokens.equal_symbol, a_source_field)
				l_equality_expression.set_index (l_index)
				print_operand (l_equality_expression)
				fill_call_operands (1)
				print_indentation_assign_to_result
				l_boolean_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.boolean_type, current_type.base_type)
				print_declaration_type_cast (l_boolean_type, current_file)
				current_file.put_character ('(')
				print_result_name (current_file)
				print_and_then
				print_expression (call_operands.first)
				current_file.put_character (')')
				print_semicolon_newline
				call_operands.wipe_out
				free_equality_expression (l_equality_expression)
				extra_dynamic_type_sets.remove_last
			else
				print_operand (a_target_field)
				print_operand (a_source_field)
				fill_call_operands (2)
				print_indentation_assign_to_result
				l_boolean_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.boolean_type, current_type.base_type)
				print_declaration_type_cast (l_boolean_type, current_file)
				current_file.put_character ('(')
				print_result_name (current_file)
				print_and_then
				current_file.put_character ('(')
				print_target_expression (call_operands.first, l_field_type, False)
				print_equal_to
				print_attachment_expression (call_operands.item (2), a_field_type_set, l_field_type)
				current_file.put_character (')')
				current_file.put_character (')')
				print_semicolon_newline
				call_operands.wipe_out
			end
		end

	print_builtin_any_standard_is_equal_attributes (a_source, a_target: ET_EXPRESSION)
			-- Print code to check whether the attributes of objects `a_target' and
			-- `a_source' can be considered equal when calling 'standard_is_equal'.
			-- `a_target' and `a_source' are expected to have the same dynamic type
			-- `current_type'.
		require
			a_source_not_void: a_source /= Void
			a_target_not_void: a_target /= Void
		local
			i, nb: INTEGER
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_attribute: ET_DYNAMIC_FEATURE
			l_name: ET_FEATURE_NAME
			l_index: INTEGER
			l_target_attribute: ET_QUALIFIED_CALL_EXPRESSION
			l_source_attribute: ET_QUALIFIED_CALL_EXPRESSION
		do
			nb := current_type.attribute_count
			l_queries := current_type.queries
			from i := 1 until i > nb loop
				l_attribute := l_queries.item (i)
				if not attached l_attribute.result_type_set as l_attribute_type_set then
						-- Internal error: should never happen: queries have a result type set.
					set_fatal_error
					error_handler.report_giaaa_error
				else
						-- Prepare dynamic type sets of attribute expressions.
					extra_dynamic_type_sets.force_last (l_attribute_type_set)
					l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
						-- Prepare call expressions to attribute feature.
					l_name := l_attribute.static_feature.name
					l_name.set_seed (l_attribute.static_feature.first_seed)
					l_target_attribute := new_qualified_call_expression (a_target, l_name, Void)
					l_target_attribute.set_index (l_index)
					l_source_attribute := new_qualified_call_expression (a_source, l_name, Void)
					l_source_attribute.set_index (l_index)
					print_builtin_any_standard_is_equal_field (l_target_attribute, l_source_attribute, l_attribute_type_set)
					free_qualified_call_expression (l_target_attribute)
					free_qualified_call_expression (l_source_attribute)
					extra_dynamic_type_sets.remove_last
				end
				i := i + 1
			end
		end

	print_builtin_any_standard_is_equal_special_items (a_source, a_target: ET_EXPRESSION; a_special_type: ET_DYNAMIC_SPECIAL_TYPE)
			-- Print code to check whether the items of objects `a_target' and
			-- `a_source' can be considered equal when calling 'standard_is_equal'.
			-- `a_target' and `a_source' are expected to have the same dynamic type
			-- `a_special_type'.
		require
			a_source_not_void: a_source /= Void
			a_target_not_void: a_target /= Void
			a_special_type: a_special_type = current_type
		local
			l_count_temp: ET_IDENTIFIER
			l_other_count_temp: ET_IDENTIFIER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_count_type: ET_DYNAMIC_PRIMARY_TYPE
			l_i_temp: ET_IDENTIFIER
			l_item_type: ET_DYNAMIC_PRIMARY_TYPE
			l_name: ET_FEATURE_NAME
			l_index: INTEGER
			l_target_item: ET_QUALIFIED_CALL_EXPRESSION
			l_source_item: ET_QUALIFIED_CALL_EXPRESSION
			l_actual_argument_list: ET_ACTUAL_ARGUMENT_LIST
			l_boolean_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if a_special_type.attribute_count < 2 then
					-- Internal error: class "SPECIAL" should have at least the
					-- features 'count' and 'capacity' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached current_type.seeded_dynamic_query (current_system.special_item_seed, current_dynamic_system) as l_item_routine then
					-- Internal error: this error should have been reported
					-- in `ET_DYNAMIC_SYSTEM.compile_kernel'.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached l_item_routine.result_type_set as l_item_type_set then
					-- Internal error: should never happen: queries have a result type set.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_item_type := l_item_type_set.static_type.primary_type
				l_dynamic_type_set := result_type_set_in_feature (a_special_type.queries.first)
				l_count_type := l_dynamic_type_set.static_type.primary_type
				l_count_temp := new_temp_variable (l_count_type)
				mark_temp_variable_frozen (l_count_temp)
				l_other_count_temp := new_temp_variable (l_count_type)
				mark_temp_variable_frozen (l_other_count_temp)
				print_assign_special_count_attribute_to_temp_variable (l_other_count_temp, a_source, a_special_type, True)
				print_assign_special_count_attribute_to_temp_variable (l_count_temp, a_target, a_special_type, False)
				print_indentation_if
				print_temp_name (l_other_count_temp, current_file)
				print_equal_to
				print_temp_name (l_count_temp, current_file)
				print_then_newline
				indent
				if a_special_type.has_item_nested_custom_standard_is_equal_routine then
					print_indentation
					l_i_temp := new_temp_variable (l_count_type)
					mark_temp_variable_frozen (l_i_temp)
					print_indentation
					current_file.put_string (c_for)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_temp_name (l_i_temp, current_file)
					print_assign_to
					print_temp_name (l_count_temp, current_file)
					print_minus
					current_file.put_character ('1')
					current_file.put_character (';')
					current_file.put_character (' ')
					print_temp_name (l_i_temp, current_file)
					print_greater_than_or_equal
					current_file.put_character ('0')
					current_file.put_character (';')
					current_file.put_character (' ')
					print_temp_name (l_i_temp, current_file)
					current_file.put_character ('-')
					current_file.put_character ('-')
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
						-- Prepare dynamic type sets of item expressions.
					extra_dynamic_type_sets.force_last (l_count_type)
					l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
					l_i_temp.set_index (l_index)
					extra_dynamic_type_sets.force_last (l_item_type_set)
					l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
						-- Prepare call expressions to 'item' feature.
					l_name := l_item_routine.static_feature.name
					l_name.set_seed (l_item_routine.static_feature.first_seed)
					l_actual_argument_list := new_actual_argument_list
					l_actual_argument_list.force_first (l_i_temp)
					l_target_item := new_qualified_call_expression (a_target, l_name, l_actual_argument_list)
					l_target_item.set_index (l_index)
					l_source_item := new_qualified_call_expression (a_source, l_name, l_actual_argument_list)
					l_source_item.set_index (l_index)
					print_builtin_any_standard_is_equal_field (l_target_item, l_source_item, l_item_type_set)
					free_qualified_call_expression (l_target_item)
					free_qualified_call_expression (l_source_item)
					free_actual_argument_list (l_actual_argument_list)
					extra_dynamic_type_sets.remove_last
					extra_dynamic_type_sets.remove_last
					mark_temp_variable_unfrozen (l_i_temp)
					mark_temp_variable_free (l_i_temp)
					dedent
					print_indentation_end_newline
				else
					print_indentation_assign_to_result
					l_boolean_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.boolean_type, current_type.base_type)
					print_declaration_type_cast (l_boolean_type, current_file)
					current_file.put_character ('(')
					print_result_name (current_file)
					print_and_then
					current_file.put_character ('(')
					current_file.put_character ('!')
					current_file.put_string (c_memcmp)
					current_file.put_character ('(')
					print_attribute_special_item_access (a_target, a_special_type, False)
					print_comma
					print_attribute_special_item_access (a_source, a_special_type, False)
					print_comma
					print_temp_name (l_count_temp, current_file)
					print_times
					current_file.put_string (c_sizeof)
					current_file.put_character ('(')
					print_type_declaration (l_item_type, current_file)
					current_file.put_character (')')
					current_file.put_character (')')
					current_file.put_character (')')
					current_file.put_character (')')
					print_semicolon_newline
				end
				dedent
				print_indentation_else_newline
				indent
				print_indentation_assign_to_result
				current_file.put_string (c_eif_false)
				print_semicolon_newline
				dedent
				print_indentation_end_newline
				mark_temp_variable_unfrozen (l_count_temp)
				mark_temp_variable_free (l_count_temp)
				mark_temp_variable_unfrozen (l_other_count_temp)
				mark_temp_variable_free (l_other_count_temp)
			end
		end

	print_builtin_any_standard_is_equal_tuple_items (a_source, a_target: ET_EXPRESSION; a_tuple_type: ET_DYNAMIC_TUPLE_TYPE)
			-- Print code to check whether the items of objects `a_target' and
			-- `a_source' can be considered equal when calling 'standard_is_equal'.
			-- `a_target' and `a_source' are expected to have the same dynamic type
			-- `a_tuple_type'.
		require
			a_source_not_void: a_source /= Void
			a_target_not_void: a_target /= Void
			a_tuple_typed: a_tuple_type = current_type
		local
			i, nb: INTEGER
			l_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_item_type_set: ET_DYNAMIC_TYPE_SET
			l_index: INTEGER
			l_tuple_label: ET_IDENTIFIER
			l_target_item: ET_QUALIFIED_CALL_EXPRESSION
			l_source_item: ET_QUALIFIED_CALL_EXPRESSION
		do
			l_item_type_sets := a_tuple_type.item_type_sets
			nb := l_item_type_sets.count
			from i := 1 until i > nb loop
				l_item_type_set := l_item_type_sets.item (i)
					-- Prepare dynamic type sets of item expressions.
				extra_dynamic_type_sets.force_last (l_item_type_set)
				l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
					-- Prepare call expressions to attribute feature.
				l_tuple_label := new_tuple_label (i)
				l_target_item := new_qualified_call_expression (a_target, l_tuple_label, Void)
				l_target_item.set_index (l_index)
				l_source_item := new_qualified_call_expression (a_target, l_tuple_label, Void)
				l_source_item.set_index (l_index)
				print_builtin_any_standard_is_equal_field (l_target_item, l_source_item, l_item_type_set)
				free_qualified_call_expression (l_target_item)
				free_qualified_call_expression (l_source_item)
				free_tuple_label (l_tuple_label)
				extra_dynamic_type_sets.remove_last
				i := i + 1
			end
		end

	print_builtin_any_standard_is_equal_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ANY.standard_is_equal'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				l_argument_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				l_result_type := l_result_type_set.static_type.primary_type
					-- The postcondition of 'ANY.standard_is_equal' indicates
					-- that the objects need to be of the same type.
				if not l_argument_type_set.has_type (a_target_type) then
						-- The objects are not of the same type.
					current_file.put_string (c_eif_false)
				elseif a_target_type.has_nested_custom_standard_is_equal_routine or l_argument_type_set.count /= 1 then
					register_called_feature (a_feature)
					print_routine_name (a_feature, a_target_type, current_file)
					current_file.put_character ('(')
					current_file.put_string (c_ac)
					print_comma
					print_target_expression (l_target, a_target_type, a_check_void_target)
					print_comma
					print_attachment_expression (l_argument, l_argument_type_set, l_formal_type)
					current_file.put_character (')')
				elseif a_target_type.is_expanded then
					print_builtin_any_standard_is_equal_call_with_expanded (a_feature, a_target_type, l_formal_type, l_argument_type_set, l_result_type, a_check_void_target)
				else
						-- `a_target_type' is one of the types held in `l_argument_type_set'
						-- (see the if-branch above). So when `l_argument_type_set' has
						-- exactly one type then we know that `a_target_type' is this type.
					print_builtin_any_standard_is_equal_call_with_reference (a_feature, a_target_type, l_formal_type, l_argument_type_set, l_result_type, a_check_void_target)
				end
			end
		end

	print_builtin_any_standard_is_equal_call_with_expanded (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE;
		a_argument_formal_type: ET_DYNAMIC_TYPE; a_argument_type_set: ET_DYNAMIC_TYPE_SET;
		a_result_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ANY.standard_is_equal'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_argument_formal_type' is the formal type of the argument.
			-- `a_argument_type_set' is the dynamic type set of the argument.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			a_target_type_is_expanded: a_target_type.is_expanded
			a_target_type_not_has_nested_custom_standard_is_equal_routine: not a_target_type.has_nested_custom_standard_is_equal_routine
			a_argument_formal_type_not_void: a_argument_formal_type /= Void
			a_argument_type_set_not_void: a_argument_type_set /= Void
			a_argument_type_set_has_target_type: a_argument_type_set.has_type (a_target_type)
			a_result_type_not_void: a_result_type /= Void
			two_operands: call_operands.count = 2
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
		do
				-- We know that the only type which conforms to an expanded
				-- type is the type itself. So no need to check that the
				-- objects are of the same type here. This will be done through
				-- a check for CAT-call in the call to `print_attachment_expression'
				-- below.
			if
				not a_target_type.is_basic and then
				a_target_type.attribute_count = 0
			then
				current_file.put_string (c_eif_true)
			elseif total_order_on_reals_mode and then a_target_type = current_dynamic_system.real_32_type and then not a_target_type.has_redefined_is_equal_routine then
				include_runtime_header_file ("ge_real.h", False, header_file)
				print_builtin_query_c_call (a_feature, c_ge_real_32_is_equal, False, a_target_type, a_check_void_target)
			elseif total_order_on_reals_mode and then a_target_type = current_dynamic_system.real_64_type and then not a_target_type.has_redefined_is_equal_routine then
				include_runtime_header_file ("ge_real.h", False, header_file)
				print_builtin_query_c_call (a_feature, c_ge_real_64_is_equal, False, a_target_type, a_check_void_target)
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_declaration_type_cast (a_result_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				current_file.put_character (')')
				print_equal_to
				current_file.put_character ('(')
				print_attachment_expression (l_argument, a_argument_type_set, a_argument_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_any_standard_is_equal_call_with_reference (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE;
		a_argument_formal_type: ET_DYNAMIC_TYPE; a_argument_type_set: ET_DYNAMIC_TYPE_SET;
		a_result_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ANY.standard_is_equal'.
			-- `The target and the argument are expected to have the same dynamic type.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_argument_formal_type' is the formal type of the argument.
			-- `a_argument_type_set' is the dynamic type set of the argument.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			a_target_type_is_reference: not a_target_type.is_expanded
			a_target_type_not_has_nested_custom_standard_is_equal_routine: not a_target_type.has_nested_custom_standard_is_equal_routine
			a_argument_formal_type_not_void: a_argument_formal_type /= Void
			a_argument_type_set_not_void: a_argument_type_set /= Void
			a_argument_type_set_has_target_type: a_argument_type_set.has_type (a_target_type)
			a_result_type_not_void: a_result_type /= Void
			two_operands: call_operands.count = 2
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
		do
			if a_target_type.base_class.is_type_class then
					-- Cannot have two instances of class "TYPE" representing the same Eiffel type.
				current_file.put_string (c_eif_true)
			elseif
				a_target_type.attribute_count = 0 and then
				not a_target_type.is_special and then
				(not attached {ET_DYNAMIC_TUPLE_TYPE} a_target_type as l_tuple_type or else l_tuple_type.item_type_sets.is_empty)
			then
					-- No field to compare with.
				current_file.put_string (c_eif_true)
			else
					-- Note: do not compare the flags.
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_declaration_type_cast (a_result_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('!')
				current_file.put_string (c_memcmp)
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_string (c_char)
				current_file.put_character ('*')
				current_file.put_character (')')
				current_file.put_character ('(')
				print_target_expression (l_target, a_target_type, a_check_void_target)
				current_file.put_character (')')
				current_file.put_character (')')
				print_plus
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				print_eif_any_type_name (current_file)
				current_file.put_character (')')
				print_comma
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_string (c_char)
				current_file.put_character ('*')
				current_file.put_character (')')
				current_file.put_character ('(')
				print_attachment_expression (l_argument, a_argument_type_set, a_argument_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
				print_plus
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				print_eif_any_type_name (current_file)
				current_file.put_character (')')
				print_comma
				if attached {ET_DYNAMIC_SPECIAL_TYPE} a_target_type as l_special_type then
					print_attribute_special_offset_access (l_target, l_special_type, False)
					print_plus
					print_attribute_special_count_access (l_target, l_special_type, False)
					print_times
					current_file.put_string (c_sizeof)
					current_file.put_character ('(')
					print_type_declaration (l_special_type.item_type_set.static_type.primary_type, current_file)
					current_file.put_character (')')
				else
					current_file.put_string (c_sizeof)
					current_file.put_character ('(')
					print_type_name (a_target_type, current_file)
					current_file.put_character (')')
				end
				print_minus
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				print_eif_any_type_name (current_file)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_any_standard_copy_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'ANY.standard_copy'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_argument: ET_IDENTIFIER
		do
			l_arguments := a_feature.arguments
			if l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by ET_BUILTIN_FEATURE_CHECKER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif current_type.base_class.is_type_class then
				-- Cannot have two instances of class "TYPE" representing the same Eiffel type.
			elseif attached {ET_DYNAMIC_SPECIAL_TYPE} current_type as l_special_type then
				l_argument := l_arguments.formal_argument (1).name
				print_builtin_any_standard_copy_body_with_special (l_argument, tokens.current_keyword, l_special_type)
			else
				l_argument := l_arguments.formal_argument (1).name
				print_builtin_any_standard_copy_body_with_no_special (l_argument, tokens.current_keyword)
			end
		end

	print_builtin_any_standard_copy_body_with_no_special (a_source, a_target: ET_EXPRESSION)
			-- Print to `current_file' the body of the built-in feature 'ANY.standard_copy'
			-- where attributes and items of `a_source' are copied to object `a_target'.
			-- `a_source' and `a_target' are expected to have the same static type `current_type'.
		require
			a_source_not_void: a_source /= Void
			a_target_not_void: a_target /= Void
			not_special_type: not current_type.is_special
		do
			if
				not current_type.is_basic and then
				current_type.attribute_count = 0 and then
				(not attached {ET_DYNAMIC_TUPLE_TYPE} current_type as l_tuple_type or else l_tuple_type.item_type_sets.is_empty)
			then
				-- Nothing to be copied.
			else
				print_assign_flags_attribute_to_temp_variable (a_target, current_type, False)
				print_assign_onces_attribute_to_temp_variable (a_target, current_type, False)
				print_indentation
				if current_type.is_expanded then
					print_unboxed_expression (a_target, current_type, False)
				else
					current_file.put_character ('*')
					print_type_cast (current_type, current_file)
					current_file.put_character ('(')
					print_procedure_target_expression (a_target, current_type, False)
					current_file.put_character (')')
				end
				print_assign_to
				if not current_type.is_expanded then
					current_file.put_character ('*')
				end
				print_type_cast (current_type, current_file)
				current_file.put_character ('(')
				print_attachment_expression (a_source, dynamic_type_set (a_source), current_type)
				current_file.put_character (')')
				print_semicolon_newline
				print_assign_temp_variable_to_flags_attribute (a_target, current_type, False)
				print_assign_temp_variable_to_onces_attribute (a_target, current_type, False)
				print_builtin_any_standard_copy_custom_attributes (a_source, a_target)
				if attached {ET_DYNAMIC_TUPLE_TYPE} current_type as l_tuple_type then
					print_builtin_any_standard_copy_custom_tuple_items (a_source, a_target, l_tuple_type)
				end
			end
		end

	print_builtin_any_standard_copy_body_with_special (a_source, a_target: ET_EXPRESSION; a_special_type: ET_DYNAMIC_SPECIAL_TYPE)
			-- Print to `current_file' the body of the built-in feature 'ANY.standard_copy'
			-- where attributes and items of `a_source' are copied to object `a_target'.
			-- `a_source' and `a_target' are expected to have the same static type `a_special_type'.
			--
			-- Implementation of ISE's behavior:
			-- * When target has more items than source:
			--   * All items of source will be copied to target.
			--   * The other items in target will not be changed.
			--   * The 'count' and 'capacity' in target will not change.
			--   * It violates 'copy's postcondition "is_equal".
			-- * When target has less items than source:
			--   * All items of target will be replaced by the first items in source.
			--   * The 'count' and 'capacity' in target will not change.
			--   * It violates 'copy's postcondition "is_equal".
		require
			a_source_not_void: a_source /= Void
			a_target_not_void: a_target /= Void
			a_special_type: a_special_type = current_type
		local
			l_count_temp: ET_IDENTIFIER
			l_capacity_temp: ET_IDENTIFIER
			l_other_count_temp: ET_IDENTIFIER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_count_type: ET_DYNAMIC_PRIMARY_TYPE
			l_capacity_type: ET_DYNAMIC_PRIMARY_TYPE
			l_item_type_set: ET_DYNAMIC_TYPE_SET
			l_item_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if a_special_type.attribute_count < 2 then
					-- Internal error: class "SPECIAL" should have at least the
					-- features 'count' and 'capacity' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_item_type_set := a_special_type.item_type_set
				l_item_type := l_item_type_set.static_type.primary_type
				l_dynamic_type_set := result_type_set_in_feature (a_special_type.queries.first)
				l_count_type := l_dynamic_type_set.static_type.primary_type
				l_count_temp := new_temp_variable (l_count_type)
				mark_temp_variable_frozen (l_count_temp)
				l_other_count_temp := new_temp_variable (l_count_type)
				mark_temp_variable_frozen (l_other_count_temp)
				l_dynamic_type_set := result_type_set_in_feature (a_special_type.queries.item (2))
				l_capacity_type := l_dynamic_type_set.static_type.primary_type
				l_capacity_temp := new_temp_variable (l_capacity_type)
				mark_temp_variable_frozen (l_capacity_temp)
				print_assign_special_count_attribute_to_temp_variable (l_other_count_temp, a_source, a_special_type, True)
				print_assign_special_count_attribute_to_temp_variable (l_count_temp, a_target, a_special_type, False)
				print_indentation_if
				print_temp_name (l_other_count_temp, current_file)
				print_greater_than
				print_temp_name (l_count_temp, current_file)
				print_then_newline
				indent
				print_indentation
				print_temp_name (l_other_count_temp, current_file)
				print_assign_to
				print_temp_name (l_count_temp, current_file)
				print_semicolon_newline
				dedent
				print_indentation_end_newline
				print_indentation_if
				print_temp_name (l_other_count_temp, current_file)
				print_greater_than
				current_file.put_character ('0')
				print_then_newline
				indent
				print_assign_flags_attribute_to_temp_variable (a_target, a_special_type, False)
				print_assign_onces_attribute_to_temp_variable (a_target, a_special_type, False)
				print_assign_special_capacity_attribute_to_temp_variable (l_capacity_temp, a_target, a_special_type, False)
					-- Do not use "*(T123*)(C) = *(T123*)(a1);" because we might overwrite
					-- some items due to struct padding (unused bytes added at the end of
					-- the struct for memory alignment).
				print_indentation
				current_file.put_string (c_memcpy)
				current_file.put_character ('(')
				print_procedure_target_expression (a_target, a_special_type, False)
				print_comma
				print_attachment_expression (a_source, dynamic_type_set (a_source), a_special_type)
				print_comma
				print_attribute_special_offset_access (a_target, a_special_type, False)
				print_plus
				print_temp_name (l_other_count_temp, current_file)
				print_times
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				print_type_declaration (l_item_type, current_file)
				current_file.put_character (')')
				current_file.put_character (')')
				print_semicolon_newline
				print_assign_temp_variable_to_flags_attribute (a_target, a_special_type, False)
				print_assign_temp_variable_to_onces_attribute (a_target, a_special_type, False)
				print_assign_temp_variable_to_special_count_attribute (l_count_temp, a_target, a_special_type, False)
				print_assign_temp_variable_to_special_capacity_attribute (l_capacity_temp, a_target, a_special_type, False)
				print_builtin_any_standard_copy_custom_attributes (a_source, a_target)
				print_builtin_any_standard_copy_custom_special_items (a_source, a_target, a_special_type, l_other_count_temp)
				mark_temp_variable_unfrozen (l_count_temp)
				mark_temp_variable_free (l_count_temp)
				mark_temp_variable_unfrozen (l_other_count_temp)
				mark_temp_variable_free (l_other_count_temp)
				mark_temp_variable_unfrozen (l_capacity_temp)
				mark_temp_variable_free (l_capacity_temp)
				dedent
				print_indentation_end_newline
			end
		end

	print_builtin_any_standard_copy_custom_attributes (a_source, a_target: ET_EXPRESSION)
			-- Special treatment, if any, when copying the attributes of object
			-- `a_source' to object `a_target' when calling 'standard_copy'.
			-- `a_source' and `a_target' are expected to have the same dynamic type `current_type'.
			-- Note that the attributes of `a_source' and `a_target' are expected to have
			-- the same content in memory when calling this routine. So only attributes
			-- which require a different semantics need to be processed.
		require
			a_source_not_void: a_source /= Void
			a_target_not_void: a_target /= Void
		local
			i, nb: INTEGER
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_attribute: ET_DYNAMIC_FEATURE
			l_attribute_type: ET_DYNAMIC_PRIMARY_TYPE
			l_name: ET_FEATURE_NAME
			l_index: INTEGER
			l_source_attribute: ET_QUALIFIED_CALL_EXPRESSION
			l_attribute_already_copied: BOOLEAN
		do
			nb := current_type.attribute_count
			l_queries := current_type.queries
			from i := 1 until i > nb loop
				l_attribute := l_queries.item (i)
				if not attached l_attribute.result_type_set as l_attribute_type_set then
						-- Internal error: should never happen: queries have a result type set.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					l_attribute_type := l_attribute_type_set.static_type.primary_type
					if l_attribute_type.is_basic then
						l_attribute_already_copied := True
					elseif l_attribute_type.is_expanded then
						l_attribute_already_copied := not l_attribute_type.has_redefined_copy_routine and not l_attribute_type.has_nested_custom_standard_copy_routine
					elseif l_attribute_type_set.has_expanded then
						-- Reference field which may be attached to an object with copy semantics.
					else
						l_attribute_already_copied := True
					end
					if not l_attribute_already_copied then
							-- Prepare dynamic type sets of attribute expression.
						extra_dynamic_type_sets.force_last (l_attribute_type_set)
						l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
							-- Prepare call expression to attribute feature.
						l_name := l_attribute.static_feature.name
						l_name.set_seed (l_attribute.static_feature.first_seed)
						l_source_attribute := new_qualified_call_expression (a_source, l_name, Void)
						l_source_attribute.set_index (l_index)
							-- The clone of the item with copy semantics occurs here
							-- in the call to `print_attachment_operand'.
						print_attachment_operand (l_source_attribute, l_attribute_type_set)
						fill_call_operands (1)
						print_indentation
						print_attribute_access (l_attribute, a_target, current_type, False)
						print_assign_to
						print_attachment_expression (call_operands.first, l_attribute_type_set, l_attribute_type)
						print_semicolon_newline
							-- Clean up.
						call_operands.wipe_out
						free_qualified_call_expression (l_source_attribute)
						extra_dynamic_type_sets.remove_last
					end
				end
				i := i + 1
			end
		end

	print_builtin_any_standard_copy_custom_special_items (a_source, a_target: ET_EXPRESSION; a_special_type: ET_DYNAMIC_SPECIAL_TYPE; a_item_count: ET_IDENTIFIER)
			-- Special treatment, if any, when copying the items of object
			-- `a_source' to object `a_target' when calling 'standard_copy'.
			-- `a_source' and `a_target' are expected to have the same dynamic type `a_special_type'.
			-- `a_item_count' is a temporary variable containing the number of items to be processed.
			-- Note that the items of `a_source' and `a_target' are expected to have
			-- the same content in memory when calling this routine. So only items
			-- which require a different semantics need to be processed.
		require
			a_source_not_void: a_source /= Void
			a_target_not_void: a_target /= Void
			a_special_type: a_special_type = current_type
			a_item_count_not_void: a_item_count /= Void
			a_item_count_is_temporary: a_item_count.is_temporary
			a_item_count_is_known: is_temp_variable_known (a_item_count)
		local
			l_i_temp: ET_IDENTIFIER
			l_item_type: ET_DYNAMIC_PRIMARY_TYPE
			l_name: ET_FEATURE_NAME
			l_index: INTEGER
			l_source_item: ET_QUALIFIED_CALL_EXPRESSION
			l_actual_argument_list: ET_ACTUAL_ARGUMENT_LIST
			l_item_already_copied: BOOLEAN
		do
			if not a_special_type.has_item_nested_custom_standard_copy_routine then
				-- Items already copied.
			elseif not attached current_type.seeded_dynamic_query (current_system.special_item_seed, current_dynamic_system) as l_item_routine then
					-- Internal error: this error should have been reported
					-- in `ET_DYNAMIC_SYSTEM.compile_kernel'.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached l_item_routine.result_type_set as l_item_type_set then
					-- Internal error: should never happen: queries have a result type set.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached temp_variable_type (a_item_count) as l_item_count_type then
					-- Internal error: the type of the temporary
					-- variable should be known at this stage.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_item_type := l_item_type_set.static_type.primary_type
				if l_item_type.is_basic then
					l_item_already_copied := True
				elseif l_item_type.is_expanded then
					l_item_already_copied := not l_item_type.has_redefined_copy_routine and not l_item_type.has_nested_custom_standard_copy_routine
				elseif l_item_type_set.has_expanded then
					-- Reference item which may be attached to an object with copy semantics.
				else
					l_item_already_copied := True
				end
				if not l_item_already_copied then
					l_i_temp := new_temp_variable (l_item_count_type)
					mark_temp_variable_frozen (l_i_temp)
					print_indentation
					current_file.put_string (c_for)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_temp_name (l_i_temp, current_file)
					print_assign_to
					print_temp_name (a_item_count, current_file)
					print_minus
					current_file.put_character ('1')
					current_file.put_character (';')
					current_file.put_character (' ')
					print_temp_name (l_i_temp, current_file)
					print_greater_than_or_equal
					current_file.put_character ('0')
					current_file.put_character (';')
					current_file.put_character (' ')
					print_temp_name (l_i_temp, current_file)
					current_file.put_character ('-')
					current_file.put_character ('-')
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					indent
						-- Prepare dynamic type sets of item expression.
					extra_dynamic_type_sets.force_last (l_item_count_type)
					l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
					l_i_temp.set_index (l_index)
					extra_dynamic_type_sets.force_last (l_item_type_set)
					l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
						-- Prepare call expressions to 'item' feature.
					l_name := l_item_routine.static_feature.name
					l_name.set_seed (l_item_routine.static_feature.first_seed)
					l_actual_argument_list := new_actual_argument_list
					l_actual_argument_list.force_first (l_i_temp)
					l_source_item := new_qualified_call_expression (a_source, l_name, l_actual_argument_list)
					l_source_item.set_index (l_index)
						-- The clone of the item with copy semantics occurs here
						-- in the call to `print_attachment_operand'.
					print_attachment_operand (l_source_item, l_item_type_set)
					fill_call_operands (1)
					print_indentation
					print_attribute_special_item_access (a_target, a_special_type, False)
					current_file.put_character ('[')
					print_temp_name (l_i_temp, current_file)
					current_file.put_character (']')
					print_assign_to
					print_attachment_expression (call_operands.first, l_item_type_set, l_item_type)
					print_semicolon_newline
					dedent
					print_indentation_end_newline
						-- Clean up.
					call_operands.wipe_out
					free_qualified_call_expression (l_source_item)
					free_actual_argument_list (l_actual_argument_list)
					extra_dynamic_type_sets.remove_last
					extra_dynamic_type_sets.remove_last
					mark_temp_variable_unfrozen (l_i_temp)
					mark_temp_variable_free (l_i_temp)
				end
			end
		end

	print_builtin_any_standard_copy_custom_tuple_items (a_source, a_target: ET_EXPRESSION; a_tuple_type: ET_DYNAMIC_TUPLE_TYPE)
			-- Special treatment, if any, when copying the items of object
			-- `a_source' to object `a_target' when calling 'standard_copy'.
			-- `a_source' and `a_target' are expected to have the same dynamic type `a_tuple_type'.
			-- Note that the items of `a_source' and `a_target' are expected to have
			-- the same content in memory when calling this routine. So only items
			-- which require a different semantics need to be processed.
		require
			a_source_not_void: a_source /= Void
			a_target_not_void: a_target /= Void
			a_tuple_typed: a_tuple_type = current_type
		local
			i, nb: INTEGER
			l_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_item_type_set: ET_DYNAMIC_TYPE_SET
			l_item_type: ET_DYNAMIC_PRIMARY_TYPE
			l_index: INTEGER
			l_tuple_label: ET_IDENTIFIER
			l_source_item: ET_QUALIFIED_CALL_EXPRESSION
			l_item_already_copied: BOOLEAN
		do
			l_item_type_sets := a_tuple_type.item_type_sets
			nb := l_item_type_sets.count
			from i := 1 until i > nb loop
				l_item_type_set := l_item_type_sets.item (i)
				l_item_type := l_item_type_set.static_type.primary_type
				if l_item_type.is_basic then
					l_item_already_copied := True
				elseif l_item_type.is_expanded then
					l_item_already_copied := not l_item_type.has_redefined_copy_routine and not l_item_type.has_nested_custom_standard_copy_routine
				elseif l_item_type_set.has_expanded then
					-- Reference item which may be attached to an object with copy semantics.
				else
					l_item_already_copied := True
				end
				if not l_item_already_copied then
						-- Prepare dynamic type sets of item expressions.
					extra_dynamic_type_sets.force_last (l_item_type_set)
					l_index := current_dynamic_type_sets.count + extra_dynamic_type_sets.count
						-- Prepare call expression to tuple item.
					l_tuple_label := new_tuple_label (i)
					l_source_item := new_qualified_call_expression (a_source, l_tuple_label, Void)
					l_source_item.set_index (l_index)
						-- The clone of the item with copy semantics occurs here
						-- in the call to `print_attachment_operand'.
					print_attachment_operand (l_source_item, l_item_type_set)
					fill_call_operands (1)
					print_indentation
					print_attribute_tuple_item_access (i, a_target, a_tuple_type, False)
					print_assign_to
					print_attachment_expression (call_operands.first, l_item_type_set, l_item_type)
					print_semicolon_newline
						-- Clean up.
					call_operands.wipe_out
					free_qualified_call_expression (l_source_item)
					free_tuple_label (l_tuple_label)
					extra_dynamic_type_sets.remove_last
				end
				i := i + 1
			end
		end

	print_builtin_any_standard_copy_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static_binding) to `a_feature'
			-- corresponding to built-in feature 'ANY.standard_copy'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif a_target_type.base_class.is_type_class then
				-- Cannot have two instances of class "TYPE" representing the same Eiffel type.
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				l_argument_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				if a_target_type.is_special or a_target_type.has_nested_custom_standard_copy_routine then
					register_called_feature (a_feature)
					print_indentation
					print_routine_name (a_feature, a_target_type, current_file)
					current_file.put_character ('(')
					current_file.put_string (c_ac)
					print_comma
					print_procedure_target_expression (l_target, a_target_type, a_check_void_target)
					print_comma
					print_attachment_expression (l_argument, l_argument_type_set, l_formal_type)
					current_file.put_character (')')
					print_semicolon_newline
				elseif
					not a_target_type.is_basic and then
					a_target_type.attribute_count = 0 and then
					(not attached {ET_DYNAMIC_TUPLE_TYPE} a_target_type as l_tuple_type or else l_tuple_type.item_type_sets.is_empty)
				then
					-- Nothing to be copied.
				elseif a_target_type.is_expanded then
					print_indentation
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
					print_assign_to
					current_file.put_character ('(')
					print_attachment_expression (l_argument, l_argument_type_set, l_formal_type)
					current_file.put_character (')')
					print_semicolon_newline
				else
						-- Note: do not copy the flags.
					print_indentation
					current_file.put_string (c_memcpy)
					current_file.put_character ('(')
					current_file.put_character ('(')
					current_file.put_character ('(')
					current_file.put_string (c_char)
					current_file.put_character ('*')
					current_file.put_character (')')
					current_file.put_character ('(')
					print_procedure_target_expression (l_target, a_target_type, a_check_void_target)
					current_file.put_character (')')
					current_file.put_character (')')
					print_plus
					current_file.put_string (c_sizeof)
					current_file.put_character ('(')
					print_eif_any_type_name (current_file)
					current_file.put_character (')')
					print_comma
					current_file.put_character ('(')
					current_file.put_character ('(')
					current_file.put_string (c_char)
					current_file.put_character ('*')
					current_file.put_character (')')
					current_file.put_character ('(')
					print_attachment_expression (l_argument, l_argument_type_set, l_formal_type)
					current_file.put_character (')')
					current_file.put_character (')')
					print_plus
					current_file.put_string (c_sizeof)
					current_file.put_character ('(')
					print_eif_any_type_name (current_file)
					current_file.put_character (')')
					print_comma
					current_file.put_string (c_sizeof)
					current_file.put_character ('(')
					print_type_name (a_target_type, current_file)
					current_file.put_character (')')
					print_minus
					current_file.put_string (c_sizeof)
					current_file.put_character ('(')
					print_eif_any_type_name (current_file)
					current_file.put_character (')')
					current_file.put_character (')')
					print_semicolon_newline
				end
			end
		end

	print_builtin_any_standard_twin_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'ANY.standard_twin'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			if current_type.base_class.is_type_class then
					-- Cannot have two instances of class TYPE representing the same Eiffel type.
				print_indentation_assign_to_result
				print_current_name (current_file)
				print_semicolon_newline
			elseif attached {ET_DYNAMIC_SPECIAL_TYPE} current_type as l_special_type then
				print_builtin_any_standard_twin_body_with_special (l_special_type)
			else
				print_builtin_any_standard_twin_body_with_no_special
			end
		end

	print_builtin_any_standard_twin_body_with_no_special
			-- Print to `current_file' the body of the built-in feature
			-- 'ANY.standard_twin' for objects of static type `current_type'.
		require
			not_special_type: not current_type.is_special
		local
			l_result: ET_RESULT
		do
			l_result := new_result_expression
			extra_dynamic_type_sets.force_last (current_type)
			l_result.set_index (current_dynamic_type_sets.count + extra_dynamic_type_sets.count)
			if not current_type.is_expanded then
				print_indentation_assign_to_result
				current_file.put_string (c_ge_new)
				current_file.put_integer (current_type.id)
				current_file.put_character ('(')
				current_file.put_string (c_eif_true)
				current_file.put_character (')')
				print_semicolon_newline
			end
			print_builtin_any_standard_copy_body_with_no_special (tokens.current_keyword, l_result)
			free_result_expression (l_result)
			extra_dynamic_type_sets.remove_last
		end

	print_builtin_any_standard_twin_body_with_special (a_special_type: ET_DYNAMIC_SPECIAL_TYPE)
			-- Print to `current_file' the body of the built-in feature
			-- 'ANY.standard_twin' for objects of static type `a_special_type'.
			--
			-- Implementation of ISE's behavior:
			-- * 'standard_twin' sets the capacity of the new object to
			--   the same value as 'count', which may be different from
			--   the capacity of the original object.
		require
			a_special_type: a_special_type = current_type
		local
			l_count_temp: ET_IDENTIFIER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_count_type: ET_DYNAMIC_PRIMARY_TYPE
			l_result: ET_RESULT
		do
			if a_special_type.attribute_count < 2 then
					-- Internal error: class "SPECIAL" should have at least the
					-- features 'count' and 'capacity' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_result := new_result_expression
				extra_dynamic_type_sets.force_last (a_special_type)
				l_result.set_index (current_dynamic_type_sets.count + extra_dynamic_type_sets.count)
				l_dynamic_type_set := result_type_set_in_feature (a_special_type.queries.first)
				l_count_type := l_dynamic_type_set.static_type.primary_type
				l_count_temp := new_temp_variable (l_count_type)
				mark_temp_variable_frozen (l_count_temp)
				print_assign_special_count_attribute_to_temp_variable (l_count_temp, tokens.current_keyword, a_special_type, False)
				print_indentation_assign_to_result
				current_file.put_string (c_ge_new)
				current_file.put_integer (a_special_type.id)
				current_file.put_character ('(')
				print_temp_name (l_count_temp, current_file)
				print_comma
				current_file.put_string (c_eif_true)
				current_file.put_character (')')
				print_semicolon_newline
				print_indentation
				print_attribute_special_count_access (l_result, a_special_type, False)
				print_assign_to
				print_temp_name (l_count_temp, current_file)
				print_semicolon_newline
				mark_temp_variable_unfrozen (l_count_temp)
				mark_temp_variable_free (l_count_temp)
				print_builtin_any_standard_copy_body_with_special (tokens.current_keyword, l_result, a_special_type)
				free_result_expression (l_result)
				extra_dynamic_type_sets.remove_last
			end
		end

	print_builtin_any_standard_twin_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ANY.standard_twin'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if call_operands.count /= 1 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				if a_target_type.base_class.is_type_class then
						-- Cannot have two instances of class "TYPE" representing the same Eiffel type.
					print_target_expression (l_target, a_target_type, a_check_void_target)
				elseif a_target_type.is_expanded and then not a_target_type.has_nested_custom_standard_copy_routine then
					current_file.put_character ('*')
					print_target_expression (l_target, a_target_type, a_check_void_target)
				else
					register_called_feature (a_feature)
					print_routine_name (a_feature, a_target_type, current_file)
					current_file.put_character ('(')
					current_file.put_string (c_ac)
					print_comma
					print_target_expression (l_target, a_target_type, a_check_void_target)
					current_file.put_character (')')
				end
			end
		end

	print_builtin_any_tagged_out_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'ANY.tagged_out'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_string: STRING
		do
-- TODO
			print_indentation
			print_result_name (current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			l_string := current_type.base_type.unaliased_to_text
			if attached current_feature.result_type_set as l_result_type_set and then l_result_type_set.static_type.base_type.same_named_type_with_type_marks (current_system.string_32_type, tokens.implicit_detachable_type_mark, current_system.any_type, tokens.implicit_detachable_type_mark, current_system.any_type) then
				current_file.put_string (c_ge_ms32)
			else
				current_file.put_string (c_ge_ms8)
			end
			current_file.put_character ('(')
			print_escaped_string (l_string)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_integer (l_string.count)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
			if attached {ET_DYNAMIC_SPECIAL_TYPE} current_type as l_special_type then
			elseif current_type.is_basic then
			else
			end
		end

	print_builtin_any_twin_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'ANY.twin'.
			--
			-- Implementation of ISE's behavior:
			-- * Create a blank object (and not a memcopy of the target object).
			-- * Call 'copy' on this object..
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			if not current_type.has_redefined_copy_routine then
				print_builtin_any_standard_twin_body (a_feature)
			elseif current_type.base_class.is_type_class then
					-- Cannot have two instances of class TYPE representing the same Eiffel type.
				print_indentation_assign_to_result
				print_current_name (current_file)
				print_semicolon_newline
			elseif not attached current_type.seeded_dynamic_procedure (current_system.copy_seed, current_dynamic_system) as l_copy_feature then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif attached {ET_DYNAMIC_SPECIAL_TYPE} current_type as l_special_type then
				print_builtin_any_twin_body_with_special (l_special_type, l_copy_feature)
			elseif current_type.is_expanded then
				print_builtin_any_twin_body_with_expanded (l_copy_feature)
			else
				print_builtin_any_twin_body_with_reference (l_copy_feature)
			end
		end

	print_builtin_any_twin_body_with_expanded (a_copy_feature: ET_DYNAMIC_FEATURE)
			-- Print to `current_file' the body of the built-in feature
			-- 'ANY.twin' for objects of expanded static type `current_type'.
			-- `a_copy_feature' is the version of feature 'copy' in `current_type'.
		require
			not_special_type: not current_type.is_special
			expanded_type: current_type.is_expanded
			a_copy_feature_not_void: a_copy_feature /= Void
		do
				-- Call 'copy'.
			register_called_feature (a_copy_feature)
			print_indentation
			print_routine_name (a_copy_feature, current_type, current_file)
			current_file.put_character ('(')
			current_file.put_string (c_ac)
			print_comma
			current_file.put_character ('&')
			print_result_name (current_file)
			print_comma
				-- We don't call `print_attachment_operand' here to avoid calling
				-- 'twin' (and hence 'copy') on the argument of 'copy'. Otherwise we
				-- will call 'copy' recursively (possibly infinite recursion) with the
				-- same argument.
			current_file.put_character ('*')
			print_current_name (current_file)
			current_file.put_character (')')
			print_semicolon_newline
		end

	print_builtin_any_twin_body_with_reference (a_copy_feature: ET_DYNAMIC_FEATURE)
			-- Print to `current_file' the body of the built-in feature
			-- 'ANY.twin' for objects of reference static type `current_type'.
			-- `a_copy_feature' is the version of feature 'copy' in `current_type'.
		require
			not_special_type: not current_type.is_special
			reference_type: not current_type.is_expanded
			a_copy_feature_not_void: a_copy_feature /= Void
		do
				-- Create new object.
			print_indentation_assign_to_result
			current_file.put_string (c_ge_new)
			current_file.put_integer (current_type.id)
			current_file.put_character ('(')
			current_file.put_string (c_eif_true)
			current_file.put_character (')')
			print_semicolon_newline
				-- Call 'copy'.
			register_called_feature (a_copy_feature)
			print_indentation
			print_routine_name (a_copy_feature, current_type, current_file)
			current_file.put_character ('(')
			current_file.put_string (c_ac)
			print_comma
				-- We don't call `print_attachment_operand' here to avoid calling
				-- 'twin' (and hence 'copy') on the argument of 'copy'. Otherwise we
				-- will call 'copy' recursively (possibly infinite recursion) with the
				-- same argument.
			print_result_name (current_file)
			print_comma
			print_current_name (current_file)
			current_file.put_character (')')
			print_semicolon_newline
		end

	print_builtin_any_twin_body_with_special (a_special_type: ET_DYNAMIC_SPECIAL_TYPE; a_copy_feature: ET_DYNAMIC_FEATURE)
			-- Print to `current_file' the body of the built-in feature
			-- 'ANY.twin' for objects of static type `a_special_type'.
			-- `a_copy_feature' is the version of feature 'copy' in `a_special_type'.
			--
			-- Implementation of ISE's behavior:
			-- * 'twin' sets the capacity of the new object to
			--   the same value as 'count', which may be different from
			--   the capacity of the original object.
		require
			a_special_type: a_special_type = current_type
			a_copy_feature_not_void: a_copy_feature /= Void
		local
			l_count_temp: ET_IDENTIFIER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_count_type: ET_DYNAMIC_PRIMARY_TYPE
			l_result: ET_RESULT
		do
			if a_special_type.attribute_count < 2 then
					-- Internal error: class "SPECIAL" should have at least the
					-- features 'count' and 'capacity' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
					-- Create new object.
				l_result := new_result_expression
				extra_dynamic_type_sets.force_last (a_special_type)
				l_result.set_index (current_dynamic_type_sets.count + extra_dynamic_type_sets.count)
				l_dynamic_type_set := result_type_set_in_feature (a_special_type.queries.first)
				l_count_type := l_dynamic_type_set.static_type.primary_type
				l_count_temp := new_temp_variable (l_count_type)
				mark_temp_variable_frozen (l_count_temp)
				print_assign_special_count_attribute_to_temp_variable (l_count_temp, tokens.current_keyword, a_special_type, False)
				print_indentation_assign_to_result
				current_file.put_string (c_ge_new)
				current_file.put_integer (a_special_type.id)
				current_file.put_character ('(')
				print_temp_name (l_count_temp, current_file)
				print_comma
				current_file.put_string (c_eif_true)
				current_file.put_character (')')
				print_semicolon_newline
				print_indentation
				print_attribute_special_count_access (l_result, a_special_type, False)
				print_assign_to
				print_temp_name (l_count_temp, current_file)
				print_semicolon_newline
				mark_temp_variable_unfrozen (l_count_temp)
				mark_temp_variable_free (l_count_temp)
				free_result_expression (l_result)
				extra_dynamic_type_sets.remove_last
					-- Call 'copy'.
				register_called_feature (a_copy_feature)
				print_indentation
				print_routine_name (a_copy_feature, current_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				print_comma
				print_result_name (current_file)
				print_comma
					-- We don't call `print_attachment_operand' here to avoid calling
					-- 'twin' (and hence 'copy') on the argument of 'copy'. Otherwise we
					-- will call 'copy' recursively (possibly infinite recursion) with the
					-- same argument.
				print_current_name (current_file)
				current_file.put_character (')')
				print_semicolon_newline
			end
		end

	print_builtin_any_twin_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ANY.twin'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not a_target_type.has_redefined_copy_routine then
				print_builtin_any_standard_twin_call (a_feature, a_target_type, a_check_void_target)
			elseif call_operands.count /= 1 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				if a_target_type.base_class.is_type_class then
						-- Cannot have two instances of class "TYPE" representing the same Eiffel type.
					print_target_expression (l_target, a_target_type, a_check_void_target)
				else
					register_called_feature (a_feature)
					print_routine_name (a_feature, a_target_type, current_file)
					current_file.put_character ('(')
					current_file.put_string (c_ac)
					print_comma
					print_target_expression (l_target, a_target_type, a_check_void_target)
					current_file.put_character (')')
				end
			end
		end

	print_builtin_arguments_32_argument_count_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ARGUMENTS_32.argument_count'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_result_type_set: detachable ET_DYNAMIC_TYPE_SET
		do
			l_result_type_set := a_feature.result_type_set
			if l_result_type_set = Void then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ge_argc)
				current_file.put_character (' ')
				current_file.put_character ('-')
				current_file.put_character (' ')
				current_file.put_character ('1')
				current_file.put_character (')')
			end
		end

	print_builtin_arguments_32_i_th_argument_pointer_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ARGUMENTS_32.i_th_argument_pointer'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_result_type_set: detachable ET_DYNAMIC_TYPE_SET
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			l_result_type_set := a_feature.result_type_set
			if l_result_type_set = Void then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ge_argv)
				current_file.put_character ('[')
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (']')
				current_file.put_character (')')
			end
		end

	print_builtin_arguments_32_i_th_argument_string_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ARGUMENTS_32.i_th_argument_string'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				current_file.put_string (c_ge_istr32_from_nstr)
				current_file.put_character ('(')
				current_file.put_string (c_ge_argv)
				current_file.put_character ('[')
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (']')
				current_file.put_character (')')
			end
		end

	print_builtin_boolean_and_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'BOOLEAN.conjuncted alias "and"'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('&')
				current_file.put_character ('&')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_boolean_and_then_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'BOOLEAN.conjuncted_semistrict alias "and then"'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('&')
				current_file.put_character ('&')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_boolean_implies_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'BOOLEAN.implication alias "implies"'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_character ('!')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_character ('|')
				current_file.put_character ('|')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_boolean_not_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'BOOLEAN.negated alias "not"'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('!')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_boolean_or_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'BOOLEAN.disjuncted alias "or"'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('|')
				current_file.put_character ('|')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_boolean_or_else_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'BOOLEAN.disjuncted_semistrict alias "or else"'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('|')
				current_file.put_character ('|')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_boolean_xor_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'BOOLEAN.disjuncted_exclusive alias "xor"'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('^')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_boolean_ref_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'BOOLEAN_REF.item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			l_target := call_operands.first
			if a_target_type.is_basic then
				print_unboxed_expression (l_target, a_target_type, a_check_void_target)
			else
					-- Internal attribute.
				print_adapted_attribute_access (a_feature, l_target, a_target_type, a_check_void_target)
			end
		end

	print_builtin_boolean_ref_set_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'BOOLEAN_REF.set_item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_query: ET_DYNAMIC_FEATURE
			l_item_attribute: detachable ET_DYNAMIC_FEATURE
			i, nb: INTEGER
			l_builtin_item_class_code: NATURAL_8
			l_builtin_item_feature_code: NATURAL_8
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				if a_target_type.is_basic then
					print_indentation
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				else
					l_builtin_item_class_code := tokens.builtin_boolean_ref_class
					l_builtin_item_feature_code := tokens.builtin_boolean_ref_item
					l_queries := a_target_type.queries
					nb := a_target_type.attribute_count
					from i := 1 until i > nb loop
						l_query := l_queries.item (i)
						if
							l_query.builtin_class_code = l_builtin_item_class_code and then
							l_query.builtin_feature_code = l_builtin_item_feature_code
						then
							l_item_attribute := l_query
							i := nb + 1
						else
							i := i + 1
						end
					end
					if l_item_attribute /= Void then
							-- Set the built-in attribute 'item'.
						print_indentation
						print_adapted_attribute_access (l_item_attribute, l_target, a_target_type, a_check_void_target)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_character ('(')
						print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					else
						-- If `l_item_attribute' is Void, it means that it is never used,
						-- therefore there is no need to set it.
-- TODO: it can also be that it has been redefined.
					end
				end
			end
		end

	print_builtin_character_n_code_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'CHARACTER_N.code'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_character_n_natural_32_code_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'CHARACTER_N.natural_32_code'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_character_n_to_character_8_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'CHARACTER_N.to_character_8'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_character_n_to_character_32_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'CHARACTER_N.to_character_32'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_character_n_ref_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'CHARACTER_N_REF.item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			l_target := call_operands.first
			if a_target_type.is_basic then
				print_unboxed_expression (l_target, a_target_type, a_check_void_target)
			else
					-- Internal attribute.
				print_adapted_attribute_access (a_feature, l_target, a_target_type, a_check_void_target)
			end
		end

	print_builtin_character_n_ref_set_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'CHARACTER_N_REF.set_item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_query: ET_DYNAMIC_FEATURE
			l_item_attribute: detachable ET_DYNAMIC_FEATURE
			i, nb: INTEGER
			l_builtin_item_class_code: NATURAL_8
			l_builtin_item_feature_code: NATURAL_8
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				l_argument_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				if a_target_type.is_basic then
					print_indentation
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_attachment_expression (l_argument, l_argument_type_set, l_formal_type)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				else
					l_builtin_item_class_code := a_feature.builtin_class_code
					l_builtin_item_feature_code := tokens.builtin_character_n_ref_item
					l_queries := a_target_type.queries
					nb := a_target_type.attribute_count
					from i := 1 until i > nb loop
						l_query := l_queries.item (i)
						if
							l_query.builtin_class_code = l_builtin_item_class_code and then
							l_query.builtin_feature_code = l_builtin_item_feature_code
						then
							l_item_attribute := l_query
							i := nb + 1
						else
							i := i + 1
						end
					end
					if l_item_attribute /= Void then
							-- Set the built-in attribute 'item'.
						print_indentation
						print_adapted_attribute_access (l_item_attribute, l_target, a_target_type, a_check_void_target)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_character ('(')
						print_attachment_expression (l_argument, l_argument_type_set, l_formal_type)
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					else
						-- If `l_item_attribute' is Void, it means that it is never used,
						-- therefore there is no need to set it.
-- TODO: it can also be that it has been redefined.
					end
				end
			end
		end

	print_builtin_com_failure_c_strlen_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'COM_FAILURE.c_strlen.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				include_runtime_header_file ("ge_com_failure.h", False, header_file)
				print_declaration_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_ge_ccom_strlen)
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
			end
		end

	print_builtin_com_failure_ccom_hresult_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'COM_FAILURE.ccom_hresult.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				include_runtime_header_file ("ge_com_failure.h", False, header_file)
				print_declaration_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_ge_ccom_hresult)
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
			end
		end

	print_builtin_com_failure_ccom_hresult_code_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'COM_FAILURE.ccom_hresult_code.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				include_runtime_header_file ("ge_com_failure.h", False, header_file)
				print_declaration_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_ge_ccom_hresult_code)
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
			end
		end

	print_builtin_com_failure_ccom_hresult_facility_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'COM_FAILURE.ccom_hresult_facility.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				include_runtime_header_file ("ge_com_failure.h", False, header_file)
				print_declaration_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_ge_ccom_hresult_facility)
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
			end
		end

	print_builtin_com_failure_character_size_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'COM_FAILURE.character_size.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if call_operands.count /= 1 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				include_runtime_header_file ("ge_com_failure.h", False, header_file)
				print_declaration_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_ge_ccom_character_size)
				current_file.put_character ('(')
				current_file.put_character (')')
			end
		end

	print_builtin_com_failure_cwin_error_text_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'COM_FAILURE.cwin_error_text.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				include_runtime_header_file ("ge_com_failure.h", False, header_file)
				print_declaration_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_ge_ccom_error_text)
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
			end
		end

	print_builtin_com_failure_cwin_local_free_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'COM_FAILURE.cwin_local_free'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				include_runtime_header_file ("ge_com_failure.h", False, header_file)
				print_indentation
				current_file.put_string (c_ge_ccom_local_free)
				current_file.put_character ('(')
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_exception_manager_catch_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'EXCEPTION_MANAGER.catch'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
		do
			l_arguments := a_feature.arguments
				-- Feature 'ISE_EXCEPTION_MANAGER.catch' is called internally.
			if not attached current_dynamic_system.ise_exception_manager_type.seeded_dynamic_query (a_feature.first_seed, current_dynamic_system) as l_ise_exception_manager_feature then
					-- Internal error: "ISE_EXCEPTION_MANAGER" is a descendant of
					-- "EXCEPTION_MANAGER". So it should have a feature with the
					-- same seed as `a_feature'. Otherwise we get an error when
					-- flattening the features of "ISE_EXCEPTION_MANAGER".
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				register_called_feature (l_ise_exception_manager_feature)
				print_routine_name (l_ise_exception_manager_feature, current_dynamic_system.ise_exception_manager_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_exception_manager)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_exception_manager_exception_from_code_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'EXCEPTION_MANAGER.exception_from_code'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
		do
			l_arguments := a_feature.arguments
				-- Feature 'ISE_EXCEPTION_MANAGER.exception_from_code' is called internally.
			if not attached current_dynamic_system.ise_exception_manager_type.seeded_dynamic_query (a_feature.first_seed, current_dynamic_system) as l_ise_exception_manager_feature then
					-- Internal error: "ISE_EXCEPTION_MANAGER" is a descendant of
					-- "EXCEPTION_MANAGER". So it should have a feature with the
					-- same seed as `a_feature'. Otherwise we get an error when
					-- flattening the features of "ISE_EXCEPTION_MANAGER".
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				register_called_feature (l_ise_exception_manager_feature)
				print_routine_name (l_ise_exception_manager_feature, current_dynamic_system.ise_exception_manager_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_exception_manager)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_exception_manager_ignore_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'EXCEPTION_MANAGER.ignore'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
		do
			l_arguments := a_feature.arguments
				-- Feature 'ISE_EXCEPTION_MANAGER.ignore' is called internally.
			if not attached current_dynamic_system.ise_exception_manager_type.seeded_dynamic_query (a_feature.first_seed, current_dynamic_system) as l_ise_exception_manager_feature then
					-- Internal error: "ISE_EXCEPTION_MANAGER" is a descendant of
					-- "EXCEPTION_MANAGER". So it should have a feature with the
					-- same seed as `a_feature'. Otherwise we get an error when
					-- flattening the features of "ISE_EXCEPTION_MANAGER".
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				register_called_feature (l_ise_exception_manager_feature)
				print_routine_name (l_ise_exception_manager_feature, current_dynamic_system.ise_exception_manager_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_exception_manager)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_exception_manager_is_caught_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'EXCEPTION_MANAGER.is_caught'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
		do
			l_arguments := a_feature.arguments
				-- Feature 'ISE_EXCEPTION_MANAGER.is_caught' is called internally.
			if not attached current_dynamic_system.ise_exception_manager_type.seeded_dynamic_query (a_feature.first_seed, current_dynamic_system) as l_ise_exception_manager_feature then
					-- Internal error: "ISE_EXCEPTION_MANAGER" is a descendant of
					-- "EXCEPTION_MANAGER". So it should have a feature with the
					-- same seed as `a_feature'. Otherwise we get an error when
					-- flattening the features of "ISE_EXCEPTION_MANAGER".
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				register_called_feature (l_ise_exception_manager_feature)
				print_routine_name (l_ise_exception_manager_feature, current_dynamic_system.ise_exception_manager_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_exception_manager)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_exception_manager_is_ignorable_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'EXCEPTION_MANAGER.is_ignorable'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
		do
			l_arguments := a_feature.arguments
				-- Feature 'ISE_EXCEPTION_MANAGER.is_ignorable' is called internally.
			if not attached current_dynamic_system.ise_exception_manager_type.seeded_dynamic_query (a_feature.first_seed, current_dynamic_system) as l_ise_exception_manager_feature then
					-- Internal error: "ISE_EXCEPTION_MANAGER" is a descendant of
					-- "EXCEPTION_MANAGER". So it should have a feature with the
					-- same seed as `a_feature'. Otherwise we get an error when
					-- flattening the features of "ISE_EXCEPTION_MANAGER".
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				register_called_feature (l_ise_exception_manager_feature)
				print_routine_name (l_ise_exception_manager_feature, current_dynamic_system.ise_exception_manager_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_exception_manager)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_exception_manager_is_ignored_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'EXCEPTION_MANAGER.is_ignored'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
		do
			l_arguments := a_feature.arguments
				-- Feature 'ISE_EXCEPTION_MANAGER.is_ignored' is called internally.
			if not attached current_dynamic_system.ise_exception_manager_type.seeded_dynamic_query (a_feature.first_seed, current_dynamic_system) as l_ise_exception_manager_feature then
					-- Internal error: "ISE_EXCEPTION_MANAGER" is a descendant of
					-- "EXCEPTION_MANAGER". So it should have a feature with the
					-- same seed as `a_feature'. Otherwise we get an error when
					-- flattening the features of "ISE_EXCEPTION_MANAGER".
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				register_called_feature (l_ise_exception_manager_feature)
				print_routine_name (l_ise_exception_manager_feature, current_dynamic_system.ise_exception_manager_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_exception_manager)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_exception_manager_is_raisable_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'EXCEPTION_MANAGER.is_raisable'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
		do
			l_arguments := a_feature.arguments
				-- Feature 'ISE_EXCEPTION_MANAGER.is_raisable' is called internally.
			if not attached current_dynamic_system.ise_exception_manager_type.seeded_dynamic_query (a_feature.first_seed, current_dynamic_system) as l_ise_exception_manager_feature then
					-- Internal error: "ISE_EXCEPTION_MANAGER" is a descendant of
					-- "EXCEPTION_MANAGER". So it should have a feature with the
					-- same seed as `a_feature'. Otherwise we get an error when
					-- flattening the features of "ISE_EXCEPTION_MANAGER".
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				register_called_feature (l_ise_exception_manager_feature)
				print_routine_name (l_ise_exception_manager_feature, current_dynamic_system.ise_exception_manager_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_exception_manager)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_exception_manager_last_exception_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'EXCEPTION_MANAGER.last_exception'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
				-- Feature 'ISE_EXCEPTION_MANAGER.last_exception' is called internally.
			if not attached current_dynamic_system.ise_exception_manager_type.seeded_dynamic_query (a_feature.first_seed, current_dynamic_system) as l_ise_exception_manager_feature then
					-- Internal error: "ISE_EXCEPTION_MANAGER" is a descendant of
					-- "EXCEPTION_MANAGER". So it should have a feature with the
					-- same seed as `a_feature'. Otherwise we get an error when
					-- flattening the features of "ISE_EXCEPTION_MANAGER".
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				register_called_feature (l_ise_exception_manager_feature)
				print_routine_name (l_ise_exception_manager_feature, current_dynamic_system.ise_exception_manager_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_exception_manager)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_exception_manager_raise_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'EXCEPTION_MANAGER.raise'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
		do
			l_arguments := a_feature.arguments
				-- Feature 'ISE_EXCEPTION_MANAGER.raise' is called internally.
			if not attached current_dynamic_system.ise_exception_manager_type.seeded_dynamic_query (a_feature.first_seed, current_dynamic_system) as l_ise_exception_manager_feature then
					-- Internal error: "ISE_EXCEPTION_MANAGER" is a descendant of
					-- "EXCEPTION_MANAGER". So it should have a feature with the
					-- same seed as `a_feature'. Otherwise we get an error when
					-- flattening the features of "ISE_EXCEPTION_MANAGER".
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				register_called_feature (l_ise_exception_manager_feature)
				print_routine_name (l_ise_exception_manager_feature, current_dynamic_system.ise_exception_manager_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_exception_manager)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_exception_manager_set_is_ignored_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'EXCEPTION_MANAGER.set_is_ignored'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
		do
			l_arguments := a_feature.arguments
				-- Feature 'ISE_EXCEPTION_MANAGER.set_is_ignored' is called internally.
			if not attached current_dynamic_system.ise_exception_manager_type.seeded_dynamic_query (a_feature.first_seed, current_dynamic_system) as l_ise_exception_manager_feature then
					-- Internal error: "ISE_EXCEPTION_MANAGER" is a descendant of
					-- "EXCEPTION_MANAGER". So it should have a feature with the
					-- same seed as `a_feature'. Otherwise we get an error when
					-- flattening the features of "ISE_EXCEPTION_MANAGER".
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 2 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				register_called_feature (l_ise_exception_manager_feature)
				print_routine_name (l_ise_exception_manager_feature, current_dynamic_system.ise_exception_manager_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_exception_manager)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (2).name, current_file)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_exception_manager_type_of_code_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'EXCEPTION_MANAGER.type_of_code'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
		do
			l_arguments := a_feature.arguments
				-- Feature 'ISE_EXCEPTION_MANAGER.type_of_code' is called internally.
			if not attached current_dynamic_system.ise_exception_manager_type.seeded_dynamic_query (a_feature.first_seed, current_dynamic_system) as l_ise_exception_manager_feature then
					-- Internal error: "ISE_EXCEPTION_MANAGER" is a descendant of
					-- "EXCEPTION_MANAGER". So it should have a feature with the
					-- same seed as `a_feature'. Otherwise we get an error when
					-- flattening the features of "ISE_EXCEPTION_MANAGER".
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				register_called_feature (l_ise_exception_manager_feature)
				print_routine_name (l_ise_exception_manager_feature, current_dynamic_system.ise_exception_manager_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ac)
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_exception_manager)
				current_file.put_character (',')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_exception_manager_factory_exception_manager_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'EXCEPTION_MANAGER_FACTORY.exception_manager'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_indentation
			print_result_name (current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_string (c_ac)
			current_file.put_string (c_arrow)
			current_file.put_string (c_exception_manager)
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_builtin_function_fast_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'FUNCTION.fast_item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
				-- Not supported.
			current_file.put_character ('(')
			current_file.put_string (c_ge_raise)
			current_file.put_character ('(')
			current_file.put_string (c_ge_ex_prog)
			current_file.put_character (')')
			current_file.put_character (',')
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_typed_default_entity_value (l_result_type_set.static_type.primary_type, current_file)
			end
			current_file.put_character (')')
		end

	print_builtin_function_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_feature' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'FUNCTION.item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_routine_call_call (a_feature, a_target_type, a_check_void_target)
		end

	print_builtin_identified_routines_eif_current_object_id_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'IDENTIFIED_ROUTINES.eif_current_object_id'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			include_runtime_header_file ("ge_identified.h", False, header_file)
			current_file.put_string (c_ge_object_id)
			current_file.put_character ('(')
			print_attachment_expression (call_operands.first, dynamic_type_set (call_operands.first), current_dynamic_system.any_type)
			current_file.put_character (')')
		end

	print_builtin_identified_routines_eif_id_object_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'IDENTIFIED_ROUTINES.eif_id_object'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				include_runtime_header_file ("ge_identified.h", False, header_file)
				current_file.put_string (c_ge_id_object)
				current_file.put_character ('(')
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
			end
		end

	print_builtin_identified_routines_eif_is_object_id_of_current_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'IDENTIFIED_ROUTINES.eif_is_object_id_of_current'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				include_runtime_header_file ("ge_identified.h", False, header_file)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_ge_id_object)
				current_file.put_character ('(')
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				if a_target_type.is_expanded then
-- TODO: `is_equal` comparison.
					current_file.put_character (',')
					current_file.put_string (c_eif_false)
				else
					current_file.put_character ('=')
					current_file.put_character ('=')
					current_file.put_character ('(')
					print_target_expression (call_operands.first, a_target_type, a_check_void_target)
					current_file.put_character (')')
				end
				current_file.put_character (')')
			end
		end

	print_builtin_identified_routines_eif_object_id_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'IDENTIFIED_ROUTINES.eif_object_id'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				include_runtime_header_file ("ge_identified.h", False, header_file)
				current_file.put_string (c_ge_object_id)
				current_file.put_character ('(')
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
			end
		end

	print_builtin_identified_routines_eif_object_id_free_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'IDENTIFIED_ROUTINES.eif_object_id_free'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				include_runtime_header_file ("ge_identified.h", False, header_file)
				print_indentation
				current_file.put_string (c_ge_object_id_free)
				current_file.put_character ('(')
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (';')
			end
		end

	print_builtin_integer_n_as_integer_8_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.as_integer_8'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_as_integer_16_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.as_integer_16'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_as_integer_32_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.as_integer_32'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_as_integer_64_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.as_integer_64'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_as_natural_8_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.as_natural_8'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_as_natural_16_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.as_natural_16'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_as_natural_32_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_Nx.as_natural_32'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_as_natural_64_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.as_natural_64'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_bit_and_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.bit_and'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('&')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_bit_not_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.bit_not'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('~')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_bit_or_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.bit_or'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('|')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_bit_shift_left_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.bit_shift_left'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('<')
				current_file.put_character ('<')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_bit_shift_right_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.bit_shift_right'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('>')
				current_file.put_character ('>')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_bit_xor_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.bit_xor'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('^')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_identity_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.identity'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_integer_quotient_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.integer_quotient'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('/')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_integer_remainder_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.integer_remainder'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('%%')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_is_less_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.is_less'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			--
			-- Note: in C, the operators '<', '<=', '>', and '>=' always
			-- return either 0 or 1. No need to call "EIF_TEST" on them.
			-- See https://stackoverflow.com/questions/18097922/return-value-of-operator-in-c
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('<')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_minus_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.minus'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('-')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_opposite_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.opposite'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('-')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_plus_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.plus'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('+')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_power_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.power'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				include_runtime_header_file ("ge_integer.h", False, header_file)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_ge_power)
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_string (c_double)
				current_file.put_character (')')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_character ('(')
				current_file.put_string (c_double)
				current_file.put_character (')')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_product_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.product'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('*')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_quotient_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.quotient_'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_string (c_double)
				current_file.put_character (')')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('/')
				current_file.put_character ('(')
				current_file.put_string (c_double)
				current_file.put_character (')')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_to_character_8_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.to_character_8'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_to_character_32_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.to_character_32'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_to_double_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.to_double'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_to_real_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.to_real'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_to_real_32_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.to_real_32'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_to_real_64_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N.to_real_64'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_integer_n_ref_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N_REF.item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			l_target := call_operands.first
			if a_target_type.is_basic then
				print_unboxed_expression (l_target, a_target_type, a_check_void_target)
			else
					-- Internal attribute.
				print_adapted_attribute_access (a_feature, l_target, a_target_type, a_check_void_target)
			end
		end

	print_builtin_integer_n_ref_set_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static file) to `a_feature'
			-- corresponding to built-in feature 'INTEGER_N_REF.set_item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_query: ET_DYNAMIC_FEATURE
			l_item_attribute: detachable ET_DYNAMIC_FEATURE
			i, nb: INTEGER
			l_builtin_item_class_code: NATURAL_8
			l_builtin_item_feature_code: NATURAL_8
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				l_argument_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				if a_target_type.is_basic then
					print_indentation
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_attachment_expression (l_argument, l_argument_type_set, l_formal_type)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				else
					l_builtin_item_class_code := a_feature.builtin_class_code
					l_builtin_item_feature_code := tokens.builtin_integer_n_ref_item
					l_queries := a_target_type.queries
					nb := a_target_type.attribute_count
					from i := 1 until i > nb loop
						l_query := l_queries.item (i)
						if
							l_query.builtin_class_code = l_builtin_item_class_code and then
							l_query.builtin_feature_code = l_builtin_item_feature_code
						then
							l_item_attribute := l_query
							i := nb + 1
						else
							i := i + 1
						end
					end
					if l_item_attribute /= Void then
							-- Set the built-in attribute 'item'.
						print_indentation
						print_adapted_attribute_access (l_item_attribute, l_target, a_target_type, a_check_void_target)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_character ('(')
						print_attachment_expression (l_argument, l_argument_type_set, l_formal_type)
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					else
						-- If `l_item_attribute' is Void, it means that it is never used,
						-- therefore there is no need to set it.
-- TODO: it can also be that it has been redefined.
					end
				end
			end
		end

	print_builtin_ise_exception_manager_developer_raise_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_EXCEPTION_MANAGER.developer_raise'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			include_runtime_header_file ("ge_exception.h", False, header_file)
			print_builtin_procedure_c_call (a_feature, c_ge_developer_raise, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_attached_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.attached_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_attached_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_boolean_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.boolean_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_boolean_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_boolean_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.boolean_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_boolean_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_character_8_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.character_8_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_character_8_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_character_8_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.character_8_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_character_8_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_character_32_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.character_32_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_character_32_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_character_32_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.character_32_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_character_32_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_check_assert_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.check_assert'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_check_assert, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_compiler_version_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.compiler_version'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_compiler_version, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_detachable_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.detachable_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_non_attached_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_dynamic_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.dynamic_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_object_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_dynamic_type_at_offset_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.dynamic_type_at_offset'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_object_encoded_type_at_offset, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_eif_gen_param_id_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.eif_gen_param_id'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_generic_parameters_used := True
			print_builtin_query_c_call (a_feature, c_ge_generic_parameter_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_field_count_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.field_count_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			print_builtin_query_c_call (a_feature, c_ge_field_count_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_field_name_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.field_name_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_name_used := True
			print_builtin_query_c_call (a_feature, c_ge_field_name_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_field_offset_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.field_offset_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_field_offset_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_field_static_type_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.field_static_type_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_type_id_used := True
			print_builtin_query_c_call (a_feature, c_ge_field_static_type_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_field_type_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.field_type_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_type_id_used := True
			print_builtin_query_c_call (a_feature, c_ge_field_type_kind_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_generating_type_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.generating_type_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_name_used := True
			print_builtin_query_c_call (a_feature, c_ge_generating_type_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_generating_type_8_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.generating_type_8_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_name_used := True
			print_builtin_query_c_call (a_feature, c_ge_generating_type_8_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_generator_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.generator_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_generator_used := True
			print_builtin_query_c_call (a_feature, c_ge_generator_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_generator_8_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.generator_8_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_generator_used := True
			print_builtin_query_c_call (a_feature, c_ge_generator_8_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_generic_parameter_count_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.generic_parameter_count'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_generic_parameters_used := True
			print_builtin_query_c_call (a_feature, c_ge_generic_parameter_count_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_in_assertion_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'ISE_RUNTIME.in_assertion'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_indentation
			print_result_name (current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_string (c_eif_test)
			current_file.put_character ('(')
			current_file.put_string (c_ac)
			current_file.put_string (c_arrow)
			current_file.put_string (c_in_assertion)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_builtin_ise_runtime_integer_8_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.integer_8_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_integer_8_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_integer_8_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.integer_8_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_integer_8_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_integer_16_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.integer_16_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_integer_16_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_integer_16_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.integer_16_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_integer_16_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_integer_32_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.integer_32_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_integer_32_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_integer_32_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.integer_32_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_integer_32_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_integer_64_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.integer_64_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_integer_64_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_integer_64_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.integer_64_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_integer_64_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_attached_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_attached_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_is_attached_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_copy_semantics_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_copy_semantics_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_is_copy_semantics_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_expanded_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_expanded'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_is_expanded_object, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_field_expanded_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_field_expanded_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_type_id_used := True
			print_builtin_query_c_call (a_feature, c_ge_is_field_expanded_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_field_transient_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_field_transient_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_is_field_transient_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_object_marked_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_object_marked'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_is_object_marked, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_special_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_special'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_is_special_object, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_special_copy_semantics_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_special_copy_semantics_item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_is_special_copy_semantics_item, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_special_of_expanded_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_special_of_expanded'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_generic_parameters_used := True
			print_builtin_query_c_call (a_feature, c_ge_is_special_of_expanded_object, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_special_of_reference_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_special_of_reference'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_generic_parameters_used := True
			print_builtin_query_c_call (a_feature, c_ge_is_special_of_reference_object, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_special_of_reference_or_basic_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_special_of_reference_or_basic_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_generic_parameters_used := True
			print_builtin_query_c_call (a_feature, c_ge_is_special_of_reference_or_basic_expanded_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_special_of_reference_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_special_of_reference_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_generic_parameters_used := True
			print_builtin_query_c_call (a_feature, c_ge_is_special_of_reference_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_tuple_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_tuple'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_is_tuple_object, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_is_tuple_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.is_tuple_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_is_tuple_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_lock_marking_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.lock_marking'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_lock_marking, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_mark_object_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.mark_object'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_mark_object, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_natural_8_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.natural_8_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_natural_8_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_natural_8_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.natural_8_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_natural_8_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_natural_16_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.natural_16_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_natural_16_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_natural_16_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.natural_16_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_natural_16_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_natural_32_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.natural_32_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_natural_32_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_natural_32_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.natural_32_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_natural_32_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_natural_64_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.natural_64_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_natural_64_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_natural_64_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.natural_64_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_natural_64_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_new_instance_of_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.new_instance_of'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_new_instance_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_new_special_of_reference_instance_of_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.new_special_of_reference_instance_of'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_new_special_of_reference_instance_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_new_tuple_instance_of_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.new_tuple_instance_of'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_new_tuple_instance_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_new_type_instance_of_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.new_type_instance_of'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_new_type_instance_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_object_size_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.object_size'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_object_size_used := True
			type_info_generic_parameters_used := True
			print_builtin_query_c_call (a_feature, c_ge_object_size, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_once_objects_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.once_objects'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			include_runtime_header_file ("eif_traverse.h", False, header_file)
			print_builtin_query_c_call (a_feature, c_once_objects, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_persistent_field_count_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.persistent_field_count_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			print_builtin_query_c_call (a_feature, c_ge_persistent_field_count_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_pointer_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.pointer_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_pointer_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_pointer_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.pointer_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_pointer_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_pre_ecma_mapping_status_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'ISE_RUNTIME.pre_ecma_mapping_status'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_indentation
			print_result_name (current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_string (c_eif_test)
			current_file.put_character ('(')
			current_file.put_string (c_ac)
			current_file.put_string (c_arrow)
			current_file.put_string (c_pre_ecma_mapping_status)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_builtin_ise_runtime_raw_reference_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.raw_reference_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_raw_reference_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_raw_reference_field_at_offset_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.raw_reference_field_at_offset'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_raw_object_at_offset, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_real_32_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.real_32_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_real_32_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_real_32_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.real_32_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_real_32_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_real_64_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.real_64_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_real_64_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_real_64_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.real_64_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_real_64_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_reference_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.reference_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_query_c_call (a_feature, c_ge_reference_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_reference_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.reference_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_reference_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_reference_field_at_offset_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.reference_field_at_offset'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_object_at_offset, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_boolean_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_boolean_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_boolean_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_boolean_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_boolean_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_boolean_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_character_8_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_character_8_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_character_8_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_character_8_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_character_8_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_character_8_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_character_32_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_character_32_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_character_32_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_character_32_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_character_32_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_character_32_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_integer_8_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_integer_8_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_integer_8_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_integer_8_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_integer_8_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_integer_8_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_integer_16_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_integer_16_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_integer_16_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_integer_16_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_integer_16_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_integer_16_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_integer_32_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_integer_32_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_integer_32_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_integer_32_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_integer_32_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_integer_32_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_integer_64_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_integer_64_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_integer_64_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_integer_64_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_integer_64_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_integer_64_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_natural_8_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_natural_8_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_natural_8_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_natural_8_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_natural_8_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_natural_8_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_natural_16_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_natural_16_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_natural_16_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_natural_16_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_natural_16_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_natural_16_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_natural_32_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_natural_32_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_natural_32_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_natural_32_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_natural_32_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_natural_32_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_natural_64_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_natural_64_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_natural_64_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_natural_64_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_natural_64_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_natural_64_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_pointer_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_pointer_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_pointer_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_pointer_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_pointer_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_pointer_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_pre_ecma_mapping_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'ISE_RUNTIME.set_pre_ecma_mapping'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			if not attached a_feature.arguments as l_arguments or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_pre_ecma_mapping_status)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_ise_runtime_set_real_32_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_real_32_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_real_32_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_real_32_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_real_32_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_real_32_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_real_64_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_real_64_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_real_64_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_real_64_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_real_64_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_real_64_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_reference_field_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_reference_field'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_attributes_used := True
			type_info_attribute_offset_used := True
			print_builtin_procedure_c_call (a_feature, c_ge_set_reference_field, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_set_reference_field_at_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.set_reference_field_at'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_set_reference_field_at, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_storable_version_of_type_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.storable_version_of_type'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_query_c_call (a_feature, c_ge_storable_version_of_encoded_type, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_type_conforms_to_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.type_conforms_to'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_ancestors_used := True
			print_builtin_query_c_call (a_feature, c_ge_encoded_type_conforms_to, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_type_id_from_name_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.type_id_from_name'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_name_used := True
			print_builtin_query_c_call (a_feature, c_ge_encoded_type_from_name, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_unlock_marking_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.unlock_marking'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_unlock_marking, True, a_target_type, a_check_void_target)
		end

	print_builtin_ise_runtime_unmark_object_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'ISE_RUNTIME.unmark_object'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_procedure_c_call (a_feature, c_ge_unmark_object, True, a_target_type, a_check_void_target)
		end

	print_builtin_memory_find_referers_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'MEMORY.find_referers'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_result_type_set: detachable ET_DYNAMIC_TYPE_SET
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			l_result_type_set := a_feature.result_type_set
			if call_operands.count /= 3 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_result_type_set = Void then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				include_runtime_header_file ("eif_traverse.h", False, header_file)
				print_declaration_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_find_referers)
				current_file.put_character ('(')
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (',')
				current_file.put_character (' ')
				l_argument := call_operands.item (3)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (2, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
			end
		end

	print_builtin_memory_free_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'MEMORY.free'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			include_runtime_header_file ("eif_memory.h", False, header_file)
			print_builtin_procedure_c_call (a_feature, c_eif_mem_free, True, a_target_type, a_check_void_target)
		end

	print_builtin_platform_boolean_bytes_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.boolean_bytes'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				current_file.put_string (c_eif_boolean)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_platform_character_bytes_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.character_bytes'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				current_file.put_string (c_eif_character_8)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_platform_double_bytes_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.double_bytes'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				current_file.put_string (c_eif_real_64)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_platform_integer_bytes_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.integer_bytes'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				current_file.put_string (c_eif_integer_32)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_platform_is_64_bits_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.s_64_bits'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			current_file.put_string (c_ge_is_64_bits)
		end

	print_builtin_platform_is_dotnet_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.is_dotnet'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			current_file.put_string (c_eif_false)
		end

	print_builtin_platform_is_mac_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.is_mac'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			current_file.put_string (c_eif_is_mac)
		end

	print_builtin_platform_is_scoop_capable_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.is_scoop_capable'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			current_file.put_string (c_eif_false)
		end

	print_builtin_platform_is_thread_capable_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.is_thread_capable'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if multithreaded_mode then
				current_file.put_string (c_eif_true)
			else
				current_file.put_string (c_eif_false)
			end
		end

	print_builtin_platform_is_unix_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.is_unix'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			current_file.put_string (c_eif_is_unix)
		end

	print_builtin_platform_is_vms_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.is_vms'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			current_file.put_string (c_eif_is_vms)
		end

	print_builtin_platform_is_vxworks_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.is_vxworks'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			current_file.put_string (c_eif_is_vxworks)
		end

	print_builtin_platform_is_windows_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.is_windows'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			current_file.put_string (c_eif_is_windows)
		end

	print_builtin_platform_pointer_bytes_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.pointer_bytes'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				current_file.put_string (c_eif_pointer)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_platform_real_bytes_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.real_bytes'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				current_file.put_string (c_eif_real_32)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_platform_wide_character_bytes_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PLATFORM.wide_character_bytes'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				current_file.put_string (c_eif_character_32)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_pointer_hash_code_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'POINTER.hash_code'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_integer_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_integer_type := l_result_type_set.static_type.primary_type
				print_type_cast (l_integer_type, current_file)
				current_file.put_character ('(')
				print_type_cast (l_integer_type, current_file)
				current_file.put_string ("(intptr_t)")
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
					-- Clear the sign bit to ensure that the hash-code is positive.
				current_file.put_character ('&')
				print_type_cast (l_integer_type, current_file)
				current_file.put_character ('(')
				if l_integer_type = current_dynamic_system.integer_8_type then
					current_file.put_string ("0x7F")
				elseif l_integer_type = current_dynamic_system.integer_16_type then
					current_file.put_string ("0x7FFF")
				elseif l_integer_type = current_dynamic_system.integer_64_type then
					current_file.put_string ("0x7FFFFFFFFFFFFFFF")
				else
					current_file.put_string ("0x7FFFFFFF")
				end
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_pointer_is_default_pointer_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'POINTER.is_default_pointer'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
			l_pointer_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_result_type := l_result_type_set.static_type.primary_type
				print_type_cast (l_result_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character ('=')
				current_file.put_character ('=')
				l_pointer_type := current_dynamic_system.dynamic_primary_type (current_universe_impl.pointer_type, current_type.base_type)
				print_type_cast (l_pointer_type, current_file)
				current_file.put_character ('0')
				current_file.put_character (')')
			end
		end

	print_builtin_pointer_out_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'POINTER.out'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
-- TODO: make sure that it works with descendants of POINTER.
-- Use 'item' for that.
			print_indentation
			current_file.put_string (c_char)
			current_file.put_character (' ')
			current_file.put_character ('s')
			current_file.put_character ('[')
			current_file.put_character ('2')
			current_file.put_character ('0')
			current_file.put_character (']')
			current_file.put_character (';')
			current_file.put_new_line
			print_indentation
			if current_type.is_basic then
				current_file.put_string ("int l = snprintf(s,20,%"0x%%lX%",(unsigned long)(intptr_t)*C);")
			else
-- TODO: use feature `item'.
				set_fatal_error
				error_handler.report_giaaa_error
			end
			current_file.put_new_line
			print_indentation
			print_result_name (current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			if attached current_feature.result_type_set as l_result_type_set and then l_result_type_set.static_type.base_type.same_named_type_with_type_marks (current_system.string_32_type, tokens.implicit_detachable_type_mark, current_system.any_type, tokens.implicit_detachable_type_mark, current_system.any_type) then
				current_file.put_string (c_ge_ms32)
			else
				current_file.put_string (c_ge_ms8)
			end
			current_file.put_string ("(s,l)")
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_builtin_pointer_plus_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'POINTER.plus alias "+"'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_string (c_char)
				current_file.put_character ('*')
				current_file.put_character (')')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_character ('+')
				current_file.put_character ('(')
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_pointer_to_integer_32_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'POINTER.to_integer_32'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string ("(intptr_t)")
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_pointer_ref_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'POINTER_REF.item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			l_target := call_operands.first
			if a_target_type.is_basic then
				print_unboxed_expression (l_target, a_target_type, a_check_void_target)
			else
					-- Internal attribute.
				print_adapted_attribute_access (a_feature, l_target, a_target_type, a_check_void_target)
			end
		end

	print_builtin_pointer_ref_set_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'POINTER_REF.set_item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_query: ET_DYNAMIC_FEATURE
			l_item_attribute: detachable ET_DYNAMIC_FEATURE
			i, nb: INTEGER
			l_builtin_item_class_code: NATURAL_8
			l_builtin_item_feature_code: NATURAL_8
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				if a_target_type.is_basic then
					print_indentation
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				else
					l_builtin_item_class_code := tokens.builtin_pointer_ref_class
					l_builtin_item_feature_code := tokens.builtin_pointer_ref_item
					l_queries := a_target_type.queries
					nb := a_target_type.attribute_count
					from i := 1 until i > nb loop
						l_query := l_queries.item (i)
						if
							l_query.builtin_class_code = l_builtin_item_class_code and then
							l_query.builtin_feature_code = l_builtin_item_feature_code
						then
							l_item_attribute := l_query
							i := nb + 1
						else
							i := i + 1
						end
					end
					if l_item_attribute /= Void then
							-- Set the built-in attribute 'item'.
						print_indentation
						print_adapted_attribute_access (l_item_attribute, l_target, a_target_type, a_check_void_target)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_character ('(')
						print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					else
						-- If `l_item_attribute' is Void, it means that it is never used,
						-- therefore there is no need to set it.
-- TODO: it can also be that it has been redefined.
					end
				end
			end
		end

	print_builtin_procedure_call_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PROCEDURE.call'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_builtin_routine_call_call (a_feature, a_target_type, a_check_void_target)
		end

	print_builtin_procedure_fast_call_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'PROCEDURE.fast_call'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
				-- Not supported.
			print_indentation
			current_file.put_string (c_ge_raise)
			current_file.put_character ('(')
			current_file.put_string (c_ge_ex_prog)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_builtin_real_n_ceiling_real_32_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.ceiling_real_32'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				include_runtime_header_file ("ge_real.h", False, header_file)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_ge_ceiling)
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_string (c_double)
				current_file.put_character (')')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_ceiling_real_64_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.ceiling_real_64'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				include_runtime_header_file ("ge_real.h", False, header_file)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_ge_ceiling)
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_string (c_double)
				current_file.put_character (')')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_floor_real_32_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.floor_real_32'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				include_runtime_header_file ("ge_real.h", False, header_file)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_ge_floor)
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_string (c_double)
				current_file.put_character (')')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_floor_real_64_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.floor_real_64'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				include_runtime_header_file ("ge_real.h", False, header_file)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_ge_floor)
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_string (c_double)
				current_file.put_character (')')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_identity_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.identity'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_ieee_is_equal_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.ieee_is_equal'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_c_function_name: STRING
		do
			if a_target_type = current_dynamic_system.real_32_type then
				l_c_function_name := c_ge_real_32_ieee_is_equal
			else
				l_c_function_name := c_ge_real_64_ieee_is_equal
			end
			include_runtime_header_file ("ge_real.h", False, header_file)
			print_builtin_query_c_call (a_feature, l_c_function_name, False, a_target_type, a_check_void_target)
		end

	print_builtin_real_n_ieee_is_greater_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.ieee_is_greater'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_c_function_name: STRING
		do
			if a_target_type = current_dynamic_system.real_32_type then
				l_c_function_name := c_ge_real_32_ieee_is_greater
			else
				l_c_function_name := c_ge_real_64_ieee_is_greater
			end
			include_runtime_header_file ("ge_real.h", False, header_file)
			print_builtin_query_c_call (a_feature, l_c_function_name, False, a_target_type, a_check_void_target)
		end

	print_builtin_real_n_ieee_is_greater_equal_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.ieee_is_greater_equal'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_c_function_name: STRING
		do
			if a_target_type = current_dynamic_system.real_32_type then
				l_c_function_name := c_ge_real_32_ieee_is_greater_equal
			else
				l_c_function_name := c_ge_real_64_ieee_is_greater_equal
			end
			include_runtime_header_file ("ge_real.h", False, header_file)
			print_builtin_query_c_call (a_feature, l_c_function_name, False, a_target_type, a_check_void_target)
		end

	print_builtin_real_n_ieee_is_less_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.ieee_is_less'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_c_function_name: STRING
		do
			if a_target_type = current_dynamic_system.real_32_type then
				l_c_function_name := c_ge_real_32_ieee_is_less
			else
				l_c_function_name := c_ge_real_64_ieee_is_less
			end
			include_runtime_header_file ("ge_real.h", False, header_file)
			print_builtin_query_c_call (a_feature, l_c_function_name, False, a_target_type, a_check_void_target)
		end

	print_builtin_real_n_ieee_is_less_equal_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.ieee_is_less_equal'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_c_function_name: STRING
		do
			if a_target_type = current_dynamic_system.real_32_type then
				l_c_function_name := c_ge_real_32_ieee_is_less_equal
			else
				l_c_function_name := c_ge_real_64_ieee_is_less_equal
			end
			include_runtime_header_file ("ge_real.h", False, header_file)
			print_builtin_query_c_call (a_feature, l_c_function_name, False, a_target_type, a_check_void_target)
		end

	print_builtin_real_n_ieee_maximum_number_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.ieee_maximum_number'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_c_function_name: STRING
		do
			if a_target_type = current_dynamic_system.real_32_type then
				l_c_function_name := c_ge_real_32_ieee_maximum_number
			else
				l_c_function_name := c_ge_real_64_ieee_maximum_number
			end
			include_runtime_header_file ("ge_real.h", False, header_file)
			print_builtin_query_c_call (a_feature, l_c_function_name, False, a_target_type, a_check_void_target)
		end

	print_builtin_real_n_ieee_minimum_number_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.ieee_minimum_number'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_c_function_name: STRING
		do
			if a_target_type = current_dynamic_system.real_32_type then
				l_c_function_name := c_ge_real_32_ieee_minimum_number
			else
				l_c_function_name := c_ge_real_64_ieee_minimum_number
			end
			include_runtime_header_file ("ge_real.h", False, header_file)
			print_builtin_query_c_call (a_feature, l_c_function_name, False, a_target_type, a_check_void_target)
		end

	print_builtin_real_n_is_less_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.is_less'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
			--
			-- Note: in C, the operators '<', '<=', '>', and '>=' always
			-- return either 0 or 1. No need to call "EIF_TEST" on them.
			-- See https://stackoverflow.com/questions/18097922/return-value-of-operator-in-c
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif total_order_on_reals_mode and then a_target_type = current_dynamic_system.real_32_type then
				include_runtime_header_file ("ge_real.h", False, header_file)
				print_builtin_query_c_call (a_feature, c_ge_real_32_is_less, False, a_target_type, a_check_void_target)
			elseif total_order_on_reals_mode and then a_target_type = current_dynamic_system.real_64_type then
				include_runtime_header_file ("ge_real.h", False, header_file)
				print_builtin_query_c_call (a_feature, c_ge_real_64_is_less, False, a_target_type, a_check_void_target)
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('<')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_is_nan_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.is_nan'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				include_runtime_header_file ("ge_real.h", False, header_file)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					if a_target_type = current_dynamic_system.real_32_type then
						current_file.put_string (c_ge_real_32_is_nan)
					else
						current_file.put_string (c_ge_real_64_is_nan)
					end
					current_file.put_character ('(')
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
					current_file.put_character (')')
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_is_negative_infinity_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.is_negative_infinity'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				include_runtime_header_file ("ge_real.h", False, header_file)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					if a_target_type = current_dynamic_system.real_32_type then
						current_file.put_string (c_ge_real_32_is_negative_infinity)
					else
						current_file.put_string (c_ge_real_64_is_negative_infinity)
					end
					current_file.put_character ('(')
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
					current_file.put_character (')')
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_is_positive_infinity_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.is_positive_infinity'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				include_runtime_header_file ("ge_real.h", False, header_file)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					if a_target_type = current_dynamic_system.real_32_type then
						current_file.put_string (c_ge_real_32_is_positive_infinity)
					else
						current_file.put_string (c_ge_real_64_is_positive_infinity)
					end
					current_file.put_character ('(')
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
					current_file.put_character (')')
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_minus_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.minus'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('-')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_opposite_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.opposite'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('-')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_out_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'REAL_N.out' from a sized real type.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_indentation
			current_file.put_string (c_char)
			current_file.put_character (' ')
			current_file.put_character ('s')
			current_file.put_character ('[')
			current_file.put_character ('4')
			current_file.put_character ('0')
			current_file.put_character (']')
			current_file.put_character (';')
			current_file.put_new_line
			print_indentation
			if current_type.is_basic then
				if current_system.real_32_type.same_named_type (current_type.base_type, current_type.base_class, current_type.base_class) then
					current_file.put_string ("int l = snprintf(s,40,%"%%g%",*C);")
				else
					current_file.put_string ("int l = snprintf(s,40,%"%%.17g%",*C);")
				end
			else
-- TODO: use feature `item'.
				set_fatal_error
				error_handler.report_giaaa_error
			end
			current_file.put_new_line
			print_indentation
			print_result_name (current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			if attached current_feature.result_type_set as l_result_type_set and then l_result_type_set.static_type.base_type.same_named_type_with_type_marks (current_system.string_32_type, tokens.implicit_detachable_type_mark, current_system.any_type, tokens.implicit_detachable_type_mark, current_system.any_type) then
				current_file.put_string (c_ge_ms32)
			else
				current_file.put_string (c_ge_ms8)
			end
			current_file.put_string ("(s,l)")
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_builtin_real_n_plus_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.plus'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('+')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_power_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.power"'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				include_runtime_header_file ("ge_real.h", False, header_file)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_string (c_ge_power)
				current_file.put_character ('(')
				current_file.put_character ('(')
				current_file.put_string (c_double)
				current_file.put_character (')')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_character ('(')
				current_file.put_string (c_double)
				current_file.put_character (')')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_product_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.product'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('*')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_quotient_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.quotient'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
				current_file.put_character ('/')
				current_file.put_character ('(')
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_to_double_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.to_double'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_truncated_to_integer_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.truncated_to_integer'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_truncated_to_integer_64_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.truncated_to_integer_64'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_truncated_to_real_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N.truncated_to_real'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				if a_target_type.is_basic then
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
				else
-- TODO: use feature `item'.
					set_fatal_error
					error_handler.report_giaaa_error
				end
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_ref_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N_REF.item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
		do
			l_target := call_operands.first
			if a_target_type.is_basic then
				print_unboxed_expression (l_target, a_target_type, a_check_void_target)
			else
					-- Internal attribute.
				print_adapted_attribute_access (a_feature, l_target, a_target_type, a_check_void_target)
			end
		end

	print_builtin_real_n_ref_nan_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N_REF.nan'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				include_runtime_header_file ("ge_real.h", False, header_file)
				l_result_type := l_result_type_set.static_type.primary_type
				print_type_cast (l_result_type, current_file)
				current_file.put_character ('(')
				if l_result_type = current_dynamic_system.real_32_type then
					current_file.put_string (c_ge_real_32_nan)
				else
					current_file.put_string (c_ge_real_64_nan)
				end
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_ref_negative_infinity_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N_REF.negative_infinity'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				include_runtime_header_file ("ge_real.h", False, header_file)
				l_result_type := l_result_type_set.static_type.primary_type
				print_type_cast (l_result_type, current_file)
				current_file.put_character ('(')
				if l_result_type = current_dynamic_system.real_32_type then
					current_file.put_string (c_ge_real_32_negative_infinity)
				else
					current_file.put_string (c_ge_real_64_negative_infinity)
				end
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_ref_positive_infinity_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N_REF.positive_infinity'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				include_runtime_header_file ("ge_real.h", False, header_file)
				l_result_type := l_result_type_set.static_type.primary_type
				print_type_cast (l_result_type, current_file)
				current_file.put_character ('(')
				if l_result_type = current_dynamic_system.real_32_type then
					current_file.put_string (c_ge_real_32_positive_infinity)
				else
					current_file.put_string (c_ge_real_64_positive_infinity)
				end
				current_file.put_character (')')
			end
		end

	print_builtin_real_n_ref_set_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'REAL_N_REF.set_item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_query: ET_DYNAMIC_FEATURE
			l_item_attribute: detachable ET_DYNAMIC_FEATURE
			i, nb: INTEGER
			l_builtin_item_class_code: NATURAL_8
			l_builtin_item_feature_code: NATURAL_8
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				l_argument_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				if a_target_type.is_basic then
					print_indentation
					print_unboxed_expression (l_target, a_target_type, a_check_void_target)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_attachment_expression (l_argument, l_argument_type_set, l_formal_type)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				else
					l_builtin_item_class_code := a_feature.builtin_class_code
					l_builtin_item_feature_code := tokens.builtin_real_n_ref_item
					l_queries := a_target_type.queries
					nb := a_target_type.attribute_count
					from i := 1 until i > nb loop
						l_query := l_queries.item (i)
						if
							l_query.builtin_class_code = l_builtin_item_class_code and then
							l_query.builtin_feature_code = l_builtin_item_feature_code
						then
							l_item_attribute := l_query
							i := nb + 1
						else
							i := i + 1
						end
					end
					if l_item_attribute /= Void then
							-- Set the built-in attribute 'item'.
						print_indentation
						print_adapted_attribute_access (l_item_attribute, l_target, a_target_type, a_check_void_target)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_character ('(')
						print_attachment_expression (l_argument, l_argument_type_set, l_formal_type)
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					else
						-- If `l_item_attribute' is Void, it means that it is never used,
						-- therefore there is no need to set it.
-- TODO: it can also be that it has been redefined.
					end
				end
			end
		end

	print_builtin_routine_call_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'FUNCTION.item' or 'PROCEDURE.call'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_open_operand_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_routine_object: ET_EXPRESSION
			l_tuple: ET_EXPRESSION
			l_tuple_target_type: ET_DYNAMIC_TYPE
			l_tuple_target_primary_type: detachable ET_DYNAMIC_TUPLE_TYPE
			l_tuple_target_type_set: ET_DYNAMIC_TYPE_SET
			l_tuple_source_type: ET_DYNAMIC_TYPE
			l_tuple_source_type_set: ET_DYNAMIC_TYPE_SET
			l_tuple_conforming_type_set: ET_DYNAMIC_STANDALONE_TYPE_SET
			l_has_non_conforming_types: BOOLEAN
			i, nb: INTEGER
			nb_open_operands: INTEGER
			j, nb2: INTEGER
			old_tuple_index: INTEGER
			l_tuple_label: ET_IDENTIFIER
			l_tuple_item_expression: ET_QUALIFIED_CALL_EXPRESSION
			l_tuple_item_type_set: ET_DYNAMIC_STANDALONE_TYPE_SET
			l_query_call: detachable ET_DYNAMIC_QUALIFIED_QUERY_CALL
			l_query_target_type_set: ET_DYNAMIC_STANDALONE_TYPE_SET
			l_operand: ET_EXPRESSION
			l_operand_type_set: ET_DYNAMIC_TYPE_SET
			l_old_extra_dynamic_type_sets_count: INTEGER
			l_temp: detachable ET_IDENTIFIER
		do
			if not attached {ET_DYNAMIC_ROUTINE_TYPE} a_target_type as l_routine_type then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
					-- This built-in feature can only be in class PROCEDURE (and
					-- its descendants) or class FUNCTION (and its descendants).
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_routine_type.attribute_count < 2 then
					-- Internal error: the Agent type should have at least the attributes
					-- 'rout_disp' and 'closed_operands' as first queries.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached l_routine_type.queries.item (2).result_type_set as l_closed_operands_type_set then
					-- Internal error: 'closed_operands' should be an attribute,
					-- hence with a result type set.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_routine_object := call_operands.first
				l_tuple := call_operands.item (2)
				l_tuple_source_type_set := dynamic_type_set (l_tuple)
				l_tuple_source_type := l_tuple_source_type_set.static_type
				l_tuple_target_type_set := argument_type_set_in_feature (1, a_feature)
				l_tuple_target_type := l_tuple_target_type_set.static_type
				l_open_operand_type_sets := l_routine_type.open_operand_type_sets
				nb_open_operands := l_open_operand_type_sets.count
				if system_processor.is_ise then
						-- ISE Eiffel does not type-check the tuple operand of Agent calls even at
						-- execution time. It only checks whether the tuple has enough items and
						-- these items are of the expected types, regardless of the type of the tuple
						-- itself. For example it is OK to pass a "TUPLE [ANY]" to an Agent which expects
						-- a "TUPLE [STRING]" provided that the dynamic type of the item of this tuple
						-- conforms to type STRING.
					if
						not attached {ET_DYNAMIC_TUPLE_TYPE} l_tuple_source_type.primary_type as l_tuple_source_primary_type or else
						l_tuple_source_primary_type.item_type_sets.count < nb_open_operands
					then
							-- There is not enough items in the tuple. Keep the real tuple target
							-- type so that a proper CAT-call error is emitted.
						if attached {ET_DYNAMIC_TUPLE_TYPE} l_tuple_target_type.primary_type as l_tuple_type then
							l_tuple_target_primary_type := l_tuple_type
						end
					else
						l_tuple_target_type := l_tuple_source_type
						l_tuple_target_primary_type := l_tuple_source_primary_type
					end
				elseif attached {ET_DYNAMIC_TUPLE_TYPE} l_tuple_target_type.primary_type as l_tuple_type then
					l_tuple_target_primary_type := l_tuple_type
				end
				if l_tuple_target_primary_type = Void then
						-- Internal error: The formal argument of this built-in
						-- routine should be a Tuple type.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					if not a_feature.is_procedure then
						current_file.put_character ('(')
					end
						-- Mark the temporary variable representing the tuple and the
						-- routine object as frozen so that they are not reused in any
						-- way when printing the tuple item extract expressions.
					mark_call_operands_frozen
						-- When printing the tuple item extract expressions, `call_operands'
						-- will be used and then wiped out. Put `call_operands' in a clean state
						-- beforehand to avoid any problem.
					call_operands.wipe_out
						-- Extract the items of the tuple to pass them as argument
						-- of the function to fill in the open operands of the agent.
					l_old_extra_dynamic_type_sets_count := extra_dynamic_type_sets.count
						-- Try to find out whether there are some potential CAT-calls,
						-- e.g. there are some Tuple types that don't conform to the
						-- expected type. Here is an example where this can occur:
						--
						--   p: PROCEDURE [ANY, TUPLE [ANY]]
						--   f (a: ANY; i: INTEGER) do ... end
						--   p := agent f
						--   p.call (["gobo"])
						--
						-- The dynamic type of 'p' is PROCEDURE [ANY, TUPLE [ANY, INTEGER]].
						-- So the expected argument type is 'TUPLE [ANY, INTEGER]'. But what
						-- is passed is of type 'TUPLE [STRING]'. We are missing the integer
						-- field to pass to feature 'f'.
					l_tuple_conforming_type_set := conforming_type_set (l_tuple_source_type_set, l_tuple_target_type)
					l_has_non_conforming_types := l_tuple_conforming_type_set.count /= l_tuple_source_type_set.count
					if l_has_non_conforming_types then
							-- Force the generation of a CAT-call check.
						if a_feature.is_procedure then
							l_temp := new_temp_variable (l_tuple_target_primary_type)
							print_indentation
							print_temp_name (l_temp, current_file)
							print_assign_to
							print_attachment_expression (l_tuple, l_tuple_source_type_set, l_tuple_target_type)
							print_semicolon_newline
							mark_temp_variable_free (l_temp)
						else
							print_attachment_expression (l_tuple, l_tuple_source_type_set, l_tuple_target_type)
							print_comma
						end
					end
					if a_feature.is_procedure then
						print_indentation
					end
					current_file.put_character ('(')
					current_file.put_character ('(')
					if attached l_routine_type.result_type_set as l_result_type_set then
						print_type_declaration (l_result_type_set.static_type.primary_type, current_file)
					else
						current_file.put_string (c_void)
					end
					current_file.put_character (' ')
					current_file.put_character ('(')
					current_file.put_character ('*')
					current_file.put_character (')')
					current_file.put_character ('(')
					current_file.put_string (c_ge_context)
					current_file.put_character ('*')
					print_comma
					print_type_declaration (l_closed_operands_type_set.static_type.primary_type, current_file)
					print_comma
					print_type_declaration (l_tuple_target_primary_type, current_file)
					from i := 1 until i > nb_open_operands loop
						print_comma
						print_type_declaration (l_open_operand_type_sets.item (i).static_type.primary_type, current_file)
						i := i + 1
					end
					current_file.put_character (')')
					current_file.put_character (')')
					current_file.put_character ('(')
						-- Print attribute 'rout_disp'.
					print_adapted_attribute_access (l_routine_type.queries.first, l_routine_object, l_routine_type, False)
					current_file.put_character (')')
					current_file.put_character (')')
					current_file.put_character ('(')
					current_file.put_string (c_ac)
					print_comma
						-- Print attribute 'closed_operands'.
						-- Note that if some closed arguments have the copy semantics, they will
						-- be cloned within the execution of 'root_disp' when passing the actual
						-- arguments to the underlying Eiffel routine (i.e. `f` when the agent is
						-- `agent a.f (b)`).
					print_adapted_attribute_access (l_routine_type.queries.item (2), l_routine_object, l_routine_type, False)
					print_comma
					if attached {ET_MANIFEST_TUPLE} l_tuple as l_manifest_tuple then
							-- There is no tuple for open operands here.
						current_file.put_character ('0')
						nb := l_manifest_tuple.count.min (nb_open_operands)
						from i := 1 until i > nb loop
							l_operand := l_manifest_tuple.expression (i)
							l_operand_type_set := dynamic_type_set (l_operand)
								-- Note that we did not call `print_attachment_operand' because
								-- the open arguments with copy semantics will be cloned within
								-- the execution of 'root_disp' when passing the actual arguments
								-- to the underlying Eiffel routine (i.e. `f` when the agent is
								-- `agent a.f (?)`). But we call `print_attachment_expression' below
								-- because we still need to box or unbox the arguments when needed
								-- (e.g. the formal argument of `f` is declared of type `ANY` but
								-- we call the agent with `my_agent.call ([2])`).
							print_comma
							print_attachment_expression (l_operand, l_operand_type_set, l_open_operand_type_sets.item (i).static_type)
							i := i + 1
						end
					elseif nb_open_operands > 0 then
							-- Tuple with the open operands.
						print_expression (l_tuple)
							-- Prepare dynamic type sets.
							-- Temporarily change the dynamic type set of the
							-- tuple so that it is only made up of conforming
							-- type, without any remaining CAT-call problems.
						extra_dynamic_type_sets.force_last (l_tuple_conforming_type_set)
						old_tuple_index := l_tuple.index
						l_tuple.set_index (current_dynamic_type_sets.count + extra_dynamic_type_sets.count)
							-- Extract the items of the tuple to pass them as argument
							-- of the function to fill in the open operands of the agent.
						from i := 1 until i > nb_open_operands loop
								-- Prepare the expression to extract the tuple item.
							create l_tuple_item_type_set.make_empty (l_tuple_target_primary_type.item_type_sets.item (i).static_type)
							nb2 := l_tuple_conforming_type_set.count
							from j := 1 until j > nb2 loop
								if attached {ET_DYNAMIC_TUPLE_TYPE} l_tuple_conforming_type_set.dynamic_type (j) as l_tuple_conforming_type then
									l_tuple_item_type_set.put_types (l_tuple_conforming_type.item_type_sets.item (i))
								else
										-- Internal error: The types conforming a tuple type should be tuple types.
									set_fatal_error
									error_handler.report_giaaa_error
								end
								j := j + 1
							end
							extra_dynamic_type_sets.force_last (l_tuple_item_type_set)
							l_tuple_label := new_tuple_label (i)
							l_tuple_item_expression := new_qualified_call_expression (l_tuple, l_tuple_label, Void)
							l_tuple_item_expression.set_index (current_dynamic_type_sets.count + extra_dynamic_type_sets.count)
								-- Register the call to extract the tuple item so that
								-- it is handled correctly in case of polymorphism.
								-- The special treatment for polymorphism only occurs
								-- when the target has more than 2 possible dynamic types.
							l_query_call := Void
							if l_tuple_conforming_type_set.count > 2 then
								create l_query_target_type_set.make (l_tuple_target_type)
								if l_tuple_conforming_type_set.is_never_void then
									l_query_target_type_set.set_never_void
								end
								l_query_target_type_set.put_types (l_tuple_conforming_type_set)
								create l_query_call.make (l_tuple_item_expression, l_query_target_type_set, l_tuple_item_type_set, current_feature, current_type)
								l_tuple_conforming_type_set.static_type.primary_type.put_query_call (l_query_call)
							end
								-- Print the actual call to extract the tuple item.
								-- Note that we did not call `print_attachment_operand' because
								-- the open arguments with copy semantics will be cloned within
								-- the execution of 'root_disp' when passing the actual arguments
								-- to the underlying Eiffel routine (i.e. `f` when the agent is
								-- `agent a.f (?)`). But we call `print_attachment_expression' below
								-- because we still need to box or unbox the arguments when needed
								-- (e.g. the formal argument of `f` is declared of type `ANY` but
								-- we call the agent with `my_agent.call ([2])`).
							print_comma
							print_attachment_expression (l_tuple_item_expression, l_tuple_item_type_set, l_open_operand_type_sets.item (i).static_type)
								-- Clean up.
							if l_query_call = Void then
								free_qualified_call_expression (l_tuple_item_expression)
								free_tuple_label (l_tuple_label)
							end
								-- No need to check for the void-ness of the tuple when accessing
								-- each of its items. The first time is enough.
							l_tuple_conforming_type_set.set_never_void
							i := i + 1
						end
							-- Clean up.
						l_tuple.set_index (old_tuple_index)
					else
							-- No need to pass the empty tuple with open operands here.
						current_file.put_character ('0')
					end
					current_file.put_character (')')
					if a_feature.is_procedure then
						print_semicolon_newline
					else
						current_file.put_character (')')
					end
						-- Clean up.
					extra_dynamic_type_sets.keep_first (l_old_extra_dynamic_type_sets_count)
						-- Put back the operands of current call onto `call_operands'.
					call_operands.wipe_out
					call_operands.put_last (l_routine_object)
					call_operands.put_last (l_tuple)
						-- Unfreeze the temporary variables corresponding to the tuple
						-- and routine objects.
					mark_call_operands_unfrozen
					free_standalone_type_set (l_tuple_conforming_type_set)
				end
			end
		end

	print_builtin_special_aliased_resized_area_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'SPECIAL.aliased_resized_area'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_argument_name: ET_IDENTIFIER
			l_item_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp: ET_IDENTIFIER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_capacity_type: ET_DYNAMIC_PRIMARY_TYPE
			l_result: ET_RESULT
		do
			l_arguments := a_feature.arguments
			if not attached {ET_DYNAMIC_SPECIAL_TYPE} current_type as l_special_type then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
					-- This built-in can only be in class SPECIAL (and its descendants).
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parse.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_special_type.attribute_count < 2 then
					-- Internal error: class "SPECIAL" should have at least the
					-- features 'count' and 'capacity' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_dynamic_type_set := result_type_set_in_feature (l_special_type.queries.item (2))
				l_capacity_type := l_dynamic_type_set.static_type.primary_type
				l_temp := new_temp_variable (l_capacity_type)
				l_item_type := l_special_type.item_type_set.static_type.primary_type
				print_indentation
				print_temp_name (l_temp, current_file)
				print_assign_to
				print_attribute_special_capacity_access (tokens.current_keyword, l_special_type, False)
				print_semicolon_newline
				print_indentation_if
				l_argument_name := l_arguments.formal_argument (1).name
				print_argument_name (l_argument_name, current_file)
				print_greater_than
				print_temp_name (l_temp, current_file)
				print_then_newline
				indent
					-- Need to allocate a new object.
				l_result := new_result_expression
				extra_dynamic_type_sets.force_last (l_special_type)
				l_result.set_index (current_dynamic_type_sets.count + extra_dynamic_type_sets.count)
				print_indentation_assign_to_result
				current_file.put_string (c_ge_new)
				current_file.put_integer (l_special_type.id)
				current_file.put_character ('(')
				print_argument_name (l_argument_name, current_file)
				print_comma
				current_file.put_string (c_eif_true)
				current_file.put_character (')')
				print_semicolon_newline
				mark_temp_variable_free (l_temp)
					-- Set count.
				print_indentation
				print_attribute_special_count_access (tokens.result_keyword, l_special_type, False)
				print_assign_to
				print_attribute_special_count_access (tokens.current_keyword, l_special_type, False)
				print_semicolon_newline
					-- Copy items.
				print_builtin_any_standard_copy_body_with_special (tokens.current_keyword, l_result, l_special_type)
					-- Clean up.
				free_result_expression (l_result)
				extra_dynamic_type_sets.remove_last
				dedent
				print_indentation_else_newline
				indent
					-- Keep the same object.
				print_indentation_assign_to_result
				print_current_name (current_file)
				print_semicolon_newline
					-- Set new count if needed.
				print_indentation
				print_temp_name (l_temp, current_file)
				print_assign_to
				print_attribute_special_count_access (tokens.result_keyword, l_special_type, False)
				print_semicolon_newline
				print_indentation_if
				print_temp_name (l_temp, current_file)
				print_greater_than
				print_argument_name (l_argument_name, current_file)
				print_then_newline
				indent
				print_indentation
				print_attribute_special_count_access (tokens.result_keyword, l_special_type, False)
				print_assign_to
				print_argument_name (l_argument_name, current_file)
				print_semicolon_newline
					-- Clear discarded item slots if needed.
				current_file.put_string (c_ifndef)
				current_file.put_character (' ')
				if l_special_type.has_nested_reference_fields then
					current_file.put_line (c_ge_malloc_cleared)
				else
					current_file.put_line (c_ge_malloc_atomic_cleared)
				end
				print_indentation
				current_file.put_string (c_memset)
				current_file.put_character ('(')
				print_attribute_special_item_access (tokens.result_keyword, l_special_type, False)
				print_plus
				print_argument_name (l_argument_name, current_file)
				print_comma
				current_file.put_character ('0')
				print_comma
				current_file.put_character ('(')
				print_temp_name (l_temp, current_file)
				print_minus
				print_argument_name (l_argument_name, current_file)
				current_file.put_character (')')
				print_times
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				print_type_declaration (l_item_type, current_file)
				current_file.put_character (')')
				current_file.put_character (')')
				print_semicolon_newline
				current_file.put_line (c_endif)
				dedent
				print_indentation_end_newline
				dedent
				print_indentation_end_newline
					-- Set new capacity.
				print_indentation
				print_attribute_special_capacity_access (tokens.result_keyword, l_special_type, False)
				print_assign_to
				print_argument_name (l_argument_name, current_file)
				print_semicolon_newline
			end
		end

	print_builtin_special_base_address_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'SPECIAL.base_address'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				print_attribute_special_item_access (call_operands.first, a_target_type, a_check_void_target)
				current_file.put_character (')')
			end
		end

	print_builtin_special_capacity_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'SPECIAL.capacity'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_attribute_special_capacity_access (call_operands.first, a_target_type, a_check_void_target)
		end

	print_builtin_special_count_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'SPECIAL.count'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			print_attribute_special_count_access (call_operands.first, a_target_type, a_check_void_target)
		end

	print_builtin_special_element_size_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'SPECIAL.element_size'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if not attached {ET_DYNAMIC_SPECIAL_TYPE} a_target_type as l_special_type then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
					-- This built-in can only be in class SPECIAL (and its descendants).
				set_fatal_error
				error_handler.report_giaaa_error
			else
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				print_type_declaration (l_special_type.item_type_set.static_type.primary_type, current_file)
				current_file.put_character (')')
			end
		end

	print_builtin_special_extend_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'SPECIAL.extend'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_temp: ET_IDENTIFIER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_count_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			l_arguments := a_feature.arguments
			if not attached {ET_DYNAMIC_SPECIAL_TYPE} current_type as l_special_type then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
					-- This built-in can only be in class SPECIAL (and its descendants).
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parse.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_special_type.attribute_count < 2 then
					-- Internal error: class "SPECIAL" should have at least the
					-- features 'count' and 'capacity' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_dynamic_type_set := result_type_set_in_feature (l_special_type.queries.first)
				l_count_type := l_dynamic_type_set.static_type.primary_type
				l_temp := new_temp_variable (l_count_type)
					-- Increment 'count'.
				print_indentation
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attribute_special_count_access (tokens.current_keyword, l_special_type, False)
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				print_attribute_special_count_access (tokens.current_keyword, l_special_type, False)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('+')
				current_file.put_character (' ')
				current_file.put_character ('1')
				current_file.put_character (' ')
				current_file.put_character (';')
				current_file.put_new_line
					-- Put the new item.
				print_indentation
				print_attribute_special_item_access (tokens.current_keyword, l_special_type, False)
				current_file.put_character ('[')
				print_temp_name (l_temp, current_file)
				current_file.put_character (']')
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_builtin_special_item_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'SPECIAL.item'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attribute_special_item_access (call_operands.first, a_target_type, a_check_void_target)
				current_file.put_character ('[')
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (']')
			end
		end

	print_builtin_special_put_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'SPECIAL.put'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			if call_operands.count /= 3 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				print_attribute_special_item_access (call_operands.first, a_target_type, a_check_void_target)
				current_file.put_character ('[')
				l_argument := call_operands.item (3)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (2, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (']')
				print_assign_to
				current_file.put_character ('(')
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
				current_file.put_character (')')
				print_semicolon_newline
			end
		end

	print_builtin_special_set_count_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'SPECIAL.set_count'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_argument_name: ET_IDENTIFIER
			l_temp: ET_IDENTIFIER
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_count_type: ET_DYNAMIC_PRIMARY_TYPE
			l_item_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			l_arguments := a_feature.arguments
			if not attached {ET_DYNAMIC_SPECIAL_TYPE} current_type as l_special_type then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
					-- This built-in can only be in class SPECIAL (and its descendants).
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parse.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_special_type.attribute_count < 2 then
					-- Internal error: class "SPECIAL" should have at least the
					-- features 'count' and 'capacity' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_dynamic_type_set := result_type_set_in_feature (l_special_type.queries.first)
				l_count_type := l_dynamic_type_set.static_type.primary_type
				l_temp := new_temp_variable (l_count_type)
					-- Set new 'count'.
				print_indentation
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attribute_special_count_access (tokens.current_keyword, l_special_type, False)
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				print_attribute_special_count_access (tokens.current_keyword, l_special_type, False)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				l_argument_name := l_arguments.formal_argument (1).name
				print_argument_name (l_argument_name, current_file)
				current_file.put_character (';')
				current_file.put_new_line
					-- Clear discarded item slots if needed.
				current_file.put_string (c_ifndef)
				current_file.put_character (' ')
				if l_special_type.has_nested_reference_fields then
					current_file.put_line (c_ge_malloc_cleared)
				else
					current_file.put_line (c_ge_malloc_atomic_cleared)
				end
				print_indentation
				current_file.put_string (c_memset)
				current_file.put_character ('(')
				print_attribute_special_item_access (tokens.current_keyword, l_special_type, False)
				current_file.put_character ('+')
				print_argument_name (l_argument_name, current_file)
				current_file.put_character (',')
				current_file.put_character ('0')
				current_file.put_character (',')
				current_file.put_character ('(')
				print_temp_name (l_temp, current_file)
				current_file.put_character ('-')
				print_argument_name (l_argument_name, current_file)
				current_file.put_character (')')
				current_file.put_character ('*')
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				l_item_type := l_special_type.item_type_set.static_type.primary_type
				print_type_declaration (l_item_type, current_file)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				current_file.put_line (c_endif)
			end
		end

	print_builtin_tuple_basic_expanded_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to a built-in feature of class "TUPLE" that returns the items
			-- of basic expanded types.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_result_type: ET_DYNAMIC_TYPE
			i, nb: INTEGER
		do
			l_arguments := a_feature.arguments
			if not attached {ET_DYNAMIC_TUPLE_TYPE} current_type as l_tuple_type then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
					-- This built-in feature can only be in class TUPLE (and
					-- its descendants).
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached current_feature.result_type_set as l_result_type_set then
					-- Internal error: `current_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_result_type := l_result_type_set.static_type
				print_indentation
				current_file.put_string (c_switch)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				l_item_type_sets := l_tuple_type.item_type_sets
				nb := l_item_type_sets.count
				from i := 1 until i > nb loop
					if l_item_type_sets.item (i).static_type = l_result_type then
						print_indentation
						current_file.put_string (c_case)
						current_file.put_character (' ')
						current_file.put_integer (i)
						current_file.put_character (':')
						current_file.put_new_line
						indent
						print_indentation
						print_result_name (current_file)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						print_attribute_tuple_item_access (i, tokens.current_keyword, l_tuple_type, False)
						current_file.put_character (';')
						current_file.put_new_line
						print_indentation
						current_file.put_string (c_break)
						current_file.put_character (';')
						current_file.put_new_line
						dedent
					end
					i := i + 1
				end
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
			end
		end

	print_builtin_tuple_boolean_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.boolean_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_character_8_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.character_8_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_character_32_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.character_32_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_count_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TUPLE.count'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			if not attached {ET_DYNAMIC_TUPLE_TYPE} a_target_type as l_tuple_type then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
					-- This built-in feature can only be in class TUPLE (and
					-- its descendants).
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_integer (l_tuple_type.item_type_sets.count)
				current_file.put_character (')')
			end
		end

	print_builtin_tuple_integer_8_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.integer_8_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_integer_16_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.integer_16_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_integer_32_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.integer_32_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_integer_64_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.integer_64_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_item_code_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.item_code'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_item_type: ET_DYNAMIC_PRIMARY_TYPE
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
		do
			l_arguments := a_feature.arguments
			if not attached {ET_DYNAMIC_TUPLE_TYPE} current_type as l_tuple_type then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
					-- This built-in feature can only be in class TUPLE (and
					-- its descendants).
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached current_feature.result_type_set as l_result_type_set then
					-- Internal error: `current_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				current_file.put_string (c_switch)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				l_item_type_sets := l_tuple_type.item_type_sets
				l_result_type := l_result_type_set.static_type.primary_type
				nb := l_item_type_sets.count
				from i := 1 until i > nb loop
					print_indentation
					current_file.put_string (c_case)
					current_file.put_character (' ')
					current_file.put_integer (i)
					current_file.put_character (':')
					current_file.put_new_line
					indent
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_type_cast (l_result_type, current_file)
					l_item_type := l_item_type_sets.item (i).static_type.primary_type
					if current_universe_impl.boolean_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x01)
					elseif current_universe_impl.character_8_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x02)
					elseif current_universe_impl.character_32_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x0E)
					elseif current_universe_impl.integer_8_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x06)
					elseif current_universe_impl.integer_16_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x07)
					elseif current_universe_impl.integer_32_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x08)
					elseif current_universe_impl.integer_64_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x09)
					elseif current_universe_impl.natural_8_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x0A)
					elseif current_universe_impl.natural_16_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x0B)
					elseif current_universe_impl.natural_32_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x0C)
					elseif current_universe_impl.natural_64_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x0D)
					elseif current_universe_impl.pointer_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x05)
					elseif current_universe_impl.real_32_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x04)
					elseif current_universe_impl.real_64_type.same_named_type (l_item_type.base_type, current_type.base_type, current_type.base_type) then
						current_file.put_integer (0x03)
					else
						current_file.put_integer (0x00)
					end
					current_file.put_character (';')
					current_file.put_new_line
					print_indentation
					current_file.put_string (c_break)
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					i := i + 1
				end
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
			end
		end

	print_builtin_tuple_natural_8_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.natural_8_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_natural_16_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.natural_16_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_natural_32_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.natural_32_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_natural_64_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.natural_64_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_object_comparison_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TUPLE.object_comparison'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
				-- Internal attribute.
			print_adapted_attribute_access (a_feature, call_operands.first, a_target_type, a_check_void_target)
		end

	print_builtin_tuple_pointer_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.pointer_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_basic_expanded_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to a built-in feature of class "TUPLE" that sets the items
			-- of basic expanded types.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_argument_type: ET_DYNAMIC_TYPE
			i, nb: INTEGER
		do
			l_arguments := a_feature.arguments
			if not attached {ET_DYNAMIC_TUPLE_TYPE} current_type as l_tuple_type then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
					-- This built-in feature can only be in class TUPLE (and
					-- its descendants).
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 2 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument_type := argument_type_set (1).static_type
				print_indentation
				current_file.put_string (c_switch)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_argument_name (l_arguments.formal_argument (2).name, current_file)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				l_item_type_sets := l_tuple_type.item_type_sets
				nb := l_item_type_sets.count
				from i := 1 until i > nb loop
					if l_item_type_sets.item (i).static_type = l_argument_type then
						print_indentation
						current_file.put_string (c_case)
						current_file.put_character (' ')
						current_file.put_integer (i)
						current_file.put_character (':')
						current_file.put_new_line
						indent
						print_indentation
						print_attribute_tuple_item_access (i, tokens.current_keyword, l_tuple_type, False)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						print_argument_name (l_arguments.formal_argument (1).name, current_file)
						current_file.put_character (';')
						current_file.put_new_line
						print_indentation
						current_file.put_string (c_break)
						current_file.put_character (';')
						current_file.put_new_line
						dedent
					end
					i := i + 1
				end
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
			end
		end

	print_builtin_tuple_put_boolean_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_boolean'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_character_8_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_character_8'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_character_32_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_character_32'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_integer_8_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_integer_8'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_integer_16_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_integer_16'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_integer_32_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_integer_32'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_integer_64_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_integer_64'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_natural_8_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_natural_8'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_natural_16_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_natural_16'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_natural_32_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_natural_32'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_natural_64_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_natural_64'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_pointer_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_pointer'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_real_32_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_real_32'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_real_64_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_real_64'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_put_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_put_reference_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.put_reference'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_item_type: ET_DYNAMIC_TYPE
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_argument: ET_IDENTIFIER
			i, nb: INTEGER
		do
			l_arguments := a_feature.arguments
			if not attached {ET_DYNAMIC_TUPLE_TYPE} current_type as l_tuple_type then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
					-- This built-in feature can only be in class TUPLE (and
					-- its descendants).
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 2 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_argument := l_arguments.formal_argument (1).name
				l_argument_type_set := argument_type_set (1)
				print_indentation
				current_file.put_string (c_switch)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_argument_name (l_arguments.formal_argument (2).name, current_file)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				l_item_type_sets := l_tuple_type.item_type_sets
				nb := l_item_type_sets.count
				from i := 1 until i > nb loop
					l_item_type := l_item_type_sets.item (i).static_type
					if not l_item_type.is_expanded or else not l_item_type.is_basic then
						print_indentation
						current_file.put_string (c_case)
						current_file.put_character (' ')
						current_file.put_integer (i)
						current_file.put_character (':')
						current_file.put_new_line
						indent
						print_indentation
						print_attribute_tuple_item_access (i, tokens.current_keyword, l_tuple_type, False)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
							-- Note: Use `print_attachment_expression' here so that there is no object of the wrong type
							-- assigned to the Tuple item (this is forbidden by the preconditions, but we should not let them
							-- go through when preconditions are turn off for example). We did not call
							-- `print_attachment_operand' beforehand because we don't want to clone the object if it
							-- has copy semantics. This should already have been done when calling `a_feature'.
						print_attachment_expression (l_argument, l_argument_type_set, l_item_type)
						current_file.put_character (';')
						current_file.put_new_line
						print_indentation
						current_file.put_string (c_break)
						current_file.put_character (';')
						current_file.put_new_line
						dedent
					end
					i := i + 1
				end
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
			end
		end

	print_builtin_tuple_real_32_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.real_32_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_real_64_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.real_64_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_builtin_tuple_basic_expanded_item_body (a_feature)
		end

	print_builtin_tuple_reference_item_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TUPLE.reference_item'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_item_type_set: ET_DYNAMIC_TYPE_SET
			l_item_type: ET_DYNAMIC_TYPE
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			i, nb: INTEGER
		do
			l_arguments := a_feature.arguments
			if not attached {ET_DYNAMIC_TUPLE_TYPE} current_type as l_tuple_type then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
					-- This built-in feature can only be in class TUPLE (and
					-- its descendants).
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_indentation
				current_file.put_string (c_switch)
				current_file.put_character (' ')
				current_file.put_character ('(')
				print_argument_name (l_arguments.formal_argument (1).name, current_file)
				current_file.put_character (')')
				current_file.put_character (' ')
				current_file.put_character ('{')
				current_file.put_new_line
				l_item_type_sets := l_tuple_type.item_type_sets
				nb := l_item_type_sets.count
				from i := 1 until i > nb loop
					l_item_type_set := l_item_type_sets.item (i)
					l_item_type := l_item_type_set.static_type
					if not l_item_type.is_expanded or else not l_item_type.is_basic then
						print_indentation
						current_file.put_string (c_case)
						current_file.put_character (' ')
						current_file.put_integer (i)
						current_file.put_character (':')
						current_file.put_new_line
						indent
						print_indentation
						print_result_name (current_file)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						print_adapted_expression_with_agent (agent print_attribute_tuple_item_access (i, tokens.current_keyword, l_tuple_type, False), l_item_type_set, current_dynamic_system.any_type, False)
						current_file.put_character (';')
						current_file.put_new_line
						print_indentation
						current_file.put_string (c_break)
						current_file.put_character (';')
						current_file.put_new_line
						dedent
					end
					i := i + 1
				end
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
			end
		end

	print_builtin_tuple_set_object_comparison_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TUPLE.set_object_comparison'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_target: ET_EXPRESSION
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_query: ET_DYNAMIC_FEATURE
			l_object_comparison_attribute: detachable ET_DYNAMIC_FEATURE
			i, nb: INTEGER
			l_builtin_object_comparison_class_code: NATURAL_8
			l_builtin_object_comparison_feature_code: NATURAL_8
		do
			if call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_target := call_operands.first
				l_argument := call_operands.item (2)
				l_actual_type_set := dynamic_type_set (l_argument)
				l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
				l_builtin_object_comparison_class_code := tokens.builtin_tuple_class
				l_builtin_object_comparison_feature_code := tokens.builtin_tuple_object_comparison
				l_queries := a_target_type.queries
				nb := a_target_type.attribute_count
				from i := 1 until i > nb loop
					l_query := l_queries.item (i)
					if
						l_query.builtin_class_code = l_builtin_object_comparison_class_code and then
						l_query.builtin_feature_code = l_builtin_object_comparison_feature_code
					then
						l_object_comparison_attribute := l_query
						i := nb + 1
					else
						i := i + 1
					end
				end
				if l_object_comparison_attribute /= Void then
						-- Set the built-in attribute 'object_comparison'.
					print_indentation
					print_adapted_attribute_access (l_object_comparison_attribute, l_target, a_target_type, a_check_void_target)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
				else
					-- If `l_object_comparison_attribute' is Void, it means that it is never used,
					-- therefore there is no need to set it.
				end
			end
		end

	print_builtin_type_default_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TYPE.default'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_parameters: detachable ET_ACTUAL_PARAMETERS
			l_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			l_parameters := a_target_type.base_type.actual_parameters
			if l_parameters = Void or else l_parameters.count < 1 then
					-- Internal error: we should have already checked by now
					-- that class TYPE has a generic parameter.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_type := current_dynamic_system.dynamic_primary_type (l_parameters.type (1), a_target_type.base_type)
				print_typed_default_entity_value (l_type, current_file)
			end
		end

	print_builtin_type_generic_parameter_type_body (a_feature: ET_EXTERNAL_ROUTINE)
			-- Print to `current_file' the body of `a_feature' corresponding
			-- to built-in feature 'TYPE.generic_parameter_type'.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		local
			l_parameters: detachable ET_ACTUAL_PARAMETERS
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_base_type: ET_BASE_TYPE
			l_type: ET_TYPE
			l_dynamic_type: ET_DYNAMIC_TYPE
			l_meta_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
		do
			l_arguments := a_feature.arguments
			l_parameters := current_type.base_type.actual_parameters
			if l_parameters = Void or else l_parameters.count < 1 then
					-- Internal error: we should have already checked by now
					-- that class TYPE has a generic parameter.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif l_arguments = Void or else l_arguments.count /= 1 then
					-- Internal error: this error should have been reported by the parser.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_base_type := l_parameters.type (1).shallow_base_type (current_type.base_type)
				l_parameters := l_base_type.actual_parameters
				if l_parameters = Void or else l_parameters.is_empty then
					-- Do nothing.
				elseif l_parameters.count = 1 then
					l_type := l_parameters.type (1)
					l_dynamic_type := current_dynamic_system.dynamic_type (l_type, current_type.base_type)
					l_meta_type := l_dynamic_type.meta_type
					if l_meta_type = Void then
							-- Internal error: the meta type of the generic parameter should
							-- have been computed when analyzing the dynamic type sets of
							-- `a_feature'.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						print_indentation
						print_result_name (current_file)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_character ('(')
						print_type_declaration (l_meta_type, current_file)
						current_file.put_character (')')
						current_file.put_character ('&')
						current_file.put_character ('(')
						current_file.put_string (c_ge_types)
						current_file.put_character ('[')
						current_file.put_integer (l_dynamic_type.primary_type.id)
						current_file.put_character (']')
						current_file.put_character ('[')
						if l_dynamic_type = l_dynamic_type.primary_type then
							current_file.put_integer (0)
						else
							current_file.put_integer (1)
						end
						current_file.put_character (']')
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					end
				else
					print_indentation
					current_file.put_string (c_switch)
					current_file.put_character (' ')
					current_file.put_character ('(')
					print_argument_name (l_arguments.formal_argument (1).name, current_file)
					current_file.put_character (')')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_new_line
					nb := l_parameters.count
					from i := 1 until i > nb loop
						l_type := l_parameters.type (i)
						l_dynamic_type := current_dynamic_system.dynamic_type (l_type, current_type.base_type)
						l_meta_type := l_dynamic_type.meta_type
						if l_meta_type = Void then
								-- Internal error: the meta type of the generic parameter should
								-- have been computed when analyzing the dynamic type sets of
								-- `a_feature'.
							set_fatal_error
							error_handler.report_giaaa_error
						else
							print_indentation
							current_file.put_string (c_case)
							current_file.put_character (' ')
							current_file.put_integer (i)
							current_file.put_character (':')
							current_file.put_new_line
							indent
							print_indentation
							print_result_name (current_file)
							current_file.put_character (' ')
							current_file.put_character ('=')
							current_file.put_character (' ')
							current_file.put_character ('(')
							print_type_declaration (l_meta_type, current_file)
							current_file.put_character (')')
							current_file.put_character ('&')
							current_file.put_character ('(')
							current_file.put_string (c_ge_types)
							current_file.put_character ('[')
							current_file.put_integer (l_dynamic_type.primary_type.id)
							current_file.put_character (']')
							current_file.put_character ('[')
							if l_dynamic_type = l_dynamic_type.primary_type then
								current_file.put_integer (0)
							else
								current_file.put_integer (1)
							end
							current_file.put_character (']')
							current_file.put_character (')')
							current_file.put_character (';')
							current_file.put_new_line
							print_indentation
							current_file.put_string (c_break)
							current_file.put_character (';')
							current_file.put_new_line
							dedent
						end
						i := i + 1
					end
					print_indentation
					current_file.put_string (c_default)
					current_file.put_character (':')
					current_file.put_new_line
					indent
					print_indentation
					current_file.put_string (c_ge_raise)
					current_file.put_character ('(')
					current_file.put_string (c_ge_ex_prog)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					dedent
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
				end
			end
		end

	print_builtin_type_generic_parameter_count_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TYPE.generic_parameter_count'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_base_type: ET_BASE_TYPE
			l_parameters: detachable ET_ACTUAL_PARAMETERS
		do
			l_base_type := a_target_type.base_type
			l_parameters := l_base_type.actual_parameters
			if l_parameters = Void or else l_parameters.count < 1 then
					-- Internal error: we should have already checked by now
					-- that class TYPE has a generic parameter.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached a_feature.result_type_set as l_result_type_set then
					-- Internal error: `a_feature' is a query.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				print_type_cast (l_result_type_set.static_type.primary_type, current_file)
				current_file.put_character ('(')
				current_file.put_integer (l_parameters.type (1).base_type_actual_count (l_base_type))
				current_file.put_character (')')
			end
		end

	print_builtin_type_has_default_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TYPE.has_default'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_parameters: detachable ET_ACTUAL_PARAMETERS
		do
			l_parameters := a_target_type.base_type.actual_parameters
			if l_parameters = Void or else l_parameters.count < 1 then
					-- Internal error: we should have already checked by now
					-- that class TYPE has a generic parameter.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not current_system.attachment_type_conformance_mode then
				current_file.put_string (c_eif_true)
			elseif l_parameters.type (1).is_type_expanded (a_target_type.base_type) then
				current_file.put_string (c_eif_true)
			elseif l_parameters.type (1).is_type_detachable (a_target_type.base_type) then
				current_file.put_string (c_eif_true)
			else
				current_file.put_string (c_eif_false)
			end
		end

	print_builtin_type_is_attached_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TYPE.is_attached'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			current_file.put_string (c_ge_is_attached_encoded_type)
			current_file.put_character ('(')
			current_file.put_character ('(')
			current_file.put_character ('(')
			current_file.put_string (c_eif_type)
			current_file.put_character ('*')
			current_file.put_character (')')
			current_file.put_character ('(')
			print_target_expression (call_operands.first, a_target_type, a_check_void_target)
			current_file.put_character (')')
			current_file.put_character (')')
			current_file.put_string (c_arrow)
			current_file.put_string (c_type_id)
			current_file.put_character (')')
		end

	print_builtin_type_is_deferred_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TYPE.is_deferred'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			current_file.put_string (c_ge_is_deferred_encoded_type)
			current_file.put_character ('(')
			current_file.put_character ('(')
			current_file.put_character ('(')
			current_file.put_string (c_eif_type)
			current_file.put_character ('*')
			current_file.put_character (')')
			current_file.put_character ('(')
			print_target_expression (call_operands.first, a_target_type, a_check_void_target)
			current_file.put_character (')')
			current_file.put_character (')')
			current_file.put_string (c_arrow)
			current_file.put_string (c_type_id)
			current_file.put_character (')')
		end

	print_builtin_type_is_expanded_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TYPE.is_expanded'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_parameters: detachable ET_ACTUAL_PARAMETERS
		do
			l_parameters := a_target_type.base_type.actual_parameters
			if l_parameters = Void or else l_parameters.count < 1 then
					-- Internal error: we should have already checked by now
					-- that class TYPE has a generic parameter.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				if l_parameters.type (1).is_type_expanded (a_target_type.base_type) then
					current_file.put_string (c_eif_true)
				else
					current_file.put_string (c_eif_false)
				end
			end
		end

	print_builtin_type_new_instance_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TYPE.new_instance'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_parameters: detachable ET_ACTUAL_PARAMETERS
			l_type: ET_DYNAMIC_TYPE
			l_primary_type: ET_DYNAMIC_PRIMARY_TYPE
			l_meta_type: detachable ET_DYNAMIC_PRIMARY_TYPE
		do
			l_parameters := a_target_type.base_type.actual_parameters
			if l_parameters = Void or else l_parameters.count < 1 then
					-- Internal error: we should have already checked by now
					-- that class TYPE has a generic parameter.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_type := current_dynamic_system.dynamic_type (l_parameters.type (1), a_target_type.base_type)
				l_primary_type := l_type.primary_type
				if attached {ET_DYNAMIC_SPECIAL_TYPE} l_primary_type as l_special_type then
						-- This should never happen according to the precondition
						-- of TYPE.new_instance.
						-- Raise an exception and return Void.
					current_file.put_character ('(')
					current_file.put_string (c_ge_raise)
					current_file.put_character ('(')
					current_file.put_string (c_ge_ex_fatal)
					current_file.put_character (')')
					current_file.put_character (',')
					current_file.put_character (' ')
					print_typed_default_entity_value (l_primary_type, current_file)
					current_file.put_character (')')
				elseif l_primary_type.base_class.is_deferred or l_primary_type.base_class.is_none then
					current_file.put_character ('(')
					current_file.put_string (c_ge_raise)
					current_file.put_character ('(')
					current_file.put_string (c_ge_ex_cdef)
					current_file.put_character (')')
					current_file.put_character (',')
					current_file.put_character (' ')
					print_typed_default_entity_value (l_primary_type, current_file)
					current_file.put_character (')')
				elseif not current_dynamic_system.is_new_instance_type (l_primary_type) then
						-- Raise an exception and return Void when the result type has not been
						-- specified as a type which can have instances created by 'TYPE.new_instance'
						-- or 'TYPE.new_special_any_instance'.
					current_file.put_character ('(')
					current_file.put_string (c_ge_raise)
					current_file.put_character ('(')
					current_file.put_string (c_ge_ex_fatal)
					current_file.put_character (')')
					current_file.put_character (',')
					current_file.put_character (' ')
					print_typed_default_entity_value (l_primary_type, current_file)
					current_file.put_character (')')
				elseif not l_primary_type.is_alive then
						-- Raise an exception and return Void when the result type is not alive
						-- (i.e. no object of that type has been otherwise created in the system).
					current_file.put_character ('(')
					current_file.put_string (c_ge_raise)
					current_file.put_character ('(')
					current_file.put_string (c_ge_ex_fatal)
					current_file.put_character (')')
					current_file.put_character (',')
					current_file.put_character (' ')
					print_typed_default_entity_value (l_primary_type, current_file)
					current_file.put_character (')')
				elseif l_primary_type.base_class.is_type_class then
						-- Cannot have two instances of class TYPE representing the same Eiffel type.
					l_meta_type := l_type.meta_type
					if l_meta_type = Void then
							-- Internal error: the meta type of the result type should
							-- have been computed when analyzing the dynamic type sets
							-- of `a_feature'.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						current_file.put_character ('(')
						print_type_declaration (l_meta_type, current_file)
						current_file.put_character (')')
						current_file.put_character ('&')
						current_file.put_character ('(')
						current_file.put_string (c_ge_types)
						current_file.put_character ('[')
						current_file.put_integer (l_primary_type.id)
						current_file.put_character (']')
						current_file.put_character ('[')
						if l_type = l_primary_type then
							current_file.put_integer (0)
						else
							current_file.put_integer (1)
						end
						current_file.put_character (']')
						current_file.put_character (')')
					end
				elseif l_primary_type.is_expanded then
					print_typed_default_entity_value (l_primary_type, current_file)
				else
					current_file.put_character ('(')
					print_type_declaration (l_primary_type, current_file)
					current_file.put_character (')')
					current_file.put_character ('(')
					current_file.put_string (c_ge_new)
					current_file.put_integer (l_primary_type.id)
					current_file.put_character ('(')
					current_file.put_string (c_eif_true)
					current_file.put_character (')')
					current_file.put_character (')')
				end
			end
		end

	print_builtin_type_new_special_any_instance_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TYPE.new_special_any_instance'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		local
			l_parameters: detachable ET_ACTUAL_PARAMETERS
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
			l_argument: ET_EXPRESSION
			l_actual_type_set: ET_DYNAMIC_TYPE_SET
			l_formal_type: ET_DYNAMIC_TYPE
		do
			l_parameters := a_target_type.base_type.actual_parameters
			if l_parameters = Void or else l_parameters.count < 1 then
					-- Internal error: we should have already checked by now
					-- that class TYPE has a generic parameter.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif call_operands.count /= 2 then
					-- Internal error: this should already have been reported in ET_FEATURE_FLATTENER.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_result_type := current_dynamic_system.dynamic_primary_type (l_parameters.type (1), a_target_type.base_type)
				if not attached {ET_DYNAMIC_SPECIAL_TYPE} l_result_type as l_special_type or else l_special_type.item_type_set.static_type.is_expanded then
						-- This should never happen according to the precondition
						-- of TYPE.new_special_any_instance.
						-- Raise an exception and return Void or
						-- the default value when the type is expanded.
					current_file.put_character ('(')
					current_file.put_string (c_ge_raise)
					current_file.put_character ('(')
					current_file.put_string (c_ge_ex_fatal)
					current_file.put_character (')')
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_string (c_eif_void)
					current_file.put_character (')')
				elseif not current_dynamic_system.is_new_instance_type (l_result_type) then
						-- Raise an exception and return Void when the result type has not been
						-- specified as a type which can have instances created by 'TYPE.new_instance'
						-- or 'TYPE.new_special_any_instance'.
					current_file.put_character ('(')
					current_file.put_string (c_ge_raise)
					current_file.put_character ('(')
					current_file.put_string (c_ge_ex_fatal)
					current_file.put_character (')')
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_string (c_eif_void)
					current_file.put_character (')')
				elseif not l_result_type.is_alive then
						-- Raise an exception and return Void when the result type is not alive
						-- (i.e. no object of that type has been otherwise created in the system).
					current_file.put_character ('(')
					current_file.put_string (c_ge_raise)
					current_file.put_character ('(')
					current_file.put_string (c_ge_ex_fatal)
					current_file.put_character (')')
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_string (c_eif_void)
					current_file.put_character (')')
				else
					current_file.put_character ('(')
					print_type_declaration (l_special_type, current_file)
					current_file.put_character (')')
					current_file.put_character ('(')
					current_file.put_string (c_ge_new)
					current_file.put_integer (l_special_type.id)
					current_file.put_character ('(')
					l_argument := call_operands.item (2)
					l_actual_type_set := dynamic_type_set (l_argument)
					l_formal_type := argument_type_set_in_feature (1, a_feature).static_type
					print_attachment_expression (l_argument, l_actual_type_set, l_formal_type)
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_string (c_eif_true)
					current_file.put_character (')')
					current_file.put_character (')')
				end
			end
		end

	print_builtin_type_runtime_name_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TYPE.runtime_name'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			type_info_name_used := True
			current_file.put_string (c_ge_generating_type_of_encoded_type)
			current_file.put_character ('(')
			current_file.put_character ('(')
			current_file.put_character ('(')
			current_file.put_string (c_eif_type)
			current_file.put_character ('*')
			current_file.put_character (')')
			current_file.put_character ('(')
			print_target_expression (call_operands.first, a_target_type, a_check_void_target)
			current_file.put_character (')')
			current_file.put_character (')')
			current_file.put_string (c_arrow)
			current_file.put_string (c_type_id)
			current_file.put_character (')')
		end

	print_builtin_type_type_id_call (a_feature: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print to `current_file' a call (static binding) to `a_feature'
			-- corresponding to built-in feature 'TYPE.type_id'.
			-- `a_target_type' is the dynamic type of the target.
			-- `a_check_void_target' means that we need to check whether the target is Void or not.
			-- Operands can be found in `call_operands'.
		require
			a_feature_not_void: a_feature /= Void
			a_target_type_not_void: a_target_type /= Void
			call_operands_not_empty: not call_operands.is_empty
		do
			current_file.put_character ('(')
			current_file.put_character ('(')
			current_file.put_string (c_eif_type)
			current_file.put_character ('*')
			current_file.put_character (')')
			current_file.put_character ('(')
			print_target_expression (call_operands.first, a_target_type, a_check_void_target)
			current_file.put_character (')')
			current_file.put_character (')')
			current_file.put_string (c_arrow)
			current_file.put_string (c_type_id)
		end

feature {NONE} -- C function generation

	print_main_function
			-- Print 'main' function to `current_file'.
		local
			l_root_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_root_creation_procedure: detachable ET_DYNAMIC_FEATURE
			l_root_creation: ET_CREATE_EXPRESSION
			l_root_call: ET_QUALIFIED_CALL
			l_temp: ET_IDENTIFIER
			i: INTEGER
		do
			current_file.put_line ("int GE_main(int argc, EIF_NATIVE_CHAR** argv)")
			current_file.put_character ('{')
			current_file.put_new_line
			l_root_type := current_dynamic_system.root_type
			l_root_creation_procedure := current_dynamic_system.root_creation_procedure
			if l_root_type /= Void and l_root_creation_procedure /= Void then
				indent
				print_indentation
				print_type_declaration (l_root_type, current_file)
				current_file.put_character (' ')
				l_temp := temp_variable
				print_temp_name (l_temp, current_file)
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_string (c_ge_context)
				current_file.put_character (' ')
				current_file.put_string (c_tc)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_string (c_ge_default_context)
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_string (c_ge_context)
				current_file.put_character ('*')
				current_file.put_character (' ')
				current_file.put_string (c_ac)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_string (c_tc_address)
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_string ("GE_type_info_count = ");
				current_file.put_integer (dynamic_types.count)
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_line ("GE_argc = argc;")
				print_indentation
				current_file.put_line ("GE_argv = argv;")
				print_indentation
				current_file.put_line ("GE_main_context = ac;")
				print_indentation
				current_file.put_string ("GE_system_name = ")
				print_escaped_string (system_name)
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_string ("GE_root_class_name = ")
				print_escaped_string (l_root_type.base_class.upper_name)
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_line ("GE_init_gc();")
					-- Exception handling.
				print_indentation
				current_file.put_string (c_ge_new_exception_manager)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('&')
				current_file.put_string (c_ge_new)
				current_file.put_integer (current_dynamic_system.ise_exception_manager_type.id)
				current_file.put_character (';')
				current_file.put_new_line
				if attached current_dynamic_system.ise_exception_manager_init_exception_manager_feature as l_ise_exception_manager_init_exception_manager_feature then
					print_indentation
					current_file.put_string (c_ge_init_exception_manager)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('&')
					print_routine_name (l_ise_exception_manager_init_exception_manager_feature, current_dynamic_system.ise_exception_manager_type, current_file)
					current_file.put_character (';')
					current_file.put_new_line
				end
				if attached current_dynamic_system.ise_exception_manager_last_exception_feature as l_ise_exception_manager_last_exception_feature then
					print_indentation
					current_file.put_string (c_ge_last_exception)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('&')
					print_routine_name (l_ise_exception_manager_last_exception_feature, current_dynamic_system.ise_exception_manager_type, current_file)
					current_file.put_character (';')
					current_file.put_new_line
				end
				if attached current_dynamic_system.ise_exception_manager_once_raise_feature as l_ise_exception_manager_once_raise_feature then
					print_indentation
					current_file.put_string (c_ge_once_raise)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('&')
					print_routine_name (l_ise_exception_manager_once_raise_feature, current_dynamic_system.ise_exception_manager_type, current_file)
					current_file.put_character (';')
					current_file.put_new_line
				end
				if attached current_dynamic_system.ise_exception_manager_set_exception_data_feature as l_ise_exception_manager_set_exception_data_feature then
					print_indentation
					current_file.put_string (c_ge_set_exception_data)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('&')
					print_routine_name (l_ise_exception_manager_set_exception_data_feature, current_dynamic_system.ise_exception_manager_type, current_file)
					current_file.put_character (';')
					current_file.put_new_line
				end
				print_indentation
				current_file.put_string ("GE_init_onces")
				current_file.put_character ('(')
				from i := 1 until i > once_kind_count loop
					if i /= 1 then
						current_file.put_character (',')
						current_file.put_character (' ')
					end
					current_file.put_integer (once_per_process_counts.item (i))
					i := i + 1
				end
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				if multithreaded_mode then
					print_indentation
					current_file.put_string (c_ge_once_per_object_data_mutex)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_ge_mutex_create)
					current_file.put_character ('(')
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					print_indentation
					current_file.put_string (c_ge_thread_onces_set_counts)
					current_file.put_character ('(')
					from i := 1 until i > once_kind_count loop
						if i /= 1 then
							current_file.put_character (',')
							current_file.put_character (' ')
						end
						current_file.put_integer (once_per_thread_counts.item (i))
						i := i + 1
					end
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
						-- We need to call 'GE_init_thread' before 'GE_init_exception' because
						-- 'GE_init_exception' will create once-per-thread objects.
					print_indentation
					current_file.put_line ("GE_init_thread(ac);")
				end
				print_indentation
				current_file.put_line ("GE_init_exception(ac);")
				print_indentation
				current_file.put_line ("GE_init_console();")
				print_indentation
				current_file.put_line ("GE_init_identified();")
				print_indentation
				current_file.put_line ("GE_init_const();")

				if trace_mode then
						-- We need to make sure that a DOS console is available (for
						-- Windows application) when in trace mode, because information
						-- will be displayed to the console.
					current_file.put_string (c_ifdef)
					current_file.put_character (' ')
					current_file.put_line (c_eif_trace)
					print_show_console_call
					current_file.put_line (c_endif)
				end
					-- Create root object.
				print_indentation
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				create l_root_call.make (l_root_creation_procedure.static_feature.name, Void)
				l_root_call.name.set_seed (l_root_creation_procedure.static_feature.first_seed)
				create l_root_creation.make (l_root_type.base_type, l_root_call)
					-- Prepare dynamic type sets of the root creation expression.
				extra_dynamic_type_sets.force_last (l_root_type)
				l_root_creation.set_index (current_dynamic_type_sets.count + extra_dynamic_type_sets.count)
				print_creation_expression (l_root_creation)
					-- Clean up.
				extra_dynamic_type_sets.remove_last
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				current_file.put_character ('0')
				current_file.put_character (';')
				current_file.put_new_line
				dedent
			end
			current_file.put_character ('}')
			current_file.put_new_line
		end

	print_new_string_8_function
			-- Print 'GE_new_str8' function to `current_file' and its signature to `header_file'.
			-- 'GE_new_str8' create an empty string which is then used to create manifest strings
			-- of type "STRING_8".
		local
			l_string_type: ET_DYNAMIC_PRIMARY_TYPE
			l_area_type: ET_DYNAMIC_PRIMARY_TYPE
			l_count_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp: ET_IDENTIFIER
			l_queries: ET_DYNAMIC_FEATURE_LIST
			old_file: KI_TEXT_OUTPUT_STREAM
		do
			l_string_type := current_dynamic_system.string_8_type
			l_area_type := current_dynamic_system.special_character_8_type
			if l_string_type.attribute_count < 2 then
					-- Internal error: class "STRING_8" should have at least the
					-- features 'area' and 'count' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached l_string_type.queries.item (2).result_type_set as l_dynamic_type_set then
					-- Error in feature 'count', already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_count_type := l_dynamic_type_set.static_type.primary_type
					-- Print signature to `header_file' and `current_file'.
				old_file := current_file
				current_file := current_function_header_buffer
				header_file.put_string (c_extern)
				header_file.put_character (' ')
				print_type_declaration (l_string_type, header_file)
				print_type_declaration (l_string_type, current_file)
				header_file.put_character (' ')
				current_file.put_character (' ')
				header_file.put_string (c_ge_new_str8)
				current_file.put_string (c_ge_new_str8)
				header_file.put_character ('(')
				current_file.put_character ('(')
				print_type_declaration (l_count_type, header_file)
				print_type_declaration (l_count_type, current_file)
				header_file.put_character (' ')
				current_file.put_character (' ')
				header_file.put_character ('c')
				current_file.put_character ('c')
				header_file.put_character (')')
				current_file.put_character (')')
				header_file.put_character (';')
				header_file.put_new_line
				current_file.put_new_line
					-- Print body to `current_file'.
				current_file.put_character ('{')
				current_file.put_new_line
				indent
				print_indentation
				print_type_declaration (l_string_type, current_file)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				current_file := current_function_body_buffer
				if l_string_type.is_alive then
					l_temp := new_temp_variable (l_area_type)
						-- Create 'area'.
					print_indentation
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_ge_new)
					current_file.put_integer (l_area_type.id)
					current_file.put_character ('(')
					current_file.put_character ('c')
					current_file.put_character ('+')
					current_file.put_character ('1')
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_string (c_eif_false)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
						-- Default initialization of header part of 'area'.
					print_indentation
					current_file.put_character ('*')
					print_type_cast (l_area_type, current_file)
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_default_name (l_area_type, current_file)
					current_file.put_character (';')
					current_file.put_new_line
						-- Set 'capacity' of 'area'.
					print_indentation
					print_attribute_special_capacity_access (l_temp, l_area_type, False)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
					current_file.put_character ('c')
					current_file.put_character ('+')
					current_file.put_character ('1')
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
						-- Set 'count' of 'area'.
					print_indentation
					print_attribute_special_count_access (l_temp, l_area_type, False)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('0')
					current_file.put_character (';')
					current_file.put_new_line
						-- Create string object.
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_ge_new)
					current_file.put_integer (l_string_type.id)
					current_file.put_character ('(')
					current_file.put_string (c_eif_true)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					if l_string_type.attribute_count < 2 then
							-- Internal error: the "STRING_8" type should have at least
							-- the attributes 'area' and 'count' as first features.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						l_queries := l_string_type.queries
							-- Set 'area'.
						print_indentation
						print_attribute_access (l_queries.first, tokens.result_keyword, l_string_type, False)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						print_temp_name (l_temp, current_file)
						current_file.put_character (';')
						current_file.put_new_line
					end
					mark_temp_variable_free (l_temp)
				else
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_void)
					current_file.put_character (';')
					current_file.put_new_line
				end
					-- Return the string.
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				dedent
				current_file.put_character ('}')
				current_file.put_new_line
				current_file := old_file
				reset_temp_variables
			end
		end

	print_new_string_32_function
			-- Print 'GE_new_str32' function to `current_file' and its signature to `header_file'.
			-- 'GE_new_str32' create an empty string which is then used to create manifest strings
			-- of type "STRING_32".
		local
			l_string_type: ET_DYNAMIC_PRIMARY_TYPE
			l_area_type: ET_DYNAMIC_PRIMARY_TYPE
			l_count_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp: ET_IDENTIFIER
			l_queries: ET_DYNAMIC_FEATURE_LIST
			old_file: KI_TEXT_OUTPUT_STREAM
		do
			l_string_type := current_dynamic_system.string_32_type
			l_area_type := current_dynamic_system.special_character_32_type
			if l_string_type.attribute_count < 2 then
					-- Internal error: class "STRING_32" should have at least the
					-- features 'area' and 'count' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached l_string_type.queries.item (2).result_type_set as l_dynamic_type_set then
					-- Error in feature 'count', already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_count_type := l_dynamic_type_set.static_type.primary_type
					-- Print signature to `header_file' and `current_file'.
				old_file := current_file
				current_file := current_function_header_buffer
				header_file.put_string (c_extern)
				header_file.put_character (' ')
				print_type_declaration (l_string_type, header_file)
				print_type_declaration (l_string_type, current_file)
				header_file.put_character (' ')
				current_file.put_character (' ')
				header_file.put_string (c_ge_new_str32)
				current_file.put_string (c_ge_new_str32)
				header_file.put_character ('(')
				current_file.put_character ('(')
				print_type_declaration (l_count_type, header_file)
				print_type_declaration (l_count_type, current_file)
				header_file.put_character (' ')
				current_file.put_character (' ')
				header_file.put_character ('c')
				current_file.put_character ('c')
				header_file.put_character (')')
				current_file.put_character (')')
				header_file.put_character (';')
				header_file.put_new_line
				current_file.put_new_line
					-- Print body to `current_file'.
				current_file.put_character ('{')
				current_file.put_new_line
				indent
				print_indentation
				print_type_declaration (l_string_type, current_file)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				current_file := current_function_body_buffer
				if l_string_type.is_alive then
					l_temp := new_temp_variable (l_area_type)
						-- Create 'area'.
					print_indentation
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_ge_new)
					current_file.put_integer (l_area_type.id)
					current_file.put_character ('(')
					current_file.put_character ('c')
					current_file.put_character ('+')
					current_file.put_character ('1')
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_string (c_eif_false)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
						-- Default initialization of header part of 'area'.
					print_indentation
					current_file.put_character ('*')
					print_type_cast (l_area_type, current_file)
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_default_name (l_area_type, current_file)
					current_file.put_character (';')
					current_file.put_new_line
						-- Set 'capacity' of 'area'.
					print_indentation
					print_attribute_special_capacity_access (l_temp, l_area_type, False)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
					current_file.put_character ('c')
					current_file.put_character ('+')
					current_file.put_character ('1')
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
						-- Set 'count' of 'area'.
					print_indentation
					print_attribute_special_count_access (l_temp, l_area_type, False)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('0')
					current_file.put_character (';')
					current_file.put_new_line
						-- Create string object.
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_ge_new)
					current_file.put_integer (l_string_type.id)
					current_file.put_character ('(')
					current_file.put_string (c_eif_true)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					if l_string_type.attribute_count < 2 then
							-- Internal error: the "STRING_32" type should have at least
							-- the attributes 'area' and 'count' as first features.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						l_queries := l_string_type.queries
							-- Set 'area'.
						print_indentation
						print_attribute_access (l_queries.first, tokens.result_keyword, l_string_type, False)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						print_temp_name (l_temp, current_file)
						current_file.put_character (';')
						current_file.put_new_line
					end
					mark_temp_variable_free (l_temp)
				else
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_void)
					current_file.put_character (';')
					current_file.put_new_line
				end
					-- Return the string.
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				dedent
				current_file.put_character ('}')
				current_file.put_new_line
				current_file := old_file
				reset_temp_variables
			end
		end

	print_new_immutable_string_8_function
			-- Print 'GE_new_istr8' function to `current_file' and its signature to `header_file'.
			-- 'GE_new_istr8' creates an empty string which is then used to create manifest strings
			-- of type "IMMUTABLE_STRING_8".
		local
			l_string_type: ET_DYNAMIC_PRIMARY_TYPE
			l_area_type: ET_DYNAMIC_PRIMARY_TYPE
			l_count_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp: ET_IDENTIFIER
			l_queries: ET_DYNAMIC_FEATURE_LIST
			old_file: KI_TEXT_OUTPUT_STREAM
		do
			l_string_type := current_dynamic_system.immutable_string_8_type
			l_area_type := current_dynamic_system.special_character_8_type
			if l_string_type.attribute_count < 2 then
					-- Internal error: class "IMMUTABLE_STRING_8" should have at least the
					-- features 'area' and 'count' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached l_string_type.queries.item (2).result_type_set as l_dynamic_type_set then
					-- Error in feature 'count', already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_count_type := l_dynamic_type_set.static_type.primary_type
					-- Print signature to `header_file' and `current_file'.
				old_file := current_file
				current_file := current_function_header_buffer
				header_file.put_string (c_extern)
				header_file.put_character (' ')
				print_type_declaration (l_string_type, header_file)
				print_type_declaration (l_string_type, current_file)
				header_file.put_character (' ')
				current_file.put_character (' ')
				header_file.put_string (c_ge_new_istr8)
				current_file.put_string (c_ge_new_istr8)
				header_file.put_character ('(')
				current_file.put_character ('(')
				print_type_declaration (l_count_type, header_file)
				print_type_declaration (l_count_type, current_file)
				header_file.put_character (' ')
				current_file.put_character (' ')
				header_file.put_character ('c')
				current_file.put_character ('c')
				header_file.put_character (')')
				current_file.put_character (')')
				header_file.put_character (';')
				header_file.put_new_line
				current_file.put_new_line
					-- Print body to `current_file'.
				current_file.put_character ('{')
				current_file.put_new_line
				indent
				print_indentation
				print_type_declaration (l_string_type, current_file)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				current_file := current_function_body_buffer
				if l_string_type.is_alive then
					l_temp := new_temp_variable (l_area_type)
						-- Create 'area'.
					print_indentation
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_ge_new)
					current_file.put_integer (l_area_type.id)
					current_file.put_character ('(')
					current_file.put_character ('c')
					current_file.put_character ('+')
					current_file.put_character ('1')
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_string (c_eif_false)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
						-- Default initialization of header part of 'area'.
					print_indentation
					current_file.put_character ('*')
					print_type_cast (l_area_type, current_file)
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_default_name (l_area_type, current_file)
					current_file.put_character (';')
					current_file.put_new_line
						-- Set 'capacity' of 'area'.
					print_indentation
					print_attribute_special_capacity_access (l_temp, l_area_type, False)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
					current_file.put_character ('c')
					current_file.put_character ('+')
					current_file.put_character ('1')
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
						-- Set 'count' of 'area'.
					print_indentation
					print_attribute_special_count_access (l_temp, l_area_type, False)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('0')
					current_file.put_character (';')
					current_file.put_new_line
						-- Create string object.
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_ge_new)
					current_file.put_integer (l_string_type.id)
					current_file.put_character ('(')
					current_file.put_string (c_eif_true)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					if l_string_type.attribute_count < 2 then
							-- Internal error: the "IMMUTABLE_STRING_8" type should have at least
							-- the attributes 'area' and 'count' as first features.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						l_queries := l_string_type.queries
							-- Set 'area'.
						print_indentation
						print_attribute_access (l_queries.first, tokens.result_keyword, l_string_type, False)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						print_temp_name (l_temp, current_file)
						current_file.put_character (';')
						current_file.put_new_line
					end
					mark_temp_variable_free (l_temp)
				else
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_void)
					current_file.put_character (';')
					current_file.put_new_line
				end
					-- Return the string.
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				dedent
				current_file.put_character ('}')
				current_file.put_new_line
				current_file := old_file
				reset_temp_variables
			end
		end

	print_new_immutable_string_32_function
			-- Print 'GE_new_istr32' function to `current_file' and its signature to `header_file'.
			-- 'GE_new_istr32' creates an empty string which is then used to create manifest strings
			-- of type "IMMUTABLE_STRING_32".
		local
			l_string_type: ET_DYNAMIC_PRIMARY_TYPE
			l_area_type: ET_DYNAMIC_PRIMARY_TYPE
			l_count_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp: ET_IDENTIFIER
			l_queries: ET_DYNAMIC_FEATURE_LIST
			old_file: KI_TEXT_OUTPUT_STREAM
		do
			l_string_type := current_dynamic_system.immutable_string_32_type
			l_area_type := current_dynamic_system.special_character_32_type
			if l_string_type.attribute_count < 2 then
					-- Internal error: class "IMMUTABLE_STRING_32" should have at least the
					-- features 'area' and 'count' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached l_string_type.queries.item (2).result_type_set as l_dynamic_type_set then
					-- Error in feature 'count', already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_count_type := l_dynamic_type_set.static_type.primary_type
					-- Print signature to `header_file' and `current_file'.
				old_file := current_file
				current_file := current_function_header_buffer
				header_file.put_string (c_extern)
				header_file.put_character (' ')
				print_type_declaration (l_string_type, header_file)
				print_type_declaration (l_string_type, current_file)
				header_file.put_character (' ')
				current_file.put_character (' ')
				header_file.put_string (c_ge_new_istr32)
				current_file.put_string (c_ge_new_istr32)
				header_file.put_character ('(')
				current_file.put_character ('(')
				print_type_declaration (l_count_type, header_file)
				print_type_declaration (l_count_type, current_file)
				header_file.put_character (' ')
				current_file.put_character (' ')
				header_file.put_character ('c')
				current_file.put_character ('c')
				header_file.put_character (')')
				current_file.put_character (')')
				header_file.put_character (';')
				header_file.put_new_line
				current_file.put_new_line
					-- Print body to `current_file'.
				current_file.put_character ('{')
				current_file.put_new_line
				indent
				print_indentation
				print_type_declaration (l_string_type, current_file)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				current_file := current_function_body_buffer
				if l_string_type.is_alive then
					l_temp := new_temp_variable (l_area_type)
						-- Create 'area'.
					print_indentation
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_ge_new)
					current_file.put_integer (l_area_type.id)
					current_file.put_character ('(')
					current_file.put_character ('c')
					current_file.put_character ('+')
					current_file.put_character ('1')
					current_file.put_character (',')
					current_file.put_character (' ')
					current_file.put_string (c_eif_false)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
						-- Default initialization of header part of 'area'.
					print_indentation
					current_file.put_character ('*')
					print_type_cast (l_area_type, current_file)
					print_temp_name (l_temp, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					print_default_name (l_area_type, current_file)
					current_file.put_character (';')
					current_file.put_new_line
						-- Set 'capacity' of 'area'.
					print_indentation
					print_attribute_special_capacity_access (l_temp, l_area_type, False)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('(')
					current_file.put_character ('c')
					current_file.put_character ('+')
					current_file.put_character ('1')
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
						-- Set 'count' of 'area'.
					print_indentation
					print_attribute_special_count_access (l_temp, l_area_type, False)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('0')
					current_file.put_character (';')
					current_file.put_new_line
						-- Create string object.
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_ge_new)
					current_file.put_integer (l_string_type.id)
					current_file.put_character ('(')
					current_file.put_string (c_eif_true)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					if l_string_type.attribute_count < 2 then
							-- Internal error: the "IMMUTABLE_STRING_32" type should have at least
							-- the attributes 'area' and 'count' as first features.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						l_queries := l_string_type.queries
							-- Set 'area'.
						print_indentation
						print_attribute_access (l_queries.first, tokens.result_keyword, l_string_type, False)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						print_temp_name (l_temp, current_file)
						current_file.put_character (';')
						current_file.put_new_line
					end
					mark_temp_variable_free (l_temp)
				else
					print_indentation
					print_result_name (current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_string (c_eif_void)
					current_file.put_character (';')
					current_file.put_new_line
				end
					-- Return the string.
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				dedent
				current_file.put_character ('}')
				current_file.put_new_line
				current_file := old_file
				reset_temp_variables
			end
		end

	print_manifest_array_function (an_array_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print 'GE_ma' function to `current_file' and its signature to `header_file'.
			-- 'GE_ma<type-id>' is used to create manifest arrays of type 'type-id'.
		require
			an_array_type_not_void: an_array_type /= Void
		local
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_integer_type: ET_DYNAMIC_PRIMARY_TYPE
			l_item_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp: ET_IDENTIFIER
		do
			l_queries := an_array_type.queries
			if an_array_type.attribute_count < 3 then
					-- Internal error: class "ARRAY" should have at least the
					-- features 'area', 'lower' and 'upper' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached l_queries.first.result_type_set as l_dynamic_type_set then
					-- Error in feature 'area', already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached {ET_DYNAMIC_SPECIAL_TYPE} l_dynamic_type_set.static_type.primary_type as l_special_type then
				-- Internal error: it has already been checked in ET_DYNAMIC_SYSTEM.compile_kernel
					-- that the attribute `area' is of SPECIAL type.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached l_queries.item (3).result_type_set as l_integer_dynamic_type_set then
					-- Error in feature 'upper', already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_integer_type := l_integer_dynamic_type_set.static_type.primary_type
				l_item_type := l_special_type.item_type_set.static_type.primary_type
					-- Print signature to `header_file' and `current_file'.
				header_file.put_string (c_extern)
				header_file.put_character (' ')
				print_type_declaration (an_array_type, header_file)
				print_type_declaration (an_array_type, current_file)
				header_file.put_character (' ')
				current_file.put_character (' ')
				header_file.put_string (c_ge_ma)
				current_file.put_string (c_ge_ma)
				header_file.put_integer (an_array_type.id)
				current_file.put_integer (an_array_type.id)
					-- Use varargs rather than inlining the code, this
					-- makes the C compilation with the -O2 faster and
					-- the resulting application is not slower.
				header_file.put_character ('(')
				print_type_declaration (l_integer_type, header_file)
				header_file.put_string (" c, ")
				print_type_declaration (l_integer_type, header_file)
				header_file.put_string (" n, ...)")
				current_file.put_character ('(')
				print_type_declaration (l_integer_type, current_file)
				current_file.put_string (" c, ")
				print_type_declaration (l_integer_type, current_file)
				current_file.put_string (" n, ...)")
				header_file.put_character (';')
				header_file.put_new_line
				current_file.put_new_line
					-- Print body to `current_file'.
				current_file.put_character ('{')
				current_file.put_new_line
				indent
				print_indentation
				print_type_declaration (an_array_type, current_file)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				l_temp := temp_variable
				print_indentation
				print_type_declaration (l_special_type, current_file)
				current_file.put_character (' ')
				print_temp_name (l_temp, current_file)
				current_file.put_character (';')
				current_file.put_new_line
					-- Create 'area'.
				print_indentation
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_string (c_ge_new)
				current_file.put_integer (l_special_type.id)
				current_file.put_character ('(')
				current_file.put_character ('c')
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_string (c_eif_false)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
					-- Default initialization of header part of 'area'.
				print_indentation
				current_file.put_character ('*')
				print_type_cast (l_special_type, current_file)
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_default_name (l_special_type, current_file)
				current_file.put_character (';')
				current_file.put_new_line
					-- Set 'capacity' of 'area'.
				print_indentation
				print_attribute_special_capacity_access (l_temp, l_special_type, False)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('c')
				current_file.put_character (';')
				current_file.put_new_line
					-- Set 'count' of 'area'.
				print_indentation
				print_attribute_special_count_access (l_temp, l_special_type, False)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('c')
				current_file.put_character (';')
				current_file.put_new_line
					-- Copy items to 'area'.
				print_indentation
				current_file.put_line ("if (n!=0) {")
				indent
				print_indentation
				current_file.put_line ("va_list v;")
				print_indentation
				print_type_declaration (l_integer_type, current_file)
				current_file.put_line (" j = n;")
				print_indentation
				print_type_declaration (l_item_type, current_file)
				current_file.put_character (' ')
				current_file.put_character ('*')
				current_file.put_character ('i')
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attribute_special_item_access (l_temp, l_special_type, False)
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_line ("va_start(v, n);")
				print_indentation
				current_file.put_line ("while (j--) {")
				indent
				print_indentation
				if
					l_item_type = current_dynamic_system.boolean_type or
					l_item_type = current_dynamic_system.character_8_type or
					l_item_type = current_dynamic_system.integer_8_type or
					l_item_type = current_dynamic_system.natural_8_type or
					l_item_type = current_dynamic_system.integer_16_type or
					l_item_type = current_dynamic_system.natural_16_type
				then
						-- ISO C 99 says that through "..." the types are promoted to
						-- 'int', and that promotion to 'int' leaves the type unchanged
						-- if all values cannot be represented with an 'int' or
						-- 'unsigned int'.
					current_file.put_string ("*(i++) = ")
					print_type_cast (l_item_type, current_file)
					current_file.put_string ("va_arg(v, int")
				elseif
					l_item_type = current_dynamic_system.real_32_type
				then
						-- ISO C 99 says that 'float' is promoted to 'double' when
						-- passed as argument of a function.
					current_file.put_string ("*(i++) = ")
					print_type_cast (l_item_type, current_file)
					current_file.put_string ("va_arg(v, double")
				else
					current_file.put_string ("*(i++) = va_arg(v, ")
					print_type_declaration (l_item_type, current_file)
				end
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				dedent
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
				print_indentation
				current_file.put_line ("va_end(v);")
				dedent
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
					-- Create array object.
				print_indentation
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_string (c_ge_new)
				current_file.put_integer (an_array_type.id)
				current_file.put_character ('(')
				current_file.put_string (c_eif_true)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
					-- Set 'area'.
				print_indentation
				print_attribute_access (l_queries.first, tokens.result_keyword, an_array_type, False)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_temp_name (l_temp, current_file)
				current_file.put_character (';')
				current_file.put_new_line
					-- Set 'lower'.
				print_indentation
				print_attribute_access (l_queries.item (2), tokens.result_keyword, an_array_type, False)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('1')
				current_file.put_character (';')
				current_file.put_new_line
					-- Set 'upper'.
				print_indentation
				print_attribute_access (l_queries.item (3), tokens.result_keyword, an_array_type, False)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('c')
				current_file.put_character (';')
				current_file.put_new_line
					-- Return the array.
				print_indentation
				current_file.put_string (c_return)
				current_file.put_character (' ')
				print_result_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
				dedent
				current_file.put_character ('}')
				current_file.put_new_line
			end
		end

	print_big_manifest_array_function (an_array_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print 'GE_bma' function to `current_file' and its signature to `header_file'.
			-- 'GE_bma<type-id>' is used to create big manifest arrays of type 'type-id'.
			-- "Big" means more elements than the number of function arguments that the
			-- underlying C compiler can support.
		require
			an_array_type_not_void: an_array_type /= Void
		local
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_integer_type: ET_DYNAMIC_PRIMARY_TYPE
			l_item_type: ET_DYNAMIC_PRIMARY_TYPE
			l_temp: ET_IDENTIFIER
		do
			l_queries := an_array_type.queries
			if an_array_type.attribute_count < 3 then
					-- Internal error: class "ARRAY" should have at least the
					-- features 'area', 'lower' and 'upper' as first features.
					-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached l_queries.first.result_type_set as l_dynamic_type_set then
					-- Error in feature 'area', already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached {ET_DYNAMIC_SPECIAL_TYPE} l_dynamic_type_set.static_type.primary_type as l_special_type then
					-- Internal error: it has already been checked in ET_DYNAMIC_SYSTEM.compile_kernel
					-- that the attribute `area' is of SPECIAL type.
				set_fatal_error
				error_handler.report_giaaa_error
			elseif not attached l_queries.item (3).result_type_set as l_integer_dynamic_type_set then
					-- Error in feature 'upper', already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
				set_fatal_error
				error_handler.report_giaaa_error
			else
				l_integer_type := l_integer_dynamic_type_set.static_type.primary_type
				l_item_type := l_special_type.item_type_set.static_type.primary_type
					-- Print signature to `header_file' and `current_file'.
				header_file.put_string (c_extern)
				header_file.put_character (' ')
				header_file.put_string (c_void)
				current_file.put_string (c_void)
				header_file.put_character (' ')
				current_file.put_character (' ')
				header_file.put_string (c_ge_bma)
				current_file.put_string (c_ge_bma)
				header_file.put_integer (an_array_type.id)
				current_file.put_integer (an_array_type.id)
					-- Use varargs rather than inlining the code, this
					-- makes the C compilation with the -O2 faster and
					-- the resulting application is not slower.
				header_file.put_character ('(')
				print_type_declaration (an_array_type, header_file)
				header_file.put_string (" C, ")
				print_type_declaration (l_integer_type, header_file)
				header_file.put_string (" s, ")
				print_type_declaration (l_integer_type, header_file)
				header_file.put_string (" n, ...)")
				current_file.put_character ('(')
				print_type_declaration (an_array_type, current_file)
				current_file.put_string (" C, ")
				print_type_declaration (l_integer_type, current_file)
				current_file.put_string (" s, ")
				print_type_declaration (l_integer_type, current_file)
				current_file.put_string (" n, ...)")
				header_file.put_character (';')
				header_file.put_new_line
				current_file.put_new_line
					-- Print body to `current_file'.
				current_file.put_character ('{')
				current_file.put_new_line
				indent
					-- Copy items to 'area'.
				print_indentation
				current_file.put_line ("if (n!=0) {")
				indent
				print_indentation
				current_file.put_line ("va_list v;")
				print_indentation
				print_type_declaration (l_integer_type, current_file)
				current_file.put_line (" j = n;")
				l_temp := temp_variable
				print_indentation
				print_type_declaration (l_special_type, current_file)
				current_file.put_character (' ')
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attribute_access (l_queries.first, tokens.current_keyword, an_array_type, False)
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				print_type_declaration (l_item_type, current_file)
				current_file.put_character (' ')
				current_file.put_character ('*')
				current_file.put_character ('i')
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_attribute_special_item_access (l_temp, l_special_type, False)
				current_file.put_character (' ')
				current_file.put_character ('+')
				current_file.put_character (' ')
				current_file.put_character ('s')
				current_file.put_character (';')
				current_file.put_new_line
				print_indentation
				current_file.put_line ("va_start(v, n);")
				print_indentation
				current_file.put_line ("while (j--) {")
				indent
				print_indentation
				if
					l_item_type = current_dynamic_system.boolean_type or
					l_item_type = current_dynamic_system.character_8_type or
					l_item_type = current_dynamic_system.integer_8_type or
					l_item_type = current_dynamic_system.natural_8_type or
					l_item_type = current_dynamic_system.integer_16_type or
					l_item_type = current_dynamic_system.natural_16_type
				then
						-- ISO C 99 says that through "..." the types are promoted to
						-- 'int', and that promotion to 'int' leaves the type unchanged
						-- if all values cannot be represented with an 'int' or
						-- 'unsigned int'.
					current_file.put_string ("*(i++) = ")
					print_type_cast (l_item_type, current_file)
					current_file.put_string ("va_arg(v, int")
				elseif
					l_item_type = current_dynamic_system.real_32_type
				then
						-- ISO C 99 says that 'float' is promoted to 'double' when
						-- passed as argument of a function.
					current_file.put_string ("*(i++) = ")
					print_type_cast (l_item_type, current_file)
					current_file.put_string ("va_arg(v, double")
				else
					current_file.put_string ("*(i++) = va_arg(v, ")
					print_type_declaration (l_item_type, current_file)
				end
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				dedent
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
				print_indentation
				current_file.put_line ("va_end(v);")
				dedent
				print_indentation
				current_file.put_character ('}')
				current_file.put_new_line
				dedent
				current_file.put_character ('}')
				current_file.put_new_line
			end
		end

	print_manifest_tuple_function (a_tuple_type: ET_DYNAMIC_TUPLE_TYPE)
			-- Print 'GE_mt' function to `current_file' and its signature to `header_file'.
			-- 'GE_mt<type-id>' is used to create manifest tuples of type 'type-id'.
		require
			a_tuple_type_not_void: a_tuple_type /= Void
		local
			l_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_item_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
		do
			l_item_type_sets := a_tuple_type.item_type_sets
				-- Print signature to `header_file' and `current_file'.
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			print_type_declaration (a_tuple_type, header_file)
			print_type_declaration (a_tuple_type, current_file)
			header_file.put_character (' ')
			current_file.put_character (' ')
			header_file.put_string (c_ge_mt)
			current_file.put_string (c_ge_mt)
			header_file.put_integer (a_tuple_type.id)
			current_file.put_integer (a_tuple_type.id)
			header_file.put_character ('(')
			current_file.put_character ('(')
			nb := l_item_type_sets.count
			if nb = 0 then
				header_file.put_string (c_void)
				current_file.put_string (c_void)
			else
				from i := 1 until i > nb loop
					if i /= 1 then
						header_file.put_character (',')
						header_file.put_character (' ')
						current_file.put_character (',')
						current_file.put_character (' ')
					end
					l_item_type := l_item_type_sets.item (i).static_type.primary_type
					print_type_declaration (l_item_type, header_file)
					print_type_declaration (l_item_type, current_file)
					header_file.put_character (' ')
					header_file.put_character ('a')
					header_file.put_integer (i)
					current_file.put_character (' ')
					current_file.put_character ('a')
					current_file.put_integer (i)
					i := i + 1
				end
			end
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
				-- Print body to `current_file'.
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_indentation
			print_type_declaration (a_tuple_type, current_file)
			current_file.put_character (' ')
			print_result_name (current_file)
			current_file.put_character (';')
			current_file.put_new_line
				-- Create tuple object.
			print_indentation
			print_result_name (current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_string (c_ge_new)
			current_file.put_integer (a_tuple_type.id)
			current_file.put_character ('(')
			current_file.put_string (c_eif_true)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
				-- Set fields.
			from i := 1 until i > nb loop
				print_indentation
				print_attribute_tuple_item_access (i, tokens.result_keyword, a_tuple_type, False)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('a')
				current_file.put_integer (i)
				current_file.put_character (';')
				current_file.put_new_line
				i := i + 1
			end
				-- Return the tuple.
			print_indentation
			current_file.put_string (c_return)
			current_file.put_character (' ')
			print_result_name (current_file)
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
		end

	print_boxed_function (a_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print 'GE_boxed' function to `current_file' and its signature to `header_file'.
			-- 'GE_boxed<type-id>' is used to create boxed objects of type `a_type' (with id <type_id>).
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
		require
			a_type_not_void: a_type /= Void
			a_type_expanded: a_type.is_expanded
		do
				-- Print signature to `header_file' and `current_file'.
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			print_boxed_type_declaration (a_type, header_file)
			print_boxed_type_declaration (a_type, current_file)
			header_file.put_character (' ')
			current_file.put_character (' ')
			header_file.put_string (c_ge_boxed)
			current_file.put_string (c_ge_boxed)
			header_file.put_integer (a_type.id)
			current_file.put_integer (a_type.id)
			header_file.put_character ('(')
			current_file.put_character ('(')
			print_type_declaration (a_type, header_file)
			print_type_declaration (a_type, current_file)
			header_file.put_character (' ')
			header_file.put_character ('a')
			header_file.put_character ('1')
			current_file.put_character (' ')
			current_file.put_character ('a')
			current_file.put_character ('1')
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
				-- Print body to `current_file'.
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_indentation
			print_boxed_type_declaration (a_type, current_file)
			current_file.put_character (' ')
			print_result_name (current_file)
			print_semicolon_newline
				-- Create boxed object.
			print_indentation_assign_to_result
			current_file.put_character ('(')
			print_boxed_type_declaration (a_type, current_file)
			current_file.put_character (')')
			current_file.put_string (c_ge_malloc)
			current_file.put_character ('(')
			current_file.put_string (c_sizeof)
			current_file.put_character ('(')
			print_boxed_type_name (a_type, current_file)
			current_file.put_character (')')
			current_file.put_character (')')
			print_semicolon_newline
				-- Set type id.
			print_indentation
			print_boxed_attribute_type_id_access (tokens.result_keyword, a_type, False)
			print_assign_to
			current_file.put_integer (a_type.id)
			print_semicolon_newline
				-- Set flags.
			print_indentation
			print_boxed_attribute_flags_access (tokens.result_keyword, a_type, False)
			print_assign_to
			current_file.put_character ('0')
			print_semicolon_newline
				-- Set pointer.
			print_indentation
			print_boxed_attribute_pointer_access (tokens.result_keyword, a_type, False)
			print_assign_to
			current_file.put_character ('&')
			current_file.put_character ('(')
			print_boxed_attribute_item_access (tokens.result_keyword, a_type, False)
			current_file.put_character (')')
			print_semicolon_newline
				-- Set item.
			print_indentation
			print_boxed_attribute_item_access (tokens.result_keyword, a_type, False)
			print_assign_to
			current_file.put_character ('a')
			current_file.put_character ('1')
			print_semicolon_newline
				-- Return the boxed object.
			print_indentation
			current_file.put_string (c_return)
			current_file.put_character (' ')
			print_result_name (current_file)
			print_semicolon_newline
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
		end

	print_boxed_pointer_function (a_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print 'GE_boxed_pointer' function to `current_file' and its signature to `header_file'.
			-- 'GE_boxed_pointer<type-id>' is used to create boxed objects of type `a_type' (with id <type_id>).
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
		require
			a_type_not_void: a_type /= Void
			a_type_expanded: a_type.is_expanded
		do
				-- Print signature to `header_file' and `current_file'.
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			print_boxed_type_declaration (a_type, header_file)
			print_boxed_type_declaration (a_type, current_file)
			header_file.put_character (' ')
			current_file.put_character (' ')
			header_file.put_string (c_ge_boxed_pointer)
			current_file.put_string (c_ge_boxed_pointer)
			header_file.put_integer (a_type.id)
			current_file.put_integer (a_type.id)
			header_file.put_character ('(')
			current_file.put_character ('(')
			print_type_declaration (a_type, header_file)
			header_file.put_character ('*')
			print_type_declaration (a_type, current_file)
			current_file.put_character ('*')
			header_file.put_character (' ')
			header_file.put_character ('a')
			header_file.put_character ('1')
			current_file.put_character (' ')
			current_file.put_character ('a')
			current_file.put_character ('1')
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
				-- Print body to `current_file'.
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_indentation
			print_boxed_type_declaration (a_type, current_file)
			current_file.put_character (' ')
			print_result_name (current_file)
			print_semicolon_newline
				-- Create boxed object.
			print_indentation_assign_to_result
			current_file.put_character ('(')
			print_boxed_type_declaration (a_type, current_file)
			current_file.put_character (')')
			current_file.put_string (c_ge_malloc)
			current_file.put_character ('(')
			current_file.put_string (c_sizeof)
			current_file.put_character ('(')
			print_boxed_type_name (a_type, current_file)
			current_file.put_character (')')
			current_file.put_character (')')
			print_semicolon_newline
				-- Set type id.
			print_indentation
			print_boxed_attribute_type_id_access (tokens.result_keyword, a_type, False)
			print_assign_to
			current_file.put_integer (a_type.id)
			print_semicolon_newline
				-- Set flags.
			print_indentation
			print_boxed_attribute_flags_access (tokens.result_keyword, a_type, False)
			print_assign_to
			current_file.put_character ('0')
			print_semicolon_newline
				-- Set pointer.
			print_indentation
			print_boxed_attribute_pointer_access (tokens.result_keyword, a_type, False)
			print_assign_to
			current_file.put_character ('a')
			current_file.put_character ('1')
			print_semicolon_newline
				-- Set item.
			print_indentation
			print_boxed_attribute_item_access (tokens.result_keyword, a_type, False)
			print_assign_to
			current_file.put_character ('*')
			current_file.put_character ('a')
			current_file.put_character ('1')
			print_semicolon_newline
				-- Return the boxed object.
			print_indentation
			current_file.put_string (c_return)
			current_file.put_character (' ')
			print_result_name (current_file)
			print_semicolon_newline
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
		end

	print_init_const_function
			-- Print 'GE_init_const' function to `current_file', and its signature to `header_file'.
			-- 'GE_init_const' is called to initialize the value of the non-expanded constant attributes
			-- and inline constants (such as once manifest strings).
		local
			l_feature: ET_FEATURE
			l_constant: ET_INLINE_CONSTANT
			l_manifest_constant: ET_CONSTANT
			l_constant_type: ET_DYNAMIC_PRIMARY_TYPE
			old_constant_index: INTEGER
			l_type: detachable ET_TYPE
			l_context: ET_TYPE_CONTEXT
			l_threshold: INTEGER
			i, j: INTEGER
		do
				-- We have a threshold here, because if the C function is too large,
				-- then the MSC C compiler takes a long time to compile it.
			l_threshold := 100
			j := 1
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			header_file.put_string (c_void)
			header_file.put_character (' ')
			header_file.put_string (c_ge_init_const)
			header_file.put_integer (j)
			header_file.put_character ('(')
			header_file.put_string (c_void)
			header_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_string (c_void)
			current_file.put_character (' ')
			current_file.put_string (c_ge_init_const)
			current_file.put_integer (j)
			current_file.put_character ('(')
			current_file.put_character (')')
			current_file.put_new_line
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			from constant_features.start until constant_features.after loop
				l_feature := constant_features.key_for_iteration
				if once_features.has (l_feature) then
					i := i + 1
					if i > l_threshold then
						dedent
						current_file.put_character ('}')
						current_file.put_new_line
						current_file.put_new_line
						flush_to_c_file
						j := j + 1
						i := 0
						header_file.put_string (c_extern)
						header_file.put_character (' ')
						header_file.put_string (c_void)
						header_file.put_character (' ')
						header_file.put_string (c_ge_init_const)
						header_file.put_integer (j)
						header_file.put_character ('(')
						header_file.put_string (c_void)
						header_file.put_character (')')
						header_file.put_character (';')
						header_file.put_new_line
						current_file.put_string (c_void)
						current_file.put_character (' ')
						current_file.put_string (c_ge_init_const)
						current_file.put_integer (j)
						current_file.put_character ('(')
						current_file.put_character (')')
						current_file.put_new_line
						current_file.put_character ('{')
						current_file.put_new_line
						indent
					end
					print_indentation
					print_once_status_name (l_feature, current_file)
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('%'')
					current_file.put_character ('\')
					current_file.put_character ('1')
					current_file.put_character ('%'')
					current_file.put_character (';')
					current_file.put_new_line
				end
				i := i + 1
				if i > l_threshold then
					dedent
					current_file.put_character ('}')
					current_file.put_new_line
					current_file.put_new_line
					flush_to_c_file
					j := j + 1
					i := 0
					header_file.put_string (c_extern)
					header_file.put_character (' ')
					header_file.put_string (c_void)
					header_file.put_character (' ')
					header_file.put_string (c_ge_init_const)
					header_file.put_integer (j)
					header_file.put_character ('(')
					header_file.put_string (c_void)
					header_file.put_character (')')
					header_file.put_character (';')
					header_file.put_new_line
					current_file.put_string (c_void)
					current_file.put_character (' ')
					current_file.put_string (c_ge_init_const)
					current_file.put_integer (j)
					current_file.put_character ('(')
					current_file.put_character (')')
					current_file.put_new_line
					current_file.put_character ('{')
					current_file.put_new_line
					indent
				end
				print_indentation
				print_once_value_name (l_feature, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('(')
				l_type := l_feature.type
				if l_type = Void then
						-- Internal feature: a constant attribute has a type.
					set_fatal_error
					error_handler.report_giaaa_error
				else
						-- The deanchored form of the type of the constant feature
						-- should not contain any formal generic parameter (in fact
						-- it should be one of BOOLEAN, CHARACTER possibly sized,
						-- INTEGER possibly sized, NATURAL possible sized, REAL possibly
						-- sized, STRING possibly sized, or 'TYPE [X]' where X is a
						-- stand-alone type), therefore it is OK to use the class where
						-- this feature has been written as context.
					l_context := l_feature.implementation_class
					l_constant_type := current_dynamic_system.dynamic_primary_type (l_type, l_context)
					l_manifest_constant := constant_features.item_for_iteration
					extra_dynamic_type_sets.force_last (l_constant_type)
					old_constant_index := l_manifest_constant.index
					l_manifest_constant.set_index (current_dynamic_type_sets.count + extra_dynamic_type_sets.count)
					l_manifest_constant.process (Current)
					l_manifest_constant.set_index (old_constant_index)
					extra_dynamic_type_sets.remove_last
				end
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				constant_features.forth
			end
			from inline_constants.start until inline_constants.after loop
				l_constant := inline_constants.key_for_iteration
				i := i + 1
				if i > l_threshold then
					dedent
					current_file.put_character ('}')
					current_file.put_new_line
					current_file.put_new_line
					flush_to_c_file
					j := j + 1
					i := 0
					header_file.put_string (c_extern)
					header_file.put_character (' ')
					header_file.put_string (c_void)
					header_file.put_character (' ')
					header_file.put_string (c_ge_init_const)
					header_file.put_integer (j)
					header_file.put_character ('(')
					header_file.put_string (c_void)
					header_file.put_character (')')
					header_file.put_character (';')
					header_file.put_new_line
					current_file.put_string (c_void)
					current_file.put_character (' ')
					current_file.put_string (c_ge_init_const)
					current_file.put_integer (j)
					current_file.put_character ('(')
					current_file.put_character (')')
					current_file.put_new_line
					current_file.put_character ('{')
					current_file.put_new_line
					indent
				end
				print_indentation
				print_inline_constant_name (l_constant, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('(')
				l_manifest_constant := l_constant.constant
				l_constant_type := inline_constants.item_for_iteration
				extra_dynamic_type_sets.force_last (l_constant_type)
				old_constant_index := l_manifest_constant.index
				l_manifest_constant.set_index (current_dynamic_type_sets.count + extra_dynamic_type_sets.count)
				l_manifest_constant.process (Current)
				l_manifest_constant.set_index (old_constant_index)
				extra_dynamic_type_sets.remove_last
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				inline_constants.forth
			end
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
				-- Main 'GE_init_const' function.
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			header_file.put_string (c_void)
			current_file.put_string (c_void)
			header_file.put_character (' ')
			current_file.put_character (' ')
			header_file.put_string (c_ge_init_const)
			current_file.put_string (c_ge_init_const)
			header_file.put_character ('(')
			current_file.put_character ('(')
			header_file.put_string (c_void)
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			from i := 1 until i > j loop
				print_indentation
				current_file.put_string (c_ge_init_const)
				current_file.put_integer (i)
				current_file.put_character ('(')
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				i := i + 1
			end
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
		end

	print_dynamic_type_id_set_constants
			-- Print 'GE_dtsN' constants to `current_file', and their signature to `header_file'.
			-- 'GE_dtsN' are dymamic type id sets whose ids are sorted in increasing order.
		local
			l_dts_name: STRING
			l_dts_ids: STRING
			l_size: INTEGER
		do
			if not dynamic_type_id_set_names.is_empty then
				from dynamic_type_id_set_names.start until dynamic_type_id_set_names.after loop
					l_dts_name := dynamic_type_id_set_names.item_for_iteration
					l_dts_ids := dynamic_type_id_set_names.key_for_iteration
					l_size := l_dts_ids.occurrences (',') + 1
					header_file.put_string (c_extern)
					header_file.put_character (' ')
					header_file.put_string (c_eif_type_index)
					header_file.put_character (' ')
					header_file.put_string (l_dts_name)
					header_file.put_character ('[')
					header_file.put_integer (l_size)
					header_file.put_character (']')
					header_file.put_character (';')
					header_file.put_new_line
					current_file.put_string (c_eif_type_index)
					current_file.put_character (' ')
					current_file.put_string (l_dts_name)
					current_file.put_character ('[')
					current_file.put_integer (l_size)
					current_file.put_character (']')
					current_file.put_character (' ')
					current_file.put_character ('=')
					current_file.put_character (' ')
					current_file.put_character ('{')
					current_file.put_string (l_dts_ids)
					current_file.put_character ('}')
					current_file.put_character (';')
					current_file.put_new_line
					dynamic_type_id_set_names.forth
				end
				header_file.put_new_line
				current_file.put_new_line
			end
		end

feature {NONE} -- Memory allocation

	print_malloc_current (a_feature: ET_FEATURE)
			-- Print memory allocation of 'Current' with `a_feature' as creation procedure.
			-- Do not call the creation procedure.
		require
			a_feature_not_void: a_feature /= Void
		local
			l_temp: ET_IDENTIFIER
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_argument_name: ET_IDENTIFIER
			old_file: KI_TEXT_OUTPUT_STREAM
		do
			if current_type.is_expanded then
					-- Variable declarations.
					-- They go to the beginning of the generated C function.
				old_file := current_file
				current_file := current_function_header_buffer
				l_temp := new_temp_variable (current_type)
				print_indentation
				print_type_declaration (current_type, current_file)
				current_file.put_character ('*')
				current_file.put_character (' ')
				print_current_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('&')
				print_temp_name (l_temp, current_file)
				current_file.put_character (';')
				current_file.put_new_line
					-- Instructions.
					-- They go to the current location in the generated C function.
				current_file := old_file
				print_indentation
				print_temp_name (l_temp, current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				print_default_name (current_type, current_file)
				current_file.put_character (';')
				current_file.put_new_line
			else
					-- Variable declarations.
					-- They go to the beginning of the generated C function.
				old_file := current_file
				current_file := current_function_header_buffer
				print_indentation
				print_type_declaration (current_type, current_file)
				current_file.put_character (' ')
				print_current_name (current_file)
				current_file.put_character (';')
				current_file.put_new_line
					-- Instructions.
					-- They go to the current location in the generated C function.
				current_file := old_file
				print_indentation
				print_current_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_string (c_ge_new)
				current_file.put_integer (current_type.id)
				current_file.put_character ('(')
				if attached {ET_DYNAMIC_SPECIAL_TYPE} current_type as l_special_type then
					l_arguments := a_feature.arguments
					if l_arguments = Void or else l_arguments.is_empty then
						current_file.put_character ('0')
					elseif not l_arguments.formal_argument (l_arguments.count).type.same_syntactical_type (current_universe.integer_type, current_type.base_type, current_type.base_type) then
						current_file.put_character ('0')
					else
						l_argument_name := l_arguments.formal_argument (l_arguments.count).name
						print_argument_name (l_argument_name, current_file)
					end
					current_file.put_character (',')
					current_file.put_character (' ')
				end
				current_file.put_string (c_eif_true)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_object_allocation_functions
			-- For each non-expanded type that is alive (e.g. that can have instances at runtime),
			-- print a 'GE_new<type-id>' function to `current_file', and its signature to `header_file'.
			-- 'GE_new<type-id>' creates a new instance of type type-id, with possible default
			-- initialization depending on its argument. It also registers the 'dispose' feature
			-- for that object to the GC if such feature exists.
		local
			l_dynamic_types: DS_ARRAYED_LIST [ET_DYNAMIC_PRIMARY_TYPE]
			l_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
		do
			l_dynamic_types := dynamic_types
			nb := l_dynamic_types.count
			from i := 1 until i > nb loop
				l_type := l_dynamic_types.item (i)
				if l_type.is_alive and then not l_type.is_expanded then
					if not l_type.base_class.is_type_class then
						print_object_allocation_function (l_type)
					end
				end
				i := i + 1
			end
		end

	print_object_allocation_function (a_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print 'GE_new<type-id>' function to `current_file', and its signature to `header_file'.
			-- 'GE_new<type-id>' creates a new instance of type `a_type', with possible default
			-- initialization depending on its argument. It also registers the 'dispose' feature
			-- for that object to the GC if such feature exists.
		require
			a_type_not_void: a_type /= Void
			a_type_reference: not a_type.is_expanded
		local
			l_special_type: detachable ET_DYNAMIC_SPECIAL_TYPE
			l_item_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_has_nested_reference_fields: BOOLEAN
			l_dynamic_type_set: ET_DYNAMIC_TYPE_SET
			l_integer_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if attached {ET_DYNAMIC_SPECIAL_TYPE} a_type as l_attached_special_type then
				l_special_type := l_attached_special_type
			end
				-- Print signature to `header_file' and `current_file'.
			header_file.put_string ("/* New instance of type ")
			header_file.put_string (a_type.base_type.unaliased_to_text)
			header_file.put_line (" */")
			current_file.put_string ("/* New instance of type ")
			current_file.put_string (a_type.base_type.unaliased_to_text)
			current_file.put_line (" */")
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			print_type_declaration (a_type, header_file)
			print_type_declaration (a_type, current_file)
			header_file.put_character (' ')
			current_file.put_character (' ')
			header_file.put_string (c_ge_new)
			current_file.put_string (c_ge_new)
			header_file.put_integer (a_type.id)
			current_file.put_integer (a_type.id)
			header_file.put_character ('(')
			current_file.put_character ('(')
			if l_special_type /= Void then
				if l_special_type.attribute_count < 2 then
						-- Internal error: class "SPECIAL" should have at least the
						-- features 'count' and 'capacity' as first features.
						-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					l_dynamic_type_set := result_type_set_in_feature (l_special_type.queries.item (2))
					l_integer_type := l_dynamic_type_set.static_type.primary_type
					print_type_declaration (l_integer_type, header_file)
					print_type_declaration (l_integer_type, current_file)
					header_file.put_character (' ')
					header_file.put_character ('a')
					header_file.put_character ('1')
					header_file.put_character (',')
					header_file.put_character (' ')
					current_file.put_character (' ')
					current_file.put_character ('a')
					current_file.put_character ('1')
					current_file.put_character (',')
					current_file.put_character (' ')
				end
			end
			print_type_declaration (current_dynamic_system.boolean_type, header_file)
			print_type_declaration (current_dynamic_system.boolean_type, current_file)
			header_file.put_character (' ')
			header_file.put_string (c_initialize)
			current_file.put_character (' ')
			current_file.put_string (c_initialize)
			header_file.put_character (')')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
				-- Print body to `current_file'.
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_indentation
			print_type_declaration (a_type, current_file)
			current_file.put_character (' ')
			print_result_name (current_file)
			current_file.put_character (';')
			current_file.put_new_line
			print_indentation
			print_result_name (current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_character ('(')
			print_type_declaration (a_type, current_file)
			current_file.put_character (')')
			if a_type.has_nested_reference_fields then
				l_has_nested_reference_fields := True
				current_file.put_string (c_ge_malloc)
			else
				current_file.put_string (c_ge_malloc_atomic)
			end
			current_file.put_character ('(')
			current_file.put_string (c_sizeof)
			current_file.put_character ('(')
			print_type_name (a_type, current_file)
			current_file.put_character (')')
			if l_special_type /= Void then
				l_item_type := l_special_type.item_type_set.static_type.primary_type
				current_file.put_character ('+')
				current_file.put_character ('(')
				current_file.put_character ('a')
				current_file.put_character ('1')
				current_file.put_character ('>')
				current_file.put_character ('1')
				current_file.put_character ('?')
				current_file.put_character ('(')
				current_file.put_character ('a')
				current_file.put_character ('1')
				current_file.put_character ('-')
				current_file.put_character ('1')
				current_file.put_character (')')
				current_file.put_character (':')
				current_file.put_character ('0')
				current_file.put_character (')')
				current_file.put_character ('*')
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				print_type_declaration (l_item_type, current_file)
				current_file.put_character (')')
			end
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
				-- Dispose routine.
			print_dispose_registration (tokens.result_keyword, a_type)
				-- Default initialization.
			print_indentation
			current_file.put_string (c_if)
			current_file.put_character (' ')
			current_file.put_character ('(')
			current_file.put_string (c_initialize)
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_indentation
			current_file.put_character ('*')
			print_type_cast (a_type, current_file)
			print_result_name (current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			print_default_name (a_type, current_file)
			current_file.put_character (';')
			current_file.put_new_line
			if l_special_type /= Void and l_item_type /= Void then
					-- Set 'capacity'.
				print_indentation
				print_attribute_special_capacity_access (tokens.result_keyword, l_special_type, False)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('a')
				current_file.put_character ('1')
				current_file.put_character (';')
				current_file.put_new_line
					-- Set 'count'.
				print_indentation
				print_attribute_special_count_access (tokens.result_keyword, l_special_type, False)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				current_file.put_character ('0')
				current_file.put_character (';')
				current_file.put_new_line
					-- Initialize items if needed.
				current_file.put_string (c_ifndef)
				current_file.put_character (' ')
				if l_has_nested_reference_fields then
					current_file.put_line (c_ge_malloc_cleared)
				else
					current_file.put_line (c_ge_malloc_atomic_cleared)
				end
				print_indentation
				current_file.put_string (c_memset)
				current_file.put_character ('(')
				print_attribute_special_item_access (tokens.result_keyword, l_special_type, False)
				current_file.put_character (',')
				current_file.put_character ('0')
				current_file.put_character (',')
				current_file.put_character ('a')
				current_file.put_character ('1')
				current_file.put_character ('*')
				current_file.put_string (c_sizeof)
				current_file.put_character ('(')
				print_type_declaration (l_item_type, current_file)
				current_file.put_character (')')
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
				current_file.put_line (c_endif)
			end
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
				-- Return the new object.
			print_indentation
			current_file.put_string (c_return)
			current_file.put_character (' ')
			print_result_name (current_file)
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
				-- Flush to file.
			flush_to_c_file
		end

	print_once_per_object_data_allocation_functions
			-- For each type with once-per-object routines that is alive (e.g. that can have instances at runtime),
			-- print 'GE_new_once_per_object_data<type-id>' function to `current_file', and its signature to `header_file'.
			-- 'GE_new_once_per_object_data<type-id>' creates the once-per-object data for an object of type type-id.
			-- It also registers a 'dispose' function for that data to the GC if necessary.
		local
			l_dynamic_types: DS_ARRAYED_LIST [ET_DYNAMIC_PRIMARY_TYPE]
			l_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
		do
			l_dynamic_types := dynamic_types
			nb := l_dynamic_types.count
			from i := 1 until i > nb loop
				l_type := l_dynamic_types.item (i)
				if l_type.is_alive and then l_type.has_once_per_object_routines then
					print_once_per_object_data_allocation_function (l_type)
				end
				i := i + 1
			end
		end

	print_once_per_object_data_allocation_function (a_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print 'GE_new_once_per_object_data<type-id>' function to `current_file', and its signature to `header_file'.
			-- 'GE_new_once_per_object_data<type-id>' creates the once-per-object data for an object of type `a_type'.
			-- It also registers a 'dispose' function for that data to the GC if necessary.
		require
			a_type_not_void: a_type /= Void
			a_type_has_once_per_object_routines: a_type.has_once_per_object_routines
		local
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_query: ET_DYNAMIC_FEATURE
			l_procedures: ET_DYNAMIC_FEATURE_LIST
			l_procedure: ET_DYNAMIC_FEATURE
			i, nb: INTEGER
		do
				-- Print signature to `header_file' and `current_file'.
			header_file.put_string ("/* New once-per-object data for type ")
			header_file.put_string (a_type.base_type.unaliased_to_text)
			header_file.put_line (" */")
			current_file.put_string ("/* New once-per-object data for type ")
			current_file.put_string (a_type.base_type.unaliased_to_text)
			current_file.put_line (" */")
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			print_once_per_object_data_type_name (a_type, header_file)
			print_once_per_object_data_type_name (a_type, current_file)
			header_file.put_character ('*')
			header_file.put_character (' ')
			current_file.put_character ('*')
			current_file.put_character (' ')
			header_file.put_string (c_ge_new_once_per_object_data)
			current_file.put_string (c_ge_new_once_per_object_data)
			header_file.put_integer (a_type.id)
			current_file.put_integer (a_type.id)
			header_file.put_character ('(')
			header_file.put_character (')')
			current_file.put_character ('(')
			current_file.put_character (')')
			header_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
				-- Print body to `current_file'.
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_indentation
			print_once_per_object_data_type_name (a_type, current_file)
			current_file.put_character ('*')
			current_file.put_character (' ')
			print_result_name (current_file)
			current_file.put_character (';')
			current_file.put_new_line
			print_indentation
			print_result_name (current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_character ('(')
			print_once_per_object_data_type_name (a_type, current_file)
			current_file.put_character ('*')
			current_file.put_character (')')
			current_file.put_string (c_ge_malloc)
			current_file.put_character ('(')
			current_file.put_string (c_sizeof)
			current_file.put_character ('(')
			print_once_per_object_data_type_name (a_type, current_file)
			current_file.put_character (')')
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
				-- Make sure that the allocated memory is cleared (with zeros).
			current_file.put_string (c_ifndef)
			current_file.put_character (' ')
			current_file.put_line (c_ge_malloc_cleared)
			print_indentation
			current_file.put_string (c_memset)
			current_file.put_character ('(')
			print_result_name (current_file)
			current_file.put_character (',')
			current_file.put_character ('0')
			current_file.put_character (',')
			current_file.put_string (c_sizeof)
			current_file.put_character ('(')
			print_once_per_object_data_type_name (a_type, current_file)
			current_file.put_character (')')
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
			current_file.put_line (c_endif)
				-- Initialize mutexes.
			if multithreaded_mode then
				l_queries := a_type.queries
				i := a_type.attribute_count + 1
				nb := l_queries.count
				from
				until
					i > nb
				loop
					l_query := l_queries.item (i)
					if l_query.is_once_per_object then
						print_indentation
						print_result_name (current_file)
						current_file.put_string (c_arrow)
						print_once_per_object_mutex_name (l_query, a_type, current_file)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_string (c_ge_mutex_create)
						current_file.put_character ('(')
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					end
					i := i + 1
				end
				l_procedures := a_type.procedures
				nb := l_procedures.count
				from
					i := 1
				until
					i > nb
				loop
					l_procedure := l_procedures.item (i)
					if l_procedure.is_once_per_object then
						print_indentation
						print_result_name (current_file)
						current_file.put_string (c_arrow)
						print_once_per_object_mutex_name (l_procedure, a_type, current_file)
						current_file.put_character (' ')
						current_file.put_character ('=')
						current_file.put_character (' ')
						current_file.put_string (c_ge_mutex_create)
						current_file.put_character ('(')
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					end
					i := i + 1
				end
					-- Register 'dispose' function to destroy the mutexes.
				print_indentation
				current_file.put_string (c_ge_register_dispose_once_per_object_data)
				current_file.put_character ('(')
				print_result_name (current_file)
				current_file.put_character (',')
				current_file.put_character ('&')
				current_file.put_string (c_ge_dispose_once_per_object_data)
				current_file.put_integer (a_type.id)
				current_file.put_character (')')
				current_file.put_character (';')
				current_file.put_new_line
			end
				-- Return the new data.
			print_indentation
			current_file.put_string (c_return)
			current_file.put_character (' ')
			print_result_name (current_file)
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			current_file.put_character ('}')
			current_file.put_new_line
			current_file.put_new_line
				-- Flush to file.
			flush_to_c_file
		end

	print_once_per_object_data_dispose_functions
			-- For each type with once-per-object routines that is alive (e.g. that can have instances at runtime),
			-- print 'GE_dispose_once_per_object_data<type-id>' function to `current_file', and its signature to `header_file'.
			-- 'GE_dispose_once_per_object_data<type-id>' destroys the mutexes used in multithreaded mode.
		local
			l_dynamic_types: DS_ARRAYED_LIST [ET_DYNAMIC_PRIMARY_TYPE]
			l_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
		do
			l_dynamic_types := dynamic_types
			nb := l_dynamic_types.count
			from i := 1 until i > nb loop
				l_type := l_dynamic_types.item (i)
				if l_type.is_alive and then l_type.has_once_per_object_routines then
					print_once_per_object_data_dispose_function (l_type)
				end
				i := i + 1
			end
		end

	print_once_per_object_data_dispose_function (a_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print 'GE_dispose_once_per_object_data<type-id>' function to `current_file', and its signature to `header_file'.
			-- 'GE_dispose_once_per_object_data<type-id>' destroys the mutexes used in multithreaded mode.
		require
			a_type_not_void: a_type /= Void
			a_type_has_once_per_object_routines: a_type.has_once_per_object_routines
		local
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_query: ET_DYNAMIC_FEATURE
			l_procedures: ET_DYNAMIC_FEATURE_LIST
			l_procedure: ET_DYNAMIC_FEATURE
			i, nb: INTEGER
		do
			if multithreaded_mode then
					-- Print signature to `header_file' and `current_file'.
				header_file.put_string ("/* Dispose once-per-object data for type ")
				header_file.put_string (a_type.base_type.unaliased_to_text)
				header_file.put_line (" */")
				current_file.put_string ("/* Dispose once-per-object data for type ")
				current_file.put_string (a_type.base_type.unaliased_to_text)
				current_file.put_line (" */")
				header_file.put_string (c_extern)
				header_file.put_character (' ')
				header_file.put_string (c_void)
				current_file.put_string (c_void)
				header_file.put_character (' ')
				current_file.put_character (' ')
				header_file.put_string (c_ge_dispose_once_per_object_data)
				current_file.put_string (c_ge_dispose_once_per_object_data)
				header_file.put_integer (a_type.id)
				current_file.put_integer (a_type.id)
				header_file.put_character ('(')
				current_file.put_character ('(')
				print_once_per_object_data_type_name (a_type, header_file)
				print_once_per_object_data_type_name (a_type, current_file)
				header_file.put_character ('*')
				header_file.put_character (' ')
				current_file.put_character ('*')
				current_file.put_character (' ')
				header_file.put_character ('a')
				header_file.put_character ('1')
				current_file.put_character ('a')
				current_file.put_character ('1')
				header_file.put_character (')')
				current_file.put_character (')')
				header_file.put_character (';')
				header_file.put_new_line
				current_file.put_new_line
					-- Print body to `current_file'.
				current_file.put_character ('{')
				current_file.put_new_line
				indent
				l_queries := a_type.queries
				i := a_type.attribute_count + 1
				nb := l_queries.count
				from
				until
					i > nb
				loop
					l_query := l_queries.item (i)
					if l_query.is_once_per_object then
						print_indentation
						current_file.put_string (c_ge_mutex_destroy)
						current_file.put_character ('(')
						current_file.put_character ('a')
						current_file.put_character ('1')
						current_file.put_string (c_arrow)
						print_once_per_object_mutex_name (l_query, a_type, current_file)
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					end
					i := i + 1
				end
				l_procedures := a_type.procedures
				nb := l_procedures.count
				from
					i := 1
				until
					i > nb
				loop
					l_procedure := l_procedures.item (i)
					if l_procedure.is_once_per_object then
						print_indentation
						current_file.put_string (c_ge_mutex_destroy)
						current_file.put_character ('(')
						current_file.put_character ('a')
						current_file.put_character ('1')
						current_file.put_string (c_arrow)
						print_once_per_object_mutex_name (l_procedure, a_type, current_file)
						current_file.put_character (')')
						current_file.put_character (';')
						current_file.put_new_line
					end
					i := i + 1
				end
				dedent
				current_file.put_character ('}')
				current_file.put_new_line
				current_file.put_new_line
			end
				-- Flush to file.
			flush_to_c_file
		end

	print_dispose_registration (an_object: ET_EXPRESSION; a_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print to `current_file' the registration to the GC of the
			-- 'dispose' feature, if it exists, to be called when the
			-- object `an_object' of type `a_type' will be reclaimed.
			-- Note that no 'dispose' routine will be registered if `a_type'
			-- is expanded (according to ECMA Eiffel, expanded classes should
			-- not inherit from class DISPOSABLE), or if the routine has an
			-- empty compound (optimization).
		require
			an_object_not_void: an_object /= Void
			a_type_not_void: a_type /= Void
		local
			l_dispose_seed: INTEGER
			l_dispose_procedure: detachable ET_DYNAMIC_FEATURE
		do
			if not a_type.is_expanded then
				dispose_procedures.search (a_type)
				if dispose_procedures.found then
					l_dispose_procedure := dispose_procedures.found_item
				else
					l_dispose_seed := current_system.dispose_seed
					if l_dispose_seed <= 0 then
						l_dispose_procedure := Void
					elseif not attached a_type.seeded_dynamic_procedure (l_dispose_seed, current_dynamic_system) as l_procedure then
							-- There is no feature 'dispose' available.
						l_dispose_procedure := Void
					elseif not attached {ET_INTERNAL_PROCEDURE} l_procedure.static_feature as l_internal_procedure then
							-- The feature 'dispose' is an external routine.
							-- Register it to the GC.
						l_dispose_procedure := l_procedure
					elseif attached l_internal_procedure.compound as l_compound and then not l_compound.is_empty then
							-- The feature 'dispose' is not empty.
							-- Register it to the GC.
						l_dispose_procedure := l_procedure
					end
					dispose_procedures.force_last_new (l_dispose_procedure, a_type)
				end
				if l_dispose_procedure /= Void then
					print_indentation
					current_file.put_string (c_ge_register_dispose)
					current_file.put_character ('(')
					print_expression (an_object)
					current_file.put_character (',')
					current_file.put_character ('&')
					print_routine_name (l_dispose_procedure, a_type, current_file)
					current_file.put_character (')')
					current_file.put_character (';')
					current_file.put_new_line
					register_called_feature (l_dispose_procedure)
				end
			end
		end

	dispose_procedures: DS_HASH_TABLE [detachable ET_DYNAMIC_FEATURE, ET_DYNAMIC_PRIMARY_TYPE]
			-- 'dispose' procedures indexed by types, or Void if we figured out
			-- that there was no 'dispose' procedure for the given type

feature {NONE} -- Trace generation

	print_feature_trace_message_call (in: BOOLEAN)
			-- Print to `current_file' a call that will display to 'stderr' information
			-- about the fact that we just entered (if `in' is True) or just exited
			-- (if `in' is False) the current feature.
		do
			if trace_mode then
				current_file.put_string (c_ifdef)
				current_file.put_character (' ')
				current_file.put_line (c_eif_trace)
				if in then
						-- Instruction to print the trace message.
					print_indentation
					print_unindented_feature_info_message_call ("->")
					current_file.put_new_line
						-- Put the body of the feature in a block so that there
						-- is no C compilation error if some variables are
						-- declared after the instruction to print the trace message.
					print_indentation
					current_file.put_character ('{')
					current_file.put_new_line
				else
						-- Close the block containing the body of the feature.
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
						-- Instruction to print the trace message.
					print_indentation
					print_unindented_feature_info_message_call ("<-")
					current_file.put_new_line
				end
				current_file.put_line (c_endif)
			end
		end

	print_agent_trace_message_call (an_agent: ET_AGENT; in: BOOLEAN)
			-- Print to `current_file' a call that will display to 'stderr' information
			-- about the fact that we just entered (if `in' is True) or just exited
			-- (if `in' is False) the agent `an_agent' appearing in current feature.
		require
			an_agent_not_void: an_agent /= Void
		local
			l_agent_message: STRING
		do
			if trace_mode then
				if an_agent.is_inline_agent then
					l_agent_message := " inline agent at line " + an_agent.position.line.out + " in"
				else
					l_agent_message := " agent at line " + an_agent.position.line.out + " in"
				end
				current_file.put_string (c_ifdef)
				current_file.put_character (' ')
				current_file.put_line (c_eif_trace)
				if in then
						-- Instruction to print the trace message.
					print_indentation
					print_unindented_feature_info_message_call ("->" + l_agent_message)
					current_file.put_new_line
						-- Put the body of the feature in a block so that there
						-- is no C compilation error if some variables are
						-- declared after the instruction to print the trace message.
					print_indentation
					current_file.put_character ('{')
					current_file.put_new_line
				else
						-- Close the block containing the body of the feature.
					print_indentation
					current_file.put_character ('}')
					current_file.put_new_line
						-- Instruction to print the trace message.
					print_indentation
					print_unindented_feature_info_message_call ("<-" + l_agent_message)
					current_file.put_new_line
				end
				current_file.put_line (c_endif)
			end
		end

	print_feature_info_message_call (a_message: STRING)
			-- Print to `current_file' a call that will display to 'stderr' information
			-- about the current feature of form: `a_message' TYPE.feature
		require
			a_message_not_void: a_message /= Void
		do
			print_show_console_call
			print_indentation
			print_unindented_feature_info_message_call (a_message)
			current_file.put_new_line
		end

	print_unindented_feature_info_message_call (a_message: STRING)
			-- Print to `current_file' a call that will display to 'stderr' information
			-- about the current feature of form: `a_message' TYPE.feature
			-- This call is not indented nor finished by a new-line.
			-- Useful to define a macro.
		require
			a_message_not_void: a_message /= Void
		do
			current_file.put_string (c_fprintf)
			current_file.put_character ('(')
			current_file.put_string (c_stderr)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_string ("%"%%s %%s.%%s\n%"")
			current_file.put_character (',')
			current_file.put_character (' ')
			print_escaped_string (a_message)
			current_file.put_character (',')
			current_file.put_character (' ')
			print_escaped_string (current_type.base_type.unaliased_to_text)
			current_file.put_character (',')
			current_file.put_character (' ')
			print_escaped_string (current_feature.static_feature.lower_name)
			current_file.put_character (')')
			current_file.put_character (';')
		end

	print_info_message_call (a_message: STRING)
			-- Print to `current_file' a call that will display to 'stderr'
			-- information form: `a_message'
		require
			a_message_not_void: a_message /= Void
		do
			print_show_console_call
			print_indentation
			current_file.put_string (c_fprintf)
			current_file.put_character ('(')
			current_file.put_string (c_stderr)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_string ("%"%%s\n%"")
			current_file.put_character (',')
			current_file.put_character (' ')
			print_escaped_string (a_message)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_show_console_call
			-- Print to `current_file' a call to 'GE_show_console' to make sure
			-- that a DOS console is available for Windows applications.
		do
			print_indentation
			current_file.put_string (c_ge_show_console)
			current_file.put_character ('(')
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
		end

feature {NONE} -- Once feature generation

	print_once_per_process_status (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once status of `a_feature'.
			-- `a_feature' is expected to be a once-per-process routine of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_process: a_feature.is_once_per_process
		local
			l_once_prefix: STRING
		do
			l_once_prefix := once_prefixes.item (a_once_kind)
			current_file.put_string (c_ge_process_onces)
			current_file.put_string (c_arrow)
			current_file.put_string (l_once_prefix)
			current_file.put_string (c_status_suffix)
			current_file.put_character ('[')
			current_file.put_integer (a_once_index)
			current_file.put_character (']')
		end

	print_thread_safe_once_per_process_status (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once status of `a_feature'.
			-- `a_feature' is expected to be a once-per-process routine of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
			-- In multithreaded mode, use the version cached in the current thread.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_process: a_feature.is_once_per_process
		local
			l_once_prefix: STRING
		do
			l_once_prefix := once_prefixes.item (a_once_kind)
			if multithreaded_mode then
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_process_onces)
			else
				current_file.put_string (c_ge_process_onces)
			end
			current_file.put_string (c_arrow)
			current_file.put_string (l_once_prefix)
			current_file.put_string (c_status_suffix)
			current_file.put_character ('[')
			current_file.put_integer (a_once_index)
			current_file.put_character (']')
		end

	print_once_per_thread_status (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once status of `a_feature'.
			-- `a_feature' is expected to be a once-per-thread routine of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_thread: a_feature.is_once_per_thread
		local
			l_once_prefix: STRING
		do
			l_once_prefix := once_prefixes.item (a_once_kind)
			if multithreaded_mode then
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_thread_onces)
			else
				current_file.put_string (c_ge_process_onces)
			end
			current_file.put_string (c_arrow)
			current_file.put_string (l_once_prefix)
			current_file.put_string (c_status_suffix)
			current_file.put_character ('[')
			current_file.put_integer (a_once_index)
			current_file.put_character (']')
		end

	print_once_per_object_status (a_feature: ET_DYNAMIC_FEATURE)
			-- Print variable containing once status of `a_feature'.
			-- `a_feature' is expected to be a once-per-object routine.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_object: a_feature.is_once_per_object
		do
			print_attribute_onces_access (tokens.current_keyword, current_type, False)
			current_file.put_string (C_arrow)
			print_once_per_object_status_name (a_feature, current_type, current_file)
		end

	print_once_status (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once status of `a_feature'.
			-- In case of one-per-process or once-per-thread, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
		do
			if a_feature.is_once_per_object then
				print_once_per_object_status (a_feature)
			elseif a_feature.is_once_per_process then
				print_once_per_process_status (a_feature, a_once_kind, a_once_index)
			elseif a_feature.is_once_per_thread then
				print_once_per_thread_status (a_feature, a_once_kind, a_once_index)
			else
					-- Internal error: this once key is not supported.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_once_per_process_value (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once value of `a_feature'.
			-- `a_feature' is expected to be a once-per-process function of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_process: a_feature.is_once_per_process
			a_feature_is_function: a_feature.is_function
		local
			l_once_prefix: STRING
		do
			l_once_prefix := once_prefixes.item (a_once_kind)
			current_file.put_string (c_ge_process_onces)
			current_file.put_string (c_arrow)
			current_file.put_string (l_once_prefix)
			current_file.put_string (c_value_suffix)
			current_file.put_character ('[')
			current_file.put_integer (a_once_index)
			current_file.put_character (']')
		end

	print_thread_safe_once_per_process_value (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once value of `a_feature'.
			-- `a_feature' is expected to be a once-per-process function of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
			-- In case of once-per-process in multithreaded mode,
			-- use the version cached in the current thread.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_process: a_feature.is_once_per_process
			a_feature_is_function: a_feature.is_function
		local
			l_once_prefix: STRING
		do
			l_once_prefix := once_prefixes.item (a_once_kind)
			if multithreaded_mode then
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_process_onces)
			else
				current_file.put_string (c_ge_process_onces)
			end
			current_file.put_string (c_arrow)
			current_file.put_string (l_once_prefix)
			current_file.put_string (c_value_suffix)
			current_file.put_character ('[')
			current_file.put_integer (a_once_index)
			current_file.put_character (']')
		end

	print_once_per_thread_value (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once value of `a_feature'.
			-- `a_feature' is expected to be a once-per-thread function of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_thread: a_feature.is_once_per_thread
			a_feature_is_function: a_feature.is_function
		local
			l_once_prefix: STRING
		do
			l_once_prefix := once_prefixes.item (a_once_kind)
			if multithreaded_mode then
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_thread_onces)
			else
				current_file.put_string (c_ge_process_onces)
			end
			current_file.put_string (c_arrow)
			current_file.put_string (l_once_prefix)
			current_file.put_string (c_value_suffix)
			current_file.put_character ('[')
			current_file.put_integer (a_once_index)
			current_file.put_character (']')
		end

	print_once_per_object_value (a_feature: ET_DYNAMIC_FEATURE)
			-- Print variable containing once value of `a_feature'.
			-- `a_feature' is expected to be a once-per-object function.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_function: a_feature.is_function
			a_feature_is_once_per_object: a_feature.is_once_per_object
		do
			print_attribute_onces_access (tokens.current_keyword, current_type, False)
			current_file.put_string (C_arrow)
			print_once_per_object_value_name (a_feature, current_type, current_file)
		end

	print_once_value (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once value of `a_feature'.
			-- In case of one-per-process or once-per-thread, `a_feature'
			-- is expected to be a once function of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
			a_feature_is_function: a_feature.is_function
		do
			if a_feature.is_once_per_object then
				print_once_per_object_value (a_feature)
			elseif a_feature.is_once_per_process then
				print_once_per_process_value (a_feature, a_once_kind, a_once_index)
			elseif a_feature.is_once_per_thread then
				print_once_per_thread_value (a_feature, a_once_kind, a_once_index)
			else
					-- Internal error: this once key is not supported.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_once_per_process_exception (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once exception of `a_feature'.
			-- `a_feature' is expected to be a once-per-process routine of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_process: a_feature.is_once_per_process
		local
			l_once_prefix: STRING
		do
			l_once_prefix := once_prefixes.item (a_once_kind)
			current_file.put_string (c_ge_process_onces)
			current_file.put_string (c_arrow)
			current_file.put_string (l_once_prefix)
			current_file.put_string (c_exception_suffix)
			current_file.put_character ('[')
			current_file.put_integer (a_once_index)
			current_file.put_character (']')
		end

	print_thread_safe_once_per_process_exception (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once exception of `a_feature'.
			-- `a_feature' is expected to be a once-per_process routine of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
			-- In case of once-per-process in multithreaded mode,
			-- use the version cached in the current thread.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_process: a_feature.is_once_per_process
		local
			l_once_prefix: STRING
		do
			l_once_prefix := once_prefixes.item (a_once_kind)
			if multithreaded_mode then
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_process_onces)
			else
				current_file.put_string (c_ge_process_onces)
			end
			current_file.put_string (c_arrow)
			current_file.put_string (l_once_prefix)
			current_file.put_string (c_exception_suffix)
			current_file.put_character ('[')
			current_file.put_integer (a_once_index)
			current_file.put_character (']')
		end

	print_once_per_thread_exception (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once exception of `a_feature'.
			-- `a_feature' is expected to be a once-per-thread routine of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_thread: a_feature.is_once_per_thread
		local
			l_once_prefix: STRING
		do
			l_once_prefix := once_prefixes.item (a_once_kind)
			if multithreaded_mode then
				current_file.put_string (c_ac)
				current_file.put_string (c_arrow)
				current_file.put_string (c_thread_onces)
			else
				current_file.put_string (c_ge_process_onces)
			end
			current_file.put_string (c_arrow)
			current_file.put_string (l_once_prefix)
			current_file.put_string (c_exception_suffix)
			current_file.put_character ('[')
			current_file.put_integer (a_once_index)
			current_file.put_character (']')
		end

	print_once_per_object_exception (a_feature: ET_DYNAMIC_FEATURE)
			-- Print variable containing once exception of `a_feature'.
			-- `a_feature' is expected to be a once-per-object routine.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_object: a_feature.is_once_per_object
		do
			print_attribute_onces_access (tokens.current_keyword, current_type, False)
			current_file.put_string (C_arrow)
			print_once_per_object_exception_name (a_feature, current_type, current_file)
		end

	print_once_exception (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once exception of `a_feature'.
			-- In case of one-per-process or once-per-thread, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
		do
			if a_feature.is_once_per_object then
				print_once_per_object_exception (a_feature)
			elseif a_feature.is_once_per_process then
				print_once_per_process_exception (a_feature, a_once_kind, a_once_index)
			elseif a_feature.is_once_per_thread then
				print_once_per_thread_exception (a_feature, a_once_kind, a_once_index)
			else
					-- Internal error: this once key is not supported.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_assign_called_to_once_status (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to set the once status of `a_feature' to "called".
			-- In case of one-per-process or once-per-thread, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
		do
			print_indentation
			print_once_status (a_feature, a_once_kind, a_once_index)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_character ('%'')
			current_file.put_character ('\')
			current_file.put_character ('1')
			current_file.put_character ('%'')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_assign_called_to_thread_safe_once_status (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to set the once status of `a_feature' to "called".
			-- `a_feature' is expected to be a once-per-process routine of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
			-- Use the version cached in the current thread.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once_per_process
			multithreaded_mode: multithreaded_mode
		do
			print_indentation
			print_thread_safe_once_per_process_status (a_feature, a_once_kind, a_once_index)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_character ('%'')
			current_file.put_character ('\')
			current_file.put_character ('1')
			current_file.put_character ('%'')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_assign_completed_to_once_status (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to set the once status of `a_feature' to "completed".
			-- In case of one-per-process or once-per-thread, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
		do
			print_indentation
			print_once_status (a_feature, a_once_kind, a_once_index)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_character ('%'')
			current_file.put_character ('\')
			current_file.put_character ('2')
			current_file.put_character ('%'')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_once_status_is_completed (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to test whether the once status of `a_feature' is set to "completed".
			-- In case of one-per-process or once-per-thread, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
		do
			print_once_status (a_feature, a_once_kind, a_once_index)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_character ('%'')
			current_file.put_character ('\')
			current_file.put_character ('2')
			current_file.put_character ('%'')
		end

	print_assign_result_to_once_value (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to set the once value of `a_feature' to "Result".
			-- In case of one-per-process or once-per-thread, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
			a_feature_is_function: a_feature.is_function
		local
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if attached a_feature.result_type_set as l_result_type_set then
				print_indentation
				print_once_value (a_feature, a_once_kind, a_once_index)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				l_result_type := l_result_type_set.static_type.primary_type
				if not a_feature.is_once_per_object and then l_result_type.is_expanded and then not l_result_type.is_basic then
						-- The value needs to be boxed.
					current_file.put_string (c_ge_boxed)
					current_file.put_integer (l_result_type.id)
					current_file.put_character ('(')
					print_result_name (current_file)
					current_file.put_character (')')
				else
					print_result_name (current_file)
				end
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_assign_result_to_thread_safe_once_value (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to set the once value of `a_feature' to "Result".
			-- `a_feature' is expected to be a once-per-process routine of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
			-- Use the version cached in the current thread.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once_per_process
			a_feature_is_function: a_feature.is_function
			multithreaded_mode: multithreaded_mode
		local
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if attached a_feature.result_type_set as l_result_type_set then
				print_indentation
				print_thread_safe_once_per_process_value (a_feature, a_once_kind, a_once_index)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				l_result_type := l_result_type_set.static_type.primary_type
				if l_result_type.is_expanded and then not l_result_type.is_basic then
						-- The value needs to be boxed.
					current_file.put_string (c_ge_boxed)
					current_file.put_integer (l_result_type.id)
					current_file.put_character ('(')
					print_result_name (current_file)
					current_file.put_character (')')
				else
					print_result_name (current_file)
				end
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_assign_once_value_to_result (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to set "Result" to the once value of `a_feature'.
			-- In case of one-per-process or once-per-thread, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
			a_feature_is_function: a_feature.is_function
		local
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			if attached a_feature.result_type_set as l_result_type_set then
				print_indentation
				print_result_name (current_file)
				current_file.put_character (' ')
				current_file.put_character ('=')
				current_file.put_character (' ')
				l_result_type := l_result_type_set.static_type.primary_type
				if not a_feature.is_once_per_object and then l_result_type.is_expanded and then not l_result_type.is_basic then
						-- The value has been boxed.
						-- It needs to be unboxed.
					current_file.put_character ('(')
					print_boxed_type_cast (l_result_type, current_file)
					current_file.put_character ('(')
					print_once_value (a_feature, a_once_kind, a_once_index)
					current_file.put_character (')')
					current_file.put_character (')')
					current_file.put_string (c_arrow)
					print_boxed_attribute_item_name (l_result_type, current_file)
				else
					print_once_value (a_feature, a_once_kind, a_once_index)
				end
				current_file.put_character (';')
				current_file.put_new_line
			end
		end

	print_return_once_value (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print C instruction to return the once value of `a_feature'.
			-- if it is a function, or just an empty return if it's
			-- a procedure.
			-- In case of one-per-process or once-per-thread, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
		local
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			print_indentation
			current_file.put_string (c_return)
			if attached a_feature.result_type_set as l_result_type_set then
				current_file.put_character (' ')
				l_result_type := l_result_type_set.static_type.primary_type
				if not a_feature.is_once_per_object and then l_result_type.is_expanded and then not l_result_type.is_basic then
						-- The value has been boxed.
						-- It needs to be unboxed.
					current_file.put_character ('(')
					print_boxed_type_cast (l_result_type, current_file)
					current_file.put_character ('(')
					print_once_value (a_feature, a_once_kind, a_once_index)
					current_file.put_character (')')
					current_file.put_character (')')
					current_file.put_string (c_arrow)
					print_boxed_attribute_item_name (l_result_type, current_file)
				else
					print_once_value (a_feature, a_once_kind, a_once_index)
				end
			end
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_return_thread_safe_once_value (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print C instruction to return the once value of `a_feature'.
			-- if it is a function, or just an empty return if it's
			-- a procedure.
			-- `a_feature' is expected to be a once-per-process routine of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
			-- Use the version cached in the current thread.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once_per_process
			multithreaded_mode: multithreaded_mode
		local
			l_result_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			print_indentation
			current_file.put_string (c_return)
			if attached a_feature.result_type_set as l_result_type_set then
				current_file.put_character (' ')
				l_result_type := l_result_type_set.static_type.primary_type
				if l_result_type.is_expanded and then not l_result_type.is_basic then
						-- The value has been boxed.
						-- It needs to be unboxed.
					current_file.put_character ('(')
					print_boxed_type_cast (l_result_type, current_file)
					current_file.put_character ('(')
					print_thread_safe_once_per_process_value (a_feature, a_once_kind, a_once_index)
					current_file.put_character (')')
					current_file.put_character (')')
					current_file.put_string (c_arrow)
					print_boxed_attribute_item_name (l_result_type, current_file)
				else
					print_thread_safe_once_per_process_value (a_feature, a_once_kind, a_once_index)
				end
			end
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_assign_last_exception_to_once_exception (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to set the once exception of `a_feature' to the last exception raised.
			-- In case of one-per-process or once-per-thread, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
		do
			print_indentation
			print_once_exception (a_feature, a_once_kind, a_once_index)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_string (c_ge_last_exception_raised)
			current_file.put_character ('(')
			current_file.put_string (c_ac)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_assign_once_exception_to_thread_safe_once_exception (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to set the version of once exception cached in
			-- current thread to the non-cached version of once exception.
			-- `a_feature' is expected to be a once-per-process routine of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once_per_process
			multithreaded_mode: multithreaded_mode
		do
			print_indentation
			print_thread_safe_once_per_process_exception (a_feature, a_once_kind, a_once_index)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			print_once_exception (a_feature, a_once_kind, a_once_index)
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_raise_once_exception_if_any (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to raise the once exception if any.
			-- In case of one-per-process or once-per-thread, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
		do
			print_indentation
			current_file.put_string (c_if)
			current_file.put_character (' ')
			current_file.put_character ('(')
			print_once_exception (a_feature, a_once_kind, a_once_index)
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_indentation
			current_file.put_string (c_ge_raise_once_exception)
			current_file.put_character ('(')
			current_file.put_string (c_ac)
			current_file.put_character (',')
			current_file.put_character (' ')
			print_once_exception (a_feature, a_once_kind, a_once_index)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
		end

	print_raise_thread_safe_once_exception_if_any (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to raise the once exception if any.
			-- `a_feature' is expected to be a once-per_thread routine of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
			-- Use the version cached in the current thread.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_process: a_feature.is_once_per_process
			multithreaded_mode: multithreaded_mode
		do
			print_indentation
			current_file.put_string (c_if)
			current_file.put_character (' ')
			current_file.put_character ('(')
			print_thread_safe_once_per_process_exception (a_feature, a_once_kind, a_once_index)
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
			indent
			print_indentation
			current_file.put_string (c_ge_raise_once_exception)
			current_file.put_character ('(')
			current_file.put_string (c_ac)
			current_file.put_character (',')
			current_file.put_character (' ')
			print_thread_safe_once_per_process_exception (a_feature, a_once_kind, a_once_index)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
			dedent
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
		end

	print_once_per_process_mutex (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once mutex of `a_feature'.
			-- `a_feature' is expected to be a once-per_thread routine of kind
			-- `a_once_kind' with index `a_once_index' in 'GE_onces'
			-- C struct.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_process: a_feature.is_once_per_process
			multithreaded_mode: multithreaded_mode
		local
			l_once_prefix: STRING
		do
			l_once_prefix := once_prefixes.item (a_once_kind)
			current_file.put_string (c_ge_process_once_mutexes)
			current_file.put_string (c_arrow)
			current_file.put_string (l_once_prefix)
			current_file.put_string (c_mutex_suffix)
			current_file.put_character ('[')
			current_file.put_integer (a_once_index)
			current_file.put_character (']')
		end

	print_once_per_object_mutex (a_feature: ET_DYNAMIC_FEATURE)
			-- Print variable containing once mutex of `a_feature'.
			-- `a_feature' is expected to be a once-per-object routine.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once_per_object: a_feature.is_once_per_object
			multithreaded_mode: multithreaded_mode
		do
			print_attribute_onces_access (tokens.current_keyword, current_type, False)
			current_file.put_string (C_arrow)
			print_once_per_object_mutex_name (a_feature, current_type, current_file)
		end

	print_once_mutex (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print variable containing once mutex of `a_feature'.
			-- In case of one-per-process, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
			a_feature_not_once_per_thread: not a_feature.is_once_per_thread
			multithreaded_mode: multithreaded_mode
		do
			if a_feature.is_once_per_object then
				print_once_per_object_mutex (a_feature)
			elseif a_feature.is_once_per_process then
				print_once_per_process_mutex (a_feature, a_once_kind, a_once_index)
			else
					-- Internal error: this once key is not supported.
				set_fatal_error
				error_handler.report_giaaa_error
			end
		end

	print_once_mutex_try_lock (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to try to lock the once mutex of `a_feature'.
			-- In case of one-per-process, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
			a_feature_not_once_per_thread: not a_feature.is_once_per_thread
			multithreaded_mode: multithreaded_mode
		do
			current_file.put_string (c_ge_mutex_try_lock)
			current_file.put_character ('(')
			print_once_mutex (a_feature, a_once_kind, a_once_index)
			current_file.put_character (')')
		end

	print_once_mutex_lock (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to lock the once mutex of `a_feature'.
			-- In case of one-per-process, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
			a_feature_not_once_per_thread: not a_feature.is_once_per_thread
			multithreaded_mode: multithreaded_mode
		do
			print_indentation
			current_file.put_string (c_ge_mutex_lock)
			current_file.put_character ('(')
			print_once_mutex (a_feature, a_once_kind, a_once_index)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_once_mutex_unlock (a_feature: ET_DYNAMIC_FEATURE; a_once_kind, a_once_index: INTEGER)
			-- Print code to unlock the once mutex of `a_feature'.
			-- In case of one-per-process, `a_feature'
			-- is expected to be a once routine of kind `a_once_kind' with
			-- index `a_once_index' in 'GE_onces' C struct.
			-- Otherwise, `a_once_kind' and `a_once_index' are ignored
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
			a_feature_not_once_per_thread: not a_feature.is_once_per_thread
			multithreaded_mode: multithreaded_mode
		do
			print_indentation
			current_file.put_string (c_ge_mutex_unlock)
			current_file.put_character ('(')
			print_once_mutex (a_feature, a_once_kind, a_once_index)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_once_per_object_global_mutex_lock
			-- Print code to lock the global mutex to protect
			-- creation of once-per-object data.
		require
			multithreaded_mode: multithreaded_mode
		do
			print_indentation
			current_file.put_string (c_ge_mutex_lock)
			current_file.put_character ('(')
			current_file.put_string (c_ge_once_per_object_data_mutex)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_once_per_object_global_mutex_unlock
			-- Print code to unlock the global mutex to protect
			-- creation of once-per-object data.
		require
			multithreaded_mode: multithreaded_mode
		do
			print_indentation
			current_file.put_string (c_ge_mutex_unlock)
			current_file.put_character ('(')
			current_file.put_string (c_ge_once_per_object_data_mutex)
			current_file.put_character (')')
			current_file.put_character (';')
			current_file.put_new_line
		end

feature {NONE} -- Type generation

	sort_types
			-- Sort types in `dynamic_types' so that all types to which a given type
			-- conforms are before this type is the list.
		local
			l_sorter: DS_HASH_TOPOLOGICAL_SORTER [ET_DYNAMIC_PRIMARY_TYPE]
			i, nb: INTEGER
			l_type, l_other_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			nb := dynamic_types.count
			create l_sorter.make (nb)
			from i := 1 until i > nb loop
				l_sorter.force (dynamic_types.item (i))
				i := i + 1
			end
			from i := 1 until i > nb loop
				l_type := dynamic_types.item (i)
				across l_type.conforming_ancestors as i_conforming_ancestor loop
					l_other_type := i_conforming_ancestor
					if l_other_type /= l_type then
						l_sorter.put_relation (l_other_type, l_type)
					end
				end
				l_other_type := current_dynamic_system.none_type
				if l_other_type /= l_type and then l_type.conforms_to_primary_type (l_other_type) then
					l_sorter.put_relation (l_other_type, l_type)
				end
				i := i + 1
			end
			l_sorter.sort
			if l_sorter.has_cycle then
					-- Internal error: this should already have been taken care of
					-- by ET_ANCESTOR_BUILDER (see validity rule VHPR-1).
				set_fatal_error
				error_handler.report_giaaa_error
			elseif attached l_sorter.sorted_items as l_sorted_types then
				dynamic_types := l_sorted_types
			else
				check is_sorted: False end
			end
		end

	print_types (a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print declarations of types of `current_dynamic_system' to `a_file'.
		require
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_dynamic_types: DS_ARRAYED_LIST [ET_DYNAMIC_PRIMARY_TYPE]
			l_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_query: ET_DYNAMIC_FEATURE
			l_attribute_type: ET_DYNAMIC_PRIMARY_TYPE
			j, nb2: INTEGER
			l_expanded_sorter: DS_HASH_TOPOLOGICAL_SORTER [ET_DYNAMIC_PRIMARY_TYPE]
		do
				-- Type with just the type_id attribute 'id'.
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_eif_any_type_name (a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_any)
			a_file.put_new_line
				-- Aliased basic types, as specified in `current_system'.
			print_aliased_character_type_definition (current_system, a_file)
			print_aliased_wide_character_type_definition (current_system, a_file)
			print_aliased_integer_type_definition (current_system, a_file)
			print_aliased_natural_type_definition (current_system, a_file)
			print_aliased_real_type_definition (current_system, a_file)
			print_aliased_double_type_definition (current_system, a_file)
				-- Alive types.
			create l_expanded_sorter.make_default
			l_dynamic_types := dynamic_types
			nb := l_dynamic_types.count
			from i := 1 until i > nb loop
				l_type := l_dynamic_types.item (i)
				if l_type.is_alive then
					a_file.put_character ('/')
					a_file.put_character ('*')
					a_file.put_character (' ')
					a_file.put_string (l_type.base_type.unaliased_to_text)
					a_file.put_character (' ')
					a_file.put_character ('*')
					a_file.put_character ('/')
					a_file.put_new_line
					if l_type.is_expanded then
							-- Keep track of expanded types.
						if not l_expanded_sorter.has (l_type) then
							l_expanded_sorter.force (l_type)
						end
							-- For expanded types, there is no type other than themselves
							-- that conform to them. Therefore  we do not keep the type-id
							-- in each object for those types because if it is used as
							-- static type of an entity there will be no polymorphic call.
							-- A boxed version (containing the type-id) is nevertheless
							-- generated when those objects are attached to entities of
							-- reference types (which might be polymorphic).
						if l_type = current_dynamic_system.boolean_type then
							print_boolean_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.character_8_type then
							print_character_8_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.character_32_type then
							print_character_32_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.integer_8_type then
							print_integer_8_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.integer_16_type then
							print_integer_16_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.integer_32_type then
							print_integer_32_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.integer_64_type then
							print_integer_64_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.natural_8_type then
							print_natural_8_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.natural_16_type then
							print_natural_16_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.natural_32_type then
							print_natural_32_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.natural_64_type then
							print_natural_64_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.real_32_type then
							print_real_32_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.real_64_type then
							print_real_64_type_definition (l_type, a_file)
						elseif l_type = current_dynamic_system.pointer_type then
							print_pointer_type_definition (l_type, a_file)
						else
							print_type_definition (l_type, a_file)
								-- Keep track of dependencies between expanded types.
							l_queries := l_type.queries
							nb2 := l_type.attribute_count
							from j := 1 until j > nb2 loop
								l_query := l_queries.item (j)
								if not attached l_query.result_type_set as l_result_type_set then
										-- Internal error: queries should have a result type.
									set_fatal_error
									error_handler.report_giaaa_error
								else
									l_attribute_type := l_result_type_set.static_type.primary_type
									if l_attribute_type.is_expanded then
										l_expanded_sorter.force_relation (l_attribute_type, l_type)
									end
								end
								j := j + 1
							end
						end
						print_boxed_function (l_type)
						print_boxed_pointer_function (l_type)
					end
					if l_type.base_class.is_type_class then
						print_type_type_definition (l_type, a_file)
					else
						print_boxed_type_definition (l_type, a_file)
					end
					if l_type.has_once_per_object_routines then
						print_once_per_object_data_type_definition (l_type, a_file)
					end
					a_file.put_new_line
				end
				i := i + 1
			end
			l_expanded_sorter.sort
			if l_expanded_sorter.has_cycle then
					-- Internal error: this should already have been taken care of, either by
					-- Eiffel validity rule (see VLEC in ETL2), or by proper handling if ECMA
					-- relaxed this rule (through the introduction of attached types).
				set_fatal_error
				error_handler.report_giaaa_error
			elseif attached l_expanded_sorter.sorted_items as l_expanded_types then
					-- Struct for each expanded type.
				nb := l_expanded_types.count
				from i := 1 until i > nb loop
					l_type := l_expanded_types.item (i)
					print_type_struct (l_type, a_file)
					print_boxed_type_struct (l_type, a_file)
					if l_type.has_once_per_object_routines then
						print_once_per_object_data_struct (l_type, a_file)
					end
					i := i + 1
				end
			else
				check is_sorted: False end
			end
				-- Struct for each non-expanded type.
			nb := l_dynamic_types.count
			from i := 1 until i > nb loop
				l_type := l_dynamic_types.item (i)
				if l_type.is_alive and not l_type.is_expanded then
					print_type_struct (l_type, a_file)
					if l_type.has_once_per_object_routines then
						print_once_per_object_data_struct (l_type, a_file)
					end
				end
				i := i + 1
			end
		end

	print_aliased_character_type_definition (a_universe: ET_UNIVERSE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of aliased type "CHARACTER"
			-- as specified in `a_universe'.
		require
			a_universe_not_void: a_universe /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_class_type: ET_CLASS_TYPE
			l_any_type: ET_CLASS_TYPE
		do
			l_any_type := a_universe.any_type
			l_class_type := a_universe.character_type
			if l_class_type.same_named_type (a_universe.character_8_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.character_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_character)
				a_file.put_character (' ')
				a_file.put_line (c_eif_character_8)
				a_file.put_new_line
			elseif l_class_type.same_named_type (a_universe.character_32_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.character_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_character)
				a_file.put_character (' ')
				a_file.put_line (c_eif_character_32)
				a_file.put_new_line
			end
		end

	print_aliased_wide_character_type_definition (a_universe: ET_UNIVERSE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of aliased type "WIDE_CHARACTER"
			-- as specified in `a_universe'.
		require
			a_universe_not_void: a_universe /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_class_type: ET_CLASS_TYPE
			l_any_type: ET_CLASS_TYPE
		do
			l_any_type := a_universe.any_type
			l_class_type := a_universe.wide_character_type
			if l_class_type.same_named_type (a_universe.character_8_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.wide_character_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_wide_char)
				a_file.put_character (' ')
				a_file.put_line (c_eif_character_8)
				a_file.put_new_line
			elseif l_class_type.same_named_type (a_universe.character_32_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.wide_character_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_wide_char)
				a_file.put_character (' ')
				a_file.put_line (c_eif_character_32)
				a_file.put_new_line
			end
		end

	print_aliased_integer_type_definition (a_universe: ET_UNIVERSE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of aliased type "INTEGER"
			-- as specified in `a_universe'.
		require
			a_universe_not_void: a_universe /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_class_type: ET_CLASS_TYPE
			l_any_type: ET_CLASS_TYPE
		do
			l_any_type := a_universe.any_type
			l_class_type := a_universe.integer_type
			if l_class_type.same_named_type (a_universe.integer_8_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.integer_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_integer)
				a_file.put_character (' ')
				a_file.put_line (c_eif_integer_8)
				a_file.put_new_line
			elseif l_class_type.same_named_type (a_universe.integer_16_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.integer_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_integer)
				a_file.put_character (' ')
				a_file.put_line (c_eif_integer_16)
				a_file.put_new_line
			elseif l_class_type.same_named_type (a_universe.integer_32_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.integer_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_integer)
				a_file.put_character (' ')
				a_file.put_line (c_eif_integer_32)
				a_file.put_new_line
			elseif l_class_type.same_named_type (a_universe.integer_64_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.integer_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_integer)
				a_file.put_character (' ')
				a_file.put_line (c_eif_integer_64)
				a_file.put_new_line
			end
		end

	print_aliased_natural_type_definition (a_universe: ET_UNIVERSE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of aliased type "NATURAL"
			-- as specified in `a_universe'.
		require
			a_universe_not_void: a_universe /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_class_type: ET_CLASS_TYPE
			l_any_type: ET_CLASS_TYPE
		do
			l_any_type := a_universe.any_type
			l_class_type := a_universe.natural_type
			if l_class_type.same_named_type (a_universe.natural_8_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.natural_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_natural)
				a_file.put_character (' ')
				a_file.put_line (c_eif_natural_8)
				a_file.put_new_line
			elseif l_class_type.same_named_type (a_universe.natural_16_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.natural_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_natural)
				a_file.put_character (' ')
				a_file.put_line (c_eif_natural_16)
				a_file.put_new_line
			elseif l_class_type.same_named_type (a_universe.natural_32_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.natural_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_natural)
				a_file.put_character (' ')
				a_file.put_line (c_eif_natural_32)
				a_file.put_new_line
			elseif l_class_type.same_named_type (a_universe.natural_64_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.natural_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_natural)
				a_file.put_character (' ')
				a_file.put_line (c_eif_natural_64)
				a_file.put_new_line
			end
		end

	print_aliased_real_type_definition (a_universe: ET_UNIVERSE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of aliased type "REAL"
			-- as specified in `a_universe'.
		require
			a_universe_not_void: a_universe /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_class_type: ET_CLASS_TYPE
			l_any_type: ET_CLASS_TYPE
		do
			l_any_type := a_universe.any_type
			l_class_type := a_universe.real_type
			if l_class_type.same_named_type (a_universe.real_32_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.real_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_real)
				a_file.put_character (' ')
				a_file.put_line (c_eif_real_32)
				a_file.put_new_line
			elseif l_class_type.same_named_type (a_universe.real_64_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.real_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_real)
				a_file.put_character (' ')
				a_file.put_line (c_eif_real_64)
				a_file.put_new_line
			end
		end

	print_aliased_double_type_definition (a_universe: ET_UNIVERSE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of aliased type "DOUBLE"
			-- as specified in `a_universe'.
		require
			a_universe_not_void: a_universe /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_class_type: ET_CLASS_TYPE
			l_any_type: ET_CLASS_TYPE
		do
			l_any_type := a_universe.any_type
			l_class_type := a_universe.double_type
			if l_class_type.same_named_type (a_universe.real_32_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.double_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_double)
				a_file.put_character (' ')
				a_file.put_line (c_eif_real_32)
				a_file.put_new_line
			elseif l_class_type.same_named_type (a_universe.real_64_type, l_any_type, l_any_type) then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string (tokens.double_class_name.upper_name)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_define)
				a_file.put_character (' ')
				a_file.put_string (c_eif_double)
				a_file.put_character (' ')
				a_file.put_line (c_eif_real_64)
				a_file.put_new_line
			end
		end

	print_boolean_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "BOOLEAN".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_boolean)
		end

	print_character_8_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "CHARACTER_8".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_character_8)
		end

	print_character_32_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "CHARACTER_32".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_character_32)
		end

	print_integer_8_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "INTEGER_8".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_integer_8)
		end

	print_integer_16_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "INTEGER_16".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_integer_16)
		end

	print_integer_32_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "INTEGER_32".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_integer_32)
		end

	print_integer_64_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "INTEGER_64".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_integer_64)
		end

	print_natural_8_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "NATURAL_8".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_natural_8)
		end

	print_natural_16_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "NATURAL_16".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_natural_16)
		end

	print_natural_32_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "NATURAL_32".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_natural_32)
		end

	print_natural_64_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "NATURAL_64".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_natural_64)
		end

	print_real_32_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "REAL_32".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_real_32)
		end

	print_real_64_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "REAL_64".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_real_64)
		end

	print_pointer_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "POINTER".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_pointer)
		end

	print_type_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type "TYPE [X]".
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_define)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_line (c_eif_type)
		end

	print_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the definition of type `a_type'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_typedef)
			a_file.put_character (' ')
			a_file.put_string (c_struct)
			a_file.put_character (' ')
			print_struct_name (a_type, a_file)
			a_file.put_character (' ')
			print_type_name (a_type, a_file)
			a_file.put_character (';')
			a_file.put_new_line
		end

	print_boxed_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the type definition of the boxed version of `a_type'.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_typedef)
			a_file.put_character (' ')
			a_file.put_string (c_struct)
			a_file.put_character (' ')
			print_boxed_struct_name (a_type, a_file)
			a_file.put_character (' ')
			print_boxed_type_name (a_type, a_file)
			a_file.put_character (';')
			a_file.put_new_line
		end

	print_once_per_object_data_type_definition (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the type definition of the once-per-object data of `a_type'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_string (c_typedef)
			a_file.put_character (' ')
			a_file.put_string (c_struct)
			a_file.put_character (' ')
			print_once_per_object_data_struct_name (a_type, a_file)
			a_file.put_character (' ')
			print_once_per_object_data_type_name (a_type, a_file)
			a_file.put_character (';')
			a_file.put_new_line
		end

	print_type_struct (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' declaration of C struct corresponding to `a_type', if_any.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_item_type_set: ET_DYNAMIC_TYPE_SET
			l_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_query: ET_DYNAMIC_FEATURE
			i, nb: INTEGER
			l_empty_struct: BOOLEAN
		do
			if
				not a_type.base_class.is_type_class and
				(not a_type.is_expanded or else not a_type.is_basic)
			then
				l_empty_struct := True
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string ("Struct for type ")
				a_file.put_string (a_type.base_type.unaliased_to_text)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_struct)
				a_file.put_character (' ')
				print_struct_name (a_type, a_file)
				a_file.put_character (' ')
				a_file.put_character ('{')
				a_file.put_new_line
				if not a_type.is_expanded then
					a_file.put_character ('%T')
					a_file.put_string (c_eif_type_index)
					a_file.put_character (' ')
					print_attribute_type_id_name (a_type, a_file)
					a_file.put_character (';')
					a_file.put_new_line
					a_file.put_character ('%T')
					a_file.put_string (c_uint16_t)
					a_file.put_character (' ')
					print_attribute_flags_name (a_type, a_file)
					a_file.put_character (';')
					a_file.put_new_line
					l_empty_struct := False
				end
				if a_type.is_special then
						-- Offset (in bytes) of the first item.
					a_file.put_character ('%T')
					a_file.put_string (c_uint32_t)
					a_file.put_character (' ')
					a_file.put_string (c_offset)
					a_file.put_character (';')
					a_file.put_new_line
					l_empty_struct := False
				end
				l_queries := a_type.queries
				nb := a_type.attribute_count
				from i := 1 until i > nb loop
					l_query := l_queries.item (i)
					if not attached l_query.result_type_set as l_result_type_set then
							-- Internal error: queries should have a result type.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						a_file.put_character ('%T')
						print_type_declaration (l_result_type_set.static_type.primary_type, a_file)
						a_file.put_character (' ')
						print_attribute_name (l_query, a_type, a_file)
						a_file.put_character (';')
						a_file.put_character (' ')
						a_file.put_character ('/')
						a_file.put_character ('*')
						a_file.put_character (' ')
						a_file.put_string (l_query.static_feature.name.name)
						a_file.put_character (' ')
						a_file.put_character ('*')
						a_file.put_character ('/')
						a_file.put_new_line
						l_empty_struct := False
					end
					i := i + 1
				end
				if a_type.has_once_per_object_routines then
					a_file.put_character ('%T')
					print_once_per_object_data_type_name (a_type, a_file)
					a_file.put_character ('*')
					a_file.put_character (' ')
					print_attribute_onces_name (a_type, a_file)
					a_file.put_character (';')
					a_file.put_new_line
				end
				if attached {ET_DYNAMIC_SPECIAL_TYPE} a_type as l_special_type then
						-- We use the "struct hack" to represent SPECIAL
						-- object header. The last member of the struct
						-- is an array of size 1, but we malloc the needed
						-- space when creating the SPECIAL object. We use
						-- an array of size 1 because some compilers don't
						-- like having an array of size 0 here. Note that
						-- the "struct hack" is superseded by the concept
						-- of "flexible array member" in ISO C 99.
					a_file.put_character ('%T')
					l_item_type_set := l_special_type.item_type_set
					print_type_declaration (l_item_type_set.static_type.primary_type, a_file)
					a_file.put_character (' ')
					print_attribute_special_item_name (l_special_type, a_file)
					a_file.put_character ('[')
					a_file.put_character ('1')
					a_file.put_character (']')
					a_file.put_character (';')
					a_file.put_character (' ')
					a_file.put_character ('/')
					a_file.put_character ('*')
					a_file.put_character (' ')
					a_file.put_character ('i')
					a_file.put_character ('t')
					a_file.put_character ('e')
					a_file.put_character ('m')
					a_file.put_character (' ')
					a_file.put_character ('*')
					a_file.put_character ('/')
					a_file.put_new_line
					l_empty_struct := False
				elseif attached {ET_DYNAMIC_TUPLE_TYPE} a_type as l_tuple_type then
					l_item_type_sets := l_tuple_type.item_type_sets
					nb := l_item_type_sets.count
					from i := 1 until i > nb loop
						a_file.put_character ('%T')
						print_type_declaration (l_item_type_sets.item (i).static_type.primary_type, a_file)
						a_file.put_character (' ')
						print_attribute_tuple_item_name (i, l_tuple_type, a_file)
						a_file.put_character (';')
						a_file.put_new_line
						l_empty_struct := False
						i := i + 1
					end
				end
				if l_empty_struct then
						-- Add a dummy field so that the struct is not empty (not allowed in C99).
					a_file.put_character ('%T')
					a_file.put_string (c_char)
					a_file.put_character (' ')
					a_file.put_string ("dummy")
					a_file.put_character (';')
					a_file.put_new_line
				end
				a_file.put_character ('}')
				a_file.put_character (';')
				a_file.put_new_line
				a_file.put_new_line
			end
		end

	print_boxed_type_struct (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' declaration of C struct corresponding to boxed version of `a_type', if_any.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if a_type.is_expanded then
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_string ("Struct for boxed version of type ")
				a_file.put_string (a_type.base_type.unaliased_to_text)
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_string (c_struct)
				a_file.put_character (' ')
				print_boxed_struct_name (a_type, a_file)
				a_file.put_character (' ')
				a_file.put_character ('{')
				a_file.put_new_line
				a_file.put_character ('%T')
				a_file.put_string (c_eif_type_index)
				a_file.put_character (' ')
				print_attribute_type_id_name (a_type, a_file)
				a_file.put_character (';')
				a_file.put_new_line
				a_file.put_character ('%T')
				a_file.put_string (c_uint16_t)
				a_file.put_character (' ')
				print_attribute_flags_name (a_type, a_file)
				a_file.put_character (';')
				a_file.put_new_line
				a_file.put_character ('%T')
				print_type_declaration (a_type, a_file)
				a_file.put_character ('*')
				a_file.put_character (' ')
				print_boxed_attribute_pointer_name (a_type, a_file)
				a_file.put_character (';')
				a_file.put_character (' ')
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_character ('p')
				a_file.put_character ('o')
				a_file.put_character ('i')
				a_file.put_character ('n')
				a_file.put_character ('t')
				a_file.put_character ('e')
				a_file.put_character ('r')
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_character ('%T')
				print_type_declaration (a_type, a_file)
				a_file.put_character (' ')
				print_boxed_attribute_item_name (a_type, a_file)
				a_file.put_character (';')
				a_file.put_character (' ')
				a_file.put_character ('/')
				a_file.put_character ('*')
				a_file.put_character (' ')
				a_file.put_character ('i')
				a_file.put_character ('t')
				a_file.put_character ('e')
				a_file.put_character ('m')
				a_file.put_character (' ')
				a_file.put_character ('*')
				a_file.put_character ('/')
				a_file.put_new_line
				a_file.put_character ('}')
				a_file.put_character (';')
				a_file.put_new_line
				a_file.put_new_line
			end
		end

	print_once_per_object_data_struct (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' declaration of C struct corresponding to once-per-object data of `a_type'.
			-- Note that the data for once-per-object is not embedded in the object struct
			-- put stored in a different struct so that we can easy process it independently
			-- of the object's attributes. It can also have its own GC dispose function
			-- (to free mutexes for example).
		require
			a_type_not_void: a_type /= Void
			a_type_has_once_per_object_routines: a_type.has_once_per_object_routines
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_query: ET_DYNAMIC_FEATURE
			l_procedures: ET_DYNAMIC_FEATURE_LIST
			l_procedure: ET_DYNAMIC_FEATURE
			i, nb: INTEGER
		do
			a_file.put_character ('/')
			a_file.put_character ('*')
			a_file.put_character (' ')
			a_file.put_string ("Struct for once-per-object data of type ")
			a_file.put_string (a_type.base_type.unaliased_to_text)
			a_file.put_character (' ')
			a_file.put_character ('*')
			a_file.put_character ('/')
			a_file.put_new_line
			a_file.put_string (c_struct)
			a_file.put_character (' ')
			print_once_per_object_data_struct_name (a_type, a_file)
			a_file.put_character (' ')
			a_file.put_character ('{')
			a_file.put_new_line
			l_queries := a_type.queries
			i := a_type.attribute_count + 1
			nb := l_queries.count
			from until i > nb loop
				l_query := l_queries.item (i)
				if not l_query.is_once_per_object then
					-- Nothing to be done.
				elseif not attached l_query.result_type_set as l_result_type_set then
						-- Internal error: queries should have a result type.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					a_file.put_character ('%T')
					a_file.put_string (c_unsigned)
					a_file.put_character (' ')
					a_file.put_string (c_char)
					a_file.put_character (' ')
					print_once_per_object_status_name (l_query, a_type, a_file)
					a_file.put_character (';')
					a_file.put_character (' ')
					a_file.put_character ('/')
					a_file.put_character ('*')
					a_file.put_character (' ')
					a_file.put_string (l_query.static_feature.name.name)
					a_file.put_character (' ')
					a_file.put_character ('*')
					a_file.put_character ('/')
					a_file.put_new_line
					a_file.put_character ('%T')
					print_type_declaration (l_result_type_set.static_type.primary_type, a_file)
					a_file.put_character (' ')
					print_once_per_object_value_name (l_query, a_type, a_file)
					a_file.put_character (';')
					a_file.put_new_line
					a_file.put_character ('%T')
					a_file.put_string (c_eif_reference)
					a_file.put_character (' ')
					print_once_per_object_exception_name (l_query, a_type, a_file)
					a_file.put_character (';')
					a_file.put_new_line
					if multithreaded_mode then
						a_file.put_character ('%T')
						a_file.put_string (c_eif_pointer)
						a_file.put_character (' ')
						print_once_per_object_mutex_name (l_query, a_type, a_file)
						a_file.put_character (';')
						a_file.put_new_line
					end
				end
				i := i + 1
			end
			l_procedures := a_type.procedures
			nb := l_procedures.count
			from i := 1 until i > nb loop
				l_procedure := l_procedures.item (i)
				if l_procedure.is_once_per_object then
					a_file.put_character ('%T')
					a_file.put_string (c_unsigned)
					a_file.put_character (' ')
					a_file.put_string (c_char)
					a_file.put_character (' ')
					print_once_per_object_status_name (l_procedure, a_type, a_file)
					a_file.put_character (';')
					a_file.put_character (' ')
					a_file.put_character ('/')
					a_file.put_character ('*')
					a_file.put_character (' ')
					a_file.put_string (l_procedure.static_feature.name.name)
					a_file.put_character (' ')
					a_file.put_character ('*')
					a_file.put_character ('/')
					a_file.put_new_line
					a_file.put_character ('%T')
					a_file.put_string (c_eif_reference)
					a_file.put_character (' ')
					print_once_per_object_exception_name (l_procedure, a_type, a_file)
					a_file.put_character (';')
					a_file.put_new_line
					if multithreaded_mode then
						a_file.put_character ('%T')
						a_file.put_string (c_eif_pointer)
						a_file.put_character (' ')
						print_once_per_object_mutex_name (l_procedure, a_type, a_file)
						a_file.put_character (';')
						a_file.put_new_line
					end
				end
				i := i + 1
			end
			a_file.put_character ('}')
			a_file.put_character (';')
			a_file.put_new_line
			a_file.put_new_line
		end

	print_types_array
			-- Print 'GE_types' array to `current_file' and its declaration to `header_file'.
		local
			l_dynamic_types: DS_ARRAYED_LIST [ET_DYNAMIC_PRIMARY_TYPE]
			l_type, l_other_type: ET_DYNAMIC_PRIMARY_TYPE
			l_meta_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_attached_type: ET_DYNAMIC_TYPE
			i, j, nb: INTEGER
			l_dispose_procedure: detachable ET_DYNAMIC_FEATURE
			l_ancestors: DS_ARRAYED_LIST [ET_DYNAMIC_PRIMARY_TYPE]
			l_ancestor_count: INTEGER
			l_sorter: DS_QUICK_SORTER [ET_DYNAMIC_PRIMARY_TYPE]
			l_attribute: ET_DYNAMIC_FEATURE
			l_attribute_count: INTEGER
			l_comma_needed: BOOLEAN
			l_generic_parameter_count: INTEGER
			l_parameter_type: ET_TYPE
			l_dynamic_type: ET_DYNAMIC_TYPE
			l_name: STRING
		do
			l_dynamic_types := dynamic_types
			nb := l_dynamic_types.count
			current_file.put_string (c_eif_type)
			current_file.put_character (' ')
			current_file.put_string (c_ge_types)
			current_file.put_character ('[')
			current_file.put_integer (nb + 1)
			current_file.put_character (']')
			current_file.put_character ('[')
			current_file.put_integer (2)
			current_file.put_character (']')
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
				-- Dummy type at index 0.
			current_file.put_character ('{')
			current_file.put_character ('{')
			current_file.put_integer (0)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_integer (0)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_string (c_eif_false)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_integer (0)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_integer (0)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_integer (0)
			current_file.put_character ('}')
			current_file.put_character (',')
			current_file.put_character ('{')
			current_file.put_integer (0)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_integer (0)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_string (c_eif_false)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_integer (0)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_integer (0)
			current_file.put_character (',')
			current_file.put_character (' ')
			current_file.put_integer (0)
			current_file.put_character ('}')
			current_file.put_character ('}')
			current_file.put_character (',')
			current_file.put_new_line
-- TODO: here we might include types that are used in our system!
			from i := 1 until i > nb loop
				l_type := l_dynamic_types.item (i)
				current_file.put_character ('{')
				current_file.put_character ('{')
					-- id.
				l_meta_type := l_type.meta_type
				if l_meta_type /= Void then
					current_file.put_integer (l_meta_type.id)
				else
					current_file.put_integer (0)
				end
				current_file.put_character (',')
				current_file.put_character (' ')
					-- type_id.
				current_file.put_integer (l_type.id)
				current_file.put_character (',')
				current_file.put_character (' ')
					-- is_special.
				if l_type.is_special then
					current_file.put_string (c_eif_true)
				else
					current_file.put_string (c_eif_false)
				end
				current_file.put_character (',')
				current_file.put_character (' ')
					-- dispose.
				l_dispose_procedure := Void
				dispose_procedures.search (l_type)
				if dispose_procedures.found then
					l_dispose_procedure := dispose_procedures.found_item
				end
				if l_dispose_procedure /= Void then
					current_file.put_character ('&')
					print_routine_name (l_dispose_procedure, l_type, current_file)
				else
					current_file.put_integer (0)
				end
					-- Attribute `internal_name'.
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_integer (0)
					-- Attribute `internal_name_32'.
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_integer (0)
				current_file.put_character ('}')
				current_file.put_character (',')
				current_file.put_character ('{')
					-- id.
				l_attached_type := l_type.attached_type
				if l_attached_type /= Void then
					l_meta_type := l_attached_type.meta_type
				else
					l_meta_type := Void
				end
				if l_meta_type /= Void then
					current_file.put_integer (l_meta_type.id)
				else
					current_file.put_integer (0)
				end
				current_file.put_character (',')
				current_file.put_character (' ')
					-- type_id.
				current_file.put_integer ((1 |<< 16) | l_type.id)
				current_file.put_character (',')
				current_file.put_character (' ')
					-- is_special.
				if l_type.is_special then
					current_file.put_string (c_eif_true)
				else
					current_file.put_string (c_eif_false)
				end
				current_file.put_character (',')
				current_file.put_character (' ')
					-- dispose.
				l_dispose_procedure := Void
				dispose_procedures.search (l_type)
				if dispose_procedures.found then
					l_dispose_procedure := dispose_procedures.found_item
				end
				if l_dispose_procedure /= Void then
					current_file.put_character ('&')
					print_routine_name (l_dispose_procedure, l_type, current_file)
				else
					current_file.put_integer (0)
				end
					-- Attribute `internal_name'.
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_integer (0)
					-- Attribute `internal_name_32'.
				current_file.put_character (',')
				current_file.put_character (' ')
				current_file.put_integer (0)
				current_file.put_character ('}')
				current_file.put_character ('}')
				if i /= nb then
					current_file.put_character (',')
				end
				current_file.put_new_line
				i := i + 1
			end
			current_file.put_character ('}')
			current_file.put_character (';')
			current_file.put_new_line
			current_file.put_new_line

			create l_ancestors.make (100)
			create l_sorter.make (dynamic_primary_type_comparator)
			nb := l_dynamic_types.count
			from i := 1 until i > nb loop
				l_type := l_dynamic_types.item (i)
					-- Ancestors.
				if type_info_ancestors_used then
					if l_type /= current_dynamic_system.none_type then
						l_ancestors.append_last (l_type.conforming_ancestors)
						l_ancestor_count := l_ancestors.count
						if l_ancestor_count > 0 then
							l_ancestors.sort (l_sorter)
							from j := 1 until j > l_ancestor_count loop
								l_other_type := l_ancestors.item (j)
								current_file.put_string ("GE_ancestor T")
								current_file.put_integer (l_type.id)
								current_file.put_string ("xx")
								current_file.put_integer (l_other_type.id)
								current_file.put_string (" = {")
								current_file.put_integer (l_other_type.id)
								current_file.put_line (", EIF_TRUE, 0};")
								j := j + 1
							end
							current_file.put_string ("GE_ancestor* T")
							current_file.put_integer (l_type.id)
							current_file.put_string ("xx[")
							current_file.put_integer (l_ancestor_count)
							current_file.put_string ("] = {")
							from j := 1 until j > l_ancestor_count loop
								l_other_type := l_ancestors.item (j)
								current_file.put_string ("&T")
								current_file.put_integer (l_type.id)
								current_file.put_string ("xx")
								current_file.put_integer (l_other_type.id)
								if j /= l_ancestor_count then
									current_file.put_string (", ")
								end
								j := j + 1
							end
							current_file.put_line ("};")
						end
						l_ancestors.wipe_out
					end
				end
					-- Attributes.
				if type_info_attributes_used and l_type.is_alive then
					l_attribute_count := l_type.attribute_count
					if l_attribute_count > 0 then
						from j := 1 until j > l_attribute_count loop
							l_comma_needed := False
							l_attribute := l_type.queries.item (j)
							current_file.put_string ("GE_attribute T")
							current_file.put_integer (l_type.id)
							current_file.put_string ("aa")
							current_file.put_integer (j)
							current_file.put_string (" = {")
								-- name.
							if type_info_attribute_name_used then
								print_escaped_string (l_attribute.static_feature.lower_name)
								l_comma_needed := True
							end
								-- type_id.
							if type_info_attribute_type_id_used then
								if l_comma_needed then
									current_file.put_character (',')
									current_file.put_character (' ')
								end
								if attached l_attribute.result_type_set as l_result_type_set then
									current_file.put_integer (l_result_type_set.static_type.type_id)
								else
										-- Internal error: attributes have a result type.
									set_fatal_error
									error_handler.report_giaaa_error
									current_file.put_character ('0')
								end
								l_comma_needed := True
							end
								-- offset.
							if type_info_attribute_offset_used then
								if l_comma_needed then
									current_file.put_character (',')
									current_file.put_character (' ')
								end
									-- Note: if `offsetof' is not supported, then we can use: ((int)&(((T317*) 0)->a2))
									-- See: http://stackoverflow.com/questions/142016/c-c-structure-offset
								current_file.put_string (c_offsetof)
								current_file.put_character ('(')
								if l_type.is_basic then
									print_boxed_type_name (l_type, current_file)
									current_file.put_character (',')
									current_file.put_character (' ')
									print_boxed_attribute_item_name (l_type, current_file)
								else
									print_type_name (l_type, current_file)
									current_file.put_character (',')
									current_file.put_character (' ')
									print_attribute_name (l_attribute, l_type, current_file)
								end
								current_file.put_character (')')
							end
							current_file.put_character ('}')
							current_file.put_character (';')
							current_file.put_new_line
							j := j + 1
						end
					end
					if attached {ET_DYNAMIC_SPECIAL_TYPE} l_type as l_special_type then
								-- One more attribute: 'item'.
						l_attribute_count := l_attribute_count + 1
						l_comma_needed := False
						current_file.put_string ("GE_attribute T")
						current_file.put_integer (l_type.id)
						current_file.put_string ("aa")
						current_file.put_integer (l_attribute_count)
						current_file.put_string (" = {")
							-- name.
						if type_info_attribute_name_used then
							print_escaped_string (tokens.item_feature_name.lower_name)
							l_comma_needed := True
						end
							-- type_id.
						if type_info_attribute_type_id_used then
							if l_comma_needed then
								current_file.put_character (',')
								current_file.put_character (' ')
							end
							current_file.put_integer (l_special_type.item_type_set.static_type.type_id)
							l_comma_needed := True
						end
							-- offset.
						if type_info_attribute_offset_used then
							if l_comma_needed then
								current_file.put_character (',')
								current_file.put_character (' ')
							end
								-- Note: if `offsetof' is not supported, then we can use: ((int)&(((T317*) 0)->a2))
								-- See: http://stackoverflow.com/questions/142016/c-c-structure-offset
							current_file.put_string (c_offsetof)
							current_file.put_character ('(')
							print_type_name (l_type, current_file)
							current_file.put_character (',')
							current_file.put_character (' ')
							print_attribute_special_item_name (l_type, current_file)
							current_file.put_character (')')
						end
						current_file.put_character ('}')
						current_file.put_character (';')
						current_file.put_new_line
					end
					if l_attribute_count > 0 then
						current_file.put_string ("GE_attribute* T")
						current_file.put_integer (l_type.id)
						current_file.put_string ("aa[")
						current_file.put_integer (l_attribute_count)
						current_file.put_string ("] = {")
						from j := 1 until j > l_attribute_count loop
							current_file.put_string ("&T")
							current_file.put_integer (l_type.id)
							current_file.put_string ("aa")
							current_file.put_integer (j)
							if j /= l_attribute_count then
								current_file.put_string (", ")
							end
							j := j + 1
						end
						current_file.put_line ("};")
					end
				end
					-- Generic parameters.
				if type_info_generic_parameters_used then
					if attached l_type.base_type.actual_parameters as l_parameters then
						l_generic_parameter_count := l_parameters.count
						if l_generic_parameter_count > 0 then
							current_file.put_string ("EIF_ENCODED_TYPE T")
							current_file.put_integer (l_type.id)
							current_file.put_string ("gen[")
							current_file.put_integer (l_generic_parameter_count)
							current_file.put_string ("] = {")
							from j := 1 until j > l_generic_parameter_count loop
								l_parameter_type := l_parameters.type (j)
								l_dynamic_type := current_dynamic_system.dynamic_type (l_parameter_type, l_type.base_type)
								current_file.put_integer (l_dynamic_type.type_id)
								if j /= l_generic_parameter_count then
									current_file.put_string (", ")
								end
								j := j + 1
							end
							current_file.put_line ("};")
						end
					end
				end
				i := i + 1
			end
			current_file.put_new_line
			current_file.put_string ("GE_type_info")
			current_file.put_character (' ')
			current_file.put_string (c_ge_type_infos)
			current_file.put_character ('[')
			current_file.put_integer (nb + 1)
			current_file.put_character (']')
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
				-- Dummy type at index 0.
			current_file.put_character ('{')
				-- type_id.
			current_file.put_character ('0')
			current_file.put_character (',')
			current_file.put_character (' ')
				-- flags.
			current_file.put_character ('0')
			if type_info_generator_used then
				current_file.put_character (',')
				current_file.put_character (' ')
					-- generator.
				current_file.put_character ('0')
			end
			if type_info_name_used then
				current_file.put_character (',')
				current_file.put_character (' ')
					-- name.
				current_file.put_character ('0')
			end
			if type_info_generic_parameters_used then
				current_file.put_character (',')
				current_file.put_character (' ')
					-- generic_parameters.
				current_file.put_character ('0')
				current_file.put_character (',')
				current_file.put_character (' ')
					-- generic_parameter_count.
				current_file.put_character ('0')
			end
			if type_info_ancestors_used then
				current_file.put_character (',')
				current_file.put_character (' ')
					-- ancestors.
				current_file.put_character ('0')
				current_file.put_character (',')
				current_file.put_character (' ')
					-- ancestor_count.
				current_file.put_character ('0')
			end
			if type_info_attributes_used then
				current_file.put_character (',')
				current_file.put_character (' ')
					-- attributes.
				current_file.put_character ('0')
				current_file.put_character (',')
				current_file.put_character (' ')
					-- attribute_count.
				current_file.put_character ('0')
			end
			if type_info_object_size_used then
				current_file.put_character (',')
				current_file.put_character (' ')
					-- object_size.
				current_file.put_character ('0')
			end
			current_file.put_character (',')
			current_file.put_character (' ')
				-- new_instance.
			current_file.put_character ('0')
			current_file.put_character (',')
			current_file.put_character (' ')
				-- dispose.
			current_file.put_character ('0')
			current_file.put_character ('}')
			current_file.put_character (',')
			current_file.put_new_line
			from i := 1 until i > nb loop
				l_type := l_dynamic_types.item (i)
				current_file.put_character ('{')
					-- type_id.
				current_file.put_integer (l_type.id)
				current_file.put_character (',')
				current_file.put_character (' ')
					-- flags.
				if l_type.is_special then
					current_file.put_string (c_ge_type_flag_special)
				elseif l_type.is_tuple then
					current_file.put_string (c_ge_type_flag_tuple)
				elseif l_type = current_dynamic_system.none_type then
					current_file.put_string (c_ge_type_flag_none)
				elseif l_type.is_expanded then
					if l_type = current_dynamic_system.boolean_type then
						current_file.put_string (c_ge_type_flag_boolean)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.character_8_type then
						current_file.put_string (c_ge_type_flag_character_8)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.character_32_type then
						current_file.put_string (c_ge_type_flag_character_32)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.integer_8_type then
						current_file.put_string (c_ge_type_flag_integer_8)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.integer_16_type then
						current_file.put_string (c_ge_type_flag_integer_16)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.integer_32_type then
						current_file.put_string (c_ge_type_flag_integer_32)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.integer_64_type then
						current_file.put_string (c_ge_type_flag_integer_64)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.natural_8_type then
						current_file.put_string (c_ge_type_flag_natural_8)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.natural_16_type then
						current_file.put_string (c_ge_type_flag_natural_16)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.natural_32_type then
						current_file.put_string (c_ge_type_flag_natural_32)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.natural_64_type then
						current_file.put_string (c_ge_type_flag_natural_64)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.pointer_type then
						current_file.put_string (c_ge_type_flag_pointer)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.real_32_type then
						current_file.put_string (c_ge_type_flag_real_32)
						current_file.put_character ('|')
					elseif l_type = current_dynamic_system.real_64_type then
						current_file.put_string (c_ge_type_flag_real_64)
						current_file.put_character ('|')
					end
					current_file.put_string (c_ge_type_flag_expanded)
				elseif l_type.base_class.is_deferred then
					current_file.put_string (c_ge_type_flag_deferred)
				else
					current_file.put_character ('0')
				end
				if type_info_generator_used then
					current_file.put_character (',')
					current_file.put_character (' ')
						-- generator.
					print_escaped_string (l_type.base_class.upper_name)
				end
				if type_info_name_used then
					current_file.put_character (',')
					current_file.put_character (' ')
						-- name.
						-- Generate a '!' character first, in case we are interested
						-- in the attached version of the type. Otherwise this first
						-- character will be skipped at runtime (see ge_types.c).
					l_name := "!" + l_type.base_type.runtime_name_to_text
					print_escaped_string (l_name)
				end
				if type_info_generic_parameters_used then
					current_file.put_character (',')
					current_file.put_character (' ')
						-- generic_parameters.
					if attached l_type.base_type.actual_parameters as l_parameters then
						l_generic_parameter_count := l_parameters.count
					else
						l_generic_parameter_count := 0
					end
					if l_generic_parameter_count > 0 then
						current_file.put_string ("T")
						current_file.put_integer (l_type.id)
						current_file.put_string ("gen")
					else
						current_file.put_character ('0')
					end
					current_file.put_character (',')
					current_file.put_character (' ')
						-- generic_parameter_count.
					current_file.put_integer (l_generic_parameter_count)
				end
				if type_info_ancestors_used then
					current_file.put_character (',')
					current_file.put_character (' ')
						-- ancestors.
					if i /= nb or else l_type /= current_dynamic_system.none_type then
						l_ancestor_count := l_type.conforming_ancestors.count
					else
						l_ancestor_count := 0
					end
					if l_ancestor_count > 0 then
						current_file.put_string ("T")
						current_file.put_integer (l_type.id)
						current_file.put_string ("xx")
					else
						current_file.put_character ('0')
					end
					current_file.put_character (',')
					current_file.put_character (' ')
						-- ancestor_count.
					current_file.put_integer (l_ancestor_count)
				end
				if type_info_attributes_used then
					current_file.put_character (',')
					current_file.put_character (' ')
						-- attributes.
					if l_type.is_alive then
						l_attribute_count := l_type.attribute_count
						if l_type.is_special then
								-- One more attribute: 'item'.
							l_attribute_count := l_attribute_count + 1
						end
					else
						l_attribute_count := 0
					end
					if l_attribute_count > 0 then
						current_file.put_string ("T")
						current_file.put_integer (l_type.id)
						current_file.put_string ("aa")
					else
						current_file.put_character ('0')
					end
					current_file.put_character (',')
					current_file.put_character (' ')
						-- attribute_count.
					current_file.put_integer (l_attribute_count)
				end
				if type_info_object_size_used then
					current_file.put_character (',')
					current_file.put_character (' ')
						-- object_size.
					if l_type.is_alive then
						current_file.put_string (c_sizeof)
						current_file.put_character ('(')
						if l_type.is_basic then
							print_boxed_type_name (l_type, current_file)
						else
							print_type_name (l_type, current_file)
						end
						current_file.put_character (')')
					else
						current_file.put_character ('0')
					end
				end
				current_file.put_character (',')
				current_file.put_character (' ')
					-- new_instance.
				if not l_type.is_alive then
					current_file.put_character ('0')
				elseif l_type.is_expanded then
					current_file.put_character ('0')
				elseif not current_dynamic_system.is_new_instance_type (l_type) then
						-- Raise an exception and return Void when the result type has not been
						-- specified as a type which can have instances created by 'new_instance'.
					current_file.put_character ('0')
				elseif l_type.base_class.is_type_class then
						-- Cannot have two instances of class TYPE representing the same Eiffel type.
					current_file.put_character ('0')
				else
					current_file.put_string ("(EIF_REFERENCE (*)())")
					current_file.put_character ('&')
					current_file.put_string (c_ge_new)
					current_file.put_integer (l_type.id)
				end
				current_file.put_character (',')
				current_file.put_character (' ')
					-- dispose.
				l_dispose_procedure := Void
				dispose_procedures.search (l_type)
				if dispose_procedures.found then
					l_dispose_procedure := dispose_procedures.found_item
				end
				if l_dispose_procedure /= Void then
					current_file.put_character ('&')
					print_routine_name (l_dispose_procedure, l_type, current_file)
				else
					current_file.put_character ('0')
				end
				current_file.put_character ('}')
				if i /= nb then
					current_file.put_character (',')
				end
				current_file.put_new_line
				i := i + 1
			end
			current_file.put_character ('}')
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_type_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of `a_type' to `a_file'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('T')
				a_file.put_integer (a_type.id)
			else
-- TODO: long names
				short_names := True
				print_type_name (a_type, a_file)
				short_names := False
			end
		end

	print_boxed_type_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of boxed version of `a_type' to `a_file'.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('T')
				if a_type.is_expanded then
					a_file.put_character ('b')
				end
				a_file.put_integer (a_type.id)
			else
-- TODO: long names
				short_names := True
				print_boxed_type_name (a_type, a_file)
				short_names := False
			end
		end

	print_once_per_object_data_type_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of once-per-object data of `a_type' to `a_file'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('T')
				a_file.put_character ('o')
				a_file.put_integer (a_type.id)
			else
-- TODO: long names
				short_names := True
				print_once_per_object_data_type_name (a_type, a_file)
				short_names := False
			end
		end

	print_struct_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of C struct corresponding to `a_type' to `a_file'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('S')
				a_file.put_integer (a_type.id)
			else
-- TODO: long names
				short_names := True
				print_struct_name (a_type, a_file)
				short_names := False
			end
		end

	print_boxed_struct_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of C struct corresponding to the boxed version of `a_type' to `a_file'.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('S')
				if a_type.is_expanded then
					a_file.put_character ('b')
				end
				a_file.put_integer (a_type.id)
			else
-- TODO: long names
				short_names := True
				print_boxed_struct_name (a_type, a_file)
				short_names := False
			end
		end

	print_once_per_object_data_struct_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of C struct corresponding to the once-per-object data of `a_type' to `a_file'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('S')
				a_file.put_character ('o')
				a_file.put_integer (a_type.id)
			else
-- TODO: long names
				short_names := True
				print_once_per_object_data_struct_name (a_type, a_file)
				short_names := False
			end
		end

	print_eif_any_type_name (a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of type 'EIF_ANY' to `a_file'.
		require
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('T')
				a_file.put_character ('0')
			else
-- TODO: long names
				short_names := True
				print_eif_any_type_name (a_file)
				short_names := False
			end
		end

	print_type_declaration (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print declaration of `a_type' to `a_file'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if a_type.is_expanded then
				print_type_name (a_type, a_file)
			else
				print_eif_any_type_name (a_file)
				a_file.put_character ('*')
			end
		end

	print_boxed_type_declaration (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print declaration of boxed version of `a_type' to `a_file'.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			print_eif_any_type_name (a_file)
			a_file.put_character ('*')
		end

	print_type_cast (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print type cast of `a_type' to `a_file'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_character ('(')
			print_type_name (a_type, a_file)
			if not a_type.is_expanded then
				a_file.put_character ('*')
			end
			a_file.put_character (')')
		end

	print_boxed_type_cast (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print type cast of boxed version of `a_type' to `a_file'.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_character ('(')
			print_boxed_type_name (a_type, a_file)
			a_file.put_character ('*')
			a_file.put_character (')')
		end

	print_declaration_type_cast (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print type cast of type declaration of `a_type' to `a_file'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_character ('(')
			print_type_declaration (a_type, a_file)
			a_file.put_character (')')
		end

	print_declaration_type_cast_from_c (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print type cast of type declaration of `a_type' to `a_file'.
			-- Convert C boolean value to Eiffel boolean value if needed.
			--
			-- Note that C boolean operators such as '==', '!=', '&&', '||',
			-- '!', '<', '<=', '>' and '>=' return 1 if true and 0 if false.
			-- See https://stackoverflow.com/questions/18097922/return-value-of-operator-in-c.
			-- There is no need to conversion in that case. So a call to
			-- `print_declaration_type_cast' is enough.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if a_type.is_boolean then
				a_file.put_string (c_eif_test)
			else
				print_declaration_type_cast (a_type, a_file)
			end
		end

feature {NONE} -- Default initialization values generation

	print_default_declarations
			-- Print default initialization declaration of each type
			-- to `current_file' and their signature to `header_file'.
		local
			l_dynamic_types: DS_ARRAYED_LIST [ET_DYNAMIC_PRIMARY_TYPE]
			l_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
		do
			l_dynamic_types := dynamic_types
			nb := l_dynamic_types.count
			from i := 1 until i > nb loop
				l_type := l_dynamic_types.item (i)
				if l_type.is_alive then
					if not l_type.base_class.is_type_class then
						print_default_declaration (l_type)
					end
				end
				i := i + 1
			end
		end

	print_default_declaration (a_type: ET_DYNAMIC_PRIMARY_TYPE)
			-- Print default initialization declaration of `a_type'
			-- to `current_file' and their signature to `header_file'.
		require
			a_type_not_void: a_type /= Void
		do
			header_file.put_string (c_extern)
			header_file.put_character (' ')
			print_type_name (a_type, header_file)
			print_type_name (a_type, current_file)
			header_file.put_character (' ')
			current_file.put_character (' ')
			print_default_name (a_type, header_file)
			print_default_name (a_type, current_file)
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
			print_default_object_value (a_type, current_file)
			header_file.put_character (';')
			current_file.put_character (';')
			header_file.put_new_line
			current_file.put_new_line
		end

	print_default_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' name of default initialization for object of type `a_type'.
			-- (In case of expanded types being involved, this default initialization
			-- does not take into account possible calls to 'default_create' which need
			-- to be applied subsequently.)
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_string (c_ge_default)
				a_file.put_integer (a_type.id)
			else
-- TODO: long names
				short_names := True
				print_default_name (a_type, a_file)
				short_names := False
			end
		end

	print_default_object_value (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' default initialization value for objects of type `a_type'.
			-- (In case of expanded types being involved, this default initialization
			-- does not take into account possible calls to 'default_create' which need
			-- to be applied subsequently.)
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			i, nb: INTEGER
			l_item_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			l_queries: ET_DYNAMIC_FEATURE_LIST
			l_query: ET_DYNAMIC_FEATURE
			l_empty_struct: BOOLEAN
		do
			if not a_type.is_expanded or else not a_type.is_basic then
				l_empty_struct := True
				a_file.put_character ('{')
				if not a_type.is_expanded then
						-- Type id.
					a_file.put_integer (a_type.id)
						-- Flags.
					a_file.put_character (',')
					a_file.put_character ('0')
					l_empty_struct := False
				end
				if a_type.is_special then
						-- Offset (in bytes) to the first item.
					if not l_empty_struct then
						a_file.put_character (',')
					end
						-- Note: if `offsetof' is not supported, then we can use: ((int)&(((T317*) 0)->a2))
						-- See: http://stackoverflow.com/questions/142016/c-c-structure-offset
					a_file.put_string (c_offsetof)
					a_file.put_character ('(')
					print_type_name (a_type, current_file)
					a_file.put_character (',')
					a_file.put_character (' ')
					print_attribute_special_item_name (a_type, current_file)
					a_file.put_character (')')
					l_empty_struct := False
				end
					-- Attributes.
				l_queries := a_type.queries
				nb := a_type.attribute_count
				from i := 1 until i > nb loop
					l_query := l_queries.item (i)
					if not attached l_query.result_type_set as l_result_type_set then
							-- Internal error: queries should have a result type.
						set_fatal_error
						error_handler.report_giaaa_error
					else
						if not l_empty_struct then
							a_file.put_character (',')
						end
						print_default_attribute_value (l_result_type_set.static_type.primary_type, a_file)
						l_empty_struct := False
					end
					i := i + 1
				end
				if a_type.has_once_per_object_routines then
						-- We put the 'onces' pseudo attributes after the attributes so that
						-- types such as "STRING" and "SPECIAL" can get some attributes
						-- such as 'count' and 'capacity' at the expected positions.
					if not l_empty_struct then
						a_file.put_character (',')
					end
					a_file.put_character ('0')
					l_empty_struct := False
				end
				if attached {ET_DYNAMIC_SPECIAL_TYPE} a_type as l_special_type then
						-- Items.
					if not l_empty_struct then
						a_file.put_character (',')
					end
					a_file.put_character ('{')
					print_default_attribute_value (l_special_type.item_type_set.static_type.primary_type, a_file)
					a_file.put_character ('}')
					l_empty_struct := False
				elseif attached {ET_DYNAMIC_TUPLE_TYPE} a_type as l_tuple_type then
					l_item_type_sets := l_tuple_type.item_type_sets
					nb := l_item_type_sets.count
					from i := 1 until i > nb loop
						if not l_empty_struct then
							a_file.put_character (',')
						end
						print_default_attribute_value (l_item_type_sets.item (i).static_type.primary_type, a_file)
						l_empty_struct := False
						i := i + 1
					end
				end
				if l_empty_struct then
						-- Add a dummy field so that the struct is not empty (not allowed in C99).
					a_file.put_character ('0')
				end
				a_file.put_character ('}')
			else
				a_file.put_character ('0')
			end
		end

	print_default_entity_value (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' default initialization value for entities declared of type `a_type'.
			-- Note that an entity can be a reference to an object, and in that case
			-- its default initialization value is 'Void', and not a default initialized
			-- object of the corresponding type.
			-- This routine is mainly useful when declaring entities of non-basic expanded types.
			-- (In case of expanded types being involved, this default initialization
			-- does not take into account possible calls to 'default_create' which need
			-- to be applied subsequently.)
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if not a_type.is_expanded or else a_type.is_basic then
				a_file.put_character ('0')
			else
				print_default_name (a_type, a_file)
			end
		end

	print_typed_default_entity_value (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' a typed version of the default initialization value for
			-- entities declared of type `a_type'.
			-- Same as `print_default_entity_value', but preceded by a type cast if needed.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if not a_type.is_expanded or else a_type.is_basic then
				current_file.put_character ('(')
				print_type_declaration (a_type, current_file)
				current_file.put_character (')')
				a_file.put_character ('0')
			else
				print_default_name (a_type, a_file)
			end
		end

	print_default_attribute_value (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' default initialization value for attributes declared of type `a_type'.
			-- Note that an attribute is a special kind of entity, and therefore
			-- it can be a reference to an object, and in that case its default initialization
			-- value is 'Void', and not a default initialized object of the corresponding type.
			-- This routine is mainly useful for attributes of non-basic expanded types
			-- when determining the default initialization value of their enclosing objects.
			-- (In case of expanded types being involved, this default initialization
			-- does not take into account possible calls to 'default_create' which need
			-- to be applied subsequently.)
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if not a_type.is_expanded or else a_type.is_basic then
				a_file.put_character ('0')
			else
				print_default_object_value (a_type, a_file)
			end
		end

feature {NONE} -- Feature name generation

	print_routine_name (a_routine: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of `a_routine' from `a_type' to `a_file'.
		require
			a_routine_not_void: a_routine /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				print_type_name (a_type, a_file)
				a_file.put_character ('f')
				if attached {ET_DYNAMIC_PRECURSOR} a_routine as l_precursor then
					a_file.put_integer (l_precursor.current_feature.id)
					a_file.put_character ('p')
				end
				a_file.put_integer (a_routine.id)
			else
-- TODO: long names
				short_names := True
				print_routine_name (a_routine, a_type, a_file)
				short_names := False
			end
		end

	print_static_routine_name (a_routine: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of static feature `a_feature' from `a_type' to `a_file'.
		require
			a_routine_not_void: a_routine /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				print_type_name (a_type, a_file)
				a_file.put_character ('s')
				if attached {ET_DYNAMIC_PRECURSOR} a_routine as l_precursor then
					a_file.put_integer (l_precursor.current_feature.id)
					a_file.put_character ('p')
				end
				a_file.put_integer (a_routine.id)
			else
-- TODO: long names
				short_names := True
				print_static_routine_name (a_routine, a_type, a_file)
				short_names := False
			end
		end

	print_creation_procedure_name (a_procedure: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of creation procedure `a_procedure' from `a_type' to `a_file'.
		require
			a_procedure_not_void: a_procedure /= Void
			a_procedure_creation: a_procedure.is_creation
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				print_type_name (a_type, a_file)
				a_file.put_character ('c')
				a_file.put_integer (a_procedure.id)
			else
-- TODO: long names
				short_names := True
				print_creation_procedure_name (a_procedure, a_type, a_file)
				short_names := False
			end
		end

	print_address_routine_name (a_routine: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of `a_routine' from `a_type' when used to get its feature address.
		require
			a_routine_not_void: a_routine /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				print_type_name (a_type, a_file)
				a_file.put_character ('a')
				a_file.put_integer (a_routine.id)
			else
-- TODO: long names
				short_names := True
				print_address_routine_name (a_routine, a_type, a_file)
				short_names := False
			end
		end

	print_inline_routine_name (a_routine: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of C inline feature `a_feature' from `a_type' to `a_file'.
		require
			a_routine_not_void: a_routine /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				print_type_name (a_type, a_file)
				a_file.put_character ('i')
				if attached {ET_DYNAMIC_PRECURSOR} a_routine as l_precursor then
					a_file.put_integer (l_precursor.current_feature.id)
					a_file.put_character ('p')
				end
				a_file.put_integer (a_routine.id)
			else
-- TODO: long names
				short_names := True
				print_inline_routine_name (a_routine, a_type, a_file)
				short_names := False
			end
		end

	print_attribute_name (an_attribute: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of `an_attribute' for objects of type `a_type'.
		require
			an_attribute_not_void: an_attribute /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('a')
				a_file.put_integer (an_attribute.id)
			else
-- TODO: long names
				short_names := True
				print_attribute_name (an_attribute, a_type, a_file)
				short_names := False
			end
		end

	print_attribute_type_id_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of the 'type_id' pseudo attribute for objects of type `a_type'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_string (c_id)
			else
-- TODO: long names
				short_names := True
				print_attribute_type_id_name (a_type, a_file)
				short_names := False
			end
		end

	print_attribute_flags_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of the 'flags' pseudo attribute for objects of type `a_type'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_string (c_flags)
			else
-- TODO: long names
				short_names := True
				print_attribute_flags_name (a_type, a_file)
				short_names := False
			end
		end

	print_attribute_onces_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of the 'onces' pseudo attribute for objects of type `a_type'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_string (c_onces)
			else
-- TODO: long names
				short_names := True
				print_attribute_onces_name (a_type, a_file)
				short_names := False
			end
		end

	print_attribute_special_item_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of the 'item' pseudo attribute for 'SPECIAL' objects of type `a_type'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('z')
				a_file.put_character ('2')
			else
-- TODO: long names
				short_names := True
				print_attribute_special_item_name (a_type, a_file)
				short_names := False
			end
		end

	print_attribute_special_capacity_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of the 'capacity' pseudo attribute for 'SPECIAL' objects of type `a_type'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				if a_type.attribute_count < 2 then
						-- Internal error: class "SPECIAL" should have at least the
						-- feature 'capacity' as second feature.
						-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					print_attribute_name (a_type.queries.item (2), a_type, a_file)
				end
			else
-- TODO: long names
				short_names := True
				print_attribute_special_capacity_name (a_type, a_file)
				short_names := False
			end
		end

	print_attribute_special_count_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of the 'count' pseudo attribute for 'SPECIAL' objects of type `a_type'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				if a_type.attribute_count < 1 then
						-- Internal error: class "SPECIAL" should have at least the
						-- feature 'count' as first feature.
						-- Already reported in ET_DYNAMIC_SYSTEM.compile_kernel.
					set_fatal_error
					error_handler.report_giaaa_error
				else
					print_attribute_name (a_type.queries.first, a_type, a_file)
				end
			else
-- TODO: long names
				short_names := True
				print_attribute_special_count_name (a_type, a_file)
				short_names := False
			end
		end

	print_attribute_special_offset_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of the 'offset' pseudo attribute for 'SPECIAL' objects of type `a_type'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_string (c_offset)
			else
-- TODO: long names
				short_names := True
				print_attribute_special_offset_name (a_type, a_file)
				short_names := False
			end
		end

	print_attribute_tuple_item_name (i: INTEGER; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' then name of the `i'-th 'item' pseudo attribute for 'TUPLE' objects of type `a_type'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('z')
				a_file.put_integer (i)
			else
-- TODO: long names
				short_names := True
				print_attribute_tuple_item_name (i, a_type, a_file)
				short_names := False
			end
		end

	print_attribute_routine_function_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of the function pointer pseudo attribute for 'ROUTINE' objects of type `a_type'.
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('f')
			else
-- TODO: long names
				short_names := True
				print_attribute_routine_function_name (a_type, a_file)
				short_names := False
			end
		end

	print_boxed_attribute_item_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' then name of the 'item' pseudo attribute of boxed version of `a_type'.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('z')
				a_file.put_character ('1')
			else
-- TODO: long names
				short_names := True
				print_boxed_attribute_item_name (a_type, a_file)
				short_names := False
			end
		end

	print_boxed_attribute_pointer_name (a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' then name of the 'pointer' pseudo attribute of boxed version of `a_type'.
			-- (The boxed version of a type makes sure that each object
			-- of that type contains its type-id. It can be the type itself
			-- if it already contains its type-id, or a wrapper otherwise.)
		require
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('p')
				a_file.put_character ('1')
			else
-- TODO: long names
				short_names := True
				print_boxed_attribute_pointer_name (a_type, a_file)
				short_names := False
			end
		end

	print_once_per_object_value_name (a_once_routine: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of the value of the once-per-object
			-- routine `a_once_routine' for objects of type `a_type'.
		require
			an_attribute_not_void: a_once_routine /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('v')
				a_file.put_integer (a_once_routine.id)
			else
-- TODO: long names
				short_names := True
				print_once_per_object_value_name (a_once_routine, a_type, a_file)
				short_names := False
			end
		end

	print_once_per_object_status_name (a_once_routine: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of the status of the once-per-object
			-- routine `a_once_routine' for objects of type `a_type'.
		require
			an_attribute_not_void: a_once_routine /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('s')
				a_file.put_integer (a_once_routine.id)
			else
-- TODO: long names
				short_names := True
				print_once_per_object_status_name (a_once_routine, a_type, a_file)
				short_names := False
			end
		end

	print_once_per_object_exception_name (a_once_routine: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of the exception of the once-per-object
			-- routine `a_once_routine' for objects of type `a_type'.
		require
			an_attribute_not_void: a_once_routine /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('e')
				a_file.put_integer (a_once_routine.id)
			else
-- TODO: long names
				short_names := True
				print_once_per_object_exception_name (a_once_routine, a_type, a_file)
				short_names := False
			end
		end

	print_once_per_object_mutex_name (a_once_routine: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the name of the mutex of the once-per-object
			-- routine `a_once_routine' for objects of type `a_type'.
		require
			an_attribute_not_void: a_once_routine /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('m')
				a_file.put_integer (a_once_routine.id)
			else
-- TODO: long names
				short_names := True
				print_once_per_object_mutex_name (a_once_routine, a_type, a_file)
				short_names := False
			end
		end

	print_call_name (a_call: ET_CALL_COMPONENT; a_caller: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_force_result_boxing: BOOLEAN; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of `a_call' appearing in `a_caller' with `a_target_type' as target static type to `a_file'.
			-- `a_force_result_boxing' indicates that `a_call' is an attribute, tuple label or "SPECIAL"
			-- item, the call result type is expanded, and it is the target of another call.
			-- In that case we force the result type to be of reference type in order
			-- to force the boxing of the result so that the other call is applied on
			-- the result object itself and not on a copy of this object.
		require
			a_call_not_void: a_call /= Void
			a_caller_not_void: a_caller /= Void
			a_target_type_not_void: a_target_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_arguments: detachable ET_ARGUMENT_OPERANDS
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_argument_type: ET_DYNAMIC_PRIMARY_TYPE
			l_seed: INTEGER
			i, nb: INTEGER
		do
			if short_names then
				l_seed := a_call.name.seed
				l_arguments := a_call.arguments
				print_type_name (a_target_type, a_file)
				a_file.put_character ('x')
				if a_force_result_boxing then
					a_file.put_character ('b')
				end
				if a_call.is_tuple_label then
					a_file.put_character ('t')
					a_file.put_integer (l_seed)
				elseif
					(l_seed = current_system.routine_call_seed or l_seed = current_system.function_item_seed) and then
					not a_call.is_call_agent and then
					(l_arguments /= Void and then l_arguments.count = 1) and then
					attached {ET_MANIFEST_TUPLE} l_arguments.actual_argument (1) as l_manifest_tuple
				then
					a_file.put_character ('m')
					a_file.put_character ('t')
					a_file.put_integer (l_seed)
					l_argument_type_set := dynamic_type_set_in_feature (l_manifest_tuple, a_caller)
					print_type_name (l_argument_type_set.static_type.primary_type, a_file)
					l_arguments := Void
				else
					a_file.put_integer (l_seed)
				end
				if l_arguments /= Void then
					nb := l_arguments.count
					from i := 1 until i > nb loop
						l_argument_type_set := dynamic_type_set_in_feature (l_arguments.actual_argument (i), a_caller)
						l_argument_type := l_argument_type_set.static_type.primary_type
						if l_argument_type.is_expanded then
							print_type_name (l_argument_type, a_file)
						else
							print_eif_any_type_name (a_file)
						end
						i := i + 1
					end
				end
			else
-- TODO: long names
				short_names := True
				print_call_name (a_call, a_caller, a_target_type, a_force_result_boxing, a_file)
				short_names := False
			end
		end

	print_argument_name (a_name: ET_IDENTIFIER; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of argument `a_name' to `a_file'.
		require
			a_name_not_void: a_name /= Void
			a_name_argument: a_name.is_argument
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('a')
				a_file.put_integer (a_name.seed)
			else
-- TODO: long names
				short_names := True
				print_argument_name (a_name, a_file)
				short_names := False
			end
		end

	print_local_name (a_name: ET_IDENTIFIER; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of local variable `a_name' to `a_file'.
		require
			a_name_not_void: a_name /= Void
			a_name_local: a_name.is_local
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('l')
				a_file.put_integer (a_name.seed)
			else
-- TODO: long names
				short_names := True
				print_local_name (a_name, a_file)
				short_names := False
			end
		end

	print_iteration_cursor_name (a_name: ET_IDENTIFIER; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of iteration cursor `a_name' to `a_file'.
		require
			a_name_not_void: a_name /= Void
			a_name_iteration_item: a_name.is_iteration_item
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('u')
				a_file.put_integer (a_name.seed)
			else
-- TODO: long names
				short_names := True
				print_iteration_cursor_name (a_name, a_file)
				short_names := False
			end
		end

	print_object_test_local_name (a_name: ET_IDENTIFIER; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of object-test local `a_name' to `a_file'.
		require
			a_name_not_void: a_name /= Void
			a_name_object_test_local: a_name.is_object_test_local
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('m')
				a_file.put_integer (a_name.seed)
			else
-- TODO: long names
				short_names := True
				print_object_test_local_name (a_name, a_file)
				short_names := False
			end
		end

	print_object_test_function_name (i: INTEGER; a_routine: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of `i'-th object-test function appearing in `a_routine' from `a_type' to `a_file'.
		require
			a_routine_not_void: a_routine /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if in_static_feature then
				print_static_routine_name (a_routine, a_type, a_file)
			else
				print_routine_name (a_routine, a_type, a_file)
			end
			a_file.put_character ('o')
			a_file.put_character ('t')
			a_file.put_integer (i)
		end

	print_equality_function_name (i: INTEGER; a_routine: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of `i'-th equality ('=' or '/=' as well as '~' or '/~') function
			-- appearing in `a_routine' from `a_type' to `a_file'.
		require
			a_routine_not_void: a_routine /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if in_static_feature then
				print_static_routine_name (a_routine, a_type, a_file)
			else
				print_routine_name (a_routine, a_type, a_file)
			end
			a_file.put_character ('e')
			a_file.put_integer (i)
		end

	print_temp_name (a_name: ET_IDENTIFIER; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of temporary variable `a_name' to `a_file'.
		require
			a_name_not_void: a_name /= Void
			a_name_temp: a_name.is_temporary
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('t')
				a_file.put_integer (a_name.seed)
			else
-- TODO: long names
				short_names := True
				print_temp_name (a_name, a_file)
				short_names := False
			end
		end

	print_current_name (a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of 'Current' to `a_file'.
		require
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('C')
			else
-- TODO: long names
				short_names := True
				print_current_name (a_file)
				short_names := False
			end
		end

	print_result_name (a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of 'Result' to `a_file'.
		require
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if short_names then
				a_file.put_character ('R')
			else
-- TODO: long names
				short_names := True
				print_result_name (a_file)
				short_names := False
			end
		end

	print_once_status_name (a_feature: ET_FEATURE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of variable holding the status of execution
			-- of the once-feature `a_feature' to `a_file'.
		require
			a_feature_not_void: a_feature /= Void
			implementation_feature: a_feature = a_feature.implementation_feature
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_character ('g')
			a_file.put_character ('e')
			a_file.put_integer (a_feature.implementation_class.id)
			a_file.put_character ('o')
			a_file.put_character ('s')
			a_file.put_integer (a_feature.first_seed)
		end

	print_once_value_name (a_feature: ET_FEATURE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of variable holding the value of first
			-- execution of the once-feature `a_feature' to `a_file'.
		require
			a_feature_not_void: a_feature /= Void
			implementation_feature: a_feature = a_feature.implementation_feature
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_character ('g')
			a_file.put_character ('e')
			a_file.put_integer (a_feature.implementation_class.id)
			a_file.put_character ('o')
			a_file.put_character ('v')
			a_file.put_integer (a_feature.first_seed)
		end

	print_agent_creation_name (i: INTEGER; a_routine: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of creation function of `i'-th agent appearing in `a_routine' from `a_type' to `a_file'.
		require
			a_routine_not_void: a_routine /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if in_static_feature then
				print_static_routine_name (a_routine, a_type, a_file)
			else
				print_routine_name (a_routine, a_type, a_file)
			end
			a_file.put_character ('a')
			a_file.put_character ('c')
			a_file.put_integer (i)
		end

	print_agent_function_name (i: INTEGER; a_routine: ET_DYNAMIC_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of function associated with `i'-th agent appearing in `a_routine' from `a_type' to `a_file'.
		require
			a_routine_not_void: a_routine /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if in_static_feature then
				print_static_routine_name (a_routine, a_type, a_file)
			else
				print_routine_name (a_routine, a_type, a_file)
			end
			a_file.put_character ('a')
			a_file.put_character ('f')
			a_file.put_integer (i)
		end

	print_inline_constant_name (a_constant: ET_INLINE_CONSTANT; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of variable holding the value of inline constant
			-- `a_constant' (such as a once manifest string) to `a_file'.
		require
			a_constant_not_void: a_constant /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_character ('g')
			a_file.put_character ('e')
			a_file.put_character ('i')
			a_file.put_character ('c')
			a_file.put_integer (a_constant.id)
		end

	print_feature_name_comment (a_feature: ET_FEATURE; a_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of `a_feature' from `a_type' as a C comment to `a_file'.
		require
			a_feature_not_void: a_feature /= Void
			a_type_not_void: a_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_character ('/')
			a_file.put_character ('*')
			a_file.put_character (' ')
			a_file.put_string (a_type.base_type.unaliased_to_text)
			a_file.put_character ('.')
			a_file.put_string (STRING_.replaced_all_substrings (a_feature.lower_name, "*/", "star/"))
			a_file.put_character (' ')
			a_file.put_character ('*')
			a_file.put_character ('/')
			a_file.put_new_line
		end

	print_call_name_comment (a_call: ET_CALL_COMPONENT; a_caller: ET_DYNAMIC_FEATURE; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print name of `a_call', appearing in `a_caller' with `a_type' as target static type, as a C comment to `a_file'.
		require
			a_call_not_void: a_call /= Void
			a_caller_not_void: a_caller /= Void
			a_target_type_not_void: a_target_type /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_arguments: detachable ET_ARGUMENT_OPERANDS
			l_argument_type_set: ET_DYNAMIC_TYPE_SET
			l_seed: INTEGER
		do
			a_file.put_character ('/')
			a_file.put_character ('*')
			a_file.put_character (' ')
			a_file.put_character ('C')
			a_file.put_character ('a')
			a_file.put_character ('l')
			a_file.put_character ('l')
			a_file.put_character (' ')
			a_file.put_character ('t')
			a_file.put_character ('o')
			a_file.put_character (' ')
			a_file.put_string (a_target_type.base_type.unaliased_to_text)
			a_file.put_character ('.')
			a_file.put_string (STRING_.replaced_all_substrings (a_call.name.lower_name, "*/", "star/"))
			l_seed := a_call.name.seed
			if a_call.is_tuple_label then
				a_file.put_string (once " (label on item #")
				a_file.put_integer (l_seed)
				a_file.put_character (')')
			elseif (l_seed = current_system.routine_call_seed or l_seed = current_system.function_item_seed) and then not a_call.is_call_agent then
				l_arguments := a_call.arguments
				if l_arguments /= Void and then l_arguments.count = 1 then
					if attached {ET_MANIFEST_TUPLE} l_arguments.actual_argument (1) as l_manifest_tuple then
						a_file.put_string (once " with a manifest tuple argument")
						l_argument_type_set := dynamic_type_set_in_feature (l_manifest_tuple, a_caller)
						a_file.put_string (once " of type ")
						a_file.put_string (l_argument_type_set.static_type.primary_type.base_type.unaliased_to_text)
					end
				end
			end
			a_file.put_character (' ')
			a_file.put_character ('*')
			a_file.put_character ('/')
			a_file.put_new_line
		end

feature {NONE} -- String generation

	print_escaped_string (a_string: STRING)
			-- Print escaped version of `a_string'.
			--
			-- Note: There was an attempt to make sure that two identical
			-- C strings were not generated twice by creating one C constant
			-- per string, and also by creating a C array of string constants.
			-- Both solutions resulted in a larger executable, and the execution
			-- speed of the resulting executable was the same. So these solutions
			-- have not been kept.
		require
			a_string_not_void: a_string /= Void
		local
			i, nb: INTEGER
			c: CHARACTER
			l_code: INTEGER
			l_splitted: BOOLEAN
			l_split_size: INTEGER
		do
			l_split_size := 512
			current_file.put_character ('%"')
			nb := a_string.count
			from i := 1 until i > nb loop
				if (i \\ l_split_size) = 1 and i /= 1 then
						-- Some C compilers don't accept too big strings.
						-- Split them in several smaller ones.
					current_file.put_character ('%"')
					if not l_splitted then
						l_splitted := True
						indent
					end
					current_file.put_new_line
					print_indentation
					current_file.put_character ('%"')
				end
				c := a_string.item (i)
				inspect c
				when ' ', '!', '#', '$', '&', '('..'>', '@'..'[', ']'..'~' then
					current_file.put_character (c)
				when '%N' then
					current_file.put_character ('\')
					current_file.put_character ('n')
				when '%R' then
					current_file.put_character ('\')
					current_file.put_character ('r')
				when '%T' then
					current_file.put_character ('\')
					current_file.put_character ('t')
				when '%U' then
					current_file.put_character ('\')
					current_file.put_character ('0')
					current_file.put_character ('0')
					current_file.put_character ('0')
				when '\' then
					current_file.put_character ('\')
					current_file.put_character ('\')
				when '%'' then
					current_file.put_character ('\')
					current_file.put_character ('%'')
				when '%"' then
					current_file.put_character ('\')
					current_file.put_character ('%"')
				when '?' then
						-- Make sure that ? is not recognized as
						-- part of a trigraph sequence.
					current_file.put_character ('\')
					current_file.put_character ('?')
				else
					current_file.put_character ('\')
					l_code := c.code
					if l_code < 8 then
						current_file.put_character ('0')
						current_file.put_character ('0')
					elseif l_code < 64 then
						current_file.put_character ('0')
					end
					INTEGER_FORMATTER_.put_octal_integer (current_file, l_code)
				end
				i := i + 1
			end
			current_file.put_character ('%"')
			if l_splitted then
				dedent
			end
		end

	print_utf8_as_escaped_string_8 (a_utf8_string: STRING_8)
			-- Print `a_utf8_string' as an escaped version of
			-- its STRING_8 representation.
		require
			a_utf8_string_not_void: a_utf8_string /= Void
			a_utf8_string_is_string: {KL_ANY_ROUTINES}.same_types (a_utf8_string, "")
			valid_utf8: {UC_UTF8_ROUTINES}.valid_utf8 (a_utf8_string)
		local
			i, j, nb: INTEGER
			c1, c2: CHARACTER_8
			l_code: NATURAL_32
			l_splitted: BOOLEAN
			l_split_size: INTEGER
		do
			l_split_size := 512
			current_file.put_character ('%"')
			nb := a_utf8_string.count
			j := 1
			from i := 1 until i > nb loop
				if (j \\ l_split_size) = 1 and j /= 1 then
						-- Some C compilers don't accept too big strings.
						-- Split them in several smaller ones.
					current_file.put_character ('%"')
					if not l_splitted then
						l_splitted := True
						indent
					end
					current_file.put_new_line
					print_indentation
					current_file.put_character ('%"')
				end
				c1 := a_utf8_string.item (i)
				inspect c1
				when ' ', '!', '#', '$', '&', '('..'>', '@'..'[', ']'..'~' then
					current_file.put_character (c1)
				when '%N' then
					current_file.put_character ('\')
					current_file.put_character ('n')
				when '%R' then
					current_file.put_character ('\')
					current_file.put_character ('r')
				when '%T' then
					current_file.put_character ('\')
					current_file.put_character ('t')
				when '%U' then
					current_file.put_character ('\')
					current_file.put_character ('0')
					current_file.put_character ('0')
					current_file.put_character ('0')
				when '\' then
					current_file.put_character ('\')
					current_file.put_character ('\')
				when '%'' then
					current_file.put_character ('\')
					current_file.put_character ('%'')
				when '%"' then
					current_file.put_character ('\')
					current_file.put_character ('%"')
				when '?' then
						-- Make sure that ? is not recognized as
						-- part of a trigraph sequence.
					current_file.put_character ('\')
					current_file.put_character ('?')
				else
					l_code := c1.natural_32_code
					if {UC_UTF8_ROUTINES}.encoded_byte_count (c1) = 2 then
							-- 110xxxxx 10xxxxxx
						i := i + 1
						c2 := a_utf8_string.item (i)
						l_code := {UC_UTF8_ROUTINES}.two_byte_character_code (c1, c2)
					end
					current_file.put_character ('\')
					if l_code < 8 then
						current_file.put_character ('0')
						current_file.put_character ('0')
					elseif l_code < 64 then
						current_file.put_character ('0')
					end
					INTEGER_FORMATTER_.put_octal_natural_32 (current_file, l_code)
				end
				i := i + 1
				j := j + 1
			end
			current_file.put_character ('%"')
			if l_splitted then
				dedent
			end
		end

	print_utf8_as_escaped_string_32 (a_utf8_string: STRING_8)
			-- Print `a_utf8_string' as an escaped version of
			-- its STRING_32 representation.
		require
			a_utf8_string_not_void: a_utf8_string /= Void
			a_utf8_string_is_string: {KL_ANY_ROUTINES}.same_types (a_utf8_string, "")
			valid_utf8: {UC_UTF8_ROUTINES}.valid_utf8 (a_utf8_string)
		local
			i, j, nb: INTEGER
			c1: CHARACTER_8
			l_code, l_code2: NATURAL_32
			l_splitted: BOOLEAN
			l_split_size: INTEGER
			l_byte_count: INTEGER
		do
			l_split_size := 512
			current_file.put_character ('%"')
			nb := a_utf8_string.count
			j := 1
			from i := 1 until i > nb loop
				if (j \\ l_split_size) = 1 and j /= 1 then
						-- Some C compilers don't accept too big strings.
						-- Split them in several smaller ones.
					current_file.put_character ('%"')
					if not l_splitted then
						l_splitted := True
						indent
					end
					current_file.put_new_line
					print_indentation
					current_file.put_character ('%"')
				end
				c1 := a_utf8_string.item (i)
				l_byte_count := {UC_UTF8_ROUTINES}.encoded_byte_count (c1)
				inspect l_byte_count
				when 1 then
						-- 0xxxxxxx
					l_code := c1.natural_32_code
				when 2 then
						-- 110xxxxx 10xxxxxx
					l_code := {UC_UTF8_ROUTINES}.two_byte_character_code (c1, a_utf8_string.item (i + 1))
				when 3 then
						-- 1110xxxx 10xxxxxx 10xxxxxx
					l_code := {UC_UTF8_ROUTINES}.three_byte_character_code (c1, a_utf8_string.item (i + 1), a_utf8_string.item (i + 2))
				when 4 then
						-- 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
					l_code := {UC_UTF8_ROUTINES}.four_byte_character_code (c1, a_utf8_string.item (i + 1), a_utf8_string.item (i + 2), a_utf8_string.item (i + 3))
				else
						-- Should never happen.
					l_code := 0
					i := i + 1
				end
				c1 := (l_code & 0x000000FF).to_character_8
				inspect c1
				when ' ', '!', '#', '$', '&', '('..'>', '@'..'[', ']'..'~' then
					current_file.put_character (c1)
				when '%N' then
					current_file.put_character ('\')
					current_file.put_character ('n')
				when '%R' then
					current_file.put_character ('\')
					current_file.put_character ('r')
				when '%T' then
					current_file.put_character ('\')
					current_file.put_character ('t')
				when '%U' then
					current_file.put_character ('\')
					current_file.put_character ('0')
					current_file.put_character ('0')
					current_file.put_character ('0')
				when '\' then
					current_file.put_character ('\')
					current_file.put_character ('\')
				when '%'' then
					current_file.put_character ('\')
					current_file.put_character ('%'')
				when '%"' then
					current_file.put_character ('\')
					current_file.put_character ('%"')
				when '?' then
						-- Make sure that ? is not recognized as
						-- part of a trigraph sequence.
					current_file.put_character ('\')
					current_file.put_character ('?')
				else
					l_code2 := c1.natural_32_code
					current_file.put_character ('\')
					if l_code2 < 8 then
						current_file.put_character ('0')
						current_file.put_character ('0')
					elseif l_code2 < 64 then
						current_file.put_character ('0')
					end
					INTEGER_FORMATTER_.put_octal_natural_32 (current_file, l_code2)
				end
				c1 := ((l_code & 0x0000FF00) |>> 8).to_character_8
				l_code2 := c1.natural_32_code
				current_file.put_character ('\')
				if l_code2 < 8 then
					current_file.put_character ('0')
					current_file.put_character ('0')
				elseif l_code2 < 64 then
					current_file.put_character ('0')
				end
				INTEGER_FORMATTER_.put_octal_natural_32 (current_file, l_code2)
				c1 := ((l_code & 0x00FF0000) |>> 16).to_character_8
				l_code2 := c1.natural_32_code
				current_file.put_character ('\')
				if l_code2 < 8 then
					current_file.put_character ('0')
					current_file.put_character ('0')
				elseif l_code2 < 64 then
					current_file.put_character ('0')
				end
				INTEGER_FORMATTER_.put_octal_natural_32 (current_file, l_code2)
				c1 := ((l_code & 0xFF000000) |>> 24).to_character_8
				l_code2 := c1.natural_32_code
				current_file.put_character ('\')
				if l_code2 < 8 then
					current_file.put_character ('0')
					current_file.put_character ('0')
				elseif l_code2 < 64 then
					current_file.put_character ('0')
				end
				INTEGER_FORMATTER_.put_octal_natural_32 (current_file, l_code2)
				i := i + l_byte_count
				j := j + 1
			end
			current_file.put_character ('%"')
			if l_splitted then
				dedent
			end
		end

	print_escaped_character_8 (c: CHARACTER_8)
			-- Print escaped version of `c'.
		local
			l_code: INTEGER
		do
			current_file.put_character ('%'')
			inspect c
			when ' ', '!', '#', '$', '&', '('..'[', ']'..'~' then
				current_file.put_character (c)
			when '%N' then
				current_file.put_character ('\')
				current_file.put_character ('n')
			when '%R' then
				current_file.put_character ('\')
				current_file.put_character ('r')
			when '%T' then
				current_file.put_character ('\')
				current_file.put_character ('t')
			when '%U' then
				current_file.put_character ('\')
				current_file.put_character ('0')
				current_file.put_character ('0')
				current_file.put_character ('0')
			when '\' then
				current_file.put_character ('\')
				current_file.put_character ('\')
			when '%'' then
				current_file.put_character ('\')
				current_file.put_character ('%'')
			when '%"' then
				current_file.put_character ('\')
				current_file.put_character ('%"')
			else
				current_file.put_character ('\')
				l_code := c.code
				if l_code < 8 then
					current_file.put_character ('0')
					current_file.put_character ('0')
				elseif l_code < 64 then
					current_file.put_character ('0')
				end
				INTEGER_FORMATTER_.put_octal_integer (current_file, l_code)
			end
			current_file.put_character ('%'')
		end

feature {NONE} -- Misc C code

	print_return_statement (a_feature: ET_INTERNAL_FEATURE)
			-- Print C instruction to return the result of `a_feature'
			-- if it is a function, or just an empty return if it's
			-- a procedure.
		require
			a_feature_not_void: a_feature /= Void
			valid_feature: current_feature.static_feature = a_feature
		do
			print_indentation
			current_file.put_string (c_return)
			if current_feature.result_type_set /= Void then
				current_file.put_character (' ')
				print_result_name (current_file)
			end
			print_semicolon_newline
		end

	print_position (a_position: ET_POSITION; a_class: ET_CLASS)
			-- Print "#line" instruction corresponding to `a_position' in `a_class'.
		require
			a_position_not_void: a_position /= Void
			a_class_not_void: a_class /= Void
		local
			l_line: INTEGER
			l_filename: STRING
		do
			l_line := a_position.line
			l_filename := a_class.filename
			if l_line /= 0 and l_filename /= Void then
				current_file.put_string (c_line)
				current_file.put_character (' ')
				current_file.put_integer (l_line)
				current_file.put_character (' ')
				print_escaped_string (l_filename)
				current_file.put_new_line
			end
		end

	print_assign_temp_variable_to_special_count_attribute (a_temp_variable: ET_IDENTIFIER; a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print C code to assign temporary variable `a_temp_variable' to the 'count'
			-- pseudo attribute of class "SPECIAL" applied of object `a_target' of type `a_target_type'.
			-- `a_check_void_target` means that we need to check whether the target is Void or not.
		require
			a_temp_variable_not_void: a_temp_variable /= Void
			a_temp_variable_is_temoprary_variable: a_temp_variable.is_temporary
			a_temp_variable_known: is_temp_variable_known (a_temp_variable)
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			print_indentation
			print_attribute_special_count_access (a_target, a_target_type, a_check_void_target)
			print_assign_to
			print_temp_name (a_temp_variable, current_file)
			print_semicolon_newline
		end

	print_assign_special_count_attribute_to_temp_variable (a_temp_variable: ET_IDENTIFIER; a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print C code to assign the 'count' pseudo attribute of class "SPECIAL" applied of
			-- object `a_target' of type `a_target_type' to temporary variable `a_temp_variable'.
			-- `a_check_void_target` means that we need to check whether the target is Void or not.
		require
			a_temp_variable_not_void: a_temp_variable /= Void
			a_temp_variable_is_temoprary_variable: a_temp_variable.is_temporary
			a_temp_variable_known: is_temp_variable_known (a_temp_variable)
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			print_indentation
			print_temp_name (a_temp_variable, current_file)
			print_assign_to
			print_attribute_special_count_access (a_target, a_target_type, a_check_void_target)
			print_semicolon_newline
		end

	print_assign_temp_variable_to_special_capacity_attribute (a_temp_variable: ET_IDENTIFIER; a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print C code to assign a temporary variable `a_temp_variable' to the 'capacity'
			-- pseudo attribute of class "SPECIAL" applied of object `a_target' of type `a_target_type'.
			-- `a_check_void_target` means that we need to check whether the target is Void or not.
		require
			a_temp_variable_not_void: a_temp_variable /= Void
			a_temp_variable_is_temoprary_variable: a_temp_variable.is_temporary
			a_temp_variable_known: is_temp_variable_known (a_temp_variable)
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			print_indentation
			print_attribute_special_capacity_access (a_target, a_target_type, a_check_void_target)
			print_assign_to
			print_temp_name (a_temp_variable, current_file)
			print_semicolon_newline
		end

	print_assign_special_capacity_attribute_to_temp_variable (a_temp_variable: ET_IDENTIFIER; a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print C code to assign the 'capacity' pseudo attribute of class "SPECIAL" applied of
			-- object `a_target' of type `a_target_type' to temporary variable `a_temp_variable'.
			-- `a_check_void_target` means that we need to check whether the target is Void or not.
		require
			a_temp_variable_not_void: a_temp_variable /= Void
			a_temp_variable_is_temoprary_variable: a_temp_variable.is_temporary
			a_temp_variable_known: is_temp_variable_known (a_temp_variable)
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			print_indentation
			print_temp_name (a_temp_variable, current_file)
			print_assign_to
			print_attribute_special_capacity_access (a_target, a_target_type, a_check_void_target)
			print_semicolon_newline
		end

	print_assign_flags_attribute_to_temp_variable (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print C code to assign the 'flags' pseudo attribute of
			-- object `a_target' of type `a_target_type', if any, to a temporary variable.
			-- `a_check_void_target` means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if not a_target_type.is_expanded then
				print_indentation
				current_file.put_string (c_uint16_t)
				current_file.put_character (' ')
				current_file.put_character ('f')
				current_file.put_character ('1')
				print_assign_to
				print_attribute_flags_access (a_target, current_type, a_check_void_target)
				print_semicolon_newline
			end
		end

	print_assign_temp_variable_to_flags_attribute (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print C code to assign a temporary variable to the 'flags'
			-- pseudo attribute of object `a_target' of type `a_target_type', if any.
			-- `a_check_void_target` means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if not a_target_type.is_expanded then
				print_indentation
				print_attribute_flags_access (a_target, a_target_type, a_check_void_target)
				print_assign_to
				current_file.put_character ('f')
				current_file.put_character ('1')
				print_semicolon_newline
			end
		end

	print_assign_onces_attribute_to_temp_variable (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print C code to assign the 'onces' pseudo attribute of
			-- object `a_target' of type `a_target_type', if any, to a temporary variable.
			-- `a_check_void_target` means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if a_target_type.has_once_per_object_routines then
				print_indentation
				print_once_per_object_data_type_name (a_target_type, current_file)
				current_file.put_character ('*')
				current_file.put_character (' ')
				current_file.put_character ('o')
				current_file.put_character ('1')
				print_assign_to
				print_attribute_onces_access (a_target, a_target_type, a_check_void_target)
				print_semicolon_newline
			end
		end

	print_assign_temp_variable_to_onces_attribute (a_target: ET_EXPRESSION; a_target_type: ET_DYNAMIC_PRIMARY_TYPE; a_check_void_target: BOOLEAN)
			-- Print C code to assign a temporary variable to the 'onces'
			-- pseudo attribute of object `a_target' of type `a_target_type', if any.
			-- `a_check_void_target` means that we need to check whether the target is Void or not.
		require
			a_target_not_void: a_target /= Void
			a_target_type_not_void: a_target_type /= Void
		do
			if a_target_type.has_once_per_object_routines then
				print_indentation
				print_attribute_onces_access (a_target, a_target_type, a_check_void_target)
				print_assign_to
				current_file.put_character ('o')
				current_file.put_character ('1')
				print_semicolon_newline
			end
		end

	print_indentation_assign_to_result
			-- Print indentation followed by "R = " to `current_file'.
		do
			print_indentation
			print_result_name (current_file)
			print_assign_to
		end

	print_indentation_if
			-- Print indentation followed by "if (" to `current_file'.
		do
			print_indentation
			current_file.put_string (c_if)
			current_file.put_character (' ')
			current_file.put_character ('(')
		end

	print_then_newline
			-- Print ") {" followed by a_newline to `current_file'.
		do
			current_file.put_character (')')
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
		end

	print_indentation_else_newline
			-- Print indentation followed by "} else {" followed by a newline to `current_file'.
		do
			print_indentation
			current_file.put_character ('}')
			current_file.put_character (' ')
			current_file.put_string (c_else)
			current_file.put_character (' ')
			current_file.put_character ('{')
			current_file.put_new_line
		end

	print_indentation_end_newline
			-- Print indentation followed by "}" followed by a newline to `current_file'.
		do
			print_indentation
			current_file.put_character ('}')
			current_file.put_new_line
		end

	print_assign_to
			-- Print " = " to `current_file'.
		do
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character (' ')
		end

	print_equal_to
			-- Print " == " to `current_file'.
		do
			current_file.put_character (' ')
			current_file.put_character ('=')
			current_file.put_character ('=')
			current_file.put_character (' ')
		end

	print_less_than
			-- Print " < " to `current_file'.
		do
			current_file.put_character (' ')
			current_file.put_character ('<')
			current_file.put_character (' ')
		end

	print_less_than_or_equal
			-- Print " <= " to `current_file'.
		do
			current_file.put_character (' ')
			current_file.put_character ('<')
			current_file.put_character ('=')
			current_file.put_character (' ')
		end

	print_greater_than
			-- Print " > " to `current_file'.
		do
			current_file.put_character (' ')
			current_file.put_character ('>')
			current_file.put_character (' ')
		end

	print_greater_than_or_equal
			-- Print " >= " to `current_file'.
		do
			current_file.put_character (' ')
			current_file.put_character ('>')
			current_file.put_character ('=')
			current_file.put_character (' ')
		end

	print_and_then
			-- Print " && " to `current_file'.
		do
			current_file.put_character (' ')
			current_file.put_character ('&')
			current_file.put_character ('&')
			current_file.put_character (' ')
		end

	print_or_else
			-- Print " || " to `current_file'.
		do
			current_file.put_character (' ')
			current_file.put_character ('|')
			current_file.put_character ('|')
			current_file.put_character (' ')
		end

	print_plus
			-- Print " + " to `current_file'.
		do
			current_file.put_character (' ')
			current_file.put_character ('+')
			current_file.put_character (' ')
		end

	print_minus
			-- Print " - " to `current_file'.
		do
			current_file.put_character (' ')
			current_file.put_character ('-')
			current_file.put_character (' ')
		end

	print_times
			-- Print " * " to `current_file'.
		do
			current_file.put_character (' ')
			current_file.put_character ('*')
			current_file.put_character (' ')
		end

	print_comma
			-- Print ", " to `current_file'.
		do
			current_file.put_character (',')
			current_file.put_character (' ')
		end

	print_semicolon_newline
			-- Print a semicolon followed by a newline to `current_file'.
		do
			current_file.put_character (';')
			current_file.put_new_line
		end

	print_start_extern_c (a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the beginning of 'extern "C"' section.
		require
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_line ("#ifdef __cplusplus")
			a_file.put_line ("extern %"C%" {")
			a_file.put_line (c_endif)
			a_file.put_new_line
		end

	print_end_extern_c (a_file: KI_TEXT_OUTPUT_STREAM)
			-- Print to `a_file' the end of 'extern "C"' section.
		require
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			a_file.put_new_line
			a_file.put_line ("#ifdef __cplusplus")
			a_file.put_line ("}")
			a_file.put_line (c_endif)
		end

feature {NONE} -- Indentation

	indentation: INTEGER
			-- Indentation in `current_file'

	indent
			-- Increment indentation.
		do
			indentation := indentation + 1
		end

	dedent
			-- Decrement indentation.
		do
			indentation := indentation - 1
		end

	print_indentation
			-- Print indentation to `current_file'.
		local
			i, nb: INTEGER
		do
			nb := indentation
			from i := 1 until i > nb loop
				current_file.put_character ('%T')
				i := i + 1
			end
		end

feature {NONE} -- Include files

	include_file (a_filename: STRING; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Include content of file `a_filename' to `a_file'.
		require
			a_filename_not_void: a_filename /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_other_file: KL_TEXT_INPUT_FILE
		do
			create l_other_file.make (a_filename)
			l_other_file.open_read
			if l_other_file.is_open_read then
				a_file.append (l_other_file)
				l_other_file.close
			else
				set_fatal_error
				report_cannot_read_error (a_filename)
			end
		end

	include_runtime_file (a_filename: STRING; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Include content of runtime file `a_filename' to `a_file'.
		require
			a_filename_not_void: a_filename /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_full_pathname: STRING
		do
			l_full_pathname := file_system.nested_pathname ("${GOBO}", <<"tool", "gec", "runtime", "c", a_filename>>)
			l_full_pathname := Execution_environment.interpreted_string (l_full_pathname)
			include_file (l_full_pathname, a_file)
		end

	include_header_filename (a_filename: STRING; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Include header filename `a_filename' to `a_file'.
		require
			a_filename_not_void: a_filename /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if not included_header_filenames.has (a_filename) then
				if included_cpp_header_filenames.has (a_filename) then
					included_cpp_header_filenames.remove (a_filename)
					included_header_filenames.force_last (a_filename)
				elseif a_filename.same_string ("%"eif_cecil.h%"") then
					include_runtime_header_file ("eif_cecil.h", False, a_file)
				elseif a_filename.same_string ("%"eif_console.h%"") then
					include_runtime_header_file ("eif_console.h", False, a_file)
				elseif a_filename.same_string ("%"eif_constants.h%"") then
					include_runtime_header_file ("eif_constants.h", False, a_file)
				elseif a_filename.same_string ("%"eif_dir.h%"") then
					include_runtime_header_file ("eif_dir.h", False, a_file)
				elseif a_filename.same_string ("%"eif_eiffel.h%"") then
					include_runtime_header_file ("eif_eiffel.h", False, a_file)
				elseif a_filename.same_string ("%"eif_except.h%"") then
					include_runtime_header_file ("eif_except.h", False, a_file)
				elseif a_filename.same_string ("%"eif_file.h%"") then
					include_runtime_header_file ("eif_file.h", False, a_file)
				elseif a_filename.same_string ("%"eif_globals.h%"") then
					include_runtime_header_file ("eif_globals.h", False, a_file)
				elseif a_filename.same_string ("%"eif_hector.h%"") then
					include_runtime_header_file ("eif_hector.h", False, a_file)
				elseif a_filename.same_string ("%"eif_lmalloc.h%"") then
					include_runtime_header_file ("eif_lmalloc.h", False, a_file)
				elseif a_filename.same_string ("%"eif_main.h%"") then
					include_runtime_header_file ("eif_main.h", False, a_file)
				elseif a_filename.same_string ("%"eif_memory.h%"") then
					include_runtime_header_file ("eif_memory.h", False, a_file)
				elseif a_filename.same_string ("%"eif_misc.h%"") then
					include_runtime_header_file ("eif_misc.h", False, a_file)
				elseif a_filename.same_string ("%"eif_path_name.h%"") then
					include_runtime_header_file ("eif_path_name.h", False, a_file)
				elseif a_filename.same_string ("%"eif_plug.h%"") then
					include_runtime_header_file ("eif_plug.h", False, a_file)
				elseif a_filename.same_string ("%"eif_portable.h%"") then
					include_runtime_header_file ("eif_portable.h", False, a_file)
				elseif a_filename.same_string ("%"eif_retrieve.h%"") then
					include_runtime_header_file ("eif_retrieve.h", False, a_file)
				elseif a_filename.same_string ("<eif_retrieve.h>") then
						-- This notation is used in class MISMATCH_INFORMATION.
					include_runtime_header_file ("eif_retrieve.h", False, a_file)
				elseif a_filename.same_string ("%"eif_sig.h%"") then
					include_runtime_header_file ("eif_sig.h", False, a_file)
				elseif a_filename.same_string ("%"eif_size.h%"") then
					include_runtime_header_file ("eif_size.h", False, a_file)
				elseif a_filename.same_string ("%"eif_store.h%"") then
					include_runtime_header_file ("eif_store.h", False, a_file)
				elseif a_filename.same_string ("<eif_system.h>") then
					include_runtime_header_file ("eif_system.h", False, a_file)
				elseif a_filename.same_string ("%"eif_threads.h%"") then
					include_runtime_header_file ("eif_threads.h", False, a_file)
				elseif a_filename.same_string ("<eif_threads.h>") then
						-- This notation is used in class THREAD_ENVIRONMENT.
					include_runtime_header_file ("eif_threads.h", False, a_file)
				elseif a_filename.same_string ("%"eif_traverse.h%"") then
					include_runtime_header_file ("eif_traverse.h", False, a_file)
				elseif a_filename.same_string ("%"eif_types.h%"") then
					include_runtime_header_file ("eif_types.h", False, a_file)
				elseif a_filename.same_string ("%"ge_time.h%"") then
					include_runtime_header_file ("ge_time.h", False, a_file)
				else
					included_header_filenames.force_last (a_filename)
				end
			end
		end

	include_cpp_header_filename (a_filename: STRING; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Include C++ header filename `a_filename' to `a_file'.
		require
			a_filename_not_void: a_filename /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			if not included_cpp_header_filenames.has (a_filename) and not included_header_filenames.has (a_filename) then
				include_header_filename (a_filename, a_file)
				if included_header_filenames.has (a_filename) then
					included_header_filenames.remove (a_filename)
					included_cpp_header_filenames.force_last (a_filename)
				end
			end
		end

	include_runtime_header_file (a_filename: STRING; a_force: BOOLEAN; a_file: KI_TEXT_OUTPUT_STREAM)
			-- Add runtime header file `a_filename' to `included_runtime_header_files',
			-- and the header files it depends on. Also add associated C file,
			-- if any, to `included_runtime_c_files'.
			-- `a_force' means that the file should be written now to `a_file'.
		require
			a_filename_not_void: a_filename /= Void
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		local
			l_c_filename: detachable STRING
			l_has_file: BOOLEAN
		do
			l_has_file := included_runtime_header_files.has (a_filename)
			if not l_has_file or else (a_force and then not included_runtime_header_files.item (a_filename)) then
				if a_filename.same_string ("eif_cecil.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "eif_cecil.c"
				elseif a_filename.same_string ("eif_console.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "eif_console.c"
				elseif a_filename.same_string ("eif_dir.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "eif_dir.c"
				elseif a_filename.same_string ("eif_eiffel.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					include_runtime_header_file ("ge_string.h", a_force, a_file)
					include_runtime_header_file ("eif_globals.h", a_force, a_file)
					include_runtime_header_file ("eif_except.h", a_force, a_file)
				elseif a_filename.same_string ("eif_except.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					include_runtime_header_file ("ge_exception.h", a_force, a_file)
					l_c_filename := "eif_except.c"
				elseif a_filename.same_string ("eif_file.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "eif_file.c"
				elseif a_filename.same_string ("eif_globals.h") then
					include_runtime_header_file ("ge_exception.h", a_force, a_file)
					include_runtime_header_file ("eif_threads.h", a_force, a_file)
					include_runtime_header_file ("eif_size.h", a_force, a_file)
				elseif a_filename.same_string ("eif_main.h") then
					l_c_filename := "eif_main.c"
				elseif a_filename.same_string ("eif_memory.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "eif_memory.c"
				elseif a_filename.same_string ("eif_misc.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					include_runtime_header_file ("ge_dll.h", a_force, a_file)
					l_c_filename := "eif_misc.c"
				elseif a_filename.same_string ("eif_path_name.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "eif_path_name.c"
				elseif a_filename.same_string ("eif_plug.h") then
					l_c_filename := "eif_plug.c"
				elseif a_filename.same_string ("eif_project.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
				elseif a_filename.same_string ("eif_retrieve.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "eif_retrieve.c"
				elseif a_filename.same_string ("eif_sig.h") then
					l_c_filename := "eif_sig.c"
				elseif a_filename.same_string ("eif_store.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "eif_store.c"
				elseif a_filename.same_string ("eif_threads.h") then
					include_runtime_header_file ("eif_cecil.h", a_force, a_file)
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					if multithreaded_mode then
						include_runtime_header_file ("ge_thread.h", False, a_file)
					end
				elseif a_filename.same_string ("eif_traverse.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "eif_traverse.c"
				elseif a_filename.same_string ("ge_arguments.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "ge_arguments.c"
				elseif a_filename.same_string ("ge_com_failure.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "ge_com_failure.c"
				elseif a_filename.same_string ("ge_console.h") then
					l_c_filename := "ge_console.c"
				elseif a_filename.same_string ("ge_deep.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "ge_deep.c"
				elseif a_filename.same_string ("ge_exception.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					include_runtime_header_file ("ge_once.h", a_force, a_file)
					if multithreaded_mode then
						include_runtime_header_file ("ge_thread_types.h", a_force, a_file)
					end
					l_c_filename := "ge_exception.c"
				elseif a_filename.same_string ("ge_gc.h") then
					include_runtime_header_file ("ge_exception.h", a_force, a_file)
					l_c_filename := "ge_gc.c"
				elseif a_filename.same_string ("ge_identified.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "ge_identified.c"
				elseif a_filename.same_string ("ge_main.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "ge_main.c"
				elseif a_filename.same_string ("ge_native_string.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
				elseif a_filename.same_string ("ge_once.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "ge_once.c"
				elseif a_filename.same_string ("ge_real.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "ge_real.c"
				elseif a_filename.same_string ("ge_string.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					l_c_filename := "ge_string.c"
				elseif a_filename.same_string ("ge_thread.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					include_runtime_header_file ("ge_thread_types.h", a_force, a_file)
					include_runtime_header_file ("ge_exception.h", a_force, a_file)
					l_c_filename := "ge_thread.c"
				elseif a_filename.same_string ("ge_thread_types.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
				elseif a_filename.same_string ("ge_types.h") then
					include_runtime_header_file ("ge_eiffel.h", a_force, a_file)
					include_runtime_header_file ("ge_exception.h", a_force, a_file)
					l_c_filename := "ge_types.c"
				end
				if a_force then
					include_runtime_file (a_filename, a_file)
					if l_has_file then
						included_runtime_header_files.replace (True, a_filename)
					else
						included_runtime_header_files.force_last (True, a_filename)
					end
				else
					included_runtime_header_files.force_last (False, a_filename)
				end
				if l_c_filename /= Void then
					include_runtime_c_file (l_c_filename, a_file)
				end
			end
		ensure
			included: included_runtime_header_files.has (a_filename)
		end

	include_runtime_c_file (a_filename: STRING; a_header_file: KI_TEXT_OUTPUT_STREAM)
			-- Add runtime C file `a_filename' to `included_runtime_c_files'.
			-- Also add the header files it depends on to `included_runtime_header_files'.
			-- `a_header_file' is where to include these header files.
		require
			a_filename_not_void: a_filename /= Void
			a_header_file_not_void: a_header_file /= Void
			a_header_file_open_write: a_header_file.is_open_write
		do
			if not included_runtime_c_files.has (a_filename) then
				if a_filename.same_string ("eif_cecil.c") then
					include_runtime_header_file ("eif_cecil.h", False, a_header_file)
					include_runtime_header_file ("ge_console.h", False, a_header_file)
				elseif a_filename.same_string ("eif_console.c") then
					include_runtime_header_file ("eif_console.h", False, a_header_file)
					include_runtime_header_file ("eif_file.h", False, a_header_file)
					include_runtime_header_file ("ge_console.h", False, a_header_file)
				elseif a_filename.same_string ("eif_dir.c") then
					include_runtime_header_file ("eif_dir.h", False, a_header_file)
				elseif a_filename.same_string ("eif_except.c") then
					include_runtime_header_file ("eif_except.h", False, a_header_file)
					include_runtime_header_file ("ge_string.h", False, a_header_file)
				elseif a_filename.same_string ("eif_file.c") then
					include_runtime_header_file ("eif_file.h", False, a_header_file)
					include_runtime_header_file ("eif_except.h", False, a_header_file)
					include_runtime_header_file ("ge_string.h", False, a_header_file)
					include_runtime_header_file ("ge_real.h", False, a_header_file)
				elseif a_filename.same_string ("eif_main.c") then
					include_runtime_header_file ("eif_main.h", False, a_header_file)
				elseif a_filename.same_string ("eif_memory.c") then
					include_runtime_header_file ("eif_memory.h", False, a_header_file)
					include_runtime_header_file ("ge_gc.h", False, a_header_file)
				elseif a_filename.same_string ("eif_misc.c") then
					include_runtime_header_file ("eif_misc.h", False, a_header_file)
				elseif a_filename.same_string ("eif_path_name.c") then
					include_runtime_header_file ("eif_path_name.h", False, a_header_file)
					include_runtime_header_file ("ge_string.h", False, a_header_file)
				elseif a_filename.same_string ("eif_plug.c") then
					include_runtime_header_file ("eif_plug.h", False, a_header_file)
				elseif a_filename.same_string ("eif_retrieve.c") then
					include_runtime_header_file ("eif_retrieve.h", False, a_header_file)
					include_runtime_header_file ("ge_console.h", False, a_header_file)
				elseif a_filename.same_string ("eif_sig.c") then
					include_runtime_header_file ("eif_sig.h", False, a_header_file)
					include_runtime_header_file ("ge_console.h", False, a_header_file)
				elseif a_filename.same_string ("eif_store.c") then
					include_runtime_header_file ("eif_store.h", False, a_header_file)
					include_runtime_header_file ("ge_console.h", False, a_header_file)
				elseif a_filename.same_string ("eif_traverse.c") then
					include_runtime_header_file ("eif_traverse.h", False, a_header_file)
					include_runtime_header_file ("ge_console.h", False, a_header_file)
				elseif a_filename.same_string ("ge_arguments.c") then
					include_runtime_header_file ("ge_arguments.h", False, a_header_file)
				elseif a_filename.same_string ("ge_com_failure.c") then
					include_runtime_header_file ("ge_com_failure.h", False, a_header_file)
				elseif a_filename.same_string ("ge_console.c") then
					include_runtime_header_file ("ge_console.h", False, a_header_file)
					include_runtime_header_file ("ge_eiffel.h", False, a_header_file)
					include_runtime_header_file ("ge_exception.h", False, a_header_file)
					if multithreaded_mode then
						include_runtime_header_file ("ge_thread.h", False, a_header_file)
					end
				elseif a_filename.same_string ("ge_deep.c") then
					include_runtime_header_file ("ge_deep.h", False, a_header_file)
					include_runtime_header_file ("ge_gc.h", False, a_header_file)
				elseif a_filename.same_string ("ge_exception.c") then
					include_runtime_header_file ("ge_exception.h", False, a_header_file)
					include_runtime_header_file ("ge_gc.h", False, a_header_file)
					include_runtime_header_file ("ge_string.h", False, a_header_file)
					include_runtime_header_file ("ge_console.h", False, a_header_file)
					if multithreaded_mode then
						include_runtime_header_file ("ge_thread.h", False, a_header_file)
					end
				elseif a_filename.same_string ("ge_gc.c") then
					include_runtime_header_file ("ge_gc.h", False, a_header_file)
					include_runtime_header_file ("ge_types.h", False, a_header_file)
				elseif a_filename.same_string ("ge_identified.c") then
					include_runtime_header_file ("ge_identified.h", False, a_header_file)
					include_runtime_header_file ("ge_gc.h", False, a_header_file)
				elseif a_filename.same_string ("ge_main.c") then
					include_runtime_header_file ("ge_main.h", False, a_header_file)
					include_runtime_header_file ("ge_arguments.h", False, a_header_file)
					include_runtime_header_file ("ge_native_string.h", False, a_header_file)
					include_runtime_header_file ("ge_gc.h", False, a_header_file)
				elseif a_filename.same_string ("ge_once.c") then
					include_runtime_header_file ("ge_once.h", False, a_header_file)
					include_runtime_header_file ("ge_gc.h", False, a_header_file)
					if multithreaded_mode then
						include_runtime_header_file ("ge_thread.h", False, a_header_file)
					end
				elseif a_filename.same_string ("ge_real.c") then
					include_runtime_header_file ("ge_real.h", False, a_header_file)
				elseif a_filename.same_string ("ge_string.c") then
					include_runtime_header_file ("ge_string.h", False, a_header_file)
					include_runtime_header_file ("ge_native_string.h", False, a_header_file)
					include_runtime_header_file ("ge_gc.h", False, a_header_file)
				elseif a_filename.same_string ("ge_thread.c") then
					include_runtime_header_file ("ge_thread.h", False, a_header_file)
					include_runtime_header_file ("ge_gc.h", False, a_header_file)
					include_runtime_header_file ("ge_once.h", False, a_header_file)
					include_runtime_header_file ("ge_time.h", False, a_header_file)
				elseif a_filename.same_string ("ge_types.c") then
					include_runtime_header_file ("ge_types.h", False, a_header_file)
					include_runtime_header_file ("ge_string.h", False, a_header_file)
				end
				included_runtime_c_files.force_last (a_filename)
			end
		ensure
			included: included_runtime_c_files.has (a_filename)
		end

	included_header_filenames: DS_HASH_SET [STRING]
			-- Name of header filenames already included

	included_cpp_header_filenames: DS_HASH_SET [STRING]
			-- Name of C++ header filenames already included

	included_runtime_header_files: DS_HASH_TABLE [BOOLEAN, STRING]
			-- Name of runtime header files already included;
			-- True means that it has already been printed

	included_runtime_c_files: DS_HASH_SET [STRING]
			-- Name of runtime C files already included

feature {NONE} -- Output files/buffers

	header_file: KI_TEXT_OUTPUT_STREAM
			-- Header file

	c_file: detachable KL_TEXT_OUTPUT_FILE
			-- C file
			-- (May be Void if not open yet.)

	c_file_size: INTEGER
			-- Number of bytes already written to `c_file'

	c_filenames: DS_HASH_TABLE [STRING, STRING]
			-- List of C (or C++) filenames generated;
			-- The key is the filename without the extension,
			-- the item is the file extension

	open_c_file
			-- Open C file if necessary.
		do
		end

	flush_to_c_file
			-- Open C file if not already done, and then
			-- flush to C file the code written in buffers.
			-- Take into account where we are in `split_mode' or not.
		local
			l_c_file: like c_file
			l_buffer: STRING
			l_filename: STRING
			l_header_filename: detachable STRING
		do
			l_c_file := c_file
			if l_c_file = Void then
				c_file_size := 0
				l_header_filename := system_name + h_file_extension
				l_filename := system_name + (c_filenames.count + 1).out
				c_filenames.force_last (c_file_extension, l_filename)
				create l_c_file.make (l_filename + c_file_extension)
				l_c_file.open_write
				c_file := l_c_file
			elseif not l_c_file.is_open_write then
				l_c_file.open_append
			end
			if not l_c_file.is_open_write then
				set_fatal_error
				report_cannot_write_error (l_c_file.name)
				c_file := Void
			else
				if l_header_filename /= Void then
					l_c_file.put_string ("#include %"")
					l_c_file.put_string (l_header_filename)
					l_c_file.put_character ('%"')
					l_c_file.put_new_line
					l_c_file.put_new_line
					c_file_size := c_file_size + l_header_filename.count + 13
					print_start_extern_c (l_c_file)
					c_file_size := c_file_size + 40
				end
				l_buffer := current_function_header_buffer.string
				l_c_file.put_string (l_buffer)
				c_file_size := c_file_size + l_buffer.count
				STRING_.wipe_out (l_buffer)
				l_buffer := current_function_body_buffer.string
				l_c_file.put_string (l_buffer)
				c_file_size := c_file_size + l_buffer.count
				STRING_.wipe_out (l_buffer)
				if split_mode and then c_file_size >= split_threshold then
					close_c_file
				end
			end
		ensure
			flushed1: not has_fatal_error implies current_function_header_buffer.string.is_empty
			flushed2: not has_fatal_error implies current_function_body_buffer.string.is_empty
		end

	close_c_file
			-- Close C file if not already done.
		do
			if attached c_file as l_c_file and then not l_c_file.is_closed then
				print_end_extern_c (l_c_file)
				l_c_file.close
			end
			c_file := Void
			c_file_size := 0
		ensure
			c_file_reset: c_file = Void
			c_file_size_reset: c_file_size = 0
		end

	cpp_file: detachable KL_TEXT_OUTPUT_FILE
			-- C++ file
			-- (May be Void if not open yet.)

	cpp_file_size: INTEGER
			-- Number of bytes already written to `cpp_file'

	open_cpp_file
			-- Open C++ file if necessary.
		do
		end

	flush_to_cpp_file
			-- Open C++ file if not already done, and then
			-- flush to C++ file the code written in buffers.
			-- Take into account where we are in `split_mode' or not.
		local
			l_buffer: STRING
			l_filename: STRING
			l_header_filename: detachable STRING
			l_cpp_file: like cpp_file
		do
			l_cpp_file := cpp_file
			if l_cpp_file = Void then
				cpp_file_size := 0
				l_header_filename := system_name + h_file_extension
				l_filename := system_name + (c_filenames.count + 1).out
				c_filenames.force_last (cpp_file_extension, l_filename)
				create l_cpp_file.make (l_filename + cpp_file_extension)
				l_cpp_file.open_write
				cpp_file := l_cpp_file
			elseif not l_cpp_file.is_open_write then
				l_cpp_file.open_append
			end
			if not l_cpp_file.is_open_write then
				set_fatal_error
				report_cannot_write_error (l_cpp_file.name)
				cpp_file := Void
			else
				if l_header_filename /= Void then
					l_cpp_file.put_string ("#include %"")
					l_cpp_file.put_string (l_header_filename)
					l_cpp_file.put_character ('%"')
					l_cpp_file.put_new_line
					l_cpp_file.put_new_line
					cpp_file_size := cpp_file_size + l_header_filename.count + 13
					print_start_extern_c (l_cpp_file)
					cpp_file_size := cpp_file_size + 40
				end
				l_buffer := current_function_header_buffer.string
				l_cpp_file.put_string (l_buffer)
				cpp_file_size := cpp_file_size + l_buffer.count
				STRING_.wipe_out (l_buffer)
				l_buffer := current_function_body_buffer.string
				l_cpp_file.put_string (l_buffer)
				cpp_file_size := cpp_file_size + l_buffer.count
				STRING_.wipe_out (l_buffer)
				if split_mode and then cpp_file_size >= split_threshold then
					close_cpp_file
				end
			end
		ensure
			flushed1: not has_fatal_error implies current_function_header_buffer.string.is_empty
			flushed2: not has_fatal_error implies current_function_body_buffer.string.is_empty
		end

	close_cpp_file
			-- Close C++ file if not already done.
		do
			if attached cpp_file as l_cpp_file and then not l_cpp_file.is_closed then
				print_end_extern_c (l_cpp_file)
				l_cpp_file.close
			end
			cpp_file := Void
			cpp_file_size := 0
		ensure
			cpp_file_reset: cpp_file = Void
			cpp_file_size_reset: cpp_file_size = 0
		end

	current_file: KI_TEXT_OUTPUT_STREAM
			-- Output file;
			-- In fact, it's not a file but a buffer (either `current_function_header_buffer'
			-- or `current_function_body_buffer'), which is then flushed to either a C or
			-- C++ file.

	current_function_header_buffer: KL_STRING_OUTPUT_STREAM
			-- Buffer to write the C header of the current function;
			-- This is useful when we need to declare local variables
			-- on the fly while generating the code for the body of
			-- the C function.

	current_function_body_buffer: KL_STRING_OUTPUT_STREAM
			-- Buffer to write the C body of the current function

feature {ET_AST_NODE} -- Processing

	process_across_expression (an_expression: ET_ACROSS_EXPRESSION)
			-- Process `an_expression'.
		do
			print_across_expression (an_expression)
		end

	process_across_instruction (an_instruction: ET_ACROSS_INSTRUCTION)
			-- Process `an_instruction'.
		do
			print_across_instruction (an_instruction)
		end

	process_assigner_instruction (an_instruction: ET_ASSIGNER_INSTRUCTION)
			-- Process `an_instruction'.
		do
			print_assigner_instruction (an_instruction)
		end

	process_assignment (an_instruction: ET_ASSIGNMENT)
			-- Process `an_instruction'.
		do
			print_assignment (an_instruction)
		end

	process_assignment_attempt (an_instruction: ET_ASSIGNMENT_ATTEMPT)
			-- Process `an_instruction'.
		do
			print_assignment_attempt (an_instruction)
		end

	process_attribute (a_feature: ET_ATTRIBUTE)
			-- Process `a_feature'.
		do
			print_attribute (a_feature)
		end

	process_bang_instruction (an_instruction: ET_BANG_INSTRUCTION)
			-- Process `an_instruction'.
		do
			print_bang_instruction (an_instruction)
		end

	process_binary_integer_constant (a_constant: ET_BINARY_INTEGER_CONSTANT)
			-- Process `a_constant'.
		do
			print_binary_integer_constant (a_constant)
		end

	process_bracket_expression (an_expression: ET_BRACKET_EXPRESSION)
			-- Process `an_expression'.
		do
			print_bracket_expression (an_expression)
		end

	process_c1_character_constant (a_constant: ET_C1_CHARACTER_CONSTANT)
			-- Process `a_constant'.
		do
			print_character_constant (a_constant)
		end

	process_c2_character_constant (a_constant: ET_C2_CHARACTER_CONSTANT)
			-- Process `a_constant'.
		do
			print_character_constant (a_constant)
		end

	process_c3_character_constant (a_constant: ET_C3_CHARACTER_CONSTANT)
			-- Process `a_constant'.
		do
			print_character_constant (a_constant)
		end

	process_call_agent (an_expression: ET_CALL_AGENT)
			-- Process `an_expression'.
		do
			if an_expression = current_agent then
				print_call_agent_body_declaration (an_expression)
			else
				print_call_agent (an_expression)
			end
		end

	process_check_instruction (an_instruction: ET_CHECK_INSTRUCTION)
			-- Process `an_instruction'.
		do
			print_check_instruction (an_instruction)
		end

	process_constant_attribute (a_feature: ET_CONSTANT_ATTRIBUTE)
			-- Process `a_feature'.
		do
			print_constant_attribute (a_feature)
		end

	process_convert_builtin_expression (an_expression: ET_CONVERT_BUILTIN_EXPRESSION)
			-- Process `an_expression'.
		do
			print_convert_builtin_expression (an_expression)
		end

	process_convert_from_expression (an_expression: ET_CONVERT_FROM_EXPRESSION)
			-- Process `an_expression'.
		do
			print_convert_from_expression (an_expression)
		end

	process_convert_to_expression (an_expression: ET_CONVERT_TO_EXPRESSION)
			-- Process `an_expression'.
		do
			print_convert_to_expression (an_expression)
		end

	process_create_expression (an_expression: ET_CREATE_EXPRESSION)
			-- Process `an_expression'.
		do
			print_create_expression (an_expression)
		end

	process_create_instruction (an_instruction: ET_CREATE_INSTRUCTION)
			-- Process `an_instruction'.
		do
			print_create_instruction (an_instruction)
		end

	process_current (an_expression: ET_CURRENT)
			-- Process `an_expression'.
		do
			print_current (an_expression)
		end

	process_current_address (an_expression: ET_CURRENT_ADDRESS)
			-- Process `an_expression'.
		do
			print_current_address (an_expression)
		end

	process_debug_instruction (an_instruction: ET_DEBUG_INSTRUCTION)
			-- Process `an_instruction'.
		do
			print_debug_instruction (an_instruction)
		end

	process_deferred_function (a_feature: ET_DEFERRED_FUNCTION)
			-- Process `a_feature'.
		do
			print_deferred_function (a_feature)
		end

	process_deferred_procedure (a_feature: ET_DEFERRED_PROCEDURE)
			-- Process `a_feature'.
		do
			print_deferred_procedure (a_feature)
		end

	process_do_function (a_feature: ET_DO_FUNCTION)
			-- Process `a_feature'.
		do
			print_do_function (a_feature)
		end

	process_do_function_inline_agent (an_agent: ET_DO_FUNCTION_INLINE_AGENT)
			-- Process `an_agent'.
		do
			if an_agent = current_agent then
				print_do_function_inline_agent_body_declaration (an_agent)
			else
				print_do_function_inline_agent (an_agent)
			end
		end

	process_do_procedure (a_feature: ET_DO_PROCEDURE)
			-- Process `a_feature'.
		do
			print_do_procedure (a_feature)
		end

	process_do_procedure_inline_agent (an_agent: ET_DO_PROCEDURE_INLINE_AGENT)
			-- Process `an_agent'.
		do
			if an_agent = current_agent then
				print_do_procedure_inline_agent_body_declaration (an_agent)
			else
				print_do_procedure_inline_agent (an_agent)
			end
		end

	process_equality_expression (an_expression: ET_EQUALITY_EXPRESSION)
			-- Process `an_expression'.
		do
			print_equality_expression (an_expression)
		end

	process_explicit_convert_from_expression (an_expression: ET_EXPLICIT_CONVERT_FROM_EXPRESSION)
			-- Process `an_expression'.
		do
			print_convert_from_expression (an_expression)
		end

	process_explicit_convert_to_expression (an_expression: ET_EXPLICIT_CONVERT_TO_EXPRESSION)
			-- Process `an_expression'.
		do
			print_convert_to_expression (an_expression)
		end

	process_expression_address (an_expression: ET_EXPRESSION_ADDRESS)
			-- Process `an_expression'.
		do
			print_expression_address (an_expression)
		end

	process_extended_attribute (a_feature: ET_EXTENDED_ATTRIBUTE)
			-- Process `a_feature'.
		do
			print_extended_attribute (a_feature)
		end

	process_external_function (a_feature: ET_EXTERNAL_FUNCTION)
			-- Process `a_feature'.
		do
			print_external_function (a_feature)
		end

	process_external_function_inline_agent (an_agent: ET_EXTERNAL_FUNCTION_INLINE_AGENT)
			-- Process `an_agent'.
		do
			if an_agent = current_agent then
				print_external_function_inline_agent_body_declaration (an_agent)
			else
				print_external_function_inline_agent (an_agent)
			end
		end

	process_external_procedure (a_feature: ET_EXTERNAL_PROCEDURE)
			-- Process `a_feature'.
		do
			print_external_procedure (a_feature)
		end

	process_external_procedure_inline_agent (an_agent: ET_EXTERNAL_PROCEDURE_INLINE_AGENT)
			-- Process `an_agent'.
		do
			if an_agent = current_agent then
				print_external_procedure_inline_agent_body_declaration (an_agent)
			else
				print_external_procedure_inline_agent (an_agent)
			end
		end

	process_false_constant (a_constant: ET_FALSE_CONSTANT)
			-- Process `a_constant'.
		do
			print_false_constant (a_constant)
		end

	process_feature_address (an_expression: ET_FEATURE_ADDRESS)
			-- Process `an_expression'.
		do
			print_feature_address (an_expression)
		end

	process_hexadecimal_integer_constant (a_constant: ET_HEXADECIMAL_INTEGER_CONSTANT)
			-- Process `a_constant'.
		do
			print_hexadecimal_integer_constant (a_constant)
		end

	process_identifier (an_identifier: ET_IDENTIFIER)
			-- Process `an_identifier'.
		do
			if an_identifier.is_argument then
				print_formal_argument (an_identifier)
			elseif an_identifier.is_temporary then
				print_temporary_variable (an_identifier)
			elseif an_identifier.is_local then
				print_local_variable (an_identifier)
			elseif an_identifier.is_object_test_local then
				print_object_test_local (an_identifier)
			elseif an_identifier.is_iteration_item then
				print_iteration_item (an_identifier)
			elseif an_identifier.is_agent_open_operand then
				print_agent_open_operand (an_identifier)
			elseif an_identifier.is_agent_closed_operand then
				print_agent_closed_operand (an_identifier)
			end
		end

	process_if_expression (a_expression: ET_IF_EXPRESSION)
			-- Process `a_expression'.
		do
			print_if_expression (a_expression)
		end

	process_if_instruction (an_instruction: ET_IF_INSTRUCTION)
			-- Process `an_instruction'.
		do
			print_if_instruction (an_instruction)
		end

	process_infix_cast_expression (an_expression: ET_INFIX_CAST_EXPRESSION)
			-- Process `an_expression'.
		do
			print_infix_cast_expression (an_expression)
		end

	process_infix_expression (an_expression: ET_INFIX_EXPRESSION)
			-- Process `an_expression'.
		do
			print_infix_expression (an_expression)
		end

	process_inspect_expression (a_expression: ET_INSPECT_EXPRESSION)
			-- Process `a_expression'.
		do
			print_inspect_expression (a_expression)
		end

	process_inspect_instruction (an_instruction: ET_INSPECT_INSTRUCTION)
			-- Process `an_instruction'.
		do
			print_inspect_instruction (an_instruction)
		end

	process_iteration_cursor (a_iteration_cursor: ET_ITERATION_CURSOR)
			-- Process `a_iteration_cursor'.
		do
			print_iteration_cursor (a_iteration_cursor)
		end

	process_loop_instruction (an_instruction: ET_LOOP_INSTRUCTION)
			-- Process `an_instruction'.
		do
			print_loop_instruction (an_instruction)
		end

	process_manifest_array (an_expression: ET_MANIFEST_ARRAY)
			-- Process `an_expression'.
		do
			print_manifest_array (an_expression)
		end

	process_manifest_tuple (an_expression: ET_MANIFEST_TUPLE)
			-- Process `an_expression'.
		do
			print_manifest_tuple (an_expression)
		end

	process_manifest_type (an_expression: ET_MANIFEST_TYPE)
			-- Process `an_expression'.
		do
			print_manifest_type (an_expression)
		end

	process_named_object_test (an_expression: ET_NAMED_OBJECT_TEST)
			-- Process `an_expression'.
		do
			print_object_test (an_expression)
		end

	process_object_equality_expression (an_expression: ET_OBJECT_EQUALITY_EXPRESSION)
			-- Process `an_expression'.
		do
			print_object_equality_expression (an_expression)
		end

	process_object_test (an_expression: ET_OBJECT_TEST)
			-- Process `an_expression'.
		do
			print_object_test (an_expression)
		end

	process_octal_integer_constant (a_constant: ET_OCTAL_INTEGER_CONSTANT)
			-- Process `a_constant'.
		do
			print_octal_integer_constant (a_constant)
		end

	process_old_expression (an_expression: ET_OLD_EXPRESSION)
			-- Process `an_expression'.
		do
			print_old_expression (an_expression)
		end

	process_old_object_test (an_expression: ET_OLD_OBJECT_TEST)
			-- Process `an_expression'.
		do
			print_object_test (an_expression)
		end

	process_once_function (a_feature: ET_ONCE_FUNCTION)
			-- Process `a_feature'.
		do
			print_once_function (a_feature)
		end

	process_once_function_inline_agent (an_agent: ET_ONCE_FUNCTION_INLINE_AGENT)
			-- Process `an_agent'.
		do
			if an_agent = current_agent then
				print_once_function_inline_agent_body_declaration (an_agent)
			else
				print_once_function_inline_agent (an_agent)
			end
		end

	process_once_manifest_string (an_expression: ET_ONCE_MANIFEST_STRING)
			-- Process `an_expression'.
		do
			print_once_manifest_string (an_expression)
		end

	process_once_procedure (a_feature: ET_ONCE_PROCEDURE)
			-- Process `a_feature'.
		do
			print_once_procedure (a_feature)
		end

	process_once_procedure_inline_agent (an_agent: ET_ONCE_PROCEDURE_INLINE_AGENT)
			-- Process `an_agent'.
		do
			if an_agent = current_agent then
				print_once_procedure_inline_agent_body_declaration (an_agent)
			else
				print_once_procedure_inline_agent (an_agent)
			end
		end

	process_parenthesis_expression (an_expression: ET_PARENTHESIS_EXPRESSION)
			-- Process `an_expression'.
		do
			print_qualified_call_expression (an_expression)
		end

	process_parenthesis_instruction (an_instruction: ET_PARENTHESIS_INSTRUCTION)
			-- Process `an_instruction'.
		do
			print_qualified_call_instruction (an_instruction)
		end

	process_parenthesized_expression (an_expression: ET_PARENTHESIZED_EXPRESSION)
			-- Process `an_expression'.
		do
			print_parenthesized_expression (an_expression)
		end

	process_precursor_expression (an_expression: ET_PRECURSOR_EXPRESSION)
			-- Process `an_expression'.
		do
			if attached an_expression.parenthesis_call as l_parenthesis_call then
				print_qualified_call_expression (l_parenthesis_call)
			else
				print_precursor_expression (an_expression)
			end
		end

	process_precursor_instruction (an_instruction: ET_PRECURSOR_INSTRUCTION)
			-- Process `an_instruction'.
		do
			if attached an_instruction.parenthesis_call as l_parenthesis_call then
				print_qualified_call_instruction (l_parenthesis_call)
			else
				print_precursor_instruction (an_instruction)
			end
		end

	process_prefix_expression (an_expression: ET_PREFIX_EXPRESSION)
			-- Process `an_expression'.
		do
			print_prefix_expression (an_expression)
		end

	process_qualified_call_expression (an_expression: ET_QUALIFIED_CALL_EXPRESSION)
			-- Process `an_expression'.
		do
			if attached an_expression.parenthesis_call as l_parenthesis_call then
				print_qualified_call_expression (l_parenthesis_call)
			else
				print_qualified_call_expression (an_expression)
			end
		end

	process_qualified_call_instruction (an_instruction: ET_QUALIFIED_CALL_INSTRUCTION)
			-- Process `an_instruction'.
		do
			if attached an_instruction.parenthesis_call as l_parenthesis_call then
				print_qualified_call_instruction (l_parenthesis_call)
			else
				print_qualified_call_instruction (an_instruction)
			end
		end

	process_quantifier_expression (a_expression: ET_QUANTIFIER_EXPRESSION)
			-- Process `a_expression'.
		do
			print_quantifier_expression (a_expression)
		end

	process_regular_integer_constant (a_constant: ET_REGULAR_INTEGER_CONSTANT)
			-- Process `a_constant'.
		do
			print_regular_integer_constant (a_constant)
		end

	process_regular_manifest_string (a_string: ET_REGULAR_MANIFEST_STRING)
			-- Process `a_string'.
		do
			print_regular_manifest_string (a_string)
		end

	process_regular_real_constant (a_constant: ET_REGULAR_REAL_CONSTANT)
			-- Process `a_constant'.
		do
			print_regular_real_constant (a_constant)
		end

	process_repeat_instruction (a_instruction: ET_REPEAT_INSTRUCTION)
			-- Process `a_instruction'.
		do
			print_repeat_instruction (a_instruction)
		end

	process_result (an_expression: ET_RESULT)
			-- Process `an_expression'.
		do
			print_result (an_expression)
		end

	process_result_address (an_expression: ET_RESULT_ADDRESS)
			-- Process `an_expression'.
		do
			print_result_address (an_expression)
		end

	process_retry_instruction (an_instruction: ET_RETRY_INSTRUCTION)
			-- Process `an_instruction'.
		do
			print_retry_instruction (an_instruction)
		end

	process_semicolon_symbol (a_symbol: ET_SEMICOLON_SYMBOL)
			-- Process `a_symbol'.
		do
			-- Do nothing.
		end

	process_special_manifest_string (a_string: ET_SPECIAL_MANIFEST_STRING)
			-- Process `a_string'.
		do
			print_special_manifest_string (a_string)
		end

	process_static_call_expression (an_expression: ET_STATIC_CALL_EXPRESSION)
			-- Process `an_expression'.
		do
			if attached an_expression.parenthesis_call as l_parenthesis_call then
				print_qualified_call_expression (l_parenthesis_call)
			else
				print_static_call_expression (an_expression)
			end
		end

	process_static_call_instruction (an_instruction: ET_STATIC_CALL_INSTRUCTION)
			-- Process `an_instruction'.
		do
			if attached an_instruction.parenthesis_call as l_parenthesis_call then
				print_qualified_call_instruction (l_parenthesis_call)
			else
				print_static_call_instruction (an_instruction)
			end
		end

	process_strip_expression (an_expression: ET_STRIP_EXPRESSION)
			-- Process `an_expression'.
		do
			print_strip_expression (an_expression)
		end

	process_true_constant (a_constant: ET_TRUE_CONSTANT)
			-- Process `a_constant'.
		do
			print_true_constant (a_constant)
		end

	process_underscored_integer_constant (a_constant: ET_UNDERSCORED_INTEGER_CONSTANT)
			-- Process `a_constant'.
		do
			print_underscored_integer_constant (a_constant)
		end

	process_underscored_real_constant (a_constant: ET_UNDERSCORED_REAL_CONSTANT)
			-- Process `a_constant'.
		do
			print_underscored_real_constant (a_constant)
		end

	process_unique_attribute (a_feature: ET_UNIQUE_ATTRIBUTE)
			-- Process `a_feature'.
		do
			print_unique_attribute (a_feature)
		end

	process_unqualified_call_expression (an_expression: ET_UNQUALIFIED_CALL_EXPRESSION)
			-- Process `an_expression'.
		do
			if attached an_expression.parenthesis_call as l_parenthesis_call then
				print_qualified_call_expression (l_parenthesis_call)
			else
				print_unqualified_call_expression (an_expression)
			end
		end

	process_unqualified_call_instruction (an_instruction: ET_UNQUALIFIED_CALL_INSTRUCTION)
			-- Process `an_instruction'.
		do
			if attached an_instruction.parenthesis_call as l_parenthesis_call then
				print_qualified_call_instruction (l_parenthesis_call)
			else
				print_unqualified_call_instruction (an_instruction)
			end
		end

	process_verbatim_string (a_string: ET_VERBATIM_STRING)
			-- Process `a_string'.
		do
			print_verbatim_string (a_string)
		end

	process_void (an_expression: ET_VOID)
			-- Process `an_expression'.
		do
			print_void (an_expression)
		end

feature {NONE} -- Error handling

	set_fatal_error
			-- Report a fatal error.
		do
			has_fatal_error := True
		ensure
			has_fatal_error: has_fatal_error
		end

	report_cannot_read_error (a_filename: STRING)
			-- Report that `a_filename' cannot be
			-- opened in read mode.
		require
			a_filename_not_void: a_filename /= Void
		local
			an_error: UT_CANNOT_READ_FILE_ERROR
		do
			create an_error.make (a_filename)
			error_handler.report_error (an_error)
		end

	report_cannot_write_error (a_filename: STRING)
			-- Report that `a_filename' cannot be
			-- opened in write mode.
		require
			a_filename_not_void: a_filename /= Void
		local
			an_error: UT_CANNOT_WRITE_TO_FILE_ERROR
		do
			create an_error.make (a_filename)
			error_handler.report_error (an_error)
		end

feature {NONE} -- Access

	system_name: STRING
			-- Name of the system being compiled

	current_feature: ET_DYNAMIC_FEATURE
			-- Feature being processed

	current_type: ET_DYNAMIC_PRIMARY_TYPE
			-- Type where `current_feature' belongs

	current_universe: ET_UNIVERSE
			-- Universe where the base class of `current_type' is declared
		do
			Result := current_type.base_class.universe
		ensure
			current_universe_not_void: Result /= Void
		end

	current_universe_impl: ET_UNIVERSE
			-- Universe where the class in which `current_feature' is written is declared
		do
			Result := current_feature.static_feature.implementation_class.universe
		ensure
			current_universe_impl_not_void: Result /= Void
		end

	current_agent: detachable ET_AGENT
			-- Agent being processed if any, Void otherwise

	current_agents: DS_ARRAYED_LIST [ET_AGENT]
			-- Agents already processed in `current_feature'

	current_object_tests: DS_ARRAYED_LIST [ET_OBJECT_TEST]
			-- Object-tests appearing in `current_feature' for which
			-- a function needs to be generated

	current_equalities: DS_ARRAYED_LIST [ET_DYNAMIC_EQUALITY_TYPES]
			-- Equalities ('=' or '/=', as well as '~' or '/~') appearing in
			-- `current_feature' for which  a function needs to be generated

	dynamic_types: DS_ARRAYED_LIST [ET_DYNAMIC_PRIMARY_TYPE]
			-- Dynamic types in the system

	manifest_array_types: DS_HASH_SET [ET_DYNAMIC_PRIMARY_TYPE]
			-- Types of manifest arrays

	big_manifest_array_types: DS_HASH_SET [ET_DYNAMIC_PRIMARY_TYPE]
			-- Types of big manifest arrays

	manifest_tuple_types: DS_HASH_SET [ET_DYNAMIC_TUPLE_TYPE]
			-- Types of manifest tuples

	constant_features: DS_HASH_TABLE [ET_CONSTANT, ET_FEATURE]
			-- Features returning a constant

	inline_constants: DS_HASH_TABLE [ET_DYNAMIC_PRIMARY_TYPE, ET_INLINE_CONSTANT]
			-- Inline constants (such as once manifest strings), with their types

	dynamic_type_id_set_names: DS_HASH_TABLE [STRING, STRING]
			-- Names of C arrays made up of dynamic type ids, indexed by those dynamic type ids;
			-- Those dynamic type ids which are used as keys are of the form
			-- "<type-id1>,<type-id2>,...,<type-idN>" and are sorted in increasing order.

feature {NONE} -- Once features

	once_features: DS_HASH_TABLE [INTEGER, ET_FEATURE]
			-- Once-per-process or once-per-thread features already generated,
			-- with their indexes in 'GE_onces' C struct's arrays.
			-- The key is the implementation feature of the once feature.

	register_once_feature (a_feature: ET_DYNAMIC_FEATURE)
			-- Register `a_feature' in `once_features' if it's a once-per-process
			-- on once-per-thread feature.
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
		local
			l_once_feature: ET_FEATURE
			l_is_once_per_process: BOOLEAN
			l_is_once_per_thread: BOOLEAN
			l_once_kind: INTEGER
			l_once_index: INTEGER
		do
			l_is_once_per_process := a_feature.is_once_per_process
			l_is_once_per_thread := a_feature.is_once_per_thread
			if l_is_once_per_process or l_is_once_per_thread then
				l_once_feature := a_feature.static_feature.implementation_feature
				once_features.search (l_once_feature)
				if not once_features.found then
					l_once_kind := once_kind (a_feature)
					if not multithreaded_mode or else l_is_once_per_process then
						l_once_index := once_per_process_counts.item (l_once_kind)
						once_per_process_counts.put (l_once_index + 1, l_once_kind)
					else
						l_once_index := once_per_thread_counts.item (l_once_kind)
						once_per_thread_counts.put (l_once_index + 1, l_once_kind)
					end
					once_features.force_last (l_once_index, l_once_feature)
				end
			end
		ensure
			registered: (a_feature.is_once_per_process or a_feature.is_once_per_thread) implies once_features.has (a_feature.static_feature.implementation_feature)
		end

	once_kind (a_feature: ET_DYNAMIC_FEATURE): INTEGER
			-- Once kind of `a_feature' in 'GE_onces' C struct,
			-- in the same order as in 'GE_onces' C struct
			-- (1 for boolean functions, 2 for character_8 functions,
			-- ..., 15 for reference functions and 16 for procedures)
		require
			a_feature_not_void: a_feature /= Void
			a_feature_is_once: a_feature.is_once
		local
			l_result_type: ET_DYNAMIC_TYPE
			l_base_class: ET_CLASS
		do
			if attached a_feature.result_type_set as l_result_type_set then
				l_result_type := l_result_type_set.static_type
				if l_result_type.is_basic then
					l_base_class := l_result_type.base_class
					if l_base_class.is_boolean_class then
						Result := 1
					elseif l_base_class.is_character_8_class then
						Result := 2
					elseif l_base_class.is_character_32_class then
						Result := 3
					elseif l_base_class.is_integer_8_class then
						Result := 4
					elseif l_base_class.is_integer_16_class then
						Result := 5
					elseif l_base_class.is_integer_32_class then
						Result := 6
					elseif l_base_class.is_integer_64_class then
						Result := 7
					elseif l_base_class.is_natural_8_class then
						Result := 8
					elseif l_base_class.is_natural_16_class then
						Result := 9
					elseif l_base_class.is_natural_32_class then
						Result := 10
					elseif l_base_class.is_natural_64_class then
						Result := 11
					elseif l_base_class.is_pointer_class then
						Result := 12
					elseif l_base_class.is_real_32_class then
						Result := 13
					elseif l_base_class.is_real_64_class then
						Result := 14
					end
				else
					Result := 15
				end
			else
				Result := 16
			end
		ensure
			valid_once_kind: Result >= 1 and Result <= once_kind_count
		end

	once_kind_count: INTEGER = 16
			-- Number of different once kinds in 'GE_onces' C struct

	once_per_process_counts: ARRAY [INTEGER]
			-- Number of once-per-process features for each kind of onces,
			-- in the same order as in 'GE_onces' C struct

	once_per_thread_counts: ARRAY [INTEGER]
			-- Number of once-per-thread features for each kind of onces,
			-- in the same order as in 'GE_onces' C struct

	once_prefixes: ARRAY [STRING]
			-- Prefix to be used when accessing information about
			-- once features in 'GE_onces' C struct, in the same
			-- order as in 'GE_onces' C struct
		once
			Result := <<"boolean",
				"character_8",
				"character_32",
				"integer_8",
				"integer_16",
				"integer_32",
				"integer_64",
				"natural_8",
				"natural_16",
				"natural_32",
				"natural_64",
				"pointer",
				"real_32",
				"real_64",
				"reference",
				"procedure">>
		ensure
			once_prefixes_not_void: Result /= Void
			no_void_prefix: across Result as i_prefix all i_prefix /= Void end
			lower: Result.lower = 1
			upper: Result.upper = once_kind_count
		end

feature {NONE} -- Dynamic type sets

	dynamic_type_set (an_operand: ET_OPERAND): ET_DYNAMIC_TYPE_SET
			-- Dynamic type set associated with `an_operand' in feature being printed;
			-- Report a fatal error if not known
		require
			an_operand_not_void: an_operand /= Void
		local
			i, j: INTEGER
		do
			if an_operand.is_current then
				Result := current_type
			else
				i := an_operand.index
				if current_dynamic_type_sets.valid_index (i) then
					Result := current_dynamic_type_sets.item (i)
				else
					j := i - current_dynamic_type_sets.count
					if extra_dynamic_type_sets.valid_index (j) then
						Result := extra_dynamic_type_sets.item (j)
					else
							-- Internal error: dynamic type set not known.
						set_fatal_error
						error_handler.report_giaaa_error
						Result := current_dynamic_system.unknown_type
					end
				end
			end
		ensure
			dynamic_type_set_not_void: Result /= Void
		end

	dynamic_type_set_in_feature (an_operand: ET_OPERAND; a_feature: ET_DYNAMIC_FEATURE): ET_DYNAMIC_TYPE_SET
			-- Dynamic type set associated with `an_operand' in `a_feature';
			-- Report a fatal error if not known
		require
			an_operand_not_void: an_operand /= Void
			a_feature_not_void: a_feature /= Void
		do
			if an_operand.is_current then
				Result := a_feature.target_type
			elseif attached a_feature.dynamic_type_set (an_operand) as l_dynamic_type_set then
				Result := l_dynamic_type_set
			else
					-- Internal error: dynamic type set not known.
				set_fatal_error
				error_handler.report_giaaa_error
				Result := current_dynamic_system.unknown_type
			end
		ensure
			dynamic_type_set_not_void: Result /= Void
		end

	argument_type_set (i: INTEGER): ET_DYNAMIC_TYPE_SET
			-- Dynamic type set of `i'-th argument of feature being printed;
			-- Report a fatal error if not known
		local
			j: INTEGER
		do
			if current_dynamic_type_sets.valid_index (i) then
				Result := current_dynamic_type_sets.item (i)
			else
				j := i - current_dynamic_type_sets.count
				if extra_dynamic_type_sets.valid_index (j) then
					Result := extra_dynamic_type_sets.item (j)
				else
						-- Internal error: dynamic type set not known.
					set_fatal_error
					error_handler.report_giaaa_error
					Result := current_dynamic_system.unknown_type
				end
			end
		ensure
			argument_type_set_not_void: Result /= Void
		end

	argument_type_set_in_feature (i: INTEGER; a_feature: ET_DYNAMIC_FEATURE): ET_DYNAMIC_TYPE_SET
			-- Dynamic type set of `i'-th argument of `a_feature';
			-- Report a fatal error if not known
		require
			a_feature_not_void: a_feature /= Void
		do
			if attached  a_feature.argument_type_set (i) as l_argument_type_set then
				Result := l_argument_type_set
			else
					-- Internal error: dynamic type set not known.
				set_fatal_error
				error_handler.report_giaaa_error
				Result := current_dynamic_system.unknown_type
			end
		ensure
			argument_type_set_not_void: Result /= Void
		end

	result_type_set_in_feature (a_feature: ET_DYNAMIC_FEATURE): ET_DYNAMIC_TYPE_SET
			-- Dynamic type set of result of `a_feature';
			-- Report a fatal error if not known
		require
			a_feature_not_void: a_feature /= Void
		do
			if attached a_feature.result_type_set as l_result_type_set then
				Result := l_result_type_set
			else
					-- Internal error: dynamic type set not known.
				set_fatal_error
				error_handler.report_giaaa_error
				Result := current_dynamic_system.unknown_type
			end
		ensure
			result_type_set_not_void: Result /= Void
		end

	current_dynamic_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			-- Dynamic type sets of expressions within feature being printed

	extra_dynamic_type_sets: ET_DYNAMIC_TYPE_SET_LIST
			-- Extra dynamic type sets used internally when needed

	conforming_type_set (a_source_type_set: ET_DYNAMIC_TYPE_SET; a_target_type: ET_DYNAMIC_TYPE): ET_DYNAMIC_STANDALONE_TYPE_SET
			-- New dynamic type set containing the dynamic types of
			-- `a_source_type_set' which conform to `a_target_type'.
			--
			-- When not needed anymore, the result object can be reused
			-- after calling `free_standalone_type_set'.
		require
			a_source_type_set_not_void: a_source_type_set /= Void
			a_target_type_not_void: a_target_type /= Void
		local
			l_source_type: ET_DYNAMIC_TYPE
			l_target_primary_type: ET_DYNAMIC_PRIMARY_TYPE
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
		do
			l_source_type := a_source_type_set.static_type
			if l_source_type.conforms_to_type (a_target_type) then
					-- All dynamic types conform to the expected one.
				Result := new_standalone_type_set (l_source_type)
				Result.put_types (a_source_type_set)
			else
				Result := new_standalone_type_set (a_target_type)
				l_target_primary_type := a_target_type.primary_type
				nb := a_source_type_set.count
				from i := 1 until i > nb loop
					l_dynamic_type := a_source_type_set.dynamic_type (i)
					if l_dynamic_type.conforms_to_primary_type (l_target_primary_type) then
						Result.put_type (l_dynamic_type)
					end
					i := i + 1
				end
			end
				-- Make sure that `is_never_void' is not set when the type set is empty.
			if a_source_type_set.is_never_void and not Result.is_empty then
				Result.set_never_void
			end
		ensure
			conforming_type_subset_not_void: Result /= Void
			count: Result.count <= a_source_type_set.count
			never_void: Result.is_never_void = (a_source_type_set.is_never_void and not Result.is_empty)
		end

	conforming_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			-- Types conforming to the target of the current assignment attempt or
			-- types to which the target of the current call to 'ANY.conforms_to' conforms.
			-- Also used for object-tests.

	non_conforming_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			-- Types non-conforming to the target of the current assignment attempt or
			-- types to which the target of the current call to 'ANY.conforms_to' do not conform.
			-- Also used for object-tests.

	attachment_dynamic_type_ids: DS_ARRAYED_LIST [INTEGER]
			-- List of dynamic type ids of the source of an attachment

	target_dynamic_type_ids: DS_ARRAYED_LIST [INTEGER]
			-- List of dynamic type ids of the target of a  call

	target_dynamic_types: DS_HASH_TABLE [ET_DYNAMIC_PRIMARY_TYPE, INTEGER]
			-- Dynamic types of the target of a call indexed by type ids

	equality_type_set: ET_DYNAMIC_STANDALONE_TYPE_SET
			-- Dynamic type set of argument of feature 'is_equal' when invoked
			-- as part of an object-equality ('~' or '/~') or of an equality
			-- ('=' or '/=') with expanded operands

	equality_common_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			-- List of types that are part of the dynamic type set of both
			-- operands in an equality expression ('=', '/=', '~' or '/~')

	equality_common_reference_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
			-- List of reference types that are part of the dynamic type set of both
			-- operands in an equality expression ('=', '/=', '~' or '/~')

	dynamic_type_id_sorter: DS_QUICK_SORTER [INTEGER]
			-- Dynamic type id sorter
		local
			l_comparator: KL_COMPARABLE_COMPARATOR [INTEGER]
		once
			create l_comparator.make
			create Result.make (l_comparator)
		ensure
			sorter_not_void: Result /= Void
		end

	standalone_type_sets: ET_DYNAMIC_STANDALONE_TYPE_SET_LIST
			-- Standalone type sets to be used as argument type sets when processing
			-- polymorphic calls, or as target type sets when attributes are deep twined

	new_standalone_type_set (a_static_type: ET_DYNAMIC_TYPE): ET_DYNAMIC_STANDALONE_TYPE_SET
			-- New standalone type set
		require
			a_static_type_not_void: a_static_type /= Void
		local
			l_types: ET_DYNAMIC_PRIMARY_TYPE_HASH_LIST
		do
			if unused_standalone_type_sets.is_empty then
				create Result.make (a_static_type)
				create l_types.make_with_capacity (100)
				Result.reset_with_types (a_static_type, l_types)
			else
				Result := unused_standalone_type_sets.last
				unused_standalone_type_sets.remove_last
				Result.reset (a_static_type)
			end
		ensure
			new_standalone_type_set_not_void: Result /= Void
			static_type_set: Result.static_type = a_static_type
			count_set: Result.count = 0
			not_never_void: not Result.is_never_void
		end

	free_standalone_type_set (a_type_set: ET_DYNAMIC_STANDALONE_TYPE_SET)
			-- Free `a_type_set' so that it can be reused.
		require
			a_type_set_not_void: a_type_set /= Void
		do
			unused_standalone_type_sets.force_last (a_type_set)
			a_type_set.reset (current_dynamic_system.unknown_type)
		end

feature {NONE} -- Dynamic type sets (Implementation)

	unused_standalone_type_sets: DS_ARRAYED_LIST [ET_DYNAMIC_STANDALONE_TYPE_SET]
			-- Standalone type sets that are not currently used

feature {NONE} -- Temporary variables

	new_temp_variable (a_type: ET_DYNAMIC_PRIMARY_TYPE): ET_IDENTIFIER
			-- New temporary variable of type `a_type'
		local
			i, nb: INTEGER
			l_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			l_temp: detachable ET_IDENTIFIER
		do
			nb := free_temp_variables.count
			from i := 1 until i > nb loop
				if frozen_temp_variables.item (i) = 0 then
					l_type := free_temp_variables.item (i)
					if l_type /= Void and then same_declaration_types (a_type, l_type) then
						used_temp_variables.replace (a_type, i)
						free_temp_variables.replace (Void, i)
						l_temp := temp_variables.item (i)
						i := nb + 1
					end
				end
				i := i + 1
			end
			if l_temp /= Void then
				Result := l_temp
			else
				nb := nb + 1
				if nb <= temp_variables.count then
					Result := temp_variables.item (nb)
				elseif short_names then
					create Result.make ("t" + nb.out)
					Result.set_temporary (True)
					Result.set_seed (nb)
					temp_variables.force_last (Result)
				else
-- TODO: long names
					create Result.make ("t" + nb.out)
					Result.set_temporary (True)
					Result.set_seed (nb)
					temp_variables.force_last (Result)
				end
				free_temp_variables.force_last (Void)
				used_temp_variables.force_last (a_type)
				frozen_temp_variables.force_last (0)
				current_function_header_buffer.put_character ('%T')
				print_type_declaration (a_type, current_function_header_buffer)
				current_function_header_buffer.put_character (' ')
				print_temp_name (Result, current_function_header_buffer)
				current_function_header_buffer.put_character (';')
				current_function_header_buffer.put_new_line
			end
		ensure
			variable_not_void: Result /= Void
			variable_is_temporary: Result.is_temporary
			known_variable: is_temp_variable_known (Result)
			used_variable: is_temp_variable_used (Result)
			not_free_variable: not is_temp_variable_free (Result)
			not_frozen_variable: not is_temp_variable_frozen (Result)
		end

	is_temp_variable_known (a_temp: ET_IDENTIFIER): BOOLEAN
			-- Is temporary variable `a_temp' under management?
		require
			a_temp_not_void: a_temp /= Void
			a_temp_is_temporary: a_temp.is_temporary
		local
			l_seed: INTEGER
		do
			l_seed := a_temp.seed
			Result := (l_seed >= 1 and l_seed <= used_temp_variables.count)
		end

	is_temp_variable_used (a_temp: ET_IDENTIFIER): BOOLEAN
			-- Is temporary variable `a_temp' currently used?
		require
			a_temp_not_void: a_temp /= Void
			a_temp_is_temporary: a_temp.is_temporary
			a_temp_known: is_temp_variable_known (a_temp)
		do
			Result := used_temp_variables.item (a_temp.seed) /= Void
		ensure
			definition: Result = (used_temp_variables.item (a_temp.seed) /= Void)
		end

	is_temp_variable_free (a_temp: ET_IDENTIFIER): BOOLEAN
			-- Is temporary variable `a_temp' currently free?
		require
			a_temp_not_void: a_temp /= Void
			a_temp_is_temporary: a_temp.is_temporary
			a_temp_known: is_temp_variable_known (a_temp)
		do
			Result := free_temp_variables.item (a_temp.seed) /= Void
		ensure
			definition: Result = (free_temp_variables.item (a_temp.seed) /= Void)
		end

	is_temp_variable_frozen (a_temp: ET_IDENTIFIER): BOOLEAN
			-- Is temporary variable `a_temp' currently frozen?
		require
			a_temp_not_void: a_temp /= Void
			a_temp_is_temporary: a_temp.is_temporary
			a_temp_known: is_temp_variable_known (a_temp)
		do
			Result := frozen_temp_variables.item (a_temp.seed) /= 0
		ensure
			definition: Result = (frozen_temp_variables.item (a_temp.seed) /= 0)
		end

	temp_variable_type (a_temp: ET_IDENTIFIER): detachable ET_DYNAMIC_PRIMARY_TYPE
			-- Type of `a_temp', if known
		require
			a_temp_not_void: a_temp /= Void
			a_temp_is_temporary: a_temp.is_temporary
			a_temp_known: is_temp_variable_known (a_temp)
		local
			l_seed: INTEGER
		do
			l_seed := a_temp.seed
			Result := used_temp_variables.item (l_seed)
			if Result = Void then
				Result := free_temp_variables.item (l_seed)
			end
		end

	mark_temp_variable_used (a_temp: ET_IDENTIFIER)
			-- Mark temporary variable `a_temp' as used.
		require
			a_temp_not_void: a_temp /= Void
			a_temp_is_temporary: a_temp.is_temporary
			a_temp_known: is_temp_variable_known (a_temp)
			a_temp_free: is_temp_variable_free (a_temp)
			a_temp_not_frozen: not is_temp_variable_frozen (a_temp)
		local
			l_seed: INTEGER
		do
			l_seed := a_temp.seed
			used_temp_variables.replace (free_temp_variables.item (l_seed), l_seed)
			free_temp_variables.replace (Void, l_seed)
		ensure
			a_temp_used: is_temp_variable_used (a_temp)
		end

	mark_temp_variable_free (a_temp: ET_IDENTIFIER)
			-- Mark temporary variable `a_temp' as free.
		require
			a_temp_not_void: a_temp /= Void
			a_temp_is_temporary: a_temp.is_temporary
			a_temp_known: is_temp_variable_known (a_temp)
			a_temp_used: is_temp_variable_used (a_temp)
			a_temp_not_frozen: not is_temp_variable_frozen (a_temp)
		local
			l_seed: INTEGER
		do
			l_seed := a_temp.seed
			free_temp_variables.replace (used_temp_variables.item (l_seed), l_seed)
			used_temp_variables.replace (Void, l_seed)
		ensure
			a_temp_free: is_temp_variable_free (a_temp)
		end

	mark_temp_variable_frozen (a_temp: ET_IDENTIFIER)
			-- Mark temporary variable `a_temp' as frozen.
		require
			a_temp_not_void: a_temp /= Void
			a_temp_is_temporary: a_temp.is_temporary
			a_temp_known: is_temp_variable_known (a_temp)
		local
			l_seed: INTEGER
		do
			l_seed := a_temp.seed
			frozen_temp_variables.replace (frozen_temp_variables.item (l_seed) + 1, l_seed)
		ensure
			a_temp_frozen: is_temp_variable_frozen (a_temp)
			a_temp_frozen_by_one_level: frozen_temp_variables.item (a_temp.seed) = old frozen_temp_variables.item (a_temp.seed) + 1
		end

	mark_temp_variable_unfrozen (a_temp: ET_IDENTIFIER)
			-- Mark temporary variable `a_temp' as not frozen.
		require
			a_temp_not_void: a_temp /= Void
			a_temp_is_temporary: a_temp.is_temporary
			a_temp_known: is_temp_variable_known (a_temp)
			a_temp_frozen: is_temp_variable_frozen (a_temp)
		local
			l_seed: INTEGER
		do
			l_seed := a_temp.seed
			frozen_temp_variables.replace (frozen_temp_variables.item (l_seed) - 1, l_seed)
		ensure
			a_temp_unfrozen_by_one_level: frozen_temp_variables.item (a_temp.seed) = old frozen_temp_variables.item (a_temp.seed) - 1
		end

	mark_call_operands_frozen
			-- Mark temporary variables in `call_operands' as frozen.
		local
			i, nb: INTEGER
		do
			nb := call_operands.count
			from i := 1 until i > nb loop
				if attached {ET_IDENTIFIER} call_operands.item (i) as l_temp and then l_temp.is_temporary then
					mark_temp_variable_frozen (l_temp)
				end
				i := i + 1
			end
		end

	mark_call_operands_unfrozen
			-- Mark temporary variables in `call_operands' as unfrozen.
		local
			i, nb: INTEGER
		do
			nb := call_operands.count
			from i := 1 until i > nb loop
				if attached {ET_IDENTIFIER} call_operands.item (i) as l_temp and then l_temp.is_temporary and then is_temp_variable_frozen (l_temp) then
					mark_temp_variable_unfrozen (l_temp)
				end
				i := i + 1
			end
		end

	mark_expressions_frozen (a_expressions: ET_EXPRESSIONS)
			-- Mark temporary variables in `a_expressions' as frozen.
		require
			a_expressions_not_void: a_expressions /= Void
		local
			i, nb: INTEGER
		do
			nb := a_expressions.count
			from i := 1 until i > nb loop
				if attached {ET_IDENTIFIER} a_expressions.expression (i) as l_temp and then l_temp.is_temporary then
					mark_temp_variable_frozen (l_temp)
				end
				i := i + 1
			end
		end

	mark_expressions_unfrozen (a_expressions: ET_EXPRESSIONS)
			-- Mark temporary variables in `a_expressions' as unfrozen.
		require
			a_expressions_not_void: a_expressions /= Void
		local
			i, nb: INTEGER
		do
			nb := a_expressions.count
			from i := 1 until i > nb loop
				if attached {ET_IDENTIFIER} a_expressions.expression (i) as l_temp and then l_temp.is_temporary and then is_temp_variable_frozen (l_temp) then
					mark_temp_variable_unfrozen (l_temp)
				end
				i := i + 1
			end
		end

	reset_temp_variables
			-- Reset temporary variables so that they can be used again in another function.
		do
			used_temp_variables.wipe_out
			free_temp_variables.wipe_out
			frozen_temp_variables.wipe_out
		end

feature {NONE} -- Temporary variables (Implementation)

	temp_variables: DS_ARRAYED_LIST [ET_IDENTIFIER]
			-- Names of temporary variables generated so far

	used_temp_variables: DS_ARRAYED_LIST [detachable ET_DYNAMIC_PRIMARY_TYPE]
			-- Temporary variables currently in used by the current feature

	free_temp_variables: DS_ARRAYED_LIST [detachable ET_DYNAMIC_PRIMARY_TYPE]
			-- Temporary variables declared for the current feature but
			-- not currently used

	frozen_temp_variables: DS_ARRAYED_LIST [INTEGER]
			-- Temporary variables currently marked as frozen (which means
			-- that we need to keep them in their current used/free state);
			-- 0 means not frozen, N means that it was been frozen N times.

	temp_variable: ET_IDENTIFIER
			-- Shared temporary variable, to be used in non-Eiffel features
			-- such as 'GE_new_str*', 'GE_ma*' or 'GE_main'.
		once
			if short_names then
				create Result.make ("t1")
			else
-- TODO: long names
				create Result.make ("t1")
			end
			Result.set_temporary (True)
			Result.set_seed (1)
		ensure
			temp_variable_not_void: Result /= Void
		end

feature {NONE} -- Equality expressions

	new_equality_expression (a_left: ET_EXPRESSION; a_operator: ET_SYMBOL; a_right: ET_EXPRESSION): ET_EQUALITY_EXPRESSION
			-- New equality expression
		require
			a_left_not_void: a_left /= Void
			a_operator_not_void: a_operator /= Void
			a_right_not_void: a_right /= Void
		do
			if unused_equality_expressions.is_empty then
				create Result.make (a_left, a_operator, a_right)
			else
				Result := unused_equality_expressions.last
				unused_equality_expressions.remove_last
				Result.set_left (a_left)
				Result.set_operator (a_operator)
				Result.set_right (a_right)
			end
		ensure
			new_equality_expression_not_void: Result /= Void
			left_set: Result.left = a_left
			operator_set: Result.operator = a_operator
			right_set: Result.right = a_right
		end

	free_equality_expression (a_expression: ET_EQUALITY_EXPRESSION)
			-- Free `a_expression' so that it can be reused.
		require
			a_expression_not_void: a_expression /= Void
		do
			unused_equality_expressions.force_last (a_expression)
			a_expression.set_left (tokens.current_keyword)
			a_expression.set_operator (tokens.equal_symbol)
			a_expression.set_right (tokens.current_keyword)
		end

feature {NONE} -- Equality expressions (Implementation)

	unused_equality_expressions: DS_ARRAYED_LIST [ET_EQUALITY_EXPRESSION]
			-- Equality expressions that are not currently used

feature {NONE} -- Equality types

	new_equality_types (a_left: ET_DYNAMIC_TYPE_SET; a_right: ET_DYNAMIC_TYPE_SET): ET_DYNAMIC_EQUALITY_TYPES
			-- New equality types
		require
			a_left_not_void: a_left /= Void
			a_right_not_void: a_right /= Void
		do
			if unused_equality_types.is_empty then
				create Result.make (a_left, a_right)
			else
				Result := unused_equality_types.last
				unused_equality_types.remove_last
				Result.set_left (a_left)
				Result.set_right (a_right)
			end
		ensure
			new_equality_types_not_void: Result /= Void
			left_set: Result.left = a_left
			right_set: Result.right = a_right
		end

	free_equality_types (a_types: ET_DYNAMIC_EQUALITY_TYPES)
			-- Free `a_types' so that it can be reused.
		require
			a_types_not_void: a_types /= Void
		do
			unused_equality_types.force_last (a_types)
			a_types.set_left (current_dynamic_system.unknown_type)
			a_types.set_right (current_dynamic_system.unknown_type)
		end

feature {NONE} -- Equality types (Implementation)

	unused_equality_types: DS_ARRAYED_LIST [ET_DYNAMIC_EQUALITY_TYPES]
			-- Dynamic type sets of equality operands that are not currently used

feature {NONE} -- Object equality types

	new_object_equality_types (a_left: ET_DYNAMIC_TYPE_SET; a_right: ET_DYNAMIC_TYPE_SET): ET_DYNAMIC_OBJECT_EQUALITY_TYPES
			-- New object equality types
		require
			a_left_not_void: a_left /= Void
			a_right_not_void: a_right /= Void
		do
			if unused_object_equality_types.is_empty then
				create Result.make (a_left, a_right)
			else
				Result := unused_object_equality_types.last
				unused_object_equality_types.remove_last
				Result.set_left (a_left)
				Result.set_right (a_right)
			end
		ensure
			new_object_equality_types_not_void: Result /= Void
			left_set: Result.left = a_left
			right_set: Result.right = a_right
		end

	free_object_equality_types (a_types: ET_DYNAMIC_OBJECT_EQUALITY_TYPES)
			-- Free `a_types' so that it can be reused.
		require
			a_types_not_void: a_types /= Void
		do
			unused_object_equality_types.force_last (a_types)
			a_types.set_left (current_dynamic_system.unknown_type)
			a_types.set_right (current_dynamic_system.unknown_type)
		end

feature {NONE} -- Object equality types (Implementation)

	unused_object_equality_types: DS_ARRAYED_LIST [ET_DYNAMIC_OBJECT_EQUALITY_TYPES]
			-- Dynamic type sets of object equality operands that are not currently used

feature {NONE} -- Unqualified call expressions

	new_unqualified_call_expression (a_name: ET_FEATURE_NAME; a_arguments: detachable ET_ACTUAL_ARGUMENT_LIST): ET_UNQUALIFIED_CALL_EXPRESSION
			-- New unqualified call expression
		require
			a_name_not_void: a_name /= Void
		do
			if unused_unqualified_call_expressions.is_empty then
				create Result.make (a_name, a_arguments)
			else
				Result := unused_unqualified_call_expressions.last
				unused_unqualified_call_expressions.remove_last
				Result.set_name (a_name)
				Result.set_arguments (a_arguments)
			end
		ensure
			new_unqualified_call_expression_not_void: Result /= Void
			name_set: Result.name = a_name
			arguments_set: Result.arguments = a_arguments
		end

	free_unqualified_call_expression (a_call: ET_UNQUALIFIED_CALL_EXPRESSION)
			-- Free `a_call' so that it can be reused.
		require
			a_call_not_void: a_call /= Void
		do
			unused_unqualified_call_expressions.force_last (a_call)
			a_call.set_arguments (Void)
		end

feature {NONE} -- Unqualified call expressions (Implementation)

	unused_unqualified_call_expressions: DS_ARRAYED_LIST [ET_UNQUALIFIED_CALL_EXPRESSION]
			-- Unqualified call expressions that are not currently used

feature {NONE} -- Qualified call expressions

	new_qualified_call_expression (a_target: ET_EXPRESSION; a_name: ET_FEATURE_NAME; a_arguments: detachable ET_ACTUAL_ARGUMENT_LIST): ET_QUALIFIED_CALL_EXPRESSION
			-- New qualified call expression
		require
			a_target_not_void: a_target /= Void
			a_name_not_void: a_name /= Void
		do
			if unused_qualified_call_expressions.is_empty then
				create Result.make (a_target, a_name, a_arguments)
			else
				Result := unused_qualified_call_expressions.last
				unused_qualified_call_expressions.remove_last
				Result.set_target (a_target)
				Result.set_name (a_name)
				Result.set_arguments (a_arguments)
			end
		ensure
			new_qualified_call_expression_not_void: Result /= Void
			target_set: Result.target = a_target
			name_set: Result.name = a_name
			arguments_set: Result.arguments = a_arguments
		end

	free_qualified_call_expression (a_call: ET_QUALIFIED_CALL_EXPRESSION)
			-- Free `a_call' so that it can be reused.
		require
			a_call_not_void: a_call /= Void
		do
			unused_qualified_call_expressions.force_last (a_call)
			a_call.set_target (tokens.current_keyword)
			a_call.set_arguments (Void)
		end

feature {NONE} -- Qualified call expressions (Implementation)

	unused_qualified_call_expressions: DS_ARRAYED_LIST [ET_QUALIFIED_CALL_EXPRESSION]
			-- Qualified call expressions that are not currently used

feature {NONE} -- Tuple labels

	new_tuple_label (i: INTEGER): ET_IDENTIFIER
			-- New tuple label to extract the `i'-th item of a tuple
		require
			i_large_enough: i >= 1
		do
			Result := new_identifier (tuple_label_name)
			Result.set_tuple_label (True)
			Result.set_seed (i)
		ensure
			new_tuple_label_not_void: Result /= Void
			is_tuple_label: Result.is_tuple_label
			ith_item: Result.seed = i
		end

	free_tuple_label (a_tuple_label: ET_IDENTIFIER)
			-- Free `a_tuple_label' so that it can be reused.
		require
			a_tuple_label_not_void: a_tuple_label /= Void
			is_tuple_label: a_tuple_label.is_tuple_label
		do
			free_identifier (a_tuple_label)
		end

feature {NONE} -- Tuple labels (Implementation)

	tuple_label_name: STRING = "l"
			-- Tuple label name

feature {NONE} -- Actual argument lists

	new_actual_argument_list: ET_ACTUAL_ARGUMENT_LIST
			-- New actual argument list
		do
			if unused_actual_argument_lists.is_empty then
				create Result.make_with_capacity (10)
			else
				Result := unused_actual_argument_lists.last
				unused_actual_argument_lists.remove_last
			end
		ensure
			new_actual_argument_list_not_void: Result /= Void
		end

	free_actual_argument_list (a_arguments: ET_ACTUAL_ARGUMENT_LIST)
			-- Free `a_arguments' so that it can be reused.
		require
			a_arguments_not_void: a_arguments /= Void
		do
			unused_actual_argument_lists.force_last (a_arguments)
			a_arguments.wipe_out
		end

feature {NONE} -- Actual argument lists (Implementation)

	unused_actual_argument_lists: DS_ARRAYED_LIST [ET_ACTUAL_ARGUMENT_LIST]
			-- Actual argument lists that are not currently used

feature {NONE} -- Result expressions

	new_result_expression: ET_RESULT
			-- New 'Result' expression
		do
			if unused_result_expressions.is_empty then
				create Result.make
			else
				Result := unused_result_expressions.last
				unused_result_expressions.remove_last
			end
		ensure
			new_result_expression_not_void: Result /= Void
		end

	free_result_expression (a_result: ET_RESULT)
			-- Free `a_result' so that it can be reused.
		require
			a_result_not_void: a_result /= Void
		do
			unused_result_expressions.force_last (a_result)
		end

feature {NONE} -- Result expressions (Implementation)

	unused_result_expressions: DS_ARRAYED_LIST [ET_RESULT]
			-- Result expressions that are not currently used

feature {NONE} -- Identifiers

	new_identifier (a_name: STRING): ET_IDENTIFIER
			-- New identifier
		require
			a_name_not_void: a_name /= Void
			a_name_not_empty: a_name.count > 0
			a_name_is_string_8: a_name.same_type ({STRING_8} "")
			valid_utf8_name: {UC_UTF8_ROUTINES}.valid_utf8 (a_name)
		do
			if unused_identifiers.is_empty then
				create Result.make (a_name)
			else
				Result := unused_identifiers.last
				unused_identifiers.remove_last
				Result.set_name (a_name)
			end
		ensure
			new_identifier_not_void: Result /= Void
			name_set: Result.name = a_name
		end

	free_identifier (a_identifier: ET_IDENTIFIER)
			-- Free `a_identifier' so that it can be reused.
		require
			a_identifier_not_void: a_identifier /= Void
		do
			unused_identifiers.force_last (a_identifier)
		end

feature {NONE} -- Identifiers (Implementation)

	unused_identifiers: DS_ARRAYED_LIST [ET_IDENTIFIER]
			-- Identifiers that are not currently used

feature {NONE} -- 'twin' feature name

	new_twin_feature_name: ET_IDENTIFIER
			-- New 'twin' feature name
		do
			Result := new_identifier (tokens.twin_name)
			Result.set_feature_name (True)
			Result.set_seed (current_system.twin_seed)
		ensure
			new_twin_feature_name_not_void: Result /= Void
			is_feature_name: Result.is_feature_name
			seed_set: Result.seed = current_system.twin_seed
		end

	free_twin_feature_name (a_twin_feature_name: ET_IDENTIFIER)
			-- Free `a_twin_feature_name' so that it can be reused.
		require
			a_twin_feature_name_not_void: a_twin_feature_name /= Void
		do
			free_identifier (a_twin_feature_name)
		end

feature {NONE} -- Formal arguments

	formal_argument (i: INTEGER): ET_IDENTIFIER
			-- Formal argument at index `i'
			-- Note: to be used only in functions that are not the direct
			-- translation of an Eiffel feature. Otherwise we should use
			-- the corresponding formal argument in that Eiffel feature
			-- (e.g. 'a_feature.arguments.formal_argument (i).name').
		require
			i_large_enough: i >= 1
		local
			nb: INTEGER
			l_arg: ET_IDENTIFIER
		do
			nb := formal_arguments.count
			if i > nb then
				if i > formal_arguments.capacity then
					formal_arguments.resize (i)
				end
				from until nb = i loop
					nb := nb + 1
					create l_arg.make ("a" + nb.out)
					l_arg.set_argument (True)
					l_arg.set_seed (nb)
					l_arg.set_index (nb)
					formal_arguments.put_last (l_arg)
				end
			end
			Result := formal_arguments.item (i)
		ensure
			formal_argument_not_void: Result /= Void
		end

feature {NONE} -- Formal arguments (Implementation)

	formal_arguments: DS_ARRAYED_LIST [ET_IDENTIFIER]
			-- Formal arguments indexed by argument position

feature {NONE} -- Called feature registration

	register_called_feature (a_feature: ET_DYNAMIC_FEATURE)
			-- Make sure that `a_feature' is added to `called_features' if not already done.
		require
			a_feature_not_void: a_feature /= Void
		do
			if not a_feature.is_generated then
				a_feature.set_generated (True)
				called_features.force_last (a_feature)
			end
		ensure
			a_feature_registered: a_feature.is_generated
			added_in_stack: called_features.has (a_feature)
			last_in_stack: not old a_feature.is_generated implies called_features.last = a_feature
		end

	register_polymorphic_called_features (a_call: ET_DYNAMIC_QUALIFIED_CALL)
			-- Make sure that the features potentially called through dynamic binding
			-- when calling `a_call' are added to `called_features' if not already done.
		require
			a_call_not_void: a_call /= Void
		local
			l_target_type_set: ET_DYNAMIC_TYPE_SET
			l_dynamic_type: ET_DYNAMIC_PRIMARY_TYPE
			i, nb: INTEGER
		do
			if not a_call.is_generated then
				a_call.set_generated (True)
				if not a_call.is_tuple_label then
					l_target_type_set := a_call.target_type_set
					nb := l_target_type_set.count
					from i := 1 until i > nb loop
						l_dynamic_type := l_target_type_set.dynamic_type (i)
						if attached a_call.dynamic_feature (l_dynamic_type, current_dynamic_system) as l_dynamic_feature then
							register_called_feature (l_dynamic_feature)
						end
						i := i + 1
					end
				end
			end
		end

	called_features: DS_ARRAYED_LIST [ET_DYNAMIC_FEATURE]
			-- Features being called

	register_called_static_feature (a_feature: ET_DYNAMIC_FEATURE)
			-- Make sure that `a_feature' is added to `called_static_features' if not already done.
		require
			a_feature_not_void: a_feature /= Void
		do
			if not a_feature.is_static_generated then
				a_feature.set_static_generated (True)
				called_static_features.force_last (a_feature)
			end
		ensure
			a_feature_registered: a_feature.is_static_generated
			added_in_stack: called_static_features.has (a_feature)
			last_in_stack: not old a_feature.is_generated implies called_static_features.last = a_feature
		end

	called_static_features: DS_ARRAYED_LIST [ET_DYNAMIC_FEATURE]
			-- Features being called statically (i.e. calls of the form {A}.f).

feature {NONE} -- Operand stack

	operand_stack: DS_ARRAYED_STACK [ET_EXPRESSION]
			-- Operand stack

	call_operands: DS_ARRAYED_LIST [ET_EXPRESSION]
			-- Operands of call being processed

	fill_call_operands (nb: INTEGER)
			-- Fill `call_operands' with nb operands found in `operand_stack'.
		require
			nb_not_negative: nb >= 0
		local
			i, j: INTEGER
			l_operand: ET_EXPRESSION
		do
			call_operands.wipe_out
			if call_operands.capacity < nb then
				call_operands.resize (nb)
			end
			if nb > operand_stack.count then
					-- Internal error.
				set_fatal_error
				error_handler.report_giaaa_error
					-- Put some dummy operands to preserve the postcondition.
				from i := 1 until i > nb loop
					call_operands.put_last (tokens.current_keyword)
					i := i + 1
				end
			else
				j := operand_stack.count - nb + 1
				from i := 1 until i > nb loop
					l_operand := operand_stack.i_th (j)
					if attached {ET_IDENTIFIER} l_operand as l_temp and then l_temp.is_temporary then
						if not is_temp_variable_frozen (l_temp) then
							mark_temp_variable_free (l_temp)
						end
					end
					call_operands.put_last (l_operand)
					j := j + 1
					i := i + 1
				end
				operand_stack.prune (nb)
			end
		ensure
			filled: call_operands.count = nb
		end

	fill_call_formal_arguments (a_feature: ET_FEATURE)
			-- Fill `call_operands' with 'Current' followed by the formal arguments of `a_feature'.
		require
			a_feature_not_void: a_feature /= Void
		local
			i, nb: INTEGER
			l_arguments: detachable ET_FORMAL_ARGUMENT_LIST
			l_name: ET_IDENTIFIER
		do
			l_arguments := a_feature.arguments
			if l_arguments /= Void then
				nb := l_arguments.count
			end
			call_operands.wipe_out
			if call_operands.capacity < nb + 1 then
				call_operands.resize (nb + 1)
			end
			call_operands.put_last (tokens.current_keyword)
			if l_arguments /= Void then
				from i := 1 until i > nb loop
					l_name := l_arguments.formal_argument (i).name
					l_name.set_index (i)
					call_operands.force_last (l_name)
					i := i + 1
				end
			end
		ensure
			filled: call_operands.count = a_feature.arguments_count + 1
		end

feature {NONE} -- Tuple arguments in agent routines

	agent_manifest_tuple: ET_MANIFEST_TUPLE
			-- Manifest tuple used when calling an agent

feature {NONE} -- Rescue clauses

	has_rescue: BOOLEAN
			-- Does current feature or inline agent have a rescue clause?

	has_retry: BOOLEAN
			-- Is 'retry' being called in the rescue clause of current feature or inline agent?

	locals_written: DS_HASH_SET [ET_IDENTIFIER]
			-- Local variables which are written in the code of current feature
			-- or inline agent being processed
			-- (Can be either in the body or the rescue clause of the
			-- current feature or inline agent.)

	locals_written_in_body: DS_HASH_SET [ET_IDENTIFIER]
			-- Local variables which are written in the body of
			-- current feature or inline agent

	locals_written_in_rescue: DS_HASH_SET [ET_IDENTIFIER]
			-- Local variables which are written in the rescue clause of
			-- current feature or inline agent

	locals_read: DS_HASH_SET [ET_IDENTIFIER]
			-- Local variables which are read in the code of current feature
			-- or inline agent being processed
			-- (Can be either in the body or the rescue clause of the
			-- current feature or inline agent.)

	locals_read_in_body: DS_HASH_SET [ET_IDENTIFIER]
			-- Local variables which are read in the body of current feature or inline agent

	locals_read_in_rescue: DS_HASH_SET [ET_IDENTIFIER]
			-- Local variables which are read in the rescue clause of
			-- current feature or inline agent

	result_written: BOOLEAN
			-- Is the Result entity written in the code of current feature
			-- or inline agent being processed?
			-- (Can be either in the body or the rescue clause of the
			-- current feature or inline agent.)

	result_read: BOOLEAN
			-- Is the Result entity read in the code of current feature
			-- or inline agent being processed?
			-- (Can be either in the body or the rescue clause of the
			-- current feature or inline agent.)

	make_rescue_data
			-- Create data to determine the 'volatile' status of local variables
			-- in features or inline agents with a rescue clause.
		do
			create locals_written_in_body.make (50)
			locals_written_in_body.set_equality_tester (identifier_tester)
			create locals_written_in_rescue.make (50)
			locals_written_in_rescue.set_equality_tester (identifier_tester)
			locals_written := locals_written_in_body
			create locals_read_in_body.make (50)
			locals_read_in_body.set_equality_tester (identifier_tester)
			create locals_read_in_rescue.make (50)
			locals_read_in_rescue.set_equality_tester (identifier_tester)
			locals_read := locals_read_in_body
		end

	reset_rescue_data
			-- Reset data to determine the 'volatile' status of local variables
			-- in features or inline agents with a rescue clause.
		do
			locals_written_in_body.wipe_out
			locals_read_in_body.wipe_out
			locals_written_in_rescue.wipe_out
			locals_read_in_rescue.wipe_out
			result_written := False
			result_read := False
			has_retry := False
			has_rescue := False
		end

feature {NONE} -- Implementation

	in_static_feature: BOOLEAN
			-- Is a feature to be used in a static call being printed?

	in_operand: BOOLEAN
			-- Is an operand being processed?

	in_call_target: BOOLEAN
			-- Is the target of a call being processed?
		do
			Result := (call_target_type /= Void)
		ensure
			definition: Result = (call_target_type /= Void)
		end

	in_procedure_call_target: BOOLEAN
			-- Is the target of a procedure call being processed?

	call_target_type: detachable ET_DYNAMIC_PRIMARY_TYPE
			-- Dynamic type of the target of the call being processed, if any
			-- (The dynamic type is the type of the object, not its declared
			-- type, also called static type.)

	call_target_check_void: BOOLEAN
			-- Do we need to check whether the target is Void or not
			-- when both `in_call_target' and not `in_operand' mode?

	is_twin_needed_in_attachment (a_source_type_set: ET_DYNAMIC_TYPE_SET): BOOLEAN
			-- Is a call to 'twin' needed when the dynamic type set of
			-- the source of an attachment is `a_source_type_set'?
		require
			a_source_type_set_not_void: a_source_type_set /= Void
		local
			l_source_type: ET_DYNAMIC_PRIMARY_TYPE
		do
			l_source_type := a_source_type_set.static_type.primary_type
			if l_source_type.is_expanded and then l_source_type.is_basic then
					-- Optimization: do not call 'twin' for basic types
					-- when 'copy' has not been redefined.
				Result := l_source_type.has_redefined_copy_routine
			else
				Result := a_source_type_set.has_expanded
			end
		end

	assignment_target: detachable ET_WRITABLE
			-- Target of assignment currently being processed, if any

	same_declaration_types (a_type1, a_type2: ET_DYNAMIC_PRIMARY_TYPE): BOOLEAN
			-- Do `a_type1' and `a_type2' have the same declaration type?
		require
			a_type1_not_void: a_type1 /= Void
			a_type2_not_void: a_type2 /= Void
		do
			if a_type1 = a_type2 then
				Result := True
			else
				Result := (not a_type1.is_expanded and not a_type2.is_expanded)
			end
		end

	type_info_ancestors_used: BOOLEAN
			-- Is the list of ancestors of types used in the system?

	type_info_attributes_used: BOOLEAN
			-- Is the list of attributes in each type used in the system?

	type_info_attribute_name_used: BOOLEAN
			-- Is the name of attributes used in the system?

	type_info_attribute_type_id_used: BOOLEAN
			-- Is the type id of attributes used in the system?

	type_info_attribute_offset_used: BOOLEAN
			-- Is the offset of attributes used in the system?

	type_info_generator_used: BOOLEAN
			-- Is the generator of types used in the system?

	type_info_generic_parameters_used: BOOLEAN
			-- Are the generic parameters of types used in the system?

	type_info_name_used: BOOLEAN
			-- Is the name of types used in the system?

	type_info_object_size_used: BOOLEAN
			-- Is the object size of types used in the system?

	dummy_feature: ET_DYNAMIC_FEATURE
			-- Dummy feature
		local
			l_name: ET_FEATURE_NAME
			l_feature: ET_FEATURE
		once
			create {ET_IDENTIFIER} l_name.make ("**dummy**")
			create {ET_DO_PROCEDURE} l_feature.make (l_name, Void, current_type.base_class)
			create Result.make (l_feature, current_type, current_dynamic_system)
		ensure
			dummy_feature_not_void: Result /= Void
		end

feature {NONE} -- External regexp

	external_c_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: C [blocking] [signature ["(" {<type> "," ...}* ")"] [":" <type>]] [use {<include> "," ...}+]
			-- \5: has signature arguments
			-- \6: signature arguments
			-- \11: signature result
			-- \18: include files

	external_c_macro_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: C [blocking] macro [signature ["(" {<type> "," ...}* ")"] [":" <type>]] use {<include> "," ...}+
			-- \4: has signature arguments
			-- \5: signature arguments
			-- \10: signature result
			-- \17: include files

	external_c_struct_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: C struct <struct-type> (access|get) <field-name> [type <field-type>] use {<include> "," ...}+
			-- \1: struct type
			-- \6: field name
			-- \9: field type
			-- \16: include files

	external_c_inline_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: C [blocking] inline [use {<include> "," ...}+]
			-- \3: include files

	old_external_c_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: C [["(" {<type> "," ...}* ")"] [":" <type>]] ["|" {<include> "," ...}+]
			-- \1: has signature
			-- \2: has signature arguments
			-- \3: signature arguments
			-- \5: signature result
			-- \7: include files

	old_external_c_macro_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: C "[" macro <include> "]" ["(" {<type> "," ...}* ")"] [":" <type>]
			-- \1: include file
			-- \2: has signature arguments
			-- \3: signature arguments
			-- \5: signature result

	old_external_c_struct_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: C "[" struct <include> "]" "(" {<type> "," ...}+ ")" [":" <type>]
			-- \1: include file
			-- \2: signature arguments
			-- \4: signature result

	external_cpp_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: C++ [blocking] <class_type> [signature ["(" {<type> "," ...}* ")"] [":" <type>]] [use {<include> "," ...}+]
			-- \2: class type
			-- \5: has signature arguments
			-- \6: signature arguments
			-- \11: signature result
			-- \18: include files

	external_cpp_macro_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: C++ [blocking] macro [signature ["(" {<type> "," ...}* ")"] [":" <type>]] use {<include> "," ...}+
			-- \4: has signature arguments
			-- \5: signature arguments
			-- \10: signature result
			-- \17: include files

	external_cpp_struct_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: C++ struct <struct-type> (access|get) <field-name> [type <field-type>] use {<include> "," ...}+
			-- \1: struct type
			-- \6: field name
			-- \9: field type
			-- \16: include files

	external_cpp_inline_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: C++ [blocking] inline [use {<include> "," ...}+]
			-- \3: include files

	external_dllwin_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: [blocking] dllwin <dll_file> [signature ["(" {<type> "," ...}* ")"] [":" <type>]] [use {<include> "," ...}+]
			-- \2: dll file
			-- \5: has signature arguments
			-- \6: signature arguments
			-- \11: signature result
			-- \18: include files

	old_external_dllwin32_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regexp: C "[" dllwin32 <dll_file> "]" ["(" {<type> "," ...}* ")"] [":" <type>]
			-- \1: dll file
			-- \2: has signature arguments
			-- \3: signature arguments
			-- \5: signature result

	make_external_regexps
			-- Create external regular expressions.
		do
				-- Regexp: C [blocking] [signature ["(" {<type> "," ...}* ")"] [":" <type>]] [use {<include> "," ...}+]
			create external_c_regexp.make
			external_c_regexp.compile ("[ \t\r\n]*[Cc]([ \t\r\n]+|$)(blocking([ \t\r\n]+|$))?(signature[ \t\r\n]*(\((([ \t\r\n]*[^ \t\r\n,)])+([ \t\r\n]*,([ \t\r\n]*[^ \t\r\n,)])+)*)?[ \t\r\n]*\))?[ \t\r\n]*(:[ \t\r\n]*((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$)((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$))*))?)?(use[ \t\r\n]*((.|\n)+))?")
				-- Regexp: C [blocking] macro [signature ["(" {<type> "," ...}* ")"] [":" <type>]] use {<include> "," ...}+
			create external_c_macro_regexp.make
			external_c_macro_regexp.compile ("[ \t\r\n]*[Cc][ \t\r\n]+(blocking[ \t\r\n]+)?macro([ \t\r\n]+|$)(signature[ \t\r\n]*(\((([ \t\r\n]*[^ \t\r\n,)])+([ \t\r\n]*,([ \t\r\n]*[^ \t\r\n,)])+)*)?[ \t\r\n]*\))?[ \t\r\n]*(:[ \t\r\n]*((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$)((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$))*))?)?(use[ \t\r\n]*((.|\n)+))")
				-- Regexp: C struct <struct-type> (access|get) <field-name> [type <field-type>] use {<include> "," ...}+
			create external_c_struct_regexp.make
			external_c_struct_regexp.compile ("[ \t\r\n]*[Cc][ \t\r\n]+struct[ \t\r\n]+((a|ac|acc|acce|acces|g|ge|[^ag \t\r\n][^ \t\r\n]*|g[^e \t\r\n][^ \t\r\n]*|ge[^t \t\r\n][^ \t\r\n]*|get[^ \t\r\n]+|a[^c \t\r\n][^ \t\r\n]*|ac[^c \t\r\n][^ \t\r\n]*|acc[^e \t\r\n][^ \t\r\n]*|acce[^s \t\r\n][^ \t\r\n]*|acces[^s \t\r\n][^ \t\r\n]*|access[^ \t\r\n]+)[ \t\r\n]+((a|ac|acc|acce|acces|g|ge|[^ag \t\r\n][^ \t\r\n]*|g[^e \t\r\n][^ \t\r\n]*|ge[^t \t\r\n][^ \t\r\n]*|get[^ \t\r\n]+|a[^c \t\r\n][^ \t\r\n]*|ac[^c \t\r\n][^ \t\r\n]*|acc[^e \t\r\n][^ \t\r\n]*|acce[^s \t\r\n][^ \t\r\n]*|acces[^s \t\r\n][^ \t\r\n]*|access[^ \t\r\n]+)[ \t\r\n]+)*)(access|get)[ \t\r\n]+([^ \t\r\n]+)([ \t\r\n]+|$)(type[ \t\r\n]+((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$)((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$))*))?(use[ \t\r\n]*((.|\n)+))")
				-- Regexp: C [blocking] inline [use {<include> "," ...}+]
			create external_c_inline_regexp.make
			external_c_inline_regexp.compile ("[ \t\r\n]*[Cc][ \t\r\n]+(blocking[ \t\r\n]+)?inline([ \t\r\n]+use[ \t\r\n]*((.|\n)+))?")
				-- Regexp: C [["(" {<type> "," ...}* ")"] [":" <type>]] ["|" {<include> "," ...}+]
			create old_external_c_regexp.make
			old_external_c_regexp.compile ("[ \t\r\n]*[Cc][ \t\r\n]*((\(([^)]*)\))?[ \t\r\n]*(:[ \t\r\n]*([^|]+))?)?[ \t\r\n]*(\|[ \t\r\n]*((.|\n)+))?")
				-- Regexp: C "[" macro <include> "]" ["(" {<type> "," ...}* ")"] [":" <type>]
			create old_external_c_macro_regexp.make
			old_external_c_macro_regexp.compile ("[ \t\r\n]*[Cc][ \t\r\n]*\[[ \t\r\n]*macro[ \t\r\n]*([^]]+)\][ \t\r\n]*(\(([^)]*)\))?[ \t\r\n]*(:[ \t\r\n]*((.|\n)+))?")
				-- Regexp: C "[" struct <include> "]" "(" {<type> "," ...}+ ")" [":" <type>]
			create old_external_c_struct_regexp.make
			old_external_c_struct_regexp.compile ("[ \t\r\n]*[Cc][ \t\r\n]*\[[ \t\r\n]*struct[ \t\r\n]*([^]]+)\][ \t\r\n]*\(([^)]+)\)[ \t\r\n]*(:[ \t\r\n]*((.|\n)+))?")
				-- Regexp: C++ [blocking] <class_type> [signature ["(" {<type> "," ...}* ")"] [":" <type>]] [use {<include> "," ...}+]
			create external_cpp_regexp.make
			external_cpp_regexp.compile ("[ \t\r\n]*[Cc]\+\+[ \t\r\n]+(blocking[ \t\r\n]+)?([^ \t\r\n]+)([ \t\r\n]+|$)(signature[ \t\r\n]*(\((([ \t\r\n]*[^ \t\r\n,)])+([ \t\r\n]*,([ \t\r\n]*[^ \t\r\n,)])+)*)?[ \t\r\n]*\))?[ \t\r\n]*(:[ \t\r\n]*((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$)((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$))*))?)?(use[ \t\r\n]*((.|\n)+))?")
				-- Regexp: C++ [blocking] macro [signature ["(" {<type> "," ...}* ")"] [":" <type>]] use {<include> "," ...}+
			create external_cpp_macro_regexp.make
			external_cpp_macro_regexp.compile ("[ \t\r\n]*[Cc]\+\+[ \t\r\n]+(blocking[ \t\r\n]+)?macro([ \t\r\n]+|$)(signature[ \t\r\n]*(\((([ \t\r\n]*[^ \t\r\n,)])+([ \t\r\n]*,([ \t\r\n]*[^ \t\r\n,)])+)*)?[ \t\r\n]*\))?[ \t\r\n]*(:[ \t\r\n]*((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$)((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$))*))?)?(use[ \t\r\n]*((.|\n)+))")
				-- Regexp: C++ struct <struct-type> (access|get) <field-name> [type <field-type>] use {<include> "," ...}+
			create external_cpp_struct_regexp.make
			external_cpp_struct_regexp.compile ("[ \t\r\n]*[Cc]\+\+[ \t\r\n]+struct[ \t\r\n]+((a|ac|acc|acce|acces|g|ge|[^ag \t\r\n][^ \t\r\n]*|g[^e \t\r\n][^ \t\r\n]*|ge[^t \t\r\n][^ \t\r\n]*|get[^ \t\r\n]+|a[^c \t\r\n][^ \t\r\n]*|ac[^c \t\r\n][^ \t\r\n]*|acc[^e \t\r\n][^ \t\r\n]*|acce[^s \t\r\n][^ \t\r\n]*|acces[^s \t\r\n][^ \t\r\n]*|access[^ \t\r\n]+)[ \t\r\n]+((a|ac|acc|acce|acces|g|ge|[^ag \t\r\n][^ \t\r\n]*|g[^e \t\r\n][^ \t\r\n]*|ge[^t \t\r\n][^ \t\r\n]*|get[^ \t\r\n]+|a[^c \t\r\n][^ \t\r\n]*|ac[^c \t\r\n][^ \t\r\n]*|acc[^e \t\r\n][^ \t\r\n]*|acce[^s \t\r\n][^ \t\r\n]*|acces[^s \t\r\n][^ \t\r\n]*|access[^ \t\r\n]+)[ \t\r\n]+)*)(access|get)[ \t\r\n]+([^ \t\r\n]+)([ \t\r\n]+|$)(type[ \t\r\n]+((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$)((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$))*))?(use[ \t\r\n]*((.|\n)+))")
				-- Regexp: C++ [blocking] inline [use {<include> "," ...}+]
			create external_cpp_inline_regexp.make
			external_cpp_inline_regexp.compile ("[ \t\r\n]*[Cc]\+\+[ \t\r\n]+(blocking[ \t\r\n]+)?inline([ \t\r\n]+use[ \t\r\n]*((.|\n)+))?")
				-- Regexp: [blocking] dllwin <dll_file> [signature ["(" {<type> "," ...}* ")"] [":" <type>]] [use {<include> "," ...}+]
			create external_dllwin_regexp.make
			external_dllwin_regexp.compile ("[ \t\r\n]*(blocking[ \t\r\n]+)?[Dd][Ll][Ll][Ww][Ii][Nn][ \t\r\n]+([^ \t\r\n]+)([ \t\r\n]+|$)(signature[ \t\r\n]*(\((([ \t\r\n]*[^ \t\r\n,)])+([ \t\r\n]*,([ \t\r\n]*[^ \t\r\n,)])+)*)?[ \t\r\n]*\))?[ \t\r\n]*(:[ \t\r\n]*((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$)((u|us|use[^ \t\r\n<%"]+|[^u \t\r\n][^ \t\r\n]*|u[^s \t\r\n][^ \t\r\n]*|us[^e \t\r\n][^ \t\r\n]*)([ \t\r\n]+|$))*))?)?(use[ \t\r\n]*((.|\n)+))?")
				-- Regexp: C "[" dllwin32 <dll_file> "]" ["(" {<type> "," ...}* ")"] [":" <type>]
			create old_external_dllwin32_regexp.make
			old_external_dllwin32_regexp.compile ("[ \t\r\n]*[Cc][ \t\r\n]*\[[ \t\r\n]*dllwin32[ \t\r\n]*([^]]+)\][ \t\r\n]*(\(([^)]*)\))?[ \t\r\n]*(:[ \t\r\n]*((.|\n)+))?")
		ensure
			external_c_regexp_not_void: external_c_regexp /= Void
			external_c_regexp_compiled: external_c_regexp.is_compiled
			external_c_macro_regexp_not_void: external_c_macro_regexp /= Void
			external_c_macro_regexp_compiled: external_c_macro_regexp.is_compiled
			external_c_struct_regexp_not_void: external_c_struct_regexp /= Void
			external_c_struct_regexp_compiled: external_c_struct_regexp.is_compiled
			external_c_inline_regexp_not_void: external_c_inline_regexp /= Void
			external_c_inline_regexp_compiled: external_c_inline_regexp.is_compiled
			old_external_c_regexp_not_void: old_external_c_regexp /= Void
			old_external_c_regexp_compiled: old_external_c_regexp.is_compiled
			old_external_c_macro_regexp_not_void: old_external_c_macro_regexp /= Void
			old_external_c_macro_regexp_compiled: old_external_c_macro_regexp.is_compiled
			old_external_c_struct_regexp_not_void: old_external_c_struct_regexp /= Void
			old_external_c_struct_regexp_compiled: old_external_c_struct_regexp.is_compiled
			external_cpp_regexp_not_void: external_cpp_regexp /= Void
			external_cpp_regexp_compiled: external_cpp_regexp.is_compiled
			external_cpp_inline_regexp_not_void: external_cpp_inline_regexp /= Void
			external_cpp_inline_regexp_compiled: external_cpp_inline_regexp.is_compiled
			external_dllwin_regexp_not_void: external_dllwin_regexp /= Void
			external_dllwin_regexp_compiled: external_dllwin_regexp.is_compiled
			old_external_dllwin32_regexp_not_void: old_external_dllwin32_regexp /= Void
			old_external_dllwin32_regexp_compiled: old_external_dllwin32_regexp.is_compiled
		end

feature {NONE} -- Constants

	c_ac: STRING = "ac"
	c_and_then: STRING = "&&"
	c_arrow: STRING = "->"
	c_attributes: STRING = "attributes"
	c_break: STRING = "break"
	c_call: STRING = "call"
	c_caller: STRING = "caller"
	c_case: STRING = "case"
	c_char: STRING = "char"
	c_const: STRING = "const"
	c_cplusplus: STRING = "__cplusplus"
	c_default: STRING = "default"
	c_define: STRING = "#define"
	c_double: STRING = "double"
	c_eif_any: STRING = "EIF_ANY"
	c_eif_boolean: STRING = "EIF_BOOLEAN"
	c_eif_character: STRING = "EIF_CHARACTER"
	c_eif_character_8: STRING = "EIF_CHARACTER_8"
	c_eif_character_32: STRING = "EIF_CHARACTER_32"
	c_eif_double: STRING = "EIF_DOUBLE"
	c_eif_exception_trace: STRING = "EIF_EXCEPTION_TRACE"
	c_eif_false: STRING = "EIF_FALSE"
	c_eif_integer: STRING = "EIF_INTEGER"
	c_eif_integer_8: STRING = "EIF_INTEGER_8"
	c_eif_integer_16: STRING = "EIF_INTEGER_16"
	c_eif_integer_32: STRING = "EIF_INTEGER_32"
	c_eif_integer_64: STRING = "EIF_INTEGER_64"
	c_eif_is_mac: STRING = "EIF_IS_MAC"
	c_eif_is_unix: STRING = "EIF_IS_UNIX"
	c_eif_is_vms: STRING = "EIF_IS_VMS"
	c_eif_is_vxworks: STRING = "EIF_IS_VXWORKS"
	c_eif_is_windows: STRING = "EIF_IS_WINDOWS"
	c_eif_mem_free: STRING = "eif_mem_free"
	c_eif_native_char: STRING = "EIF_NATIVE_CHAR"
	c_eif_natural: STRING = "EIF_NATURAL"
	c_eif_natural_8: STRING = "EIF_NATURAL_8"
	c_eif_natural_16: STRING = "EIF_NATURAL_16"
	c_eif_natural_32: STRING = "EIF_NATURAL_32"
	c_eif_natural_64: STRING = "EIF_NATURAL_64"
	c_eif_object: STRING = "EIF_OBJECT"
	c_eif_pointer: STRING = "EIF_POINTER"
	c_eif_real: STRING = "EIF_REAL"
	c_eif_real_32: STRING = "EIF_REAL_32"
	c_eif_real_64: STRING = "EIF_REAL_64"
	c_eif_reference: STRING = "EIF_REFERENCE"
	c_eif_test: STRING = "EIF_TEST"
	c_eif_trace: STRING = "EIF_TRACE"
	c_eif_true: STRING = "EIF_TRUE"
	c_eif_type: STRING = "EIF_TYPE_OBJ"
	c_eif_type_index: STRING = "EIF_TYPE_INDEX"
	c_eif_void: STRING = "EIF_VOID"
	c_eif_wide_char: STRING = "EIF_WIDE_CHAR"
	c_eif_windows: STRING = "EIF_WINDOWS"
	c_else: STRING = "else"
	c_endif: STRING = "#endif"
	c_equal: STRING = "=="
	c_exception_code: STRING = "exception_code"
	c_exception_manager: STRING = "exception_manager"
	c_exception_suffix: STRING = "_exception"
	c_exception_tag: STRING = "exception_tag"
	c_extern: STRING = "extern"
	c_find_referers: STRING = "find_referers"
	c_flags: STRING = "flags"
	c_float: STRING = "float"
	c_for: STRING = "for"
	c_fprintf: STRING = "fprintf"
	c_ge_argc: STRING = "GE_argc"
	c_ge_argv: STRING = "GE_argv"
	c_ge_attached_encoded_type: STRING = "GE_attached_encoded_type"
	c_ge_bma: STRING = "GE_bma"
	c_ge_boolean_field: STRING = "GE_boolean_field"
	c_ge_boolean_field_at: STRING = "GE_boolean_field_at"
	c_ge_boxed: STRING = "GE_boxed"
	c_ge_boxed_pointer: STRING = "GE_boxed_pointer"
	c_ge_call: STRING = "GE_call"
	c_ge_case_int64: STRING = "GE_case_int64"
	c_ge_case_nat64: STRING = "GE_case_nat64"
	c_ge_catcall: STRING = "GE_catcall"
	c_ge_ccom_character_size: STRING = "GE_ccom_character_size"
	c_ge_ccom_error_text: STRING = "GE_ccom_error_text"
	c_ge_ccom_hresult: STRING = "GE_ccom_hresult"
	c_ge_ccom_hresult_code: STRING = "GE_ccom_hresult_code"
	c_ge_ccom_hresult_facility: STRING = "GE_ccom_hresult_facility"
	c_ge_ccom_local_free: STRING = "GE_ccom_local_free"
	c_ge_ccom_strlen: STRING = "GE_ccom_strlen"
	c_ge_ceiling: STRING = "GE_ceiling"
	c_ge_character_8_field: STRING = "GE_character_8_field"
	c_ge_character_8_field_at: STRING = "GE_character_8_field_at"
	c_ge_character_32_field: STRING = "GE_character_32_field"
	c_ge_character_32_field_at: STRING = "GE_character_32_field_at"
	c_ge_check_assert: STRING = "GE_check_assert"
	c_ge_compiler_version: STRING = "GE_compiler_version"
	c_ge_context: STRING = "GE_context"
	c_ge_current_context: STRING = "GE_current_context"
	c_ge_decoded_type: STRING = "GE_decoded_type"
	c_ge_deep: STRING = "GE_deep"
	c_ge_deep_twin: STRING = "GE_deep_twin"
	c_ge_default: STRING = "GE_default"
	c_ge_default_context: STRING = "GE_default_context"
	c_ge_developer_raise: STRING = "GE_developer_raise"
	c_ge_dispose_once_per_object_data: STRING = "GE_dispose_once_per_object_data"
	c_ge_dts: STRING = "GE_dts"
	c_ge_encoded_type_conforms_to: STRING = "GE_encoded_type_conforms_to"
	c_ge_encoded_type_from_name: STRING = "GE_encoded_type_from_name"
	c_ge_ex_cdef: STRING = "GE_EX_CDEF"
	c_ge_ex_check: STRING = "GE_EX_CHECK"
	c_ge_ex_fail: STRING = "GE_EX_FAIL"
	c_ge_ex_fatal: STRING = "GE_EX_FATAL"
	c_ge_ex_prog: STRING = "GE_EX_PROG"
	c_ge_ex_when: STRING = "GE_EX_WHEN"
	c_ge_field_count_of_encoded_type: STRING = "GE_field_count_of_encoded_type"
	c_ge_field_name_of_encoded_type: STRING = "GE_field_name_of_encoded_type"
	c_ge_field_offset_of_encoded_type: STRING = "GE_field_offset_of_encoded_type"
	c_ge_field_static_type_of_encoded_type: STRING = "GE_field_static_type_of_encoded_type"
	c_ge_field_type_kind_of_encoded_type: STRING = "GE_field_type_kind_of_encoded_type"
	c_ge_floor: STRING = "GE_floor"
	c_ge_generating_type_of_encoded_type: STRING = "GE_generating_type_of_encoded_type"
	c_ge_generating_type_8_of_encoded_type: STRING = "GE_generating_type_8_of_encoded_type"
	c_ge_generator_of_encoded_type: STRING = "GE_generator_of_encoded_type"
	c_ge_generator_8_of_encoded_type: STRING = "GE_generator_8_of_encoded_type"
	c_ge_generic_parameter_of_encoded_type: STRING = "GE_generic_parameter_of_encoded_type"
	c_ge_generic_parameter_count_of_encoded_type: STRING = "GE_generic_parameter_count_of_encoded_type"
	c_ge_id_object: STRING = "GE_id_object"
	c_ge_ims8: STRING = "GE_ims8"
	c_ge_ims32_from_utf32le: STRING = "GE_ims32_from_utf32le"
	c_ge_init_const: STRING = "GE_init_const"
	c_ge_init_exception_manager: STRING = "GE_init_exception_manager"
	c_ge_int8: STRING = "GE_int8"
	c_ge_int16: STRING = "GE_int16"
	c_ge_int32: STRING = "GE_int32"
	c_ge_int64: STRING = "GE_int64"
	c_ge_integer_8_field: STRING = "GE_integer_8_field"
	c_ge_integer_8_field_at: STRING = "GE_integer_8_field_at"
	c_ge_integer_16_field: STRING = "GE_integer_16_field"
	c_ge_integer_16_field_at: STRING = "GE_integer_16_field_at"
	c_ge_integer_32_field: STRING = "GE_integer_32_field"
	c_ge_integer_32_field_at: STRING = "GE_integer_32_field_at"
	c_ge_integer_64_field: STRING = "GE_integer_8_field"
	c_ge_integer_64_field_at: STRING = "GE_integer_64_field_at"
	c_ge_is_64_bits: STRING = "GE_IS_64_BITS"
	c_ge_is_attached_encoded_type: STRING = "GE_is_attached_encoded_type"
	c_ge_is_copy_semantics_field: STRING = "GE_is_copy_semantics_field"
	c_ge_is_expanded_object: STRING = "GE_is_expanded_object"
	c_ge_is_deferred_encoded_type: STRING = "GE_is_deferred_encoded_type"
	c_ge_is_field_expanded_of_encoded_type: STRING = "GE_is_field_expanded_of_encoded_type"
	c_ge_is_field_transient_of_encoded_type: STRING = "GE_is_field_transient_of_encoded_type"
	c_ge_is_object_marked: STRING = "GE_is_object_marked"
	c_ge_is_special_copy_semantics_item: STRING = "GE_is_special_copy_semantics_item"
	c_ge_is_special_object: STRING = "GE_is_special_object"
	c_ge_is_special_of_expanded_object: STRING = "GE_is_special_of_expanded_object"
	c_ge_is_special_of_reference_encoded_type: STRING = "GE_is_special_of_reference_encoded_type"
	c_ge_is_special_of_reference_object: STRING = "GE_is_special_of_reference_object"
	c_ge_is_special_of_reference_or_basic_expanded_encoded_type: STRING = "GE_is_special_of_reference_or_basic_expanded_encoded_type"
	c_ge_is_tuple_encoded_type: STRING = "GE_is_tuple_encoded_type"
	c_ge_is_tuple_object: STRING = "GE_is_tuple_object"
	c_ge_istr32_from_nstr: STRING = "GE_istr32_from_nstr"
	c_ge_jump_to_last_rescue: STRING = "GE_jump_to_last_rescue"
	c_ge_last_exception: STRING = "GE_last_exception"
	c_ge_last_exception_raised: STRING = "GE_last_exception_raised"
	c_ge_lock_marking: STRING = "GE_lock_marking"
	c_ge_ma: STRING = "GE_ma"
	c_ge_malloc: STRING = "GE_malloc"
	c_ge_malloc_atomic: STRING = "GE_malloc_atomic"
	c_ge_malloc_cleared: STRING = "GE_malloc_cleared"
	c_ge_malloc_atomic_cleared: STRING = "GE_malloc_atomic_cleared"
	c_ge_mark_object: STRING = "GE_mark_object"
	c_ge_min_int32: STRING = "GE_min_int32"
	c_ge_min_int64: STRING = "GE_min_int64"
	c_ge_ms8: STRING = "GE_ms8"
	c_ge_ms32: STRING = "GE_ms32"
	c_ge_ms32_from_utf32le: STRING = "GE_ms32_from_utf32le"
	c_ge_mt: STRING = "GE_mt"
	c_ge_mutex_create: STRING = "GE_mutex_create"
	c_ge_mutex_destroy: STRING = "GE_mutex_destroy"
	c_ge_mutex_lock: STRING = "GE_mutex_lock"
	c_ge_mutex_try_lock: STRING = "GE_mutex_try_lock"
	c_ge_mutex_unlock: STRING = "GE_mutex_unlock"
	c_ge_nat8: STRING = "GE_nat8"
	c_ge_nat16: STRING = "GE_nat16"
	c_ge_nat32: STRING = "GE_nat32"
	c_ge_nat64: STRING = "GE_nat64"
	c_ge_natural_8_field: STRING = "GE_natural_8_field"
	c_ge_natural_8_field_at: STRING = "GE_natural_8_field_at"
	c_ge_natural_16_field: STRING = "GE_natural_16_field"
	c_ge_natural_16_field_at: STRING = "GE_natural_16_field_at"
	c_ge_natural_32_field: STRING = "GE_natural_32_field"
	c_ge_natural_32_field_at: STRING = "GE_natural_32_field_at"
	c_ge_natural_64_field: STRING = "GE_natural_64_field"
	c_ge_natural_64_field_at: STRING = "GE_natural_64_field_at"
	c_ge_new: STRING = "GE_new"
	c_ge_new_exception_manager: STRING = "GE_new_exception_manager"
	c_ge_new_instance_of_encoded_type: STRING = "GE_new_instance_of_encoded_type"
	c_ge_new_istr8: STRING = "GE_new_istr8"
	c_ge_new_istr32: STRING = "GE_new_istr32"
	c_ge_new_once_per_object_data: STRING = "GE_new_once_per_object_data"
	c_ge_new_special_of_reference_instance_of_encoded_type: STRING = "GE_new_special_of_reference_instance_of_encoded_type"
	c_ge_new_str8: STRING = "GE_new_str8"
	c_ge_new_str32: STRING = "GE_new_str32"
	c_ge_new_tuple_instance_of_encoded_type: STRING = "GE_new_tuple_instance_of_encoded_type"
	c_ge_new_type_instance_of_encoded_type: STRING = "GE_new_type_instance_of_encoded_type"
	c_ge_non_attached_encoded_type: STRING = "GE_non_attached_encoded_type"
	c_ge_object_at_offset: STRING = "GE_object_at_offset"
	c_ge_object_encoded_type: STRING = "GE_object_encoded_type"
	c_ge_object_encoded_type_at_offset: STRING = "GE_object_encoded_type_at_offset"
	c_ge_object_id: STRING = "GE_object_id"
	c_ge_object_id_free: STRING = "GE_object_id_free"
	c_ge_object_size: STRING = "GE_object_size"
	c_ge_once_per_object_data_mutex: STRING = "GE_once_per_object_data_mutex"
	c_ge_once_raise: STRING = "GE_once_raise"
	c_ge_persistent_field_count_of_encoded_type: STRING = "GE_persistent_field_count_of_encoded_type"
	c_ge_pointer_field: STRING = "GE_pointer_field"
	c_ge_pointer_field_at: STRING = "GE_pointer_field_at"
	c_ge_power: STRING = "GE_power"
	c_ge_process_once_mutexes: STRING = "GE_process_once_mutexes"
	c_ge_process_onces: STRING = "GE_process_onces"
	c_ge_raise: STRING = "GE_raise"
	c_ge_raise_once_exception: STRING = "GE_raise_once_exception"
	c_ge_raw_object_at_offset: STRING = "GE_raw_object_at_offset"
	c_ge_raw_reference_field_at: STRING = "GE_raw_reference_field_at"
	c_ge_register_dispose: STRING = "GE_register_dispose"
	c_ge_register_dispose_once_per_object_data: STRING = "GE_register_dispose_once_per_object_data"
	c_ge_real_32_field: STRING = "GE_real_32_field"
	c_ge_real_32_field_at: STRING = "GE_real_32_field_at"
	c_ge_real_32_ieee_is_equal: STRING = "GE_real_32_ieee_is_equal"
	c_ge_real_32_ieee_is_greater: STRING = "GE_real_32_ieee_is_greater"
	c_ge_real_32_ieee_is_greater_equal: STRING = "GE_real_32_ieee_is_greater_equal"
	c_ge_real_32_ieee_is_less: STRING = "GE_real_32_ieee_is_less"
	c_ge_real_32_ieee_is_less_equal: STRING = "GE_real_32_ieee_is_less_equal"
	c_ge_real_32_ieee_maximum_number: STRING = "GE_real_32_ieee_maximum_number"
	c_ge_real_32_ieee_minimum_number: STRING = "GE_real_32_ieee_minimum_number"
	c_ge_real_32_is_equal: STRING = "GE_real_32_is_equal"
	c_ge_real_32_is_less: STRING = "GE_real_32_is_less"
	c_ge_real_32_is_nan: STRING = "GE_real_32_is_nan"
	c_ge_real_32_is_negative_infinity: STRING = "GE_real_32_is_negative_infinity"
	c_ge_real_32_is_positive_infinity: STRING = "GE_real_32_is_positive_infinity"
	c_ge_real_32_nan: STRING = "GE_real_32_nan"
	c_ge_real_32_negative_infinity: STRING = "GE_real_32_negative_infinity"
	c_ge_real_32_positive_infinity: STRING = "GE_real_32_positive_infinity"
	c_ge_real_64_field: STRING = "GE_real_64_field"
	c_ge_real_64_field_at: STRING = "GE_real_64_field_at"
	c_ge_real_64_ieee_is_equal: STRING = "GE_real_64_ieee_is_equal"
	c_ge_real_64_ieee_is_greater: STRING = "GE_real_64_ieee_is_greater"
	c_ge_real_64_ieee_is_greater_equal: STRING = "GE_real_64_ieee_is_greater_equal"
	c_ge_real_64_ieee_is_less: STRING = "GE_real_64_ieee_is_less"
	c_ge_real_64_ieee_is_less_equal: STRING = "GE_real_64_ieee_is_less_equal"
	c_ge_real_64_ieee_maximum_number: STRING = "GE_real_64_ieee_maximum_number"
	c_ge_real_64_ieee_minimum_number: STRING = "GE_real_64_ieee_minimum_number"
	c_ge_real_64_is_equal: STRING = "GE_real_64_is_equal"
	c_ge_real_64_is_less: STRING = "GE_real_64_is_less"
	c_ge_real_64_is_nan: STRING = "GE_real_64_is_nan"
	c_ge_real_64_is_negative_infinity: STRING = "GE_real_64_is_negative_infinity"
	c_ge_real_64_is_positive_infinity: STRING = "GE_real_64_is_positive_infinity"
	c_ge_real_64_nan: STRING = "GE_real_64_nan"
	c_ge_real_64_negative_infinity: STRING = "GE_real_64_negative_infinity"
	c_ge_real_64_positive_infinity: STRING = "GE_real_64_positive_infinity"
	c_ge_reference_field: STRING = "GE_reference_field"
	c_ge_reference_field_at: STRING = "GE_reference_field_at"
	c_ge_rescue: STRING = "GE_rescue"
	c_ge_retry: STRING = "GE_retry"
	c_ge_setjmp: STRING = "GE_setjmp"
	c_ge_set_boolean_field: STRING = "GE_set_boolean_field"
	c_ge_set_boolean_field_at: STRING = "GE_set_boolean_field_at"
	c_ge_set_character_8_field: STRING = "GE_set_character_8_field"
	c_ge_set_character_8_field_at: STRING = "GE_set_character_8_field_at"
	c_ge_set_character_32_field: STRING = "GE_set_character_32_field"
	c_ge_set_character_32_field_at: STRING = "GE_set_character_32_field_at"
	c_ge_set_exception_data: STRING = "GE_set_exception_data"
	c_ge_set_integer_8_field: STRING = "GE_set_integer_8_field"
	c_ge_set_integer_8_field_at: STRING = "GE_set_integer_8_field_at"
	c_ge_set_integer_16_field: STRING = "GE_set_integer_16_field"
	c_ge_set_integer_16_field_at: STRING = "GE_set_integer_16_field_at"
	c_ge_set_integer_32_field: STRING = "GE_set_integer_32_field"
	c_ge_set_integer_32_field_at: STRING = "GE_set_integer_32_field_at"
	c_ge_set_integer_64_field: STRING = "GE_set_integer_8_field"
	c_ge_set_integer_64_field_at: STRING = "GE_set_integer_64_field_at"
	c_ge_set_natural_8_field: STRING = "GE_set_natural_8_field"
	c_ge_set_natural_8_field_at: STRING = "GE_set_natural_8_field_at"
	c_ge_set_natural_16_field: STRING = "GE_set_natural_16_field"
	c_ge_set_natural_16_field_at: STRING = "GE_set_natural_16_field_at"
	c_ge_set_natural_32_field: STRING = "GE_set_natural_32_field"
	c_ge_set_natural_32_field_at: STRING = "GE_set_natural_32_field_at"
	c_ge_set_natural_64_field: STRING = "GE_set_natural_64_field"
	c_ge_set_natural_64_field_at: STRING = "GE_set_natural_64_field_at"
	c_ge_set_pointer_field: STRING = "GE_set_pointer_field"
	c_ge_set_pointer_field_at: STRING = "GE_set_pointer_field_at"
	c_ge_set_real_32_field: STRING = "GE_set_real_32_field"
	c_ge_set_real_32_field_at: STRING = "GE_set_real_32_field_at"
	c_ge_set_real_64_field: STRING = "GE_set_real_64_field"
	c_ge_set_real_64_field_at: STRING = "GE_set_real_64_field_at"
	c_ge_set_reference_field: STRING = "GE_set_reference_field"
	c_ge_set_reference_field_at: STRING = "GE_set_reference_field_at"
	c_ge_show_console: STRING = "GE_show_console"
	c_ge_storable_version_of_encoded_type: STRING = "GE_storable_version_of_encoded_type"
	c_ge_thread_onces_set_counts: STRING = "GE_thread_onces_set_counts"
	c_ge_type_flag_boolean: STRING = "GE_TYPE_FLAG_BOOLEAN"
	c_ge_type_flag_character_8: STRING = "GE_TYPE_FLAG_CHARACTER_8"
	c_ge_type_flag_character_32: STRING = "GE_TYPE_FLAG_CHARACTER_32"
	c_ge_type_flag_deferred: STRING = "GE_TYPE_FLAG_DEFERRED"
	c_ge_type_flag_expanded: STRING = "GE_TYPE_FLAG_EXPANDED"
	c_ge_type_flag_integer_8: STRING = "GE_TYPE_FLAG_INTEGER_8"
	c_ge_type_flag_integer_16: STRING = "GE_TYPE_FLAG_INTEGER_16"
	c_ge_type_flag_integer_32: STRING = "GE_TYPE_FLAG_INTEGER_32"
	c_ge_type_flag_integer_64: STRING = "GE_TYPE_FLAG_INTEGER_64"
	c_ge_type_flag_natural_8: STRING = "GE_TYPE_FLAG_NATURAL_8"
	c_ge_type_flag_natural_16: STRING = "GE_TYPE_FLAG_NATURAL_16"
	c_ge_type_flag_natural_32: STRING = "GE_TYPE_FLAG_NATURAL_32"
	c_ge_type_flag_natural_64: STRING = "GE_TYPE_FLAG_NATURAL_64"
	c_ge_type_flag_none: STRING = "GE_TYPE_FLAG_NONE"
	c_ge_type_flag_pointer: STRING = "GE_TYPE_FLAG_POINTER"
	c_ge_type_flag_real_32: STRING = "GE_TYPE_FLAG_REAL_32"
	c_ge_type_flag_real_64: STRING = "GE_TYPE_FLAG_REAL_64"
	c_ge_type_flag_special: STRING = "GE_TYPE_FLAG_SPECIAL"
	c_ge_type_flag_tuple: STRING = "GE_TYPE_FLAG_TUPLE"
	c_ge_type_infos: STRING = "GE_type_infos"
	c_ge_types: STRING = "GE_types"
	c_ge_unlock_marking: STRING = "GE_unlock_marking"
	c_ge_unmark_object: STRING = "GE_unmark_object"
	c_ge_use_ancestors: STRING = "GE_USE_ANCESTORS"
	c_ge_use_attribute_name: STRING = "GE_USE_ATTRIBUTE_NAME"
	c_ge_use_attribute_offset: STRING = "GE_USE_ATTRIBUTE_OFFSET"
	c_ge_use_attribute_type_id: STRING = "GE_USE_ATTRIBUTE_TYPE_ID"
	c_ge_use_attributes: STRING = "GE_USE_ATTRIBUTES"
	c_ge_use_boehm_gc: STRING = "GE_USE_BOEHM_GC"
	c_ge_use_posix_threads: STRING = "GE_USE_POSIX_THREADS"
	c_ge_use_threads: STRING = "GE_USE_THREADS"
	c_ge_use_type_generator: STRING = "GE_USE_TYPE_GENERATOR"
	c_ge_use_type_generic_parameters: STRING = "GE_USE_TYPE_GENERIC_PARAMETERS"
	c_ge_use_type_name: STRING = "GE_USE_TYPE_NAME"
	c_ge_use_type_object_size: STRING = "GE_USE_TYPE_OBJECT_SIZE"
	c_ge_void: STRING = "GE_void"
	c_goto: STRING = "goto"
	c_id: STRING = "id"
	c_if: STRING = "if"
	c_ifdef: STRING = "#ifdef"
	c_ifndef: STRING = "#ifndef"
	c_in_assertion: STRING = "in_assertion"
	c_in_rescue: STRING = "in_rescue"
	c_include: STRING = "#include"
	c_initialize: STRING = "initialize"
	c_int: STRING = "int"
	c_int8_t: STRING = "int8_t"
	c_int16_t: STRING = "int16_t"
	c_int32_t: STRING = "int32_t"
	c_int64_t: STRING = "int64_t"
	c_is_special: STRING = "is_special"
	c_last_rescue: STRING = "last_rescue"
	c_line: STRING = "#line"
	c_memcmp: STRING = "memcmp"
	c_memcpy: STRING = "memcpy"
	c_memset: STRING = "memset"
	c_mutex_suffix: STRING = "_mutex"
	c_not: STRING = "!"
	c_not_equal: STRING = "!="
	c_not_not: STRING = ""
	c_object: STRING = "object"
	c_offset: STRING = "offset"
	c_offsetof: STRING = "offsetof"
	c_once_objects: STRING = "once_objects"
	c_onces: STRING = "onces"
	c_or_else: STRING = "||"
	c_pre_ecma_mapping_status: STRING = "pre_ecma_mapping_status"
	c_previous: STRING = "previous"
	c_process_onces: STRING = "process_onces"
	c_return: STRING = "return"
	c_sizeof: STRING = "sizeof"
	c_status_suffix: STRING = "_status"
	c_stderr: STRING = "stderr"
	c_struct: STRING = "struct"
	c_switch: STRING = "switch"
	c_tc: STRING = "tc"
	c_tc_address: STRING = "&tc"
	c_thread_onces: STRING = "thread_onces"
	c_tr: STRING = "tr"
	c_type_id: STRING = "type_id"
	c_typedef: STRING = "typedef"
	c_uint16_t: STRING = "uint16_t"
	c_uint32_t: STRING = "uint32_t"
	c_undef: STRING = "#undef"
	c_unsigned: STRING = "unsigned"
	c_value_suffix: STRING = "_value"
	c_void: STRING = "void"
	c_volatile: STRING = "volatile"
	c_while: STRING = "while"
			-- String constants

	default_split_threshold: INTEGER = 2500000
			-- Default value for `split_threshold'

	bat_file_extension: STRING = ".bat"
	c_file_extension: STRING = ".c"
	cfg_file_extension: STRING = ".cfg"
	cpp_file_extension: STRING = ".cpp"
	h_file_extension: STRING = ".h"
	res_file_extension: STRING = ".res"
	resx_file_extension: STRING = ".resx"
	rc_file_extension: STRING = ".rc"
	sh_file_extension: STRING = ".sh"
	make_file_extension: STRING = ".make"
			-- File extensions

	makefile_template: STRING = "[
#
# Makefile generated by gec.
#

.PHONY: all install clean distclean uninstall

all: $exe

prefix = /usr/local
exec_prefix = $(prefix)
bindir = $(exec_prefix)/bin
mandir = $(prefix)/man
man1dir = $(mandir)/man1

INSTALL = install -C
INSTALL_PROGRAM = $(INSTALL)
INSTALL_DATA = $(INSTALL)

CFLAGS += $cflags $includes $gc_includes
LDFLAGS += $lflags $lflags_threads
LDLIBS += -lm $gc_libs $libs
OBJS = $objs

$exe: $(OBJS) $header
	$(CC) $(LDFLAGS) -o $exe $(OBJS) $(LDLIBS)

clean:
	-rm $(OBJS) $exe

distclean: clean

install:
	$(INSTALL_PROGRAM) -d $(DESTDIR)$(bindir)
	$(INSTALL_PROGRAM) $exe $(DESTDIR)$(bindir)

uninstall:
	-rm -f $(DESTDIR)$(bindir)/$exe

.c.o:
	$(CC) $(CFLAGS) -c $<
]"

invariant

	current_dynamic_system_not_void: current_dynamic_system /= Void
	system_processor_not_void: system_processor /= Void
	dynamic_types_not_void: dynamic_types /= Void
	no_void_dynamic_type: not dynamic_types.has_void
	current_file_not_void: current_file /= Void
	current_file_open_write: current_file.is_open_write
	header_file_not_void: header_file /= Void
	header_file_open_write: header_file.is_open_write
	current_function_header_buffer_not_void: current_function_header_buffer /= Void
	current_function_body_buffer_not_void: current_function_body_buffer /= Void
	current_feature_not_void: current_feature /= Void
	current_type_not_void: current_type /= Void
	operand_stack_not_void: operand_stack /= Void
	no_void_operand: not operand_stack.has_void
	call_operands_not_void: call_operands /= Void
	no_void_call_operand: not call_operands.has_void
	conforming_types_not_void: conforming_types /= Void
	non_conforming_types_not_void: non_conforming_types /= Void
	attachment_dynamic_type_ids_not_void: attachment_dynamic_type_ids /= Void
	target_dynamic_type_ids_not_void: target_dynamic_type_ids /= Void
	target_dynamic_types_not_void: target_dynamic_types /= Void
	no_void_target_dynamic_type: not target_dynamic_types.has_void_item
	equality_type_set_not_void: equality_type_set /= Void
	equality_common_types_not_void: equality_common_types /= Void
	equality_common_reference_types_not_void: equality_common_reference_types /= Void
	standalone_type_sets_not_void: standalone_type_sets /= Void
	unused_standalone_type_sets_not_void: unused_standalone_type_sets /= Void
	no_void_unused_standalone_type_set: not unused_standalone_type_sets.has_void
	deep_twin_types_not_void: deep_twin_types /= Void
	no_void_deep_twin_type: not deep_twin_types.has_void
	deep_equal_types_not_void: deep_equal_types /= Void
	no_void_deep_equal_type: not deep_equal_types.has_void
	deep_feature_target_type_sets_not_void: deep_feature_target_type_sets /= Void
	no_void_deep_feature_target_type_set: not deep_feature_target_type_sets.has_void_item
	no_void_deep_feature_static_target_type: not deep_feature_target_type_sets.has_void
	current_agents_not_void: current_agents /= Void
	no_void_agent: not current_agents.has_void
	agent_instruction_not_void: agent_instruction /= Void
	agent_expression_not_void: agent_expression /= Void
	agent_target_not_void: agent_target /= Void
	agent_arguments_not_void: agent_arguments /= Void
	agent_open_operands_not_void: agent_open_operands /= Void
	no_void_agent_open_operand: not agent_open_operands.has_void
	agent_closed_operands_not_void: agent_closed_operands /= Void
	no_void_agent_closed_operand: not agent_closed_operands.has_void
	agent_closed_operands_type_not_void: agent_closed_operands_type /= Void
	manifest_array_types_not_void: manifest_array_types /= Void
	no_void_manifest_array_type: not manifest_array_types.has_void
	big_manifest_array_types_not_void: big_manifest_array_types /= Void
	no_void_big_manifest_array_type: not big_manifest_array_types.has_void
	manifest_tuple_types_not_void: manifest_tuple_types /= Void
	no_void_manifest_tuple_type: not manifest_tuple_types.has_void
	current_object_tests_not_void: current_object_tests /= Void
	no_void_current_object_test: not current_object_tests.has_void
	current_equalities_not_void: current_equalities /= Void
	no_void_current_equality: not current_equalities.has_void
	called_features_not_void: called_features /= Void
	no_void_called_feature: not called_features.has_void
	called_static_features_not_void: called_static_features /= Void
	no_void_called_static_feature: not called_static_features.has_void
	once_features_not_void: once_features /= Void
	no_void_once_feature: not once_features.has_void
	once_feature_constraint: across once_features as i_feature all @i_feature.key = @i_feature.key.implementation_feature end
	once_per_process_counts_not_void: once_per_process_counts /= Void
	once_per_process_counts_bounds: once_per_process_counts.lower = 1 and once_per_process_counts.upper = once_kind_count
	no_negative_once_per_process_count: across once_per_process_counts as l_once_per_process_counts all l_once_per_process_counts.item >= 0 end
	once_per_thread_counts_not_void: once_per_thread_counts /= Void
	once_per_thread_counts_bounds: once_per_thread_counts.lower = 1 and once_per_thread_counts.upper = once_kind_count
	no_negative_once_per_thread_count: across once_per_thread_counts as l_once_per_thread_counts all l_once_per_thread_counts.item >= 0 end
	constant_features_not_void: constant_features /= Void
	no_void_constant_feature: not constant_features.has_void
	constant_feature_constraint: across constant_features as i_feature all @i_feature.key = @i_feature.key.implementation_feature end
	inline_constants_not_void: inline_constants /= Void
	no_void_inline_constant: not inline_constants.has_void
	no_void_inline_constant_type: not inline_constants.has_void_item
	dispose_procedures_not_void: dispose_procedures /= Void
		-- Temporary variables.
	temp_variables_not_void: temp_variables /= Void
	no_void_temp_variable: not temp_variables.has_void
	used_temp_variables_not_void: used_temp_variables /= Void
	used_temp_variables_count: used_temp_variables.count <= temp_variables.count
	free_temp_variables_not_void: free_temp_variables /= Void
	free_temp_variables_count: free_temp_variables.count = used_temp_variables.count
	frozen_temp_variables_not_void: frozen_temp_variables /= Void
	frozen_temp_variables_count: frozen_temp_variables.count = used_temp_variables.count
		-- Equality expressions.
	unused_equality_expressions_not_void: unused_equality_expressions /= Void
	no_void_unused_equality_expression: not unused_equality_expressions.has_void
		-- Equality types.
	unused_equality_types_not_void: unused_equality_types /= Void
	no_void_unused_equality_types: not unused_equality_types.has_void
		-- Object equality types.
	unused_object_equality_types_not_void: unused_object_equality_types /= Void
	no_void_unused_object_equality_types: not unused_object_equality_types.has_void
		-- Unqualified call expressions.
	unused_unqualified_call_expressions_not_void: unused_unqualified_call_expressions /= Void
	no_void_unused_unqualified_call_expression: not unused_unqualified_call_expressions.has_void
		-- Qualified call expressions.
	unused_qualified_call_expressions_not_void: unused_qualified_call_expressions /= Void
	no_void_unused_qualified_call_expression: not unused_qualified_call_expressions.has_void
		-- Actual argument lists.
	unused_actual_argument_lists_not_void: unused_actual_argument_lists /= Void
	no_void_unused_actual_argument_list: not unused_actual_argument_lists.has_void
		-- Result expressions.
	unused_result_expressions_not_void: unused_result_expressions /= Void
	no_void_unused_result_expression: not unused_result_expressions.has_void
		-- Identifiers.
	unused_identifiers_not_void: unused_identifiers /= Void
	no_void_unused_identifier: not unused_identifiers.has_void
		-- C/C++ files.
	included_header_filenames_not_void: included_header_filenames /= Void
	no_void_included_header_filename: not included_header_filenames.has_void
	included_cpp_header_filenames_not_void: included_cpp_header_filenames /= Void
	no_void_included_cpp_header_filename: not included_cpp_header_filenames.has_void
	included_runtime_header_files_not_void: included_runtime_header_files /= Void
	no_void_included_runtime_header_file: not included_runtime_header_files.has_void
	included_runtime_c_files_not_void: included_runtime_c_files /= Void
	no_void_included_runtime_c_file: not included_runtime_c_files.has_void
	c_filenames_not_void: c_filenames /= Void
	no_void_c_filename: not c_filenames.has_void
	no_void_c_file_extension: not c_filenames.has_void_item
		-- Rescue clauses.
	locals_written_not_void: locals_written /= Void
	no_void_local_written: not locals_written.has_void
	locals_written_in_body_not_void: locals_written_in_body /= Void
	no_void_local_written_in_body: not locals_written_in_body.has_void
	locals_written_in_rescue_not_void: locals_written_in_rescue /= Void
	no_void_local_written_in_rescue: not locals_written_in_rescue.has_void
	locals_read_not_void: locals_read /= Void
	no_void_local_read: not locals_read.has_void
	locals_read_in_body_not_void: locals_read_in_body /= Void
	no_void_local_read_in_body: not locals_read_in_body.has_void
	locals_read_in_rescue_not_void: locals_read_in_rescue /= Void
	no_void_local_read_in_rescue: not locals_read_in_rescue.has_void
		-- Regular expressions for external features.
	external_c_regexp_not_void: external_c_regexp /= Void
	external_c_regexp_compiled: external_c_regexp.is_compiled
	external_c_macro_regexp_not_void: external_c_macro_regexp /= Void
	external_c_macro_regexp_compiled: external_c_macro_regexp.is_compiled
	external_c_struct_regexp_not_void: external_c_struct_regexp /= Void
	external_c_struct_regexp_compiled: external_c_struct_regexp.is_compiled
	external_c_inline_regexp_not_void: external_c_inline_regexp /= Void
	external_c_inline_regexp_compiled: external_c_inline_regexp.is_compiled
	old_external_c_regexp_not_void: old_external_c_regexp /= Void
	old_external_c_regexp_compiled: old_external_c_regexp.is_compiled
	old_external_c_macro_regexp_not_void: old_external_c_macro_regexp /= Void
	old_external_c_macro_regexp_compiled: old_external_c_macro_regexp.is_compiled
	old_external_c_struct_regexp_not_void: old_external_c_struct_regexp /= Void
	old_external_c_struct_regexp_compiled: old_external_c_struct_regexp.is_compiled
	external_cpp_regexp_not_void: external_cpp_regexp /= Void
	external_cpp_regexp_compiled: external_cpp_regexp.is_compiled
	external_cpp_macro_regexp_not_void: external_cpp_macro_regexp /= Void
	external_cpp_macro_regexp_compiled: external_cpp_macro_regexp.is_compiled
	external_cpp_struct_regexp_not_void: external_cpp_struct_regexp /= Void
	external_cpp_struct_regexp_compiled: external_cpp_struct_regexp.is_compiled
	external_cpp_inline_regexp_not_void: external_cpp_inline_regexp /= Void
	external_cpp_inline_regexp_compiled: external_cpp_inline_regexp.is_compiled
	external_dllwin_regexp_not_void: external_dllwin_regexp /= Void
	external_dllwin_regexp_compiled: external_dllwin_regexp.is_compiled
	old_external_dllwin32_regexp_not_void: old_external_dllwin32_regexp /= Void
	old_external_dllwin32_regexp_compiled: old_external_dllwin32_regexp.is_compiled
		--
	split_threshold_positive: split_threshold > 0
	system_name_not_void: system_name /= Void
	dynamic_type_id_set_names_not_void: dynamic_type_id_set_names /= Void
	no_void_dynamic_type_id_set_name: not dynamic_type_id_set_names.has_void_item
	no_void_dynamic_type_id_set: not dynamic_type_id_set_names.has_void
	current_dynamic_type_sets_not_void: current_dynamic_type_sets /= Void
	extra_dynamic_type_sets_not_void: extra_dynamic_type_sets /= Void
	agent_manifest_tuple_not_void: agent_manifest_tuple /= Void
	formal_arguments_not_void: formal_arguments /= Void
	no_void_formal_argument: not formal_arguments.has_void

end
